/**
 * SECTION:applet-calendar-main
 * @short_description: 日历applet总入口
 *
 * 提供给applet-manager调用。
 *
 * 实现日期与时间的显示，系统“日期和时间”的设置入口。
 *
 */
#include "common.h"
#include "applet-calendar-main.h"
#include "cdos-popup-menu-manager.h"
#include "cdos-popup-menu.h"
#include "cdos-popup-separator-menu-item.h"
#include "cdos-util.h"

#include "applet-calendar.h"
// modify by linjj for remove clock begin
// #include "applet-calendar-clock.h"
// modify by linjj for remove clock end

#include "applet-calendar-private.h"

#define UPDATE_INTERVAL_FOR_SEC (500)
#define UPDATE_INTERVAL_FOR_NOSEC (10000)

struct _AppletCalendarMainPrivate
{
    ClutterActor *actor;
    CdosPopupMenu *popup_menu;
    GSettings *settings;
    gboolean need_show_seconds;
    gboolean is_use_24h;
    gboolean need_show_date;
    gboolean date_use_long_format;
    gboolean date_use_short_format;
    gboolean need_show_week;
    const gchar *format_class_name;

    StWidget *title;
    StWidget *datetimeSet;
    StWidget *actortimelabel;
    StWidget *actordatelabel;

    guint datetimeupdateid;

    AppletCalendar *calendar;
    gint year;
    gint month;
    gint date;
    StWidget *timeLabel;
    StWidget *currentDateBtn;
    CdosPopupMenuItem *scheduleManageBtn;
    CdosPopupMenuItem *timeSettingBtn;
    CdosPopupSeparatorMenuItem *separatorItem;
    //语音控制
    GDBusProxy *vc_proxy;
};

static const gchar *calendar_background[] =
{
    "applet-calendar-bg-74",
    "applet-calendar-bg-94",
    "applet-calendar-bg-94",
    "applet-calendar-bg-94",
    "applet-calendar-bg-48",
    "applet-calendar-bg-68",
    "applet-calendar-bg-94",
    "applet-calendar-bg-94"
};


G_DEFINE_TYPE_WITH_PRIVATE (AppletCalendarMain, applet_calendar_main, CDOS_TYPE_APPLET)

static void load_settings (AppletCalendarMain *self);
static void apply_settings (AppletCalendarMain *self);
static void update_datetime (AppletCalendarMain *self);
static void update_hover_background (AppletCalendarMain *self, gchar *key);

static void gsetting_value_changed (GSettings *settings, gchar *key, gpointer user_data);

static gboolean
callUpdateTime (gpointer user_data)
{
    AppletCalendarMain *self = APPLET_CALENDAR_MAIN (user_data);
    update_datetime (self);
    return TRUE;
}

static void
onScheculeManageClicked (ClutterActor *actor, ClutterEvent *event, gboolean keep, gpointer data)
{
    cdos_util_spawn_command_line ("gnome-calendar");
}

static void
onDateSettingClicked (ClutterActor *actor, ClutterEvent *event, gboolean keep, gpointer data)
{
    cdos_util_spawn_command_line ("found-control-center datetime");
}

static void
onCurrentDateClicked (StWidget *button, ClutterEvent *event, gpointer user_data)
{
    AppletCalendarMainPrivate *priv = APPLET_CALENDAR_MAIN (user_data)->priv;
    applet_calendar_update (priv->calendar);
}

static void
vc_proxy_on_response_cb (GDBusProxy *proxy,
                        char       *sender_name,
                        char       *signal_name,
                        GVariant   *parameters,
                        gpointer    user_data)
{
    AppletCalendarMain *self = APPLET_CALENDAR_MAIN (user_data);
    AppletCalendarMainPrivate *priv = self->priv;
    gchar *action_name, *operation;
    GVariant *v;

    if (g_strcmp0 (signal_name, "Request") == 0)
    {
        g_variant_get (parameters, "(ssv)", &action_name, &operation, &v);

        if (g_strcmp0 (action_name, "open-calender") == 0)
        {
            cdos_popup_menu_open ((CdosPopupMenuBase *)priv->popup_menu, 0);
        }
        g_free (action_name);
        g_free (operation);
        g_variant_unref (v);
    }
}

static void
vc_proxy_ready_cb (GObject *source_object,
                   GAsyncResult *res,
                   gpointer user_data)
{
    AppletCalendarMain *self = APPLET_CALENDAR_MAIN (user_data);
    AppletCalendarMainPrivate *priv = self->priv;
    GError *error = NULL;

    priv->vc_proxy = g_dbus_proxy_new_for_bus_finish (res, &error);
    if (error)
    {
        g_warning ("Connect ai assistant failed: %s", error->message);
        g_error_free (error);
        return;
    }
    if (priv->vc_proxy)
        g_signal_connect (priv->vc_proxy,
                        "g-signal", 
                        G_CALLBACK (vc_proxy_on_response_cb), 
                        self);
}

static void
applet_calendar_main_dispose (GObject *gobject)
{
    AppletCalendarMain *self = APPLET_CALENDAR_MAIN (gobject);
    if (self->priv->datetimeupdateid)
    {
        g_source_remove (self->priv->datetimeupdateid);
        self->priv->datetimeupdateid = 0;
    }
    if (self->priv->settings)
    {
        g_object_unref (self->priv->settings);
        self->priv->settings = NULL;
    }
    if (self->priv->vc_proxy)
    {
        g_object_unref (self->priv->vc_proxy);
        self->priv->vc_proxy = NULL;
    }
    G_OBJECT_CLASS (applet_calendar_main_parent_class)->dispose (gobject);
}

static void
applet_calendar_main_finalize (GObject *calendarapplet)
{
    G_OBJECT_CLASS (applet_calendar_main_parent_class)->finalize (calendarapplet);
}

static void
applet_calendar_main_class_init (AppletCalendarMainClass *klass)
{
    GObjectClass *objcls = G_OBJECT_CLASS (klass);

    objcls->dispose = applet_calendar_main_dispose;
    objcls->finalize = applet_calendar_main_finalize;
}

static void
applet_calendar_main_init (AppletCalendarMain *self)
{
    AppletCalendarMainPrivate *priv;
    CdosPopupMenu *context_menu;
    cdos_applet_set_popup_menu_active (CDOS_APPLET (self), TRUE);
    cdos_applet_set_context_menu_active (CDOS_APPLET (self), TRUE);
    priv = self->priv = applet_calendar_main_get_instance_private (self);

    priv->settings = g_settings_new (GSETTING_SCHEMA_NAME);
    load_settings (self);
    priv->date = 0;
    priv->month = 0;
    priv->year = 0;
    g_signal_connect (priv->settings, "changed::" SHOW_DATE_KEY, G_CALLBACK (gsetting_value_changed), self);
    g_signal_connect (priv->settings, "changed::" SHOW_SEC_KEY, G_CALLBACK (gsetting_value_changed), self);
    g_signal_connect (priv->settings, "changed::" USE_24H_KEY, G_CALLBACK (gsetting_value_changed), self);
    g_signal_connect (priv->settings, "changed::" USE_LONG_FORMAT_KEY, G_CALLBACK (gsetting_value_changed), self);
    g_signal_connect (priv->settings, "changed::" USE_SHORT_FORMAT_KEY, G_CALLBACK (gsetting_value_changed), self);
    g_signal_connect (priv->settings, "changed::" CLOCK_SHOW_WEEK, G_CALLBACK (gsetting_value_changed), self);

    priv->actor = (ClutterActor *)st_bin_new ();
    clutter_actor_add_child (CLUTTER_ACTOR (self), CLUTTER_ACTOR (priv->actor));
    st_widget_add_style_class_name(ST_WIDGET(priv->actor), "applet-calendar-bg");

    priv->actortimelabel = st_label_new ("");
    st_widget_set_style_class_name (priv->actortimelabel, "applet-label");
    clutter_actor_set_x_align (CLUTTER_ACTOR (priv->actortimelabel), CLUTTER_ACTOR_ALIGN_CENTER);
    clutter_actor_set_y_align (CLUTTER_ACTOR (priv->actortimelabel), CLUTTER_ACTOR_ALIGN_CENTER);

    priv->actordatelabel = st_label_new ("");
    st_widget_set_style_class_name (priv->actordatelabel, "applet-label");
    clutter_actor_set_x_align (CLUTTER_ACTOR (priv->actordatelabel), CLUTTER_ACTOR_ALIGN_CENTER);
    clutter_actor_set_y_align (CLUTTER_ACTOR (priv->actordatelabel), CLUTTER_ACTOR_ALIGN_CENTER);

    ClutterText *clutter_text = (ClutterText *)st_label_get_clutter_text ((StLabel *)priv->actortimelabel);
    clutter_text_set_ellipsize (clutter_text, PANGO_ELLIPSIZE_NONE);

    clutter_text = (ClutterText *)st_label_get_clutter_text ((StLabel *)priv->actordatelabel);
    clutter_text_set_ellipsize (clutter_text, PANGO_ELLIPSIZE_NONE);

    priv->popup_menu = cdos_applet_get_popup_menu (CDOS_APPLET (self));

    StBoxLayout *calendarbox = cdos_popup_menu_base_get_box (CDOS_POPUP_MENU_BASE (priv->popup_menu));

    StWidget *mainLayout = st_box_layout_new ();
    st_box_layout_set_vertical (ST_BOX_LAYOUT (mainLayout), TRUE);

    priv->timeLabel = st_label_new ("");
    st_widget_set_style_class_name (ST_WIDGET (priv->timeLabel), "datemenu-date-label");

    priv->currentDateBtn = st_label_new ("");
    st_widget_set_style_class_name (ST_WIDGET (priv->currentDateBtn), "calendar-time-current-date");
    g_signal_connect (priv->currentDateBtn, "button-release-event", G_CALLBACK (onCurrentDateClicked), self);

    StWidget *calendarLayout = st_box_layout_new ();
    st_box_layout_set_vertical (ST_BOX_LAYOUT (calendarLayout), FALSE);
    priv->calendar = applet_calendar_new ();
    clutter_actor_insert_child_at_index (CLUTTER_ACTOR (calendarLayout), CLUTTER_ACTOR (priv->calendar->actor), -1);

    // modify by linjj for remove clock begin
    // AppletCalendarClock* clock = applet_calendar_clock_new();
    // st_box_layout_insert_actor(ST_BOX_LAYOUT(calendarLayout), CLUTTER_ACTOR(clock->actor), -1);
    // modify by linjj for remove clock end

    // StWidget* timeSettingBtn = st_button_new_with_label(_("Date and Time Settings"));
    // st_widget_set_style_class_name(ST_WIDGET(timeSettingBtn), "calendar-time-setting-button");
    // clutter_actor_set_x_align(CLUTTER_ACTOR(timeSettingBtn), CLUTTER_ACTOR_ALIGN_CENTER);
    // st_widget_set_track_hover (timeSettingBtn, TRUE);
    // g_signal_connect(G_OBJECT(timeSettingBtn), "clicked", G_CALLBACK(onDateSettingClicked), self);

    _add_actor (mainLayout, priv->timeLabel);
    _add_actor (mainLayout, priv->currentDateBtn);

    CdosPopupSeparatorMenuItem *separatorItem = cdos_popup_separator_menu_item_new ();
    st_widget_add_style_class_name (ST_WIDGET (separatorItem), "calendar-h-separator");
    _add_actor (mainLayout, separatorItem);

    _add_actor (mainLayout, calendarLayout);

    g_object_set (priv->timeLabel, "x-expand", TRUE, "x-align", CLUTTER_ACTOR_ALIGN_FILL, "y-align", CLUTTER_ACTOR_ALIGN_CENTER, NULL);
    g_object_set (priv->currentDateBtn, "reactive", TRUE, "x-expand", TRUE, "x-align", CLUTTER_ACTOR_ALIGN_START, "y-align", CLUTTER_ACTOR_ALIGN_CENTER, "track-hover", TRUE, NULL);

    clutter_actor_add_child (CLUTTER_ACTOR (calendarbox), CLUTTER_ACTOR (mainLayout));

    context_menu = cdos_applet_get_context_menu (CDOS_APPLET (self));
    GFile *file = g_file_new_for_path ("/usr/bin/gnome-calendar");
    if (g_file_query_exists (file, NULL))
    {
        priv->scheduleManageBtn = cdos_popup_menu_item_new ();
        cdos_popup_menu_item_set_label_align (priv->scheduleManageBtn, ST_ALIGN_MIDDLE, 0);
        cdos_popup_menu_item_set_label (priv->scheduleManageBtn, _("Schedule Manage"));
        StLabel *label_schedule = cdos_popup_menu_item_get_label (priv->scheduleManageBtn);
        st_widget_add_style_class_name (ST_WIDGET (label_schedule), "popup-menu-item-label-link");

        g_signal_connect (priv->scheduleManageBtn, "activate", G_CALLBACK (onScheculeManageClicked), self);
        cdos_popup_menu_base_add_menu_item (CDOS_POPUP_MENU_BASE (context_menu), 
                                            CDOS_POPUP_BASE_MENU_ITEM (priv->scheduleManageBtn), 
                                            -1);
    }
    g_object_unref (file);

    /* priv->separatorItem = cdos_popup_separator_menu_item_new ();
    st_widget_add_style_class_name (ST_WIDGET (priv->separatorItem), "calendar-h-separator");
    cdos_popup_menu_base_add_menu_item (
        CDOS_POPUP_MENU_BASE (priv->popup_menu), CDOS_POPUP_BASE_MENU_ITEM (priv->separatorItem), -1); */

    priv->timeSettingBtn = cdos_popup_menu_item_new ();
    cdos_popup_menu_item_set_label_align (priv->timeSettingBtn, ST_ALIGN_MIDDLE, 0);
    cdos_popup_menu_item_set_label (priv->timeSettingBtn, _("Date and Time Settings"));
    StLabel *label = cdos_popup_menu_item_get_label (priv->timeSettingBtn);
    st_widget_add_style_class_name (ST_WIDGET (label), "popup-menu-item-label-link");

    g_signal_connect (priv->timeSettingBtn, "activate", G_CALLBACK (onDateSettingClicked), self);
    cdos_popup_menu_base_add_menu_item (CDOS_POPUP_MENU_BASE (context_menu), 
                                        CDOS_POPUP_BASE_MENU_ITEM (priv->timeSettingBtn), 
                                        -1);

    //    static const ClutterColor blue_color = { 0x00, 0x00, 0xff, 0xff };
    //    clutter_actor_set_background_color(CLUTTER_ACTOR(calendarbox), &blue_color);

    StWidget *datetimegroup = st_box_layout_new ();
    st_box_layout_set_vertical (ST_BOX_LAYOUT (datetimegroup), TRUE);
    clutter_actor_set_x_align (CLUTTER_ACTOR (datetimegroup), CLUTTER_ACTOR_ALIGN_CENTER);
    clutter_actor_set_y_align (CLUTTER_ACTOR (datetimegroup), CLUTTER_ACTOR_ALIGN_CENTER);

    _add_actor (datetimegroup, priv->actortimelabel);
    _add_actor (datetimegroup, priv->actordatelabel);

    g_object_set (priv->actortimelabel,
                  "x-expand", TRUE,
                  "x-align", CLUTTER_ACTOR_ALIGN_CENTER,
                  "y_align", CLUTTER_ACTOR_ALIGN_CENTER,
                  NULL);
    g_object_set (priv->actordatelabel,
                  "x-expand", TRUE,
                  "x-align", CLUTTER_ACTOR_ALIGN_CENTER,
                  "y_align", CLUTTER_ACTOR_ALIGN_CENTER,
                  NULL);

    st_bin_set_child ((StBin *)priv->actor, CLUTTER_ACTOR (datetimegroup));
    clutter_actor_set_reactive( CLUTTER_ACTOR (priv->actor), TRUE);

    update_hover_background (self, NULL);

    apply_settings (self);

    //连接语音控制
    g_dbus_proxy_new_for_bus (G_BUS_TYPE_SESSION,
                          G_DBUS_PROXY_FLAGS_NONE,
                          NULL,
                          "org.freedesktop.AIAssistant",
                          "/org/freedesktop/AIAssistant",
                          "org.freedesktop.AIAssistant",
                          NULL,
                          (GAsyncReadyCallback)vc_proxy_ready_cb,
                          self);
}

/**
 * applet_calendar_main_new:
 * @orientation:（未使用）
 * @panel_height:任务栏的高度
 *
 * 构建日历applet对象，用于在任务栏中显示日期和时间。
 *
 * Returns: 日历applet对象
 */
CdosApplet *
applet_calendar_main_new (StSide orientation)
{
    return g_object_new (APPLET_TYPE_CALENDAR_MAIN, "orientation", orientation, NULL);
}

/**
  * applets_calendar_main:
  * @{meta}:(未使用)
  * @orientation:(未使用)
  * @panel_height:任务栏高度
  *
  * 外部模块调用applet-calendar模块入口。
  *
  * Return value:日历applet对象
  */
CdosApplet *
applets_calendar_main (MetaData *meta, StSide orientation, gfloat panel_height)
{
    return applet_calendar_main_new (orientation);
}

static void
load_settings (AppletCalendarMain *self)
{
    AppletCalendarMainPrivate *priv = self->priv;
    priv->need_show_date = g_settings_get_boolean (priv->settings, SHOW_DATE_KEY);
    priv->need_show_seconds = g_settings_get_boolean (priv->settings, SHOW_SEC_KEY);
    priv->is_use_24h = g_settings_get_boolean (priv->settings, USE_24H_KEY);
    priv->date_use_long_format = g_settings_get_boolean (priv->settings, USE_LONG_FORMAT_KEY);
    priv->date_use_short_format = g_settings_get_boolean (priv->settings, USE_SHORT_FORMAT_KEY);
    priv->need_show_week = g_settings_get_boolean (priv->settings, CLOCK_SHOW_WEEK);
}

static void
apply_settings (AppletCalendarMain *self)
{
    AppletCalendarMainPrivate *priv = self->priv;

    update_datetime (self);

    if (0 != priv->datetimeupdateid)
    {
        g_source_remove (priv->datetimeupdateid);
        priv->datetimeupdateid = 0;
    }
    priv->datetimeupdateid = g_timeout_add (
        (priv->need_show_seconds ? UPDATE_INTERVAL_FOR_SEC : UPDATE_INTERVAL_FOR_NOSEC), callUpdateTime, self);
}

static void
update_datetime (AppletCalendarMain *self)
{
    AppletCalendarMainPrivate *priv;
    GDateTime *now;
    gint date;
    gint month;
    gint year;
    gchar* datetimeformat;
    gchar* datetimeformat_with_seconds;
    gchar *timestrtmp;
    gchar *timestrtmp_with_seconds;
    gchar *datestrtmp;
    gchar *datestrformat;
    gchar *currentstrtmp;

    priv = self->priv;

    now = g_date_time_new_now_local ();
    date = g_date_time_get_day_of_month (now);
    month = g_date_time_get_month (now);
    year = g_date_time_get_year (now);
    if (date != priv->date || month != priv->month || year != priv->year)
    {
        applet_calendar_update (priv->calendar);
        priv->date = date;
        priv->month = month;
        priv->year = year;
    }
    datetimeformat = g_strdup_printf("%s:%%M",
                                     priv->is_use_24h ? "%H" : "%p %l");
    datetimeformat_with_seconds = g_strdup_printf("%s:%%M%s",
                                     priv->is_use_24h ? "%H" : "%p %l",
                                     priv->need_show_seconds ? ":%S" : "");
    timestrtmp = g_date_time_format (now, datetimeformat);
    timestrtmp_with_seconds = g_date_time_format (now, datetimeformat_with_seconds);

    if (priv->date_use_long_format)
    {
        datestrformat = g_strdup ("%x");
    }
    else if (priv->date_use_short_format)
    {
        datestrformat = g_strdup ("%Y-%m-%d");
    }
    else
    {
        datestrformat = g_strdup ("%Y/%m/%d");
    }

    if (priv->need_show_week)
    {
        datestrformat = g_strdup_printf ("%s %%A", datestrformat);
    }

    datestrtmp = g_date_time_format (now, datestrformat);

    currentstrtmp = g_date_time_format (now, "%x\n%A");

    st_label_set_text (ST_LABEL (priv->actortimelabel), timestrtmp_with_seconds);
    st_label_set_text (ST_LABEL (priv->timeLabel), timestrtmp_with_seconds);
    st_label_set_text (ST_LABEL (priv->actordatelabel), datestrtmp);
    st_label_set_text (ST_LABEL (priv->currentDateBtn), currentstrtmp);

    if (TRUE == priv->need_show_date)
    {
        clutter_actor_show (CLUTTER_ACTOR (priv->actordatelabel));
    }
    else
    {
        clutter_actor_hide (CLUTTER_ACTOR (priv->actordatelabel));
    }
    cdos_applet_set_applet_tooltip (CDOS_APPLET (self), currentstrtmp);
    g_free (currentstrtmp);
    g_free (datestrtmp);
    g_free (datestrformat);
    g_free (datetimeformat_with_seconds);
    g_free (timestrtmp);
    g_free (timestrtmp_with_seconds);
    g_free (datetimeformat);
    g_date_time_unref (now);
}

static void
gsetting_value_changed (GSettings *settings, gchar *key, gpointer user_data)
{
    load_settings (APPLET_CALENDAR_MAIN (user_data));
    update_hover_background (APPLET_CALENDAR_MAIN (user_data), key);
    apply_settings (APPLET_CALENDAR_MAIN (user_data));
}

static void
update_hover_background (AppletCalendarMain *self, gchar *key)
{
    AppletCalendarMainPrivate *priv = self->priv;
    int index = (priv->is_use_24h << 2) | (priv->need_show_date << 1) | (priv->need_show_seconds << 0);
    if (priv->format_class_name != NULL)
    {
        st_widget_remove_style_class_name (ST_WIDGET (priv->actor), priv->format_class_name);
    }
    st_widget_add_style_class_name (ST_WIDGET (priv->actor), calendar_background[index]);
    priv->format_class_name = calendar_background[index];
}
