/**
 * Copyright (c) 2016-2018 CPU and Fundamental Software Research Center, CAS
 *
 * This software is published by the license of CPU-OS License，you can use and
 * distibuted this software under this License. See CPU-OS Liense for more detail.
 *
 * You should have receive a copy of CPU-OS License. If not, please contact us
 * by email <support_os@cpu-os.ac.cn>
 *
 **/
//
// Created by zhangtao on 18-1-25.
// Email: zhangtao@cpu-os.ac.cn
//
#include <gio/gio.h>
#include "applet-application-button.h"
#include "applet-menu-private.h"
#include "applet-menu.h"
#include "cdos-popup-menu-manager.h"
#include "cdos-popup-image-menu-item.h"
#include "cdos-utils-private.h"
#include "cdos-util.h"
#include "cdos-message-tray.h"
#include "cdos-panel.h"
#include "cdos-ui-plugin.h"
#include "applet-menu-helper.h"
#include "applet-favorites.h"
#include "cdos-drag-drop.h"
#include "applet-menu-types.h"

#define APPLET_APPLICATION_BUTTON_GET_PRIVATE(obj)      \
(G_TYPE_INSTANCE_GET_PRIVATE((obj), APPLET_TYPE_APPLICATION_BUTTON, AppletApplicationButtonPrivate))

G_DEFINE_TYPE(AppletApplicationButton, applet_application_button, APPLET_TYPE_MENU_BUTTON_BASE);

struct _AppletApplicationButtonPrivate
{
    gboolean                is_draggable;
    gboolean                dragging;
    gboolean                context_menu_enable;
    CdosApp                 *app;
    CdosPopupMenu           *context_menu;
    gulong                  context_menu_activate_id;
    CdosPopupMenuManager    *popup_menu_manager;
    GList                   *context_menu_items;
    ClutterDragAction       *drag_action;
    ClutterDropAction       *drop_action;

    gboolean                can_drop;
    ClutterDragAction       *dragging_action;
    gulong                  settings_launchers_changed_id;
    gboolean                enter_context_menu;
    guint                   context_menu_close_id;
    gulong                  applet_menu_id;
};

static void applet_application_button_set_app (AppletApplicationButton *self, CdosApp *app, gint icon_size);
static void button_on_activate (AppletApplicationButton *self, gpointer user_data);
static gboolean button_on_context_menu (ClutterActor *actor, ClutterEvent *event, gpointer user_data);
static void applet_application_button_add_context_menu_items (AppletApplicationButton *self);
static void button_on_context_menu_open_state_changed (CdosPopupMenuBase *popup_menu_base,
                                                      gboolean state,
                                                      AppletApplicationButton *self);
static void context_menu_item_on_favorite_activate (CdosPopupBaseMenuItem *item,
				                                ClutterEvent          *event,
                                                gboolean               keepMenu,
                                                AppletApplicationButton *self);
static void context_menu_item_on_desktop_activate (CdosPopupBaseMenuItem *item,
				                                ClutterEvent          *event,
                                                gboolean               keepMenu,
                                                AppletApplicationButton *self);
static void context_menu_item_on_panel_activate (CdosPopupBaseMenuItem *item,
				                                ClutterEvent          *event,
                                                gboolean               keepMenu,
                                                AppletApplicationButton *self);

static void context_menu_item_on_uninstall_activate (CdosPopupBaseMenuItem *item,
                                                ClutterEvent          *event,
                                                gboolean               keepMenu,
                                                AppletApplicationButton *self);

//获取右键菜单项的icon和label文本
static void context_menu_update_favorite_item (AppletApplicationButton *self, CdosPopupImageMenuItem *item);
static void context_menu_update_desktop_item (AppletApplicationButton *self, CdosPopupImageMenuItem *item);
static void context_menu_update_panel_item (AppletApplicationButton *self, CdosPopupImageMenuItem *item);
static void context_menu_item_update_icon_and_label (CdosPopupImageMenuItem *item,
                                                    const gchar *icon_name,
                                                    const gchar *label_text);
static void context_menu_item_on_panel_launchers_changed (GSettings *settings,
                                                         gchar *key,
                                                         AppletApplicationButton *self);
static gboolean check_desktop_contained_app (AppletApplicationButton *self);
static gboolean
action_on_can_drop (ClutterDropAction *action,
               ClutterActor      *actor,
               gfloat             event_x,
               gfloat             event_y,
               gpointer           user_data);
static void
action_on_drop (ClutterDropAction *action,
               ClutterActor      *actor,
               gfloat             event_x,
               gfloat             event_y,
               gpointer           user_data);
static void
action_on_drop_cancel (ClutterDropAction *action,
               ClutterActor      *actor,
               gfloat             event_x,
               gfloat             event_y,
               gpointer           user_data);
static void
action_on_drag_begin (ClutterDragAction  *action,
                    ClutterActor *actor,
                    gfloat event_x,
                    gfloat event_y,
                    ClutterModifierType modifiers,
                    gpointer user_data);

static void
action_on_drag_motion (ClutterDragAction  *action,
                    ClutterActor *actor,
                    gfloat motion_x,
                    gfloat motion_y,
                    gpointer user_data);

static void
action_on_drag_end (ClutterDragAction  *action,
                    ClutterActor *actor,
                    gfloat event_x,
                    gfloat event_y,
                    ClutterModifierType modifiers,
                    gpointer user_data);

static void
button_on_focus_out (ClutterActor *actor, gpointer user_data);


static void
button_on_applet_menu_open_state_changed (CdosPopupMenuBase *popup_menu_base,
                                          gboolean state,
                                          AppletApplicationButton *self)
{
    AppletApplicationButtonPrivate *priv = self->priv;
    if (state == FALSE)
    {
        cdos_popup_menu_close (CDOS_POPUP_MENU_BASE (priv->context_menu), FALSE);
    }
}

static void
applet_application_button_init (AppletApplicationButton *self)
{
    AppletApplicationButtonPrivate *priv;
    CdosBoxPointer *bp = NULL;
    priv = self->priv = APPLET_APPLICATION_BUTTON_GET_PRIVATE (self);

    applet_menu_button_base_set_layout_mode ((AppletMenuButtonBase *)self, BUTTON_LAYOUT_V);
    st_widget_set_style_class_name ((StWidget *)self, "menu-app-button");

    priv->context_menu = cdos_popup_menu_new ((ClutterActor *)self);
    priv->popup_menu_manager = cdos_popup_menu_manager_new ((ClutterActor *)self);
    cdos_popup_menu_manager_add_menu (priv->popup_menu_manager, CDOS_POPUP_MENU_BASE(priv->context_menu), -1);
    applet_application_button_set_context_menu_enable (self, TRUE);

    bp = cdos_popup_menu_get_box_pointer (priv->context_menu);
    st_widget_add_style_class_name ((StWidget *)bp, "menu-app-button-contextmenu");
    g_signal_connect (priv->context_menu,
                     "open-state-changed",
                     G_CALLBACK (button_on_context_menu_open_state_changed),
                     self);

    AppletMenu* applet_menu = applet_menu_menu_get_default ();
    CdosPopupMenu *popup_menu = cdos_applet_get_popup_menu ((CdosApplet *)applet_menu);
    priv->applet_menu_id = \
        g_signal_connect (popup_menu,
                        "open-state-changed",
                        G_CALLBACK (button_on_applet_menu_open_state_changed),
                        self);

    g_signal_connect (self,
                      "key-focus-out",
                      G_CALLBACK (button_on_focus_out),
                      self);
    priv->drag_action = cdos_drag_action_new ((ClutterActor *)self);
    g_signal_connect (priv->drag_action,
                     "drag-begin",
                     G_CALLBACK (action_on_drag_begin),
                     self);
    /*g_signal_connect (priv->drag_action,
                     "drag-motion",
                     G_CALLBACK (action_on_drag_motion),
                     self);*/
    g_signal_connect (priv->drag_action,
                     "drag-end",
                     G_CALLBACK (action_on_drag_end),
                     self);
    //使用cdos_drop_action_new，会导致组容器无法接收drop相关信号；
    //priv->drop_action = (ClutterDropAction *) cdos_drop_action_new ((ClutterActor *)self);
    priv->drop_action = (ClutterDropAction *) clutter_drop_action_new ();
    clutter_actor_add_action ((ClutterActor *)self, CLUTTER_ACTION(priv->drop_action));
    applet_application_button_set_can_drop (self, FALSE);
    g_signal_connect (priv->drop_action,
                    "can-drop",
                    G_CALLBACK (action_on_can_drop),
                    self);
    g_signal_connect (priv->drop_action,
                    "drop",
                    G_CALLBACK (action_on_drop),
                    self);
    g_signal_connect (priv->drop_action,
                    "drop-cancel",
                    G_CALLBACK (action_on_drop_cancel),
                    self);
    //拖拽会影响触摸屏的手势控制。
    applet_application_button_set_draggable (self, FALSE);
    priv->dragging = FALSE;
}

static void
applet_application_button_dispose (GObject *object)
{
    AppletApplicationButton *self = APPLET_APPLICATION_BUTTON (object);
    AppletApplicationButtonPrivate *priv = self->priv;

    AppletMenu* applet_menu = applet_menu_menu_get_default ();
    CdosPopupMenu *popup_menu = cdos_applet_get_popup_menu ((CdosApplet *)applet_menu);
    if (priv->applet_menu_id)
    {
        g_signal_handler_disconnect (popup_menu, priv->applet_menu_id);
        priv->applet_menu_id = 0;
    }

    if (priv->context_menu_close_id)
    {
        g_source_remove (priv->context_menu_close_id);
        priv->context_menu_close_id = 0;
    }
    if (clutter_actor_has_key_focus ((ClutterActor *)self))
    {
        applet_menu_grab_key_focus_main_box ();
    }
    if (priv->dragging_action && priv->dragging)
    {
        action_on_drag_end (priv->drag_action, (ClutterActor *)self, 0, 0, CLUTTER_MODIFIER_MASK, self);
    }
    clutter_actor_clear_actions ((ClutterActor *)self);
    priv->drag_action = NULL;
    priv->drop_action = NULL;
    if (priv->context_menu_items)
    {
        g_list_free (priv->context_menu_items);
        priv->context_menu_items = NULL;
    }
    // After removed menu, The manager will release it and it's items, so we do't need release them again;
    if (priv->popup_menu_manager)
    {
        cdos_popup_menu_manager_remove_menu (priv->popup_menu_manager, CDOS_POPUP_MENU_BASE(priv->context_menu));
        g_clear_object (&priv->popup_menu_manager);
    }
    if (priv->context_menu_activate_id > 0)
    {
        g_signal_handler_disconnect (self, priv->context_menu_activate_id);
        priv->context_menu_activate_id = 0;
    }
    if (priv->settings_launchers_changed_id)
    {
    	CdosGlobal *global = cdos_global_get();
        GSettings *settings = cdos_global_get_settings (global);
        g_signal_handler_disconnect (settings, priv->settings_launchers_changed_id);
        priv->settings_launchers_changed_id = 0;
    }


    G_OBJECT_CLASS (applet_application_button_parent_class)->dispose (object);
}

static void
applet_application_button_finalize (GObject *object)
{
    G_OBJECT_CLASS (applet_application_button_parent_class)->finalize (object);
}

static void
applet_application_button_class_init (AppletApplicationButtonClass *klass)
{
    GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
    g_type_class_add_private (klass, sizeof (AppletApplicationButtonPrivate));

    gobject_class->dispose = applet_application_button_dispose;
    gobject_class->finalize = applet_application_button_finalize;
}

static void
applet_application_button_set_app (AppletApplicationButton *self, CdosApp *app, gint icon_size)
{
    AppletApplicationButtonPrivate *priv = self->priv;
    priv->app = app;
    icon_size = icon_size > 0 ? icon_size : DEFAULT_ICON_SIZE;
    ClutterActor *icon = cdos_app_create_icon_texture (priv->app, icon_size);
    const char *name = cdos_app_get_name (priv->app);
    applet_menu_button_base_set_icon ((AppletMenuButtonBase *)self, icon);
    applet_menu_button_base_set_label_text ((AppletMenuButtonBase *)self, name);
    applet_menu_button_base_set_tooltip ((AppletMenuButtonBase *)self, name);
    applet_application_button_add_context_menu_items (self);
    g_signal_connect (self,
                      "activate",
                      G_CALLBACK (button_on_activate),
                      NULL);
}

static void
button_on_activate (AppletApplicationButton *self, gpointer user_data)
{
    AppletApplicationButtonPrivate *priv = self->priv;
    if (!CDOS_POPUP_MENU_BASE (priv->context_menu)->is_open)
    {
        cdos_app_open_new_window (priv->app, -1);
        applet_menu_close ();
    }
}

static gboolean
button_on_context_menu (ClutterActor *actor, ClutterEvent *event, gpointer user_data)
{
    AppletApplicationButton *self = user_data;
    AppletApplicationButtonPrivate *priv = self->priv;

    if (priv->context_menu_enable)
    {
        guint32 button = clutter_event_get_button (event);
        ClutterModifierType state = clutter_event_get_state (event);
        //clutter_actor_grab_key_focus ((ClutterActor *)self);
        if ((state & CLUTTER_BUTTON1_MASK) || button == 1)
        {
            if (CDOS_POPUP_MENU_BASE (priv->context_menu)->is_open)
            {
                cdos_popup_menu_base_toggle (CDOS_POPUP_MENU_BASE (priv->context_menu));
            }
        }
        else if ((state & CLUTTER_BUTTON2_MASK) || button == 2)
        {
            cdos_popup_menu_close (CDOS_POPUP_MENU_BASE (priv->context_menu), FALSE);
        }
        else if ((state & CLUTTER_BUTTON3_MASK || button == 3))
        {
            cdos_popup_menu_base_toggle (CDOS_POPUP_MENU_BASE (priv->context_menu));
        }
    }
    return FALSE;
}

void
applet_application_button_add_context_menu_item (AppletApplicationButton *self, const gchar *text, gint index,  GCallback  activate_callback,  gpointer user_data)
{
    g_return_if_fail (APPLET_IS_APPLICATION_BUTTON (self));
    AppletApplicationButtonPrivate *priv = self->priv;
    CdosPopupImageMenuItem *item;

    //for clear record;
    item = cdos_popup_image_menu_item_new ();
    context_menu_item_update_icon_and_label (item,  NULL, text);
    g_signal_connect ((CdosPopupBaseMenuItem *)item,
                    "activate",
                    G_CALLBACK (activate_callback),
                    user_data);
    cdos_popup_menu_base_add_menu_item ((CdosPopupMenuBase *)priv->context_menu, (CdosPopupBaseMenuItem *)item, index);
    g_object_set (item, 
                "x-align", CLUTTER_ACTOR_ALIGN_FILL, 
                "x-expand", TRUE,
                NULL);
    //clutter_actor_show ((ClutterActor *)item);
   priv->context_menu_items = g_list_insert  (priv->context_menu_items, item, index);
}

static void
applet_application_button_add_context_menu_items (AppletApplicationButton *self)
{
    AppletApplicationButtonPrivate *priv = self->priv;
    CdosPopupImageMenuItem *item;
    CdosGlobal *global = cdos_global_get();
    GSettings *settings = cdos_global_get_settings (global);

    //for panel;
    item = cdos_popup_image_menu_item_new ();
    context_menu_update_panel_item (self, item);
    g_signal_connect ((CdosPopupBaseMenuItem *)item,
                     "activate",
                     G_CALLBACK (context_menu_item_on_panel_activate),
                     self);
    cdos_popup_menu_base_add_menu_item ((CdosPopupMenuBase *)priv->context_menu, (CdosPopupBaseMenuItem *)item, -1);
    g_object_set (item, 
                  "x-align", CLUTTER_ACTOR_ALIGN_FILL, 
                  "x-expand", TRUE,
                  NULL);
    priv->context_menu_items = g_list_append (priv->context_menu_items, item);

    /* //for favorites;
    item = cdos_popup_image_menu_item_new ();
    context_menu_update_favorite_item (self, item);
    g_signal_connect ((CdosPopupBaseMenuItem *)item,
                     "activate",
                     G_CALLBACK (context_menu_item_on_favorite_activate),
                     self);
    cdos_popup_menu_base_add_menu_item ((CdosPopupMenuBase *)priv->context_menu, (CdosPopupBaseMenuItem *)item, -1);
    g_object_set (item, 
                  "x-align", CLUTTER_ACTOR_ALIGN_FILL, 
                  "x-expand", TRUE,
                  NULL);
    priv->context_menu_items = g_list_append (priv->context_menu_items, item); */

    //for desktop;
    item = cdos_popup_image_menu_item_new ();
    context_menu_update_desktop_item (self, item);
    g_signal_connect ((CdosPopupBaseMenuItem *)item,
                     "activate",
                     G_CALLBACK (context_menu_item_on_desktop_activate),
                     self);
    cdos_popup_menu_base_add_menu_item ((CdosPopupMenuBase *)priv->context_menu, (CdosPopupBaseMenuItem *)item, -1);
    g_object_set (item, 
                  "x-align", CLUTTER_ACTOR_ALIGN_FILL, 
                  "x-expand", TRUE,
                  NULL);
    priv->context_menu_items = g_list_append (priv->context_menu_items, item);

    //for uninstall;
     item = cdos_popup_image_menu_item_new ();
    context_menu_item_update_icon_and_label (item, "found-uninstall", _("Uninstall"));
    g_signal_connect ((CdosPopupBaseMenuItem *)item,
                     "activate",
                     G_CALLBACK (context_menu_item_on_uninstall_activate),
                     self);
    cdos_popup_menu_base_add_menu_item ((CdosPopupMenuBase *)priv->context_menu, (CdosPopupBaseMenuItem *)item, -1);
    g_object_set (item, 
                  "x-align", CLUTTER_ACTOR_ALIGN_FILL, 
                  "x-expand", TRUE,
                  NULL);
    priv->context_menu_items = g_list_append (priv->context_menu_items, item);
    
    priv->settings_launchers_changed_id = \
    g_signal_connect (settings,
                     "changed::panel-launchers",
                     G_CALLBACK (context_menu_item_on_panel_launchers_changed),
                     self); 
}

static void 
button_on_context_menu_position (AppletApplicationButton *self, MetaDisplay *display, gfloat context_h)
{
    AppletApplicationButtonPrivate *priv = self->priv;
    gint mx, my;
    gfloat panel_height;
    ClutterActorBox actor_box;
    meta_display_get_size (display, &mx, &my);
    cdos_base_util_get_transformed_allocation ((ClutterActor *)self, &actor_box);
    CdosPanel *panel = cdos_ui_plugin_get_panel ();
    panel_height = cdos_panel_get_height (panel);
    if (actor_box.y2 + context_h + panel_height + 2.0 > my)
        cdos_popup_menu_set_arrow_side (priv->context_menu, ST_SIDE_BOTTOM);
    else
        cdos_popup_menu_set_arrow_side (priv->context_menu, ST_SIDE_TOP);
}

static void
button_on_context_menu_open_state_changed (CdosPopupMenuBase *popup_menu_base,
                                          gboolean state,
                                          AppletApplicationButton *self)
{
    AppletApplicationButtonPrivate *priv = self->priv;
    if (state)
    {
        gint  x, y;
        gfloat px, py, menu_box_h;
        gfloat popup_menu_h, context_h;
        ClutterActorBox context_box;
        StBoxLayout *context_menu_box = cdos_popup_menu_base_get_box (priv->context_menu);
    	CdosGlobal *global = cdos_global_get();
        MetaDisplay *display = cdos_global_get_display (global);
        MetaCursorTracker *cursor_tracker = meta_cursor_tracker_get_for_display (display);
        meta_cursor_tracker_get_pointer (cursor_tracker, &x, &y, NULL);

        CdosBoxPointer  *box_pointer = cdos_popup_menu_get_box_pointer (priv->context_menu);
        StBoxLayout  *menu_box = applet_menu_get_main_box ();
        menu_box_h = clutter_actor_get_height ((ClutterActor *)menu_box);
        popup_menu_h = clutter_actor_get_height ((ClutterActor *)context_menu_box);
        clutter_actor_get_allocation_box ((ClutterActor *)box_pointer, &context_box);
        context_h = context_box.y2 - context_box.y1;
        button_on_context_menu_position (self, display, context_h);

        px = (float)x;
        if (y + popup_menu_h + 5.0 > menu_box_h)
        {
            py = y - popup_menu_h;
        } else {
            py = (float)y;
        }

        clutter_actor_set_position ((ClutterActor *)box_pointer, px, py);
        CdosPopupImageMenuItem *item_desktop = g_list_nth_data (priv->context_menu_items, 1);
        context_menu_update_desktop_item (self, item_desktop);
        //CdosPopupImageMenuItem *item_favorites = g_list_nth_data (priv->context_menu_items, 0);
        //context_menu_update_favorite_item (self, item_favorites);

        /* GList *items = cdos_popup_menu_base_get_menu_items (popup_menu_base);
        ClutterActor *focus = (ClutterActor *)items->data;
        clutter_actor_grab_key_focus (focus); */
        applet_menu_can_close (FALSE);
        applet_menu_button_base_set_tooltip_state ((AppletMenuButtonBase *)self, FALSE);
    } else {
        applet_menu_grab_key_focus_main_box ();
        applet_menu_can_close (TRUE);
        applet_menu_button_base_set_tooltip_state ((AppletMenuButtonBase *)self, TRUE);
    }
    priv->enter_context_menu = FALSE;
}

static void
context_menu_item_on_favorite_activate (CdosPopupBaseMenuItem *item,
                                        ClutterEvent          *event,
                                        gboolean               keepMenu,
                                        AppletApplicationButton *self)
{
    AppletApplicationButtonPrivate *priv = self->priv;
    AppletFavorites *favorites = applet_favorites_get_default ();
    gboolean is_exisit;
    gboolean result;
    GroupData *group;
    const gchar *app_id = cdos_app_get_id (priv->app);
    is_exisit = applet_favorites_app_query_exists (favorites, priv->app);
    if (is_exisit)
    {
        result = applet_favorites_remove_app (favorites, NULL, app_id);
    } else {
        group = applet_favorites_get_default_group (favorites);
        result = applet_favorites_add_app (favorites, group->group_name, app_id, TRUE);
    }
}

static void
context_menu_item_on_desktop_activate (CdosPopupBaseMenuItem *item,
                                       ClutterEvent          *event,
                                       gboolean               keepMenu,
                                       AppletApplicationButton *self)
{
    AppletApplicationButtonPrivate *priv = self->priv;
    gchar *desktop_dir;
    const gchar *dekstop_path;
    GFile *desktop_file;
    const gchar *app_id;
    gchar *app_path;
    GFile *app_file;
    gboolean result;
    gchar *chmod;
    gboolean is_exist;
    desktop_dir = applet_helper_get_xdg_desktop_dir ();
    dekstop_path =  g_desktop_app_info_get_filename (cdos_app_get_app_info (priv->app));
    desktop_file = g_file_new_for_path (dekstop_path);

    app_id = cdos_app_get_id (priv->app);
    app_path = g_strconcat(desktop_dir, "/", app_id, NULL);
    app_file = g_file_new_for_path (app_path);
    is_exist = g_file_query_exists (app_file, NULL);
    if (is_exist)
    {
        result = g_file_delete(app_file, NULL, NULL);
    } else {
        result = g_file_copy (desktop_file, app_file, G_FILE_COPY_OVERWRITE, NULL, NULL, NULL, NULL);
        if (result)
        {
            chmod = g_strconcat ("chmod +x ", desktop_dir, "/", app_id, NULL);
            cdos_util_spawn_command_line (chmod);
            g_free (chmod);
        }
    }
    g_object_unref (app_file);
    g_free (app_path);
    g_object_unref (desktop_file);
    g_free (desktop_dir);

}
static void
context_menu_item_on_panel_activate (CdosPopupBaseMenuItem *item,
                                    ClutterEvent          *event,
                                    gboolean               keepMenu,
                                    AppletApplicationButton *self)
{
    AppletApplicationButtonPrivate *priv = self->priv;
    const gchar *app_id = cdos_app_get_id (priv->app);

    CdosGlobal *global = cdos_global_get();
    GSettings *settings = cdos_global_get_settings (global);
    gchar **desktop_files = g_settings_get_strv (settings, CDOS_PANEL_LAUNCHERS);
    guint max_apps = g_settings_get_int (settings, CDOS_PANEL_MAX_LAUNCHERS);
    gchar **desktop_files_new = NULL;
    gboolean is_add = TRUE;
    gint i;
    for (i = 0; desktop_files[i] != NULL; i++)
    {
        if (g_strcmp0 (app_id, desktop_files[i]) == 0)
        {
            is_add = FALSE;
            break;
        }
    }
    if (!is_add)
    {
        desktop_files_new = cdos_strv_elem_del_by_str (desktop_files, app_id);
        g_settings_set_strv (settings, CDOS_PANEL_LAUNCHERS, (const gchar *const *)desktop_files_new);
    } else {
        if (desktop_files && g_strv_length (desktop_files) < max_apps)
        {
            desktop_files_new = cdos_strv_elem_add (desktop_files, app_id);
            g_settings_set_strv (settings, CDOS_PANEL_LAUNCHERS, (const gchar *const *)desktop_files_new);
        } else {
                cdos_notify_info (NULL, _("Too many applications added"),_("The max number is 8"));
        }
    }

    if (desktop_files_new != NULL)
    {
        //notice: you cann't use  "g_strfreev"  to free "desktop_files_new",please see the return of the function "g_ptr_array_free";
        g_free(desktop_files_new);
    }
    g_strfreev (desktop_files);
}

static void
context_menu_update_favorite_item (AppletApplicationButton *self, CdosPopupImageMenuItem *item)
{
    AppletApplicationButtonPrivate *priv = self->priv;
    if (!priv->app)
    {
        return ;
    }
    const gchar *icon_name = NULL;
    const gchar *label_text ;
    AppletFavorites *favorites = applet_favorites_get_default ();
    gboolean is_exisit = applet_favorites_app_query_exists (favorites, priv->app);

    if (is_exisit)
    {
        icon_name = "application-remove-from-favorites";
        label_text = _("Remove from quick operation");
    } else {
        icon_name = "application-add-to-favorites";
        label_text = _("Add to quick operation");
    }
    context_menu_item_update_icon_and_label (item, icon_name, label_text);
}

static void
context_menu_item_on_uninstall_activate (CdosPopupBaseMenuItem *item,
                                       ClutterEvent          *event,
                                       gboolean               keepMenu,
                                       AppletApplicationButton *self)
{
    AppletApplicationButtonPrivate *priv = self->priv;
    if (!priv->app)
    {
        return ;
    }
    const gchar *app_id = cdos_app_get_id (priv->app);
    char *cmd_line = g_strdup_printf ("%s %s", "/usr/bin/apt-tools-remove", app_id);
    cdos_util_spawn_command_line (cmd_line);
    g_free (cmd_line);
}

static void
context_menu_update_desktop_item (AppletApplicationButton *self, CdosPopupImageMenuItem *item)
{
    AppletApplicationButtonPrivate *priv = self->priv;
    if (!priv->app)
    {
        return ;
    }
    const gchar *icon_name = NULL;
    const gchar *label_text;
    gboolean is_exisit = check_desktop_contained_app (self);
    if (is_exisit)
    {
        icon_name = "application-remove-from-desktop";
        label_text = _("Remove from desktop");
    } else {
        icon_name = "application-add-to-desktop";
        label_text = _("Add to desktop");
    }
    context_menu_item_update_icon_and_label (item, icon_name, label_text);
}

static void
context_menu_update_panel_item (AppletApplicationButton *self, CdosPopupImageMenuItem *item)
{
    AppletApplicationButtonPrivate *priv = self->priv;
    if (!priv->app)
    {
    	return ;
    }
    const gchar *app_id = cdos_app_get_id (priv->app);
    const gchar *icon_name = NULL;
    const gchar *label_text;
    CdosGlobal *global = cdos_global_get();
    GSettings *global_settings = cdos_global_get_settings (global);
    gchar **launchers = g_settings_get_strv (global_settings, CDOS_PANEL_LAUNCHERS);
    gint i ;
    gboolean check = FALSE;
    if (launchers)
    {
        for (i = 0; launchers[i] != NULL; i++){
            gchar *launcher = launchers[i];
            if (g_strcmp0 (launcher, app_id) == 0)
            {
                check = TRUE;
                break;
            }
        }
        g_strfreev(launchers);
    }
    if (check)
    {
        icon_name = "found-unpin";
        label_text = _("Remove from panel");
    } else {
        icon_name = "found-pin";
        label_text = _("Add to panel");
    }
    context_menu_item_update_icon_and_label (item, icon_name, label_text);
}

static void
context_menu_item_update_icon_and_label (CdosPopupImageMenuItem *item,
                                        const gchar *icon_name,
                                        const gchar *label_text)
{
    //暂时不显示图标
    //if (icon_name)
    //    cdos_popup_image_menu_item_set_icon_by_gicon (item, icon_name);
    cdos_popup_image_menu_item_set_label (item, label_text);
}

static void
context_menu_item_on_panel_launchers_changed (GSettings *settings,
                                              gchar *key,
                                              AppletApplicationButton *self)
{
    if (g_strcmp0 (key, CDOS_PANEL_LAUNCHERS) == 0)
    {
        AppletApplicationButtonPrivate *priv = self->priv;
        CdosPopupImageMenuItem *item = g_list_nth_data (priv->context_menu_items, 0);
        context_menu_update_panel_item (self, item);
    }
}

static gboolean
check_desktop_contained_app (AppletApplicationButton *self)
{
    AppletApplicationButtonPrivate *priv = self->priv;
    gchar *desktop_dir;
    const gchar *app_id;
    gchar *app_path;
    GFile *app_file;
    gboolean is_exist = FALSE;

    desktop_dir = applet_helper_get_xdg_desktop_dir ();
    app_id = cdos_app_get_id (priv->app);
    app_path = g_strconcat(desktop_dir, "/", app_id, NULL);
    app_file = g_file_new_for_path (app_path);
    is_exist = g_file_query_exists (app_file, NULL);
    g_object_unref (app_file);
    g_free (app_path);
    g_free (desktop_dir);

    return is_exist;
}

static void
button_on_focus_out (ClutterActor *actor, gpointer user_data)
{
    AppletApplicationButton *self = user_data;
    AppletApplicationButtonPrivate *priv = self->priv;
    st_widget_hide_tooltip ((StWidget *)self);
    //st_widget_set_style_pseudo_class ((StWidget *)self, NULL);
    st_widget_add_style_pseudo_class ((StWidget *)self, "focus");
}

static gboolean
context_menu_close_delay (AppletApplicationButton *self)
{
    AppletApplicationButtonPrivate *priv = self->priv;
    if (!priv->enter_context_menu)
    {
        cdos_popup_menu_close (CDOS_POPUP_MENU_BASE (priv->context_menu), FALSE);
    }
    priv->context_menu_close_id = 0;
    return FALSE;
}

static gboolean
button_on_leave (ClutterActor *actor, ClutterEvent *event, gpointer user_data)
{
    AppletApplicationButton *self = APPLET_APPLICATION_BUTTON (user_data);
    AppletApplicationButtonPrivate *priv = self->priv;
    if (priv->context_menu_close_id)
    {
        g_source_remove (priv->context_menu_close_id);
        priv->context_menu_close_id = 0;
    }
    priv->context_menu_close_id = g_timeout_add (300, context_menu_close_delay, self);
    return FALSE;
}

static gboolean
context_menu_on_enter (ClutterActor *actor, ClutterEvent *event, gpointer user_data)
{
    AppletApplicationButton *self = APPLET_APPLICATION_BUTTON (user_data);
    AppletApplicationButtonPrivate *priv = self->priv;
    priv->enter_context_menu = TRUE;
    return FALSE;
}

static gboolean
context_menu_on_leave (ClutterActor *actor, ClutterEvent *event, gpointer user_data)
{
    AppletApplicationButton *self = APPLET_APPLICATION_BUTTON (user_data);
    AppletApplicationButtonPrivate *priv = self->priv;
    priv->enter_context_menu = FALSE;
    return FALSE;
}

static void
action_on_drag_begin (ClutterDragAction  *action,
                    ClutterActor *actor,
                    gfloat event_x,
                    gfloat event_y,
                    ClutterModifierType modifiers,
                    gpointer user_data)
{
    AppletApplicationButton *self = user_data;
    AppletApplicationButtonPrivate *priv = self->priv;
    const char *app_id = cdos_app_get_id (priv->app);
    g_object_set_data_full ((GObject *)priv->drag_action, "desktop-id", g_strdup (app_id), g_free);
    applet_menu_set_display_cursor (META_CURSOR_DND_IN_DRAG);
    priv->dragging = TRUE;
    st_widget_set_track_hover ((StWidget *)self, FALSE);
    clutter_actor_grab_key_focus ((ClutterActor *)self);
    applet_menu_button_base_set_tooltip_state ((AppletMenuButtonBase *)self, FALSE);
}

static void
copy_app_to_desktop (AppletApplicationButton *self)
{
    AppletApplicationButtonPrivate *priv = self->priv;
    gchar *desktop_dir;
    const gchar *dekstop_path;
    GFile *desktop_file;
    const gchar *app_id;
    gchar *app_path;
    GFile *app_file;
    gboolean result;
    gchar *chmod;
    gboolean is_exist;
    desktop_dir = applet_helper_get_xdg_desktop_dir ();
    dekstop_path =  g_desktop_app_info_get_filename (cdos_app_get_app_info (priv->app));
    desktop_file = g_file_new_for_path (dekstop_path);

    app_id = cdos_app_get_id (priv->app);
    app_path = g_strconcat(desktop_dir, "/", app_id, NULL);
    g_debug ("desktop dir: %s\n -- desktop path:%s, app_path %s\n", desktop_dir, dekstop_path, app_path);
    app_file = g_file_new_for_path (app_path);
    is_exist = g_file_query_exists (app_file, NULL);
    if (!is_exist)
    {
        g_debug ("app exist\n");
        result = g_file_copy (desktop_file, app_file, G_FILE_COPY_OVERWRITE, NULL, NULL, NULL, NULL);
        if (result)
        {
            chmod = g_strconcat ("chmod +x ", desktop_dir, "/", app_id, NULL);
            cdos_util_spawn_command_line (chmod);
            g_free (chmod);
        }
    }
    g_object_unref (app_file);
    g_free (app_path);
    g_object_unref (desktop_file);
    g_free (desktop_dir);
}

static gboolean 
is_showing_desktop (void)
{
    MetaWorkspaceManager *workspace_manager;
    CdosGlobal *global;
    MetaDisplay *display;
    MetaWorkspace *current_workspace;

    global = cdos_global_get ();
    display = cdos_global_get_display (global);
    workspace_manager = meta_display_get_workspace_manager (display);
    current_workspace = meta_workspace_manager_get_active_workspace (workspace_manager);
    return meta_workspace_get_showing_desktop (current_workspace);
}

static void
action_on_drag_motion (ClutterDragAction *action,
                    ClutterActor *actor,
                    gfloat delta_x,
                    gfloat delta_y,
                    gpointer user_data)
{
    AppletApplicationButton *self = user_data;
    gfloat motion_x, motion_y;
    guint screen_width, screen_height;
    MetaWorkspaceManager *workspace_manager;

    clutter_drag_action_get_motion_coords (action, &motion_x, &motion_y);

    CdosGlobal *global = cdos_global_get ();
    guint32 time = cdos_global_get_current_time (global);
    MetaDisplay *display = cdos_global_get_display (global);
    workspace_manager = meta_display_get_workspace_manager (display);
    meta_display_get_size (display, &screen_width, &screen_height);

    StBoxLayout  *menu_box = applet_menu_get_main_box ();
    guint menu_box_w = clutter_actor_get_width ((ClutterActor *)menu_box);

    gboolean is_show_desktop = is_showing_desktop ();
    if (!is_show_desktop && motion_y < screen_height - 50 && motion_x > menu_box_w){
        meta_workspace_manager_show_desktop (workspace_manager, time);
    }else if (motion_y > screen_height - 50 || motion_x < menu_box_w) {
        meta_workspace_manager_unshow_desktop (workspace_manager);
    }
}

static void
action_on_drag_end (ClutterDragAction  *action,
                    ClutterActor *actor,
                    gfloat event_x,
                    gfloat event_y,
                    ClutterModifierType modifiers,
                    gpointer user_data)
{
    AppletApplicationButton *self = user_data;
    AppletApplicationButtonPrivate *priv = self->priv;
    applet_menu_set_display_cursor (META_CURSOR_DEFAULT);
    st_widget_set_track_hover ((StWidget *)self, TRUE);
    applet_menu_button_base_set_tooltip_state ((AppletMenuButtonBase *)self, TRUE);

    applet_menu_grab_key_focus_main_box ();

    guint screen_width, screen_height;
    CdosGlobal *global = cdos_global_get ();
    MetaDisplay *display = cdos_global_get_display (global);
    meta_display_get_size (display, &screen_width, &screen_height);

    StBoxLayout  *menu_box = applet_menu_get_main_box ();
    guint menu_box_w = clutter_actor_get_width ((ClutterActor *)menu_box);

    ClutterActor *handle = clutter_drag_action_get_drag_handle (action);
    if (handle)
    {
        clutter_drag_action_set_drag_handle (action, NULL);
        clutter_actor_destroy (handle);
    }

    if (event_y < screen_height - 50 && event_x > menu_box_w){
        // 拷贝到桌面
        copy_app_to_desktop (self);
        // 关闭菜单？
        // priv->popup_menu_manager
        // cdos_popup_menu_manager_close_menu
    }
    priv->dragging = FALSE;
}

static gboolean
action_on_can_drop (ClutterDropAction *action,
               ClutterActor      *actor,
               gfloat             event_x,
               gfloat             event_y,
               gpointer           user_data)
{
    AppletApplicationButton *self = user_data;
    AppletApplicationButtonPrivate *priv = self->priv;
    if (priv->can_drop == FALSE)
        return FALSE;

    ClutterDragAction *dragging_action;
    gchar             *desktop_id = NULL;

    priv->dragging_action = NULL;
    dragging_action =  cdos_get_current_drag_action ();
    if (dragging_action)
    {
        desktop_id = g_object_get_data (dragging_action, "desktop-id");
    }
    //g_debug ("\n%s:%s:%d:===>desktop_id = %s\n", __FILE__, __func__, __LINE__, desktop_id);
    if (desktop_id)
    {
        priv->dragging_action = dragging_action;
        return  TRUE;
    }
    return FALSE;
}

static void
action_on_drop (ClutterDropAction *action,
               ClutterActor      *actor,
               gfloat             event_x,
               gfloat             event_y,
               gpointer           user_data)
{
    AppletApplicationButton *self = user_data;
    AppletApplicationButtonPrivate *priv = self->priv;
    if (!priv->dragging_action)
    {
        return ;
    }
    gchar *desktop_id = NULL;
    const gchar *target_app_id = NULL;
    AppletFavorites *favorites;
    MoveMode mode = MOVE_BEFORE;
    desktop_id = g_object_get_data (priv->dragging_action, "desktop-id");
    //g_debug ("\n%s:%s:%d:===>desktop_id = %s\n", __FILE__, __func__, __LINE__, desktop_id);
    if (desktop_id)
    {
        favorites = applet_favorites_get_default ();
        target_app_id = cdos_app_get_id (priv->app);
        applet_favorites_app_move (favorites, desktop_id, target_app_id, mode);
    }
    st_widget_set_track_hover ((StWidget *)self, TRUE);
    priv->dragging_action = NULL;
}

static void
action_on_drop_cancel (ClutterDropAction *action,
               ClutterActor      *actor,
               gfloat             event_x,
               gfloat             event_y,
               gpointer           user_data)
{
    AppletApplicationButton *self = user_data;
    //AppletApplicationButtonPrivate *priv = self->priv;
    st_widget_set_track_hover ((StWidget *)self, TRUE);
}

AppletApplicationButton *
applet_application_button_new (CdosApp *app, gint icon_size)
{
    g_return_val_if_fail (CDOS_IS_APP (app), NULL);
    AppletApplicationButton *application_button;
    application_button = g_object_new (APPLET_TYPE_APPLICATION_BUTTON, NULL);
    applet_application_button_set_app (application_button, app, icon_size);

    return application_button;
}

void
applet_application_button_set_draggable (AppletApplicationButton *self, gboolean is_draggable)
{
    g_return_if_fail (APPLET_IS_APPLICATION_BUTTON (self));
    AppletApplicationButtonPrivate *priv = self->priv;
    priv->is_draggable = is_draggable;
    clutter_actor_meta_set_enabled ((ClutterActorMeta *)priv->drag_action, priv->is_draggable);
}

gboolean
applet_application_button_get_draggable (AppletApplicationButton *self)
{
    g_return_val_if_fail (APPLET_IS_APPLICATION_BUTTON (self), FALSE);
    AppletApplicationButtonPrivate *priv = self->priv;
    return priv->is_draggable;

}

CdosApp*
applet_application_button_get_app (AppletApplicationButton *self)
{
    g_return_val_if_fail (APPLET_IS_APPLICATION_BUTTON (self), NULL);
    AppletApplicationButtonPrivate *priv = self->priv;
    return priv->app;
}

void
applet_application_button_set_context_menu_enable (AppletApplicationButton *self, gboolean enable)
{
    g_return_if_fail (APPLET_IS_APPLICATION_BUTTON (self));
    AppletApplicationButtonPrivate *priv = self->priv;
    priv->context_menu_enable = enable;
    if (priv->context_menu_enable && priv->context_menu_activate_id == 0)
    {
        priv->context_menu_activate_id = \
            g_signal_connect ((ClutterActor *)self,
                             "button-release-event",
                             G_CALLBACK (button_on_context_menu),
                             self);
    }
    else if (!priv->context_menu_enable && priv->context_menu_activate_id > 0)
    {
        cdos_popup_menu_close (CDOS_POPUP_MENU_BASE (priv->context_menu), FALSE);
        g_signal_handler_disconnect ((ClutterActor *)self, priv->context_menu_activate_id);
        priv->context_menu_activate_id = 0;
    }
}

gboolean
applet_application_button_get_context_menu_enable (AppletApplicationButton *self)
{
    g_return_val_if_fail (APPLET_IS_APPLICATION_BUTTON (self), FALSE);
    AppletApplicationButtonPrivate *priv = self->priv;
    return priv->context_menu_enable;
}

CdosPopupMenu *
applet_application_button_get_context_menu (AppletApplicationButton *self)
{
    g_return_val_if_fail (APPLET_IS_APPLICATION_BUTTON (self), NULL);
    AppletApplicationButtonPrivate *priv = self->priv;
    return priv->context_menu;
}

void
applet_application_button_set_can_drop (AppletApplicationButton *self, gboolean can_drop)
{
    g_return_if_fail (APPLET_IS_APPLICATION_BUTTON (self));
    AppletApplicationButtonPrivate *priv = self->priv;
    priv->can_drop = can_drop;
}

gboolean
applet_application_button_get_can_drop (AppletApplicationButton *self)
{
    g_return_val_if_fail (APPLET_IS_APPLICATION_BUTTON (self), FALSE);
    AppletApplicationButtonPrivate *priv = self->priv;
    return priv->can_drop;
}

ClutterDropAction *
applet_application_button_get_drop_action (AppletApplicationButton *self)
{
    g_return_val_if_fail (APPLET_IS_APPLICATION_BUTTON (self), NULL);
    AppletApplicationButtonPrivate *priv = self->priv;
    return priv->drop_action;
}

ClutterDragAction *
applet_application_button_get_drag_action (AppletApplicationButton *self)
{
    g_return_val_if_fail (APPLET_IS_APPLICATION_BUTTON (self), NULL);
    AppletApplicationButtonPrivate *priv = self->priv;
    return priv->drag_action;
}
