/**
 * Copyright (c) 2016-2018 CPU and Fundamental Software Research Center, CAS
 *
 * This software is published by the license of CPU-OS License，you can use and
 * distibuted this software under this License. See CPU-OS Liense for more detail.
 *
 * You should have receive a copy of CPU-OS License. If not, please contact us
 * by email <support_os@cpu-os.ac.cn>
 *
 **/
//
// Created by zhangtao on 18-1-25.
// Email: zhangtao@cpu-os.ac.cn
//

#include "applet-applications-category.h"
#include "applet-menu-types.h"
#include "applet-menu.h"
#include "applet-menu-private.h"
#include "applet-application-button.h"

#define APPLET_APPLICATIONS_CATEGORY_GET_PRIVATE(obj)      \
(G_TYPE_INSTANCE_GET_PRIVATE((obj), APPLET_TYPE_APPLICATIONS_CATEGORY, AppletApplicationsCategoryPrivate))

G_DEFINE_TYPE(AppletApplicationsCategory, applet_applications_category, APPLET_TYPE_CATEGORY_BASE);

#define DEFAULT_COLUMN_NUM 6

struct _AppletApplicationsCategoryPrivate
{
    GSettings       *settings;
};

static void applet_applications_category_dispose (GObject *object);
static void applet_applications_category_finalize (GObject *object);
static void menu_settings_on_changed (GSettings *settings,
                                     gchar *key,
                                     AppletApplicationsCategory *self);
static gboolean
main_box_on_key_press (ClutterActor *actor, ClutterEvent *event, gpointer user_data);

static void
applet_applications_category_init (AppletApplicationsCategory *self)
{
    AppletApplicationsCategoryPrivate *priv;
    priv = self->priv = APPLET_APPLICATIONS_CATEGORY_GET_PRIVATE (self);
    priv->settings = g_settings_new (CDOS_MENU_SCHEMA);

    applet_category_base_set_can_delete ((AppletCategoryBase *)self, FALSE);
    applet_category_base_set_can_edit_title ((AppletCategoryBase *)self, FALSE);
    applet_category_base_show_expand_button ((AppletCategoryBase *)self, FALSE);
    st_widget_set_style_class_name ((StWidget *)self, "menu-apps-category");
    g_signal_connect (priv->settings,
                     "changed",
                     G_CALLBACK (menu_settings_on_changed),
                     self);
}

static void
applet_applications_category_class_init (AppletApplicationsCategoryClass *klass)
{

    GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
    g_type_class_add_private (klass, sizeof (AppletApplicationsCategoryPrivate));

    gobject_class->dispose = applet_applications_category_dispose;
    gobject_class->finalize = applet_applications_category_finalize;

}

static void
applet_applications_category_dispose (GObject *object)
{
    AppletApplicationsCategory *self = APPLET_APPLICATIONS_CATEGORY (object);
    AppletApplicationsCategoryPrivate *priv = self->priv;
    if (priv->settings)
    {
        g_clear_object (&priv->settings);
    }

    G_OBJECT_CLASS (applet_applications_category_parent_class)->dispose (object);
}

static void
applet_applications_category_finalize (GObject *object)
{
    G_OBJECT_CLASS (applet_applications_category_parent_class)->finalize (object);
}

static void
menu_settings_on_changed (GSettings *settings,
                         gchar *key,
                         AppletApplicationsCategory *self)
{
    AppletApplicationsCategoryPrivate *priv = self->priv;
}

static gboolean
main_box_on_key_press (ClutterActor *actor, ClutterEvent *event, gpointer user_data)
{
    AppletApplicationsCategory *self = APPLET_APPLICATIONS_CATEGORY (user_data);
    guint symbol;
    ClutterActor *focused;
    ClutterStage *global_stage;

    guint        focused_index, chilren_n;
    gint         next_index, column_index, default_column_num;
    GList        *children;

    symbol = clutter_event_get_key_symbol (event);
    if (symbol != CLUTTER_KEY_Left && symbol != CLUTTER_KEY_Right)
    {
        return FALSE;
    }
    CdosGlobal   *global = cdos_global_get ();
    global_stage = cdos_global_get_stage (global);
    focused = clutter_stage_get_key_focus (CLUTTER_STAGE (global_stage));
    if (!clutter_actor_contains ((ClutterActor *)self, focused))
    {
        return FALSE;
    }

    children = applet_category_base_get_app_buttons ((AppletCategoryBase *)self);
    default_column_num = applet_category_base_get_default_column_num ((AppletCategoryBase *)self);

    focused_index = g_list_index (children, (AppletApplicationButton *)focused);
    chilren_n = g_list_length (children);
    column_index = focused_index % default_column_num;

    if (symbol == CLUTTER_KEY_Left)
    {
        if (column_index == 0)
        {
            next_index = focused_index == 0 ? chilren_n - 1 : focused_index - 1;
            applet_category_base_buttons_navigate_focus ((AppletCategoryBase *)self, next_index);
            return TRUE;
        }
    }
    else if (symbol == CLUTTER_KEY_Right)
    {
        if (column_index == default_column_num - 1 || focused_index + 1 == chilren_n)
        {
            next_index = focused_index + 1 >= chilren_n ? 0 : focused_index + 1;
            applet_category_base_buttons_navigate_focus ((AppletCategoryBase *)self, next_index);
            return TRUE;
        }
    }

    return FALSE;
}

AppletApplicationsCategory *
applet_applications_category_new (const gchar *title)
{
    AppletApplicationsCategory *self;
    self = APPLET_APPLICATIONS_CATEGORY (g_object_new (APPLET_TYPE_APPLICATIONS_CATEGORY, NULL));
    if (title != NULL)
    {
        applet_category_base_set_title ((AppletCategoryBase *)self, title);
    }

    /*StBoxLayout *main_box = applet_menu_get_main_box ();
    if (main_box)
    {
        g_signal_connect ((ClutterActor *)main_box,
               "key-press-event",
             G_CALLBACK (main_box_on_key_press),
                     self);
    }*/
    return self;
}

