/**
 * Copyright (c) 2016-2018 CPU and Fundamental Software Research Center, CAS
 *
 * This software is published by the license of CPU-OS License，you can use and
 * distibuted this software under this License. See CPU-OS Liense for more detail.
 *
 * You should have receive a copy of CPU-OS License. If not, please contact us
 * by email <support_os@cpu-os.ac.cn>
 *
 **/
//
// Created by zhangtao on 18-1-25.
// Email: zhangtao@cpu-os.ac.cn
//
//
#include <math.h>
#include <meta/meta-cursor-tracker.h>

#include "common.h"
#include "applet-category-base.h"
#include "applet-menu-helper.h"
#include "applet-menu.h"
#include "applet-application-button.h"
#include "applet-menu-private.h"

#define APPLET_CATEGORY_BASE_GET_PRIVATE(obj)      \
(G_TYPE_INSTANCE_GET_PRIVATE((obj), APPLET_TYPE_CATEGORY_BASE, AppletCategoryBasePrivate))

G_DEFINE_TYPE(AppletCategoryBase, applet_category_base, ST_TYPE_BOX_LAYOUT);

struct _AppletCategoryBasePrivate
{
    const gchar         *name;
    const gchar         *title_text;
    StLabel             *title_label;
    StIcon              *title_icon;
    StEntry             *title_entry;
    StIcon              *title_entry_icon;
    StButton            *expand_button;
    StButton            *delete_button;
    StBoxLayout         *head;
    StWidget            *body_table;
    ClutterGridLayout   *body_layout;
    ClutterAction       *click_action;
    AppletButtonLayoutMode button_layout_mode;

    gboolean            show_expand_button;
    gboolean            expand;
    gboolean            can_edit_title;
    gboolean            title_editing;
    gboolean            can_delete;
    gboolean            app_button_can_drop;
    gint                default_column_num;
    gint                default_show_num;
    GList              *children;
    GList              *source_data;
    guint               idle_id;
};

enum {
    TITLE_CHANGED,
    DELETE,
    RELAYOUT_FINISH,
    LAST_SIGNAL
};

static guint signals[LAST_SIGNAL] = {0};
static void applet_category_base_dispose (GObject *object);
static void applet_category_base_finalize (GObject *object);

static void
title_entry_on_clear (StEntry *entry, AppletCategoryBase *self);
static void
title_entry_on_focus_out (ClutterActor *actor, gpointer user_data);
static gboolean
stage_on_captured_event (ClutterActor *actor, ClutterEvent *event, gpointer user_data);
static gboolean
applet_category_base_accept_title_changed (AppletCategoryBase *self);
static void
applet_category_base_cancel_edit_title (AppletCategoryBase *self);
static gboolean
category_on_key_press_event (ClutterActor *actor, ClutterKeyEvent *event);
static gboolean
category_on_key_release_event (ClutterActor *actor, ClutterKeyEvent *event);
static void
applet_category_base_add_app_button (AppletCategoryBase *self, ClutterActor *button, gint index);
static void
applet_category_base_get_button_pos (AppletCategoryBase *self,
                                     ClutterActor *button,
                                     gint *col,
                                     gint *row);
static void
applet_category_base_set_button_pos (AppletCategoryBase *self,
                                     ClutterActor *button,
                                     gint col,
                                     gint row);
static gboolean
applet_category_base_is_child (AppletCategoryBase *self,
                              ClutterActor *button);
static MoveMode
applet_category_base_get_move_direction (AppletCategoryBase *self, ClutterActor *target, ClutterActor *handle);
static void
expand_button_on_click (StButton *button, gint clicked_button, gpointer user_data);
static void
delete_button_on_click (StButton *button, gint clicked_button, gpointer user_data);
static void
applet_category_base_remove_button (AppletCategoryBase *self, ClutterActor *button, gboolean destroy);
static gboolean
applet_category_base_relayout (AppletCategoryBase *self);
static void
head_on_clicked (ClutterClickAction   *action,
                ClutterActor         *actor,
                gpointer              user_data);
static gboolean
main_box_on_key_press (ClutterActor *actor, ClutterEvent *event, gpointer user_data);

static void
button_drop_action_on_over_in (ClutterDropAction *action,
                              ClutterActor      *actor,
                              gpointer           user_data);
static void
button_drop_action_on_over_out (ClutterDropAction *action,
                                ClutterActor      *actor,
                                gpointer           user_data);
static void
button_drag_action_on_begin  (ClutterDragAction  *action,
               ClutterActor       *actor,
               gfloat              event_x,
               gfloat              event_y,
               ClutterModifierType modifiers,
               gpointer            user_data);
static void
button_drag_action_on_motion (ClutterDragAction *action,
                                ClutterActor      *actor,
                                gfloat             delta_x,
                                gfloat             delta_y,
                                gpointer           user_data);
static void
button_drag_action_on_end (ClutterDragAction  *action,
                            ClutterActor *actor,
                            gfloat event_x,
                            gfloat event_y,
                            ClutterModifierType modifiers,
                            gpointer user_data);
static void
head_on_hover_changed (GObject    *gobject,
                        GParamSpec *pspec,
                        gpointer    user_data);

static void
body_table_on_realize (StWidget *actor, gpointer user_data)
{
    AppletCategoryBase *self = APPLET_CATEGORY_BASE (user_data);
    AppletCategoryBasePrivate *priv = self->priv;
    StThemeNode *theme_node;

    theme_node = st_widget_get_theme_node (priv->body_table);
    gdouble spacing = st_theme_node_get_length (theme_node, "spacing");
    clutter_grid_layout_set_row_spacing (priv->body_layout, spacing);
}

static void
applet_category_base_init (AppletCategoryBase *self)
{
    AppletCategoryBasePrivate *priv;
    priv = self->priv = APPLET_CATEGORY_BASE_GET_PRIVATE (self);
    priv->can_delete = TRUE;
    priv->can_edit_title = TRUE;
    priv->title_editing = FALSE;
    priv->show_expand_button = FALSE;
    priv->default_column_num = CATEGORY_DEFAULT_COLUMN_NUM;
    priv->app_button_can_drop = FALSE;

    st_widget_set_style_class_name ((StWidget *)self, "menu-category-box");

    priv->head = ST_BOX_LAYOUT (st_box_layout_new ());
    st_widget_set_style_class_name ((StWidget *)priv->head, "menu-category-head");
    priv->title_label = ST_LABEL (st_label_new (NULL));
    st_widget_set_style_class_name ((StWidget *)priv->title_label, "menu-category-title-label");
    priv->title_entry = ST_ENTRY (st_entry_new (NULL));
    st_widget_set_style_class_name ((StWidget *)priv->title_entry, "menu-category-title-entry");
    ClutterActor *clutter_text = st_entry_get_clutter_text (priv->title_entry);
    clutter_text_set_max_length ((ClutterText *)clutter_text, 48);

    StIcon *secondary_icon = ST_ICON (st_icon_new ());
    priv->title_entry_icon = secondary_icon;
    //st_icon_set_icon_type (secondary_icon, ST_ICON_FULLCOLOR);
    st_icon_set_icon_size (secondary_icon, 16);
    st_icon_set_icon_name (secondary_icon, "entry-edit-clear");
    st_entry_set_secondary_icon ((StEntry *)priv->title_entry, (ClutterActor *)secondary_icon);
    st_widget_set_style_class_name ((StWidget *)secondary_icon, "menu-title-entry-icon");
    st_widget_set_track_hover ((StWidget *)secondary_icon, TRUE);

    priv->expand_button = ST_BUTTON (st_button_new ());
    st_widget_set_style_class_name ((StWidget *)priv->expand_button, "menu-category-expand-button");
    StIcon *expand_button_icon = ST_ICON(st_icon_new ());
    //st_icon_set_icon_type (expand_button_icon, ST_ICON_FULLCOLOR);
    st_icon_set_icon_size (expand_button_icon, 16);
    st_icon_set_icon_name (expand_button_icon, "found-go-right");
    st_bin_set_child ((StBin *)priv->expand_button, (ClutterActor *)expand_button_icon);

    priv->delete_button = ST_BUTTON (st_button_new ());
    st_widget_set_style_class_name ((StWidget *)priv->delete_button, "menu-category-delete-button");
    StIcon *delete_button_icon = ST_ICON(st_icon_new ());
    //st_icon_set_icon_type (delete_button_icon, ST_ICON_FULLCOLOR);
    st_icon_set_icon_size (delete_button_icon, 16);
    st_icon_set_icon_name (delete_button_icon, "group-remove");
    st_bin_set_child ((StBin *)priv->delete_button, (ClutterActor *)delete_button_icon);

    priv->body_table = g_object_new (ST_TYPE_WIDGET, NULL);
    st_widget_set_style_class_name ((StWidget *)priv->body_table, "menu-category-body");
    priv->body_layout = (ClutterGridLayout *)clutter_grid_layout_new ();
    clutter_actor_set_layout_manager ((ClutterActor *)priv->body_table,
                            (ClutterLayoutManager *)priv->body_layout);
    clutter_grid_layout_set_row_homogeneous (priv->body_layout, TRUE);
    g_signal_connect (priv->body_table, 
                      "realize", 
                      G_CALLBACK (body_table_on_realize), 
                      self);

    //clutter_grid_layout_set_orientation (priv->body_layout, CLUTTER_ORIENTATION_VERTICAL);

    ClutterAction *click_action = clutter_click_action_new ();
    priv->click_action = click_action;
    clutter_actor_add_action ((ClutterActor *)priv->head, click_action);
    g_signal_connect ((ClutterClickAction *)click_action,
                    "clicked",
                    G_CALLBACK (head_on_clicked),
                    self);

    st_box_layout_set_vertical ((StBoxLayout *)self, TRUE);
    st_box_layout_set_vertical ((StBoxLayout *)priv->head, FALSE);
    clutter_actor_set_reactive ((ClutterActor *)self,TRUE);
    clutter_actor_set_reactive ((ClutterActor *)priv->head,TRUE);
    clutter_actor_hide ((ClutterActor *)priv->title_entry);
    clutter_actor_hide ((ClutterActor *)priv->delete_button);

    applet_category_base_set_expand (self, TRUE);
    applet_category_base_show_expand_button (self, TRUE);//priv->expand_button);
    applet_category_base_set_default_show_num (self, -1);

    st_widget_set_track_hover ((StWidget *)priv->head, TRUE);
    st_widget_set_track_hover ((StWidget *)self, TRUE);

    applet_helper_add_focus_group ((StWidget *)self);

    g_signal_connect (priv->title_entry,
                     "secondary-icon-clicked",
                     G_CALLBACK (title_entry_on_clear),
                     self);
    g_signal_connect (clutter_text,
                      "key-focus-out",
                      G_CALLBACK (title_entry_on_focus_out),
                      self);
    g_signal_connect (priv->head,
                      "notify::hover",
                      G_CALLBACK (head_on_hover_changed),
                      self);
    CdosGlobal *global = cdos_global_get();
    ClutterStage *global_stage = cdos_global_get_stage (global);
    g_signal_connect (global_stage,
                      "captured-event",
                      G_CALLBACK (stage_on_captured_event),
                      self);
    g_signal_connect (priv->expand_button,
                     "clicked",
                     G_CALLBACK (expand_button_on_click),
                     self);
    g_signal_connect (priv->delete_button,
                     "clicked",
                     G_CALLBACK (delete_button_on_click),
                     self);

    StBoxLayout *main_box = applet_menu_get_main_box ();
    g_signal_connect ((ClutterActor *)main_box,
                     "key-press-event",
                     G_CALLBACK (main_box_on_key_press),
                     self);

    menu_container_add_actor ((ClutterActor *)self, (ClutterActor *)priv->head,
                              "x-expand", TRUE,
                              "x-align", CLUTTER_ACTOR_ALIGN_FILL,
                              "y-align", CLUTTER_ACTOR_ALIGN_START,
                              NULL);
    menu_container_add_actor ((ClutterActor *)self, (ClutterActor *)priv->body_table,
                              "x-expand", TRUE,
                              "x-align", CLUTTER_ACTOR_ALIGN_FILL,
                              NULL);
    menu_container_add_actor ((ClutterActor *)priv->head, (ClutterActor *)priv->title_label,
                              "x-align", CLUTTER_ACTOR_ALIGN_FILL,
                              "y-align", CLUTTER_ACTOR_ALIGN_CENTER,
                              "x-expand", TRUE,
                              NULL);
    menu_container_add_actor ((ClutterActor *)priv->head, (ClutterActor *)priv->title_entry,
                              "x-expand", TRUE,
                              "x-align", CLUTTER_ACTOR_ALIGN_FILL,
                              NULL);
    menu_container_add_actor ((ClutterContainer *)priv->head, (ClutterActor *)priv->expand_button,
                              //"y-fill", TRUE, 
                              "x-align", ST_ALIGN_END,
                              NULL);
    menu_container_add_actor ((ClutterContainer *)priv->head, (ClutterActor *)priv->delete_button,
                              "x-align", ST_ALIGN_END,
                              NULL);

    applet_category_base_set_button_layout_mode (self, BUTTON_LAYOUT_H);
}

static void
applet_category_base_class_init (AppletCategoryBaseClass *klass)
{

    GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
    g_type_class_add_private (klass, sizeof (AppletCategoryBasePrivate));
    ClutterActorClass *actor_class = CLUTTER_ACTOR_CLASS (klass);

    gobject_class->dispose = applet_category_base_dispose;
    gobject_class->finalize = applet_category_base_finalize;
    actor_class->key_press_event = category_on_key_press_event;
    actor_class->key_release_event = category_on_key_release_event;

    signals[TITLE_CHANGED] =
        g_signal_new ("title-changed",
                     G_TYPE_FROM_CLASS (klass),
                     G_SIGNAL_RUN_LAST,
                     (guint)G_STRUCT_OFFSET (AppletCategoryBaseClass, title_changed),
                     NULL,
                     NULL,
                     g_cclosure_marshal_generic,
                     G_TYPE_NONE,
                     1,
                     G_TYPE_STRING);
    signals[DELETE] =
        g_signal_new ("delete",
                     G_TYPE_FROM_CLASS (klass),
                     G_SIGNAL_RUN_LAST,
                     0,
                     NULL,
                     NULL,
                     NULL,
                     G_TYPE_NONE,
                     0);
    signals[RELAYOUT_FINISH] =
        g_signal_new ("relayout-finish",
                     G_TYPE_FROM_CLASS (klass),
                     G_SIGNAL_RUN_LAST,
                     0,
                     NULL,
                     NULL,
                     NULL,
                     G_TYPE_NONE,
                     0);
}

static void
applet_category_base_dispose (GObject *object)
{
    AppletCategoryBase *self = APPLET_CATEGORY_BASE (object);
    AppletCategoryBasePrivate *priv = self->priv;
    ClutterActor *handle = NULL;
    CdosGlobal *global = cdos_global_get();
    ClutterStage *global_stage = cdos_global_get_stage (global);
    g_signal_handlers_disconnect_by_data ((ClutterActor *)global_stage, self);
    StBoxLayout *main_box = applet_menu_get_main_box ();
    g_signal_handlers_disconnect_by_data ((ClutterActor *)main_box, self);

    if (priv->idle_id > 0)
    {
        g_idle_remove_by_data (self);
        priv->idle_id = 0;
    }

    clutter_actor_clear_actions ((ClutterActor *)self);
    priv->click_action = NULL;

    if (priv->children)
    {
        g_list_free (priv->children);
        priv->children = NULL;
    }
    if (priv->source_data)
    {
        g_list_free (priv->source_data);
        priv->source_data = NULL;
    }
    G_OBJECT_CLASS (applet_category_base_parent_class)->dispose (object);
}

static void
applet_category_base_finalize (GObject *object)
{
    G_OBJECT_CLASS (applet_category_base_parent_class)->finalize (object);
}

static void
title_entry_on_clear (StEntry *entry, AppletCategoryBase *self)
{
    AppletCategoryBasePrivate *priv = self->priv;
    st_entry_set_text (priv->title_entry, NULL);
}

static void
head_update_expand_button_visible (AppletCategoryBase *self)
{
    AppletCategoryBasePrivate *priv = self->priv;
    if (priv->show_expand_button && !priv->title_editing && (int)g_list_length (priv->children) > priv->default_show_num)
        clutter_actor_show ((ClutterActor *)priv->expand_button);
    else
        clutter_actor_hide ((ClutterActor *)priv->expand_button);
}

static void
head_update_delete_button_visible (AppletCategoryBase *self)
{
    AppletCategoryBasePrivate *priv = self->priv;
    gboolean hover = st_widget_get_hover (priv->head);
    if (priv->can_delete && 
       !priv->show_expand_button && 
       !priv->title_editing &&
       hover)
    {
        clutter_actor_show ((ClutterActor *)priv->delete_button);
    } else {
        clutter_actor_hide ((ClutterActor *)priv->delete_button);
    }
}

static void
head_on_hover_changed (GObject    *gobject,
                        GParamSpec *pspec,
                        gpointer    user_data)
{
    AppletCategoryBase *self = APPLET_CATEGORY_BASE (user_data);
    head_update_delete_button_visible (self);
    head_update_expand_button_visible (self);
    return FALSE;
}

static void
title_entry_on_focus_out (ClutterActor *actor, gpointer user_data)
{
    AppletCategoryBase *self = APPLET_CATEGORY_BASE (user_data);
    //AppletCategoryBasePrivate *priv = self->priv;
    // You can do some things, when the key focus in.
    applet_category_base_cancel_edit_title (self);
}

static void
head_on_clicked (ClutterClickAction   *action,
                ClutterActor         *actor,
                gpointer              user_data)
{

    AppletCategoryBase *self = APPLET_CATEGORY_BASE (user_data);
    AppletCategoryBasePrivate *priv = self->priv;
    guint action_button = clutter_click_action_get_button (action);
    if (action_button != 1)
    {
        return ;
    }

    //edit title
    if (priv->can_edit_title && !priv->title_editing)
    {
        priv->title_editing = TRUE;
        clutter_actor_hide ((ClutterActor *)priv->title_label);
        clutter_actor_set_x_expand ((ClutterActor *)priv->title_label, FALSE);
        head_update_delete_button_visible (self);
        head_update_expand_button_visible (self);
        st_entry_set_text (priv->title_entry, priv->title_text);
        clutter_actor_show ((ClutterActor *)priv->title_entry);
        clutter_actor_grab_key_focus ((ClutterActor *)priv->title_entry);
        st_widget_add_style_pseudo_class ((StWidget *)priv->head, "edit");
    }

    //switch expand button;
    if (!priv->can_edit_title && priv->show_expand_button)
    {
        applet_category_base_switch_expand_button (self);
    }
}

static gboolean
stage_on_captured_event (ClutterActor *actor, ClutterEvent *event, gpointer user_data)
{
    AppletCategoryBase *self = APPLET_CATEGORY_BASE (user_data);
    AppletCategoryBasePrivate *priv = self->priv;
    ClutterActor *source = clutter_event_get_source (event);
    ClutterEventType event_type = clutter_event_type (event);

    if (event_type == CLUTTER_BUTTON_RELEASE || event_type == CLUTTER_BUTTON_PRESS)
    {
        if (!clutter_actor_contains ((ClutterActor *)priv->head, source) && priv->title_editing)
        {
            applet_category_base_accept_title_changed (self);
        }
        if (event_type == CLUTTER_BUTTON_RELEASE)
        {
            applet_menu_set_display_cursor (META_CURSOR_DEFAULT);
        }
    }

    return FALSE;
}

static gboolean
applet_category_base_accept_title_changed (AppletCategoryBase *self)
{
    AppletCategoryBasePrivate *priv = self->priv;
    if (priv->can_edit_title && priv->title_editing)
    {
        const gchar *text = st_entry_get_text (priv->title_entry);
        gchar *s = g_strdup (text);
        s= g_strstrip (s);
        if (s == NULL || g_strcmp0 (s, "") == 0)
        {
            applet_category_base_cancel_edit_title (self);
            return FALSE;
        }
        priv->title_text = (const gchar *)g_strdup(s);
        st_label_set_text (priv->title_label, priv->title_text);
        applet_category_base_cancel_edit_title (self);

        g_signal_emit (self, signals[TITLE_CHANGED], 0, priv->title_text);
        return TRUE;
    }
    return FALSE ;
}

static void
applet_category_base_cancel_edit_title (AppletCategoryBase *self)
{
    AppletCategoryBasePrivate *priv = self->priv;

    if (priv->title_editing)
    {
        priv->title_editing = FALSE;
        clutter_actor_hide ((ClutterActor *)priv->title_entry);
        clutter_actor_show ((ClutterActor *)priv->title_label);
        clutter_actor_set_x_expand ((ClutterActor *)priv->title_label, TRUE);
        st_widget_remove_style_pseudo_class ((StWidget *)priv->head, "edit");
        head_update_delete_button_visible (self);
        head_update_expand_button_visible (self);
    }
}
//Esc键只接收按下事件
static gboolean
category_on_key_press_event (ClutterActor *actor, ClutterKeyEvent *event)
{
    AppletCategoryBase *self = (AppletCategoryBase *)actor;
    AppletCategoryBasePrivate *priv = self->priv;
    if (event->keyval == CLUTTER_KEY_Escape)
    {
        if (priv->title_editing)
        {
            applet_category_base_cancel_edit_title (self);
            return TRUE;
        }
    }
    return FALSE;
}
//回车键只接受弹起事件
static gboolean
category_on_key_release_event (ClutterActor *actor, ClutterKeyEvent *event)
{
    AppletCategoryBase *self = (AppletCategoryBase *)actor;
    AppletCategoryBasePrivate *priv = self->priv;
    if (event->keyval == CLUTTER_KEY_Return)
    {
        if (priv->title_editing)
        {
            return applet_category_base_accept_title_changed (self);
        }
    }

    if (event->keyval == CLUTTER_KEY_Delete)
    {
        if (priv->title_editing)
        {
            st_entry_set_text (priv->title_entry, NULL);
            return TRUE;
        }
    }
    return FALSE;
}

static void
applet_category_base_add_app_button (AppletCategoryBase *self, ClutterActor *button, gint index)
{
    gint row, col;
    AppletCategoryBasePrivate *priv = self->priv;
    if ((priv->default_show_num > -1 &&
        index >= priv->default_show_num) &&
        priv->show_expand_button == FALSE)
    {
        return ;
    }
    if (clutter_actor_get_parent (button))
        return ;

    if (APPLET_APPLICATION_BUTTON (button) && 
        applet_menu_button_base_get_layout_mode (APPLET_MENU_BUTTON_BASE (button)) == BUTTON_LAYOUT_H)
    {
        col = 0;
        row = index;
        clutter_grid_layout_attach (CLUTTER_GRID_LAYOUT (priv->body_layout), 
                                    CLUTTER_ACTOR (button), 
                                    col, row,
                                    priv->default_column_num, 1);
    } else {
        row = (int)floor(index / priv->default_column_num);
        col = index % priv->default_column_num;
        clutter_grid_layout_attach (CLUTTER_GRID_LAYOUT (priv->body_layout), 
                                    CLUTTER_ACTOR (button), 
                                    col, row, 
                                    1, 1);
    }
}

static void
applet_category_base_get_button_pos (AppletCategoryBase *self,
                                     ClutterActor *button,
                                     gint *col,
                                     gint *row)
{
    /*
    AppletCategoryBasePrivate *priv = self->priv;
    ClutterGridLayout *grid = CLUTTER_GRID_LAYOUT (priv->body_table);
    */
}

static void
applet_category_base_set_button_pos (AppletCategoryBase *self,
                                     ClutterActor *button,
                                     gint col,
                                     gint row)
{
    //AppletCategoryBasePrivate *priv = self->priv;
    //ClutterContainer *container = CLUTTER_CONTAINER (priv->body_table);
    //StTableChild *meta = (StTableChild *) clutter_container_get_child_meta (container, button);
    //g_object_set(meta, "row", row, "col", col, NULL);
}

static gboolean
applet_category_base_is_child (AppletCategoryBase *self,
                              ClutterActor *button)
{
    AppletCategoryBasePrivate *priv = self->priv;
    GList *children = priv->children;
    while (children != NULL)
    {
        ClutterActor *btn = children->data;
        if (btn == button)
        {
            return TRUE;
        }
        children = children->next;
    }
    return FALSE;
}

static MoveMode
applet_category_base_get_move_direction (AppletCategoryBase *self, ClutterActor *target, ClutterActor *handle)
{
    AppletCategoryBasePrivate *priv = self->priv;
    gboolean is_child = FALSE;
    guint handle_index;
    guint target_index;
    is_child = applet_category_base_is_child (self, handle);
    if (!is_child)
    {
        return MOVE_BEFORE;
    }
    handle_index = g_list_index (priv->children, handle);
    target_index = g_list_index (priv->children, target);
    if (handle_index < target_index)
    {
        return MOVE_AFTER;
    }
    return MOVE_BEFORE;
}

static void
applet_category_base_remove_button (AppletCategoryBase *self, ClutterActor *button, gboolean destroy)
{
    AppletCategoryBasePrivate *priv = self->priv;
    if (destroy)
    {
        priv->children = g_list_remove (priv->children, button);
        //can't use clutter_actor_destroy.
    } else {
        g_object_ref (button);
    }
    if (clutter_actor_get_parent (button))
    {
        clutter_actor_remove_child ((ClutterActor *)priv->body_table, (ClutterActor *)button);
    }
}

static void
update_body_table_visible (AppletCategoryBase *self)
{
    AppletCategoryBasePrivate *priv = self->priv;
    if (!priv->expand || g_list_length (priv->children) == 0)
    {
        clutter_actor_hide (priv->body_table);
    } else {
        clutter_actor_show (priv->body_table);
    }
}

static void
update_show_items (AppletCategoryBase *self, gboolean expand)
{
    AppletCategoryBasePrivate *priv = self->priv;
    GList *children = priv->children;
    gint index = 0;
    while (children != NULL)
    {
        AppletApplicationButton *button = children->data;
        if (index >= priv->default_show_num)
        {
            if (expand)
            {
                applet_category_base_add_app_button (self, (ClutterActor *)button, index);
            } else {
                if (clutter_actor_has_key_focus ((ClutterActor *)button))
                {
                    applet_menu_grab_key_focus_main_box ();
                }
                applet_category_base_remove_button (self, (ClutterActor *)button, FALSE);
            }
        }
        index++;
        children = children->next;
    }
    update_body_table_visible (self);
}

static void
head_update_expand_button (AppletCategoryBase *self)
{
    AppletCategoryBasePrivate *priv = self->priv;
    StIcon *icon = (StIcon *)st_bin_get_child ((StBin *)priv->expand_button);
    if (priv->expand)
    {
        st_icon_set_icon_name (icon, "found-go-down");
    } else {
        st_icon_set_icon_name (icon, "found-go-right");
    }
    update_show_items (self, priv->expand);
}

static void
expand_button_on_click (StButton *button, gint clicked_button, gpointer user_data)
{
    AppletCategoryBase *self = user_data;
    AppletCategoryBasePrivate *priv = self->priv;
    if (priv->show_expand_button && clicked_button == 1)
    {
        applet_category_base_set_expand (self, !priv->expand);
    }
}

static void
delete_button_on_click (StButton *button, gint clicked_button, gpointer user_data)
{
    AppletCategoryBase *self = user_data;
    AppletCategoryBasePrivate *priv = self->priv;
    if (priv->title_editing)
    {
        applet_category_base_cancel_edit_title (self);
    }
    if (priv->can_delete && clicked_button == 1)
    {
        g_signal_emit (self, signals[DELETE], 0);
        clutter_actor_destroy ((ClutterActor *)self);
    }
}

static gboolean
applet_category_base_relayout (AppletCategoryBase *self)
{
    AppletCategoryBasePrivate *priv = self->priv;
    //不可强制销毁，否则导致正在拖拽的按钮无法完成拖拽过程。
    //st_container_destroy_children ((StContainer *)priv->body);
    while (priv->children != NULL)
    {
        applet_category_base_remove_button (self, (ClutterActor *)priv->children->data, TRUE);
    }
    applet_category_base_set_source_data (self, priv->source_data);
    priv->idle_id = 0;
    g_signal_emit (self, signals[RELAYOUT_FINISH], 0);
    return FALSE;
}

static gboolean
main_box_on_key_press (ClutterActor *actor, ClutterEvent *event, gpointer user_data)
{
    AppletCategoryBase *self = APPLET_CATEGORY_BASE (user_data);
    AppletCategoryBasePrivate *priv = self->priv;
    guint symbol;
    ClutterActor *focused;
    ClutterActor *next_focus;
    ClutterStage *global_stage;
    guint        children_n;
    gint         next_index = 0, column_index = 0, focused_index = -1;

    symbol = clutter_event_get_key_symbol (event);
    if (symbol != CLUTTER_KEY_Down && symbol != CLUTTER_KEY_Up)
    {
        return FALSE;
    }
    CdosGlobal *global = cdos_global_get();
    global_stage = cdos_global_get_stage (global);
    focused = clutter_stage_get_key_focus (CLUTTER_STAGE (global_stage));
    if (!clutter_actor_contains ((ClutterActor *)priv->body_table, focused))
    {
        return FALSE;
    }
    focused_index = g_list_index (priv->children, (AppletApplicationButton *)focused);
    children_n = g_list_length (priv->children);
    if (priv->button_layout_mode == BUTTON_LAYOUT_V)
    {
        column_index = focused_index % priv->default_column_num;
    }
    if (symbol == CLUTTER_KEY_Down)
    {
        if (priv->button_layout_mode == BUTTON_LAYOUT_V)
        {
            next_index = focused_index + priv->default_column_num;
        } else {
            next_index = focused_index + 1;
        }
        //get the count of shown items;
        if (priv->default_show_num > 0 && priv->default_show_num < children_n)
        {
            children_n = priv->default_show_num;
        }
        if (next_index >= children_n)
        {
            //go to the next sibing category;
            next_focus = clutter_actor_get_next_sibling ((ClutterActor *)self);
            if (next_focus)
            {
                applet_category_base_buttons_navigate_focus ((AppletCategoryBase *)next_focus, column_index);
                return TRUE;
            }
        } else {
            applet_category_base_buttons_navigate_focus ((AppletCategoryBase *)self, next_index);
            return TRUE;
        }
    }
    else if (symbol == CLUTTER_KEY_Up)
    {
        if (priv->button_layout_mode == BUTTON_LAYOUT_V)
        {
            next_index = focused_index - priv->default_column_num;
        } else {
            next_index = focused_index - 1;
        }
        if (next_index < 0)
        {
            //go to the last column of the previous sibing category;
            next_focus = clutter_actor_get_previous_sibling ((ClutterActor *)self);
            if (next_focus)
            {
                GList *t = applet_category_base_get_app_buttons ((AppletCategoryBase *)next_focus);
                guint next_focus_n = g_list_length (t);
                if (priv->button_layout_mode == BUTTON_LAYOUT_V)
                {
                    gint t1 = next_focus_n % priv->default_column_num;
                    gint t2 = next_focus_n - t1;
                    if ((t1 - 1) >= column_index)
                    {
                        column_index = t2 + column_index;
                    } else {
                        column_index = t2 - priv->default_column_num + column_index;
                    }
                } else {
                    //last row;
                    column_index = next_focus_n - 1;
                }
                applet_category_base_buttons_navigate_focus ((AppletCategoryBase *)next_focus, column_index);
                return TRUE;
            }
        }
    }
    return FALSE;
}

static void
button_drop_action_on_over_in (ClutterDropAction *action,
                               ClutterActor      *actor,
                               gpointer           user_data)
{
    //AppletCategoryBasePrivate *priv = self->priv;
    //完成应用拖拽时显示即将要被放置的位置预览。
    ClutterDragAction *dragging_action;
    gchar             *desktop_id = NULL;
    dragging_action =  cdos_get_current_drag_action ();
    if (dragging_action)
    {
        desktop_id = g_object_get_data ((GObject *)dragging_action, "desktop-id");
    }
    if (desktop_id)
    {
        if (APPLET_IS_APPLICATION_BUTTON (actor))
        {
            //applet_category_base_show_button_preview_holder (self, actor);
        }
    }
}

static void
button_drop_action_on_over_out (ClutterDropAction *action,
                                ClutterActor      *actor,
                                gpointer           user_data)
{
    AppletCategoryBase *self = APPLET_CATEGORY_BASE (user_data);
    //AppletCategoryBasePrivate *priv = self->priv;

    applet_category_base_hide_button_preview_holder (self);
}

static void
button_drag_action_on_begin  (ClutterDragAction  *action,
               ClutterActor       *actor,
               gfloat              event_x,
               gfloat              event_y,
               ClutterModifierType modifiers,
               gpointer            user_data)
{
}

static void
button_drag_action_on_motion (ClutterDragAction *action,
                                ClutterActor      *actor,
                                gfloat             delta_x,
                                gfloat             delta_y,
                                gpointer           user_data)
{
    AppletCategoryBase *self = APPLET_CATEGORY_BASE (user_data);
    if ((ClutterActor *)self == actor)
    {
        return ;
    }
    gfloat  mx, my;
    gboolean accept;
    clutter_drag_action_get_motion_coords (action, &mx, &my);
    CdosGlobal *global = cdos_global_get();
    ClutterStage  *stage = cdos_global_get_stage (global);
    ClutterActor  *over_actor = clutter_stage_get_actor_at_pos (stage, CLUTTER_PICK_REACTIVE, (int)mx, (int)my);
    gint tx, ty;

    MetaDisplay *display = cdos_global_get_display (global);
    MetaCursorTracker *cursor_tracker = meta_cursor_tracker_get_for_display (display);
    meta_cursor_tracker_get_pointer (cursor_tracker, &tx, &ty, NULL);

    if (APPLET_IS_CATEGORY_BASE (over_actor))
    {
        accept = applet_category_base_is_accept_button_drop ((AppletCategoryBase *)over_actor, mx, my);
        if (accept)
        {
            //applet_category_base_show_button_preview_holder (over_actor, NULL);
        }
    }
    else if (APPLET_IS_APPLICATION_BUTTON (over_actor))
    {
        //ClutterActor *parent = (ClutterActor *)g_object_get_data (over_actor, "parent");
        //applet_category_base_show_button_preview_holder (parent, over_actor);
    }
}

static void
button_drag_action_on_end (ClutterDragAction  *action,
                            ClutterActor *actor,
                            gfloat event_x,
                            gfloat event_y,
                            ClutterModifierType modifiers,
                            gpointer user_data)
{
    AppletCategoryBase *self = APPLET_CATEGORY_BASE (user_data);
    applet_category_base_hide_button_preview_holder (self);
}

void
applet_category_base_set_name (AppletCategoryBase *self, const gchar *name)
{
    g_return_if_fail (APPLET_IS_CATEGORY_BASE (self));
    AppletCategoryBasePrivate *priv = self->priv;
    priv->name = (const gchar *)g_strdup (name);
}

const gchar*
applet_category_base_get_name (AppletCategoryBase *self)
{
    g_return_val_if_fail (APPLET_IS_CATEGORY_BASE (self), NULL);
    AppletCategoryBasePrivate *priv = self->priv;
    return priv->name;
}

void
applet_category_base_set_title (AppletCategoryBase *self, const gchar *title)
{
    g_return_if_fail (APPLET_IS_CATEGORY_BASE (self));
    g_return_if_fail (title != NULL);
    AppletCategoryBasePrivate *priv = self->priv;
    if (g_strcmp0 (title, priv->title_text) != 0)
    {
        priv->title_text = (const gchar *)g_strdup (title);
        st_label_set_text (priv->title_label, priv->title_text);
    }
}

const gchar*
applet_category_base_get_title (AppletCategoryBase *self)
{
    g_return_val_if_fail (APPLET_IS_CATEGORY_BASE (self), NULL);
    AppletCategoryBasePrivate *priv = self->priv;
    return priv->title_text;
}

void
applet_category_base_show_expand_button (AppletCategoryBase *self, gboolean is_show)
{
    g_return_if_fail (APPLET_IS_CATEGORY_BASE (self));
    AppletCategoryBasePrivate *priv = self->priv;
    priv->show_expand_button = is_show;
    head_update_expand_button_visible (self);
}

void
applet_category_base_set_can_delete (AppletCategoryBase *self, gboolean can_delete)
{
    g_return_if_fail (APPLET_IS_CATEGORY_BASE (self));
    AppletCategoryBasePrivate *priv = self->priv;
    priv->can_delete = can_delete;
}

void
applet_category_base_set_default_show_num (AppletCategoryBase *self, gint show_num)
{
    g_return_if_fail (APPLET_IS_CATEGORY_BASE (self));
    AppletCategoryBasePrivate *priv = self->priv;
    if (priv->default_show_num == show_num)
        return ;
    priv->default_show_num = show_num;
    applet_category_base_relayout (self);
    head_update_expand_button (self);
}

gint
applet_category_base_get_default_show_num (AppletCategoryBase *self)
{
    g_return_val_if_fail (APPLET_IS_CATEGORY_BASE (self), -1);
    AppletCategoryBasePrivate *priv = self->priv;
    return priv->default_show_num;
}

void
applet_category_base_set_default_column_num (AppletCategoryBase *self, gint show_num)
{
    g_return_if_fail (APPLET_IS_CATEGORY_BASE (self));
    AppletCategoryBasePrivate *priv = self->priv;
    priv->default_column_num = show_num;
    applet_category_base_relayout (self);
}

gint
applet_category_base_get_default_column_num (AppletCategoryBase *self)
{
    g_return_val_if_fail (APPLET_IS_CATEGORY_BASE (self), -1);
    AppletCategoryBasePrivate *priv = self->priv;
    return priv->default_column_num;
}

void
applet_category_base_set_can_edit_title (AppletCategoryBase *self, gboolean can_edit)
{
    g_return_if_fail (APPLET_IS_CATEGORY_BASE (self));
    AppletCategoryBasePrivate *priv = self->priv;
    priv->can_edit_title = can_edit;
}

gboolean
applet_category_base_get_can_edit_title (AppletCategoryBase *self)
{
    g_return_val_if_fail (APPLET_IS_CATEGORY_BASE (self), FALSE);
    AppletCategoryBasePrivate *priv = self->priv;
    return priv->can_edit_title;
}

void
applet_category_base_set_source_data (AppletCategoryBase *self, GList *source_data)
{
    g_return_if_fail (APPLET_IS_CATEGORY_BASE (self));
    AppletCategoryBasePrivate *priv = self->priv;
    if (source_data == NULL)
    {
        return ;
    }
    if (priv->source_data != source_data)
    {
        g_list_free (priv->source_data);
        priv->source_data = g_list_copy (source_data);
    }
    gint i = 0;
    while (source_data != NULL)
    {
        CdosApp *app = source_data->data;
        if (app)
        {
            gint icon_size = APP_ICON_SIZE_DEFAULT;
            if (priv->button_layout_mode == BUTTON_LAYOUT_V)
            {
                icon_size = APP_ICON_SIZE_BIG;
            }
            AppletApplicationButton *button;
            button = applet_application_button_new (app, icon_size);
            applet_application_button_set_can_drop (button, priv->app_button_can_drop);
            applet_menu_button_base_set_layout_mode ((AppletMenuButtonBase *)button, priv->button_layout_mode);

            if (priv->app_button_can_drop)
            {
                ClutterDragAction *drag_action = applet_application_button_get_drag_action (button);
                /*
                ClutterDropAction *drop_action = applet_application_button_get_drop_action (button);
                g_signal_connect (drop_action,
                                 "over-in",
                                 G_CALLBACK (button_drop_action_on_over_in),
                                 self);
                g_signal_connect (drop_action,
                                 "over-out",
                                 G_CALLBACK (button_drop_action_on_over_out),
                                 self);*/
                g_signal_connect (drag_action,
                                 "drag-begin",
                                 G_CALLBACK (button_drag_action_on_begin),
                                 self);
                g_signal_connect (drag_action,
                                 "drag-motion",
                                 G_CALLBACK (button_drag_action_on_motion),
                                 self);
                g_signal_connect (drag_action,
                                 "drag-end",
                                 G_CALLBACK (button_drag_action_on_end),
                                 self);
            }
            priv->children = g_list_append (priv->children, button);
            applet_category_base_add_app_button (self, (ClutterActor *)button, i);
            i++;
        }
        source_data = source_data->next;
    }
    head_update_expand_button_visible (self);
    update_body_table_visible (self);
}

GList *
applet_category_base_get_source_data (AppletCategoryBase *self)
{
    g_return_val_if_fail (APPLET_IS_CATEGORY_BASE (self), NULL);
    AppletCategoryBasePrivate *priv = self->priv;
    return priv->source_data;
}

void
applet_category_base_source_data_changed (AppletCategoryBase *self, GList *new_source_data, gboolean free)
{
    g_return_if_fail (APPLET_IS_CATEGORY_BASE (self));
    AppletCategoryBasePrivate *priv = self->priv;
    if (priv->idle_id > 0)
    {
        g_idle_remove_by_data (self);
        priv->idle_id = 0;
    }
    if (free)
    {
        g_list_free (priv->source_data);
    }
    priv->source_data = g_list_copy (new_source_data);
    priv->idle_id = g_idle_add ((GSourceFunc)applet_category_base_relayout, self);
}

void
applet_category_base_set_app_button_can_drop (AppletCategoryBase *self, gboolean can_drop)
{
    g_return_if_fail (APPLET_IS_CATEGORY_BASE (self));
    AppletCategoryBasePrivate *priv = self->priv;
    priv->app_button_can_drop = can_drop;
    GList *children = priv->children;
    while (children != NULL)
    {
        AppletApplicationButton *button = children->data;
        applet_application_button_set_can_drop (button, can_drop);
        children = children->next;
    }
}

gboolean
applet_category_base_get_app_button_can_drop (AppletCategoryBase *self)
{
    g_return_val_if_fail (APPLET_IS_CATEGORY_BASE (self), FALSE);
    AppletCategoryBasePrivate *priv = self->priv;
    return priv->app_button_can_drop;
}

GList *
applet_category_base_get_app_buttons (AppletCategoryBase *self)
{
    g_return_val_if_fail (APPLET_IS_CATEGORY_BASE (self), NULL);
    AppletCategoryBasePrivate *priv = self->priv;
    return priv->children;
}

void
applet_category_base_buttons_navigate_focus (AppletCategoryBase *self, gint index)
{
    g_return_if_fail (APPLET_IS_CATEGORY_BASE (self));
    AppletCategoryBasePrivate *priv = self->priv;
    AppletApplicationButton *app_button;
    if (index < 0)
    {
        index = 0;
    } else {
        guint len = g_list_length (priv->children);
        if (priv->default_show_num > 0 && priv->default_show_num < len)
        {
            len = priv->default_show_num;
        }
        //get last one of shown;
        if (index >= len)
        {
            index = len - 1;
        }
    }
    app_button = g_list_nth_data (priv->children, index);
    if (app_button)
    {
        if (priv->show_expand_button &&
            !priv->expand &&
            index >= priv->default_show_num)
        {
            applet_category_base_switch_expand_button (self);
        }
        clutter_actor_grab_key_focus ((ClutterActor *)app_button);
    }
}

void
applet_category_base_show_button_preview_holder (AppletCategoryBase *self,  ClutterActor *before_button)
{
    g_warn_if_reached ();
    g_return_if_fail (APPLET_IS_CATEGORY_BASE (self));

    AppletCategoryBasePrivate *priv = self->priv;
    GList *children = priv->children;

    if (!children)
    {
        return ;
    }

    children = g_list_reverse (priv->children);
    while (children != NULL)
    {
        AppletApplicationButton *button =  children->data;
        gint col, row;
        //applet_category_base_get_button_pos (self, button, &col, &row);
        if ((ClutterActor *)button == before_button)
        {
            break ;
        } else {
            col ++;
            if (col >= priv->default_column_num)
            {
                col = 0;
                row ++;
            }
            //applet_category_base_set_button_pos (self, button, col, row);
        }
        children = children->next;
    }
}

void
applet_category_base_hide_button_preview_holder (AppletCategoryBase *self)
{
    g_return_if_fail (APPLET_IS_CATEGORY_BASE (self));
    AppletCategoryBasePrivate *priv = self->priv;
    GList *children = priv->children;
    //remove all buttons;
    while (children != NULL)
    {
        applet_category_base_remove_button (self, (ClutterActor *)children->data, FALSE);
        children = children->next;
    }
    //add them once again;
    children = priv->children;
    guint i = 0;
    while (children != NULL)
    {
        applet_category_base_add_app_button (self, (ClutterActor *)children->data, i);
        i ++;
        children = children->next;
    }
}

gboolean
applet_category_base_is_accept_button_drop (AppletCategoryBase *self, gfloat x, gfloat y)
{
    g_return_val_if_fail (APPLET_IS_CATEGORY_BASE (self), FALSE);
    AppletCategoryBasePrivate *priv = self->priv;
    gfloat w, h, x1, y1;

    GList *children = priv->children;
    GList *last = g_list_last (children);
    ClutterActor *last_actor = (ClutterActor *)last->data;

    clutter_actor_get_transformed_position ((ClutterActor *)last_actor, &x1, &y1);
    clutter_actor_get_size (last_actor, &w, &h);
    if ((x > x1 + w) && (y > y1))
    {
        return TRUE;
    }
    return FALSE;
}

void
applet_category_base_switch_expand_button (AppletCategoryBase *self)
{
    g_return_if_fail (APPLET_IS_CATEGORY_BASE (self));
    AppletCategoryBasePrivate *priv = self->priv;
    if (priv->show_expand_button)
    {
        expand_button_on_click (priv->expand_button, 1, self);
    }
}

gboolean
applet_category_base_get_expand (AppletCategoryBase *self)
{
    g_return_val_if_fail (APPLET_IS_CATEGORY_BASE (self), FALSE);
    AppletCategoryBasePrivate *priv = self->priv;
    return priv->expand;
}

void
applet_category_base_set_expand (AppletCategoryBase *self, gboolean expand)
{
    g_return_val_if_fail (APPLET_IS_CATEGORY_BASE (self), FALSE);
    AppletCategoryBasePrivate *priv = self->priv;
    priv->expand = expand;
    head_update_expand_button (self);
}

void
applet_category_base_close_all_popup_menu (AppletCategoryBase *self)
{
    g_return_if_fail (APPLET_IS_CATEGORY_BASE (self));
    AppletCategoryBasePrivate *priv = self->priv;
    GList *children = priv->children;
    while (children != NULL)
    {
        AppletApplicationButton *button = children->data;
        CdosPopupMenu *context_menu = applet_application_button_get_context_menu (button);
        if (CDOS_POPUP_MENU_BASE (context_menu)->is_open)
        {
            cdos_popup_menu_close (CDOS_POPUP_MENU_BASE (context_menu), FALSE);
        }
        children = children->next;
    }
}

void
applet_category_base_set_button_layout_mode (AppletCategoryBase *self, AppletButtonLayoutMode mode)
{
    g_return_if_fail (APPLET_IS_CATEGORY_BASE (self));
    AppletCategoryBasePrivate *priv = self->priv;
    priv->button_layout_mode = mode;
    gboolean col_homogeneous = FALSE;
    if (priv->button_layout_mode == BUTTON_LAYOUT_H)
    {
        col_homogeneous = TRUE;
    }
    clutter_grid_layout_set_column_homogeneous (priv->body_layout, col_homogeneous);
}

void 
applet_category_base_set_icon_name (AppletCategoryBase *self, const gchar *icon_name)
{
    g_return_if_fail (APPLET_IS_CATEGORY_BASE (self));
    AppletCategoryBasePrivate *priv = self->priv;
    GIcon *icon;

    if (priv->title_icon && icon_name == NULL)
    {
        clutter_actor_destroy (priv->title_icon);
        priv->title_icon = NULL;
        return ;
    }
    if (icon_name)
    {
        if (priv->title_icon == NULL)
        {
            priv->title_icon = ST_ICON (st_icon_new ());
            //st_icon_set_icon_type (priv->title_icon, ST_ICON_FULLCOLOR);
            st_icon_set_icon_size (priv->title_icon, 24);
        }
        st_icon_set_icon_name (priv->title_icon, icon_name);
        menu_container_add_actor_at_index (priv->head, 
                                           priv->title_icon, 
                                           0, NULL);
    }
}
void
applet_category_base_set_head_sensitive (AppletCategoryBase *self, gboolean sensitive)
{
    g_return_if_fail (APPLET_IS_CATEGORY_BASE (self));
    AppletCategoryBasePrivate *priv = self->priv;
    clutter_actor_set_reactive((ClutterActor *)priv->head, sensitive);
	st_widget_set_can_focus((StWidget *)priv->head, sensitive);
}

gboolean
applet_category_base_get_head_sensitive (AppletCategoryBase *self)
{
    g_return_val_if_fail (APPLET_IS_CATEGORY_BASE (self), TRUE);
    AppletCategoryBasePrivate *priv = self->priv;
    return clutter_actor_get_reactive((ClutterActor *)priv->head);
}
