/**
 * Copyright (c) 2016-2018 CPU and Fundamental Software Research Center, CAS
 *
 * This software is published by the license of CPU-OS License，you can use and
 * distibuted this software under this License. See CPU-OS Liense for more detail.
 *
 * You should have receive a copy of CPU-OS License. If not, please contact us
 * by email <support_os@cpu-os.ac.cn>
 *
 **/
//
// Created by zhangtao on 18-1-25.
// Email: zhangtao@cpu-os.ac.cn
//
#include "cdos-global.h"

#include "applet-favorites-category.h"
#include "cdos-drag-drop.h"
#include "applet-menu-types.h"
#include "applet-favorites.h"

#define APPLET_FAVORITES_CATEGORY_GET_PRIVATE(obj)      \
(G_TYPE_INSTANCE_GET_PRIVATE((obj), APPLET_TYPE_FAVORITES_CATEGORY, AppletFavoritesCategoryPrivate))

G_DEFINE_TYPE(AppletFavoritesCategory, applet_favorites_category, APPLET_TYPE_CATEGORY_BASE);

struct _AppletFavoritesCategoryPrivate
{
    ClutterDropAction   *drop_action;
    ClutterDragAction   *dragging_action;
};

static void applet_favorites_category_dispose (GObject *object);
static void applet_favorites_category_finalize (GObject *object);

static gboolean
action_on_can_drop (ClutterDropAction *action,
               ClutterActor      *actor,
               gfloat             event_x,
               gfloat             event_y,
               gpointer           user_data);
static void
action_on_drop (ClutterDropAction *action,
               ClutterActor      *actor,
               gfloat             event_x,
               gfloat             event_y,
               gpointer           user_data);
static void
action_on_over_in (ClutterDropAction *action,
                  ClutterActor      *actor,
                  gpointer           user_data);
static void
action_on_over_out (ClutterDropAction *action,
                    ClutterActor      *actor,
                    gpointer           user_data);
static void
button_drag_action_on_end (ClutterDragAction  *action,
                            ClutterActor *actor,
                            gfloat event_x,
                            gfloat event_y,
                            ClutterModifierType modifiers,
                            gpointer user_data);
static void
applet_favorites_category_init (AppletFavoritesCategory *self)
{
    AppletFavoritesCategoryPrivate *priv;

    priv = self->priv = APPLET_FAVORITES_CATEGORY_GET_PRIVATE (self);
    applet_category_base_set_default_column_num ((AppletCategoryBase *)self, 3);
    applet_category_base_set_can_delete ((AppletCategoryBase *)self, TRUE);
    applet_category_base_set_can_edit_title ((AppletCategoryBase *)self, TRUE);
    applet_category_base_show_expand_button ((AppletCategoryBase *)self, FALSE);
    st_widget_set_style_class_name ((StWidget *)self, "menu-favorites-category");
    applet_category_base_set_app_button_can_drop ((AppletCategoryBase *)self, TRUE);
    applet_category_base_set_button_layout_mode ((AppletCategoryBase *)self, BUTTON_LAYOUT_V);

    //ClutterDropAction *drop_action = (ClutterDropAction *) clutter_drop_action_new ();
    //clutter_actor_add_action ((ClutterActor *)self, drop_action);

    priv->drop_action = (ClutterDropAction *) cdos_drop_action_new ((ClutterActor *)self);
    g_signal_connect (priv->drop_action,
                    "can-drop",
                    G_CALLBACK (action_on_can_drop),
                    self);
    g_signal_connect (priv->drop_action,
                    "drop",
                    G_CALLBACK (action_on_drop),
                    self);
/*    g_signal_connect (priv->drop_action,
                    "over-in",
                    G_CALLBACK (action_on_over_in),
                    self);
    g_signal_connect (priv->drop_action,
                    "over-out",
                    G_CALLBACK (action_on_over_out),
                    self);*/
}

static void
applet_favorites_category_class_init (AppletFavoritesCategoryClass *klass)
{

    GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
    g_type_class_add_private (klass, sizeof (AppletFavoritesCategoryPrivate));

    gobject_class->dispose = applet_favorites_category_dispose;
    gobject_class->finalize = applet_favorites_category_finalize;

}

static void
applet_favorites_category_dispose (GObject *object)
{
    AppletFavoritesCategory *self = APPLET_FAVORITES_CATEGORY (object);
    AppletFavoritesCategoryPrivate *priv = self->priv;

    clutter_actor_clear_actions ((ClutterActor *)self);
    priv->drop_action = NULL;
    G_OBJECT_CLASS (applet_favorites_category_parent_class)->dispose (object);
}

static void
applet_favorites_category_finalize (GObject *object)
{
    G_OBJECT_CLASS (applet_favorites_category_parent_class)->finalize (object);
}

static gboolean
action_on_can_drop (ClutterDropAction *action,
               ClutterActor      *actor,
               gfloat             event_x,
               gfloat             event_y,
               gpointer           user_data)
{
    AppletFavoritesCategory *self = user_data;
    AppletFavoritesCategoryPrivate *priv = self->priv;
    ClutterDragAction *dragging_action;
    gchar             *source_data = NULL;
    gfloat mx, my;
    gboolean accept = FALSE;
    priv->dragging_action = NULL;
    dragging_action =  cdos_get_current_drag_action ();

    if (dragging_action)
    {
        source_data = g_object_get_data ((GObject *)dragging_action, "favorites-group-name");
        accept = TRUE;
    }
    if (source_data == NULL)
    {
        source_data = g_object_get_data ((GObject *)dragging_action, "desktop-id");
        clutter_drag_action_get_motion_coords (dragging_action, &mx, &my);
        accept = applet_category_base_is_accept_button_drop (self, mx, my);
    }
    if (source_data && accept)
    {
        priv->dragging_action = dragging_action;
        return TRUE;
    }
    return FALSE;
}

static void
action_on_drop (ClutterDropAction *action,
               ClutterActor      *actor,
               gfloat             event_x,
               gfloat             event_y,
               gpointer           user_data)
{
    AppletFavoritesCategory *self = user_data;
    AppletFavoritesCategoryPrivate *priv = self->priv;
    MoveMode mode = MOVE_BEFORE;
    if (!priv->dragging_action)
    {
        return ;
    }
    gchar *source_data = NULL;
    AppletFavorites *favorites;
    source_data = g_object_get_data ((GObject *)priv->dragging_action, "favorites-group-name");
    favorites = applet_favorites_get_default ();
    const gchar *target_group = applet_category_base_get_name ((AppletCategoryBase *)self);

    if (source_data)
    {
        gchar *section = g_object_get_data ((GObject *)self, "section");
        applet_favorites_group_move (favorites, source_data, target_group, (int)g_strtod (section, NULL), mode);
    }
    //for application drop: when a appliaction button is droped in a blank area , it will be added to the last of group;
    if (source_data == NULL)
    {
        source_data = g_object_get_data ((GObject *)priv->dragging_action, "desktop-id");
        if (source_data)
        {
    	    CdosGlobal *global = cdos_global_get();
            ClutterStage  *stage = cdos_global_get_stage (global);
            ClutterActor  *over_actor = clutter_stage_get_actor_at_pos (stage, CLUTTER_PICK_REACTIVE, (int)event_x, (int)event_y);
            gboolean res;
            if (over_actor == actor)
            {
                CdosAppSystem *app_system = cdos_app_system_get_default ();
                GroupData *source_group = applet_favorites_get_group_by_app_id (favorites, source_data);

                if (source_group && g_strcmp0 (target_group, source_group->group_name) == 0)
                {
                    //both are the sam group,the dragging source will be moved to end;
                    GList *sources = applet_category_base_get_source_data ((AppletCategoryBase *)self);
                    CdosApp *app = cdos_app_system_lookup_app (app_system, source_data);
                    GList *last = g_list_last (sources);
                    //if only one child or app is the last one ,do nothing;
                    if (last && last->data != app)
                    {
                        const gchar *source_app_id = cdos_app_get_id (app);
                        const gchar *target_app_id = cdos_app_get_id (last->data);
                        applet_favorites_app_move (favorites, source_app_id, target_app_id, MOVE_AFTER);
                    }
                } else {
                    //both are diffrent group;
                    res = applet_favorites_remove_app (favorites, NULL, source_data);
                    if (res)
                    {
                        applet_favorites_add_app (favorites, target_group, source_data, FALSE);
                    }
                }
            }
            applet_category_base_hide_button_preview_holder ((AppletCategoryBase *)self);
        }
    }
    priv->dragging_action = NULL;
}

static void
action_on_over_in (ClutterDropAction *action,
                  ClutterActor      *actor,
                  gpointer           user_data)
{
    AppletFavoritesCategory *self = user_data;

    ClutterDragAction *dragging_action;
    gchar             *desktop_id = NULL;
    gfloat      mx, my;
    gboolean    accept;
    dragging_action =  cdos_get_current_drag_action ();

    if (dragging_action)
    {
        desktop_id = g_object_get_data ((GObject *)dragging_action, "desktop-id");
    }
    if (desktop_id)
    {
        clutter_drag_action_get_motion_coords (dragging_action, &mx, &my);
        accept = applet_category_base_is_accept_button_drop (self, mx, my);
        if (accept)
        {
            applet_category_base_show_button_preview_holder (self, NULL);
        }
        g_signal_connect (dragging_action,
                        "drag-end",
                        G_CALLBACK (button_drag_action_on_end),
                        self);
    }
}

static void
action_on_over_out (ClutterDropAction *action,
                    ClutterActor      *actor,
                    gpointer           user_data)
{
    AppletFavoritesCategory *self = user_data;
    ClutterDragAction *dragging_action =  cdos_get_current_drag_action ();

    applet_category_base_hide_button_preview_holder ((AppletCategoryBase *)self);

    g_signal_handlers_disconnect_by_func (dragging_action, button_drag_action_on_end, self);

}

static void
button_drag_action_on_end (ClutterDragAction  *action,
                            ClutterActor *actor,
                            gfloat event_x,
                            gfloat event_y,
                            ClutterModifierType modifiers,
                            gpointer user_data)
{
    AppletCategoryBase *self = APPLET_CATEGORY_BASE (user_data);
    applet_category_base_hide_button_preview_holder (self);
}

AppletFavoritesCategory *
applet_favorites_category_new (const gchar *name, const gchar *title)
{
    g_return_val_if_fail (name != NULL, NULL);

    AppletFavoritesCategory *favorites_category ;
    favorites_category = g_object_new (APPLET_TYPE_FAVORITES_CATEGORY, NULL);
    applet_category_base_set_name ((AppletCategoryBase *)favorites_category, name);

    if (title != NULL)
    {
        applet_category_base_set_title ((AppletCategoryBase *)favorites_category, title);
    }
    return favorites_category;
}

ClutterDropAction *
applet_favorites_category_get_drop_action (AppletFavoritesCategory *self)
{
    g_return_val_if_fail (APPLET_IS_FAVORITES_CATEGORY (self), NULL);
    AppletFavoritesCategoryPrivate *priv = self->priv;
    return priv->drop_action;
}
