/** 
 * Copyright (c) 2016-2018 CPU and Fundamental Software Research Center, CAS
 *  
 * This software is published by the license of CPU-OS License，you can use and 
 * distibuted this software under this License. See CPU-OS Liense for more detail.
 *
 * You should have receive a copy of CPU-OS License. If not, please contact us 
 * by email <support_os@cpu-os.ac.cn>  
 * 
 **/
//
// Created by zhangtao on 18-2-24.
// Email: zhangtao@cpu-os.ac.cn
//
#ifndef __APPLET_FAVORITES_H__
#define __APPLET_FAVORITES_H__

#include <stdio.h>
#include <stdlib.h>
#include <glib-object.h>
#include <glib.h>
#include "cdos-app-system.h"
#include "applet-parser.h"
#include "applet-menu-types.h"

G_BEGIN_DECLS

// clang-format off
#define APPLET_TYPE_FAVORITES (applet_favorites_get_type())
#define APPLET_FAVORITES(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj), APPLET_TYPE_FAVORITES, AppletFavorites))
#define APPLET_FAVORITES_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST((klass), APPLET_TYPE_FAVORITES, AppletFavoritesClass))
#define APPLET_IS_FAVORITES(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), APPLET_TYPE_FAVORITES))
#define APPLET_IS_FAVORITES_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass), APPLET_TYPE_FAVORITES))
#define APPLET_FAVORITES_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS((obj), APPLET_TYPE_FAVORITES, AppletFavoritesClass))
// clang-format on
#define FAVORITES_GROUP_DEFAULT_LABEL     _("Unnamed group")

typedef struct _AppletFavorites AppletFavorites;
typedef struct _AppletFavoritesClass AppletFavoritesClass;
typedef struct _AppletFavoritesPrivate AppletFavoritesPrivate;

typedef struct _GroupData GroupData;
typedef struct _ItemData ItemData;

struct _AppletFavorites
{
    GObject parent_instance;
    AppletFavoritesPrivate *priv;
};

struct _AppletFavoritesClass
{
    GObjectClass parent_class;
    void (* app_added) (AppletFavorites *self,
                        GroupData *group_data,
                        const gchar *app_id);

    void (* app_removed) (AppletFavorites *self, 
                         GroupData *group_data,
                         const gchar *app_id);

    void (* app_moved) (AppletFavorites *self,
                       GroupData *source_group_data,
                       GroupData *dest_group_data,
                       const gchar *app_id);

    void (* group_added) (AppletFavorites *self,
                        GroupData *group_data);

    void ( *group_label_changed) (AppletFavorites *self,
                                  GroupData *group_data);

    void (*group_removed) (AppletFavorites *self, GroupData *group_data);

    void (*group_moved) (AppletFavorites *self);
};

typedef enum {
	FAVORITES_SECTION_NONE,
	FAVORITES_SECTION_PRIMARY,
	FAVORITES_SECTION_SECONDARY
} FavoritesSection;

struct _GroupData
{
    gchar *group_name;
    gchar *group_label;
    gchar *origin_group_label;
    FavoritesSection section;
    xmlNodePtr group_node;
};

struct _ItemData
{
    const gchar *app_id;
    xmlNodePtr item_node;
    //xmlNodePtr group_node;
};

GType
applet_favorites_get_type (void) G_GNUC_CONST;
AppletFavorites *
applet_favorites_get_default (void);
GList *
applet_favorites_get_groups (AppletFavorites *self);
GList *
applet_favorites_get_group_apps (AppletFavorites *self, const gchar *group_name);
GroupData *
applet_favorites_add_group (AppletFavorites *self, const gchar *group_label);
GroupData *
applet_favorites_add_group_at_position (AppletFavorites *self, 
                                        const gchar *group_label, 
                                        gint position);
gboolean applet_favorites_remove_group (AppletFavorites *self, const gchar *group_name);

gboolean
applet_favorites_set_group_label (AppletFavorites *self,
                                  const gchar *group_name,
                                  const gchar *group_label);
gchar *
applet_favorites_get_group_label (AppletFavorites *self, gchar *group_name);
gboolean
applet_favorites_add_app (AppletFavorites *self,
                          const gchar *group_name,
                          const gchar *app_id,
                          gboolean is_head);
gboolean
applet_favorites_remove_app (AppletFavorites *self,
                             gchar *group_name,
                             const gchar *app_id);
gboolean
applet_favorites_group_move (AppletFavorites *self,
                             const gchar   *source_group_name,
                             const gchar   *dest_group_name,
                             FavoritesSection   section,
                            MoveMode    mode);
gboolean
applet_favorites_app_move (AppletFavorites *self,
                           const gchar   *source_app_id,
                           const gchar   *dest_app_id,
                           MoveMode    mode);

gboolean 
applet_favorites_app_query_exists (AppletFavorites *self, CdosApp *app); 

GroupData *
applet_favorites_get_default_group (AppletFavorites *self);
GroupData *
applet_favorites_get_group_by_app_id (AppletFavorites *self, const gchar *app_id);
ItemData *
applet_favorites_get_item_by_app_id (AppletFavorites *self, const gchar *app_id);
FavoritesSection
applet_favorites_get_group_section (AppletFavorites *self, const gchar *group_name);
void
applet_favorites_set_group_section (AppletFavorites *self, const gchar *group_name, FavoritesSection section);

gboolean
applet_favorites_query_group_exist_by_label (AppletFavorites *self, const gchar *group_label);

GroupData *applet_favorites_get_group_by_label (AppletFavorites *self, const gchar *group_label);

gboolean applet_favorites_remove_group_by_label (AppletFavorites *self, const gchar *group_label);

gchar *
applet_favorites_get_group_name_by_label (AppletFavorites *self, const gchar *group_label);

gboolean
applet_favorites_set_group_label_translatable (AppletFavorites *self,
                                               const gchar *group_name,
                                               const gchar *translatable);

G_END_DECLS

#endif /* __APPLET_FAVORITES_H__ */
