/** 
 * Copyright (c) 2016-2018 CPU and Fundamental Software Research Center, CAS
 *  
 * This software is published by the license of CPU-OS License，you can use and 
 * distibuted this software under this License. See CPU-OS Liense for more detail.
 *
 * You should have receive a copy of CPU-OS License. If not, please contact us 
 * by email <support_os@cpu-os.ac.cn>  
 * 
 **/
//
// Created by zhangtao on 18-1-18.
// Email: zhangtao@cpu-os.ac.cn
//

#include <clutter/clutter.h>

#include "applet-menu-button-base.h"
#include "applet-menu-helper.h"
#include "applet-menu-private.h"
#include "applet-menu.h"

#define APPLET_MENU_BUTTON_BASE_GET_PRIVATE(obj)      \
(G_TYPE_INSTANCE_GET_PRIVATE((obj), APPLET_TYPE_MENU_BUTTON_BASE, AppletMenuButtonBasePrivate))

G_DEFINE_TYPE(AppletMenuButtonBase, applet_menu_button_base, ST_TYPE_BOX_LAYOUT);

struct _AppletMenuButtonBasePrivate
{

    ClutterActor            *icon;
    StLabel                 *label;
    const gchar             *tooltip_text;
    gint                    icon_size;
    AppletButtonLayoutMode  layout_mode; 
    gboolean                sensitive;
};
enum
{
    ACTIVATE,
    CANCEL,
    LAST_SIGNAL
};

static guint applet_menu_button_base_signals[LAST_SIGNAL] = { 0, };

static void applet_menu_button_base_dispose (GObject *object);
static void applet_menu_button_base_finalize (GObject *object);

static gboolean button_on_captured_event (ClutterActor *actor, ClutterEvent *event, gpointer user_data);
static gboolean button_key_release_event (ClutterActor *actor, ClutterKeyEvent *event);
static gboolean button_key_press_event (ClutterActor *actor, ClutterKeyEvent *event);

static void applet_menu_button_base_set_child_meta (AppletMenuButtonBase *self);

static void
applet_menu_button_base_init (AppletMenuButtonBase *self)
{
    AppletMenuButtonBasePrivate *priv;
    priv = self->priv = APPLET_MENU_BUTTON_BASE_GET_PRIVATE (self);

    priv->icon = CLUTTER_ACTOR(st_icon_new ());
    //st_icon_set_icon_type ((StIcon *)priv->icon, ST_ICON_FULLCOLOR);
    priv->icon_size = DEFAULT_ICON_SIZE;
    priv->label = ST_LABEL(st_label_new (NULL));
    priv->sensitive = TRUE;
    
    applet_menu_button_base_set_icon_size (self, priv->icon_size);
    applet_menu_button_base_set_layout_mode (self, BUTTON_LAYOUT_H);
    
    clutter_actor_set_reactive (CLUTTER_ACTOR (self), TRUE);
    st_widget_set_track_hover (ST_WIDGET (self), TRUE);
    st_widget_set_can_focus (ST_WIDGET (self), TRUE);

    menu_container_add_actor ((ClutterActor *)self, 
                              CLUTTER_ACTOR (priv->icon), 
                              NULL);
    menu_container_add_actor ((ClutterActor *)self, 
                              CLUTTER_ACTOR (priv->label), 
                              "y-align", CLUTTER_ACTOR_ALIGN_CENTER,
                              NULL);

    g_signal_connect (self, 
                      "captured-event", 
                      G_CALLBACK (button_on_captured_event), 
                      self); 
}

static void
applet_menu_button_base_class_init (AppletMenuButtonBaseClass *klass)
{

    GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
    g_type_class_add_private (klass, sizeof (AppletMenuButtonBasePrivate));
    ClutterActorClass *actor_class = CLUTTER_ACTOR_CLASS (klass);

    gobject_class->dispose = applet_menu_button_base_dispose;
    gobject_class->finalize = applet_menu_button_base_finalize;
    actor_class->key_release_event = button_key_release_event;
    actor_class->key_press_event = button_key_press_event;

    applet_menu_button_base_signals[ACTIVATE] = g_signal_new (
			"activate", 
            G_TYPE_FROM_CLASS (gobject_class), 
			G_SIGNAL_RUN_LAST, 
            G_STRUCT_OFFSET (AppletMenuButtonBaseClass, activate), 
			NULL, 
			NULL, 
			g_cclosure_marshal_VOID__VOID, 
            G_TYPE_NONE, 
            0);
    applet_menu_button_base_signals[CANCEL] = g_signal_new (
			"cancel", 
            G_TYPE_FROM_CLASS (gobject_class), 
			G_SIGNAL_RUN_LAST, 
            G_STRUCT_OFFSET (AppletMenuButtonBaseClass, cancel), 
			NULL, 
			NULL, 
			g_cclosure_marshal_VOID__VOID, 
            G_TYPE_NONE, 
            0);        
}

static void
applet_menu_button_base_dispose (GObject *object)
{
    AppletMenuButtonBase *self = APPLET_MENU_BUTTON_BASE (object);
    if (clutter_actor_has_key_focus ((ClutterActor *)self))
    {
        applet_menu_grab_key_focus_main_box ();
    }   
    G_OBJECT_CLASS (applet_menu_button_base_parent_class)->dispose (object);
}

static void
applet_menu_button_base_finalize (GObject *object)
{
    G_OBJECT_CLASS (applet_menu_button_base_parent_class)->finalize (object);
}

static gboolean 
button_on_captured_event (ClutterActor *actor, ClutterEvent *event, gpointer user_data)
{
    AppletMenuButtonBase *self = user_data;
    guint32 button;
    ClutterEventType type =  clutter_event_type(event);
    ClutterModifierType state = clutter_event_get_state(event);

    if (type == CLUTTER_BUTTON_PRESS)
    {
        button = clutter_event_get_button(event);
        if (button == 1 || state & CLUTTER_BUTTON1_MASK)
        {
            st_widget_add_style_pseudo_class ((StWidget *)self, "click");

        }
    }
    else if (type == CLUTTER_BUTTON_RELEASE)
    {
        button = clutter_event_get_button(event);
        // Make sure the button has pseduo class "click" or pressed. 
        if ((button == 1 || state & CLUTTER_BUTTON1_MASK) && st_widget_has_style_pseudo_class ((StWidget *)self, "click"))
        {
            //g_signal_emit (self, applet_menu_button_base_signals[ACTIVATE], 0);
            applet_menu_button_base_activate (self);
        }
        st_widget_remove_style_pseudo_class ((StWidget *)self, "click"); 
        //st_widget_add_style_pseudo_class (self, "selected"); 
    }
    else if (type == CLUTTER_LEAVE)
    {
        st_widget_remove_style_pseudo_class ((StWidget *)self, "click"); 
    }
    return FALSE;
}

static gboolean
button_key_release_event (ClutterActor *actor, ClutterKeyEvent *event)
{
    AppletMenuButtonBase *self = APPLET_MENU_BUTTON_BASE(actor);
    if (event->keyval == CLUTTER_KEY_Return || event->keyval == CLUTTER_KEY_BackSpace)
    {
        CdosGlobal *global = cdos_global_get();
        ClutterStage *global_stage = cdos_global_get_stage (global);
        clutter_stage_set_key_focus (global_stage, (ClutterActor *)self);
        g_signal_emit (self, applet_menu_button_base_signals[ACTIVATE], 0);
        //applet_menu_button_base_activate (self);
        return TRUE;
    }
    return FALSE;
}

static gboolean 
button_key_press_event (ClutterActor *actor, ClutterKeyEvent *event)
{
    AppletMenuButtonBase *self = APPLET_MENU_BUTTON_BASE(actor);
    if (event->keyval == CLUTTER_KEY_Escape)
    {
        //ClutterStage *global_stage = cdos_global_get_stage (global);
        //clutter_stage_set_key_focus (global_stage, NULL);
        applet_menu_grab_key_focus_main_box ();
        g_signal_emit (self, applet_menu_button_base_signals[CANCEL], 0);
        return TRUE;    
    }
    return FALSE;
}

static void 
applet_menu_button_base_set_child_meta (AppletMenuButtonBase *self)
{
    AppletMenuButtonBasePrivate *priv = self->priv;
    if (priv->layout_mode == BUTTON_LAYOUT_V)
    {
        menu_container_set_layout_child ((ClutterActor *)self, (ClutterActor *)priv->icon, 
                    "x-fill", FALSE, 
                    "y-fill", FALSE, 
                    //"x-align", ST_ALIGN_MIDDLE,
                    NULL);
        menu_container_set_layout_child ((ClutterActor *)self, (ClutterActor *)priv->label,
                    "x-fill", FALSE, 
                    "y-fill", FALSE, 
                    "x-align", ST_ALIGN_MIDDLE,
                    NULL);
        clutter_actor_set_x_align ((ClutterActor *)priv->label, CLUTTER_ACTOR_ALIGN_CENTER);
    }
    else if (priv->layout_mode == BUTTON_LAYOUT_H)
    {
        menu_container_set_layout_child ((ClutterActor *)self, (ClutterActor *)priv->icon, 
                    "x-fill", FALSE, 
                    "y-fill", FALSE, 
                    //"x-align", ST_ALIGN_START,
                    NULL);
        clutter_actor_set_x_expand (priv->icon, FALSE);
        menu_container_set_layout_child ((ClutterActor *)self, (ClutterActor *)priv->label, 
                    "expand", TRUE,
                    "x-fill", TRUE, 
                    "y-fill", FALSE, 
                    //"x-align", ST_ALIGN_START,
                    NULL);                        
        clutter_actor_set_x_expand ((ClutterActor *)priv->label, TRUE);
        clutter_actor_set_x_align ((ClutterActor *)priv->label, CLUTTER_ACTOR_ALIGN_FILL);
    }
}

void    
applet_menu_button_base_set_icon_by_name (AppletMenuButtonBase *self, const gchar *icon_name)
{
    g_return_if_fail (APPLET_IS_MENU_BUTTON_BASE(self));
    AppletMenuButtonBasePrivate *priv = self->priv;
    if (ST_IS_ICON (priv->icon))
        st_icon_set_icon_name ((StIcon *)priv->icon, icon_name);
}

void    
applet_menu_button_base_set_label_text (AppletMenuButtonBase *self, const gchar *text)
{
    g_return_if_fail (APPLET_IS_MENU_BUTTON_BASE(self));
    AppletMenuButtonBasePrivate *priv = self->priv;

    st_label_set_text ((StLabel *)priv->label, text);
}

void    
applet_menu_button_base_set_layout_mode (AppletMenuButtonBase *self, AppletButtonLayoutMode mode)
{
    g_return_if_fail (APPLET_IS_MENU_BUTTON_BASE(self));
    AppletMenuButtonBasePrivate *priv = self->priv;
    priv->layout_mode = mode;
    gboolean is_v = priv->layout_mode == BUTTON_LAYOUT_V;
    if (priv->layout_mode == BUTTON_LAYOUT_V)
    {
        st_widget_remove_style_pseudo_class ((StWidget *)self, "horizontal");
        st_widget_add_style_pseudo_class ((StWidget *)self, "vertical");
    } else {
        st_widget_remove_style_pseudo_class ((StWidget *)self, "vertical");
        st_widget_add_style_pseudo_class ((StWidget *)self, "horizontal");
    }

    st_box_layout_set_vertical ((StBoxLayout *)self, is_v);
    applet_menu_button_base_set_child_meta (self);
}

AppletButtonLayoutMode 
applet_menu_button_base_get_layout_mode (AppletMenuButtonBase *self)
{
    g_return_if_fail (APPLET_IS_MENU_BUTTON_BASE(self));
    AppletMenuButtonBasePrivate *priv = self->priv;
    return priv->layout_mode;
}

void    
applet_menu_button_base_set_sensitive (AppletMenuButtonBase *self, gboolean sensitive)
{
    g_return_if_fail (APPLET_IS_MENU_BUTTON_BASE(self));
    AppletMenuButtonBasePrivate *priv = self->priv;
    if (priv->sensitive == sensitive) 
		return;

	priv->sensitive = sensitive;
	clutter_actor_set_reactive((ClutterActor *)self, sensitive);
	st_widget_set_can_focus((StWidget *)self, sensitive);
	if (!sensitive) 
		st_widget_add_style_pseudo_class((StWidget *)self, "insensitive");
	else 	
		st_widget_remove_style_pseudo_class((StWidget *)self, "insensitive");
}

void
applet_menu_button_base_set_icon (AppletMenuButtonBase *self, ClutterActor *icon)
{
    g_return_if_fail (APPLET_IS_MENU_BUTTON_BASE(self));
    AppletMenuButtonBasePrivate *priv = self->priv;
    if (priv->icon){
        clutter_actor_remove_child ((ClutterActor *)self, (ClutterActor *)priv->icon);
    }
    priv->icon = icon;
    clutter_actor_insert_child_at_index ((ClutterActor *)self, priv->icon, 0);
    applet_menu_button_base_set_child_meta (self);
}

ClutterActor *
applet_menu_button_base_get_icon (AppletMenuButtonBase *self)
{
    g_return_val_if_fail (APPLET_IS_MENU_BUTTON_BASE(self), NULL);
    AppletMenuButtonBasePrivate *priv = self->priv;
    return priv->icon;
}

StWidget *
applet_menu_button_base_get_label (AppletMenuButtonBase *self)
{
    g_return_val_if_fail (APPLET_IS_MENU_BUTTON_BASE(self), NULL);
    AppletMenuButtonBasePrivate *priv = self->priv;
    return (StWidget *)priv->label;
}

void    
applet_menu_button_base_set_icon_size (AppletMenuButtonBase *self, gint icon_size)
{
    g_return_if_fail (APPLET_IS_MENU_BUTTON_BASE(self));
    AppletMenuButtonBasePrivate *priv = self->priv;
    priv->icon_size = icon_size;
    if (ST_IS_ICON (priv->icon))
    {
        st_icon_set_icon_size ((StIcon *)priv->icon, icon_size);
    } else {
        clutter_actor_set_size ((ClutterActor *)priv->icon, (float)priv->icon_size, (float)priv->icon_size);
    }
}

gint 
applet_menu_button_base_get_icon_size (AppletMenuButtonBase *self)
{
    g_return_val_if_fail (APPLET_IS_MENU_BUTTON_BASE(self), 0);
    AppletMenuButtonBasePrivate *priv = self->priv;
    return priv->icon_size;
}

void    
applet_menu_button_base_set_tooltip (AppletMenuButtonBase *self, const gchar *text)
{
    g_return_if_fail (APPLET_IS_MENU_BUTTON_BASE(self));
    AppletMenuButtonBasePrivate *priv = self->priv;
    if (text == NULL) 
    {
        st_widget_set_has_tooltip (ST_WIDGET (self), FALSE);
    } 
    else if (priv->sensitive ==  TRUE) 
    {
        st_widget_set_has_tooltip (ST_WIDGET (self), TRUE);
        st_widget_set_tooltip_text (ST_WIDGET (self), text);
        priv->tooltip_text = text; 
    } 
    else 
    {
        g_warning ("The widget is sensitive!");
    }
}

const gchar *    
applet_menu_button_base_get_tooltip (AppletMenuButtonBase *self)
{
    g_return_val_if_fail (APPLET_IS_MENU_BUTTON_BASE(self), NULL);
    AppletMenuButtonBasePrivate *priv = self->priv;
    return priv->tooltip_text;
}

void
applet_menu_button_base_activate (AppletMenuButtonBase *self)
{
    g_return_if_fail (APPLET_IS_MENU_BUTTON_BASE(self));
    applet_menu_grab_key_focus_main_box ();
    g_signal_emit (self, applet_menu_button_base_signals[ACTIVATE], 0);
}

void
applet_menu_button_base_set_tooltip_state (AppletMenuButtonBase *self, gboolean enabled)
{
    g_return_if_fail (APPLET_IS_MENU_BUTTON_BASE(self));
    AppletMenuButtonBasePrivate *priv = self->priv;
    gboolean sensitive = priv->sensitive;
    if (sensitive == FALSE || enabled == FALSE)
    {
        st_widget_set_has_tooltip (ST_WIDGET (self), FALSE);
        return ;
    }
    if (enabled)
    {
        applet_menu_button_base_set_tooltip (self, priv->tooltip_text);
        st_widget_hide_tooltip (ST_WIDGET (self));
    }

}

