/** 
 * Copyright (c) 2016-2018 CPU and Fundamental Software Research Center, CAS
 *  
 * This software is published by the license of CPU-OS License，you can use and 
 * distibuted this software under this License. See CPU-OS Liense for more detail.
 *
 * You should have receive a copy of CPU-OS License. If not, please contact us 
 * by email <support_os@cpu-os.ac.cn>  
 * 
 **/
//
// Created by zhangtao on 18-1-23.
// Email: zhangtao@cpu-os.ac.cn
//

#include <glib-object.h>
#include <gio/gio.h>
#include "cdos-global.h"

#include "applet-menu-helper.h"

void    
applet_menu_set_display_cursor (MetaCursor cursor)
{
    CdosGlobal *global = cdos_global_get();
    MetaDisplay *display = cdos_global_get_display (global);
    meta_display_set_cursor (display, cursor);
}

void 
menu_container_add_actor (ClutterActor *container, 
                          ClutterActor *actor,
                          const gchar *first_property_name, 
                          ...)
{
    g_return_if_fail (CLUTTER_IS_ACTOR (container));
    g_return_if_fail (CLUTTER_IS_ACTOR (actor));
    
    va_list var_args;

    clutter_actor_add_child (container, actor);
    if (first_property_name != NULL)
    {
        va_start (var_args, first_property_name);
        g_object_set_valist ((GObject *)actor, first_property_name, var_args);
        va_end (var_args);
    }
}

void 
menu_container_add_actor_at_index (ClutterActor *container, 
                                   ClutterActor *actor, 
                                   gint index, 
                                   const gchar *first_property_name, 
                                   ...)
{
    g_return_if_fail (CLUTTER_IS_ACTOR (container));
    g_return_if_fail (CLUTTER_IS_ACTOR (actor));
    
    va_list var_args;

    clutter_actor_insert_child_at_index (container, actor, index);
    if (first_property_name != NULL)
    {
        va_start (var_args, first_property_name);
        g_object_set_valist ((GObject *)actor, first_property_name, var_args);
        va_end (var_args);
    }
}


void 
menu_container_set_layout_child (ClutterContainer *container, 
                          ClutterActor *actor,
                          const gchar *first_property_name, 
                          ...)
{
    g_return_if_fail (CLUTTER_IS_CONTAINER (container));
    g_return_if_fail (CLUTTER_IS_ACTOR (actor));
    va_list var_args;
    ClutterChildMeta *child_meta;
    if (first_property_name != NULL)
    {
        child_meta = (ClutterChildMeta *)clutter_container_get_child_meta (container,actor);
        if (child_meta)
        {
            va_start (var_args, first_property_name);
            g_object_set_valist ((GObject *)child_meta, first_property_name, var_args);
            va_end (var_args);
        }   
    }
}

gchar *
applet_helper_get_xdg_desktop_dir (void)
{
    static gchar  *desktop_dir = NULL;
    if (desktop_dir)
    {
        return g_strdup (desktop_dir);
    }

    const gchar *home_path = g_get_home_dir ();
    gchar *path = g_strconcat (home_path, "/.config/user-dirs.dirs", NULL);
    GFile *file = g_file_new_for_path (path);
    gchar *tmp_dir = NULL;
    if (g_file_query_exists (file, NULL)) 
    {
        gchar *content = g_malloc (sizeof (gchar *) * 1);
        gsize length;
        gboolean flag = g_file_load_contents (file, NULL, &content, &length, NULL, NULL);
        gchar **lines = g_strsplit (content, "\n", -1);
        GRegex *regex = g_regex_new ("\"", G_REGEX_MULTILINE, G_REGEX_MATCH_NOTBOL, NULL);
        while (*lines) 
        {
            if (*lines[0] == '#') 
            {
                lines++;
                continue;
            }
            gchar *sta = g_strrstr (*lines, "XDG_DESKTOP_DIR");
            if (sta) 
            {
                gchar *tmp = g_strstr_len (sta, -1, "/");
                if (tmp){
                    tmp = g_regex_replace (regex, tmp, -1, 0, "", G_REGEX_MATCH_NOTBOL, NULL);
                    tmp_dir = g_strconcat (home_path, tmp, NULL);
                    g_free (tmp);  
                }
                break ;
            }
            lines++;    
        }
        //if free lines using "g_strfreev", it will be crash;
        g_free (content);
        g_regex_unref (regex);
    }
    g_object_unref (file);
    g_free (path);

    if (tmp_dir == NULL)
        tmp_dir = g_strconcat (home_path, "/Desktop", NULL);
    desktop_dir = tmp_dir;
    return g_strdup (desktop_dir);
}

void
applet_helper_add_focus_group (StWidget *actor)
{
    g_return_if_fail (ST_IS_WIDGET (actor));
    CdosGlobal *global = cdos_global_get();
    ClutterStage *global_stage = cdos_global_get_stage (global);
    StFocusManager *focus_manager = st_focus_manager_get_for_stage (global_stage);
    st_focus_manager_add_group (focus_manager, actor);
}

void 
applet_helper_scroll_to_focused_actor (StScrollView  *scroll_view)
{
    g_return_if_fail (ST_IS_SCROLL_VIEW (scroll_view));
    ClutterActor *scroll_bar;
    StAdjustment *adjustment;
    ClutterStage *global_stage;
    ClutterActor *focused;
    gfloat       sv_y, focused_y, focused_h, delta;
    gboolean     update = FALSE;
    gdouble      page_size, value, new_value;
    gfloat       p = 0.5; 

    CdosGlobal *global = cdos_global_get();
    global_stage = cdos_global_get_stage (global);
    focused = clutter_stage_get_key_focus (CLUTTER_STAGE (global_stage));
    if (!clutter_actor_contains ((ClutterActor *)scroll_view, focused))
    {
        return ;
    }
    scroll_bar = st_scroll_view_get_vscroll_bar (scroll_view);
    adjustment = st_scroll_bar_get_adjustment (scroll_bar);
    clutter_actor_get_transformed_size (focused, NULL, &focused_h);
    clutter_actor_get_transformed_position (focused, NULL, &focused_y);
    clutter_actor_get_transformed_position (scroll_view, NULL, &sv_y);
    
    focused_y = focused_y - sv_y;

    g_object_get (adjustment,
                 "page-size", &page_size,
                 "value", &value,
                 NULL);
    if (focused_y + focused_h > page_size)
    {
        delta = focused_y - focused_h * p;
        update = TRUE;
    } 
    else if (focused_y <= 0)
    {
        delta = focused_y - focused_h * p;
        update = TRUE;
    }

    if (update)
    {
        new_value = value + delta >= 0 ? (value + delta) : 0;
        st_adjustment_set_value (adjustment, new_value);
    }
}

/** 
 * 根据上下位置触发滚动条的滚动事件，当鼠标进入上下位置hotarea_y的距离范围时，滚动条自动滚动到相应位置。
 * 
 */
void 
applet_helper_scroll_by_handle (StScrollView  *scroll_view, gfloat handle_y, gfloat hotarea_y)
{
    return;
    /*
    g_return_if_fail (ST_IS_SCROLL_VIEW (scroll_view));
    ClutterActor *scroll_bar;
    StAdjustment *adjustment;
    
    gfloat       sv_y, sv_h, delta = 0;
    gboolean     update = FALSE;
    gdouble      value, new_value;
    
    clutter_actor_get_transformed_position (scroll_view, NULL, &sv_y);
    clutter_actor_get_transformed_size (scroll_view, NULL, &sv_h);
    if ((handle_y > sv_y + hotarea_y && handle_y < sv_y + sv_h - hotarea_y))
    {
        return ;
    }
    scroll_bar = st_scroll_view_get_vscroll_bar (scroll_view);
    adjustment = st_scroll_bar_get_adjustment (scroll_bar);
    g_object_get (adjustment,
                 "value", &value,
                 NULL);
    if (handle_y <= sv_y + hotarea_y)
    {
        delta = sv_h - hotarea_y;
        update = TRUE;
    }
    else if (handle_y >= sv_y + sv_h - hotarea_y)
    {
        delta = -(sv_h - hotarea_y);
        update = TRUE;
    }
    if (update)
    {
        new_value = value + delta >= 0 ? (value + delta) : 0;
        st_adjustment_set_value (adjustment, new_value);
    }
    */
}
