/** 
 * Copyright (c) 2016-2018 CPU and Fundamental Software Research Center, CAS
 *  
 * This software is published by the license of CPU-OS License，you can use and 
 * distibuted this software under this License. See CPU-OS Liense for more detail.
 *
 * You should have receive a copy of CPU-OS License. If not, please contact us 
 * by email <support_os@cpu-os.ac.cn>  
 * 
 **/
//
// Created by zhangtao on 18-1-18.
// Email: zhangtao@cpu-os.ac.cn
//

#include "applet-menu-search.h"
#include "menu/applet-applications.h"
#include "applet-application-button.h"
#include "applet-menu-private.h"
#include "applet-menu.h"
#include "applet-menu-helper.h"

#define APPLET_MENU_SEARCH_GET_PRIVATE(obj)      \
(G_TYPE_INSTANCE_GET_PRIVATE((obj), APPLET_TYPE_MENU_SEARCH, AppletMenuSearchPrivate))

G_DEFINE_TYPE(AppletMenuSearch, applet_menu_search, G_TYPE_OBJECT);

struct _AppletMenuSearchPrivate
{
    StEntry         *entry; 
    StScrollView    *results_box; 
    StBoxLayout     *results_list; 
    StBoxLayout     *results_none;
    StIcon          *primary_icon;
    StIcon          *secondary_icon;
};
enum {
    SEARCH, 
    SEARCH_FINISHED,
    LAST_SIGNAL 
};
static guint signals[LAST_SIGNAL] = {0};

static void applet_menu_search_dispose (GObject *object);
static void applet_menu_search_finalize (GObject *object);

static void menu_search_on_text_changed (GObject    *object,
                                        GParamSpec *pspec,
                                        AppletMenuSearch *self);
static void menu_search_on_primary_icon_clicked (StEntry *entry, AppletMenuSearch *self);
static void menu_search_on_secondary_icon_clicked (StEntry *entry, AppletMenuSearch *self);

static void menu_search_begin (AppletMenuSearch *self);
static void menu_search_clear_results (AppletMenuSearch *self);
static void menu_search_render_list (AppletMenuSearch *self, GList *results);
static AppletApplicationButton * 
menu_search_get_list_item (AppletMenuSearch *self, CdosApp *app);
static void menu_search_add_list_item (AppletMenuSearch *self, AppletApplicationButton *button);
static gboolean menu_search_on_key_release (ClutterActor *entry, ClutterKeyEvent *event, AppletMenuSearch *self);
static gboolean
main_box_on_button_release (ClutterActor *actor, ClutterEvent *event, gpointer user_data);
static void
stage_on_key_focus_changed  (GObject    *gobject,
                            GParamSpec  *pspec,
                            gpointer    user_data);
static void
scroll_bar_on_changed (GObject     *object,
                      GParamSpec  *pspec,
                      gpointer user_data);
static void
applet_menu_search_init (AppletMenuSearch *self)
{
    AppletMenuSearchPrivate *priv;
    priv = self->priv = APPLET_MENU_SEARCH_GET_PRIVATE (self);

    priv->entry = ST_ENTRY(st_entry_new (NULL));
    st_entry_set_hint_text (priv->entry, _("Search..."));
    ClutterActor *clutter_text = st_entry_get_clutter_text (priv->entry);
    clutter_text_set_max_length ((ClutterText *)clutter_text, 32);
    st_widget_set_track_hover ((StWidget *)priv->entry, TRUE);
    st_widget_add_style_class_name ((StWidget *)priv->entry, "menu-search-entry");

    StIcon *primary_icon = ST_ICON (st_icon_new ());
    priv->primary_icon = primary_icon;
    //st_icon_set_icon_type (primary_icon, ST_ICON_FULLCOLOR); 
    st_icon_set_icon_size (primary_icon, 16); 
    st_icon_set_icon_name (primary_icon, "found-entry-find"); 
    st_entry_set_primary_icon (priv->entry, (ClutterActor *)primary_icon); 
    st_widget_set_style_class_name ((StWidget *)primary_icon, "menu-search-entry-search-icon");

    StIcon *secondary_icon = ST_ICON (st_icon_new ()); 
    priv->secondary_icon = secondary_icon;
    //st_icon_set_icon_type (secondary_icon, ST_ICON_FULLCOLOR); 
    st_icon_set_icon_size (secondary_icon, 16); 
    st_icon_set_icon_name (secondary_icon, "editclear");
    st_entry_set_secondary_icon (priv->entry, (ClutterActor *)secondary_icon);
    st_widget_set_style_class_name ((StWidget *)secondary_icon, "menu-search-entry-icon"); 
    st_widget_set_track_hover ((StWidget *)secondary_icon, TRUE);
    clutter_actor_hide ((ClutterActor *)secondary_icon);

    /*g_signal_connect (priv->entry, 
                     "notify::text", 
                     G_CALLBACK (menu_search_on_text_changed), 
                     self);*/
    g_signal_connect (priv->entry, 
                     "primary-icon-clicked", 
                     G_CALLBACK (menu_search_on_primary_icon_clicked), 
                     self);
    g_signal_connect (priv->entry, 
                     "secondary-icon-clicked", 
                     G_CALLBACK (menu_search_on_secondary_icon_clicked), 
                     self);
    g_signal_connect ((ClutterActor *)priv->entry, 
                     "key-release-event", 
                     G_CALLBACK (menu_search_on_key_release), 
                     self);

    priv->results_box = g_object_new(ST_TYPE_SCROLL_VIEW,
                                    "style-class" , "vfade menu-applications-scrollbox",
                                    "x-expand", TRUE,
                                    "y-expand", TRUE,
                                    "x-align", CLUTTER_ACTOR_ALIGN_FILL,
                                    "y-align", CLUTTER_ACTOR_ALIGN_FILL,
                                    NULL);
    st_scroll_view_set_policy (priv->results_box, GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC); 
    ClutterActor *scroll_bar = st_scroll_view_get_vscroll_bar (priv->results_box);

    StAdjustment *adjustment;
    adjustment = st_scroll_bar_get_adjustment ((StScrollBar *)scroll_bar);
    g_signal_connect (adjustment, 
                     "notify::value", 
                     G_CALLBACK (scroll_bar_on_changed), 
                     self);

    priv->results_list = g_object_new (ST_TYPE_BOX_LAYOUT,
                                       "x-expand", TRUE,
                                       "x-align", CLUTTER_ACTOR_ALIGN_FILL,
                                       "y-expand", TRUE,
                                       "y-align", CLUTTER_ACTOR_ALIGN_FILL,
                                       "style-class", "menu-search-results-list",
                                       NULL);
    applet_helper_add_focus_group ((StWidget *)priv->results_list);
    st_box_layout_set_vertical (priv->results_list, TRUE);
    st_scroll_view_add_child (priv->results_box, (ClutterActor *)priv->results_list);
    
    priv->results_none = ST_BOX_LAYOUT (st_box_layout_new ());
    st_widget_set_style_class_name ((StWidget *)priv->results_none, "menu-results-none");
    StLabel *label = ST_LABEL(st_label_new (_("No match for search terms")));
    //clutter_text_set_markup ((ClutterText *)st_label_get_clutter_text (label), "<b>N</b>one <b>R</b>esults");
    menu_container_add_actor ((ClutterActor *)priv->results_none, (ClutterActor *)label, 
                             "x-expand", TRUE, 
                             "x-align", CLUTTER_ACTOR_ALIGN_FILL,
                             NULL);
    StBoxLayout *main_box = applet_menu_get_main_box ();
    g_signal_connect (main_box, 
					  "button-release-event", 
                      G_CALLBACK (main_box_on_button_release), 
                      self);
    CdosGlobal *global = cdos_global_get();
    ClutterStage *global_stage = cdos_global_get_stage (global);
    g_signal_connect (global_stage, 
					 "notify::key-focus", 
					 G_CALLBACK (stage_on_key_focus_changed), 
					 self);                                  
}

static void
applet_menu_search_class_init (AppletMenuSearchClass *klass)
{

    GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
    g_type_class_add_private (klass, sizeof (AppletMenuSearchPrivate));

    gobject_class->dispose = applet_menu_search_dispose;
    gobject_class->finalize = applet_menu_search_finalize;

    signals[SEARCH] =
        g_signal_new("search", 
                     G_TYPE_FROM_CLASS(klass), 
                     G_SIGNAL_RUN_LAST, 
                     0, 
                     NULL, 
                     NULL, 
                     g_cclosure_marshal_VOID__VOID, 
                     G_TYPE_NONE, 
                     0);
    signals[SEARCH_FINISHED] =
        g_signal_new("search-finished", 
                     G_TYPE_FROM_CLASS(klass), 
                     G_SIGNAL_RUN_LAST, 
                     0, 
                     NULL, 
                     NULL, 
                     g_cclosure_marshal_VOID__VOID, 
                     G_TYPE_NONE, 
                     0);
}

static void
applet_menu_search_dispose (GObject *object)
{
    AppletMenuSearch *self = APPLET_MENU_SEARCH (object);
    AppletMenuSearchPrivate *priv = self->priv;
    StBoxLayout *main_box = applet_menu_get_main_box ();
    g_signal_handlers_disconnect_by_data ((ClutterActor *)main_box, self);
    CdosGlobal *global = cdos_global_get();
    ClutterStage *global_stage = cdos_global_get_stage (global);
    g_signal_handlers_disconnect_by_data (global_stage, self);
    
    if (priv->entry)
    {
        clutter_actor_destroy ((ClutterActor *)priv->entry);
        priv->entry = NULL;
    }
    if (priv->results_none)
    {
        clutter_actor_destroy ((ClutterActor *)priv->results_none);
        priv->results_none = NULL;
    }
    if (priv->results_box)
    {
        clutter_actor_destroy ((ClutterActor *)priv->results_box);
        priv->results_box = NULL;
    }
    G_OBJECT_CLASS (applet_menu_search_parent_class)->dispose (object);
}

static void
applet_menu_search_finalize (GObject *object)
{
    G_OBJECT_CLASS (applet_menu_search_parent_class)->finalize (object);
}

static void 
menu_search_begin (AppletMenuSearch *self)
{
    AppletMenuSearchPrivate *priv = self->priv; 
    
    const gchar *keyword = st_entry_get_text (priv->entry);
    if (g_strcmp0 (keyword, "") != 0)
    {
        menu_search_clear_results (self);

        AppletApplications *applications = applet_applications_get_default (); 
        GList *results = applet_applications_search (applications, keyword); 
        if (results)
        {
            menu_search_render_list (self, results);
            g_list_free (results);
        } else {
            menu_search_add_list_item (self, (AppletApplicationButton *)priv->results_none);
        }
        clutter_actor_show ((ClutterActor *)priv->secondary_icon);
        g_signal_emit (self, signals[SEARCH], 0);
    } else {
        menu_search_clear_results (self);
        menu_search_add_list_item (self, (AppletApplicationButton *)priv->results_none);
        clutter_actor_hide ((ClutterActor *)priv->secondary_icon);
        g_signal_emit (self, signals[SEARCH_FINISHED], 0);
    }
}

static void 
menu_search_clear_results (AppletMenuSearch *self)
{
    AppletMenuSearchPrivate *priv = self->priv;
    ClutterActor *first_child = clutter_actor_get_first_child ((ClutterActor *)priv->results_list);
    if (first_child == (ClutterActor *)priv->results_none)
    {
        g_object_ref (priv->results_none);
        clutter_actor_remove_child ((ClutterActor *)priv->results_list, (ClutterActor *)priv->results_none);
    } else {
        clutter_actor_destroy_all_children ((ClutterActor *)priv->results_list); 
    }
}

static void 
menu_search_render_list (AppletMenuSearch *self, GList *results)
{
    while (results != NULL)
    {
        CdosApp *app = results->data;
        AppletApplicationButton *button = menu_search_get_list_item (self, app);
        menu_search_add_list_item (self, (AppletApplicationButton *)button);
        results = results->next;
    }
}

static AppletApplicationButton* 
menu_search_get_list_item (AppletMenuSearch *self, CdosApp *app)
{
    AppletApplicationButton* button = applet_application_button_new (app, APP_ICON_SIZE_DEFAULT);
    //st_widget_add_style_class_name ((StWidget *)button, "menu-search-app-button");
    clutter_actor_set_name ((ClutterActor *)button, "menu-search-app-button");
    applet_application_button_set_draggable (button, FALSE); 
    applet_menu_button_base_set_layout_mode ((AppletMenuButtonBase *)button, BUTTON_LAYOUT_H); 

    return button;
}

static void 
menu_search_add_list_item (AppletMenuSearch *self, AppletApplicationButton *button)
{
    AppletMenuSearchPrivate *priv = self->priv;
    clutter_actor_set_x_expand ((ClutterActor *)button, TRUE);
    menu_container_add_actor ((ClutterActor *)priv->results_list, (ClutterActor *)button, 
                              "x-align", CLUTTER_ACTOR_ALIGN_FILL,
                              "y-align", CLUTTER_ACTOR_ALIGN_START, 
                              NULL);
}

static void 
menu_search_on_text_changed (GObject    *object,
                            GParamSpec *pspec,
                            AppletMenuSearch *self)
{
    menu_search_begin (self);
}

static void 
menu_search_on_primary_icon_clicked (StEntry *entry, AppletMenuSearch *self)
{
    menu_search_begin (self);
}

static void 
menu_search_on_secondary_icon_clicked (StEntry *entry, AppletMenuSearch *self)
{
    AppletMenuSearchPrivate *priv = self->priv;
    st_entry_set_text (priv->entry, NULL);
    menu_search_clear_results (self);
    menu_search_add_list_item (self, (AppletApplicationButton *)priv->results_none); 
    clutter_actor_hide ((ClutterActor *)priv->secondary_icon);
    g_signal_emit (self, signals[SEARCH_FINISHED], 0);
}

static gboolean 
menu_search_on_key_release (ClutterActor *entry, ClutterKeyEvent *event, AppletMenuSearch *self)
{
    AppletMenuSearchPrivate *priv = self->priv;
    //此处不能使用clutter_actor_has_key_focus进行判断
    if (st_widget_has_style_pseudo_class ((StWidget *)entry, "focus"))
    {   
        /*if (event->keyval == CLUTTER_KEY_Return)
        {
            menu_search_begin (self);
            return TRUE;
        }*/
        if (event->keyval == CLUTTER_KEY_Delete)
        {
            st_entry_set_text (priv->entry, NULL);
            g_signal_emit (self, signals[SEARCH_FINISHED], 0);
        } else {
            menu_search_begin (self);
        }
        return TRUE;
    }
    return FALSE;
}

static gboolean
main_box_on_button_release (ClutterActor *actor, ClutterEvent *event, gpointer user_data)
{
    AppletMenuSearch *self = APPLET_MENU_SEARCH (user_data);
    AppletMenuSearchPrivate *priv = self->priv;
    ClutterActor *source = clutter_event_get_source (event);
    ClutterActor *ct = st_entry_get_clutter_text (priv->entry);
    if (!clutter_actor_has_key_focus (ct))
    {
        return FALSE;
    }
    if (!clutter_actor_contains ((ClutterActor *)priv->entry, source) && APPLET_IS_MENU_BUTTON_BASE (source) == FALSE)
    {
        applet_menu_grab_key_focus_main_box ();
        return TRUE;
    }
    return FALSE;
}

static void
stage_on_key_focus_changed  (GObject    *gobject,
                            GParamSpec  *pspec,
                            gpointer    user_data)
{
    AppletMenuSearch *self = APPLET_MENU_SEARCH (user_data);
    AppletMenuSearchPrivate *priv = self->priv;
    ClutterActor *focused;
    focused = clutter_stage_get_key_focus (CLUTTER_STAGE (gobject));
    if (APPLET_IS_APPLICATION_BUTTON (focused))
    {
        applet_helper_scroll_to_focused_actor (priv->results_box);
    }   
}

static void
scroll_bar_on_changed (GObject     *object,
                      GParamSpec  *pspec,
                      gpointer user_data)
{
    AppletMenuSearch *self = APPLET_MENU_SEARCH (user_data);
    AppletMenuSearchPrivate *priv = self->priv;
    GList *children, *l;
    children = l = clutter_actor_get_children ((ClutterActor *)priv->results_list);
    while (children != NULL)
    {
        ClutterActor *button = children->data;
        if (APPLET_IS_APPLICATION_BUTTON (button))
        {
            CdosPopupMenu *context_menu = applet_application_button_get_context_menu ((AppletApplicationButton *)button);
            if (CDOS_POPUP_MENU_BASE (context_menu)->is_open)
            {
                cdos_popup_menu_close (CDOS_POPUP_MENU_BASE (context_menu), FALSE);
            }
        }
        children = children->next;
    }
    g_list_free (l);
}

AppletMenuSearch *
applet_menu_search_new (void)
{
    AppletMenuSearch *menu_search ;
    menu_search = g_object_new (APPLET_TYPE_MENU_SEARCH, NULL);

    return menu_search;
}

StEntry * 
applet_menu_search_get_entry (AppletMenuSearch *self)
{
    g_return_val_if_fail (APPLET_IS_MENU_SEARCH (self), NULL);
    AppletMenuSearchPrivate *priv = self->priv;
    return priv->entry;
}

StWidget *
applet_menu_search_get_results_box (AppletMenuSearch *self)
{
    g_return_val_if_fail (APPLET_IS_MENU_SEARCH (self), NULL);
    AppletMenuSearchPrivate *priv = self->priv;
    return (StWidget *)priv->results_box;
}

void
applet_menu_search_cancel_search (AppletMenuSearch *self)
{
    g_return_if_fail (APPLET_IS_MENU_SEARCH (self));
    AppletMenuSearchPrivate *priv = self->priv;
    st_entry_set_text (priv->entry, "");
    clutter_actor_hide ((ClutterActor *)priv->secondary_icon);
    ClutterStage *global_stage = cdos_global_get_stage (cdos_global_get());
    clutter_stage_set_key_focus (global_stage, NULL);
    menu_search_clear_results (self);
}

void
applet_menu_search_start_key_focus (AppletMenuSearch *self)
{
    g_return_if_fail (APPLET_IS_MENU_SEARCH (self));
    AppletMenuSearchPrivate *priv = self->priv;
    GList *children = clutter_actor_get_children (priv->results_list);
    if (children && children->data != priv->results_none)
    {
        clutter_actor_grab_key_focus ((ClutterActor *)children->data);
    }
    if (children)
        g_list_free (children);
}
