/** 
 * Copyright (c) 2016-2018 CPU and Fundamental Software Research Center, CAS
 *  
 * This software is published by the license of CPU-OS License，you can use and 
 * distibuted this software under this License. See CPU-OS Liense for more detail.
 *
 * You should have receive a copy of CPU-OS License. If not, please contact us 
 * by email <support_os@cpu-os.ac.cn>  
 * 
 **/
//
// Created by zhangtao on 18-1-18.
// Email: zhangtao@cpu-os.ac.cn
//
#include <math.h>
#include "common.h"

#include <stdlib.h>
#include <stdlib.h>
#include <syslog.h>
#include <act/act.h>
#include <meta/meta-cursor-tracker.h>
#include <meta/meta-monitor-manager.h>

#include "cdos-global.h"
#include "cdos-util.h"
#include "cdos-layout-manager.h"
#include "cdos-drag-drop.h"
#include "cdos-start.h"
#include "applet-menu.h"
#include "applet-menu-private.h"
#include "applet-menu-types.h"
#include "applet-shortcut-group.h"
#include "applet-applications-manager.h"
#include "applet-favorites-manager.h"
#include "applet-menu-helper.h"
#include "applet-menu-search.h"
#include "cdos-power-dialog.h"

#define APPLET_MENU_GET_PRIVATE(obj)      \
(G_TYPE_INSTANCE_GET_PRIVATE((obj), APPLET_TYPE_MENU, AppletMenuPrivate))

G_DEFINE_TYPE(AppletMenu, applet_menu, CDOS_TYPE_ICONAPPLET);

enum {
    VIEW_MODE_CHANGED,
    PANE_SWITCH,
    LAST_SIGNAL
};

struct _AppletMenuPrivate
{
    StBoxLayout     *main_box;
    StWidget        *main_bin_box;
    StBoxLayout     *left_panel;
    StBoxLayout     *right_panel;
    StBoxLayout     *left_panel_top;
    StBoxLayout     *left_panel_middle;
    StBoxLayout     *left_panel_bottom;
    StBoxLayout     *right_panel_top;
    StBoxLayout     *pane_applications;
    StBoxLayout     *pane_favorites;
    StWidget        *pane_search;

    gboolean        can_close;

    MenuPane        menu_pane;
    ApplicationsGroupType apps_group_type;

    AppletShortcutGroup     *shortcut_group_apps;
    AppletShortcutGroup     *shortcut_group_expand;
    AppletShortcutGroup     *shortcut_group_others;
    gboolean                group_expand;

    CdosPopupMenuItem *switch_user_item;

    AppletApplicationsManager   *apps_manager;
    AppletFavoritesManager      *favorites_manager;
    AppletMenuSearch            *search;
    GSettings                   *settings;
    gboolean                     is_open;
    AppletShortcutButton        *pane_switch_button;
    AppletShortcutButton        *expand_button;
    gulong      user_changed_id;
    gulong      user_loaded_id;
    ActUser     *user;
};
static guint signals[LAST_SIGNAL] = {0};

static AppletMenu *default_menu;
static void applet_menu_dispose (GObject *object);
static void applet_menu_finalize (GObject *object);
static AppletMenu *applet_menu_new (void);
static void menu_settings_on_changed (GSettings *settings, 
                                     gchar *key, 
                                     AppletMenu *self);

static void menu_create_ui_frame (AppletMenu *self);
static void menu_resize_cancel (AppletMenu *self);

//for left panel shortcuts;
static void menu_add_apps_shortcut_group (AppletMenu *self);
static void menu_apps_shortcut_group_callback (AppletShortcutButton *button, AppletMenu *self);

static void menu_add_expand_group (AppletMenu *self);
static void menu_expand_group_callback (AppletShortcutButton *button, AppletMenu *self);
static void menu_expand_button_retract (AppletMenu *self);

static void menu_add_others_group (AppletMenu *self);
static void menu_others_group_user_add_menu_items (AppletMenu *self, AppletShortcutButton *button);
static void menu_others_group_file_callback (AppletShortcutButton *button, AppletMenu *self);
static void menu_others_group_settings_callback (AppletShortcutButton *button, AppletMenu *self);
static void menu_others_group_feedback_callback (AppletShortcutButton *button, AppletMenu *self);
static void menu_others_group_power_add_menu_items (AppletMenu *self, AppletShortcutButton *button);
static void menu_others_group_user_info_callback (AppletShortcutButton *button, AppletMenu *self);
static void menu_others_group_power_callback (AppletShortcutButton *button, AppletMenu *self);

static void menu_group_power_reboot_callback (CdosPopupMenuItem *item, AppletMenu *self);
static void menu_group_power_suspend_callback (CdosPopupMenuItem *item, AppletMenu *self);
static void menu_group_power_hibernate_callback (CdosPopupMenuItem *item, AppletMenu *self);
static void menu_group_power_off_callback (CdosPopupMenuItem *item, AppletMenu *self);

static void menu_group_user_switch_callback (CdosPopupMenuItem *item, AppletMenu *self);
static void menu_group_user_lock_callback (CdosPopupMenuItem *item, AppletMenu *self);
static void menu_group_user_logout_callback (CdosPopupMenuItem *item, AppletMenu *self);
static void menu_group_user_settings_callback (CdosPopupMenuItem *item, AppletMenu *self);

static void menu_pane_switch (AppletMenu *self, MenuPane pane);
static void menu_load_favorites (AppletMenu *self);
static void menu_on_monitors_changed (MetaDisplay *display, gpointer user_data);
static void menu_reset_right_box_width (AppletMenu *self);

//for search;
static void menu_search (AppletMenu *self);
static void menu_on_search (AppletMenuSearch *search, AppletMenu *self);
static void menu_on_search_finished (AppletMenuSearch *search, AppletMenu *self);

//for all applications
static void menu_load_applications (AppletMenu *self);

static void menu_user_face_setup (AppletMenu *self, AppletShortcutButton *button);

static void popup_menu_on_open_state_changed (CdosPopupMenuBase *popup_menu_base, 
                                             gboolean state, 
                                             AppletMenu *self);
static void
display_on_overlay_key_release (MetaDisplay *display, gpointer user_data);
static gboolean
main_box_on_navigate_focus (ClutterActor *actor, ClutterEvent *event, gpointer user_data);
static void
menu_left_panel_setup (AppletMenu *self);
static void 
menu_set_main_box_size (AppletMenu *self);

static gboolean 
right_panel_on_captured_event (ClutterActor *actor,
                               ClutterEvent *event,
                               gpointer      user_data)
{
    AppletMenu *self = APPLET_MENU (user_data);
    AppletMenuPrivate *priv = self->priv;
    ClutterEventType event_type;
    event_type = clutter_event_type (event);
    StEntry *entry = applet_menu_search_get_entry (priv->search);
    if (event_type == CLUTTER_BUTTON_PRESS || event_type == CLUTTER_BUTTON_RELEASE)
    {
        menu_expand_button_retract (self);
        if (applet_application_manager_get_is_scrolling (priv->apps_manager))
        {
            clutter_actor_grab_key_focus ((ClutterActor *)entry);
            return TRUE;
        }
    }
    return FALSE;
}

static void
bp_on_allocation_changed (ClutterActor *actor,
                          ClutterActorBox *box,
                          ClutterAllocationFlags flags,
                          gpointer user_data)
{
    menu_set_main_box_size ((AppletMenu *)user_data);
}

/**
 * 创建基本UI框架
 * 
*/
static void 
menu_create_ui_frame (AppletMenu *self)
{
    AppletMenuPrivate *priv = self->priv;

    CdosPopupMenu *popup_menu = cdos_applet_get_popup_menu ((CdosApplet *)self);
    priv->main_box = cdos_popup_menu_base_get_box ((CdosPopupMenuBase *)popup_menu);

    CdosBoxPointer *bp = cdos_popup_menu_get_box_pointer (popup_menu);
    st_widget_add_style_class_name ((StWidget *)bp, "menu-box-pointer");
    g_signal_connect (bp, 
                      "allocation-changed", 
                      G_CALLBACK (bp_on_allocation_changed), 
                      self);

    st_box_layout_set_vertical (priv->main_box, FALSE);
    clutter_actor_set_reactive(CLUTTER_ACTOR(priv->main_box),TRUE);
    st_widget_set_style_class_name ((StWidget *)priv->main_box, "menu-main-box");

    priv->main_bin_box = (StBoxLayout *)st_box_layout_new ();
    menu_container_add_actor (CLUTTER_ACTOR (priv->main_box), (ClutterActor *)priv->main_bin_box,
                              "x-expand", TRUE,
                              "y-expand", TRUE,
                              "x-align", CLUTTER_ACTOR_ALIGN_FILL,
                              "y-align", CLUTTER_ACTOR_ALIGN_FILL,
                              NULL);

    priv->left_panel = (StBoxLayout *)st_box_layout_new ();
    menu_left_panel_setup (self);

    priv->right_panel = (StBoxLayout *)st_box_layout_new ();
    st_box_layout_set_vertical (priv->right_panel, TRUE);
    menu_container_add_actor (CLUTTER_ACTOR (priv->main_bin_box), (ClutterActor *)priv->right_panel,
                              "x-expand", TRUE,
                              "y-expand", TRUE,
                              "x-align", CLUTTER_ACTOR_ALIGN_FILL,
                              "y-align", CLUTTER_ACTOR_ALIGN_FILL,
                              NULL);
    clutter_actor_set_flags (priv->right_panel, CLUTTER_ACTOR_REACTIVE);
    g_signal_connect (priv->right_panel, 
                      "captured-event", 
                      G_CALLBACK (right_panel_on_captured_event), 
                      self);
    st_widget_set_style_class_name ((StWidget *)priv->right_panel, "main-right-panel");

    applet_helper_add_focus_group ((StWidget *)priv->right_panel);
    //left panel
    priv->left_panel_top = (StBoxLayout *)st_box_layout_new ();
    priv->left_panel_middle = (StBoxLayout *)st_box_layout_new ();
    priv->left_panel_bottom = (StBoxLayout *)st_box_layout_new ();
    st_widget_set_style_class_name (ST_WIDGET(priv->left_panel_top), "menu-left-panel-top");
    st_widget_set_style_class_name (ST_WIDGET(priv->left_panel_middle), "menu-left-panel-middle");
    st_widget_set_style_class_name (ST_WIDGET(priv->left_panel_bottom), "menu-left-panel-bottom");
    menu_container_add_actor (CLUTTER_ACTOR (priv->left_panel), CLUTTER_ACTOR (priv->left_panel_top),
                    "x-expand", TRUE, 
                    "x-align", CLUTTER_ACTOR_ALIGN_FILL,
                    "y-align", CLUTTER_ACTOR_ALIGN_START,
                    NULL);
    st_box_layout_set_vertical (priv->left_panel_middle, FALSE);
    menu_container_add_actor (CLUTTER_ACTOR (priv->left_panel), CLUTTER_ACTOR (priv->left_panel_middle),
                    "x-expand", TRUE, 
                    "y-expand", TRUE,
                    "x-align", CLUTTER_ACTOR_ALIGN_FILL,
                    "y-align", CLUTTER_ACTOR_ALIGN_FILL,
                    NULL);
    st_box_layout_set_vertical (priv->left_panel_bottom, TRUE);      
    menu_container_add_actor (CLUTTER_ACTOR (priv->left_panel), CLUTTER_ACTOR (priv->left_panel_bottom),
                    "x-expand", TRUE, 
                    "x-align", CLUTTER_ACTOR_ALIGN_FILL,
                    "y-align", CLUTTER_ACTOR_ALIGN_END,  
                    NULL);     
    // right panel
    priv->right_panel_top = (StBoxLayout *)st_box_layout_new ();
    st_box_layout_set_vertical (priv->right_panel_top, FALSE);
    st_widget_set_style_class_name (ST_WIDGET(priv->right_panel_top), "menu-right-panel-top");
    menu_container_add_actor (CLUTTER_ACTOR (priv->right_panel), CLUTTER_ACTOR (priv->right_panel_top),
                    "x-expand", TRUE, 
                    "x-align", CLUTTER_ACTOR_ALIGN_FILL,
                    NULL);
    priv->pane_applications = (StBoxLayout *)st_box_layout_new ();
    menu_container_add_actor (CLUTTER_ACTOR (priv->right_panel), CLUTTER_ACTOR (priv->pane_applications),
                    "x-expand", TRUE,
                    "y-expand", TRUE, 
                    "x-align", CLUTTER_ACTOR_ALIGN_FILL, 
                    "y-align", CLUTTER_ACTOR_ALIGN_FILL, 
                    NULL);
    priv->pane_favorites = (StBoxLayout *)st_box_layout_new ();
    menu_container_add_actor (CLUTTER_ACTOR (priv->right_panel), CLUTTER_ACTOR (priv->pane_favorites),
                    "x-expand", TRUE, 
                    "x-align", CLUTTER_ACTOR_ALIGN_FILL, 
                    NULL);                
    priv->pane_search = g_object_new (ST_TYPE_BOX_LAYOUT,
                                      "x-expand", TRUE,
                                      "x-align", CLUTTER_ACTOR_ALIGN_FILL,
                                      NULL);
    _add_actor (priv->right_panel, priv->pane_search);

    clutter_actor_hide ((ClutterActor *)priv->pane_applications); 
    clutter_actor_hide ((ClutterActor *)priv->pane_favorites); 
    clutter_actor_hide ((ClutterActor *)priv->pane_search);
}

static void 
menu_resize_cancel (AppletMenu *self)
{
    AppletMenuPrivate *priv = self->priv;
    applet_menu_set_display_cursor (META_CURSOR_DEFAULT);
    menu_reset_right_box_width (self);
}

static void 
menu_set_main_box_size (AppletMenu *self)
{
    AppletMenuPrivate *priv = self->priv;
    CdosLayoutManager *layout_manager;
    const MetaRectangle *monitor;
    StThemeNode *theme_node;
    gint theme_height, h = -1;
    gfloat panel_height;
    StWidget *panel_box;

    layout_manager = cdos_layout_manager_get_default ();
    monitor = cdos_layout_manager_get_primary_monitor (layout_manager);
    if (!monitor)
        return;

    panel_box = cdos_layout_manager_get_panelbox (layout_manager);
    panel_height = clutter_actor_get_height ((ClutterActor *)panel_box);

    theme_node = st_widget_get_theme_node ((StWidget *)priv->main_box);
    theme_height = st_theme_node_get_height (theme_node);
    if (monitor->height < theme_height + (int)panel_height + 10)
    {
        h = monitor->height - (int)panel_height -10;
    }
    clutter_actor_set_height (priv->main_box, h);
}

static void
menu_on_monitors_changed (MetaDisplay *display, gpointer user_data)
{
    menu_set_main_box_size ((AppletMenu *)user_data);
}

static void
applet_menu_init (AppletMenu *self)
{
    AppletMenuPrivate *priv;
    priv = self->priv = APPLET_MENU_GET_PRIVATE (self);

    cdos_iconapplet_set_icon_name (CDOS_ICONAPPLET (self), "start-here");
    cdos_applet_set_popup_menu_active ((CdosApplet *)self, TRUE);
    st_widget_add_style_class_name (ST_WIDGET(self), "applet-box-menu");

    priv->settings = g_settings_new (CDOS_MENU_SCHEMA);
    g_signal_connect (priv->settings, 
                      "changed", 
                      G_CALLBACK (menu_settings_on_changed), 
                      self);
    if (default_menu == NULL)
    {
        default_menu = self;
    }

    priv->can_close = TRUE;
    priv->menu_pane = MENU_PANE_APPS;
    menu_create_ui_frame (self);
    //menu_reset_right_box_width (self);
    menu_set_main_box_size (self);
    
    menu_add_apps_shortcut_group (self);
    menu_add_expand_group (self);
    menu_add_others_group (self);
    menu_load_applications (self);

    //menu_load_favorites (self);
    menu_search (self);

    menu_pane_switch (self, priv->menu_pane);
    applet_menu_can_close (TRUE); 

    CdosGlobal *global = cdos_global_get();
    MetaDisplay *display = cdos_global_get_display (global);
    MetaMonitorManager *monitor_manager = meta_monitor_manager_get ();

    g_signal_connect (monitor_manager, 
                     "monitors-changed", 
                     G_CALLBACK (menu_on_monitors_changed), 
                     self);
    g_signal_connect (display, 
                     "overlay-key", 
                     G_CALLBACK (display_on_overlay_key_release), 
                     self);
    g_signal_connect (priv->main_box, 
                     "key-press-event", 
                     G_CALLBACK (main_box_on_navigate_focus), 
                     self);
    /* g_signal_connect (priv->left_panel, 
                     "key-press-event", 
                     G_CALLBACK (main_box_on_navigate_focus), 
                     self); */
    CdosPopupMenu *popup_menu = cdos_applet_get_popup_menu ((CdosApplet *)self);
    g_signal_connect (popup_menu,
                     "open-state-changed", 
                     G_CALLBACK (popup_menu_on_open_state_changed), 
                     self);
}

static void
applet_menu_class_init (AppletMenuClass *klass)
{

    GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
    g_type_class_add_private (klass, sizeof (AppletMenuPrivate));

    gobject_class->dispose = applet_menu_dispose;
    gobject_class->finalize = applet_menu_finalize;

    signals[VIEW_MODE_CHANGED] =
        g_signal_new("view-mode-changed",
                     G_TYPE_FROM_CLASS(klass),
                     G_SIGNAL_RUN_LAST,
                     0,
                     NULL,
                     NULL,
                     g_cclosure_marshal_VOID__INT,
                     G_TYPE_NONE,
                     1,
                     G_TYPE_UINT);
    signals[PANE_SWITCH] =
        g_signal_new("pane-switch",
                     G_TYPE_FROM_CLASS(klass),
                     G_SIGNAL_RUN_LAST,
                     0,
                     NULL,
                     NULL,
                     g_cclosure_marshal_VOID__INT,
                     G_TYPE_NONE,
                     1,
                     G_TYPE_UINT);
}

static void
applet_menu_dispose (GObject *object)
{
    AppletMenu *self = APPLET_MENU (object);
    AppletMenuPrivate *priv = self->priv;
    CdosGlobal *global = cdos_global_get();
    MetaDisplay *display = cdos_global_get_display (global);
    g_signal_handlers_disconnect_by_data ((ClutterActor *)display, self);
    ActUserManager *act_manager = act_user_manager_get_default ();
    g_signal_handlers_disconnect_by_data (act_manager, self);
    if (priv->user_changed_id > 0)
    {
        g_signal_handler_disconnect (priv->user, priv->user_changed_id); 
        priv->user_changed_id = 0;
    }
    if (priv->user_loaded_id > 0)
    {
        g_signal_handler_disconnect (priv->user, priv->user_loaded_id); 
        priv->user_loaded_id = 0;
    }

    g_clear_pointer (&priv->shortcut_group_apps, g_object_unref);
    g_clear_pointer (&priv->shortcut_group_expand, g_object_unref);
    g_clear_pointer (&priv->shortcut_group_others, g_object_unref);
    g_clear_pointer (&priv->apps_manager, g_object_unref);
    g_clear_pointer (&priv->favorites_manager, g_object_unref);
    /*if (priv->search)
    {
        clutter_actor_destroy ((ClutterActor *)priv->search);
        priv->search = NULL;

    };
    if (priv->main_bin_box)
     {
         clutter_actor_destroy ((ClutterActor *)priv->main_bin_box);
         priv->main_bin_box = NULL;
     } 

    if (priv->left_panel)
    {
        clutter_actor_destroy ((ClutterActor *)priv->left_panel);
        priv->left_panel = NULL;
    } */
    clutter_actor_destroy_all_children ((ClutterActor *)priv->main_box);    

    G_OBJECT_CLASS (applet_menu_parent_class)->dispose (object);
}

static void
applet_menu_finalize (GObject *object)
{
    AppletMenuPrivate *priv = APPLET_MENU (object)->priv;

    default_menu = NULL;

    G_OBJECT_CLASS (applet_menu_parent_class)->finalize (object);
}

static void 
menu_settings_on_changed (GSettings *settings, 
                         gchar *key, 
                         AppletMenu *self)
{
    AppletMenuPrivate *priv = self->priv;
    
}

//所有程序和收藏应用的切换按钮
static void 
menu_add_apps_shortcut_group (AppletMenu *self)
{
    AppletMenuPrivate *priv = self->priv;
    AppletShortcutButton *button ;
    priv->shortcut_group_apps = applet_shortcut_group_new ();

    button = applet_shortcut_group_add_button (priv->shortcut_group_apps, 
                "found-menu-categories", 
                _("Category Order"), 
                (GCallback)menu_apps_shortcut_group_callback, 
                self);
    //applet_menu_button_base_set_icon_size ((AppletMenuButtonBase *)button, 24);
    menu_container_add_actor ((ClutterActor *)priv->left_panel_top, (ClutterActor *)button,
                              "x-expand", TRUE,
                              "x-align", CLUTTER_ACTOR_ALIGN_START,
                              NULL);
    applet_menu_button_base_set_tooltip ((AppletMenuButtonBase *)button, _("Category Order"));
    applet_shortcut_button_set_tooltip_toggle (button, TRUE);  
    priv->pane_switch_button = button;
}

static void
menu_apps_shortcut_group_update (AppletMenu *self)
{
    AppletMenuPrivate *priv = self->priv;
    const gchar *icon_name = NULL;
    const gchar *label_text = NULL;
    gboolean show_frequently_used = FALSE;

    if (priv->menu_pane == MENU_PANE_APPS)
    {
        if (priv->apps_group_type == APPLICATIONS_GROUP_BY_LETTER)
        {
            icon_name = "found-menu-categories";
            label_text = _("Category Order");
        } else {
            icon_name = "found-menu-applications";
            label_text = _("Default Order");
        }
    }
    else if (priv->menu_pane == MENU_PANE_FAVORITES || priv->menu_pane == MENU_PANE_SEARCH)
    {
        icon_name = "found-menu-applications";
        label_text = _("Switch to favorites");
    }

    show_frequently_used = priv->apps_group_type == APPLICATIONS_GROUP_BY_LETTER ? TRUE : FALSE;

    applet_menu_button_base_set_icon_by_name ((AppletMenuButtonBase *)priv->pane_switch_button, icon_name);
    applet_menu_button_base_set_label_text ((AppletMenuButtonBase *)priv->pane_switch_button, label_text);
    applet_menu_button_base_set_tooltip ((AppletMenuButtonBase *)priv->pane_switch_button, label_text);
    applet_application_mananger_set_frequently_used_visible (priv->apps_manager, show_frequently_used);

    if (applet_shortcut_button_get_expand (priv->pane_switch_button))
        st_widget_set_has_tooltip (ST_WIDGET (priv->pane_switch_button), FALSE);
    if (priv->group_expand)
    {
        menu_expand_group_callback (priv->expand_button, self);
    }
}

static void 
menu_apps_shortcut_group_callback (AppletShortcutButton *button, AppletMenu *self)
{   
    AppletMenuPrivate *priv = self->priv;
    ApplicationsGroupType group_type;
    if (priv->menu_pane == MENU_PANE_APPS)
    {
        group_type = applet_applications_manager_get_group_type (priv->apps_manager);
        priv->apps_group_type = group_type == APPLICATIONS_GROUP_BY_LETTER ? APPLICATIONS_GROUP_BY_CATEGORY : APPLICATIONS_GROUP_BY_LETTER;    
    }
    menu_pane_switch (self, MENU_PANE_APPS);
    applet_applications_manager_set_group_type (priv->apps_manager, priv->apps_group_type);
    menu_apps_shortcut_group_update (self);
}

static void 
menu_add_expand_group (AppletMenu *self)
{
    AppletMenuPrivate *priv = self->priv;
    AppletShortcutButton *button ;
    priv->shortcut_group_expand = applet_shortcut_group_new ();
    priv->group_expand = applet_shortcut_group_get_expand (priv->shortcut_group_expand);
    button = applet_shortcut_group_add_button (priv->shortcut_group_expand, 
                NULL, 
                NULL, 
                (GCallback)menu_expand_group_callback, 
                self);
    st_widget_remove_style_class_name (button, "menu-shortcut-button");
    menu_container_add_actor ((ClutterActor *)priv->left_panel_middle, (ClutterActor *)button, 
                              "x-align", CLUTTER_ACTOR_ALIGN_FILL,
                              "y-align", CLUTTER_ACTOR_ALIGN_FILL,
                              "x-expand", TRUE, 
                              "y-expand", TRUE,
                              NULL);
    //applet_menu_button_base_set_tooltip ((AppletMenuButtonBase *)button, _("Expand"));
    priv->expand_button = button;
}

static void 
menu_expand_group_callback (AppletShortcutButton *button, AppletMenu *self)
{
    AppletMenuPrivate *priv = self->priv;
    priv->group_expand = !priv->group_expand;
    const gchar *label_text;
    if (priv->group_expand)
    {
        label_text = _("Retract");
        st_widget_add_style_class_name ((StWidget *)priv->left_panel, "main-left-panel-expand");
    } else {
        label_text = _("Expand");
        st_widget_remove_style_class_name ((StWidget *)priv->left_panel, "main-left-panel-expand");
    }
    //applet_menu_button_base_set_tooltip ((AppletMenuButtonBase *)button, label_text);
    applet_shortcut_group_set_expand (priv->shortcut_group_apps, priv->group_expand);
    applet_shortcut_group_set_expand (priv->shortcut_group_others, priv->group_expand);
}

//添加其它快捷操作按钮：用户详情，文件管理器，设置中心，帮助与反馈，电源管理；
static void 
menu_add_others_group (AppletMenu *self)
{
    AppletMenuPrivate *priv = self->priv;
    AppletShortcutButton *button_user, *button_file, *button_settings, *button_feedback, *button_power;
    priv->shortcut_group_others = applet_shortcut_group_new ();
    const gchar *user_name = cdos_user_get_display_name (priv->user);

    button_user = applet_shortcut_group_add_button (priv->shortcut_group_others, 
                "found-menu-user", 
                user_name, 
                NULL, 
                self);
    menu_user_face_setup (self, button_user);
    st_widget_add_style_class_name (button_user, "menu-button-user-face");
    menu_container_add_actor ((ClutterActor *)priv->left_panel_bottom, 
                              (ClutterActor *)button_user, 
                              "x-expand", TRUE,
                              "x-align", CLUTTER_ACTOR_ALIGN_FILL,
                              "y-align", CLUTTER_ACTOR_ALIGN_START,
                              NULL);
    applet_menu_button_base_set_tooltip ((AppletMenuButtonBase *)button_user, user_name);
    applet_shortcut_button_set_tooltip_toggle (button_user, TRUE);
    applet_shortcut_button_set_menu_enable (button_user, TRUE);
    menu_others_group_user_add_menu_items (self, button_user);

    button_file = applet_shortcut_group_add_button (priv->shortcut_group_others, 
                "found-menu-computer", 
                _("Computer"), 
                (GCallback)menu_others_group_file_callback, 
                self);
    //applet_menu_button_base_set_icon_size ((AppletMenuButtonBase *)button_file, 24);
    menu_container_add_actor ((ClutterActor *)priv->left_panel_bottom, 
                              (ClutterActor *)button_file, 
                              "x-expand", TRUE,
                              "x-align", CLUTTER_ACTOR_ALIGN_FILL,
                              "y-align", CLUTTER_ACTOR_ALIGN_START,
                              NULL);
    applet_menu_button_base_set_tooltip ((AppletMenuButtonBase *)button_file, _("Computer"));
    applet_shortcut_button_set_tooltip_toggle (button_file, TRUE);

    button_settings = applet_shortcut_group_add_button (priv->shortcut_group_others, 
                "found-menu-settings", 
                _("System Settings"), 
                (GCallback)menu_others_group_settings_callback, 
                self);
    //applet_menu_button_base_set_icon_size ((AppletMenuButtonBase *)button_settings, 24);            
    menu_container_add_actor ((ClutterActor *)priv->left_panel_bottom, 
                              (ClutterActor *)button_settings, 
                              "x-expand", TRUE,
                              "x-align", CLUTTER_ACTOR_ALIGN_FILL,
                              "y-align", CLUTTER_ACTOR_ALIGN_START,
                              NULL);
    applet_menu_button_base_set_tooltip ((AppletMenuButtonBase *)button_settings, _("System Settings"));
    applet_shortcut_button_set_tooltip_toggle (button_settings, TRUE);

    button_power = applet_shortcut_group_add_button (priv->shortcut_group_others, 
                "found-menu-power", 
                _("Power"), 
                (GCallback)menu_others_group_power_callback, 
                self);
    //applet_menu_button_base_set_icon_size ((AppletMenuButtonBase *)button_power, 24);
    menu_container_add_actor ((ClutterActor *)priv->left_panel_bottom, 
                              (ClutterActor *)button_power, 
                              "x-expand", TRUE,
                              "x-align", CLUTTER_ACTOR_ALIGN_FILL,
                              "y-align", CLUTTER_ACTOR_ALIGN_START,
                              NULL);
    applet_menu_button_base_set_tooltip ((AppletMenuButtonBase *)button_power, _("Power"));
    applet_shortcut_button_set_tooltip_toggle (button_power, TRUE);
    /*
    applet_shortcut_button_set_menu_enable (button_power, TRUE);
    menu_others_group_power_add_menu_items (self, button_power);
    */
}

static gboolean 
should_show_switch_user (void)
{
    ActUserManager *act_manager = act_user_manager_get_default ();
    GSList *user_list = act_user_manager_list_users (act_manager);
    guint len = g_slist_length (user_list);
    g_slist_free (user_list);
    return len > 1 ? TRUE : FALSE;
}

static void
users_on_changed (ActUserManager *gobject,
               ActUser        *user,
               gpointer        user_data)
{
    AppletMenu *self = APPLET_MENU (user_data);
    AppletMenuPrivate *priv = self->priv;
    if (can_switch_user() && should_show_switch_user())
    {
        clutter_actor_show (CLUTTER_ACTOR (priv->switch_user_item));
    }
    else
    {
        clutter_actor_hide (CLUTTER_ACTOR (priv->switch_user_item));
    }
}

static void
users_loaded (ActUserManager *gobject,
             GParamSpec *spec,
             gpointer        user_data)
{
    users_on_changed (gobject, NULL, user_data);
}

static void 
menu_others_group_user_add_menu_items (AppletMenu *self, AppletShortcutButton *button)
{
    CdosPopupMenuItem *item_switch, *item_lock, *item_logout, *item_settings;
    StWidget *item_actor;
    AppletMenuPrivate *priv = self->priv;
/*
    item_lock = applet_shortcut_button_add_menu_item (button, 
                                                       _("Lock Screen"), 
                                                       (GCallback)menu_group_user_lock_callback, 
                                                       self);
    item_actor = (StWidget *)item_lock;
    st_widget_set_can_focus (item_actor, TRUE);

    item_logout = applet_shortcut_button_add_menu_item (button, 
                                                        _("Log Out..."), 
                                                        (GCallback)menu_group_user_logout_callback, 
                                                        self);
    item_actor = (StWidget *)item_logout;
    st_widget_set_can_focus (item_actor, TRUE);

*/
    item_settings = applet_shortcut_button_add_menu_item (button, 
                                                          _("Account Details"), 
                                                          (GCallback)menu_group_user_settings_callback, 
                                                          self);
    item_actor = (StWidget *)item_settings;
    st_widget_set_can_focus (item_actor, TRUE);
/*
    item_switch = applet_shortcut_button_add_menu_item (button, 
                                                        _("Switch User"), 
                                                        (GCallback)menu_group_user_switch_callback, 
                                                        self);
    item_actor = (StWidget *)item_switch;
    st_widget_set_can_focus (item_actor, TRUE);
*/
    priv->switch_user_item = item_switch;
    clutter_actor_hide (CLUTTER_ACTOR (priv->switch_user_item));
    ActUserManager *act_manager = act_user_manager_get_default ();
    g_signal_connect (act_manager, 
                      "user-added", 
                      G_CALLBACK (users_on_changed), 
                      self);
    g_signal_connect (act_manager, 
                      "user-removed",
                      G_CALLBACK (users_on_changed), 
                      self);
    g_signal_connect (act_manager, 
                      "notify::is-loaded", 
                      G_CALLBACK (users_loaded), 
                      self);
}

static void 
menu_others_group_file_callback (AppletShortcutButton *button, AppletMenu *self)
{
    cdos_util_spawn_command_line("nemo computer:///");
    applet_menu_close (); 
}

static void 
menu_others_group_user_info_callback (AppletShortcutButton *button, AppletMenu *self)
{
    cdos_util_spawn_command_line("found-control-center user-accounts");
    applet_menu_close (); 
}
static void 
menu_others_group_settings_callback (AppletShortcutButton *button, AppletMenu *self)
{
    cdos_util_spawn_command_line("found-control-center");
    applet_menu_close (); 
}

static void 
menu_others_group_feedback_callback (AppletShortcutButton *button, AppletMenu *self)
{
    cdos_util_spawn_command_line("cdos-welcome");
    applet_menu_close (); 
}

//系统退出
static gboolean
power_dialog_run_idle_cb (AppletMenu *self)
{
    cdos_power_dialog_run (PA_NONE);
    return FALSE;
}

static void 
menu_others_group_power_callback (AppletShortcutButton *button, AppletMenu *self)
{
    g_idle_add (power_dialog_run_idle_cb, self);
    applet_menu_close (); 
}

static void 
menu_others_group_power_add_menu_items (AppletMenu *self, AppletShortcutButton *button)
{
    CdosPopupMenuItem *item_reboot, *item_sleep, *item_poweroff, *item_suspend;

    item_reboot = applet_shortcut_button_add_menu_item (button, 
                                                        _("Reboot"), 
                                                        (GCallback)menu_group_power_reboot_callback, 
                                                        self);
    st_widget_set_can_focus ((StWidget *)item_reboot, TRUE);

    if (check_acpi_supports ("S3") == TRUE)
    {
        item_suspend = applet_shortcut_button_add_menu_item (button, 
                                                       _("Suspend"), 
                                                       (GCallback)menu_group_power_suspend_callback, 
                                                       self);
        st_widget_set_can_focus ((StWidget *)item_suspend, TRUE);
    }
    if (check_acpi_supports ("S4") == TRUE)
    {
        item_sleep = applet_shortcut_button_add_menu_item (button, 
                                                       _("Hibernate"), 
                                                       (GCallback)menu_group_power_hibernate_callback, 
                                                       self);
        st_widget_set_can_focus ((StWidget *)item_sleep, TRUE);
    }
    item_poweroff = applet_shortcut_button_add_menu_item (button, 
                                                          _("Power Off..."), 
                                                          (GCallback)menu_group_power_off_callback, 
                                                          self);
    st_widget_set_can_focus ((StWidget *)item_poweroff, TRUE);

}

//重启
static void 
menu_group_power_reboot_callback (CdosPopupMenuItem *item, AppletMenu *self)
{
    cdos_util_spawn_command_line("dbus-send --session --print-reply --dest=\"org.gnome.SessionManager\" /org/gnome/SessionManager org.gnome.SessionManager.Reboot");
    applet_menu_close (); 
}

//挂起
static void
menu_group_power_suspend_callback (CdosPopupMenuItem *item, AppletMenu *self)
{
    cdos_util_spawn_command_line("dbus-send --system --print-reply --dest=\"org.freedesktop.login1\" /org/freedesktop/login1 org.freedesktop.login1.Manager.Suspend boolean:'true'");
    applet_menu_close (); 
}

//睡眠
static void 
menu_group_power_hibernate_callback (CdosPopupMenuItem *item, AppletMenu *self)
{
    cdos_util_spawn_command_line("dbus-send --system --print-reply --dest=\"org.freedesktop.login1\" /org/freedesktop/login1 org.freedesktop.login1.Manager.Hibernate boolean:'true'");
    applet_menu_close (); 
}

//关机
static void 
menu_group_power_off_callback (CdosPopupMenuItem *item, AppletMenu *self)
{
    cdos_util_spawn_command_line("dbus-send --session --print-reply --dest=\"org.gnome.SessionManager\" /org/gnome/SessionManager org.gnome.SessionManager.Shutdown");
    applet_menu_close (); 
}

//切换用户
static void 
menu_group_user_switch_callback (CdosPopupMenuItem *item, AppletMenu *self)
{
    cdos_util_spawn_command_line("dm-tool switch-to-greeter"); 
    applet_menu_close (); 
}

static gboolean
menu_group_user_lock_screen (AppletMenu *self)
{
    cdos_screen_saver_lock ();
    return FALSE;
}

//锁屏
static void 
menu_group_user_lock_callback (CdosPopupMenuItem *item, AppletMenu *self)
{
    g_idle_add (menu_group_user_lock_screen, self);
    applet_menu_close (); 
}

//注销
static void 
menu_group_user_logout_callback (CdosPopupMenuItem *item, AppletMenu *self)
{
    cdos_util_spawn_command_line("dbus-send --session --print-reply --dest=\"org.gnome.SessionManager\" /org/gnome/SessionManager org.gnome.SessionManager.Logout uint32:0");
    applet_menu_close (); 
}

//用户详情
static void
menu_group_user_settings_callback (CdosPopupMenuItem *item, AppletMenu *self)
{ 
    cdos_util_spawn_command_line("found-control-center user-accounts");
    //applet_menu_can_close (TRUE);
    applet_menu_close (); 
}

static void 
menu_load_applications (AppletMenu *self)
{
    AppletMenuPrivate *priv = self->priv;
    priv->apps_manager = applet_applications_manager_new ();
    priv->apps_group_type = applet_applications_manager_get_group_type (priv->apps_manager);
    StBoxLayout *apps_box = applet_applications_manager_get_box (priv->apps_manager);
    menu_container_add_actor ((ClutterActor *)priv->pane_applications, (ClutterActor *)apps_box, 
                              "x-expand", TRUE,
                              "x-align", CLUTTER_ACTOR_ALIGN_FILL,
                              //"x-fill", TRUE,
                              //"y-fill", TRUE,
                              NULL);
}

static void
menu_load_favorites (AppletMenu *self) 
{
    AppletMenuPrivate *priv = self->priv;
    priv->favorites_manager = applet_favorites_manager_new ();
    StBoxLayout *favorites_box = applet_favorites_manager_get_box (priv->favorites_manager);
    menu_container_add_actor ((ClutterActor *)priv->pane_favorites, (ClutterActor *)favorites_box, 
                              "x-expand", TRUE,
                              "x-align", CLUTTER_ACTOR_ALIGN_FILL,
                              //"x-fill", TRUE,
                              //"y-fill", TRUE,
                              NULL);
}

static void 
menu_pane_switch (AppletMenu *self, MenuPane pane)
{
    AppletMenuPrivate *priv = self->priv;
    priv->menu_pane = pane;
    switch (pane) 
    {
        case MENU_PANE_APPS :
            clutter_actor_hide ((ClutterActor *)priv->pane_favorites);
            clutter_actor_hide ((ClutterActor *)priv->pane_search);
            clutter_actor_show ((ClutterActor *)priv->pane_applications);
            break;
        case MENU_PANE_FAVORITES :
            clutter_actor_hide ((ClutterActor *)priv->pane_applications);
            clutter_actor_hide ((ClutterActor *)priv->pane_search);
            clutter_actor_show ((ClutterActor *)priv->pane_favorites);
            break;
        case MENU_PANE_SEARCH :
            clutter_actor_hide ((ClutterActor *)priv->pane_applications);
            clutter_actor_hide ((ClutterActor *)priv->pane_favorites);
            clutter_actor_show ((ClutterActor *)priv->pane_search);
            break;
        default :
            clutter_actor_hide ((ClutterActor *)priv->pane_applications);
            clutter_actor_hide ((ClutterActor *)priv->pane_favorites);
            clutter_actor_hide ((ClutterActor *)priv->pane_search);   
    }
    g_signal_emit (self, signals[PANE_SWITCH], 0, pane);
}

static AppletMenu *
applet_menu_new (void)
{
    static AppletMenu *menu = NULL;
    if (!menu)
    {
        menu = g_object_new (APPLET_TYPE_MENU, NULL);
    }
    return menu;
}

static void 
menu_reset_right_box_width (AppletMenu *self)
{
    CdosGlobal *global = cdos_global_get();
    guint scale = cdos_global_get_ui_scale (global);

    AppletMenuPrivate *priv = self->priv;
    gfloat width = MENU_RIGHT_PANEL_WIDTH;

    clutter_actor_set_width ((ClutterActor *)priv->right_panel, width * scale);
    clutter_actor_set_width ((ClutterActor *)priv->main_box, width * scale);
}

static void 
menu_on_search (AppletMenuSearch *search, AppletMenu *self)
{
    menu_pane_switch (self, MENU_PANE_SEARCH);
}

static void 
menu_on_search_finished (AppletMenuSearch *search, AppletMenu *self)
{
    menu_pane_switch (self, MENU_PANE_APPS);
}

static void 
menu_search (AppletMenu *self)
{
    AppletMenuPrivate *priv = self->priv;
    priv->search = applet_menu_search_new ();
    StEntry *entry = applet_menu_search_get_entry (priv->search);
    menu_container_add_actor ((ClutterActor *)priv->right_panel_top, (ClutterActor *)entry, 
                              "x-expand", TRUE,
                              "x-align", CLUTTER_ACTOR_ALIGN_FILL,
                              "y-align", CLUTTER_ACTOR_ALIGN_START,
                              NULL);

    StWidget *results_box = applet_menu_search_get_results_box (priv->search);
    _add_actor (priv->pane_search, results_box);
    g_object_set (results_box,
                  "x-expand", TRUE,
                  "y-expand", TRUE,
                  "x-align", CLUTTER_ACTOR_ALIGN_FILL,
                  "y-align", CLUTTER_ACTOR_ALIGN_START,
                  NULL); 
    /*
    menu_container_add_actor ((ClutterActor *)priv->pane_search, (ClutterActor *)results_box, 
                              "x-expand", TRUE,
                              "y-expand", TRUE,
                              "x-align", CLUTTER_ACTOR_ALIGN_FILL,
                              "y-align", CLUTTER_ACTOR_ALIGN_START,
                            NULL); 
    */
    g_signal_connect (priv->search, 
                     "search", 
                     G_CALLBACK (menu_on_search), 
                     self);
    g_signal_connect (priv->search, 
                     "search-finished", 
                     G_CALLBACK (menu_on_search_finished), 
                     self);            
}

static void
user_face_on_changed (ActUser *act_user,
                      gpointer user_data)
{
    AppletShortcutButton *button = (AppletShortcutButton *)user_data;
    ClutterActor *face = NULL;
    gint face_size = MENU_ICON_SIZE_FACE;

    guint ui_scale = 1;
    CdosGlobal *global = cdos_global_get();
    ui_scale = cdos_global_get_ui_scale (global);

    face = cdos_get_user_face_icon (act_user, face_size * (int)ui_scale);
    if (face == NULL)
    {
        face = CLUTTER_ACTOR (st_icon_new ());
        st_icon_set_icon_name ((StIcon *)face, "found-menu-user");
        //st_icon_set_icon_type ((StIcon *)face, ST_ICON_FULLCOLOR);
    }
    const gchar *user_name = cdos_user_get_display_name (act_user);
    applet_menu_button_base_set_tooltip ((AppletMenuButtonBase *)button, user_name);
    applet_menu_button_base_set_label_text ((AppletMenuButtonBase *)button, user_name);
    applet_menu_button_base_set_icon ((AppletMenuButtonBase *)button, face);
}

static void
user_loaded (ActUser *gobject,
             GParamSpec *spec,
             gpointer   user_data)
{
    user_face_on_changed ((ActUser *)gobject, user_data);
}

static void
drawing_area_on_allocation_changed (ClutterActor *actor,
                          ClutterActorBox *box,
                          ClutterAllocationFlags flags,
                          gpointer user_data)
{
    AppletMenu *self = APPLET_MENU (user_data);
    AppletMenuPrivate *priv = self->priv;
    ClutterActor *icon;
    gint face_size = MENU_ICON_SIZE_FACE;

    CdosGlobal *global = cdos_global_get();
    guint ui_scale = cdos_global_get_ui_scale (global);
    icon = applet_menu_button_base_get_icon (APPLET_MENU_BUTTON_BASE (actor)); 
    if (ST_IS_ICON (icon))
    {
        ui_scale = 1;
    }
    applet_menu_button_base_set_icon_size (APPLET_MENU_BUTTON_BASE (actor), face_size * (int)ui_scale);
}

static void
menu_user_face_setup (AppletMenu *self, AppletShortcutButton *button)
{
    AppletMenuPrivate *priv = self->priv;
    ActUserManager *user_manager;
    user_manager = act_user_manager_get_default ();
    priv->user = act_user_manager_get_user (user_manager, g_get_user_name ());
    priv->user_changed_id = \
    g_signal_connect (priv->user, 
                     "changed", 
                     G_CALLBACK (user_face_on_changed), 
                     button);
    priv->user_loaded_id = \
    g_signal_connect (priv->user, 
                     "notify::is-loaded", 
                     G_CALLBACK (user_loaded), 
                     button);
    g_signal_connect ((ClutterActor *)button, 
                    "allocation-changed",
                    G_CALLBACK (drawing_area_on_allocation_changed), 
                    self);
}

static void 
popup_menu_on_open_state_changed (CdosPopupMenuBase *popup_menu_base, 
                                  gboolean state, 
                                  AppletMenu *self)
{
    AppletMenuPrivate *priv = self->priv;
    priv->is_open = state;
    menu_expand_button_retract (self);
    if (state == FALSE)
    {
        applet_menu_search_cancel_search (priv->search);
        clutter_actor_hide ((ClutterActor *)priv->left_panel);
        if (priv->menu_pane == MENU_PANE_SEARCH)
        {
            menu_pane_switch (self, MENU_PANE_APPS);
        }
    } else {
        // StEntry *entry = applet_menu_search_get_entry (priv->search);
        clutter_actor_grab_key_focus ((ClutterActor *)priv->main_box);
        clutter_actor_show ((ClutterActor *)priv->left_panel);
        clutter_actor_set_child_above_sibling ((ClutterActor *)priv->main_bin_box, (ClutterActor *)priv->left_panel, NULL);
    }
    menu_apps_shortcut_group_update (self);
}

static gboolean
cdos_popup_menu_base_toggle_idle (CdosPopupMenuBase *self)
{
    cdos_popup_menu_base_toggle (self);
    return FALSE;
}

static void
display_on_overlay_key_release (MetaDisplay *display, gpointer user_data)
{
    AppletMenu *self = APPLET_MENU (user_data);
    CdosPopupMenu *popup_menu = cdos_applet_get_popup_menu ((CdosApplet *)self);
    CdosPanel *panel;
    gboolean hidden;

    // check  the monitor is in full screen.
    gint primary = meta_display_get_primary_monitor (display);
    if (primary < 0)
    {
        return ;
    }
    if (meta_display_get_monitor_in_fullscreen (display, primary))
    {
        return ;
    }

    // check the panel is hidden.
    panel = cdos_ui_plugin_get_panel ();
    hidden = cdos_panel_get_hidden (panel);
    if (hidden == TRUE)
    {
        cdos_panel_enable (panel, 0);
    }
    g_timeout_add (POPUP_MENU_DELAY, 
                                (GSourceFunc)cdos_popup_menu_base_toggle_idle, 
                                CDOS_POPUP_MENU_BASE(popup_menu));
}

static void
menu_pane_grab_focus_start (AppletMenu *self)
{
    AppletMenuPrivate *priv = self->priv;
    switch (priv->menu_pane)
    {
        case MENU_PANE_FAVORITES:
            applet_favorites_manager_start_key_focus (priv->favorites_manager);
            break;
        case MENU_PANE_APPS:
            applet_applications_manager_start_key_focus (priv->apps_manager);
            break;
        case MENU_PANE_SEARCH:
            applet_menu_search_start_key_focus (priv->search);
            break;
        default:
            applet_menu_grab_key_focus_main_box ();
	        break;
    }
}

static gboolean
main_box_on_navigate_focus (ClutterActor *actor, ClutterEvent *event, gpointer user_data)
{
    AppletMenu *self = APPLET_MENU (user_data);
    AppletMenuPrivate *priv = self->priv;
    if (!priv->is_open)
    {
        return FALSE;
    }
    guint symbol;
    ClutterModifierType state;
    ClutterActor *focused;
    StEntry *entry;
    symbol = clutter_event_get_key_symbol (event); 
    state = clutter_event_get_state (event);
    CdosGlobal *global = cdos_global_get();
    ClutterStage *global_stage = cdos_global_get_stage (global);
    focused = clutter_stage_get_key_focus (CLUTTER_STAGE (global_stage));
    entry = applet_menu_search_get_entry (priv->search);
    if (symbol == CLUTTER_KEY_Right)
    {
        if (!clutter_actor_contains ((ClutterActor *)priv->right_panel, focused))
        {
            clutter_actor_grab_key_focus ((ClutterActor *)entry);
        }
        menu_expand_button_retract (self);
        return TRUE;
    }
        else if (symbol == CLUTTER_KEY_Left)
    {
        if (clutter_actor_contains ((ClutterActor *)priv->right_panel, focused))
        {
            clutter_actor_grab_key_focus ((ClutterActor *)priv->pane_switch_button); 
        }
    }
    else if (symbol == CLUTTER_KEY_Down)
    {
        if (focused == (ClutterActor *)global_stage || focused == (ClutterActor *)priv->main_box)
        {
            clutter_actor_grab_key_focus ((ClutterActor *)priv->pane_switch_button);
            return TRUE;
        }
        if (clutter_actor_contains ((ClutterActor *)entry, focused))
        {
            menu_pane_grab_focus_start (self);
            return TRUE;
        }
    }
    else if (symbol == CLUTTER_KEY_Search || (symbol == CLUTTER_KEY_f && state == CLUTTER_CONTROL_MASK))
    {
        clutter_actor_grab_key_focus ((ClutterActor *)entry);
        menu_expand_button_retract (self);
        return TRUE;
    }
    return FALSE;
}

static void
left_panel_on_allocation_changed (ClutterActor *actor,
                          ClutterActorBox *box,
                          ClutterAllocationFlags flags,
                          gpointer user_data)
{
    AppletMenu *self = APPLET_MENU (user_data);
    AppletMenuPrivate *priv = self->priv;
    StThemeNode *theme_node;
    gfloat h, w = -1;
    gint border_top = 0, border_left = 0;
    CdosPopupMenu *popup_menu;
    CdosBoxPointer *bp;
    gfloat bp_x, bp_y;

    popup_menu = cdos_applet_get_popup_menu ((CdosApplet *)self);
    bp = cdos_popup_menu_get_box_pointer (popup_menu);
    clutter_actor_get_position ((ClutterActor *)bp, &bp_x, &bp_y);

    h = clutter_actor_get_height ((ClutterActor *)bp);
    theme_node = st_widget_get_theme_node ((StWidget *)priv->left_panel);
    w = st_theme_node_get_width (theme_node);

    //border_top = st_theme_node_get_border_width (theme_node, ST_SIDE_TOP);
    //border_left = st_theme_node_get_border_width (theme_node, ST_SIDE_LEFT);
    //border_left = 0;
    //border_top = 0;

    box->x2 = w + (float)border_left + bp_x;
    box->x1 = (float)border_left + bp_x;
    box->y1 = (float)border_top + bp_y;
    box->y2 = h - (float)border_top + bp_y;

    clutter_actor_allocate ((ClutterActor *)priv->left_panel, box, flags);
}

static void
menu_left_panel_setup (AppletMenu *self)
{
    AppletMenuPrivate *priv = self->priv;
    st_box_layout_set_vertical (priv->left_panel, TRUE);
    st_widget_set_style_class_name ((StWidget *)priv->left_panel, "main-left-panel");
    applet_helper_add_focus_group ((StWidget *)priv->left_panel);
    menu_container_add_actor (CLUTTER_ACTOR (priv->main_bin_box), (ClutterActor *)priv->left_panel,
                              "x-expand", TRUE,
                              "y-expand", TRUE,
                              "x-align", CLUTTER_ACTOR_ALIGN_START,
                              "y-align", CLUTTER_ACTOR_ALIGN_FILL,
                              NULL);
    g_signal_connect (priv->left_panel, 
                    "allocation-changed", 
                    G_CALLBACK (left_panel_on_allocation_changed), 
                    self);
}

static void 
menu_expand_button_retract (AppletMenu *self)
{
    AppletMenuPrivate *priv = self->priv;
    if (priv->group_expand)
    {
        menu_expand_group_callback (priv->expand_button, self);
    }
}

AppletMenu *
applet_menu_menu_get_default (void)
{
    return default_menu;
}

CdosApplet*
applets_menu_main (MetaData *metadata, StSide orientation, gfloat panel_height)
{
    AppletMenu *menu;
    menu = applet_menu_new ();
    cdos_applet_set_orientation (CDOS_APPLET (menu), orientation);

    return (CdosApplet *)menu;
}

StBoxLayout  *
applet_menu_get_main_box (void)
{
    AppletMenu *self = default_menu;
    if (!self)
        return NULL;
    AppletMenuPrivate *priv = self->priv;
    return priv->main_box;
}

void 
applet_menu_close (void)
{
    AppletMenu *menu = default_menu;
    if (!menu)
        return;
    CdosPopupMenu *popup_menu = cdos_applet_get_popup_menu ((CdosApplet *)menu);
    cdos_popup_menu_close ((CdosPopupMenuBase *)popup_menu, FALSE);
}

void 
applet_menu_can_close (gboolean can_close)
{
    AppletMenu *self = applet_menu_menu_get_default ();

    AppletMenuPrivate *priv = self->priv;
    priv->can_close = can_close;
}

CdosPopupMenuManager *
applet_menu_get_popup_menu_manager (void)
{
    AppletMenu *self = applet_menu_menu_get_default ();
    CdosPopupMenuManager *manager = cdos_applet_get_menu_manager((CdosApplet *)self);
    return manager;
}

MenuPane
applet_menu_get_current_pane (void)
{
    AppletMenu *self = applet_menu_menu_get_default ();
    AppletMenuPrivate *priv = self->priv;
    return priv->menu_pane;
}

void
applet_menu_grab_key_focus_main_box (void)
{
    AppletMenu *self = applet_menu_menu_get_default ();
    if (self)
        clutter_actor_grab_key_focus ((ClutterActor *)self->priv->main_box);
}
