/** 
 * Copyright (c) 2016-2018 CPU and Fundamental Software Research Center, CAS
 *  
 * This software is published by the license of CPU-OS License，you can use and 
 * distibuted this software under this License. See CPU-OS Liense for more detail.
 *
 * You should have receive a copy of CPU-OS License. If not, please contact us 
 * by email <support_os@cpu-os.ac.cn>  
 * 
 **/
#ifndef __APPLET_PARSER_H__
#define __APPLET_PARSER_H__

#include <stdio.h>
#include <stdlib.h>
#include <glib-object.h>
#include <glib.h>
#include <libxml/parser.h>
#include <libxml/xpath.h>

G_BEGIN_DECLS

// clang-format off
#define APPLET_TYPE_PARSER (applet_parser_get_type())
#define APPLET_PARSER(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj), APPLET_TYPE_PARSER, AppletParser))
#define APPLET_PARSER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST((klass), APPLET_TYPE_PARSER, AppletParserClass))
#define APPLET_IS_PARSER(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), APPLET_TYPE_PARSER))
#define APPLET_IS_PARSER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass), APPLET_TYPE_PARSER))
#define APPLET_PARSER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS((obj), APPLET_TYPE_PARSER, AppletParserClass))

// clang-format on
typedef struct _AppletParserPrivate AppletParserPrivate;
typedef struct _AppletParser AppletParser;
typedef struct _AppletParserClass AppletParserClass;

struct _AppletParser
{
    GObject parent_instace;
    AppletParserPrivate *priv;
};
struct _AppletParserClass
{
    GObjectClass parent_class;
};

GType
applet_parser_get_type(void) G_GNUC_CONST;
AppletParser *
applet_parser_new (const gchar *file_uri);
gchar *
applet_parser_get_node_prop (AppletParser *self,
                            xmlNodePtr cur_node,
                            const gchar *prop_name);
gboolean
applet_parser_set_node_prop (AppletParser *self,
                            xmlNodePtr cur_node,
                            const gchar *prop_name,
                            const gchar *prop_value);
xmlNodePtr
applet_parser_get_node_by_prop (AppletParser *self,
                                const gchar *xpath,
                                const gchar *prop_name,
                                const gchar *prop_value);
gboolean
applet_parser_remove_node (AppletParser *self, xmlNodePtr cur_node);
xmlNodePtr
applet_parser_move_to_after (AppletParser *self,
                            xmlNodePtr cur_node,
                            xmlNodePtr dest_node);
xmlNodePtr
applet_parser_move_to_before (AppletParser *self,
                             xmlNodePtr cur_node,
                             xmlNodePtr dest_node);
xmlNodePtr
applet_parser_add_text_child (AppletParser *self,
                             xmlNodePtr parent, 
                             const gchar *name, 
                             const gchar *content);
gchar *
applet_parser_get_node_content (AppletParser *self, xmlNodePtr cur_node);
gboolean
applet_parser_set_node_content (AppletParser *self, 
                                xmlNodePtr cur_node, 
                                const gchar *content);
xmlXPathObjectPtr
applet_parser_get_nodes_by_path (AppletParser *self, const gchar *xpath);

G_END_DECLS

#endif /* __APPLET_PARSER_H__ */
