/** 
 * Copyright (c) 2016-2018 CPU and Fundamental Software Research Center, CAS
 *  
 * This software is published by the license of CPU-OS License，you can use and 
 * distibuted this software under this License. See CPU-OS Liense for more detail.
 *
 * You should have receive a copy of CPU-OS License. If not, please contact us 
 * by email <support_os@cpu-os.ac.cn>  
 * 
 **/
//
// Created by zhangtao on 18-1-18.
// Email: zhangtao@cpu-os.ac.cn
//

#include "applet-shortcut-button.h"
#include "cdos-popup-menu-manager.h"
#include "applet-menu.h"
#include "applet-menu-private.h"

#define APPLET_SHORTCUT_BUTTON_GET_PRIVATE(obj)      \
(G_TYPE_INSTANCE_GET_PRIVATE((obj), APPLET_TYPE_SHORTCUT_BUTTON, AppletShortcutButtonPrivate))

G_DEFINE_TYPE(AppletShortcutButton, applet_shortcut_button, APPLET_TYPE_MENU_BUTTON_BASE);

struct _AppletShortcutButtonPrivate
{
    gboolean                expand;
    gboolean                menu_enable;
    gboolean                tooltip_toggle;
    CdosPopupMenu           *menu;
    gulong                  menu_activate_id;
    gulong                  applet_menu_id;
    CdosPopupMenuManager    *popup_menu_manager;

};

static void applet_shortcut_button_dispose (GObject *object);
static void applet_shortcut_button_finalize (GObject *object);

static void applet_shortcut_button_set_menu (AppletShortcutButton *self);
static void applet_shortcut_button_menu_freeze (AppletShortcutButton *self);

static void button_on_activate (AppletShortcutButton *self, gpointer user_data);
static void button_cancel (AppletShortcutButton *self);

static void 
menu_on_open_state_changed (CdosPopupMenuBase *popup_menu_base, 
                            gboolean state, 
                            AppletShortcutButton *self);

static void
applet_shortcut_button_init (AppletShortcutButton *self)
{
    AppletShortcutButtonPrivate *priv;
    priv = self->priv = APPLET_SHORTCUT_BUTTON_GET_PRIVATE (self);
    priv->expand = FALSE;
    priv->menu_enable = FALSE;
    priv->tooltip_toggle = FALSE;
    st_widget_set_style_class_name (ST_WIDGET(self), "menu-shortcut-button");
    applet_shortcut_button_set_expand (self, priv->expand);
    applet_menu_button_base_set_icon_size ((AppletMenuButtonBase *)self, DEFAULT_ICON_SIZE);
}

static void
applet_shortcut_button_class_init (AppletShortcutButtonClass *klass)
{

    GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
    g_type_class_add_private (klass, sizeof (AppletShortcutButtonPrivate));
    AppletMenuButtonBaseClass *base_class = APPLET_MENU_BUTTON_BASE_CLASS(klass);

    gobject_class->dispose = applet_shortcut_button_dispose;
    gobject_class->finalize = applet_shortcut_button_finalize;
    base_class->cancel = button_cancel;
}

static void
applet_shortcut_button_dispose (GObject *object)
{
    AppletShortcutButton *self = APPLET_SHORTCUT_BUTTON (object);
    AppletShortcutButtonPrivate *priv = self->priv;

    AppletMenu* applet_menu = applet_menu_menu_get_default ();
    CdosPopupMenu *popup_menu = cdos_applet_get_popup_menu ((CdosApplet *)applet_menu);
    if (priv->applet_menu_id)
    {
        g_signal_handler_disconnect (popup_menu, priv->applet_menu_id);
        priv->applet_menu_id = 0;
    }
    if (priv->menu)
    {
        applet_shortcut_button_menu_freeze (self);
    }
    //After removed menu, The manager will release it and it's items, so we do't need release them again;
    if (priv->popup_menu_manager)
    {
    	cdos_popup_menu_manager_remove_menu (priv->popup_menu_manager, CDOS_POPUP_MENU_BASE(priv->menu));
        g_clear_object (&priv->popup_menu_manager);
    }
    G_OBJECT_CLASS (applet_shortcut_button_parent_class)->dispose (object);
}

static void
applet_shortcut_button_finalize (GObject *object)
{
    G_OBJECT_CLASS (applet_shortcut_button_parent_class)->finalize (object);
}

static void 
applet_menu_on_open_state_changed (CdosPopupMenuBase *popup_menu_base, 
                                   gboolean state, 
                                   AppletShortcutButton *self)
{
    AppletShortcutButtonPrivate *priv = self->priv;
    if (state == FALSE)
    {
        cdos_popup_menu_close ((CdosPopupMenuBase *)priv->menu, FALSE);
    }
}

static void 
applet_shortcut_button_set_menu (AppletShortcutButton *self)
{
    AppletShortcutButtonPrivate *priv = self->priv;
    if (!priv->menu_enable)
    {
        return ;
    }
    
    if (priv->menu == NULL)
    {
        priv->menu = g_object_new (CDOS_TYPE_POPUP_MENU,
                         "source-actor", self,
                         "arrow-alignment", 1.0,
                         "arrow-side", ST_SIDE_BOTTOM,
                         "is-popup", TRUE,
                         NULL);
        priv->popup_menu_manager = cdos_popup_menu_manager_new ((ClutterActor *)self);  
        cdos_popup_menu_manager_add_menu (priv->popup_menu_manager, CDOS_POPUP_MENU_BASE(priv->menu), -1);
        cdos_popup_menu_close ((CdosPopupMenuBase *)priv->menu, FALSE);

        g_signal_connect (CDOS_POPUP_MENU_BASE(priv->menu), 
                         "open-state-changed", 
                         G_CALLBACK (menu_on_open_state_changed), 
                         self);

        //when applet menu closed, button menu closed too.
        AppletMenu* applet_menu = applet_menu_menu_get_default ();
        CdosPopupMenu *popup_menu = cdos_applet_get_popup_menu ((CdosApplet *)applet_menu);
        priv->applet_menu_id = \
            g_signal_connect (CDOS_POPUP_MENU_BASE(popup_menu), 
                            "open-state-changed", 
                            G_CALLBACK (applet_menu_on_open_state_changed), 
                            self);
    }

    if (priv->menu_activate_id == 0L)
    {
        priv->menu_activate_id = \
            g_signal_connect ((AppletMenuButtonBase *)self, 
                             "activate", 
                             G_CALLBACK (button_on_activate), 
                             NULL);
    }
}

static void 
applet_shortcut_button_menu_freeze (AppletShortcutButton *self)
{
    AppletShortcutButtonPrivate *priv = self->priv;
    if (priv->menu)
    {
        cdos_popup_menu_close ((CdosPopupMenuBase *)priv->menu, FALSE);
    }
    if (priv->menu_activate_id) 
    {
        g_signal_handler_disconnect ((AppletMenuButtonBase *)self, priv->menu_activate_id);
        priv->menu_activate_id = 0L;
    }
}

static void 
button_cancel (AppletShortcutButton *self)
{
    AppletShortcutButtonPrivate *priv = self->priv;
    if (priv->menu_enable == FALSE)
    {
        return ;
    }
    if (priv->menu)
    {
        cdos_popup_menu_close ((CdosPopupMenuBase *)priv->menu, FALSE);
        clutter_actor_grab_key_focus (CLUTTER_ACTOR (self));    
    }
}

static void 
button_on_activate (AppletShortcutButton *self, gpointer user_data)
{
    AppletShortcutButtonPrivate *priv = self->priv;
    cdos_popup_menu_base_toggle (CDOS_POPUP_MENU_BASE (priv->menu));
}

static void 
menu_on_open_state_changed (CdosPopupMenuBase *popup_menu_base, 
                            gboolean state, 
                            AppletShortcutButton *self)
{
    ClutterActor *focus;
    if (state)
    {
        GList *items = cdos_popup_menu_base_get_menu_items (popup_menu_base);
        focus = (ClutterActor *)items->data;
        g_list_free (items);
        applet_menu_can_close (FALSE);
        applet_menu_button_base_set_tooltip_state ((AppletMenuButtonBase *)self, FALSE);
    } else {
        focus = (ClutterActor *)applet_menu_get_main_box ();
        applet_menu_can_close (TRUE);
        applet_menu_button_base_set_tooltip_state ((AppletMenuButtonBase *)self, TRUE);
    }
    clutter_actor_grab_key_focus (focus);
}

AppletShortcutButton *
applet_shortcut_button_new (const gchar *icon_name, const gchar *label_text)
{
    AppletShortcutButton *shortcut_button ;
    shortcut_button = g_object_new (APPLET_TYPE_SHORTCUT_BUTTON, NULL);

    if (icon_name)
    {
        applet_menu_button_base_set_icon_by_name (APPLET_MENU_BUTTON_BASE(shortcut_button), icon_name);
    }
    if (label_text)
    {
        applet_menu_button_base_set_label_text (APPLET_MENU_BUTTON_BASE(shortcut_button), label_text);
    }
    return shortcut_button;
}

void 
applet_shortcut_button_set_expand (AppletShortcutButton *self, gboolean expand)
{
    g_return_if_fail (APPLET_IS_SHORTCUT_BUTTON(self));
    AppletShortcutButtonPrivate *priv = self->priv;
    gboolean has_tooltip =  FALSE;
    const gchar *tooltip;

    priv->expand = expand;
    StWidget *label = applet_menu_button_base_get_label ((AppletMenuButtonBase *)self);
    if (priv->tooltip_toggle)
    {
        tooltip = applet_menu_button_base_get_tooltip ((AppletMenuButtonBase *)self);
        has_tooltip = tooltip != NULL ? TRUE : FALSE;
    }
    if (priv->expand)
    {
        if (has_tooltip && priv->tooltip_toggle)
            st_widget_set_has_tooltip (ST_WIDGET(self), FALSE);
        clutter_actor_show ((ClutterActor *)label);
    } else {
        if (has_tooltip && priv->tooltip_toggle)
            applet_menu_button_base_set_tooltip ((AppletMenuButtonBase *)self, tooltip);
        clutter_actor_hide ((ClutterActor *)label);
    }
}

gboolean 
applet_shortcut_button_get_expand (AppletShortcutButton *self)
{
    g_return_val_if_fail (APPLET_IS_SHORTCUT_BUTTON(self), FALSE);
    AppletShortcutButtonPrivate *priv = self->priv;
    return priv->expand;
}

void 
applet_shortcut_button_set_menu_enable (AppletShortcutButton *self, gboolean enable)
{
    g_return_if_fail (APPLET_IS_SHORTCUT_BUTTON(self));
    AppletShortcutButtonPrivate *priv = self->priv;
    priv->menu_enable = enable;
    if (priv->menu_enable)
    {
        applet_shortcut_button_set_menu (self);
    }
    else 
    {
        applet_shortcut_button_menu_freeze (self);
    }
}

gboolean 
applet_shortcut_button_get_menu_enable (AppletShortcutButton *self)
{
    g_return_val_if_fail (APPLET_IS_SHORTCUT_BUTTON(self), FALSE);
    AppletShortcutButtonPrivate *priv = self->priv;
    return priv->menu_enable;
}

CdosPopupMenu * 
applet_shortcut_button_get_menu (AppletShortcutButton *self)
{
    g_return_val_if_fail (APPLET_IS_SHORTCUT_BUTTON(self), NULL);
    AppletShortcutButtonPrivate *priv = self->priv;
    return priv->menu;
}

CdosPopupMenuItem * 
applet_shortcut_button_add_menu_item (AppletShortcutButton *self,
                                      const gchar *text, 
                                      GCallback callback, 
                                      gpointer user_data)
{
    g_return_val_if_fail (APPLET_IS_SHORTCUT_BUTTON(self), NULL);
    g_return_val_if_fail (callback != NULL, NULL);
    AppletShortcutButtonPrivate *priv = self->priv;
    CdosPopupMenuItem *item = cdos_popup_menu_base_add_action ((CdosPopupMenuBase *)priv->menu, text, callback, user_data);
    cdos_popup_base_menu_item_set_keep_menu ((CdosPopupBaseMenuItem *)item, FALSE);
    return item ;
}

void 
applet_shortcut_button_set_tooltip_toggle (AppletShortcutButton *self, gboolean is_toggle)
{
    g_return_if_fail (APPLET_IS_SHORTCUT_BUTTON(self));
    AppletShortcutButtonPrivate *priv = self->priv;
    priv->tooltip_toggle = is_toggle;
}

gboolean 
applet_shortcut_button_get_tooltip_toggle (AppletShortcutButton *self)
{
    g_return_val_if_fail (APPLET_IS_SHORTCUT_BUTTON(self), FALSE);
    AppletShortcutButtonPrivate *priv = self->priv;
    return priv->tooltip_toggle;
}

