/** 
 * Copyright (c) 2016-2018 CPU and Fundamental Software Research Center, CAS
 *  
 * This software is published by the license of CPU-OS License，you can use and 
 * distibuted this software under this License. See CPU-OS Liense for more detail.
 *
 * You should have receive a copy of CPU-OS License. If not, please contact us 
 * by email <support_os@cpu-os.ac.cn>  
 * 
 **/
//
// Created by zhangtao on 18-1-18.
// Email: zhangtao@cpu-os.ac.cn
//

#include "applet-shortcut-group.h"

#define APPLET_SHORTCUT_GROUP_GET_PRIVATE(obj)      \
(G_TYPE_INSTANCE_GET_PRIVATE((obj), APPLET_TYPE_SHORTCUT_GROUP, AppletShortcutGroupPrivate))

G_DEFINE_TYPE(AppletShortcutGroup, applet_shortcut_group, G_TYPE_OBJECT);

struct _AppletShortcutGroupPrivate
{
    gboolean     expand;
    GList        *buttons;
};

static void applet_shortcut_group_dispose (GObject *object);
static void applet_shortcut_group_finalize (GObject *object);

static void
applet_shortcut_group_init (AppletShortcutGroup *self)
{
    AppletShortcutGroupPrivate *priv;
    priv = self->priv = APPLET_SHORTCUT_GROUP_GET_PRIVATE (self);
    priv->expand =  FALSE;
}

static void
applet_shortcut_group_class_init (AppletShortcutGroupClass *klass)
{

    GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
    g_type_class_add_private (klass, sizeof (AppletShortcutGroupPrivate));

    gobject_class->dispose = applet_shortcut_group_dispose;
    gobject_class->finalize = applet_shortcut_group_finalize;

}

static void
applet_shortcut_group_dispose (GObject *object)
{

    AppletShortcutGroupPrivate *priv = APPLET_SHORTCUT_GROUP (object)->priv;
    if (priv->buttons)
    {
        g_clear_pointer (&priv->buttons, g_list_free);
        priv->buttons = NULL;
    }
    
    G_OBJECT_CLASS (applet_shortcut_group_parent_class)->dispose (object);
}

static void
applet_shortcut_group_finalize (GObject *object)
{
    G_OBJECT_CLASS (applet_shortcut_group_parent_class)->finalize (object);
}


AppletShortcutGroup *
applet_shortcut_group_new (void)
{
    AppletShortcutGroup *shortcut_group ;
    shortcut_group = g_object_new (APPLET_TYPE_SHORTCUT_GROUP, NULL);

    return shortcut_group;
}

AppletShortcutButton*    
applet_shortcut_group_add_button (AppletShortcutGroup *self, 
                                  const gchar *icon_name, 
                                  const gchar *label_text, 
                                  GCallback  activate_callback, 
                                  gpointer user_data)
{
    g_return_val_if_fail (APPLET_IS_SHORTCUT_GROUP(self), NULL);

    AppletShortcutGroupPrivate *priv = self->priv;
    AppletShortcutButton *button = applet_shortcut_button_new (icon_name, label_text);
    if (activate_callback)
    {
        g_signal_connect (button, 
                      "activate", 
                      G_CALLBACK (activate_callback), 
                      user_data);

    }
    priv->buttons = g_list_prepend (priv->buttons, button);
    applet_shortcut_group_set_expand (self, priv->expand); 
    return button ;
}

void    
applet_shortcut_group_remove_button (AppletShortcutGroup *self, AppletShortcutButton *button, gboolean destroy)
{
    g_return_if_fail (APPLET_IS_SHORTCUT_GROUP(self));
    g_return_if_fail (APPLET_IS_SHORTCUT_BUTTON(button));

    AppletShortcutGroupPrivate *priv = self->priv;
    priv->buttons = g_list_remove (priv->buttons, button);
    if (destroy)
    {
        g_object_unref (button);
    }
}

void    
applet_shortcut_group_set_expand (AppletShortcutGroup *self, gboolean expand)
{
    g_return_if_fail (APPLET_IS_SHORTCUT_GROUP(self));
    AppletShortcutGroupPrivate *priv = self->priv;
    GList *buttons = priv->buttons;
    while (buttons != NULL)
    {
        AppletShortcutButton *button = buttons->data;
        applet_shortcut_button_set_expand (button, expand);
        buttons = buttons->next;
    }
}

gboolean    
applet_shortcut_group_get_expand (AppletShortcutGroup *self)
{
    g_return_val_if_fail (APPLET_IS_SHORTCUT_GROUP(self), FALSE);
    AppletShortcutGroupPrivate *priv = self->priv;
    return  priv->expand;
}
