#include "applet-nm-device-wireless.h"
#include "applet-nm-menu-item.h"
#include "cdos-util.h"
#include "applet-nm-connection.h"

#define HEIGHT_WIRELESS_LIST 72 // add by niyl

typedef struct _APData
{
    gulong notify_ssid;
    NMUtilsSecurityType sec_type;
} APData;

static APData *
get_ap_data (NMAccessPoint *ap)
{
    APData *data = (APData *)g_object_get_data ((GObject *)ap, "applet-ap-data");
    if (!data)
    {
        data = g_new0 (APData, 1);
        g_object_set_data_full ((GObject *)ap, "applet-ap-data", data, (GDestroyNotify)g_free);
    }
    return data;
}

static void notify_ssid_cb (NMAccessPoint *ap, gpointer spec, AppletNMDeviceWireless *self);

static gint network_sort_function (const AppletAP *a, const AppletAP *b);

static void active_ap_changed (NMDeviceWifi *device, gpointer spec, AppletNMDeviceWireless *self);

static void access_point_added (NMDeviceWifi *device, NMAccessPoint *ap, AppletNMDeviceWireless *self);

static void access_point_removed (NMDeviceWifi *device, NMAccessPoint *ap, AppletNMDeviceWireless *self);

static NMUtilsSecurityType get_ap_security_type (NMAccessPoint *ap);

static AppletAP *find_network (AppletNMDeviceWireless *self, NMAccessPoint *ap);

static void wifi_create_section (AppletNMDevice *device);

static void wifi_clear_section (AppletNMDevice *device);

static void applet_nm_device_wireless_activate (AppletNMDevice *device);

static void create_network_item (AppletNMDeviceWireless *device, AppletAP *ap_obj, gint position);

static AppletAP *find_existing_network (AppletNMDeviceWireless *self, NMAccessPoint *ap);

// add by niyl
static void
set_more_item_height (CdosPopupSubMenu *menu)
{
    if (!menu)
    {
        return;
    }

    // gfloat height = clutter_actor_get_height (menu->actor);
    guint length = CDOS_POPUP_MENU_BASE (menu)->length;
    // g_print ("\n==== Func:%s height:%f length:%d ====\n", __FUNCTION__, height, length);
    // 当 HEIGHT_WIRELESS_LIST = 72，恰好能显示三个
    if (length > 3)
    {
        clutter_actor_set_height (CLUTTER_ACTOR (menu), HEIGHT_WIRELESS_LIST);
    }
    else
    {
        clutter_actor_set_height (CLUTTER_ACTOR (menu), -1);
    }
}

// set height when Click More...
static void
on_clicked_for_more (CdosPopupSubMenu *menu, gboolean open, gpointer user_data)
{
    // gfloat height = clutter_actor_get_height (menu->actor);
    // g_print ("\n====== Func:%s open:%d height:%f =======\n", __FUNCTION__, open, height);
    set_more_item_height (menu);
}
// add end

char *
applet_nm_get_utf8_ssid (GBytes *ssid)
{
    char *ssid_utf8 = NULL;
    gconstpointer data;
    gsize len;

    if (ssid)
    {
        data = g_bytes_get_data (ssid, &len);
        ssid_utf8 = nm_utils_ssid_to_utf8 (data, len);
    }
    if (!ssid_utf8)
        ssid_utf8 = g_strdup (_("<unknown>"));
    return ssid_utf8;
}


static void
applet_nm_device_wireless_init (AppletNMDeviceWireless *self,
                                NMClient *client,
                                NMDeviceWifi *device,
                                const GPtrArray *connections)
{
    self->networks = NULL;

    AppletNMDevice *dev = (void *)self;
    dev->device = (NMDevice *)device;
    dev->category = DeviceWireless;
    // impl_manager_activate_connection ();
    GSList *valid_connections = NULL;
    for (int i = 0; i < connections->len; i++)
    {
        NMConnection *conn = connections->pdata[i];
        if (nm_device_connection_valid (dev->device, conn))
            valid_connections = g_slist_prepend (valid_connections, conn);
    }
    const GPtrArray *access_points = nm_device_wifi_get_access_points (device);
    guint i;
    for (i = 0; access_points && (i < access_points->len); i++)
    {
        NMAccessPoint *ap = access_points->pdata[i];
        GBytes *ssid = nm_access_point_get_ssid (ap);
        if (!ssid)
        {
            // hidden access point cannt be added.
            gulong notify_ssid = g_signal_connect (ap, "notify::ssid", G_CALLBACK (notify_ssid_cb), self);
            get_ap_data (ap)->notify_ssid = notify_ssid;
            continue;
        }
        AppletAP *obj = NULL;
        AppletAP *pos = find_network (self, ap);
        if (pos)
        {
            obj = pos;
            obj->ap = ap;
            obj->aps = g_slist_prepend (obj->aps, ap);
        }
        else
        {
            obj = g_slice_new0 (AppletAP);
            obj->ssid = nm_access_point_get_ssid (ap);
            obj->mode = nm_access_point_get_mode (ap);
            obj->security = get_ap_security_type (ap);
            obj->ap = ap;
            self->networks = g_slist_prepend (self->networks, obj);
        }
        GSList *valid = NULL;
        for (valid = valid_connections; valid; valid = valid->next)
        {
            NMConnection *conn = valid->data;
            if (nm_access_point_connection_valid (ap, conn) &&
                (g_slist_find (obj->connections, conn) == NULL))
            {
                obj->connections = g_slist_prepend (obj->connections, conn);
            }
        }
    }

    NMAccessPoint *active = nm_device_wifi_get_active_access_point (device);
    if (active)
    {
        AppletAP *pos = find_network (self, active);
        if (pos == NULL)
            self->active_network = NULL;
        else
            self->active_network = pos;
    }
    else
        self->active_network = NULL;

    self->networks = g_slist_sort (self->networks, (GCompareFunc)network_sort_function);
    self->ap_changed_id =
        g_signal_connect (device, "notify::active-access-point", G_CALLBACK (active_ap_changed), self);
    self->ap_added_id = g_signal_connect (device, "access-point-added", G_CALLBACK (access_point_added), self);
    self->ap_removed_id = g_signal_connect (device, "access-point-removed", G_CALLBACK (access_point_removed), self);
    // applet_nm_device_init (dev, client, (NMDevice *)device, valid_connections);
}

void
applet_nm_device_wireless_destroy (AppletNMDeviceWireless *self)
{
    AppletNMDevice *device = (void *)self;
    if (self->ap_changed_id)
    {
        g_signal_handler_disconnect (device->device, self->ap_changed_id);
        self->ap_changed_id = 0L;
    }
    if (self->ap_added_id)
    {
        g_signal_handler_disconnect (device->device, self->ap_added_id);
        self->ap_added_id = 0L;
    }
    if (self->ap_removed_id)
    {
        g_signal_handler_disconnect (device->device, self->ap_removed_id);
        self->ap_removed_id = 0L;
    }
    applet_nm_device_destroy (device);
}
/*
static void
applet_nm_device_wireless_set_enabled (AppletNMDevice *device, gboolean enabled)
{
    if (enabled)
    {
        clutter_actor_show ((ClutterActor *)CDOS_POPUP_BASE_MENU_ITEM (device->status_item)->actor);
        clutter_actor_show ((ClutterActor *)device->section->actor);
    }
    else
    {
        clutter_actor_hide ((ClutterActor *)CDOS_POPUP_BASE_MENU_ITEM (device->status_item)->actor);
        clutter_actor_hide ((ClutterActor *)device->section->actor);
    }
}
*/

static void
activate_connection_cb (GObject *source,
                        GAsyncResult *res,
                        gpointer user_data)
{

}

static void
add_and_activate_connection_cb (GObject *source,
                                GAsyncResult *res,
                                gpointer user_data)
{

}

static void 
applet_nm_device_wireless_activate (AppletNMDevice *device)
{
    AppletNMDeviceWireless *self = (AppletNMDeviceWireless *)device;
    if (device->active_connection)
        return;

    NMConnection *best = NULL;
    AppletAP *best_ap = NULL;
    guint64 best_time = 0;
    GSList *iter;
    for (iter = self->networks; iter; iter = iter->next)
    {
        AppletAP *ap_obj = iter->data;
        GSList *j;
        for (j = ap_obj->connections; j; j = j->next)
        {
            NMConnection *conn = j->data;
            NMSettingConnection *s_con = nm_connection_get_setting_connection (conn);

            if (nm_setting_connection_get_timestamp (s_con) > best_time)
            {
                best = conn;
                best_time = nm_setting_connection_get_timestamp (s_con);
                best_ap = ap_obj;
            }
        }
    }
    if (best)
    {
        NMAccessPoint *ap = best_ap->ap;
        if (nm_access_point_connection_valid (ap, best))
        {
            nm_client_activate_connection_async (device->client,
                                                 best,
                                                 device->device,
                                                 nm_object_get_path (NM_OBJECT(ap)),
                                                 NULL,
                                                 activate_connection_cb, device);
        }
        return;
    }
    /*
    if (self->networks)
    {
        AppletAP *network = self->networks->data;
        NMConnection *conn = wifi_create_auto_connection(network->ap);
        NMAccessPoint *ap = network->ap;//sort_access_points (network->ap);
        nm_client_add_and_activate_connection (device->client,
                                       conn,
                                       device->device,
                                       nm_object_get_path(NM_OBJECT(ap)),
                                       NULL, NULL);

    }
    */
}

static void
notify_ssid_cb (NMAccessPoint *ap, gpointer spec, AppletNMDeviceWireless *self)
{
    if (nm_access_point_get_ssid (ap))
    {
        gulong notify_ssid = get_ap_data (ap)->notify_ssid;
        g_signal_handler_disconnect (ap, notify_ssid);
        APData *data = get_ap_data (ap);
        data->notify_ssid = 0;
        access_point_added ((NMDeviceWifi *)((AppletNMDevice *)self)->device, ap, self);
    }
}

static void
update_active_connection_item (AppletNMDeviceWireless *device, NMAccessPoint *ap)
{
    AppletNMDevice *dev = APPLET_NM_DEVICE (device);
    if (dev->active_connection_item)
    {
        applet_nm_menu_item_update_ap (dev->active_connection_item, ap);
    }
}

static void
active_ap_changed (NMDeviceWifi *device, gpointer spec, AppletNMDeviceWireless *self)
{
    self->active_network = NULL;

    NMAccessPoint *ap = nm_device_wifi_get_active_access_point (device);
    if (ap)
    {
        AppletAP *res = find_existing_network (self, ap);
        if (res)
        {
            self->active_network = res;
            update_active_connection_item (self, ap);
        }
    }
}

static NMUtilsSecurityType
get_ap_security_type (NMAccessPoint *ap)
{
    APData *data = get_ap_data (ap);
    if (data->sec_type)
        return data->sec_type;

    guint32 ap_flags = nm_access_point_get_flags (ap);
    guint32 ap_wpa = nm_access_point_get_wpa_flags (ap);
    guint32 ap_rsn = nm_access_point_get_rsn_flags (ap);

    NMUtilsSecurityType type = NMU_SEC_NONE;
    if (ap_rsn != NM_802_11_AP_SEC_NONE)
    {
        if (ap_rsn & NM_802_11_AP_SEC_KEY_MGMT_802_1X)
            type = NMU_SEC_WPA2_ENTERPRISE;
        else if (ap_rsn & NM_802_11_AP_SEC_KEY_MGMT_PSK)
            type = NMU_SEC_WPA2_PSK;
    }
    else if (ap_wpa != NM_802_11_AP_SEC_NONE)
    {
        if (ap_wpa & NM_802_11_AP_SEC_KEY_MGMT_802_1X)
            type = NMU_SEC_WPA_ENTERPRISE;
        else if (ap_wpa & NM_802_11_AP_SEC_KEY_MGMT_PSK)
            type = NMU_SEC_WPA_PSK;
    }
    else
    {
        if (ap_flags & NM_802_11_AP_FLAGS_PRIVACY)
            type = NMU_SEC_STATIC_WEP;
        else
            type = NMU_SEC_NONE;
    }
    data->sec_type = type;
    return type;
}

static gint
network_sort_function (const AppletAP *one, const AppletAP *two)
{
    gboolean t1 = one->connections != NULL;
    gboolean t2 = two->connections != NULL;

    if (t1 && !t2)
        return -1;
    else if (!t1 && t2)
        return 1;
    guint8 strength1 = nm_access_point_get_strength (one->ap);
    guint8 strength2 = nm_access_point_get_strength (two->ap);

    return strength2 - strength1;
}

static gboolean
ssid_compare (const GBytes *one, const GBytes *two)
{
    if (!one || !two)
        return 0;
    return g_bytes_equal (one, two);
}
static gboolean
network_compare (AppletAP *network, NMAccessPoint *ap)
{
    GBytes *ssid = nm_access_point_get_ssid (ap);
    if (!ssid_compare (network->ssid, ssid))
        return 0;
    if (network->mode != nm_access_point_get_mode (ap))
        return 0;
    if (network->security != get_ap_security_type (ap))
        return 0;
    return 1;
}

static AppletAP *
find_existing_network (AppletNMDeviceWireless *self, NMAccessPoint *ap)
{
    GSList *iter = NULL;
    for (iter = self->networks; iter; iter = iter->next)
    {
        AppletAP *ap_obj = iter->data;
        if (ap_obj->ap == ap)
            return ap_obj;
        for (GSList *it = ap_obj->aps; it; it = it->next)
            if (it->data == ap)
                return ap_obj;
    }
    return NULL;
}

static AppletAP *
find_network (AppletNMDeviceWireless *self, NMAccessPoint *ap)
{
    if (nm_access_point_get_ssid (ap) == NULL)
        return NULL;
    GSList *iter = NULL;
    for (iter = self->networks; iter; iter = iter->next)
    {
        if (network_compare (iter->data, ap))
            return iter->data;
    }
    return NULL;
}

static void
resort_menu_items (AppletNMDeviceWireless *self)
{
    AppletNMDevice *device = APPLET_NM_DEVICE (self);
    CdosPopupMenuSection *section = device->section;
    if (!APPLET_NM_DEVICE (device)->overflow_item)
    {
        APPLET_NM_DEVICE (device)->overflow_item = cdos_popup_submenu_menu_item_new (_("More..."), 0);
        cdos_popup_menu_base_add_menu_item (
            (CdosPopupMenuBase *)section, (CdosPopupBaseMenuItem *)device->overflow_item, -1);
    }
    GSList *iter;
    gint index = 0;
    CdosPopupSubMenu *menu = cdos_popup_submenu_menu_item_get_menu (device->overflow_item);
    for (iter = self->networks; iter; iter = iter->next)
    {
        if (index < NUM_VISIBLE_NETWORKS - 1)
        {
            index++;
            continue;
        }
        AppletAP *ap_obj = iter->data;
        if (ap_obj->is_more == 1)
            break;
        CdosPopupBaseMenuItem *item = ap_obj->item;
        g_object_ref (item);
        cdos_popup_menu_base_remove_menu_item ((CdosPopupMenuBase *)section, item);
        cdos_popup_menu_base_add_menu_item ((CdosPopupMenuBase *)menu, item, 0);
        ap_obj->is_more = 1;
    }
    // add by niyl
    if (self->menu_open_state_id <= 0)
    {
        self->menu_open_state_id =
            g_signal_connect (menu, "open-state-changed", G_CALLBACK (on_clicked_for_more), NULL);
    }
    set_more_item_height (menu);
}    

static void
access_point_added (NMDeviceWifi *device, NMAccessPoint *ap, AppletNMDeviceWireless *self)
{
    g_debug ("device: %p; ap added (%p): %s", device, ap, applet_nm_get_utf8_ssid (nm_access_point_get_ssid(ap)));
    AppletNMDevice *dev = (void *)self;
    GBytes *ssid = nm_access_point_get_ssid (ap);
    if (!ssid)
    {
        gulong notify_ssid = g_signal_connect (ap, "notify::ssid", G_CALLBACK (notify_ssid_cb), self);
        APData *data = get_ap_data (ap);
        data->notify_ssid = notify_ssid;
        return;
    }
    AppletAP *pos = find_network (self, ap);
    AppletAP *ap_obj;
    gboolean needs_update = 0;
    if (pos)
    {
        ap_obj = pos;
        if (g_slist_find (ap_obj->aps, ap))
        {
            g_debug ("%s: Access point was already seen, not adding again", __func__);
            return;
        }
        ap_obj->ap = ap;
        ap_obj->aps = g_slist_prepend (ap_obj->aps, ap);
        if (ap_obj->item)
        {
            applet_nm_menu_item_update_ap ((AppletNMMenuItem *)ap_obj->item, ap);
        }
    }
    else
    {
        ap_obj = g_slice_new0 (AppletAP);
        ap_obj->ssid = ssid;
        ap_obj->mode = nm_access_point_get_mode (ap);
        ap_obj->security = get_ap_security_type (ap);
        ap_obj->ap = ap;
        ap_obj->aps = g_slist_prepend (ap_obj->aps, ap);
        needs_update = 1;
    }
    GSList *iter = NULL;
    for (iter = dev->connections; iter; iter = iter->next)
    {
        NMConnection *connection = iter->data;
        if (nm_access_point_connection_valid (ap, connection) &&
            !g_slist_find (ap_obj->connections, connection))
        {
            ap_obj->connections = g_slist_prepend (ap_obj->connections, connection);
            needs_update = 1;
        }
    }
    if (needs_update)
    {
        if (ap_obj->item)
        {
            cdos_popup_base_menu_item_destroy (ap_obj->item);
            ap_obj->item = NULL;
        }
        if (pos)
            self->networks = g_slist_remove (self->networks, pos);
        /*
        if (self->networks == NULL)
        {
            self->networks = g_slist_prepend (self->networks, ap_obj);
            wifi_clear_section (dev);
            wifi_create_section (dev);
            return;
        }
        */
        int menu_pos = 0;
        GSList *ptr = NULL;
        for (ptr = self->networks; ptr && network_sort_function (ptr->data, ap_obj) < 0; ptr = ptr->next)
        {
            // tmp = ptr;
            if (ptr->data != self->active_network)
                menu_pos++;
        }
        self->networks = g_slist_insert_before (self->networks, ptr, ap_obj);
        if (1 /* should_show_connection_list ()*/)
        {
            menu_pos += (dev->active_connection_item ? 1 : 0);
            create_network_item (self, ap_obj, menu_pos);
        }
        /* FIXME */
        if (menu_pos <= NUM_VISIBLE_NETWORKS && g_slist_length (ptr) > NUM_VISIBLE_NETWORKS - menu_pos)
            resort_menu_items (self);
    }
}

static void
access_point_removed (NMDeviceWifi *device, NMAccessPoint *ap, AppletNMDeviceWireless *self)
{
    g_debug ("%s: device: %p, ap: %p, %s", __func__, device, ap, applet_nm_get_utf8_ssid (nm_access_point_get_ssid (ap)));
    AppletAP *res = find_existing_network (self, ap);
    AppletNMDevice *dev = (void *)self;
    if (!res)
    {
        g_warning ("Removing an access point that was never added");
        return;
    }
    AppletAP *ap_obj = res;
    ap_obj->aps = g_slist_remove (ap_obj->aps, ap);
    if (ap_obj->aps)
    {
        ap_obj->ap = ap_obj->aps->data;
        if (ap_obj->item)
            applet_nm_menu_item_update_ap ((AppletNMMenuItem *)ap_obj->item, ap_obj->ap);
        return;
    }

    ap_obj->ap = NULL;
    if (self->active_network == ap_obj)
        self->active_network = NULL;

    if (ap_obj->item)
        cdos_popup_base_menu_item_destroy (ap_obj->item);

    if (dev->overflow_item)
    {
        CdosPopupMenuBase *menu = (void *)cdos_popup_submenu_menu_item_get_menu (dev->overflow_item);
        GList *items = cdos_popup_menu_base_get_menu_items (menu);
        if (!ap_obj->is_more)
        {
            AppletNMMenuItem *item = items->data;
            AppletAP *_ap = find_existing_network (self, item->ap);
            cdos_popup_base_menu_item_destroy ((CdosPopupBaseMenuItem *)item);
            create_network_item (self, _ap, NUM_VISIBLE_NETWORKS - 2);
        }

        items = cdos_popup_menu_base_get_menu_items (menu);
        if (!items)
        {
            cdos_popup_base_menu_item_destroy ((CdosPopupBaseMenuItem *)dev->overflow_item);
            dev->overflow_item = NULL;
        }
    }
    self->networks = g_slist_remove (self->networks, res);
    g_slist_free (ap_obj->connections);
    g_slist_free (ap_obj->aps);
    g_slice_free (AppletAP, ap_obj);
}

static void
activate_ap (AppletNMMenuItem *item, gpointer p, gboolean keep, AppletNMDeviceWireless *device)
{
    AppletNMDevice *dev = (void *)device;
    NMAccessPoint *ap = item->ap;
    if (nm_access_point_connection_valid (ap, item->connection))
    {
        nm_client_activate_connection_async (dev->client, item->connection,
                                             dev->device, NULL, NULL,
                                             activate_connection_cb, NULL);
    }
}

static CdosPopupBaseMenuItem *
create_ap_item (AppletNMDeviceWireless *self, NMConnection *connection, NMAccessPoint *ap, gboolean use_connection_name)
{
    NMSettingConnection *s_con = nm_connection_get_setting_connection (connection);
    const char *name = nm_setting_connection_get_id (s_con);
    AppletNMMenuItem *item = applet_nm_menu_item_new (connection, ap, use_connection_name ? name : NULL, NULL);
    g_signal_connect (item, "activate", G_CALLBACK (activate_ap), self);
    return (CdosPopupBaseMenuItem *)item;
}

/*
static NMConnection *
wifi_create_auto_connection (NMAccessPoint *ap)
{
    char *name = NULL;
    const gchar *ssid = nm_utils_ssid_to_utf8 (nm_access_point_get_ssid (ap));
    if (ssid)
    {
        char *tmp = g_strdup_printf ("Auto %s", ssid);
        name = _(tmp);
        g_free (tmp);
    }
    else
        name = _("Auto wireless");

    NMConnection *conn = nm_connection_new ();
    nm_connection_add_setting (conn, nm_setting_wireless_new ());
    NMSetting *s_con = nm_setting_connection_new ();
    g_object_set (s_con,
                  "autoconnect", 1,
                  "id", name,
                  "uuid", nm_utils_uuid_generate (),
                  "type", NM_SETTING_WIRELESS_SETTING_NAME,
                  NULL);
    nm_connection_add_setting (conn, s_con);
    return conn;
}
*/

static void
activate_callback (AppletNMMenuItem *item, ClutterEvent *e, gboolean k, AppletNMDeviceWireless *device)
{
    NMDevice *dev = ((AppletNMDevice *)device)->device;
    NMAccessPoint *ap = item->ap;
    NMUtilsSecurityType sec_type = get_ap_security_type (ap);
    if (sec_type == NMU_SEC_WPA2_ENTERPRISE || sec_type == NMU_SEC_WPA_ENTERPRISE)
    {
        const gchar *args[] = {"found-control-center",
                               "network",
                               "connect-8021x-wifi",
                               nm_object_get_path ((NMObject *)dev),
                               nm_object_get_path ((NMObject *)ap)};
        cdos_util_spawn (args);
    }
    else
    {
        //NMConnection *conn = wifi_create_auto_connection (ap);
        NMConnection *conn = nm_simple_connection_new ();
        nm_client_add_and_activate_connection_async (((AppletNMDevice *)device)->client,
                                                     conn, dev,
                                                     nm_object_get_path ((NMObject *)ap),
                                                     NULL,
                                                     add_and_activate_connection_cb, NULL);
    }
}

static void
create_network_item (AppletNMDeviceWireless *device, AppletAP *ap_obj, gint position)
{
    if (!ap_obj || !ap_obj->ap)
    {
        return;
    }
    if (ap_obj->connections)
    {
        if (ap_obj->connections->next == NULL)
        {
            ap_obj->item = create_ap_item (device, ap_obj->connections->data, ap_obj->ap, 0);
        }
        else
        {
            g_warning ("connections length > 1");
            ap_obj->item = create_ap_item (device, ap_obj->connections->data, ap_obj->ap, 0);
            /* connections->length > 1 */
        }
    }
    else
    {
        ap_obj->item = (CdosPopupBaseMenuItem *)applet_nm_menu_item_new (NULL, ap_obj->ap, NULL, NULL);
        g_signal_connect (ap_obj->item, "activate", G_CALLBACK (activate_callback), device);
    }
    CdosPopupMenuSection *section = ((AppletNMDevice *)device)->section;
    if (position < NUM_VISIBLE_NETWORKS)
    {
        ap_obj->is_more = 0;
        cdos_popup_menu_base_add_menu_item ((CdosPopupMenuBase *)section, ap_obj->item, position);
    }
    else
    {
        if (!APPLET_NM_DEVICE (device)->overflow_item)
        {
            APPLET_NM_DEVICE (device)->overflow_item = cdos_popup_submenu_menu_item_new (_("More..."), 0);
            cdos_popup_menu_base_add_menu_item (
                (CdosPopupMenuBase *)section, (CdosPopupBaseMenuItem *)APPLET_NM_DEVICE (device)->overflow_item, -1);
        }
        CdosPopupSubMenu *menu = cdos_popup_submenu_menu_item_get_menu (((AppletNMDevice *)device)->overflow_item);
        cdos_popup_menu_base_add_menu_item ((CdosPopupMenuBase *)menu, ap_obj->item, position - NUM_VISIBLE_NETWORKS);
        ap_obj->is_more = 1;
        // add by niyl
        if (device->menu_open_state_id <= 0)
        {
            device->menu_open_state_id =
                g_signal_connect (menu, "open-state-changed", G_CALLBACK (on_clicked_for_more), NULL);
        }

        set_more_item_height (menu);
        // add end
    }
}

static void
create_active_connection_item (AppletNMDeviceWireless *device)
{
    AppletNMDevice *dev = APPLET_NM_DEVICE (device);
    NMActiveConnection *active = dev->active_connection;
    /*
    if (!active)
        active = nm_device_get_active_connection (dev->device);
    */
    NMRemoteConnection *r_con = nm_active_connection_get_connection (active);
    NMConnection *connection = (NMConnection *)r_con;
    if (device->active_network && dev->active_connection_item)
    {
        cdos_popup_base_menu_item_destroy (dev->active_connection_item);
        dev->active_connection_item = NULL;
    }

    if (connection)
    {
        if (!device->active_network)
        {
            NMAccessPoint *ap = nm_device_wifi_get_active_access_point ((NMDeviceWifi *)dev->device);
            if (ap)
            {
                AppletAP *ap_obj = find_network (device, ap);
                device->active_network = ap_obj;
            }
        }
        if (device->active_network)
        {
            PopupBaseMenuItemInit params;
            cdos_popup_base_menu_item_get_init_params (&params);
            params.reactive = 0;
            dev->active_connection_item = (CdosPopupBaseMenuItem *)applet_nm_menu_item_new (
                connection, device->active_network->ap, NULL, &params);
            // cdos_popup_base_menu_item_set_params (dev->active_connection_item, &params);
        }
    }
    else
    {
        if (device->active_network)
        {
            PopupBaseMenuItemInit params;
            cdos_popup_base_menu_item_get_init_params (&params);
            params.reactive = 0;
            dev->active_connection_item =
                (CdosPopupBaseMenuItem *)applet_nm_menu_item_new (NULL, device->active_network->ap, NULL, &params);
        }
        else
        {
            /* FIXME */
            g_warning ("FIXME: %s", __func__);
        }
    }
    if (dev->active_connection_item)
        cdos_popup_base_menu_item_set_ornament (dev->active_connection_item, CDOS_ORNAMENT_DOT);
}

static void
wifi_create_section (AppletNMDevice *device)
{
    AppletNMDeviceWireless *wdev = (void *)device;
    gint active_offset = 0;
    if (device->active_connection)
    {
        create_active_connection_item (wdev);
        CdosPopupMenuBase *section = (CdosPopupMenuBase *)device->section;
        if (device->active_connection_item)
        {
            cdos_popup_menu_base_add_menu_item (section, device->active_connection_item, -1);
            active_offset = 1;
        }
    }
    else
    {
        g_warn_if_fail (wdev->active_network == NULL);
        wdev->active_network = NULL;
    }
    GSList *iter = NULL;
    gint j = 0;
    for (iter = wdev->networks; iter; j++, iter = iter->next)
    {
        AppletAP *ap_obj = iter->data;
        if (ap_obj == wdev->active_network)
            continue;
        create_network_item (wdev, ap_obj, j + active_offset);
    }
}

static void
wifi_clear_section (AppletNMDevice *device)
{
    AppletNMDeviceWireless *wdev = APPLET_NM_DEVICE_WIRELESS (device);
    applet_nm_device_clear_section_real (device);
    GSList *iter;
    for (iter = wdev->networks; iter; iter = iter->next)
    {
        AppletAP *ap_obj = iter->data;
        ap_obj->item = NULL;
    }
    device->overflow_item = NULL;
}

static void
wifi_add_connection (AppletNMDevice *device, NMConnection *connection)
{
    device->connections = g_slist_append (device->connections, connection);
    AppletNMDeviceWireless *self = APPLET_NM_DEVICE_WIRELESS (device);
    GSList *iter;
    for (iter = self->networks; iter; iter = iter->next)
    {
        AppletAP *ap_obj = iter->data;
        NMAccessPoint *ap = ap_obj->ap;
        if (nm_access_point_connection_valid (ap, connection))
        {
            ap_obj->connections = g_slist_append (ap_obj->connections, connection);
            break;
        }
    }
    if (iter == NULL)
        return;

    self->networks = g_slist_sort (self->networks, (GCompareFunc)network_sort_function);
    wifi_clear_section (device);
    wifi_create_section (device);
}

static NMConnection *
find_connection (AppletNMDeviceWireless *wdev, const char *uuid)
{
    GSList *iter;
    AppletNMDevice *device = APPLET_NM_DEVICE (wdev);
    for (iter = device->connections; iter; iter = iter->next)
    {
        NMConnection *conn = iter->data;
        if (g_strcmp0 (nm_connection_get_uuid (conn), uuid) == 0)
            return conn;
    }
    return NULL;
}

static void
wifi_remove_connection (AppletNMDevice *device, NMConnection *connection)
{
    const char *uuid = nm_connection_get_uuid (connection);
    if (!uuid)
        return;

    AppletNMDeviceWireless *wdev = APPLET_NM_DEVICE_WIRELESS (device);
    NMConnection *pos = find_connection (wdev, uuid);

    device->connections = g_slist_remove (device->connections, pos);

    GSList *iter;
    gboolean update = FALSE;

    for (iter = wdev->networks; iter; iter = iter->next)
    {
        AppletAP *ap_obj = iter->data;
        update = (ap_obj->connections && (ap_obj->connections->data == connection)) || update;

        GSList *k;
        for (k = ap_obj->connections; k; k = k->next)
        {
            if (g_strcmp0 (nm_connection_get_uuid (k->data), uuid) == 0)
            {
                ap_obj->connections = g_slist_remove (ap_obj->connections, connection);
                update = update || (ap_obj->connections == NULL);
                break;
            }
        }
    }
    if (update)
    {
        wdev->networks = g_slist_sort (wdev->networks, (GCompareFunc)network_sort_function);
        wifi_clear_section (device);
        wifi_create_section (device);
    }
}

AppletNMDeviceClass *
applet_nm_device_wireless_class_new (AppletNM *applet)
{
    AppletNMDeviceClass *klass = g_new0 (AppletNMDeviceClass, 1);
    *klass = *applet->base_class;
    // klass->create_auto_connection = wifi_create_auto_connection;
    klass->create_section = wifi_create_section;
    klass->clear_section = wifi_clear_section;
    klass->activate = applet_nm_device_wireless_activate;
    // parent_class->deactivate = applet_nm_device_wireless_deactivate;
    klass->add_connection = wifi_add_connection;
    klass->remove_connection = wifi_remove_connection;
    return klass;
}

AppletNMDeviceWireless *
applet_nm_device_wireless_new (AppletNM *applet,
                               NMDevice *device,
                               const GPtrArray *connections)
{
    AppletNMDeviceWireless *wdev = g_new0 (AppletNMDeviceWireless, 1);
    APPLET_NM_DEVICE (wdev)->klass = applet->wifi_class;
    applet_nm_device_wireless_init (wdev, applet->client, (NMDeviceWifi *)device, connections);
    applet_nm_device_init ((AppletNMDevice *)wdev, applet, device, connections);
    return wdev;
}
