#include "applet-nm-menu-item.h"
#include "applet-nm-device-wireless.h"

G_DEFINE_TYPE (AppletNMMenuItem, applet_nm_menu_item, CDOS_TYPE_POPUP_BASE_MENU_ITEM);

static gboolean
ap_is_encrypted (NMAccessPoint *ap)
{
    guint32 ap_flags, ap_wpa, ap_rsn;
    ap_flags = nm_access_point_get_flags (ap);
    ap_wpa = nm_access_point_get_wpa_flags (ap);
    ap_rsn = nm_access_point_get_rsn_flags (ap);
    if ((ap_flags & NM_802_11_AP_FLAGS_PRIVACY) || ap_wpa || ap_rsn)
        return 1;
    return 0;
}

const gchar *
applet_nm_get_icon (NMAccessPoint *ap, guint8 strength)
{
    static const gchar *signal_icons[] = {
        "found-network-wireless-signal-none",
        "found-network-wireless-signal-weak",
        "found-network-wireless-signal-ok",
        "found-network-wireless-signal-good",
        "found-network-wireless-signal-excellent",
        "found-network-workgroup"
    };
    static const gchar *signal_icons_encrypted[] = {
        "found-network-wireless-signal-none-encrypted",
        "found-network-wireless-signal-weak-encrypted",
        "found-network-wireless-signal-ok-encrypted",
        "found-network-wireless-signal-good-encrypted",
        "found-network-wireless-signal-excellent-encrypted",
    };
    if (nm_access_point_get_mode (ap) == NM_802_11_MODE_ADHOC)
        return signal_icons[5];
    if (ap_is_encrypted (ap))
    {
        if (strength <= 4)
            return signal_icons_encrypted[strength];
        else
            return signal_icons_encrypted[4];
    }
    else
    {
        if (strength <= 4)
            return signal_icons[strength];
        else
            return signal_icons[4];
    }

    /*
    if (ap_is_encrypted (ap))
        return g_strdup_printf ("nm-signal-%d-secure", strength);
    else
        return g_strdup_printf ("nm-signal-%d", strength);
    */
}

static void
ap_signal_updated (NMAccessPoint *ap, GParamSpec *pspec, AppletNMMenuItem *self)
{
    guint8 strength = nm_access_point_get_strength (ap);
    guint8 new_strength = applet_nm_signal_to_icon (strength);
    if (new_strength == self->last_strength)
        return;

    self->last_strength = new_strength;
    const gchar *name = applet_nm_get_icon (ap, new_strength);

    StIcon *icon = self->signal_icon;
    st_icon_set_icon_name (icon, name);
}

static void
update_title (AppletNMMenuItem *item, const char *title)
{
    char *_title;
    if (!title)
    {
        GBytes *ssid = nm_access_point_get_ssid (item->ap);
        _title = applet_nm_get_utf8_ssid (ssid);
    } else {
        _title = g_strdup (title);
    }

    if (item->label)
    {
        st_label_set_text (item->label, _title);
    } else {
        item->label = st_label_new (_title);
    }
    g_free (_title);
}

static void
applet_nm_menu_item_init_after (AppletNMMenuItem *item, NMAccessPoint *ap, const char *title)
{
    item->ap = ap; // s = g_slist_sort (aps, (GCompareFunc)sort_access_point);
    st_widget_add_style_class_name ((StWidget *)item, "nm-menu-item");
    cdos_popup_base_menu_item_set_ornament ((CdosPopupBaseMenuItem *)item, CDOS_ORNAMENT_SPACE);
    update_title (item, title);
    ActorChild ac;
    ac.expand = TRUE;
    ac.align = ST_ALIGN_START;
    cdos_popup_base_menu_item_add_actor ((CdosPopupBaseMenuItem *)item, 
            (ClutterActor *)item->label, &ac);

    guint8 strength = nm_access_point_get_strength (ap);
    item->last_strength = applet_nm_signal_to_icon (strength);
    /* TODO labelStrength */

    StWidget *icons = st_box_layout_new ();
    st_widget_set_style_class_name (icons, "nm-menu-item-icons");
    item->icons = (StBoxLayout *)icons;

    item->signal_icon = (StIcon *)st_icon_new ();
    st_widget_set_style_class_name ((StWidget *)item->signal_icon, "nm-menu-item-icon");
    const gchar *icon_name = applet_nm_get_icon (ap, item->last_strength);
    st_icon_set_icon_name (item->signal_icon, icon_name);
    clutter_actor_add_child ((ClutterActor *)icons, (ClutterActor *)item->signal_icon);

    ac.expand = FALSE;
    ac.align = ST_ALIGN_END;
    cdos_popup_base_menu_item_add_actor ((CdosPopupBaseMenuItem *)item,
            (ClutterActor *)icons, &ac);
    item->update_id = g_signal_connect (ap, "notify::strength",
            G_CALLBACK (ap_signal_updated), item);
}

AppletNMMenuItem *
applet_nm_menu_item_new (NMConnection *connection,
                         NMAccessPoint *ap,
                         const char *title,
                         PopupBaseMenuItemInit *params)
{
    AppletNMMenuItem *item = g_object_new (APPLET_TYPE_NM_MENU_ITEM, NULL);
    applet_nm_menu_item_init_after (item, ap, title);
    item->connection = connection;
    cdos_popup_base_menu_item_set_params ((CdosPopupBaseMenuItem *)item, params);

    return item;
}


static void
applet_nm_menu_item_finalize (GObject *item)
{
    // AppletNMMenuItem *self = (AppletNMMenuItem *)item;

    G_OBJECT_CLASS (applet_nm_menu_item_parent_class)->finalize (item);
}

static void
applet_nm_menu_item_dispose (GObject *item)
{
    AppletNMMenuItem *self = (AppletNMMenuItem *)item;
    if (self->ap && self->update_id > 0)
    {
        /* access-point 可能已经被移除 */
        if (NM_IS_ACCESS_POINT (self->ap))
            g_signal_handler_disconnect (self->ap, self->update_id);
        self->update_id = 0L;
        self->ap = NULL;
    }
    G_OBJECT_CLASS (applet_nm_menu_item_parent_class)->dispose (item);
}

static void
applet_nm_menu_item_init (AppletNMMenuItem *self)
{
}

static void
applet_nm_menu_item_class_init (AppletNMMenuItemClass *klass)
{
    GObjectClass *object_class = G_OBJECT_CLASS (klass);
    object_class->finalize = applet_nm_menu_item_finalize;
    object_class->dispose = applet_nm_menu_item_dispose;
}

void
applet_nm_menu_item_update_ap (AppletNMMenuItem *item, NMAccessPoint *ap)
{
    if (NM_IS_ACCESS_POINT (item->ap))
        if (item->update_id > 0)
            g_signal_handler_disconnect (item->ap, item->update_id);
    item->ap = ap;
    update_title (item, NULL);
    item->update_id = g_signal_connect (ap, "notify::strength", G_CALLBACK (ap_signal_updated), item);
}
