#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include "applet-nm-wireless-section-menu-item.h"

enum AppletNMWirelessSectionMenuItemSignals
{
    ENABLED_CHANGED,
    LAST_SIGNAL
};
static guint applet_nm_wireless_section_menu_item_signals[LAST_SIGNAL] = {0};

G_DEFINE_TYPE (AppletNMWirelessSectionMenuItem, applet_nm_wireless_section_menu_item, CDOS_TYPE_POPUP_SWITCH_MENU_ITEM);

static void 
applet_nm_wireless_section_menu_item_activate (CdosPopupBaseMenuItem *item,
                                               gpointer event,
                                               gboolean keepmenu)
{
    AppletNMWirelessSectionMenuItem *self = APPLET_NM_WIRELESS_SECTION_MENU_ITEM (item);
    /* 不能再调用基类的activate(), 否则 "activate"信号和这两个
     * activate() 函数会循环调用*/
    // cdos_popup_base_menu_item_activate (item, event, keepmenu);

    if (self->device == NULL)
    {
        g_warning ("Section title activated when there is more than one device, should be non reactive");
        return;
    }
    gboolean state = cdos_popup_switch_menu_item_get_status ((CdosPopupSwitchMenuItem *)item);
    self->set_enable_func (self->client, state);

    gboolean new_state = cdos_popup_switch_menu_item_get_status ((CdosPopupSwitchMenuItem *)item);
    if (new_state)
        applet_nm_device_activate (self->device);
    else
        applet_nm_device_deactivate (self->device);

    g_signal_emit (self, applet_nm_wireless_section_menu_item_signals[ENABLED_CHANGED], 0, new_state);
}

static void
applet_nm_wireless_section_menu_item_init (AppletNMWirelessSectionMenuItem *self)
{
}

static void
applet_nm_wireless_section_menu_item_class_init (AppletNMWirelessSectionMenuItemClass *klass)
{
    CdosPopupBaseMenuItemClass *parent_class = CDOS_POPUP_BASE_MENU_ITEM_CLASS (klass);
    parent_class->activate = applet_nm_wireless_section_menu_item_activate;

    applet_nm_wireless_section_menu_item_signals[ENABLED_CHANGED] =
        g_signal_new ("enabled-changed",
                      APPLET_TYPE_NM_WIRELESS_SECTION_MENU_ITEM,
                      G_SIGNAL_RUN_FIRST,
                      0,
                      NULL, NULL,
                      g_cclosure_marshal_VOID__UINT,
                      G_TYPE_NONE, 1,
                      G_TYPE_UINT);
}

static void
applet_nm_wireless_section_menu_item_property_changed (NMClient *client,
                                                       GParamSpec *spec,
                                                       AppletNMWirelessSectionMenuItem *self)
{
    self->software_enabled = self->get_enable_func (client);
    self->hardware_enabled = self->hardware_get_enabled (client);

    gboolean enabled = self->software_enabled && self->hardware_enabled;
    cdos_popup_switch_menu_item_set_toggle_status ((CdosPopupSwitchMenuItem *)self, enabled);
    if (!self->hardware_enabled)
        cdos_popup_base_menu_item_set_sensitive ((CdosPopupBaseMenuItem *)self, FALSE);
    else 
        cdos_popup_base_menu_item_set_sensitive ((CdosPopupBaseMenuItem *)self, TRUE);

    g_signal_emit (self, applet_nm_wireless_section_menu_item_signals[ENABLED_CHANGED], 0, self);
}

AppletNMWirelessSectionMenuItem *
applet_nm_wireless_section_menu_item_new (NMClient *client,
                                          AppletNMDeviceType type,
                                          const char *title)
{
    AppletNMWirelessSectionMenuItem *self = g_object_new (APPLET_TYPE_NM_WIRELESS_SECTION_MENU_ITEM, NULL);

    cdos_popup_switch_menu_item_set_text ((CdosPopupSwitchMenuItem *)self, title);

    self->client = client;
    // self->property = g_strconcat (property, "-enabled", NULL);
    // self->property_hardware = g_strconcat (property, "-hardware_enabled", NULL);
    self->device_type = type;
    if (type == DeviceWireless)
    {
        self->set_enable_func = nm_client_wireless_set_enabled;
        self->get_enable_func = nm_client_wireless_get_enabled;
        self->hardware_get_enabled = nm_client_wireless_hardware_get_enabled;
    }
    else if (type == DeviceWwan)
    {
        self->set_enable_func = nm_client_wwan_set_enabled;
        self->get_enable_func = nm_client_wwan_get_enabled;
        self->hardware_get_enabled = nm_client_wwan_hardware_get_enabled;
    }
    else if (type == DeviceWimax)
    {
        self->set_enable_func = nm_client_wimax_set_enabled;
        self->get_enable_func = nm_client_wimax_get_enabled;
        self->hardware_get_enabled = nm_client_wimax_hardware_get_enabled;
    }
    else
    {
        g_warning ("Unsupported property");
        return NULL;
    }
    const char *property = applet_nm_device_info[type].value;
    int len = strlen (property);
    char *tmp = g_new0 (char, len + 26);
    snprintf (tmp, len + 26, "notify::%s-enabled", property);

    g_signal_connect (self->client, tmp, G_CALLBACK (applet_nm_wireless_section_menu_item_property_changed), self);

    memset (tmp, 0, len + 26);
    snprintf (tmp, len + 26, "notify::%s-hardware-enabled", property);
    g_signal_connect (self->client, tmp, G_CALLBACK (applet_nm_wireless_section_menu_item_property_changed), self);
    g_free (tmp);
    applet_nm_wireless_section_menu_item_property_changed (client, NULL, self);

    return self;
}

void
applet_nm_wireless_section_menu_item_update_for_device (CdosPopupSwitchMenuItem *self, AppletNMDevice *device)
{
    AppletNMWirelessSectionMenuItem *item = APPLET_NM_WIRELESS_SECTION_MENU_ITEM (self);
    item->device = device;
    if (device && item->software_enabled && item->hardware_enabled)
    {
        const char *text = applet_nm_device_get_status_label (device);
        g_message ("%s: %s", nm_device_get_description (device->device), text);
    }
}
