#ifndef __APPLET_NM_H__
#define __APPLET_NM_H__

#include <gio/gio.h>
#include <NetworkManager.h>

#include "cdos-global.h"
#include <clutter/clutter.h>
#include "st.h"

#include "cdos-icon-applet.h"
#include "cdos-popup-menu-section.h"
#include "cdos-popup-switch-menu-item.h"
#include "cdos-popup-sub-menu.h"
#include "cdos-popup-submenu-menu-item.h"
#include "cdos-message-tray.h"

typedef struct
{
    CdosPopupMenuSection *section;
    GList *devices; // unsure
    CdosPopupSwitchMenuItem *item;
} AppletNMDeviceObj;

#define NUM_VISIBLE_NETWORKS 5

typedef enum _AppletNMDeviceType {
    DeviceInvalid = -1,
    DeviceWired = 0, //       *wired;
    DeviceWireless, //    *wireless;
    DeviceWwan, //        *wwan;
    DeviceWimax,
    DeviceVPN, //         *vpn;
    DeviceCount
} AppletNMDeviceType;

typedef struct _AppletEnumValue
{
    gint key;
    const char *value;
} AppletEnumValue;

extern const AppletEnumValue applet_nm_device_info[DeviceCount];

#define APPLET_NM_CONNECTION(conn) ((AppletNMConnection *)conn)
typedef struct _AppletNMConnection AppletNMConnection;
struct _AppletNMConnection
{
    NMConnection        *connection;
    const char          *name;
    const char          *type;
    const char          *uuid;
    guint64              timestamp;
    CdosPopupMenuItem   *item;
    AppletNMDeviceType   section;
    gulong               removed_id;
    gulong               updated_id;
};

#define APPLET_NM_DEVICE(obj) ((AppletNMDevice *)obj)

typedef struct _AppletNMDevice AppletNMDevice;
typedef struct _AppletNMDeviceClass AppletNMDeviceClass;

struct _AppletNMDevice
{
    CdosPopupMenuSection    *section;
    NMDevice                *device;
    NMClient                 *client;
    NMDeviceStateReason      carrier;
    
    GSList                  *connections;
    NMActiveConnection      *active_connection;    
    CdosPopupSwitchMenuItem *status_item;
    CdosPopupBaseMenuItem   *active_connection_item;
    CdosPopupSubmenuMenuItem *overflow_item;
    CdosPopupMenuItem       *auto_connection_item;
    char *auto_connection_name;
    
    CdosNotification        *notification;


    AppletNMDeviceType category;
    gulong device_state_changed_id;
    gulong destroy_id;
    gulong activation_failed_id;
    gulong carried_changed_id;
    gulong state_changed_id;
    gulong status_changed_id;
    gulong firmware_missing_id;
    gboolean create_or_clear_in_progress;
    gboolean connected;
    gpointer data;

    AppletNMDeviceClass *klass;
};


struct _AppletNMDeviceClass
{
    NMConnection *(*create_auto_connection) (AppletNMDevice *self);
    void (*add_connection) (AppletNMDevice *self, NMConnection *conn);
    void (*remove_connection) (AppletNMDevice *self, NMConnection *conn);
    void (*create_section) (AppletNMDevice *self);
    void (*clear_section) (AppletNMDevice *self);
    void (*set_enabled) (AppletNMDevice *self, gboolean enable);
    void (*activate) (AppletNMDevice *self);
    void (*deactivate) (AppletNMDevice *self);
    void (*destroy) (AppletNMDevice *self);
};

GType applet_nm_get_type (void);

#define APPLET_TYPE_NM		(applet_nm_get_type ())
#define APPLET_NM(obj)              (G_TYPE_CHECK_INSTANCE_CAST ((obj), APPLET_TYPE_NM, AppletNM))
#define APPLET_IS_NM(obj)           (G_TYPE_CHECK_INSTANCE_TYPE ((obj), APPLET_TYPE_NM))
#define APPLET_NM_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST ((klass), APPLET_TYPE_NM, AppletNMClass))
#define APPLET_IS_NM_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE ((klass), APPLET_TYPE_NM))
#define APPLET_NM_GET_CLASS(obj)    (G_TYPE_INSTANCE_GET_CLASS ((obj), APPLET_TYPE_NM, AppletNMClass))

typedef struct _AppletNM AppletNM;
typedef struct _AppletNMClass AppletNMClass;

struct _AppletNM
{
    CdosIconApplet parent_instance;
    NMClient *client;
    GDBusConnection *bus;

    AppletNMDeviceClass *base_class;
    AppletNMDeviceClass *ethernet_class;
    AppletNMDeviceClass *wifi_class;
    AppletNMDeviceClass *gdm_class;
    AppletNMDeviceClass *cdma_class;
    AppletNMDeviceClass *wimax_class;

    GPtrArray                   *connections;
    NMActiveConnection          *main_connection;
    NMAccessPoint               *active_access_point;
    gint                         connectivity;

    MetaData                    *metadata;
    CdosPopupMenuManager        *menu_manager;
    CdosPopupMenu               *menu;
    gchar                       *current_icon_name;
    CdosPopupMenuSection        *status_section;
    CdosPopupMenuItem           *status_item;
    AppletNMDeviceObj           *devices;
    gulong                       connections_read_id;
    gboolean                     inited;
    gboolean                     devices_hidden;

    gulong                      wireless_active_ap_changed;
};

struct _AppletNMClass
{
    CdosIconAppletClass parent_class;
};

CdosApplet *applet_network_main (MetaData *meta, guint orientation, guint panel_height);

gint nm_g_ptr_array_find (const GPtrArray *array, gpointer element);

GSList *applet_nm_get_all_connections (AppletNM *applet);

void applet_nm_state_changed (AppletNM *applet, AppletNMDevice *device);
#endif
