/** 
 * Copyright (c) 2016-2018 CPU and Fundamental Software Research Center, CAS
 *  
 * This software is published by the license of CPU-OS License，you can use and 
 * distibuted this software under this License. See CPU-OS Liense for more detail.
 *
 * You should have receive a copy of CPU-OS License. If not, please contact us 
 * by email <support_os@cpu-os.ac.cn>  
 * 
 **/
/**
 * SECTION: AppletNotification
 * @short_description: 通知应用
 * @include: applet-notification.h
 * 
 * AppletNotificationClass 类用来接收，展示，处理 #CdosMessageTray 发送来地消息
 */
#include <stdio.h>
#include <string.h>
#include <sys/time.h>
#include <math.h>

#include "cdos-layout-manager.h"
#include "cdos-message-tray.h"
#include "cdos-start.h"
#include "cdos-scale.h"
#include "cdos-mpris2-player.h"
#include "applet-player.h"
#include "common.h"
#include "cdos-lunar.h"

#include "applet-notifications2.h"
#include "applet-airplane-mode-button.h"
#include "applet-quiet-hours-button.h"
#include "applet-luminance-button.h"
#include "applet-project-button.h"
#include "applet-all-settings-button.h"

#define SYSTEMD_DBUS_NAME        "org.freedesktop.login1"
#define SYSTEMD_DBUS_PATH        "/org/freedesktop/login1"
#define SYSTEMD_DBUS_INTERFACE   "org.freedesktop.login1.Manager"

#define APPLET_NOTIFICATIONS2_MAX_COUNT 100

enum
{
    NOTIFICATIONS_SHOW,
    NOTIFICATIONS_HIDE,
    LAST_SIGNAL    
};

struct _AppletNotifications2Private
{
    GDBusProxy *logind_proxy;
    gpointer user_data;
};

static guint applet_notifications2_signals [LAST_SIGNAL] = {0};

G_DEFINE_TYPE_WITH_PRIVATE (AppletNotifications2, applet_notifications2, CDOS_TYPE_TEXT_ICON_APPLET)

static void applet_notifications2_update_list (AppletNotifications2 *self);
static void applet_notifications2_update_icon (AppletNotifications2 *self);

static gboolean on_calendar_release (StButton *button, gint clicked_button, gpointer data);

/*
static gchar *
applet_notifications2_timeify (GDateTime* orig_time, const char *format)
{
    GDateTime* now_timestamp = g_date_time_new_now_local();
    GTimeSpan diff = g_date_time_difference(now_timestamp, orig_time) / 1000000;
    gchar * str = NULL, *str_diff = NULL;
    gchar *orig = g_date_time_format (orig_time, format);

    if (diff <= 15) {
        str_diff = g_strdup (" (Just now)");
    } else if (diff > 15 && diff <= 59) {
        gchar s_diff[3];
        s_diff[0] = '0' + diff / 10;
        s_diff[1] = '0' + diff % 10;
        s_diff[2] = '\0';
        str_diff = g_strdup_printf (" (%s seconds ago)", s_diff);
    } else if (diff > 59 && diff <= 119) {
        gchar s_diff[2];
        s_diff[0] = '0' + diff / 60;
        s_diff[1] = '\0';
        str_diff = g_strdup_printf (" (%s minute ago)", s_diff);
    //} else if (diff > 119 && diff <= 3540){
    } else if (diff > 119 && diff <= 599){
        gchar s_diff[2];
        s_diff[0] = '0' + diff / 60;
        s_diff[1] = '\0';
        str_diff = g_strdup_printf (" (%s minutes ago)", s_diff);
    } else if (diff > 599 && diff <= 3540){
        gchar s_diff[3];
        gint d = diff / 60;
        s_diff[0] = '0' + d / 10;
        s_diff[1] = '0' + d % 10;
        s_diff[2] = '\0';
        str_diff = g_strdup_printf (" (%s minutes ago)", s_diff);
    }
    char *tmp = _(str_diff);
    g_free (str_diff);
    str = g_strconcat (orig, tmp, NULL);
    g_free (orig);
    g_free (tmp);

    g_date_time_unref(now_timestamp);
    
    return str;
}
*/

static void
notification_update_calendar (AppletNotifications2 *self)
{
    GDateTime* now = g_date_time_new_now_local();
    gint day = g_date_time_get_day_of_month (now);
    gint month = g_date_time_get_month(now);
    gint year = g_date_time_get_year(now);

    gchar *currentstrtmp = g_date_time_format(now, "%b%d  %A");
    st_label_set_text (self->timelabel, currentstrtmp);

    const gchar *ret_lunar_month = NULL, *ret_lunar_day = NULL;
    cdos_sun_to_lunar (year, month, day, NULL, &ret_lunar_month, &ret_lunar_day, NULL);
    const gchar *ganzhi_of_year = cdos_get_ganzhi_of_year (year, month, day);
    gchar lunarstr[128] = {0};
    g_snprintf (lunarstr, sizeof(lunarstr), "%s年%s%s", ganzhi_of_year, ret_lunar_month, ret_lunar_day);
    st_label_set_text (self->lunarlabel, lunarstr);
}

static void
clear_all_notifications (StButton *widget,
                         gint      button,
                         AppletNotifications2 *self)
{
    GList *notifications = self->notifications;

    if (notifications)
    {
        while (notifications != NULL)
        {
            CdosNotification *notification = NULL;
            notification = CDOS_NOTIFICATION (notifications->data);

            g_signal_handlers_disconnect_by_data (notification, self);
            cdos_notification_destroy (notification, CDOS_NOTIFICATION_DISMISSED); 
            notifications = notifications->next;
        }
        g_list_free(self->notifications);
    }

    
    self->notification_count = 0;
    self->notifications = NULL;
    applet_notifications2_update_list (self);

    applet_notifications2_update_icon (self);
}

static void
on_workarea_changed (MetaDisplay *display, AppletNotifications2 *self)
{
    MetaWorkspaceManager *wsm = meta_display_get_workspace_manager (display);
    gint index = meta_display_get_primary_monitor (display);
    MetaRectangle rect;
    MetaWorkspace *wks = meta_workspace_manager_get_active_workspace (wsm);
    meta_workspace_get_work_area_for_monitor (wks, index, &rect);

    float x, y, width, height;
    x = rect.x + rect.width;
    y = rect.y;
    width = clutter_actor_get_width (self->actor);
    height = rect.height;
    self->pos_x = x - width;
    self->pos_y = y;
    clutter_actor_set_height (self->actor, height);
    if (!clutter_actor_is_visible (self->actor))
        clutter_actor_set_position (self->actor, x, y);
    else
        clutter_actor_set_position (self->actor, self->pos_x, y);
}

static void
applet_notifications_on_player_destroyed (AppletPlayer *player, AppletNotifications2 *self)
{
    self->players = g_list_remove (self->players, player);
}

static void
applet_notifications2_add_player (AppletPlayer *player, gpointer user_data)
{
    AppletNotifications2 *self = user_data;
    clutter_actor_insert_child_above (self->actor, (ClutterActor *)player, (ClutterActor *)self->calendar);
    clutter_actor_set_y_align ((ClutterActor *)player, CLUTTER_ACTOR_ALIGN_END);
    g_signal_connect (player, "destroy", G_CALLBACK(applet_notifications_on_player_destroyed), self);

    self->players = g_list_prepend (self->players, player);
}

static void
logind_signal_cb (GDBusProxy  *proxy,
                  const gchar *sender,
                  const gchar *signal,
                  GVariant    *parameters,
                  AppletNotifications2     *self)
{
    if (g_strcmp0 (signal, "PrepareForSleep") != 0)
        return;
    gboolean is_about_to_suspend;
    g_variant_get (parameters, "(b)", &is_about_to_suspend);
    if (is_about_to_suspend)
        applet_notifications2_hide (self);
}

static void
applet_notifications2_ui_init (AppletNotifications2 *self)
{
    ClutterActor *actor;
    self->actor = (ClutterActor *)st_box_layout_new ();
    st_box_layout_set_vertical ((StBoxLayout *)self->actor, TRUE);
    actor = self->actor;
    st_widget_add_style_class_name ((StWidget *)self->actor, "notification-applet");
    clutter_actor_hide (actor);
    CdosLayoutManager *layout = cdos_layout_manager_get_default ();
    cdos_layout_manager_add_chrome (layout, actor, 4);
    clutter_actor_set_reactive (actor, TRUE);

    MetaDisplay *display = cdos_global_get_display (cdos_global_get ()) ;
    g_signal_connect (display, "workareas-changed", G_CALLBACK (on_workarea_changed), self);
    on_workarea_changed (display, self);

    guint8 opacity = clutter_actor_get_opacity (actor);
    self->orig_opacity = opacity;

    /*
    ClutterEffect *blur = meta_blur_effect_new (5);
    clutter_actor_add_effect (actor, blur);
    */

    self->calendar = g_object_new (ST_TYPE_WIDGET,
                                   "x-expand", TRUE,
                                   NULL);
    st_widget_add_style_class_name (ST_WIDGET (self->calendar), "notification-calendar");
    ClutterLayoutManager *box_layout = clutter_box_layout_new ();
    clutter_box_layout_set_orientation (CLUTTER_BOX_LAYOUT (box_layout), CLUTTER_ORIENTATION_VERTICAL);
    clutter_actor_set_layout_manager (self->calendar, box_layout);

    // test time label
    self->timelabel = (StLabel *)st_label_new ("");
    self->lunarlabel = (StLabel *)st_label_new ("");
    st_widget_add_style_class_name ((StWidget *)self->timelabel,"notification-time-label");
    st_widget_add_style_class_name ((StWidget *)self->lunarlabel,"notification-lunar-label");
    g_object_set (self->timelabel, "x-expand", TRUE, "x-align", CLUTTER_ACTOR_ALIGN_CENTER, NULL);
    g_object_set (self->lunarlabel, "x-expand", TRUE, "x-align", CLUTTER_ACTOR_ALIGN_CENTER, NULL);
    clutter_actor_add_child (self->calendar, (ClutterActor *)self->timelabel);
    clutter_actor_add_child (self->calendar, (ClutterActor *)self->lunarlabel);
    clutter_actor_add_child (actor, self->calendar);
    //clutter_actor_show (calendar_box);
    clutter_actor_set_reactive (self->calendar, TRUE);
    g_signal_connect (self->calendar, "button-press-event", G_CALLBACK (on_calendar_release), NULL);
    notification_update_calendar (self);

    self->scroll_view = g_object_new (ST_TYPE_SCROLL_VIEW,
                                      "x-fill", TRUE,
                                      "y-fill", FALSE,
                                      "y-align", ST_ALIGN_START,
                                      NULL);
    g_object_set (self->scroll_view, "style-class", "vfade", NULL);

    clutter_actor_add_child (actor, CLUTTER_ACTOR(self->scroll_view));
    clutter_actor_set_y_expand ((ClutterActor *)self->scroll_view, TRUE);
    clutter_actor_set_y_align ((ClutterActor *)self->scroll_view, CLUTTER_ACTOR_ALIGN_FILL);

    self->box = (ClutterActor *)st_box_layout_new  ();
    st_box_layout_set_vertical ((StBoxLayout *)self->box, TRUE);
    st_scroll_view_add_child ((StScrollView *)self->scroll_view, (ClutterActor *)self->box);
    st_scroll_view_set_policy (ST_SCROLL_VIEW (self->scroll_view), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);

    self->clear_all = (StButton *)st_button_new_with_label (_("Clear All"));
    clutter_actor_add_child (actor, (ClutterActor *)self->clear_all);
    g_signal_connect (self->clear_all, "clicked", G_CALLBACK (clear_all_notifications), self);
    clutter_actor_set_x_align ((ClutterActor *)self->clear_all, CLUTTER_ACTOR_ALIGN_END);
    clutter_actor_set_name ((ClutterActor *)self->clear_all, "clear");
    clutter_actor_hide ((ClutterActor *)self->clear_all);

    applet_mpris2_listen (applet_notifications2_add_player, self);

    if (!self->priv->logind_proxy)
    {
        GError *error = NULL;
        self->priv->logind_proxy = g_dbus_proxy_new_for_bus_sync (G_BUS_TYPE_SYSTEM,
                                                                  0,
                                                                  NULL,
                                                                  SYSTEMD_DBUS_NAME,
                                                                  SYSTEMD_DBUS_PATH,
                                                                  SYSTEMD_DBUS_INTERFACE,
                                                                  NULL,
                                                                  &error);
        if (error)
        {
            g_warning ("%s", error->message);
            g_error_free (error);
        }
    }
    if (self->priv->logind_proxy)
    {
        g_signal_connect (self->priv->logind_proxy, "g-signal",
                          G_CALLBACK (logind_signal_cb), self);
    }
}

static void
applet_notifications2_settings_center_init (AppletNotifications2 *self)
{
    self->settings_center_box = g_object_new (ST_TYPE_BOX_LAYOUT,
                                              "style-class", "settings-center-box",
                                              "vertical", FALSE,
                                              NULL);
    clutter_actor_add_child (self->actor, CLUTTER_ACTOR (self->settings_center_box));
    /*设置中心在通知底部显示*/
    clutter_actor_set_x_expand ((ClutterActor *)self->settings_center_box, TRUE);
    clutter_actor_set_y_align ((ClutterActor *)self->settings_center_box, CLUTTER_ACTOR_ALIGN_END);
    ClutterLayoutManager *box_layout = clutter_actor_get_layout_manager ((ClutterActor *)self->settings_center_box);
    clutter_box_layout_set_homogeneous ((ClutterBoxLayout *)box_layout, TRUE);

    self->airplane_button = applet_airplane_mode_button_new (self, TRUE, TRUE);
    _add_actor (self->settings_center_box, self->airplane_button);

    self->quiet_button = applet_quiet_hours_button_new (self, TRUE, TRUE);
    _add_actor (self->settings_center_box, self->quiet_button);

    self->luminance_button = applet_luminance_button_new (self, TRUE, FALSE);
    _add_actor (self->settings_center_box, self->luminance_button);

    self->project_button = applet_project_button_new (self, FALSE, FALSE);
    _add_actor (self->settings_center_box, self->project_button);

    self->settings_button = applet_all_settings_button_new (self, FALSE, FALSE);
    _add_actor (self->settings_center_box, self->settings_button);
    self->timer_id = 0;                                               
}

static void
applet_notifications2_update_icon (AppletNotifications2 *self)
{
    if (self->notification_unshown > 0)
    {
        gchar *text = self->notification_unshown > APPLET_NOTIFICATIONS2_MAX_COUNT - 1? 
                                    g_strdup_printf ("%d+", self->notification_unshown - 1) :
                                    g_strdup_printf ("%d", self->notification_unshown) ;
        cdos_iconapplet_set_badge ((CdosIconApplet *)self, text);
        g_free (text);
    }
    else
    {
        cdos_iconapplet_set_badge ((CdosIconApplet *)self, NULL);
    }
}

static void
applet_notifications2_update_list (AppletNotifications2 *self)
{
    if (self->notifications)
    {
        clutter_actor_show ((ClutterActor *)self->clear_all);
        clutter_actor_show ((ClutterActor *)self->scroll_view);
    }    
    else
    {
        clutter_actor_hide ((ClutterActor *)self->clear_all);
    }

    self->notification_unshown = 0;
    CdosMessageTray* messageTray = cdos_message_tray_get_default ();
    if (!messageTray->dnd)
    {
        for (GList *it = self->notifications; it; it = it->next)
        {
            CdosNotification *notification = it->data;
            if (notification->unshown)
                self->notification_unshown++;
        }
    }
    applet_notifications2_update_icon (self);
}

static void
applet_notifications2_item_destroy_cb (CdosNotification *notification,
                                       AppletNotifications2 *self)
{
    self->notifications = g_list_remove (self->notifications, notification);
    applet_notifications2_update_list (self);
}

static gchar *
strdown (const gchar *src)
{
    if (!src)
        return NULL;

    gchar *dst = g_strdup (src);
    for (gchar *ptr = dst; *ptr != '\0'; ptr++)
    {
        if (*ptr >= 'A' && *ptr <= 'Z')
            *ptr = *ptr - ('A' - 'a');
    }
    return dst;
}

static void
applet_notifications2_notification_added_cb (CdosMessageTray *message_tray,
                                             CdosNotification *notification,
                                             AppletNotifications2 *self)
{
    ClutterActor *child = CLUTTER_ACTOR (notification);
    ClutterActor *parent = clutter_actor_get_parent (child);
    g_assert (parent != NULL);
    g_assert (parent != (gpointer)self);

    if (self->ignore_transient_notifications && notification->is_transient)
    {
        clutter_actor_destroy ((ClutterActor *)notification);
        return;
    }

    if (!self->actor)
    {
        applet_notifications2_ui_init (self);
        applet_notifications2_settings_center_init (self);
    }
    if (self->notification_count == APPLET_NOTIFICATIONS2_MAX_COUNT)
    {
        GList *first = g_list_last (self->notifications );
        cdos_notification_destroy (CDOS_NOTIFICATION (first->data), 0);
        self->notification_count--;
    }

    CdosSource *source = notification->source;

    g_autofree gchar *source_name = strdown (cdos_source_get_app_name (source));
    if (source_name)
    {
        for (GList *iter = self->players; iter; iter = iter->next)
        {
            AppletPlayer *player = iter->data;
            const gchar *player_name = applet_player_get_name (player);
            g_autofree gchar *player_name1 = strdown (player_name);
            g_debug ("%s %s %s", player_name1, source_name, notification->category);
            if (g_strrstr (player_name1, source_name)
                    && g_strcmp0 (notification->category, "x-gnome.music") == 0)
            {
                clutter_actor_destroy ((ClutterActor *)notification);
                return;
            }
        }
    }

    CdosSource *s = NULL;
    for (GList *l = self->notifications; l; l = l->next)
    {
        CdosNotification *n = l->data;

        if (n->source == source)
        {
            s = source;
            break;
        }

    }
    if (s == NULL)
        clutter_actor_insert_child_at_index (self->box, (ClutterActor *)source, 0);

    g_object_ref (notification);//引用计数加1，防止被释放
    clutter_actor_remove_child (parent, child);
    /*
     * 设置引用。否则notification引用数会 +1,导致无法完全释放内存
     */
    g_object_force_floating ((GObject *)notification);

    notification->in_notification_center = TRUE;
    clutter_actor_set_x_expand (child, TRUE);
    clutter_actor_set_x_align (child, CLUTTER_ACTOR_ALIGN_FILL);

    clutter_actor_insert_child_above (self->box, child, (ClutterActor *)source);
    clutter_actor_show (CLUTTER_ACTOR (notification->time_label));

    notification->in_notification_center = TRUE;

    self->notifications = g_list_prepend (self->notifications, notification);
    self->notification_count += 1;
    g_signal_connect (notification, "destroy", G_CALLBACK (applet_notifications2_item_destroy_cb), self);

    applet_notifications2_update_list (self);
}

static void
ungrab (AppletNotifications2 *self)
{
    cdos_pop_modal ((ClutterActor *)self, 0);
    clutter_actor_grab_key_focus (self->prev_focus);
    g_signal_handler_disconnect (cdos_global_get_stage (global), self->event_captured_id);
}

static void
applet_notifications2_update_timestamp (AppletNotifications2 *self)
{
    GList *iter = NULL;
    for (iter = self->notifications; iter; iter = iter->next)
    {
        CdosNotification *notification = iter->data;
        ClutterActor *clutter_text;
        clutter_text = st_label_get_clutter_text (notification->time_label);
        gchar *date = cdos_notification_timeify (notification, NULL);
        clutter_text_set_markup ((ClutterText *)clutter_text, date);
        g_free (date);
    }
}

static gboolean
on_captured_event (ClutterStage *stage,
                   ClutterEvent *event,
                   AppletNotifications2 *self)
{
    ClutterEventType type = clutter_event_type (event);
    if (type == CLUTTER_KEY_PRESS &&
        clutter_event_get_key_symbol (event) == CLUTTER_KEY_Escape)
    {
        applet_notifications2_hide (self);
        return TRUE;
    }
    if (type == CLUTTER_BUTTON_PRESS)
    {
        if (!clutter_actor_contains (self->actor, clutter_event_get_source (event)))
        {
            applet_notifications2_hide (self);
            return TRUE;
        }
    }
    if (type == CLUTTER_KEY_PRESS && 
            (clutter_event_get_key_symbol (event) == CLUTTER_KEY_MonBrightnessUp || 
             clutter_event_get_key_symbol (event) == CLUTTER_KEY_MonBrightnessDown))
    {
        g_debug ("in CLUTTER_KEY_MonBrightnessUp or CLUTTER_KEY_MonBrightnessDown");
        //applet_notifications2_hide (self);
        //通知面板打开时，若按下亮度调节热键，显示当前亮度值。直接获取亮度值会比实际亮度值滞后一档，需要延迟后再获取。
        return FALSE;
    }
    return FALSE;
}

static void
on_transition_completed (ClutterActor *actor, AppletNotifications2 *self)
{
    if (self->transition_state == 0)
    {
        clutter_actor_hide (actor);
        menu_stack_depth--;
    }
}

static void
on_transition_stopped (ClutterTimeline *trans, gboolean finished, ClutterActor *actor)
{
    if (finished)
        return;
    clutter_actor_remove_all_transitions (actor);
}

static void
on_allocation_changed (ClutterActor *actor,
                       ClutterActorBox *box,
                       ClutterAllocationFlags flags,
                       AppletNotifications2 *self)
{
    gfloat x;
    gfloat width;
    x = box->x1;
    width = box->x2 - box->x1;
    clutter_actor_set_clip (actor,
                            0, 0,
                            width - (x - self->pos_x),
                            box->y2 - box->y1);
}

void
applet_notifications2_show (AppletNotifications2 *self)
{
    menu_stack_depth++;

    ClutterActor *actor = self->actor;
    clutter_actor_show (actor);
    applet_notifications2_update_timestamp (self);

    ClutterStage *stage = cdos_global_get_stage (global);
    ClutterActor *focus = clutter_stage_get_key_focus (stage);
    self->prev_focus = focus;
    cdos_push_modal ((ClutterActor *)self, 0, 0, CDOS_ACTION_MODE_POPUP);
    clutter_stage_set_key_focus (stage, actor);
    self->event_captured_id = g_signal_connect (stage, "captured-event", G_CALLBACK (on_captured_event), self);

    float x;
    x = self->pos_x;
    ClutterTransition *trans;
    trans = cdos_transition_simple (actor,
                                    CLUTTER_EASE_OUT,
                                    200,
                                    "x", x,
                                    "opacity", self->orig_opacity,
                                    NULL);
    self->transition_state = 1;
    g_signal_connect (actor, "transitions-completed", G_CALLBACK (on_transition_completed), self);
    g_signal_connect (actor, "allocation-changed", G_CALLBACK (on_allocation_changed), self);
    g_signal_connect (trans, "stopped", G_CALLBACK (on_transition_stopped), actor);

    notification_update_calendar (self);
    CdosMessageTray* messageTray = cdos_message_tray_get_default ();
    cdos_message_tray_set_dnd (messageTray, TRUE);
    
    g_signal_emit (self, applet_notifications2_signals[NOTIFICATIONS_SHOW], 0);
}

void
applet_notifications2_hide (AppletNotifications2 *self)
{
    if (self->transition_state == 0)
        return;

    ClutterActor *actor = self->actor;

    float x, width;
    width = clutter_actor_get_width (actor);
    x = self->pos_x + width;
    ClutterTransition *trans;
    trans = cdos_transition_simple (actor,
                                    CLUTTER_EASE_IN,
                                    200,
                                    "x", x,
                                    "opacity", 0,
                                    NULL);
    self->transition_state = 0;
    g_signal_connect (actor, "transitions-completed", G_CALLBACK (on_transition_completed), self);
    g_signal_connect (trans, "stopped", G_CALLBACK (on_transition_stopped), actor);
    ungrab (self);

    CdosMessageTray* messageTray = cdos_message_tray_get_default ();
    cdos_message_tray_set_dnd (messageTray, FALSE);
    
    g_signal_emit (self, applet_notifications2_signals[NOTIFICATIONS_HIDE], 0); 
}

static gboolean
on_button_release (ClutterActor *source,
                   ClutterEvent *event,
                   AppletNotifications2 *self)
{
    
    if (clutter_event_get_button (event) != 1)
        return FALSE;
   
    if (!self->actor)
    {
        applet_notifications2_ui_init (self);
        applet_notifications2_settings_center_init (self);
    }

    self->notification_unshown = 0;
    for (GList *it = self->notifications; it; it = it->next)
    {
        CdosNotification *notification = it->data;
        notification->unshown = FALSE;
    }
    applet_notifications2_update_icon (self);

    if (clutter_actor_is_visible (self->actor))
        applet_notifications2_hide (self);
    else
        applet_notifications2_show (self);
    return TRUE;
}

static gboolean
on_calendar_release (StButton *button, gint clicked_button, gpointer data)
{
    cdos_util_spawn_command_line ("found-control-center calendar");
    return TRUE;
}

static void
applet_notifications2_dispose (GObject *gobject)
{
    AppletNotifications2 *self = APPLET_NOTIFICATIONS2 (gobject);

    CdosMessageTray *message_tray = cdos_message_tray_get_default ();
    g_signal_handlers_disconnect_by_data (message_tray, self);

    MetaDisplay *display = cdos_global_get_display (cdos_global_get ()) ;
    g_signal_handlers_disconnect_by_data (display, self);

    for (GList *iter = self->notifications; iter; iter = iter->next)
    {
        g_signal_handlers_disconnect_by_data (iter->data, self);
    }
    g_clear_object (&self->priv->logind_proxy);

    applet_mpris2_unlisten ();

    if (self->actor)
    {
        clutter_actor_destroy (self->actor);
        self->actor = NULL;
    }

    G_OBJECT_CLASS(applet_notifications2_parent_class)->dispose (gobject);
}

static void
applet_notifications2_finalize (GObject *gobject)
{
    AppletNotifications2 *self = APPLET_NOTIFICATIONS2 (gobject);

    g_list_free (self->notifications);

    G_OBJECT_CLASS(applet_notifications2_parent_class)->finalize (gobject);
}

static void
applet_notifications2_class_init (AppletNotifications2Class *klass)
{
    GObjectClass *gobject_class = G_OBJECT_CLASS(klass);
    gobject_class->dispose = applet_notifications2_dispose;
    gobject_class->finalize = applet_notifications2_finalize;

    //CdosAppletClass *parent_class = CDOS_APPLET_CLASS (klass);
    //parent_class->on_applet_clicked = applet_notifications2_on_applet_clicked_cb;
    
    /*signals*/
    applet_notifications2_signals[NOTIFICATIONS_SHOW] = g_signal_new ("notifications-show",
                                                      G_TYPE_FROM_CLASS (klass),
                                                      G_SIGNAL_RUN_LAST,
                                                      G_STRUCT_OFFSET (AppletNotifications2Class, notifications_show),
                                                      NULL,
                                                      NULL,
                                                      g_cclosure_marshal_VOID__VOID,
                                                      G_TYPE_NONE,
                                                      0);
                                                      
    applet_notifications2_signals[NOTIFICATIONS_HIDE] = g_signal_new ("notifications-hide",
                                                                      G_TYPE_FROM_CLASS (klass),
                                                                      G_SIGNAL_RUN_LAST,
                                                                      G_STRUCT_OFFSET (AppletNotifications2Class, notifications_hide),
                                                                      NULL,
                                                                      NULL,
                                                                      g_cclosure_marshal_VOID__VOID,
                                                                      G_TYPE_NONE,
                                                                      0);
}

static void
applet_notifications2_init (AppletNotifications2 *self)
{
    self->priv = applet_notifications2_get_instance_private (self);

    self->notification_count = 0;
    self->notification_unshown = 0;
    self->notifications = NULL;
    self->settings_center_box = NULL;

    CdosMessageTray *message_tray = cdos_message_tray_get_default ();
    g_signal_connect (message_tray, "notify-applet-update",
                      G_CALLBACK (applet_notifications2_notification_added_cb), self);
}

/**
 * applet_notifications2_main:
 * @metadata: applet的元数据
 * @orientation: 方向
 * @panel_height: 面板高度
 * @instanceId： 字符串类型的id
 *
 * 新建一个 #CdosApplet 类的实例。
 *
 * Returns: 返回一个 #CdosApplet 的实例。
 */
CdosApplet *
applet_notifications2_main (MetaData *metadata,
                            StSide orientation,
                            gfloat panel_height,
                            gchar *instanceId)
{
    AppletNotifications2 *self = g_object_new (APPLET_TYPE_NOTIFICATIONS2,
                                               "orientation", orientation,
                                               NULL);

    cdos_iconapplet_set_icon_name (CDOS_ICONAPPLET (self), "empty-notif");
    cdos_applet_set_applet_tooltip (CDOS_APPLET(self), _("Notifications"));

    self->notification_unshown = 0;
    //applet_notifications2_update_icon (self);
    g_signal_connect (self, "button-release-event",
                      G_CALLBACK (on_button_release), self);

    return CDOS_APPLET (self);
}

/*
static void
applet_notifications2_update_timestamp (AppletNotifications2 *self)
{
    const gchar *dateFormat = "%p %l:%M";
    
    if (self->notifications)
    {
        GList *iter = NULL;
        for (iter = self->notifications; iter; iter = iter->next)
        {
            CdosNotification *notification = iter->data;
            GDateTime* orig_time = notification->timestamp;
            ClutterActor *clutter_text;
            clutter_text = st_label_get_clutter_text (notification->time_label);
            gchar *date = applet_notifications2_timeify (orig_time, dateFormat);
            clutter_text_set_markup ((ClutterText *)clutter_text, date);
            g_free (date);
        }
    }
}
static void
applet_notifications2_on_applet_clicked_cb (ClutterActor* actor, ClutterEvent* event, gpointer user_data)
{
    if(clutter_event_get_button(event) != 1)
        return ;
    AppletNotifications2 *self = APPLET_NOTIFICATIONS2 (user_data);

    applet_notifications2_update_timestamp (self);
}

*/
