/**
 * Copyright (c) 2016-2018 CPU and Fundamental Software Research Center, CAS
 *
 * This software is published by the license of CPU-OS License，you can use and
 * distibuted this software under this License. See CPU-OS Liense for more detail.
 *
 * You should have receive a copy of CPU-OS License. If not, please contact us
 * by email <support_os@cpu-os.ac.cn>
 *
 **/
#include <gio/gio.h>
#include <stdlib.h>

#include <meta/display.h>
#include <meta/meta-monitor-manager.h>

#include "applet-project-button.h"
#include "applet-notifications2.h"
#include "config.h"
#include "cdos-util.h"
#include "cdos-monitor-switcher.h"

/*屏幕投射按钮*/

struct _AppletProjectButtonPrivate
{
    gulong monitors_changed_id;
};

G_DEFINE_TYPE_WITH_PRIVATE (AppletProjectButton, applet_project_button, APPLET_TYPE_SETTINGS_BUTTON);

static void
on_open_changed_callback (GObject *gobject,
                          GParamSpec *pspec,
                          gpointer user_data)
{
    applet_settings_button_open_state_show (APPLET_SETTINGS_BUTTON (gobject));
}

static void
applet_project_button_activate (AppletSettingsButton *button)
{
    cdos_monitor_switcher_get_default ();
}

static void
on_monitors_changed (MetaMonitorManager *monitor_manager, AppletProjectButton *self)
{
    /*默认扩展模式，若已经连接多个显示器，则需将屏幕投影的图标更新为扩展模式图标*/
    gboolean can_switch = meta_monitor_manager_can_switch_config (monitor_manager);

    if (can_switch)
        applet_settings_button_set_icon_name (APPLET_SETTINGS_BUTTON (self), "notifications-extended");
    else
    {
        applet_settings_button_set_icon_name (APPLET_SETTINGS_BUTTON (self), "notifications-project");
        return;
    }

    MetaMonitorSwitchConfigType config_type = meta_monitor_manager_get_switch_config (monitor_manager);
    switch (config_type)
    {
        case META_MONITOR_SWITCH_CONFIG_ALL_MIRROR:
            applet_settings_button_set_icon_name (APPLET_SETTINGS_BUTTON (self), "notifications-mirror");
            break;
        case META_MONITOR_SWITCH_CONFIG_ALL_LINEAR:
            applet_settings_button_set_icon_name (APPLET_SETTINGS_BUTTON (self), "notifications-extended");
            break;
        case META_MONITOR_SWITCH_CONFIG_EXTERNAL:
            applet_settings_button_set_icon_name (APPLET_SETTINGS_BUTTON (self), "notifications-docked");
            break;
        case META_MONITOR_SWITCH_CONFIG_BUILTIN:
            applet_settings_button_set_icon_name (APPLET_SETTINGS_BUTTON (self), "notifications-internal");
            break;
        case META_MONITOR_SWITCH_CONFIG_UNKNOWN:
            applet_settings_button_set_icon_name (APPLET_SETTINGS_BUTTON (self), "notifications-project");
            break;
    }
}

static void
applet_project_button_dispose (GObject *object)
{
    AppletProjectButton *self = APPLET_PROJECT_BUTTON (object);

    MetaMonitorManager *monitor_manager = meta_monitor_manager_get ();
    g_clear_signal_handler (&self->priv->monitors_changed_id, monitor_manager);

    G_OBJECT_CLASS (applet_project_button_parent_class)->dispose (object);
}

static void
applet_project_button_init (AppletProjectButton *self)
{
    AppletProjectButtonPrivate *priv;
    priv = self->priv = applet_project_button_get_instance_private (self);
    MetaMonitorManager *monitor_manager = meta_monitor_manager_get ();
    priv->monitors_changed_id = g_signal_connect (monitor_manager,
                                                  "monitors-changed",
                                                  G_CALLBACK (on_monitors_changed),
                                                  self);
}

static void
applet_project_button_class_init (AppletProjectButtonClass *klass)
{
    GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
    gobject_class->dispose = applet_project_button_dispose;

    AppletSettingsButtonClass *parent_class = APPLET_SETTINGS_BUTTON_CLASS (klass);
    parent_class->activate = applet_project_button_activate;
}

AppletProjectButton *
applet_project_button_new (AppletNotifications2 *notification,
                           gboolean keep_notification,
                           gboolean show_open_state)
{
    AppletProjectButton *project_button;

    project_button = g_object_new (APPLET_TYPE_PROJECT_BUTTON,
                                    "icon-name", "notifications-project",
                                    "icon-size", 24,
                                    "text", _("Project"),
                                    NULL);

    applet_settings_button_set_notification (APPLET_SETTINGS_BUTTON (project_button), notification);
    applet_settings_button_keep_notification (APPLET_SETTINGS_BUTTON (project_button), keep_notification);
    applet_settings_button_show_open_state (APPLET_SETTINGS_BUTTON (project_button), show_open_state);

    g_signal_connect (project_button,
                      "notify::open",
                      G_CALLBACK (on_open_changed_callback),
                      NULL);

    return project_button;
}
