/**
 * Copyright (c) 2016-2018 CPU and Fundamental Software Research Center, CAS
 *
 * This software is published by the license of CPU-OS License，you can use and
 * distibuted this software under this License. See CPU-OS Liense for more detail.
 *
 * You should have receive a copy of CPU-OS License. If not, please contact us
 * by email <support_os@cpu-os.ac.cn>
 *
 **/
#include "applet-quiet-hours-button.h"
#include "applet-notifications2.h"
#include "config.h"

/*免打扰按钮*/

struct _AppletQuietHoursButtonPrivate
{
    GSettings *settings;
    //CdosMessageTray *message_tray;
    gboolean is_open;
};

G_DEFINE_TYPE_WITH_PRIVATE (AppletQuietHoursButton, applet_quiet_hours_button, APPLET_TYPE_SETTINGS_BUTTON);

static void
applet_quiet_hours_button_activate (AppletSettingsButton *button)
{
    AppletQuietHoursButton *quiet_button = APPLET_QUIET_HOURS_BUTTON (button);
    AppletQuietHoursButtonPrivate *priv;

    priv = quiet_button->priv;
    if (!priv->is_open)
    {
        //dnd use
        /*
           cdos_message_tray_set_dnd (priv->message_tray, TRUE);
           applet_settings_button_set_open (APPLET_SETTINGS_BUTTON (quiet_button),
           TRUE);
           */
        //GSettings use
        g_settings_set_boolean (priv->settings, "display-notifications", FALSE);

        priv->is_open = TRUE;
    }
    else
    {
        //dnd use
        /*
           cdos_message_tray_set_dnd (priv->message_tray, FALSE);
           applet_settings_button_set_open (APPLET_SETTINGS_BUTTON (quiet_button),
           FALSE);
           */
        //GSettings use
        g_settings_set_boolean (priv->settings, "display-notifications", TRUE);

        priv->is_open = FALSE;
    }
}

static void
on_open_changed_callback (GObject *gobject,
                          GParamSpec *pspec,
                          gpointer user_data)
{
    applet_settings_button_open_state_show (APPLET_SETTINGS_BUTTON (gobject));
}

static void
display_notifications_changed_cb (GSettings *settings,
                                  gchar *key,
                                  gpointer user_data)
{
    gboolean value;
    AppletQuietHoursButton *quiet_button = (AppletQuietHoursButton *)user_data;
    AppletQuietHoursButtonPrivate *priv = quiet_button->priv;

    value = g_settings_get_boolean (priv->settings, "display-notifications");

    applet_settings_button_set_open (APPLET_SETTINGS_BUTTON (quiet_button),
                                            !value);
}

static void
applet_quiet_hours_button_dispose (GObject *object)
{
    AppletQuietHoursButton *self = APPLET_QUIET_HOURS_BUTTON (object);
    g_clear_object ((GObject **)&self->priv->settings);

    G_OBJECT_CLASS (applet_quiet_hours_button_parent_class)->dispose (object);
}

static void
applet_quiet_hours_button_init (AppletQuietHoursButton *self)
{
    AppletQuietHoursButtonPrivate *priv;

    priv = self->priv = applet_quiet_hours_button_get_instance_private (self);

    priv->settings = g_settings_new ("org.cdos");
    //dnd use
    //priv->message_tray = cdos_message_tray_get_default ();
    //GSettings use
    priv->is_open = !g_settings_get_boolean (priv->settings, "display-notifications");
    //dnd use
    //priv->is_open = FALSE;
    //cdos_message_tray_set_dnd (priv->message_tray, priv->is_open);

    g_signal_connect (priv->settings,
                      "changed::display-notifications",
                      G_CALLBACK (display_notifications_changed_cb),
                      self);
}

static void
applet_quiet_hours_button_class_init (AppletQuietHoursButtonClass *klass)
{
    GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
    gobject_class->dispose = applet_quiet_hours_button_dispose;

    AppletSettingsButtonClass *parent_class = APPLET_SETTINGS_BUTTON_CLASS (klass);
    parent_class->activate = applet_quiet_hours_button_activate;
}

AppletQuietHoursButton *
applet_quiet_hours_button_new (AppletNotifications2 *notification,
                               gboolean keep_notification,
                               gboolean show_open_state)
{
    AppletQuietHoursButton *quiet_button;

    quiet_button = g_object_new (APPLET_TYPE_QUIET_HOURS_BUTTON,
                                    "icon-name", "notifications-quiet-hours",
                                    "icon-size", 24,
                                    "text", _("Quiet Hours"),
                                    NULL);

    applet_settings_button_set_notification (APPLET_SETTINGS_BUTTON (quiet_button), notification);
    applet_settings_button_keep_notification (APPLET_SETTINGS_BUTTON (quiet_button), keep_notification);
    applet_settings_button_show_open_state (APPLET_SETTINGS_BUTTON (quiet_button), show_open_state);

    g_signal_connect (quiet_button,
                      "notify::open",
                      G_CALLBACK (on_open_changed_callback),
                      NULL);

    /**/
    //GSettings use
    applet_settings_button_set_open (APPLET_SETTINGS_BUTTON (quiet_button),
                                            quiet_button->priv->is_open);
    /**/

    return quiet_button;
}
