#include "cdos-start.h"
#include "cdos-global.h"
#include "cdos-util.h"
#include "applet-drive-menu-item.h"

/**
 * SECTION: applet-drive-menu-item.c
 * @Title: CdosDriveMenuItem
 * @short_description: description
 * 
 * CdosDriveMenuItem类是可移动磁盘里的一个项,当我们为某个可移动磁盘创建item时，可以使用这个类
 */

struct _CdosDriveMenuItemPrivate
{
    ClutterActor *label;
    GMount *mount;
    MountAction action;
};


G_DEFINE_TYPE_WITH_PRIVATE (CdosDriveMenuItem, cdos_drive_menu_item, CDOS_TYPE_POPUP_BASE_MENU_ITEM);

/*
static void
cdos_drive_menu_item_eject(StButton *button,  gint click, CdosDriveMenuItem *item);
*/

static void
cdos_drive_menu_item_eject(CdosDriveMenuItem *self);
static void 
cdos_drive_menu_item_open (CdosDriveMenuItem *self);

static CdosDriveMenuItem *
cdos_drive_menu_item_set_params(CdosDriveMenuItem   *item, 
                                GMount *mount,
                                MountAction action);

static void 
cdos_drive_menu_item_class_init(CdosDriveMenuItemClass *klass)
{
}

static void
cdos_drive_menu_item_init (CdosDriveMenuItem *item)
{
    CdosDriveMenuItemPrivate *priv;
    priv = item->priv = cdos_drive_menu_item_get_instance_private (item);
    priv->label = NULL;
}

/**
 * cdos_drive_menu_item_new:
 * @place: param1_description 
 * 
 * 新建一个 #CdosDriveMenuItem 类的实例。
 *
 * Returns: 返回一个 #CdosDriveMenuItem 的实例。
 */
CdosDriveMenuItem *
cdos_drive_menu_item_new (GMount *mount, MountAction action)
{
    CdosDriveMenuItem *item;
    item = g_object_new (CDOS_TYPE_DRIVE_MENU_ITEM, NULL);
    cdos_drive_menu_item_set_params (item, mount, action);
    return item;
}

static void
on_button_clicked (StButton *button, 
                   gint click, 
                   CdosDriveMenuItem *self)
{
    CdosDriveMenuItemPrivate *priv;
    priv = self->priv;
    switch (priv->action)
    {
    case ACTION_EJECT:
        cdos_drive_menu_item_eject (self);
        break;
    case ACTION_OPEN:
        cdos_drive_menu_item_open (self);
        break;
    default:
        break;
    }
}

static void
on_activate (CdosDriveMenuItem *self, 
             gpointer event, 
             gboolean keep, 
             gpointer data)
{
    guint event_type = clutter_event_get_button((ClutterEvent *)event);
    CdosDriveMenuItemPrivate *priv;
    priv = self->priv;
    if(event_type == 1)
    {
        switch (priv->action)
        {
        case ACTION_EJECT:
            cdos_drive_menu_item_eject (self);
            break;
        case ACTION_OPEN:
            cdos_drive_menu_item_open (self);
            break;
        default:
            break;
        }
    }
}

static CdosDriveMenuItem *
cdos_drive_menu_item_set_params (CdosDriveMenuItem   *item, 
                                 GMount  *mount,
                                 MountAction action)
{
    char *name, *label;
    CdosDriveMenuItemPrivate *priv;
    priv = item->priv;

    PopupBaseMenuItemInit init;
    cdos_popup_base_menu_item_get_init_params(&init);
    cdos_popup_base_menu_item_set_params((CdosPopupBaseMenuItem *)item, &init);

    priv->action = action;
    priv->mount = mount;
    name = g_mount_get_name (mount);
    GFile *sys_file = g_mount_get_default_location (mount);
    gchar *mount_uri = g_file_get_uri (sys_file);
    GFile *root_file = g_file_new_for_uri (mount_uri);
    GFileInfo *sys_file_size = g_file_query_filesystem_info (root_file, "filesystem::*", NULL, NULL);
    guint64 sys_size_int = g_file_info_get_attribute_uint64 (sys_file_size, G_FILE_ATTRIBUTE_FILESYSTEM_SIZE);
    gchar *label_size = g_format_size (sys_size_int);
    StWidget *ejectIcon = st_icon_new ();
    switch (action)
    {
    case ACTION_EJECT:
        label = g_strconcat (_("Eject"), " ", name, NULL);
        g_object_set (ejectIcon, 
                       "icon-name", "found-media-eject",
                       "style-class", "drive-menu-item-eject-icon",
                       "icon-size", 16,
                       NULL);
        break;

    case ACTION_OPEN:
        label = g_strconcat (_("Open"), " ", name, " (", label_size, ")",NULL);
        g_object_set (ejectIcon, 
                       "icon-name", "found-media-open",
                       "style-class", "drive-menu-item-eject-icon",
                       "icon-size", 16,
                       NULL);
        break;

    default:
        break;
    }
    g_free (name);
    g_object_unref (sys_file);
    g_free (mount_uri);
    g_object_unref (root_file);
    g_object_unref (sys_file_size);
    priv->label = (ClutterActor *)st_label_new (label);
    st_widget_add_style_class_name (ST_WIDGET (priv->label), "drive-menu-item-label");
    g_free (label);

    ActorChild child;
    cdos_popup_base_menu_item_get_actor_child(&child);

    StWidget *ejectButton = st_button_new ();

    st_bin_set_child ((StBin *)ejectButton, (ClutterActor *)ejectIcon);

    //可移动磁盘优化：点击菜单项直接弹出或卸载，取消点击卷标名称打开功能，仍保留打开功能代码
    // g_signal_connect (ejectButton, "clicked",
    //                   G_CALLBACK(on_button_clicked), item);

    child.align = ST_ALIGN_START;

    cdos_popup_base_menu_item_add_actor(CDOS_POPUP_BASE_MENU_ITEM(item), (ClutterActor *)ejectButton, &child);
    child.expand = TRUE;
    cdos_popup_base_menu_item_add_actor(CDOS_POPUP_BASE_MENU_ITEM(item), priv->label, &child);

    //g_signal_connect(item, "activate", G_CALLBACK(cdos_drive_menu_item_activate), item);

    g_signal_connect (item, "activate", G_CALLBACK(on_activate), NULL);

    return item;
}

static void
unmount_finish(GMount *mount,
               GAsyncResult *res,
               CdosDriveMenuItem *self)
{
    GError *error = NULL;
    gboolean status = g_mount_unmount_with_operation_finish (mount, res, &error);
    if (!status)
    {
        if (error)
        {
            char *message;
            char *name = g_mount_get_name (mount);

            // char *msg = g_strconcat ("unmount error: ", name, NULL);
            gchar *msg = g_strdup_printf (_("Unmount error: %s"), name);
            gchar *err_msg = strstr (error->message, "exited with non-zero exit status 32: umount: ");
            if (err_msg)
            {
                glong err_msg_len = g_utf8_strlen(err_msg, -1);
                // length 45 is 'exited with non-zero exit status 32: umount: '
                message = g_strdup_printf (_("Try unmount %s"), g_utf8_substring(err_msg, 45, err_msg_len));
            }
            else
            {
                message = g_strdup_printf (_("Try unmount %s failed."), name);
            }

            g_free (name);
            cdos_notify_error (msg, message);
            g_free (msg);
            g_free (message);
            g_error_free (error);
        }
    }
}

static void
eject_finish(GMount *mount,
             GAsyncResult *res,
             CdosDriveMenuItem *self)
{
    GError *error = NULL;
    g_mount_eject_with_operation_finish (mount, res, &error);
    if (error)
    {
        char *message;
        char *name = g_mount_get_name (mount);

        // char *msg = g_strconcat ("unmount error: ", name, NULL);
        gchar *msg = g_strdup_printf (_("Unmount error: %s"), name);
        gchar *err_msg = strstr (error->message, "exited with non-zero exit status 32: umount: ");
        if (err_msg)
        {
            glong err_msg_len = g_utf8_strlen(err_msg, -1);
            // length 45 is 'exited with non-zero exit status 32: umount: '
            message = g_strdup_printf (_("Try unmount %s"), g_utf8_substring(err_msg, 45, err_msg_len));
        }
        else
        {
            message = g_strdup_printf (_("Try unmount %s failed."), name);
        }

        g_free (name);
        cdos_notify_error (msg, message);
        g_free (message);
        g_free (msg);
        g_error_free (error);
    }
}

/*
static void
cdos_drive_menu_item_eject(StButton *button, gint click, CdosDriveMenuItem *self)
{
    CdosDriveMenuItemPrivate *priv = self->priv;
    if (g_mount_can_eject(priv->mount)) 
        g_mount_eject_with_operation (priv->mount, 
                                      G_MOUNT_UNMOUNT_NONE,
                                      NULL, 
                                      NULL,
                                      (GAsyncReadyCallback)eject_finish, self);
    else 
        g_mount_unmount_with_operation (priv->mount,
                                        G_MOUNT_UNMOUNT_NONE,
                                        NULL, 
                                        NULL,
                                        (GAsyncReadyCallback )unmount_finish, self);
}
*/

static void
cdos_drive_menu_item_eject (CdosDriveMenuItem *self)
{
    CdosDriveMenuItemPrivate *priv = self->priv;
    if (g_mount_can_eject(priv->mount)) 
        g_mount_eject_with_operation (priv->mount, 
                                      G_MOUNT_UNMOUNT_NONE,
                                      NULL, 
                                      NULL,
                                      (GAsyncReadyCallback)eject_finish, self);
    else 
        g_mount_unmount_with_operation (priv->mount,
                                        G_MOUNT_UNMOUNT_NONE,
                                        NULL, 
                                        NULL,
                                        (GAsyncReadyCallback )unmount_finish, self);
}

static void
cdos_drive_menu_item_open (CdosDriveMenuItem *self)
{
    CdosDriveMenuItemPrivate *priv = self->priv;
    GFile *mount_root = g_mount_get_root (priv->mount);
    char *file_uri = g_file_get_uri (mount_root);
    const gchar *file_path = g_strconcat ("nemo ", file_uri, NULL);
    cdos_util_spawn_command_line (file_path);
    g_object_unref (mount_root);
    g_free (file_uri);
}

static void G_GNUC_UNUSED
cdos_drive_menu_item_activate (CdosDriveMenuItem *item,
                               gpointer event,
                               gboolean keep,
                               gpointer data)
{
    CdosDriveMenuItemPrivate *priv;
    priv = item->priv;

    guint event_type = clutter_event_get_button((ClutterEvent *)event);

    if(event_type == 1)
    {
        CdosGlobal *global = cdos_global_get ();
        GAppLaunchContext *context = cdos_global_create_app_launch_context (global, 0, -1);
        GFile *root = g_mount_get_root (priv->mount);
        gchar *uri = g_file_get_uri (root);
        g_object_unref (root);
        GError *error = NULL;
        gboolean ret = g_app_info_launch_default_for_uri (uri, context, &error);
        g_free (uri);
        if (!ret)
        {
            if (error)
                g_warning ("%s", error->message);
        }
    }
}

GMount *
cdos_drive_menu_item_get_mount (CdosDriveMenuItem *self)
{
    return self->priv->mount;
}
