#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "applet-removable-drives.h"
#include "cdos-popup-menu-manager.h"
#include "cdos-popup-menu-base.h"
#include "applet-drive-menu-item.h"
#include "cdos-popup-menu-section.h"
#include "cdos-app-system.h"
#include "cdos-popup-separator-menu-item.h"


typedef enum
{
    IS_NOTHING,
    IS_MOUNT_ADD,
    IS_MOUNT_REMOVE,
} MonitorMountState;

struct _AppletRemovableDrivesPrivate
{
    CdosPopupMenu   *menu;
    StBoxLayout     *content_section;
    GVolumeMonitor  *monitor;
    gulong           add_id;
    gulong           remove_id;
    GList           *items;

    MonitorMountState monitormount_state;
    guint   monitor_on_mount_id;
    char    *mount_name;
};

G_DEFINE_TYPE_WITH_PRIVATE (AppletRemovableDrives, applet_removable_drives, CDOS_TYPE_ICONAPPLET);

/*
static void
applet_removable_drives_add_item(AppletRemovableDrives *applet);
*/

static void
applet_removable_drives_update_visibility (AppletRemovableDrives *self)
{
    if (self->priv->items)
    {
        clutter_actor_show ((ClutterActor *)self->priv->content_section);
        clutter_actor_show ((ClutterActor *)self);
    }
    else
    {
        cdos_popup_menu_close ((CdosPopupMenuBase *)self->priv->menu, 0);
        clutter_actor_hide ((ClutterActor *)self);
    }
}

static gboolean
check_item_exists (AppletRemovableDrives *self, GMount *mount)
{
    AppletRemovableDrivesPrivate *priv = self->priv;
    GMount *tmp_mount = NULL;
    CdosDriveMenuItem *item = NULL;
    GList *l = priv->items;
    gboolean ret = FALSE;
    GFile *new_file = NULL;

    new_file = g_mount_get_root (mount);

    while (l)
    {
        g_autoptr(GFile) tmp_file = NULL;
        item = l->data;
        tmp_mount = cdos_drive_menu_item_get_mount (item);
        if (tmp_mount == mount)
        {
            ret = TRUE;
            break;
        }

        tmp_file = g_mount_get_root (tmp_mount);
        if (g_file_equal (tmp_file, new_file))
        {
            ret = TRUE;
            break;
        }
        l = l->next;
    }
    g_object_unref (new_file);
    return ret;
}

static gboolean
should_show (GMount *mount)
{
    GFile *root;
    char *mount_point;
    gboolean res = TRUE;
    
    root = g_mount_get_default_location (mount);
    if (root)
    {
        mount_point = g_file_get_path (root);

        if (g_strcmp0 (mount_point, "/home") == 0)
        {
            res = FALSE;
        }

        if (g_strcmp0 (mount_point, "/opt/backup-recover") == 0)
        {
            res = FALSE;
        }

        if (g_str_has_prefix (mount_point, "/media/"))
        {
            res = TRUE;
        }

        g_object_unref (root);
    }

    return res;
}

static void
chown_media_mntpoint (GMount *mount)
{
    GFile *root;

    root = g_mount_get_default_location (mount);
    if (root)
    {
        char *mount_point = g_file_get_path (root);
        if (g_str_has_prefix (mount_point, "/media/"))
        {
	    gchar *cmd = NULL;
	    cmd = g_strdup_printf("/usr/bin/pkexec /usr/bin/udisk_tools.sh %s %s", mount_point, g_get_user_name());
	    system(cmd);
	    g_free(cmd);
	}
        g_object_unref (root);
	g_free(mount_point);
    }
}

static gboolean
add_mount (AppletRemovableDrives *self,
           GMount   *mount)
{
    GVolume *volume = g_mount_get_volume (mount);
    if (check_item_exists (self, mount))
    {
        g_debug ("%s: Menu item exists, just return.", __func__);
        return FALSE;
    }

    chown_media_mntpoint (mount);

    if (!should_show (mount))
    {
        return FALSE;
    }

    if (volume != NULL )
    { // smb, ftp etc mount without volume    
        GDrive *drive = g_volume_get_drive (volume);

        g_debug ("%s: volume name: %s, mount %p", __func__, g_volume_get_name(volume), mount);

        if (!drive || (drive && g_drive_is_removable (drive)))
        {
            CdosDriveMenuItem *item_open = cdos_drive_menu_item_new (mount, ACTION_OPEN);
            CdosDriveMenuItem *item_eject = cdos_drive_menu_item_new (mount, ACTION_EJECT);
            g_object_set_data_full (item_open, 
                                    "item-eject", item_eject, 
                                    clutter_actor_destroy);
            AppletRemovableDrivesPrivate *priv = self->priv;
            priv->items = g_list_append (priv->items, item_open);
            clutter_actor_add_child ((ClutterActor *)priv->content_section, (ClutterActor *)item_open);
            clutter_actor_add_child ((ClutterActor *)priv->content_section, (ClutterActor *)item_eject);
            applet_removable_drives_update_visibility (self);
            return TRUE;
        } 
        else
        {
            g_debug ("%s: GDrive is not removable", __func__);
        }
    }
    return FALSE;
}

static gboolean
remove_mount (AppletRemovableDrives *self,
              GMount   *mount)
{
    AppletRemovableDrivesPrivate *priv = self->priv;
    for (GList *iter = priv->items; iter; iter = iter->next)
    {
        CdosDriveMenuItem *item = iter->data;
        if (cdos_drive_menu_item_get_mount (item) == mount)
        {
            priv->items = g_list_remove (priv->items, item);
            clutter_actor_destroy ((ClutterActor *)item);
            applet_removable_drives_update_visibility (self);
            return TRUE;
        }
    }
    return FALSE;
}

//可移动磁盘优化：去除打开文件管理器项，保留代码
/*
static void
applet_removable_drives_fun(ClutterEvent *event)
{
    CdosAppSystem *appSystem = cdos_app_system_get_default ();
    CdosApp *app = cdos_app_system_lookup_app(appSystem , "nemo.desktop");
    cdos_app_activate_full(app, -1, clutter_event_get_time(event));
}

static void
applet_removable_drives_add_item(AppletRemovableDrives *applet)
{
        AppletRemovableDrivesPrivate *priv = applet->priv;

        CdosPopupSeparatorMenuItem  *sep = cdos_popup_separator_menu_item_new();

        clutter_actor_add_child ((ClutterActor *)priv->content_section,
                (ClutterActor *)sep);

        CdosPopupMenuItem *item = cdos_popup_menu_base_add_action((CdosPopupMenuBase *)priv->menu,
                _("Open File Manager"),
                G_CALLBACK(applet_removable_drives_fun), NULL);
        cdos_popup_menu_item_set_label_align(item,ST_ALIGN_MIDDLE,0);
        StLabel *label = cdos_popup_menu_item_get_label(item);
        st_widget_add_style_class_name(ST_WIDGET(label),"popup-menu-item-label-link");
}
*/

static void
send_notify_message (gpointer user_data)
{
    gchar *message, *cmd;
    char *name = (gchar *)(user_data);
    message = g_strdup_printf (_("%s can be removed from the computer now"), name);
    cmd = g_strdup_printf ("notify-send -i %s \"%s\" \"%s\"", 
                               "found-removable-drives", 
                               _("Removed device safely"), 
                               message);

    cdos_util_spawn_command_line (cmd);
    g_free (message);
    g_free (cmd);
}

static gboolean
monitor_on_mount_idle_cb (AppletRemovableDrives *self)
{
    AppletRemovableDrivesPrivate *priv = self->priv;
    if (priv->monitormount_state == IS_MOUNT_ADD)
    {
        cdos_popup_menu_open (priv->menu, FALSE);
    }
    else if (priv->monitormount_state == IS_MOUNT_REMOVE)
    {
        send_notify_message (priv->mount_name);
    }
    priv->monitor_on_mount_id = 0;

    return FALSE;
}

static void
monitor_on_mount_added (GVolumeMonitor *monitor,
                        GMount   *mount,
                        AppletRemovableDrives *self)
{
    AppletRemovableDrivesPrivate *priv = self->priv;
    gboolean get_active = cdos_screen_saver_get_active ();
    if (add_mount (self, mount) && !get_active)
    {
        priv->monitormount_state = IS_MOUNT_ADD;
        g_free (priv->mount_name);
        priv->mount_name = NULL;

        if (priv->monitor_on_mount_id)
        {
            g_source_remove (priv->monitor_on_mount_id);
            priv->monitor_on_mount_id = 0;
        }
        priv->monitor_on_mount_id = g_timeout_add (1000, (GSourceFunc)monitor_on_mount_idle_cb, self);
    }
}

static void
monitor_on_mount_removed (GVolumeMonitor *monitor,
                          GMount   *mount,
                          AppletRemovableDrives *self)
{
    AppletRemovableDrivesPrivate *priv = self->priv;
    if (remove_mount (self, mount))
    {
        priv->monitormount_state = IS_MOUNT_REMOVE;
        g_free (priv->mount_name);
        priv->mount_name = g_mount_get_name (mount);

        if (priv->monitor_on_mount_id)
        {
            g_source_remove (priv->monitor_on_mount_id);
            priv->monitor_on_mount_id = 0;
        }
        priv->monitor_on_mount_id = g_timeout_add (1000, (GSourceFunc)monitor_on_mount_idle_cb, self);
    }
}

static void
applet_removable_drives_init (AppletRemovableDrives *applet)
{
    AppletRemovableDrivesPrivate *priv;
    priv = applet->priv = applet_removable_drives_get_instance_private (applet);

    priv->items = NULL;
    priv->monitormount_state = IS_NOTHING;
    cdos_applet_set_popup_menu_active (CDOS_APPLET (applet), TRUE);
    cdos_iconapplet_set_icon_name ((CdosIconApplet *)applet, "found-removable-drives");
    cdos_iconapplet_set_icon_size ((CdosIconApplet *)applet, 16);
    cdos_applet_set_applet_tooltip((CdosApplet *)applet, _("Removable drives"));

    priv->menu = cdos_applet_get_popup_menu(CDOS_APPLET(applet));

    priv->content_section = cdos_popup_menu_base_get_box((CdosPopupMenuBase *)priv->menu);

    priv->monitor = g_volume_monitor_get ();
    priv->add_id = g_signal_connect (priv->monitor, "mount-added", G_CALLBACK(monitor_on_mount_added), applet);
    priv->remove_id = g_signal_connect (priv->monitor, "mount-removed", G_CALLBACK(monitor_on_mount_removed), applet);
    GList *mounts = g_volume_monitor_get_mounts (priv->monitor);
    for (GList *iter = mounts; iter; iter = iter->next)
    {
        GMount *mount = iter->data;
        if (!g_mount_can_eject (mount) && !g_mount_can_unmount (mount))
            continue;
        if (g_mount_is_shadowed (mount))
            continue;
        GVolume *volume = g_mount_get_volume (mount);
        gboolean skip = 0;
        if (volume)
        {
            gchar *identifier = g_volume_get_identifier (volume, "class");
            if (identifier != NULL)
            {
                if (!g_strcmp0 (identifier, "network"))
                    skip = 1;
                g_free (identifier);
            }
        }
        if (!skip)
        {
            add_mount (applet, mount);
        }
    }
    g_list_free (mounts);

    /*
    applet_removable_drives_add_item (applet);
    */
    applet_removable_drives_update_visibility (applet);
}

static void
applet_removable_drives_finalize(GObject *gobject)
{
    AppletRemovableDrivesPrivate *priv = APPLET_REMOVABLE_DRIVES(gobject)->priv;

    g_signal_handler_disconnect (priv->monitor, priv->add_id);
    g_signal_handler_disconnect (priv->monitor, priv->remove_id);

    if (priv->monitor_on_mount_id)
    {
        g_source_remove (priv->monitor_on_mount_id);
        priv->monitor_on_mount_id = 0;
    }
    g_free (priv->mount_name);

    g_object_unref (priv->monitor);

    G_OBJECT_CLASS(applet_removable_drives_parent_class)->finalize(gobject);
}

static void
applet_removable_drives_class_init(AppletRemovableDrivesClass *klass)
{
    GObjectClass *gobject = G_OBJECT_CLASS(klass);
    gobject->finalize = applet_removable_drives_finalize;
}

static AppletRemovableDrives *
applet_removable_drives_new(guint orientation, gfloat panel_height)
{
    AppletRemovableDrives *applet;
    applet = g_object_new (APPLET_TYPE_REMOVABLE_DRIVES,
                           "orientation", orientation,
                           NULL);
    return applet;
}

CdosApplet *
applet_removable_drives_main(MetaData *meta, guint orientation, guint panel_height)
{
        AppletRemovableDrives *applet;
        applet = applet_removable_drives_new(orientation, panel_height);
        return CDOS_APPLET (applet);
}
