#include <gio/gio.h>

#include "cdos-icon-applet.h"
#include "cdos-overview.h"
#include "applets-scale.h"

struct _AppletScalePrivate
{
    GSettings      *settings;
    ClutterActor   *actor;
    gboolean        hover_activates;
};

G_DEFINE_TYPE_WITH_PRIVATE (AppletScale, applet_scale, CDOS_TYPE_ICONAPPLET);

static gboolean on_applet_clicked(ClutterActor *actor,ClutterEvent *event,AppletScale *self);

static void
cdos_set_hover_changed (AppletScale *self,gboolean active)
{
  AppletScalePrivate *priv;
  priv = self->priv;

    gboolean activeChanged;
    activeChanged = (active != priv->hover_activates);
    if (activeChanged)
    {
        priv->hover_activates = active;
        if (priv->hover_activates)
        {
            cdos_iconapplet_set_icon_name ((CdosIconApplet *)self, "found-workspaces-hover");
            // clutter_actor_grab_key_focus ((ClutterActor *)priv->actor);
        }
        else
        {
            cdos_iconapplet_set_icon_name ((CdosIconApplet *)self, "found-workspaces");
        }
    }
}

static void 
cdos_on_key_focus_in (ClutterActor *actor, AppletScale *self)
{
   cdos_set_hover_changed (self, TRUE);
}

static void
cdos_on_key_focus_out (ClutterActor *actor, AppletScale *self)
{
  cdos_set_hover_changed (self, FALSE);
}

static void
cdos_on_hover_changed (ClutterActor *actor, gpointer pspec, AppletScale *self)
{
   cdos_set_hover_changed (self, st_widget_get_hover ((StWidget *)actor));
}

static gboolean
on_applet_clicked (ClutterActor *actor, ClutterEvent *event, AppletScale *self)
{
    gint32 button = clutter_event_get_button (event);
    if (button == 1)
    {
        CdosOverview *overview = cdos_overview_get ();
        cdos_overview_toggle (overview);
    }
    return TRUE;
}

static void 
applet_scale_class_init( AppletScaleClass *kclass)
{
   //lutterActorClass *actor_class = CLUTTER_ACTOR_CLASS (kclass);
    //parent_class->on_applet_clicked = on_applet_clicked;
}

static void
applet_scale_init (AppletScale *self)
{
    self->priv = applet_scale_get_instance_private (self);
    cdos_applet_set_popup_menu_active (CDOS_APPLET (self),FALSE);
}


AppletScale *
applet_scale_new (MetaData *metadata, guint orientation, gfloat panel_height, gchar *instance_id)
{
    AppletScale *scale;
    CdosIconApplet *iconapplet;
    AppletScalePrivate *priv = NULL;
    scale = g_object_new (APPLET_TYPE_SCALE, "orientation", orientation, NULL);
    priv = applet_scale_get_instance_private (scale);

    iconapplet = (CdosIconApplet *)scale;
    priv->actor = (ClutterActor *)iconapplet;
    //st_widget_remove_style_class_name ((StWidget *)priv->actor,"applet-box");
    // priv->applet_icon_box = g_object_new (ST_TYPE_BIN, NULL);
    // iconapplet->applet_icon_box = cdos_iconapplet_get_applet_icon_box (iconapplet);
    // clutter_actor_set_margin_left ((ClutterActor *)iconapplet->applet_icon_box, 8.0);

    cdos_iconapplet_set_icon_name ((CdosIconApplet *)iconapplet, "found-workspaces");
    cdos_applet_set_applet_tooltip ((CdosApplet *)iconapplet, _("Workspaces and tasks"));
    priv->hover_activates =FALSE;
    g_signal_connect (scale, "button-press-event",G_CALLBACK(on_applet_clicked),scale);

    st_widget_add_style_class_name(ST_WIDGET(scale),"applet-box-scale");

    g_signal_connect (priv->actor, "key-focus-in", G_CALLBACK (cdos_on_key_focus_in), scale);
    g_signal_connect (priv->actor, "key-focus-out", G_CALLBACK (cdos_on_key_focus_out), scale);
    st_widget_set_track_hover ((StWidget *)priv->actor, 1);
    g_signal_connect (priv->actor, "notify::hover", G_CALLBACK (cdos_on_hover_changed), scale);

    return scale;
}
