#include <meta/meta-monitor-manager.h>
#include "applets-systray.h"
#include "cdos-start.h"
#include "cdos-tooltip.h"
#include "cdos-debug.h"
#include "cdos-icon-dispatcher.h"
#include "cdos-tray-icon.h"

GType applets_systray_get_type (void);

#define APPLETS_TYPE_SYSTRAY (applets_systray_get_type ())
#define APPLETS_SYSTRAY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), APPLETS_TYPE_SYSTRAY, AppletsSystray))
#define APPLETS_IS_SYSTRAY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), APPLETS_TYPE_SYSTRAY))
#define APPLETS_SYSTRAY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), APPLETS_TYPE_SYSTRAY, AppletsSystrayClass))
#define APPLETS_IS_SYSTRAY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), APPLETS_TYPE_SYSTRAY))
#define APPLETS_SYSTRAY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), APPLETS_TYPE_SYSTRAY, AppletsSystrayClass))

#define TRAY_ICON_SIZE_BASE 22

typedef struct _AppletsSystray AppletsSystray;
typedef struct _AppletsSystrayClass AppletsSystrayClass;

static gchar *placeholders[] = {"fcitx", "unregistered", NULL};

struct _AppletsSystray
{
    CdosApplet parent_instance;
    ClutterActor *manager_container;
    gulong signals_added;
    gulong signals_removed;
    gulong signals_redisplay;
    gulong signals_registered;

    GHashTable *role_position_table;
    MetaMonitorManager *monitor_manager;
    gulong monitor_changed_id;
    guint  redisplay_systray_id;
};

struct _AppletsSystrayClass
{
    CdosAppletClass parent_class;
};

G_DEFINE_TYPE (AppletsSystray, applets_systray, CDOS_TYPE_APPLET);


static void
applets_systray_insert_status_item (AppletsSystray *systray, ClutterActor *actor, gfloat position)
{
    int role_position;
    void *temp = NULL;
    int i = clutter_actor_get_n_children (systray->manager_container) - 1;
    ClutterActor *iter = clutter_actor_get_last_child (systray->manager_container);
    for (; i >= 0 && iter != NULL; i--, iter = clutter_actor_get_previous_sibling (iter))
    {
        if (g_hash_table_contains (systray->role_position_table, iter))
        {
            temp = g_hash_table_lookup (systray->role_position_table, iter);
            role_position = (int)(intptr_t)temp;
            if (position > role_position)
            {
                clutter_actor_insert_child_at_index (systray->manager_container, actor, i + 1);
                break;
            }
        }
    }

    if (i == -1)
    {
        /* If we didn't find a position, we must be first */
        clutter_actor_insert_child_at_index (systray->manager_container, actor, 0);
    }
    g_hash_table_insert (systray->role_position_table, actor, (void *)(intptr_t)position);
}

static void
applets_systray_on_tray_icon_removed (CdosIconDispatcher *manager, CdosTrayIcon *icon, gpointer user_data)
{
    AppletsSystray *systray = APPLETS_SYSTRAY (user_data);
    ClutterActor *item = clutter_actor_get_parent (CLUTTER_ACTOR (icon));
    const gchar *name = clutter_actor_get_name (item);
    gboolean has_placeholder = FALSE;

    for (int i = 0; placeholders[i] != NULL; i++)
    {
        if (name && g_strcmp0 (name, placeholders[i]) == 0)
        {
            has_placeholder = TRUE;
        }
    }
    if (has_placeholder)
    {
        g_signal_handlers_disconnect_by_data (item, icon);
        clutter_actor_hide (item);
    } else {
        clutter_actor_remove_child (systray->manager_container, item);
    }
    clutter_actor_destroy (CLUTTER_ACTOR (icon));
}

static const gchar *buggy_icons[] =
{
    "pidgin",
    "thunderbird"
};

static int
applets_systray_glist_indexof (const gchar *str)
{
    if (str)
    {
        int i;
        for (i = 0; i < G_N_ELEMENTS (buggy_icons); i++)
        {
            if (g_strcmp0 (buggy_icons[i], str) == 0)
            {
                return i;
            }
        }
    }
    return -1;
}

#if 0
static double
applets_systray_get_icon_size(AppletsSystray *systray) {

  StThemeNode *themeNode;
  double icon_size = 0;
  StWidget *st_icon = st_icon_new();
  clutter_actor_hide(st_icon);
  clutter_actor_add_child(systray->manager_container, st_icon);
  st_widget_set_style_class_name (st_icon, "applet-systray-icon");
  themeNode = st_widget_get_theme_node(st_icon);
  icon_size = st_theme_node_get_length(themeNode, "icon-size");
  clutter_actor_remove_child(systray->manager_container, st_icon);
  g_object_unref(st_icon);
  return icon_size;
}
#endif
static gboolean
on_leave (ClutterActor *actor, ClutterEvent *event, CdosTrayIcon *icon)
{
    cdos_tray_icon_leave (icon, event);
    return FALSE;
}

static gboolean
on_enter (ClutterActor *actor, ClutterEvent *event, CdosTrayIcon *icon)
{
    cdos_tray_icon_enter (icon, event);
    return FALSE;
}

static gboolean
on_button_release (ClutterActor *actor, ClutterEvent *event, CdosTrayIcon *icon)
{
    cdos_tray_icon_click (icon,event);
    return FALSE;
}

static void
create_placeholders (AppletsSystray *systray)
{
    StWidget *item;
    int i;
    for (i = 0; placeholders[i] != NULL; i++)
    {
        item = st_bin_new ();
        clutter_actor_set_name ((ClutterActor *)item, placeholders[i]);
        applets_systray_insert_status_item (systray, (ClutterActor *)item, -1);
        clutter_actor_hide (item);
    }
}

static ClutterActor *
find_placeholder (AppletsSystray *systray, const gchar *role)
{
    GList *children, *l;
    children = clutter_actor_get_children (systray->manager_container);
    l = children;
    ClutterActor *iter = NULL, *holder = NULL;
    while (l)
    {
        iter = l->data;
        const gchar *name = clutter_actor_get_name (iter);
        if (name && g_strcmp0 (role, name) == 0)
        {
            holder = iter;
            break;
        }
        l = l->next;
    }
    g_list_free (children);
    return holder;
}

static gboolean
applets_systray_on_tray_icon_added (CdosIconDispatcher *manager,
                                    CdosTrayIcon *icon,
                                    gchar *role,
                                    gpointer user_data)
{
    AppletsSystray *systray = APPLETS_SYSTRAY (user_data);
    guint ui_scale = cdos_global_get_ui_scale (cdos_global_get ());
    if (g_strcmp0 (role, "network") == 0)
    {
        return TRUE;
    }

    ClutterActor *parent = clutter_actor_get_parent (CLUTTER_ACTOR (icon));
    if (parent)
    {
        g_object_ref (icon);
        clutter_actor_remove_child (parent, CLUTTER_ACTOR (icon));
        g_object_force_floating (G_OBJECT (icon));
    }

    ClutterActor *item;
    item = find_placeholder (systray, role); 
    if (item == NULL)
    {
        item = st_bin_new ();
        applets_systray_insert_status_item (systray, item, -1);
    }

    st_widget_set_style_class_name (item, "applet-box-systray");
    clutter_actor_set_reactive ((ClutterActor *)item, TRUE);
    st_widget_set_track_hover (item, TRUE);

    st_bin_set_child ((StBin *)item, (ClutterActor *)icon);
    st_widget_set_can_focus ((StWidget *)item, TRUE);
    clutter_actor_show (item);

    g_signal_connect (item, "button-release-event", G_CALLBACK (on_button_release), icon);
    g_signal_connect (item, "enter-event", G_CALLBACK (on_enter), icon);
    g_signal_connect (item, "leave-event", G_CALLBACK (on_leave), icon);

    st_bin_set_fill ((StBin *)item, TRUE, TRUE);
    clutter_actor_set_y_expand (CLUTTER_ACTOR (item), TRUE);
    clutter_actor_set_x_align (CLUTTER_ACTOR (icon), CLUTTER_ACTOR_ALIGN_CENTER);
    clutter_actor_set_y_align (CLUTTER_ACTOR (icon), CLUTTER_ACTOR_ALIGN_CENTER);
    clutter_actor_set_pivot_point ((ClutterActor *)icon, 0.5f, 0.5f);

    CdosGlobal *global;
    global = cdos_global_get ();
    MetaDisplay *display = cdos_global_get_display (global);
    gfloat scale, max_scale = 1.0;
    gdouble s;
    gint n_monitors, i;
    MetaMonitorSwitchConfigType config = meta_monitor_manager_get_switch_config (systray->monitor_manager);
    n_monitors = meta_display_get_n_monitors (display);

    if (config == META_MONITOR_SWITCH_CONFIG_ALL_LINEAR)
    {
        //缩放是针对整体来进行的，而不是只看部分, 针对扩展模式下，所有显示器均参与计算。
        for (i = 0; i < n_monitors; i++)
        {
            scale = meta_display_get_monitor_scale (display, i);
            max_scale = MAX (max_scale, scale);
        }
        scale = max_scale;
    } else {
        scale = meta_display_get_monitor_scale (display, meta_display_get_current_monitor(display));
    }

    GSettings *settings = cdos_global_get_settings (global);

    gfloat nw, nh;
    clutter_actor_get_preferred_size (CLUTTER_ACTOR (icon), NULL, NULL, &nw, &nh);

    //图标缩放,以高度作为基准缩放,大图标缩小，小图标不变（防止模糊）
    s = (double)(TRAY_ICON_SIZE_BASE * ceil(scale) / nh);
    s = MIN (s, 1.0);

    g_message ("Systray: role = %s, scale = %.2f, s = %.2f, natural size = %.2f,%.2f", role, scale, s, nw, nh);

    if (scale > 1.f)
    {
        g_settings_set_boolean (settings, "panel-scale-text-icons", TRUE);
    }
    else if (scale <= 1.f && scale > 0)
    {
        g_settings_set_boolean (settings, "panel-scale-text-icons", FALSE);
    }
    clutter_actor_set_scale ((ClutterActor *)icon, s, s);

    /* dropbox, for some reason, refuses to provide a correct size icon in our new situation.
   * Tried even with stalonetray, same results - all systray icons I tested work fine but dropbox.  I'm
   * assuming for now it's their problem.  For us, just scale it up.
   */
    if (role && g_strcmp0 ("dropbox", role) == 0)
    {
        clutter_actor_set_scale ((ClutterActor *)icon, (gdouble)ui_scale, (gdouble)ui_scale);
    }
    return FALSE;
}

static void
applets_systray_on_before_redisplay (CdosIconDispatcher *manager, gpointer user_data)
{
    AppletsSystray *systray = APPLETS_SYSTRAY (user_data);
    GList *children = clutter_actor_get_children (systray->manager_container);
    GList *iter;
    for (iter = children; iter; iter = iter->next)
    {
        clutter_actor_destroy (iter->data);
    }
    g_list_free (children);
}

static void
applets_systray_changed_callback (CdosSystrayManager *systray_manager, gpointer user_data)
{
    CdosIconDispatcher *status_icon_dispatcher = CDOS_ICON_DISPATCHER (user_data);
    cdos_icon_dispatcher_redisplay (status_icon_dispatcher);
}

static gboolean
applets_systray_on_applet_added_to_panel (CdosApplet *applet, gboolean loaded)
{
    AppletsSystray *systray = APPLETS_SYSTRAY (applet);
    CdosIconDispatcher *status_icon_dispatcher = cdos_icon_dispatcher_get ();
    if (systray->signals_added == 0)
    {
        systray->signals_added = g_signal_connect (
            status_icon_dispatcher, "status-icon-added", G_CALLBACK (applets_systray_on_tray_icon_added), systray);
    }

    if (systray->signals_removed == 0)
    {
        systray->signals_removed = g_signal_connect (
            status_icon_dispatcher, "status-icon-removed", G_CALLBACK (applets_systray_on_tray_icon_removed), systray);
    }

    if (systray->signals_redisplay == 0)
    {
        systray->signals_redisplay = g_signal_connect (
            status_icon_dispatcher, "before-redisplay", G_CALLBACK (applets_systray_on_before_redisplay), systray);
    }

    if (systray->signals_registered == 0)
    {
        systray->signals_registered = g_signal_connect (cdos_ui_plugin_get_systray_manager (),
                                                        "changed",
                                                        G_CALLBACK (applets_systray_changed_callback),
                                                        status_icon_dispatcher);
    }
    return 1;
}

static gboolean
applets_systray_on_applet_removed_from_panel (CdosApplet *tray)
{
    AppletsSystray *systray = APPLETS_SYSTRAY (tray);
    CdosIconDispatcher *status_icon_dispatcher = cdos_icon_dispatcher_get ();

    if (systray->signals_added > 0)
    {
        g_signal_handler_disconnect (status_icon_dispatcher, systray->signals_added);
        systray->signals_added = 0;
    }

    if (systray->signals_removed > 0)
    {
        g_signal_handler_disconnect (status_icon_dispatcher, systray->signals_removed);
        systray->signals_removed = 0;
    }

    if (systray->signals_redisplay > 0)
    {
        g_signal_handler_disconnect (status_icon_dispatcher, systray->signals_redisplay);
        systray->signals_redisplay = 0;
    }

    CdosSystrayManager *systray_manager = cdos_ui_plugin_get_systray_manager ();
    if (systray->signals_registered > 0)
    {
        g_signal_handler_disconnect (systray_manager, systray->signals_registered);
        systray->signals_registered = 0;
    }
    return 1;
}

static gboolean
applets_systray_on_panel_height_changed (void)
{
    CdosIconDispatcher *status_icon_dispatcher = cdos_icon_dispatcher_get ();
    cdos_icon_dispatcher_redisplay (status_icon_dispatcher);
    return 0;
}

static gboolean
systray_redisplay_delay (AppletsSystray *self)
{
    CdosIconDispatcher *status_icon_dispatcher = cdos_icon_dispatcher_get ();
    self->redisplay_systray_id = 0;
    cdos_icon_dispatcher_redisplay (status_icon_dispatcher);
    return FALSE;
}

static void
systray_on_monitors_changed (MetaMonitorManager *monitor_manager, AppletsSystray *self)
{
    if (self->redisplay_systray_id > 0)
    {
        g_source_remove (self->redisplay_systray_id);
        self->redisplay_systray_id = 0;
    }
    self->redisplay_systray_id = \
        g_timeout_add (250, 
                       (GSourceFunc)systray_redisplay_delay, 
                       self);
}

static void
applets_systray_dispose (GObject *object)
{
    AppletsSystray *self = APPLETS_SYSTRAY (object);
    CdosIconDispatcher *status_icon_dispatcher = cdos_icon_dispatcher_get ();
    g_signal_handlers_disconnect_by_data (status_icon_dispatcher, object);
    if (self->redisplay_systray_id > 0)
    {
        g_source_remove (self->redisplay_systray_id);
        self->redisplay_systray_id = 0;
    }

    if (self->monitor_changed_id > 0)
    {
        g_signal_handler_disconnect (self->monitor_manager, self->monitor_changed_id);
        self->monitor_changed_id = 0;
    }

    G_OBJECT_CLASS (applets_systray_parent_class)->dispose (object);
}

static void
applets_systray_finalize (GObject *object)
{
    G_OBJECT_CLASS (applets_systray_parent_class)->finalize (object);
}

static void
applets_systray_init (AppletsSystray *self)
{
    st_widget_remove_style_class_name ((StWidget *)self, "applet-box");
    // st_widget_set_style ((StWidget *)CDOS_APPLET(self)->actor, "spacing: 5px");

    self->role_position_table = g_hash_table_new (NULL, NULL);

    self->signals_added = 0;
    self->signals_removed = 0;
    self->signals_redisplay = 0;
    self->signals_registered = 0;

    guint ui_scale = cdos_global_get_ui_scale (cdos_global_get ());
    ClutterLayoutManager *manager = clutter_box_layout_new ();
    clutter_box_layout_set_spacing ((ClutterBoxLayout *)manager, 2 * ui_scale);
    clutter_box_layout_set_homogeneous ((ClutterBoxLayout *)manager, TRUE);
    clutter_box_layout_set_orientation ((ClutterBoxLayout *)manager, CLUTTER_ORIENTATION_HORIZONTAL);

    ClutterActor *manager_container = clutter_actor_new ();
    clutter_actor_set_layout_manager (manager_container, manager);
    self->manager_container = manager_container;

    clutter_actor_add_child (CLUTTER_ACTOR (self), manager_container);
    clutter_actor_show (manager_container);

    cdos_applet_set_popup_menu_active (CDOS_APPLET (self), FALSE);
    create_placeholders (self);

    self->monitor_manager = meta_monitor_manager_get ();
    self->monitor_changed_id = \
        g_signal_connect_after (self->monitor_manager, 
                                "monitors-changed", 
                                G_CALLBACK (systray_on_monitors_changed), 
                                self);
}

static void
applets_systray_class_init (AppletsSystrayClass *klass)
{
    GObjectClass *object_class = G_OBJECT_CLASS (klass);
    CdosAppletClass *parent_class = CDOS_APPLET_CLASS (klass);

    object_class->dispose = applets_systray_dispose;
    object_class->finalize = applets_systray_finalize;

    // parent_class->on_applet_clicked = applets_systray_on_applet_clicked;
    parent_class->on_applet_added_to_panel = applets_systray_on_applet_added_to_panel;
    parent_class->on_applet_removed_from_panel = applets_systray_on_applet_removed_from_panel;
    parent_class->on_panel_height_changed = applets_systray_on_panel_height_changed;
}

static CdosApplet *
applets_systray_new (StSide orientation, gfloat panel_height)
{
    AppletsSystray *my_applet = g_object_new (
        APPLETS_TYPE_SYSTRAY, "orientation", orientation, NULL);
    return (CdosApplet *)my_applet;
}

CdosApplet *
applets_systray_main (MetaData *meta, StSide orientation, gfloat panel_height)
{
    CdosApplet *my_applet = applets_systray_new (orientation, panel_height);

    return my_applet;
}
