// Created by zhangtao on 16-3-15.

#include <math.h>
#include "applet-app-button.h"
#include "cdos-window-tracker.h"
#include "cdos-global.h"
#include "cdos-applet.h"

struct _AppletAppButtonPrivate
{
    MetaWorkspace *meta_workspace;
    CdosWindowTracker *tracker;
};

G_DEFINE_TYPE_WITH_PRIVATE (AppletAppButton, applet_app_button, APPLET_TYPE_ICON_LABEL_BUTTON);

static void tracker_on_notify_focus_app (GObject *tracker, GParamSpec *pspec, AppletAppButton *self);

static void
applet_app_button_dispose (GObject *object)
{
    AppletAppButton *self = APPLET_APP_BUTTON (object);
    AppletAppButtonPrivate *priv = self->priv;

    if (priv->tracker)
    {
        g_signal_handlers_disconnect_by_data (priv->tracker, self);
        priv->tracker = NULL;
    }

    G_OBJECT_CLASS (applet_app_button_parent_class)->dispose (object);
}

static void
applet_app_button_finalize (GObject *object)
{
    // AppletAppButton *self = APPLET_APP_BUTTON (object);
    // AppletAppButtonPrivate *priv = self->priv;

    G_OBJECT_CLASS (applet_app_button_parent_class)->finalize (object);
}

static void
applet_app_button_init (AppletAppButton *self)
{
    AppletAppButtonPrivate *priv;
    self->priv = applet_app_button_get_instance_private (self);
    priv = self->priv;

    priv->tracker = cdos_window_tracker_get_default ();

    g_signal_connect (priv->tracker, "notify::focus-app", G_CALLBACK (tracker_on_notify_focus_app), self);
    applet_icon_label_button_hide_label (APPLET_ICON_LABEL_BUTTON (self));
}

static void
applet_app_button_class_init (AppletAppButtonClass *klass)
{
    GObjectClass *gobject_class = G_OBJECT_CLASS (klass);

    gobject_class->dispose = applet_app_button_dispose;
    gobject_class->finalize = applet_app_button_finalize;
}

static void
tracker_on_notify_focus_app (GObject *tracker, GParamSpec *pspec, AppletAppButton *self)
{
    applet_app_button_update_actor_style (self);
}

AppletAppButton *
applet_app_button_new (CdosApp *app, gboolean is_favorite, MetaWorkspace *meta_workspace)
{
    AppletAppButton *self;
    self = g_object_new (APPLET_TYPE_APP_BUTTON,
                         "app", app,
                         "is-favorite", is_favorite,
                         "style-class", "window-list-app-button",
                         NULL);
    AppletAppButtonPrivate *priv = self->priv;
    priv->meta_workspace = meta_workspace;
    return self;
}

void
applet_app_button_update_actor_style (AppletAppButton *self)
{
    g_return_if_fail (APPLET_IS_APP_BUTTON (self));
    AppletAppButtonPrivate *priv = self->priv;
    if (!priv->meta_workspace)
    {
        return;
    }
    CdosApp *app;

    StWidget *actor = ST_WIDGET (self);
    app = applet_icon_label_button_get_app (APPLET_ICON_LABEL_BUTTON (self));
    GSList *windows = cdos_app_get_windows (app);
    gint win_num = 0;
    gint focus = FALSE;
    while (windows != NULL)
    {
        MetaWindow *meta_window = windows->data;
        MetaWorkspace *meta_workspace = meta_window_get_workspace (meta_window);
        MetaWindowType meta_window_type = meta_window_get_window_type (meta_window);
        if (meta_window_type == META_WINDOW_NORMAL && meta_workspace == priv->meta_workspace)
        {
            if (meta_window_appears_focused (meta_window) || meta_window_has_focus (meta_window))
            {
                focus = TRUE;
            }
            win_num++;
            if (win_num >= 3)
            {
                break;
            }
        }
        windows = windows->next;
    }

    st_widget_remove_style_class_name (actor, "window-list-item-box-one-active");
    st_widget_remove_style_class_name (actor, "window-list-item-box-two-active");
    st_widget_remove_style_class_name (actor, "window-list-item-box-more-active");

    if (focus)
    {
        st_widget_add_style_pseudo_class (actor, "focus");
        // notice:暂时无用；
        // st_widget_remove_style_class_name (actor, "window-list-item-demands-attention");
        // st_widget_remove_style_class_name (actor, "window-list-item-demands-attention-top");
        if (win_num == 1)
        {
            st_widget_add_style_class_name (actor, "window-list-item-box-one-active");
        }
        else if (win_num == 2)
        {
            st_widget_add_style_class_name (actor, "window-list-item-box-two-active");
        }
        else if (win_num >= 3)
        {
            st_widget_add_style_class_name (actor, "window-list-item-box-more-active");
        }
    }
    else
    {
        st_widget_remove_style_pseudo_class (actor, "focus");
    }
}
