// Created by zhangtao on 16-4-16.

#include "cdos-start.h"
#include "applet-app-context-menu.h"
#include "cdos-popup-menu-item.h"
#include "cdos-popup-separator-menu-item.h"
#include "cdos-global.h"
#include "cdos-utils-private.h"
#include "cdos-util.h"
#include "applet-window-button.h"
#include "applet-panel-launchers-manager.h"

typedef struct _AppletAppContextMenuSignalIds AppletAppContextMenuSignalIds;
typedef struct _AppletAppContextMenuItems AppletAppContextMenuItems;

struct _AppletAppContextMenuPrivate
{
    MetaWindow *meta_window;
    CdosApp *app;
    gboolean is_favorite_app;
    GList *windows;
    const gchar *app_id;
    AppletAppContextMenuItems *items;
    AppletAppContextMenuSignalIds *signal_ids;
    gboolean open_state;
    GSettings *global_settings;
    GPid force_quit_dialog_pid;
};

G_DEFINE_TYPE_WITH_PRIVATE (AppletAppContextMenu, applet_app_context_menu, CDOS_TYPE_POPUP_MENU);

struct _AppletAppContextMenuItems
{
    CdosPopupMenuItem *close_window;
    CdosPopupMenuItem *close_all_windows;
    CdosPopupMenuItem *make_above;
    CdosPopupMenuItem *minimize_window;
    CdosPopupMenuItem *maximize_window;
    CdosPopupMenuItem *move_to_left_workspace;
    CdosPopupMenuItem *move_to_right_workspace;
    CdosPopupMenuItem *on_all_workspaces;
    CdosPopupMenuItem *launch;
    CdosPopupMenuItem *move_window;
    CdosPopupMenuItem *force_quit;
    CdosPopupMenuItem *make_below;

    CdosPopupMenuItem *toggle_favorite;
    CdosPopupMenuItem *nemo_property;

    CdosPopupSeparatorMenuItem *separator;
    CdosPopupSeparatorMenuItem *separator1;

};

struct _AppletAppContextMenuSignalIds
{
    gulong menu_open_state_changed_id;
    gulong close_window_activate_id;
    gulong close_all_windows_activate_id;
    gulong make_above_activate_id;
    gulong minimize_window_activate_id;
    gulong maximize_window_activate_id;
    gulong move_to_left_workspace_activate_id;
    gulong move_to_right_workspace_activate_id;
    gulong on_all_workspaces_activate_id;
    gulong launch_activate_id;
    gulong move_window_activate_id;
    gulong force_quit_activate_id;
    gulong make_below_activate_id;

    gulong toggle_favorite_toggle_id;
    gulong nemo_property_id;

};

static void applet_app_context_menu_dispose (GObject *object);
static void applet_app_context_menu_finalize (GObject *object);

static void applet_app_context_menu_setup (AppletAppContextMenu *self);

static gboolean applet_app_context_menu_is_favorite_app (AppletAppContextMenu *self);

static void menu_on_open_state_changed (CdosPopupMenuBase *popup_menu_base, gboolean state, gpointer user_data);

static void applet_app_context_menu_refresh (AppletAppContextMenu *self);

static gboolean applet_app_context_menu_has_mata_window (AppletAppContextMenu *self);

static void applet_app_context_menu_item_on_close_window (CdosPopupBaseMenuItem *item,
                                                          ClutterEvent *event,
                                                          gboolean keep_menu,
                                                          AppletAppContextMenu *self);
static void applet_app_context_menu_item_on_close_all_windows (CdosPopupBaseMenuItem *item,
                                                               ClutterEvent *event,
                                                               gboolean keep_menu,
                                                               AppletAppContextMenu *self);
static void applet_app_context_menu_item_on_force_quit (CdosPopupBaseMenuItem *item,
                                                        ClutterEvent *event,
                                                        gboolean keep_menu,
                                                        AppletAppContextMenu *self);
static void applet_app_context_menu_item_on_make_above (CdosPopupBaseMenuItem *item,
                                                        ClutterEvent *event,
                                                        gboolean keep_menu,
                                                        AppletAppContextMenu *self);
static void applet_app_context_menu_item_on_make_below (CdosPopupBaseMenuItem *item,
                                                        ClutterEvent *event,
                                                        gboolean keep_menu,
                                                        AppletAppContextMenu *self);
static void applet_app_context_menu_item_on_minimize_window (CdosPopupBaseMenuItem *item,
                                                             ClutterEvent *event,
                                                             gboolean keep_menu,
                                                             AppletAppContextMenu *self);
static void applet_app_context_menu_item_on_maximize_window (CdosPopupBaseMenuItem *item,
                                                             ClutterEvent *event,
                                                             gboolean keep_menu,
                                                             AppletAppContextMenu *self);
static void applet_app_context_menu_item_on_move_to_left_workspace (CdosPopupBaseMenuItem *item,
                                                                    ClutterEvent *event,
                                                                    gboolean keep_menu,
                                                                    AppletAppContextMenu *self);
static void applet_app_context_menu_item_on_move_to_right_workspace (CdosPopupBaseMenuItem *item,
                                                                     ClutterEvent *event,
                                                                     gboolean keep_menu,
                                                                     AppletAppContextMenu *self);
static void applet_app_context_menu_item_on_on_all_workspaces (CdosPopupBaseMenuItem *item,
                                                               ClutterEvent *event,
                                                               gboolean keep_menu,
                                                               AppletAppContextMenu *self);
static void applet_app_context_menu_item_on_launch (CdosPopupBaseMenuItem *item,
                                                    ClutterEvent *event,
                                                    gboolean keep_menu,
                                                    AppletAppContextMenu *self);

static void applet_app_context_menu_item_on_toggle_favorite (CdosPopupBaseMenuItem *item,
                                                             ClutterEvent *event,
                                                             gboolean keep_menu,
                                                             AppletAppContextMenu *self);

static void applet_app_context_menu_item_on_nemo_property (CdosPopupBaseMenuItem *item,
                                                            ClutterEvent *event,
                                                            gboolean keep_menu,
                                                           AppletAppContextMenu *self);


static void applet_app_context_menu_add_items (AppletAppContextMenu *self);

/*
static void
panel_launchers_on_changed (GSettings *settings, gchar *key, gpointer user_data)
{
    AppletAppContextMenu *self = APPLET_APP_CONTEXT_MENU (user_data);
    AppletAppContextMenuPrivate *priv = self->priv;
    AppletAppContextMenuItems *items = priv->items;

    priv->is_favorite_app = applet_app_context_menu_is_favorite_app (self);
    if (priv->is_favorite_app)
    {
        cdos_popup_menu_item_set_label (items->toggle_favorite, _("Remove App"));
    }
    else
    {
        cdos_popup_menu_item_set_label (items->toggle_favorite, _("Add App"));
    }
}
*/

static void
applet_app_context_menu_init (AppletAppContextMenu *self)
{
    AppletAppContextMenuPrivate *priv;
    self->priv = applet_app_context_menu_get_instance_private (self);
    priv = self->priv;

    priv->signal_ids = g_new0 (AppletAppContextMenuSignalIds, 1);
    priv->items = g_new0 (AppletAppContextMenuItems, 1);

    priv->signal_ids->menu_open_state_changed_id =
        g_signal_connect (self, "open-state-changed", G_CALLBACK (menu_on_open_state_changed), self);
    priv->meta_window = NULL;

    priv->global_settings = cdos_get_cdos_settings ();

    CdosBoxPointer *bp = cdos_popup_menu_get_box_pointer (self);
    st_widget_add_style_class_name ((StWidget *)bp, "window-list-context-menu");

    /*
    g_signal_connect (priv->global_settings, "changed::panel-launchers", G_CALLBACK (panel_launchers_on_changed), self);
    */
}

static void
applet_app_context_menu_class_init (AppletAppContextMenuClass *klass)
{
    GObjectClass *gobject_class = G_OBJECT_CLASS (klass);

    gobject_class->dispose = applet_app_context_menu_dispose;
    gobject_class->finalize = applet_app_context_menu_finalize;
}

static void
applet_app_context_menu_dispose (GObject *object)
{
    AppletAppContextMenu *self = APPLET_APP_CONTEXT_MENU (object);
    AppletAppContextMenuPrivate *priv = self->priv;
    if (CDOS_POPUP_MENU_BASE (self)->is_open)
        cdos_popup_menu_close ((CdosPopupMenuBase *)self, TRUE);

    if (priv->global_settings)
    {
        g_signal_handlers_disconnect_by_data (priv->global_settings, self);
        priv->global_settings = NULL;
    }
    // applet_app_context_menu_items_disconnect_all (self);

    G_OBJECT_CLASS (applet_app_context_menu_parent_class)->dispose (object);
}

static void
applet_app_context_menu_finalize (GObject *object)
{
    AppletAppContextMenu *self = APPLET_APP_CONTEXT_MENU (object);
    AppletAppContextMenuPrivate *priv = self->priv;

    if (priv->windows != NULL)
    {
        g_list_free (priv->windows);
    }
    g_free (priv->signal_ids);
    g_free (priv->items);
    G_OBJECT_CLASS (applet_app_context_menu_parent_class)->finalize (object);
}

static gboolean
applet_app_context_menu_is_favorite_app (AppletAppContextMenu *self)
{
    AppletAppContextMenuPrivate *priv = self->priv;
    if (!priv->app)
    {
        return FALSE;
    }

    const gchar *app_id = cdos_app_get_id (priv->app);
    gchar **desktop_files = g_settings_get_strv (priv->global_settings, "panel-launchers");
    int i;
    for (i = 0; desktop_files[i] != NULL; i++)
    {
        if (g_strcmp0 (app_id, desktop_files[i]) == 0)
        {
            return TRUE;
        }
    }
    g_strfreev (desktop_files);
    return FALSE;
}

static void
applet_app_context_menu_add_items (AppletAppContextMenu *self)
{
    AppletAppContextMenuPrivate *priv = self->priv;

    AppletAppContextMenuItems *items = priv->items;
    CdosPopupMenuBase *menu = CDOS_POPUP_MENU_BASE (self);

    cdos_popup_menu_base_add_menu_item (menu, CDOS_POPUP_BASE_MENU_ITEM (items->launch), -1);
    cdos_popup_menu_base_add_menu_item (menu, CDOS_POPUP_BASE_MENU_ITEM (items->toggle_favorite), -1);

    cdos_popup_menu_base_add_menu_item (menu, CDOS_POPUP_BASE_MENU_ITEM (items->separator), -1);
    // cdos_popup_menu_base_add_menu_item (menu, CDOS_POPUP_BASE_MENU_ITEM (items->on_all_workspaces), -1);
    cdos_popup_menu_base_add_menu_item (menu, CDOS_POPUP_BASE_MENU_ITEM (items->move_to_left_workspace), -1);
    cdos_popup_menu_base_add_menu_item (menu, CDOS_POPUP_BASE_MENU_ITEM (items->move_to_right_workspace), -1);

    cdos_popup_menu_base_add_menu_item (menu, CDOS_POPUP_BASE_MENU_ITEM (items->separator1), -1);

    cdos_popup_menu_base_add_menu_item (menu, CDOS_POPUP_BASE_MENU_ITEM (items->make_above), -1);
    cdos_popup_menu_base_add_menu_item (menu, CDOS_POPUP_BASE_MENU_ITEM (items->make_below), -1);
    cdos_popup_menu_base_add_menu_item (menu, CDOS_POPUP_BASE_MENU_ITEM (items->minimize_window), -1);
    cdos_popup_menu_base_add_menu_item (menu, CDOS_POPUP_BASE_MENU_ITEM (items->maximize_window), -1);
    // cdos_popup_menu_base_add_menu_item (menu, CDOS_POPUP_BASE_MENU_ITEM (items->move_window), -1);
    cdos_popup_menu_base_add_menu_item (menu, CDOS_POPUP_BASE_MENU_ITEM (items->nemo_property), -1);
    cdos_popup_menu_base_add_menu_item (menu, CDOS_POPUP_BASE_MENU_ITEM (items->force_quit), -1);
    cdos_popup_menu_base_add_menu_item (menu, CDOS_POPUP_BASE_MENU_ITEM (items->close_window), -1);
    cdos_popup_menu_base_add_menu_item (menu, CDOS_POPUP_BASE_MENU_ITEM (items->close_all_windows), -1);
}

static void
menu_on_open_state_changed (CdosPopupMenuBase *popup_menu, gboolean state, gpointer user_data)
{
    AppletAppContextMenu *self = APPLET_APP_CONTEXT_MENU (user_data);
    AppletAppContextMenuPrivate *priv = self->priv;
    AppletAppContextMenuItems *items = priv->items;

    if (!state)
        return;

    gboolean has_window = applet_app_context_menu_has_mata_window (self);
    if (priv->meta_window == NULL || !has_window)
    {
        cdos_popup_base_menu_item_set_sensitive ((CdosPopupBaseMenuItem *)items->launch, TRUE);
        return;
    }

    CdosBoxPointer *box_pointer = cdos_popup_menu_get_box_pointer ((CdosPopupMenu *)self);
    gfloat arrow = cdos_popup_menu_get_arrow_alignment ((CdosPopupMenu *)self);
    ClutterActor *source_actor = cdos_popup_menu_base_get_source_actor ((CdosPopupMenuBase *)self);
    cdos_box_pointer_set_position (box_pointer, source_actor, arrow);

    priv->open_state = state;
    ClutterActor *actor_left = (ClutterActor *)items->move_to_left_workspace;
    ClutterActor *actor_right = (ClutterActor *)items->move_to_right_workspace;

    if (meta_window_is_on_all_workspaces (priv->meta_window))
    {
        // cdos_popup_menu_item_set_label (items->on_all_workspaces, _("Only on this workspace"));
        clutter_actor_hide (actor_left);
        clutter_actor_hide (actor_right);
    }
    else if (priv->meta_window != NULL)
    {
        // cdos_popup_menu_item_set_label (items->on_all_workspaces, _("Visible on all workspaces"));
        MetaWorkspace *meta_workspace = meta_window_get_workspace (priv->meta_window);
        MetaWorkspace *neighbor_left = meta_workspace_get_neighbor (meta_workspace, META_MOTION_LEFT);
        MetaWorkspace *neighbor_right = meta_workspace_get_neighbor (meta_workspace, META_MOTION_RIGHT);

        if (meta_workspace != neighbor_left)
        {
            clutter_actor_show (actor_left);
        }
        else
        {
            clutter_actor_hide (actor_left);
        }
        if (meta_workspace != neighbor_right)
        {
            clutter_actor_show (actor_right);
        }
        else
        {
            clutter_actor_hide (actor_right);
        }
    }

    if (meta_window_get_maximized (priv->meta_window))
    {
        cdos_popup_menu_item_set_label (items->maximize_window, _("Unmaximize"));
    }

    if (priv->meta_window != NULL)
    {
        if (meta_window_get_maximized (priv->meta_window))
        {
            cdos_popup_menu_item_set_label (items->maximize_window, _("Unmaximize"));
        }
        else
        {
            cdos_popup_menu_item_set_label (items->maximize_window, _("Maximize"));
        }
        gboolean minimized, resizeable, is_above, is_below;
        g_object_get (priv->meta_window, "minimized", &minimized, NULL);
        g_object_get (priv->meta_window, "resizeable", &resizeable, NULL);
        is_above = meta_window_is_above (priv->meta_window);
        is_below = meta_window_is_below (priv->meta_window);

        if (minimized)
        {
            cdos_popup_menu_item_set_label (items->minimize_window, _("Restore"));
            cdos_popup_base_menu_item_set_sensitive ((CdosPopupBaseMenuItem *)items->maximize_window, FALSE);
            cdos_popup_base_menu_item_set_sensitive ((CdosPopupBaseMenuItem *)items->move_window, FALSE);
        }
        else
        {
            cdos_popup_menu_item_set_label (items->minimize_window, _("Minimize"));
            cdos_popup_base_menu_item_set_sensitive ((CdosPopupBaseMenuItem *)items->maximize_window, TRUE);
            cdos_popup_base_menu_item_set_sensitive ((CdosPopupBaseMenuItem *)items->move_window, TRUE);
        }
        if (resizeable == FALSE)
        {
            cdos_popup_base_menu_item_set_sensitive ((CdosPopupBaseMenuItem *)items->maximize_window, FALSE);
        }
        if (is_above)
            cdos_popup_menu_item_set_label (items->make_above, _("Unmake above"));
        else
            cdos_popup_menu_item_set_label (items->make_above, _("Make above"));

        if (is_below)
            cdos_popup_menu_item_set_label (items->make_below, _("Unmake below"));
        else
            cdos_popup_menu_item_set_label (items->make_below, _("Make below"));
    }
}

static void
applet_app_context_menu_item_on_close_window (CdosPopupBaseMenuItem *item,
                                              ClutterEvent *event,
                                              gboolean keep_menu,
                                              AppletAppContextMenu *self)
{
    AppletAppContextMenuPrivate *priv = self->priv;
    guint32 current_time = cdos_global_get_current_time (global);
    if (priv->meta_window)
    {
        meta_window_delete (priv->meta_window, current_time);
    }
}

static void
applet_app_context_menu_item_on_close_all_windows (CdosPopupBaseMenuItem *item,
                                                   ClutterEvent *event,
                                                   gboolean keep_menu,
                                                   AppletAppContextMenu *self)
{
    AppletAppContextMenuPrivate *priv = self->priv;
    GList *windows = priv->windows;
    while (windows)
    {
        guint32 current_time = cdos_global_get_current_time (global);
        // 此处如果传入参数如果不正确，会造成 mutter CRITICAL 崩溃
        // 发生的场景是设置中心的弹出窗口在关闭时，没有通知到窗口管理器
        if (META_IS_WINDOW (windows->data)) {
            meta_window_delete (windows->data, current_time);
        }
        windows = windows->next;
    }

    g_list_free (priv->windows);
    priv->windows = NULL;
}

static void
applet_app_context_menu_item_on_dialog (GPid pid, gint status, gpointer user_data)
{
    AppletAppContextMenu *self = (AppletAppContextMenu *)user_data;
    AppletAppContextMenuPrivate *priv = self->priv;
    gboolean ok;
    ok = g_spawn_check_exit_status (status, NULL);
    if (ok)
    {
        GSList *app_id = cdos_app_get_pids (priv->app);
        for (GSList *iter = app_id; iter; iter = iter->next)
        {
            int app_pid = GPOINTER_TO_INT (iter->data);
            char *kill_pid = g_strdup_printf ("kill -9 %d", app_pid);
            cdos_util_spawn_command_line (kill_pid);
            g_free (kill_pid);
        }
    }
    priv->force_quit_dialog_pid = 0;
}

static void
applet_app_context_menu_item_on_force_quit (CdosPopupBaseMenuItem *item,
                                            ClutterEvent *event,
                                            gboolean keep_menu,
                                            AppletAppContextMenu *self)
{
    AppletAppContextMenuPrivate *priv = self->priv;
    if (priv->force_quit_dialog_pid)
        return;

    GPid pid;
    pid = meta_show_dialog ("--question",
                            _("Forcing to quit will kill all processes and windows in the application\n Possibly losing unsaved working data, whether to continue?"),
                            "-1",
                            NULL,
                            _("Yes"),
                            _("No"),
                            NULL,
                            0,
                            NULL,
                            NULL);
    priv->force_quit_dialog_pid = pid;
    g_child_watch_add (pid, applet_app_context_menu_item_on_dialog, self);
}

static void
applet_app_context_menu_item_on_make_above (CdosPopupBaseMenuItem *item,
                                            ClutterEvent *event,
                                            gboolean keep_menu,
                                            AppletAppContextMenu *self)
{
    AppletAppContextMenuPrivate *priv = self->priv;
    MetaWindow *focus_window;
    MetaDisplay *display;
    gboolean is_above = meta_window_is_above (priv->meta_window);
    gboolean is_below = meta_window_is_below (priv->meta_window);

    if (is_above)
        meta_window_unmake_above (priv->meta_window);
    else
    {
        if (is_below)
            meta_window_unmake_below (priv->meta_window);

        meta_window_make_above (priv->meta_window);
    }

    display = cdos_global_get_display (cdos_global_get ());
    focus_window = meta_display_get_focus_window (display);
    if (focus_window)
    {
       meta_window_raise (focus_window);
    }
}
static void
applet_app_context_menu_item_on_make_below (CdosPopupBaseMenuItem *item,
                                            ClutterEvent *event,
                                            gboolean keep_menu,
                                            AppletAppContextMenu *self)
{
    AppletAppContextMenuPrivate *priv = self->priv;
    MetaWindow *focus_window;
    MetaDisplay *display;
    gboolean is_above = meta_window_is_above (priv->meta_window);
    gboolean is_below = meta_window_is_below (priv->meta_window);

    if (is_below)
        meta_window_unmake_below (priv->meta_window);
    else
    {
        if (is_above)
            meta_window_unmake_above (priv->meta_window);

        meta_window_make_below (priv->meta_window);
    }
    display = cdos_global_get_display (cdos_global_get ());
    focus_window = meta_display_get_focus_window (display);
    if (focus_window)
        meta_window_raise (focus_window);
}

static void
applet_app_context_menu_item_on_minimize_window (CdosPopupBaseMenuItem *item,
                                                 ClutterEvent *event,
                                                 gboolean keep_menu,
                                                 AppletAppContextMenu *self)
{
    AppletAppContextMenuPrivate *priv = self->priv;
    g_return_if_fail (priv->meta_window != NULL);
    guint32 current_time = cdos_global_get_current_time (global);
    gboolean minimized;
    g_object_get (priv->meta_window, "minimized", &minimized, NULL);
    if (minimized)
    {
        meta_window_unminimize (priv->meta_window);
        meta_window_activate (priv->meta_window, current_time);
    }
    else
    {
        meta_window_minimize (priv->meta_window);
    }
}

static void
applet_app_context_menu_item_on_maximize_window (CdosPopupBaseMenuItem *item,
                                                 ClutterEvent *event,
                                                 gboolean keep_menu,
                                                 AppletAppContextMenu *self)
{
    AppletAppContextMenuPrivate *priv = self->priv;
    if (meta_window_get_maximized (priv->meta_window))
    {
        meta_window_unmaximize (priv->meta_window, META_MAXIMIZE_HORIZONTAL | META_MAXIMIZE_VERTICAL);
    }
    else
    {
        meta_window_maximize (priv->meta_window, META_MAXIMIZE_HORIZONTAL | META_MAXIMIZE_VERTICAL);
    }
}

static void
applet_app_context_menu_item_on_move_to_left_workspace (CdosPopupBaseMenuItem *item,
                                                        ClutterEvent *event,
                                                        gboolean keep_menu,
                                                        AppletAppContextMenu *self)
{
    AppletAppContextMenuPrivate *priv = self->priv;
    g_return_if_fail (priv->meta_window != NULL);
    MetaWorkspace *meta_workspace = meta_window_get_workspace (priv->meta_window);
    MetaWorkspace *neighbor_left = meta_workspace_get_neighbor (meta_workspace, META_MOTION_LEFT);
    if (neighbor_left)
    {
        meta_window_change_workspace (priv->meta_window, neighbor_left);
        // notice:此处需要main下的checkWorkspaces ();
    }
}

static void
applet_app_context_menu_item_on_move_to_right_workspace (CdosPopupBaseMenuItem *item,
                                                         ClutterEvent *event,
                                                         gboolean keep_menu,
                                                         AppletAppContextMenu *self)
{
    AppletAppContextMenuPrivate *priv = self->priv;
    g_return_if_fail (priv->meta_window != NULL);
    MetaWorkspace *meta_workspace = meta_window_get_workspace (priv->meta_window);
    MetaWorkspace *neighbor_right = meta_workspace_get_neighbor (meta_workspace, META_MOTION_RIGHT);
    if (neighbor_right)
    {
        meta_window_change_workspace (priv->meta_window, neighbor_right);
        // notice:此处需要main下的checkWorkspaces ();
    }
}

static void
applet_app_context_menu_item_on_on_all_workspaces (CdosPopupBaseMenuItem *item,
                                                   ClutterEvent *event,
                                                   gboolean keep_menu,
                                                   AppletAppContextMenu *self)
{
    AppletAppContextMenuPrivate *priv = self->priv;
    g_return_if_fail (priv->meta_window != NULL);
    if (meta_window_is_on_all_workspaces (priv->meta_window))
    {
        meta_window_unstick (priv->meta_window);
    }
    else
    {
        meta_window_stick (priv->meta_window);
    }
}

static void
applet_app_context_menu_item_on_launch (CdosPopupBaseMenuItem *item,
                                        ClutterEvent *event,
                                        gboolean keep_menu,
                                        AppletAppContextMenu *self)
{
    AppletAppContextMenuPrivate *priv = self->priv;
    gboolean res = cdos_app_open_new_window (priv->app, -1);
    if (res == FALSE)
    {
        gboolean minimized;
        MetaWindow *last_focused_window = priv->meta_window;
        if (!G_IS_OBJECT (last_focused_window))
        {
            g_print ("last_focused_window is not a gobject!\n");
            return;
        }
        g_object_get (last_focused_window, "minimized", &minimized, NULL);
        guint32 current_time = cdos_global_get_current_time (global);

        meta_window_activate (last_focused_window, current_time);
        if (minimized)
        {
            meta_window_unminimize (last_focused_window);
        }
        meta_window_raise (last_focused_window);
    }
}

static void
applet_app_context_menu_item_on_toggle_favorite (CdosPopupBaseMenuItem *item,
                                                 ClutterEvent *event,
                                                 gboolean keep_menu,
                                                 AppletAppContextMenu *self)
{
    AppletAppContextMenuPrivate *priv = self->priv;

    GSettings *settings = priv->global_settings;
    gchar **desktop_files = g_settings_get_strv (settings, "panel-launchers");
    guint max_apps = g_settings_get_int (settings, "panel-launchers-max-number");
    if (priv->app_id == NULL)
    {
        return;
    }
    // AppletAppContextMenuItems *items = priv->items;
    cdos_popup_menu_close (CDOS_POPUP_MENU_BASE (self), FALSE);
    gchar **desktop_files_new = NULL;

    if (priv->is_favorite_app)
    {
        desktop_files_new = cdos_strv_elem_del_by_str (desktop_files, priv->app_id);
        g_settings_set_strv (settings, "panel-launchers", (const gchar *const *)desktop_files_new);
        // cdos_popup_menu_item_set_label (items->toggle_favorite, _("Add App"));
    }
    else
    {
        if (g_strv_length (desktop_files) < max_apps)
        {
            desktop_files_new = cdos_strv_elem_add ((const gchar **)desktop_files, priv->app_id);
            g_settings_set_strv (settings, "panel-launchers", (const gchar *const *)desktop_files_new);
            // cdos_popup_menu_item_set_label (items->toggle_favorite, _("Remove App"));
        }
        else
        {
            cdos_notify_info (NULL, _("Too many applications added"), _("The max number is 8"));
        }
    }
    if (desktop_files_new != NULL)
    {
        // notice: you cann't use  "g_strfreev"  to free "desktop_files_new", please see the return of the function
        // "g_ptr_array_free";
        g_free (desktop_files_new);
        g_strfreev (desktop_files);
    }
}

static void
applet_app_context_menu_item_on_nemo_property (CdosPopupBaseMenuItem *item,
                                                ClutterEvent *event,
                                                gboolean keep_menu,
                                                AppletAppContextMenu *self)
{
    //AppletAppContextMenu *self = APPLET_APP_CONTEXT_MENU (user_data);
    AppletAppContextMenuPrivate *priv = self->priv;
    const gchar *app_id = NULL;
    app_id = cdos_app_get_id (priv->app);
    applet_panel_launchers_edit_properties (app_id);
}

static void
applet_app_context_menu_item_on_move_window (CdosPopupBaseMenuItem *item,
                                             ClutterEvent *event,
                                             gboolean keep_menu,
                                             AppletAppContextMenu *self)
{
    MetaDisplay *display;
    ClutterActor *source;
    MetaWindow *window;
    AppletAppContextMenuPrivate *priv = self->priv;
    if (!priv->meta_window)
        return;
    cdos_popup_menu_close (CDOS_POPUP_MENU_BASE (self), FALSE);
    display = cdos_global_get_display (cdos_global_get ());
    if (meta_display_get_grab_op (display) == META_GRAB_OP_NONE)
    {
        source = cdos_popup_menu_base_get_source_actor (CDOS_POPUP_MENU_BASE (self));
        if (!APPLET_IS_WINDOW_BUTTON (source))
            return;
        window = applet_icon_label_button_get_window (APPLET_ICON_LABEL_BUTTON (source));
        meta_window_begin_grab_op (window, META_GRAB_OP_KEYBOARD_MOVING, TRUE, 0);
    }
}

static void
applet_app_context_menu_refresh (AppletAppContextMenu *self)
{
    AppletAppContextMenuPrivate *priv = self->priv;

    MetaWorkspace *workspace = NULL;
    if (priv->meta_window)
    {
        workspace = meta_window_get_workspace (priv->meta_window);
    }
    GSList *windows = cdos_app_get_windows (priv->app);

    if (priv->windows)
    {
        g_list_free (priv->windows);
        priv->windows = NULL;
    }
    while (windows != NULL)
    {
        MetaWindow *win = windows->data;
        if (meta_window_get_workspace (win) == workspace)
        {
            priv->windows = g_list_append (priv->windows, win);
        }
        windows = windows->next;
    }

    if (priv->windows == NULL)
    {
        priv->meta_window = NULL;
    }

    AppletAppContextMenuItems *items = priv->items;

    if (priv->meta_window)
    {
        clutter_actor_show (CLUTTER_ACTOR (items->separator));
        // clutter_actor_show (CDOS_POPUP_BASE_MENU_ITEM (items->on_all_workspaces));
        clutter_actor_show (CLUTTER_ACTOR (items->move_to_left_workspace));
        clutter_actor_show (CLUTTER_ACTOR (items->move_to_right_workspace));

        clutter_actor_show (CLUTTER_ACTOR (items->separator1));
        clutter_actor_show (CLUTTER_ACTOR (items->make_above));
        clutter_actor_show (CLUTTER_ACTOR (items->make_below));
        clutter_actor_show (CLUTTER_ACTOR (items->minimize_window));
        clutter_actor_show (CLUTTER_ACTOR (items->maximize_window));
        clutter_actor_show (CLUTTER_ACTOR (items->force_quit));
        clutter_actor_show (CLUTTER_ACTOR (items->close_window));
        clutter_actor_show (CLUTTER_ACTOR (items->close_all_windows));
    }
    else
    {
        clutter_actor_hide (CLUTTER_ACTOR (items->separator));
        // cdos_popup_base_menu_item_hide (CDOS_POPUP_BASE_MENU_ITEM (items->on_all_workspaces));
        clutter_actor_hide (CLUTTER_ACTOR (items->move_to_left_workspace));
        clutter_actor_hide (CLUTTER_ACTOR (items->move_to_right_workspace));

        clutter_actor_hide (CLUTTER_ACTOR (items->separator1));
        clutter_actor_hide (CLUTTER_ACTOR (items->make_above));
        clutter_actor_hide (CLUTTER_ACTOR (items->make_below));
        clutter_actor_hide (CLUTTER_ACTOR (items->minimize_window));
        clutter_actor_hide (CLUTTER_ACTOR (items->maximize_window));
        clutter_actor_hide (CLUTTER_ACTOR (items->force_quit));
        clutter_actor_hide (CLUTTER_ACTOR (items->close_window));
        clutter_actor_hide (CLUTTER_ACTOR (items->close_all_windows));
    }

    if (priv->is_favorite_app)
    {
        clutter_actor_show (CLUTTER_ACTOR (items->nemo_property));
    } else {
        clutter_actor_hide (CLUTTER_ACTOR (items->nemo_property));
    }

    if (g_list_length (priv->windows) > 1)
    {
        clutter_actor_show (CLUTTER_ACTOR (items->close_all_windows));
        clutter_actor_hide (CLUTTER_ACTOR (items->close_window));
    } else {
        clutter_actor_hide (CLUTTER_ACTOR (items->close_all_windows));
    }

    if (!cdos_app_can_open_new_window (priv->app))
        clutter_actor_hide ((ClutterActor *)items->launch);
    else
        clutter_actor_show ((ClutterActor *)items->launch);
    ClutterActor *source = cdos_popup_menu_base_get_source_actor (CDOS_POPUP_MENU_BASE (self));
    if (APPLET_IS_WINDOW_BUTTON (source))
        clutter_actor_show ((ClutterActor *)items->move_window);
    else
        clutter_actor_hide ((ClutterActor *)items->move_window);
}

typedef struct _ActionData
{
    AppletAppContextMenu *self;
    gchar *action;
}ActionData;

static void
applet_app_context_menu_item_on_action (CdosPopupMenuItem *item,
                                        ClutterEvent *event,
                                        gboolean keep_menu,
                                        ActionData *data)
{
    AppletAppContextMenu *self = data->self;
    cdos_app_launch_action (self->priv->app, data->action);
}

static void
add_action (AppletAppContextMenu *self,
            GDesktopAppInfo *app_info,
            const gchar *action)
{
    gchar *action_name = g_desktop_app_info_get_action_name (app_info, action);
    PopupBaseMenuItemInit params;
    CdosPopupMenuItem *item;
    ActionData *data;

    data = g_new0 (ActionData, 1);
    data->self = self;
    data->action = g_strdup (action);

    cdos_popup_base_menu_item_get_init_params (&params);
    item = cdos_popup_menu_item_new_with_params (action_name, &params);
    g_signal_connect ((CdosPopupBaseMenuItem *)item,
                      "activate",
                      G_CALLBACK (applet_app_context_menu_item_on_action),
                      data);
    cdos_popup_menu_base_add_menu_item (CDOS_POPUP_MENU_BASE (self), CDOS_POPUP_BASE_MENU_ITEM (item), -1);
}

static void
applet_app_context_menu_setup (AppletAppContextMenu *self)
{
    g_return_if_fail (APPLET_APP_CONTEXT_MENU (self));
    AppletAppContextMenuPrivate *priv = self->priv;

    AppletAppContextMenuItems *items = priv->items;
    items->separator = cdos_popup_separator_menu_item_new ();
    items->separator1 = cdos_popup_separator_menu_item_new ();

//注册菜单项目
#define ADD_NEW_ITEM(item, label)           \
    do                                      \
    {                                       \
        PopupBaseMenuItemInit params;       \
        cdos_popup_base_menu_item_get_init_params (&params);    \
        items->item = cdos_popup_menu_item_new_with_params (_(label), &params);    \
        priv->signal_ids->item##_activate_id = g_signal_connect ((CdosPopupBaseMenuItem *)items->item,  \
                                                                 "activate",         \
                                                                 G_CALLBACK (applet_app_context_menu_item_on_##item),  \
                                                                 self);              \
    } while (0)

    ADD_NEW_ITEM (close_window, "Close");
    ADD_NEW_ITEM (close_all_windows, "Close all");
    ADD_NEW_ITEM (force_quit, "Force quit");
    ADD_NEW_ITEM (make_above, "Set top");
    ADD_NEW_ITEM (make_below, "Set bottom");
    ADD_NEW_ITEM (minimize_window, "Minimize");
    ADD_NEW_ITEM (maximize_window, "Maximize");
    ADD_NEW_ITEM (move_to_left_workspace, "Move to left workspace");
    ADD_NEW_ITEM (move_to_right_workspace, "Move to right workspace");
    ADD_NEW_ITEM (on_all_workspaces, "Visible on all workspace");
    ADD_NEW_ITEM (launch, "New Window");
    ADD_NEW_ITEM (move_window, "Move");
#undef ADD_NEW_ITEM


    gboolean show_pinned = TRUE;
    // show_pinned = cdos_provider_get_boolean (CDOS_PROVIDER (applet_settings), "show-pinned", NULL);
    if (show_pinned)
    {
        items->toggle_favorite = cdos_popup_menu_item_new ();
        items->nemo_property = cdos_popup_menu_item_new();
        cdos_popup_menu_item_set_label (items->nemo_property, _("Properties"));
        if (priv->is_favorite_app)
            cdos_popup_menu_item_set_label (items->toggle_favorite, _("Remove from panel"));
        else
            cdos_popup_menu_item_set_label (items->toggle_favorite, _("Add to panel"));
        if (priv->app_id)
        {
            cdos_popup_base_menu_item_set_sensitive (CDOS_POPUP_BASE_MENU_ITEM (items->toggle_favorite), TRUE);
        }
        else
        {
            cdos_popup_base_menu_item_set_sensitive (CDOS_POPUP_BASE_MENU_ITEM (items->toggle_favorite), FALSE);
        }
        priv->signal_ids->toggle_favorite_toggle_id =
            g_signal_connect (CDOS_POPUP_BASE_MENU_ITEM (items->toggle_favorite),
                              "activate",
                              G_CALLBACK (applet_app_context_menu_item_on_toggle_favorite),
                              self);
        
        priv->signal_ids->nemo_property_id =
            g_signal_connect (CDOS_POPUP_BASE_MENU_ITEM (items->nemo_property),
                              "activate",
                              G_CALLBACK (applet_app_context_menu_item_on_nemo_property),
                              self);
                              
    }

    GDesktopAppInfo *app_info = cdos_app_get_app_info (priv->app);
    if (app_info)
    {
        const gchar *const *actions = g_desktop_app_info_list_actions (app_info);
        gint i = 0;
        while (actions[i])
        {
            add_action (self, app_info, actions[i]);
            i++;
        }
        if (i > 0)
        {
            ADD_MENU_ITEM (self, cdos_popup_separator_menu_item_new ());
        }
    }

    applet_app_context_menu_add_items (self);
}


static gboolean
applet_app_context_menu_has_mata_window (AppletAppContextMenu *self)
{
    AppletAppContextMenuPrivate *priv = self->priv;
    MetaWorkspace *workspace = NULL;
    gboolean has_flag = FALSE;
    if (priv->meta_window == NULL)
    {
        return FALSE;
    }
    workspace = meta_window_get_workspace (priv->meta_window);
    GSList *windows = cdos_app_get_windows (priv->app);
    while (windows != NULL)
    {
        MetaWindow *win = windows->data;
        if (meta_window_get_workspace (win) == workspace)
        {
            has_flag = TRUE;
            break;
        }
        windows = windows->next;
    }
    if (windows == NULL || !has_flag)
    {
        priv->meta_window = NULL;
    }
    return has_flag;
}

AppletAppContextMenu *
applet_app_context_menu_new (CdosApp *app, StWidget *source_actor, StSide arrow_side)
{
    AppletAppContextMenu *self;
    self = g_object_new (APPLET_TYPE_APP_CONTEXT_MENU,
                         "is-popup", TRUE,
                         "source-actor", source_actor,
                         "arrow-side", arrow_side,
                         NULL);
    AppletAppContextMenuPrivate *priv = self->priv;
    priv->app = app;
    priv->app_id = cdos_app_get_id (priv->app);

    priv->is_favorite_app = applet_app_context_menu_is_favorite_app (self);

    applet_app_context_menu_setup (self);
    ClutterActor *ui_group = cdos_ui_plugin_get_ui_group ();
    clutter_actor_add_child (ui_group, (ClutterActor *)((CdosPopupMenu *)self)->actor);
    return self;
}

void
applet_app_context_menu_set_meta_window (AppletAppContextMenu *self, MetaWindow *meta_window)
{
    g_return_if_fail (APPLET_APP_CONTEXT_MENU (self));
    AppletAppContextMenuPrivate *priv = self->priv;

    priv->meta_window = meta_window;
    applet_app_context_menu_refresh (self);
}

MetaWindow *
applet_app_context_menu_get_meta_window (AppletAppContextMenu *self)
{
    g_return_val_if_fail (APPLET_IS_APP_CONTEXT_MENU (self), NULL);
    AppletAppContextMenuPrivate *priv = self->priv;
    return priv->meta_window;
}

CdosApp *
applet_app_context_menu_get_app (AppletAppContextMenu *self)
{
    g_return_val_if_fail (APPLET_IS_APP_CONTEXT_MENU (self), NULL);
    AppletAppContextMenuPrivate *priv = self->priv;
    return priv->app;
}
