// Created by zhangtao on 16-4-16.

#include <meta/window.h>
#include <meta/display.h>

#include "common.h"
#include "cdos-base-util.h"
#include "cdos-drag-drop.h"
#include "cdos-popup-menu-manager.h"
#include "cdos-window-tracker.h"
#include "cdos-global.h"
#include "cdos-ui-plugin.h"

#include "applet-app-group.h"
#include "applet-window-list.h"
#include "applet-window-button.h"
#include "applet-tree.h"


struct _AppletAppGroupPrivate
{
    gboolean is_favorite_app;
    gboolean was_favorite_app;
    CdosApp *app;
    GTree *meta_windows;
    GList *meta_workspaces;
    AppletAppContextMenu *right_click_menu;
    AppletAppThumbnailHoverMenu *hover_menu;
    MetaWindow *last_focused_window;

    AppletButtonBox *window_button_box;
    AppletAppButton *app_button;

    CdosPopupMenuManager *menu_manager;
    CdosPopupMenuManager *hover_menu_manager;
    MetaWorkspace *meta_workspace;

    // GSettings: group-apps;
    gboolean group_app_buttons;
    GSettings *settings;

    StSide orientation;
};

G_DEFINE_TYPE_WITH_PRIVATE (AppletAppGroup, applet_app_group, ST_TYPE_WIDGET);

static void applet_app_group_setup (AppletAppGroup *self);
static void applet_app_group_focus_window_change (AppletAppGroup *self, MetaWindow *meta_window);
static void meta_window_on_notify_appears_focused (GObject *meta_window, GParamSpec *pspec, AppletAppGroup *self);
static gboolean applet_app_group_is_focus_app (AppletAppGroup *self);

static void applet_app_group_window_handle (AppletAppGroup *self);
static void applet_app_group_update_buttons (AppletAppGroup *self);
static void applet_app_group_button_setup (AppletAppGroup *self, AppletIconLabelButton *button);
static AppletWindowButton *applet_app_group_create_window_button (AppletAppGroup *self, MetaWindow *meta_window);

static void
meta_windows_destroy_item (gpointer data)
{
    clutter_actor_destroy (CLUTTER_ACTOR (data));
}

static void
window_button_box_on_condense_changed (AppletButtonBox *button_box, gboolean condense, gpointer user_data)
{
    //g_debug ("%s", __func__);
    AppletAppGroup *self = APPLET_APP_GROUP (user_data);
    AppletAppGroupPrivate *priv = self->priv;
    AppletAppSwitcherItem *switcher_item = NULL;
    if (priv->group_app_buttons)
    {
        return;
    }
    switcher_item = applet_app_thumbnail_hover_menu_get_app_switcher_item (priv->hover_menu);
    if (condense)
    {
        applet_app_switcher_item_show_one (switcher_item, NULL);
        applet_app_switcher_item_refresh (switcher_item);
    }
}
/*
static void
applet_app_group_buttons_layout_changed (AppletAppGroup *self, gboolean group_app_buttons)
{
    AppletAppGroupPrivate *priv = self->priv;
    priv->group_app_buttons = group_app_buttons;
    applet_app_group_update_buttons (self);
}

static void
settings_on_changed (GSettings *settings, gchar *key, gpointer user_data)
{
    AppletAppGroup *self = APPLET_APP_GROUP (user_data);
    // AppletAppGroupPrivate *priv = self->priv;
    if (g_strcmp0 (key, CDOS_WINDOW_LIST_GROUP_APPS) == 0)
    {
        gboolean group_apps = g_settings_get_boolean (settings, CDOS_WINDOW_LIST_GROUP_APPS);
        applet_app_group_buttons_layout_changed (self, group_apps);
    }
}
*/

static gint
applet_group_key_compare (MetaWindow *a, MetaWindow *b)
{
    return (gint)((intptr_t)b - (intptr_t)a);
}

static gboolean
traverse_tree (MetaWindow *window,
               AppletWindowButton *button,
               AppletAppGroup *self)
{
    g_signal_handlers_disconnect_by_data (window, self);

    return FALSE;
}

static gboolean
traverse_windows (MetaWindow *window,
                  AppletWindowButton *button,
                  MetaRectangle *rect)
{
    meta_window_set_icon_geometry (window, rect);
    return FALSE;
}

static gboolean
traverse_windows_button (MetaWindow *window,
                         AppletWindowButton *button,
                         AppletAppGroup *self)
{
    ClutterActor *actor = CLUTTER_ACTOR (button);
    ClutterActorBox box;
    MetaRectangle rect;
    cdos_base_util_get_transformed_allocation (actor, &box);
    rect.x = box.x1;
    rect.y = box.y1;
    rect.width = box.x2 - box.x1;
    rect.height = box.y2 - box.y1;
    meta_window_set_icon_geometry (window, &rect);
    return FALSE;
}

static void
applet_app_group_dispose (GObject *object)
{
    AppletAppGroup *self = APPLET_APP_GROUP (object);
    AppletAppGroupPrivate *priv = self->priv;

    if (priv->settings)
    {
        g_signal_handlers_disconnect_by_data (priv->settings, self);
        priv->settings = NULL;
    }

    if (priv->meta_windows)
    {
        g_tree_foreach (priv->meta_windows, (GTraverseFunc)traverse_tree, self);
        g_clear_pointer (&priv->meta_windows, g_tree_unref);
    }

    if (priv->meta_workspaces)
    {
        g_list_free (priv->meta_workspaces);
        priv->meta_workspaces = NULL;
    }

    if (priv->right_click_menu)
    {
        cdos_popup_menu_base_destroy (CDOS_POPUP_MENU_BASE (priv->right_click_menu));
        priv->right_click_menu = NULL;
    }
    g_clear_object (&priv->menu_manager);

    if (priv->hover_menu)
    {
        cdos_popup_menu_base_destroy (CDOS_POPUP_MENU_BASE (priv->hover_menu));
        priv->hover_menu = NULL;
    }
    g_clear_object (&priv->hover_menu_manager);

    G_OBJECT_CLASS (applet_app_group_parent_class)->dispose (object);
}

static void
applet_app_group_get_preferred_width (ClutterActor *actor,
                                      gfloat for_height,
                                      gfloat *min_width_p,
                                      gfloat *natural_width_p)
{
    AppletAppGroup *self = APPLET_APP_GROUP (actor);
    AppletAppGroupPrivate *priv = self->priv;

    StThemeNode *theme_node = st_widget_get_theme_node ((StWidget *)actor);
    st_theme_node_adjust_for_height (theme_node, &for_height);

    if (priv->group_app_buttons || (priv->is_favorite_app && !priv->last_focused_window))
        clutter_actor_get_preferred_width ((ClutterActor *)priv->app_button, for_height, min_width_p, natural_width_p);
    else if (priv->last_focused_window)
        clutter_actor_get_preferred_width ((ClutterActor *)priv->window_button_box, for_height, min_width_p, natural_width_p);

    st_theme_node_adjust_preferred_width (theme_node, min_width_p, natural_width_p);
}

static void
applet_app_group_get_preferred_height (ClutterActor *actor,
                                       gfloat for_width,
                                       gfloat *min_height_p,
                                       gfloat *natural_height_p)
{
    AppletAppGroup *self = APPLET_APP_GROUP (actor);
    AppletAppGroupPrivate *priv = self->priv;

    StThemeNode *theme_node = st_widget_get_theme_node ((StWidget *)actor);
    st_theme_node_adjust_for_width (theme_node, &for_width);

    if (priv->group_app_buttons || (priv->is_favorite_app && !priv->last_focused_window))
        clutter_actor_get_preferred_height ((ClutterActor *)priv->app_button, for_width, min_height_p, natural_height_p);
    else if (priv->last_focused_window)
        clutter_actor_get_preferred_height ((ClutterActor *)priv->window_button_box, for_width, min_height_p, natural_height_p);

    st_theme_node_adjust_preferred_height (theme_node, min_height_p, natural_height_p);
}

static void
applet_app_group_allocate (ClutterActor *actor,
                           const ClutterActorBox *box,
                           ClutterAllocationFlags flags)
{
    AppletAppGroup *self = APPLET_APP_GROUP (actor);
    AppletAppGroupPrivate *priv = self->priv;

    StThemeNode *theme_node = st_widget_get_theme_node ((StWidget *)actor);

    clutter_actor_set_allocation (actor, box, flags);

    ClutterActorBox content_box;
    st_theme_node_get_content_box (theme_node, box, &content_box);

    if (priv->group_app_buttons || (priv->is_favorite_app && !priv->last_focused_window))
    {
        clutter_actor_allocate ((ClutterActor *)priv->app_button, &content_box, flags);

        graphene_point3d_t point = {content_box.x1, content_box.y1, 0};
        graphene_point3d_t vertex;
        clutter_actor_apply_relative_transform_to_point (actor, NULL, &point, &vertex);
        MetaRectangle rect = {vertex.x, vertex.y,
                              content_box.x2 - content_box.x1,
                              content_box.y2 - content_box.y1};
        g_tree_foreach (priv->meta_windows, (GTraverseFunc)traverse_windows, &rect);
    }
    else if (priv->last_focused_window)
    {
        clutter_actor_allocate ((ClutterActor *)priv->window_button_box, &content_box, flags);
        g_tree_foreach (priv->meta_windows, (GTraverseFunc)traverse_windows_button, self);
    }
}

static void
applet_app_group_init (AppletAppGroup *self)
{
    AppletAppGroupPrivate *priv;
    self->priv = applet_app_group_get_instance_private (self);
    priv = self->priv;

    priv->app = NULL;
    priv->meta_windows = g_tree_new_full ((GCompareDataFunc)applet_group_key_compare, NULL, NULL, meta_windows_destroy_item);
    priv->meta_workspaces = NULL;
    priv->last_focused_window = NULL;

    clutter_actor_set_reactive ((ClutterActor *)self, TRUE);
    st_widget_set_can_focus ((StWidget *)self, TRUE);
    st_widget_set_track_hover ((StWidget *)self, TRUE);
    st_widget_set_style_class_name ((StWidget *)self, "window-list-group");

    priv->window_button_box = applet_button_box_new ();
    _add_actor (self, priv->window_button_box);

    g_signal_connect (
        priv->window_button_box, "condense-changed", G_CALLBACK (window_button_box_on_condense_changed), self);

    priv->settings = applet_window_list_get_settings ();
    priv->group_app_buttons = g_settings_get_boolean (priv->settings, CDOS_WINDOW_LIST_GROUP_APPS);

    StSide side = cdos_ui_plugin_get_applet_side ();
    if (side == ST_SIDE_TOP || side == ST_SIDE_BOTTOM)
        clutter_actor_set_y_expand (CLUTTER_ACTOR (self), TRUE);
    else
        clutter_actor_set_x_expand (CLUTTER_ACTOR (self), TRUE);
}

static void
applet_app_group_class_init (AppletAppGroupClass *klass)
{
    GObjectClass *gobject_class = G_OBJECT_CLASS (klass);

    gobject_class->dispose = applet_app_group_dispose;

    ClutterActorClass *actor_class = CLUTTER_ACTOR_CLASS (klass);
    actor_class->get_preferred_width = applet_app_group_get_preferred_width;
    actor_class->get_preferred_height = applet_app_group_get_preferred_height;
    actor_class->allocate = applet_app_group_allocate;
}

static void
applet_app_group_update_buttons (AppletAppGroup *self)
{
    AppletAppGroupPrivate *priv = self->priv;
    AppletAppSwitcherItem *switcher_item = NULL;
    switcher_item = applet_app_thumbnail_hover_menu_get_app_switcher_item (priv->hover_menu);

    if (priv->group_app_buttons || (priv->is_favorite_app && !priv->last_focused_window))
    {
        cdos_popup_menu_base_set_source_actor ((CdosPopupMenuBase *)priv->hover_menu,(ClutterActor *)priv->app_button);
        cdos_popup_menu_base_set_source_actor ((CdosPopupMenuBase *)priv->right_click_menu, (ClutterActor *)priv->app_button);
        applet_app_switcher_item_show_one (switcher_item, NULL);

        clutter_actor_hide (CLUTTER_ACTOR (priv->window_button_box));
        clutter_actor_show (CLUTTER_ACTOR (priv->app_button));
    }
    else
    {
        if (priv->last_focused_window)
        {
            ClutterActor *window_button = g_tree_lookup (priv->meta_windows, priv->last_focused_window);
            cdos_popup_menu_base_set_source_actor ((CdosPopupMenuBase *)priv->hover_menu, window_button);
            cdos_popup_menu_base_set_source_actor ((CdosPopupMenuBase *)priv->right_click_menu, window_button);

            clutter_actor_hide (CLUTTER_ACTOR (priv->app_button));
            clutter_actor_show (CLUTTER_ACTOR (priv->window_button_box));
        }
    }
}

static void
on_drag_begin (ClutterDragAction   *action,
               ClutterActor        *actor,
               gfloat               event_x,
               gfloat               event_y,
               ClutterModifierType  modifiers,
               AppletAppGroup      *self)
{
    AppletAppGroupPrivate *priv;
    priv = self->priv;
    cdos_popup_menu_close ((CdosPopupMenuBase *)priv->right_click_menu, FALSE);
    cdos_popup_menu_close ((CdosPopupMenuBase *)priv->hover_menu, FALSE);
    clutter_actor_hide (actor);
}

static void
on_drag_end (ClutterDragAction   *action,
             ClutterActor        *actor,
             gfloat               event_x,
             gfloat               event_y,
             ClutterModifierType  modifiers,
             AppletAppGroup      *self)
{
    clutter_actor_show (actor);
}

static void
applet_app_group_setup (AppletAppGroup *self)
{
    AppletAppGroupPrivate *priv = self->priv;

    priv->app_button = applet_app_button_new (priv->app, priv->is_favorite_app, priv->meta_workspace);

    _add_actor (self, priv->app_button);
    applet_app_group_button_setup (self, APPLET_ICON_LABEL_BUTTON (priv->app_button));

    // hover menu;
    priv->hover_menu = applet_app_thumbnail_hover_menu_new (priv->app, priv->is_favorite_app, (ClutterActor *)priv->app_button, priv->orientation);
    priv->hover_menu_manager = cdos_popup_menu_manager_new ((ClutterActor *)self);
    cdos_popup_menu_manager_add_menu (priv->hover_menu_manager, CDOS_POPUP_MENU_BASE (priv->hover_menu), -1);
    cdos_popup_menu_manager_block_all_events (priv->hover_menu_manager, FALSE);
    // context menu;
    priv->right_click_menu = applet_app_context_menu_new (priv->app, ST_WIDGET (self), priv->orientation);
    priv->menu_manager = cdos_popup_menu_manager_new ((ClutterActor *)self);
    cdos_popup_menu_manager_add_menu (priv->menu_manager, CDOS_POPUP_MENU_BASE (priv->right_click_menu), -1);

    // new:当右键菜单激活后，不触发窗口预览。
    applet_app_thumbnail_hover_menu_set_context_menu (priv->hover_menu, priv->right_click_menu);
    applet_app_group_update_buttons (self);

    ClutterAction *action = cdos_drag_action_new ((ClutterActor *)self);
    if (priv->is_favorite_app)
    {
        const char *desktop_id = cdos_app_get_id (priv->app);
        if (desktop_id)
            g_object_set_data_full ((GObject *)action, "desktop-id", g_strdup (desktop_id), g_free);
    }
    g_signal_connect (action, "drag-begin", G_CALLBACK (on_drag_begin), self);
    g_signal_connect (action, "drag-end", G_CALLBACK (on_drag_end), self);
}

static void
group_update_window_button_box (AppletAppGroup *self, AppletIconLabelButton *button)
{
    AppletAppGroupPrivate *priv = self->priv;

    MetaWindow *meta_window = NULL;
    AppletAppSwitcherItem *switcher_item = NULL;

    cdos_popup_menu_base_set_source_actor ((CdosPopupMenuBase *)priv->hover_menu, CLUTTER_ACTOR (button));
    meta_window = applet_icon_label_button_get_window (APPLET_ICON_LABEL_BUTTON (button));

    switcher_item = applet_app_thumbnail_hover_menu_get_app_switcher_item (priv->hover_menu);
    if (applet_button_box_get_condense (priv->window_button_box))
    {
        meta_window = NULL;
    }
    applet_app_switcher_item_show_one (switcher_item, meta_window);
}

static void
app_open_new_window (AppletAppGroup *self)
{
    AppletAppGroupPrivate *priv = self->priv;
    cdos_popup_menu_close (CDOS_POPUP_MENU_BASE (priv->right_click_menu), FALSE);
    applet_app_thumbnail_hover_menu_set_should_open (priv->hover_menu, FALSE);
    applet_app_thumbnail_hover_menu_close (priv->hover_menu, FALSE);
    cdos_app_open_new_window (priv->app, -1);
}

static gboolean
group_button_on_press (ClutterActor *actor, ClutterEvent *event, gpointer user_data)
{
    AppletAppGroup *self = APPLET_APP_GROUP (user_data);
    guint32 button = clutter_event_get_button (event);

    if (button == 1 || button == 2)
    {
        gboolean is_focus_app = applet_app_group_is_focus_app (self);
        if (is_focus_app)
        {
            st_widget_remove_style_pseudo_class (ST_WIDGET (actor), "click");
            st_widget_add_style_pseudo_class (ST_WIDGET (actor), "active");
        }
    }
    return FALSE;
}

static gboolean
group_button_on_release (ClutterActor *actor, ClutterEvent *event, gpointer user_data)
{
    AppletAppGroup *self = APPLET_APP_GROUP (user_data);
    AppletAppGroupPrivate *priv = self->priv;
    ClutterModifierType state = clutter_event_get_state (event);
    guint32 button = clutter_event_get_button (event);

    st_widget_remove_style_pseudo_class (ST_WIDGET (actor), "active");

    if (((state & CLUTTER_BUTTON1_MASK || button == 1) && priv->is_favorite_app)
        || ((state & CLUTTER_SHIFT_MASK) && (state & CLUTTER_BUTTON1_MASK || button == 1)))
    {
        app_open_new_window (self);
        return TRUE;
    }

    if (priv->last_focused_window == NULL && !priv->is_favorite_app)
    {
        return FALSE;
    }

    if (!priv->group_app_buttons && !applet_button_box_get_condense (priv->window_button_box))
    {
        MetaWindow *current_window = applet_icon_label_button_get_window (APPLET_ICON_LABEL_BUTTON (actor));
        applet_app_group_focus_window_change (self, current_window);
    }

    if ((state & CLUTTER_BUTTON2_MASK || button == 2))
    {
        app_open_new_window (self);
    }
    else if (state & CLUTTER_BUTTON1_MASK || button == 1)
    {
        if (CDOS_POPUP_MENU_BASE (priv->right_click_menu)->is_open)
        {
            cdos_popup_menu_base_toggle (CDOS_POPUP_MENU_BASE (priv->right_click_menu));
        }

        applet_app_group_window_handle (self);

        gboolean minimized;
        g_object_get (priv->last_focused_window, "minimized", &minimized, NULL);
        if (minimized)
        {
            applet_app_thumbnail_hover_menu_set_should_open (priv->hover_menu, FALSE);
        }
        applet_app_thumbnail_hover_menu_toggle (priv->hover_menu);
    }
    else if (state & CLUTTER_BUTTON3_MASK || button == 3)
    {
        applet_app_thumbnail_hover_menu_set_should_open (priv->hover_menu, FALSE);
        applet_app_thumbnail_hover_menu_close (priv->hover_menu, FALSE);

        if (!priv->group_app_buttons)
        {
            ClutterActor *source_actor =
                cdos_popup_menu_base_get_source_actor (CDOS_POPUP_MENU_BASE (priv->right_click_menu));
            if (source_actor != actor)
            {
                cdos_popup_menu_base_set_source_actor ((CdosPopupMenuBase *)priv->right_click_menu, actor);
                MetaWindow *meta_window = applet_icon_label_button_get_window (APPLET_ICON_LABEL_BUTTON (actor));
                applet_app_context_menu_set_meta_window (priv->right_click_menu, meta_window);
                cdos_popup_menu_close (CDOS_POPUP_MENU_BASE (priv->right_click_menu), FALSE);
            }
        }
        cdos_popup_menu_base_toggle (CDOS_POPUP_MENU_BASE (priv->right_click_menu));
    }
    return TRUE;
}

static gboolean
group_button_on_enter (ClutterActor *actor, ClutterEvent *event, gpointer user_data)
{
    AppletAppGroup *self = APPLET_APP_GROUP (user_data);
    AppletAppGroupPrivate *priv = self->priv;
    gboolean should_open = TRUE;

    if (!priv->group_app_buttons)
    {
        group_update_window_button_box (self, APPLET_ICON_LABEL_BUTTON (actor));
    }

    if (CDOS_POPUP_MENU_BASE (priv->right_click_menu)->is_open)
    {
        should_open = FALSE;
    }

    applet_app_thumbnail_hover_menu_set_should_open (priv->hover_menu, should_open);
    applet_app_thumbnail_hover_menu_hover_open (priv->hover_menu);
    return FALSE;
}

static gboolean
group_button_on_leave (ClutterActor *actor, ClutterEvent *event, gpointer user_data)
{
    AppletAppGroup *self = APPLET_APP_GROUP (user_data);
    AppletAppGroupPrivate *priv = self->priv;

    st_widget_remove_style_pseudo_class (ST_WIDGET (actor), "active");

    applet_app_thumbnail_hover_menu_set_should_open (priv->hover_menu, FALSE);
    applet_app_thumbnail_hover_menu_hover_close (priv->hover_menu);

    return FALSE;
}

static gboolean
group_button_on_key_press (ClutterActor *actor, ClutterEvent *event, gpointer user_data)
{
    guint symbol = clutter_event_get_key_symbol (event);
    if (symbol == CLUTTER_KEY_space || symbol == CLUTTER_KEY_Return
        || symbol == CLUTTER_KEY_Escape
        || symbol == CLUTTER_KEY_Down)
    {
        return TRUE;
    }
    else
    {
        return FALSE;
    }
}

static void
applet_app_group_button_setup (AppletAppGroup *self, AppletIconLabelButton *button)
{
    g_signal_connect_after (button, "button-press-event", G_CALLBACK (group_button_on_press), self);
    g_signal_connect_after (button, "button-release-event", G_CALLBACK (group_button_on_release), self);
    g_signal_connect (button, "enter-event", G_CALLBACK (group_button_on_enter), self);
    g_signal_connect (button, "leave-event", G_CALLBACK (group_button_on_leave), self);
    g_signal_connect (button, "key-press-event", G_CALLBACK (group_button_on_key_press), self);
}

static void
applet_app_group_window_handle (AppletAppGroup *self)
{
    AppletAppGroupPrivate *priv = self->priv;
    if (priv->last_focused_window == NULL)
    {
        return;
    }
    gboolean minimized;
    MetaWindow *last_focused_window = priv->last_focused_window;
    g_object_get (last_focused_window, "minimized", &minimized, NULL);
    guint32 current_time = cdos_global_get_current_time (cdos_global_get ());

    if (meta_window_has_focus (last_focused_window) == FALSE)
    {
        if (minimized)
        {
            meta_window_unminimize (last_focused_window);
        }
        meta_window_raise (last_focused_window);
        meta_window_activate (last_focused_window, current_time);
    }
    else
    {
        if (minimized)
        {
            meta_window_unminimize (last_focused_window);
        }
        else
        {
            meta_window_minimize (last_focused_window);
        }
    }
}

static AppletWindowButton *
applet_app_group_create_window_button (AppletAppGroup *self, MetaWindow *meta_window)
{
    AppletAppGroupPrivate *priv = self->priv;
    AppletWindowButton *button = NULL;
    button = applet_window_button_new (priv->app, priv->is_favorite_app, meta_window);
    applet_button_box_add_button (priv->window_button_box, APPLET_WINDOW_BUTTON (button));
    applet_app_group_button_setup (self, APPLET_ICON_LABEL_BUTTON (button));
    return button;
}

static void
applet_app_group_calc_window_number (AppletAppGroup *self, MetaWorkspace *meta_workspace)
{
    AppletAppGroupPrivate *priv = self->priv;

    if (!priv->app_button)
    {
        return;
    }
    gint window_num = g_tree_nnodes (priv->meta_windows);

    StWidget *app_button_actor = ST_WIDGET (priv->app_button);
    st_widget_remove_style_class_name (app_button_actor, "window-list-item-box-one");
    st_widget_remove_style_class_name (app_button_actor, "window-list-item-box-two");
    st_widget_remove_style_class_name (app_button_actor, "window-list-item-box-more");
    if (window_num == 1)
    {
        st_widget_add_style_class_name (app_button_actor, "window-list-item-box-one");
    }
    else if (window_num == 2)
    {
        st_widget_add_style_class_name (app_button_actor, "window-list-item-box-two");
    }
    else if (window_num >= 3)
    {
        st_widget_add_style_class_name (app_button_actor, "window-list-item-box-more");
    }
    applet_app_button_update_actor_style (priv->app_button);
}

static gboolean
applet_app_group_window_added (AppletAppGroup *self, MetaWorkspace *meta_workspace, MetaWindow *meta_window)
{
    AppletAppGroupPrivate *priv = self->priv;
    if (!meta_workspace || !meta_window)
    {
        return FALSE;
    }
    if (g_tree_lookup (priv->meta_windows, meta_window))
        return TRUE;

    CdosWindowTracker *tracker = cdos_window_tracker_get_default ();
    CdosApp *app = cdos_window_tracker_get_window_app (tracker, meta_window);
    if (app == NULL)
    {
        return FALSE;
    }

    MetaWindowType meta_window_type = meta_window_get_window_type (meta_window);

    if (app == priv->app
        && cdos_window_tracker_is_window_interesting (meta_window)
        && meta_window_type == META_WINDOW_NORMAL)
    {
        AppletWindowButton *button = applet_app_group_create_window_button (self, meta_window);
        applet_app_group_focus_window_change (self, meta_window);

        g_signal_connect (meta_window,
                          "notify::appears-focused",
                          G_CALLBACK (meta_window_on_notify_appears_focused),
                          self);

        g_tree_insert (priv->meta_windows, meta_window, button);
        applet_app_group_update_buttons (self);

        if (priv->is_favorite_app)
        {
            applet_app_group_is_favorite (self, FALSE);
        }
        if (priv->group_app_buttons)
        {
            applet_app_group_calc_window_number (self, meta_workspace);
        }
    }
    return TRUE;
}

void
applet_app_group_window_removed (AppletAppGroup *self, MetaWorkspace *meta_workspace, MetaWindow *meta_window)
{
    AppletAppGroupPrivate *priv = self->priv;
    MetaWindow *next_window = NULL;
    AppletIconLabelButton *next_button = NULL;
    gboolean has_window = FALSE;
    has_window = g_tree_remove (priv->meta_windows, meta_window);

    if (!has_window)
    {
        g_warn_if_reached ();
        return;
    }
    next_window = applet_tree_get_first_node (priv->meta_windows);

    // the button of meta_window will be destroyed, so we should update something about this button;
    if (!priv->group_app_buttons)
    {
        next_button = g_tree_lookup (priv->meta_windows, next_window);
    }
    if (!next_button)
    {
        next_button = (AppletIconLabelButton *)priv->app_button;
    }

    g_signal_handlers_disconnect_by_data (meta_window, self);

    group_update_window_button_box (self, APPLET_ICON_LABEL_BUTTON (next_button));
    cdos_popup_menu_base_set_source_actor ((CdosPopupMenuBase *)priv->right_click_menu, (ClutterActor *)next_button);
    applet_app_context_menu_set_meta_window (priv->right_click_menu, next_window);

    gboolean is_keep_menu = next_window != NULL ? TRUE : FALSE;
    applet_app_thumbnail_hover_menu_set_is_keep_menu (priv->hover_menu, is_keep_menu);
    applet_app_group_focus_window_change (self, next_window);

    if (!priv->last_focused_window && priv->is_favorite_app)
    {
        applet_app_group_update_buttons (self);
    }

    applet_app_group_calc_window_number (self, meta_workspace);
    applet_app_thumbnail_hover_menu_refresh (priv->hover_menu);
}

static void
applet_app_group_focus_window_change (AppletAppGroup *self, MetaWindow *meta_window)
{
    AppletAppGroupPrivate *priv = self->priv;

    priv->last_focused_window = meta_window;
    applet_app_context_menu_set_meta_window (priv->right_click_menu, priv->last_focused_window);
    applet_app_thumbnail_hover_menu_set_meta_window (priv->hover_menu, priv->last_focused_window);

    applet_button_box_set_last_focused_window (priv->window_button_box, priv->last_focused_window);
}

static void
meta_window_on_notify_appears_focused (GObject *meta_window, GParamSpec *pspec, AppletAppGroup *self)
{
    applet_app_group_focus_window_change (self, (MetaWindow *)meta_window);
}

static gboolean
applet_app_group_is_focus_app (AppletAppGroup *self)
{
    AppletAppGroupPrivate *priv = self->priv;
    GSList *windows = cdos_app_get_windows (priv->app);
    gint focus = FALSE;
    while (windows)
    {
        MetaWindow *meta_window = windows->data;
        if (meta_window_appears_focused (meta_window) || meta_window_has_focus (meta_window))
        {
            focus = TRUE;
            break;
        }
        windows = windows->next;
    }
    return focus;
}

AppletAppGroup *
applet_app_group_new (CdosApp *app, gboolean is_favorite_app, MetaWorkspace *meta_workspace)
{
    g_return_val_if_fail (CDOS_IS_APP (app), NULL);

    AppletAppGroup *self;
    self = g_object_new (APPLET_TYPE_APP_GROUP, NULL);

    AppletAppGroupPrivate *priv = self->priv;
    priv->app = app;
    priv->is_favorite_app = is_favorite_app;
    priv->meta_workspace = meta_workspace;
    priv->orientation = cdos_ui_plugin_get_applet_side ();
    applet_app_group_setup (self);
    return self;
}

void
applet_app_group_update_meta_windows (AppletAppGroup *self, MetaWorkspace *meta_workspace)
{
    AppletAppGroupPrivate *priv = self->priv;
    CdosWindowTracker *tracker = cdos_window_tracker_get_default ();
    GList *window_list = meta_workspace_list_windows (meta_workspace);
    // GList *new_window_list;
    // applet_button_box_clear (priv->window_button_box);
    for (GList *it = window_list; it; it = it->next)
    {
        MetaWindow *meta_window = it->data;
        CdosApp *app = cdos_window_tracker_get_window_app (tracker, meta_window);
        if (app == priv->app && cdos_window_tracker_is_window_interesting (meta_window))
        {
            applet_app_group_window_added (self, meta_workspace, meta_window);
        }
    }
    g_list_free (window_list);

    if (!priv->last_focused_window)
        applet_app_group_focus_window_change (self, priv->last_focused_window);
}

void
applet_app_group_is_favorite (AppletAppGroup *self, gboolean is_favorite)
{
    g_return_if_fail (APPLET_IS_APP_GROUP (self));
    AppletAppGroupPrivate *priv = self->priv;

    priv->is_favorite_app = is_favorite;
    priv->was_favorite_app = is_favorite == TRUE ? FALSE : TRUE;

    AppletAppSwitcherItem *app_switcher_item = applet_app_thumbnail_hover_menu_get_app_switcher_item (priv->hover_menu);
    applet_app_switcher_item_set_is_favorite (app_switcher_item, is_favorite);
}

gboolean
applet_app_group_get_is_favorite_app (AppletAppGroup *self)
{
    g_return_val_if_fail (APPLET_IS_APP_GROUP (self), FALSE);
    AppletAppGroupPrivate *priv = self->priv;
    return priv->is_favorite_app;
}

gboolean
applet_app_group_get_was_favorite_app (AppletAppGroup *self)
{
    g_return_val_if_fail (APPLET_IS_APP_GROUP (self), FALSE);
    AppletAppGroupPrivate *priv = self->priv;
    return priv->was_favorite_app;
}

void
applet_app_group_set_buttons_condense (AppletAppGroup *self, gboolean condense)
{
    g_return_if_fail (APPLET_IS_APP_GROUP (self));
    AppletAppGroupPrivate *priv = self->priv;
    if (!priv->group_app_buttons)
    {
        applet_button_box_set_condense (priv->window_button_box, condense);
    }
}

AppletButtonBox *
applet_app_group_get_window_button_box (AppletAppGroup *self)
{
    g_return_val_if_fail (APPLET_IS_APP_GROUP (self), NULL);
    AppletAppGroupPrivate *priv = self->priv;
    return priv->window_button_box;
}

void
applet_app_group_set_orientation (AppletAppGroup *self, StSide orientation)
{
    AppletAppGroupPrivate *priv = self->priv;
    if (priv->orientation != orientation)
    {
        priv->orientation = orientation;
        cdos_popup_menu_set_arrow_side ((CdosPopupMenu *)priv->hover_menu, orientation);
        cdos_popup_menu_set_arrow_side ((CdosPopupMenu *)priv->right_click_menu, orientation);
    }
}
