// Created by zhangtao on 16-4-16.

#include <math.h>
#include "common.h"
#include "applet-app-switcher-item.h"
#include "applet-window-thumbnail.h"
#include "cdos-layout-manager.h"
#include "cdos-global.h"

#define LIKELY_THUMBNAIL_WIDTH 240
#define LIKELY_THUMBNAIL_HEIGHT_V 40

typedef struct _AppletAppSwitcherItemAppThumbnail AppletAppSwitcherItemAppThumbnail;
typedef struct _AppletAppSwitcherItemSignalIds AppletAppSwitcherItemSignalIds;

enum
{
    THUMBNAILS_LAYOUT_CHANGED,
    LAST_SIGNAL
};

struct _AppletAppSwitcherItemPrivate
{
    gboolean is_favorite_app;
    gfloat thumbnails_space;
    MetaWindow *meta_window;
    CdosApp *app;
    StWidget *box;
    StWidget *app_container;
    StWidget *app_container2;
    StWidget *app_container3;
    GSList *app_thumbnails;
    GSList *app_thumbnails2;
    GSList *app_thumbnails3;
    AppletWindowThumbnail *meta_window_thumbnail;
    AppletAppSwitcherItemSignalIds *signal_ids;
    gboolean vertical_thumbs;
    StWidget *page_up_button;
    StWidget *page_down_button;
    gint show_start_index;
    gint show_last_index;
    gint app_windows_length;
    gint page_size;
    gint monitor_width;
    gint monitor_height;
    GSList *thumbnail_actors;

    MetaWindow *show_window;
};

G_DEFINE_TYPE_WITH_PRIVATE (AppletAppSwitcherItem, applet_app_switcher_item, CDOS_TYPE_POPUP_BASE_MENU_ITEM);

struct _AppletAppSwitcherItemAppThumbnail
{
    MetaWindow *meta_window;
    AppletWindowThumbnail *thumbnail;
    gint container_num;
};

struct _AppletAppSwitcherItemSignalIds
{
    gulong change_thumbnails_layout_id;
    gulong page_up_clicked_id;
    gulong page_down_clicked_id;
    gulong page_scroll_id;
};

static gint applet_app_switcher_item_signals[LAST_SIGNAL] = {0};

static void applet_app_switcher_item_dispose (GObject *object);
static void applet_app_switcher_item_finalize (GObject *object);

static void applet_app_switcher_item_setup (AppletAppSwitcherItem *self,
                                            CdosApp *app,
                                            gboolean is_favorite,
                                            MetaWindow *meta_window);

static void applet_app_switcher_item_set_vertical_setting (AppletAppSwitcherItem *self);

static AppletAppSwitcherItemAppThumbnail *
applet_app_switcher_item_get_app_thumbnail_with_window (GSList *app_thumbnails, MetaWindow *window);

static GSList *applet_app_switcher_item_get_app_windows (AppletAppSwitcherItem *self);

static gboolean
applet_app_switcher_item_page_up_on_clicked (StButton *button, gint clicked_button, AppletAppSwitcherItem *self);
static gboolean
applet_app_switcher_item_page_down_on_clicked (StButton *button, gint clicked_button, AppletAppSwitcherItem *self);
static gboolean
applet_app_switcher_item_page_on_scroll (ClutterActor *actor, ClutterEvent *event, AppletAppSwitcherItem *self);
static void applet_app_switcher_item_page_up (AppletAppSwitcherItem *self);
static void applet_app_switcher_item_page_down (AppletAppSwitcherItem *self);
static void applet_app_switcher_item_page_connect_events (AppletAppSwitcherItem *self);
static void applet_app_switcher_item_page_disconnect_events (AppletAppSwitcherItem *self);
static void applet_app_switcher_item_set_button_sensitive (StWidget *button, gboolean sensitive);
static void applet_app_switcher_item_refresh_vertical_list (AppletAppSwitcherItem *self);
static void applet_app_switcher_item_reset_page_index (AppletAppSwitcherItem *self);
static void applet_app_switcher_item_add_windows_loop (AppletAppSwitcherItem *self,
                                                       gint i,
                                                       gint win_length,
                                                       StWidget *actor,
                                                       GSList *windows,
                                                       gint container_num);
static void applet_app_switcher_item_remove_old_windows (AppletAppSwitcherItem *self, GSList *windows);
static void applet_app_switcher_item_add_new_windows (AppletAppSwitcherItem *self, GSList *windows);

static void
applet_app_switcher_item_init (AppletAppSwitcherItem *self)
{
    AppletAppSwitcherItemPrivate *priv;
    self->priv = applet_app_switcher_item_get_instance_private (self);
    priv = self->priv;

    priv->vertical_thumbs = FALSE;
    priv->signal_ids = g_new0 (AppletAppSwitcherItemSignalIds, 1);
    priv->thumbnails_space = 0;
    priv->show_start_index = 0;
    priv->show_window = NULL;

    ClutterActor *actor = CLUTTER_ACTOR (self);
    ActorChild item_params;
    cdos_popup_base_menu_item_get_actor_child (&item_params);

    st_widget_set_style_class_name (ST_WIDGET (actor), NULL);

    priv->box = st_box_layout_new ();
    st_box_layout_set_vertical ((StBoxLayout *)priv->box, TRUE);
    clutter_actor_set_reactive ((ClutterActor *)priv->box, TRUE);
    st_widget_set_style_class_name (priv->box, "switcher-list-box");

    priv->app_container = st_box_layout_new ();
    st_widget_set_style_class_name (priv->app_container, "switcher-list");

    priv->app_container2 = st_box_layout_new ();
    st_widget_set_style_class_name (priv->app_container2, "switcher-list");
    clutter_actor_hide (CLUTTER_ACTOR (priv->app_container2));

    priv->app_container3 = st_box_layout_new ();
    st_widget_set_style_class_name (priv->app_container3, "switcher-list");
    clutter_actor_hide (CLUTTER_ACTOR (priv->app_container3));

    priv->page_up_button = st_button_new ();
    st_widget_set_style_class_name (priv->page_up_button, "window-list-switcher-page-up-button");
    st_widget_set_track_hover (priv->page_up_button, TRUE);

    priv->page_down_button = st_button_new ();
    st_widget_set_style_class_name (priv->page_down_button, "window-list-switcher-page-down-button");
    st_widget_set_track_hover (priv->page_down_button, TRUE);

    cdos_popup_base_menu_item_add_actor (CDOS_POPUP_BASE_MENU_ITEM (self), (ClutterActor *)priv->box, &item_params);
    cdos_popup_base_menu_item_set_keep_menu (CDOS_POPUP_BASE_MENU_ITEM (self), FALSE);
}

static void
applet_app_switcher_item_class_init (AppletAppSwitcherItemClass *klass)
{
    GObjectClass *gobject_class = G_OBJECT_CLASS (klass);

    gobject_class->dispose = applet_app_switcher_item_dispose;
    gobject_class->finalize = applet_app_switcher_item_finalize;

    applet_app_switcher_item_signals[THUMBNAILS_LAYOUT_CHANGED] =
        g_signal_new ("thumbnails-layout-changed",
                       G_TYPE_FROM_CLASS (gobject_class),
                       G_SIGNAL_RUN_LAST,
                       0,
                       NULL,
                       NULL,
                       g_cclosure_marshal_VOID__VOID,
                       G_TYPE_NONE,
                       0);
}

static void
applet_app_switcher_item_dispose (GObject *object)
{
    AppletAppSwitcherItem *self = APPLET_APP_SWITCHER_ITEM (object);
    AppletAppSwitcherItemPrivate *priv = self->priv;
    applet_app_switcher_item_page_disconnect_events (self);

    if (priv->app_thumbnails != NULL)
    {
        GSList *app_thumbs = priv->app_thumbnails;
        while (app_thumbs)
        {
            AppletAppSwitcherItemAppThumbnail *thumbnail = app_thumbs->data;
            g_clear_pointer ((ClutterActor **)&thumbnail->thumbnail, clutter_actor_destroy);
            g_free (thumbnail);
            app_thumbs->data = NULL;
            app_thumbs = app_thumbs->next;
        }
        g_slist_free (priv->app_thumbnails);
        priv->app_thumbnails = NULL;
    }
    if (priv->app_thumbnails2 != NULL)
    {
        GSList *app_thumbs2 = priv->app_thumbnails2;
        while (app_thumbs2)
        {
            AppletAppSwitcherItemAppThumbnail *thumbnail = app_thumbs2->data;
            g_clear_pointer ((ClutterActor **)&thumbnail->thumbnail, clutter_actor_destroy);
            g_free (thumbnail);
            app_thumbs2->data = NULL;
            app_thumbs2 = app_thumbs2->next;
        }
        g_slist_free (priv->app_thumbnails2);
        priv->app_thumbnails2 = NULL;
    }
    if (priv->app_thumbnails3 != NULL)
    {
        GSList *app_thumbs3 = priv->app_thumbnails3;
        while (app_thumbs3)
        {
            AppletAppSwitcherItemAppThumbnail *thumbnail = app_thumbs3->data;
            g_clear_pointer ((ClutterActor **)&thumbnail->thumbnail, clutter_actor_destroy);
            g_free (thumbnail);
            app_thumbs3->data = NULL;
            app_thumbs3 = app_thumbs3->next;
        }
        g_slist_free (priv->app_thumbnails3);
        priv->app_thumbnails3 = NULL;
    }

    // g_clear_object (&priv->box);
    G_OBJECT_CLASS (applet_app_switcher_item_parent_class)->dispose (object);
}

static void
applet_app_switcher_item_finalize (GObject *object)
{
    AppletAppSwitcherItem *self = APPLET_APP_SWITCHER_ITEM (object);
    AppletAppSwitcherItemPrivate *priv = self->priv;
    g_free (priv->signal_ids);
    G_OBJECT_CLASS (applet_app_switcher_item_parent_class)->finalize (object);
}

static void
applet_app_switcher_item_setup (AppletAppSwitcherItem *self,
                                CdosApp *app,
                                gboolean is_favorite,
                                MetaWindow *meta_window)
{
    AppletAppSwitcherItemPrivate *priv = self->priv;

    priv->meta_window = meta_window;
    priv->app = app;
    priv->is_favorite_app = is_favorite;

    applet_app_switcher_item_set_vertical_setting (self);
    applet_app_switcher_item_refresh (self);
}

static void
applet_app_switcher_item_on_thumbnails_layout_changed (AppletAppSwitcherItem *self)
{
    CdosLayoutManager *layout_manager = cdos_layout_manager_get_default ();
    const MetaRectangle *primary_monitor = cdos_layout_manager_get_primary_monitor (layout_manager);
    if (!primary_monitor)
        return;
    AppletAppSwitcherItemPrivate *priv = self->priv;
    priv->monitor_width = primary_monitor->width;
    priv->monitor_height = primary_monitor->height;

    priv->thumbnails_space = ceil (priv->monitor_width / LIKELY_THUMBNAIL_WIDTH);
    //计算垂直布局模式下的相关参数, 去最大高度的80%作为计算高度;
    priv->page_size = (int)ceil (priv->monitor_height * 0.8 / LIKELY_THUMBNAIL_HEIGHT_V);
    priv->show_last_index = priv->page_size - 1;

    gboolean vertical = priv->vertical_thumbs;

    GList *children = clutter_actor_get_children (CLUTTER_ACTOR (priv->box));
    if (g_list_length (children) > 0)
    {
        g_object_ref (priv->app_container3);
        g_object_ref (priv->app_container2);
        g_object_ref (priv->app_container);
        clutter_actor_remove_child (CLUTTER_ACTOR (priv->box), CLUTTER_ACTOR (priv->app_container3));
        clutter_actor_remove_child (CLUTTER_ACTOR (priv->box), CLUTTER_ACTOR (priv->app_container2));
        clutter_actor_remove_child (CLUTTER_ACTOR (priv->box), CLUTTER_ACTOR (priv->app_container));
    }

    st_widget_remove_style_class_name (ST_WIDGET (priv->app_container2), "window-list-switcher-item-container2");
    st_widget_remove_style_class_name (ST_WIDGET (priv->app_container3), "window-list-switcher-item-container3");
    st_widget_remove_style_class_name (ST_WIDGET (priv->app_container), "window-list-switcher-item-container");
    st_widget_remove_style_pseudo_class (priv->box, "vertical");

    st_box_layout_set_vertical ((StBoxLayout *)priv->box, vertical);
    st_box_layout_set_vertical ((StBoxLayout *)priv->app_container, vertical);
    st_box_layout_set_vertical ((StBoxLayout *)priv->app_container2, vertical);
    st_box_layout_set_vertical ((StBoxLayout *)priv->app_container3, vertical);

    if (vertical)
    {
        priv->show_start_index = 0;
        priv->show_last_index = priv->page_size - 1;
        clutter_actor_add_child (CLUTTER_ACTOR (priv->box), CLUTTER_ACTOR (priv->app_container2));
        st_widget_add_style_class_name (ST_WIDGET (priv->app_container2), "window-list-switcher-item-container2");
        _add_actor (priv->box, priv->app_container);
        st_widget_add_style_class_name (ST_WIDGET (priv->app_container), "window-list-switcher-item-container");
        _add_actor (priv->box, priv->app_container3);
        st_widget_add_style_class_name (ST_WIDGET (priv->app_container3), "window-list-switcher-item-container3");

        st_widget_add_style_pseudo_class (priv->box, "vertical");

        _add_actor (priv->app_container2, priv->page_up_button);
        _add_actor (priv->app_container3, priv->page_down_button);
        g_object_set (priv->page_up_button, 
                      "x-expand", TRUE, 
                      "x-align", CLUTTER_ACTOR_ALIGN_FILL, 
                      NULL);
        g_object_set (priv->page_down_button, 
                      "x-expand", TRUE, 
                      "x-align", CLUTTER_ACTOR_ALIGN_FILL, 
                      NULL);

        gint windows_length = priv->app_windows_length;

        applet_app_switcher_item_set_button_sensitive (priv->page_up_button, FALSE);
        // g_print ("\n%s:------>windows_length = %d, page_size = %d;\n", __func__, windows_length, priv->page_size);
        if (windows_length > priv->page_size)
        {
            clutter_actor_show (CLUTTER_ACTOR (priv->app_container2));
            clutter_actor_show (CLUTTER_ACTOR (priv->app_container3));
            applet_app_switcher_item_set_button_sensitive (priv->page_down_button, TRUE);
        } else {
            clutter_actor_hide (CLUTTER_ACTOR (priv->app_container2));
            clutter_actor_hide (CLUTTER_ACTOR (priv->app_container3));
            applet_app_switcher_item_set_button_sensitive (priv->page_down_button, FALSE);
        }
        applet_app_switcher_item_page_connect_events (self);
    } else {
        g_object_ref (priv->page_up_button);
        g_object_ref (priv->page_down_button);
        applet_app_switcher_item_page_disconnect_events (self);

        _remove_actor (priv->app_container2, priv->page_up_button);
        _remove_actor (priv->app_container3, priv->page_down_button);

        clutter_actor_hide (CLUTTER_ACTOR (priv->app_container2));
        clutter_actor_hide (CLUTTER_ACTOR (priv->app_container3));
        _add_actor (priv->box, priv->app_container3);
        _add_actor (priv->box, priv->app_container2);
        _add_actor (priv->box, priv->app_container);
    }

    g_list_free (children);
    GSList *app_windows = applet_app_switcher_item_get_app_windows (self);
    applet_app_switcher_item_add_new_windows (self, app_windows);
    // clutter_actor_realize (CLUTTER_ACTOR (priv->app_container));
}

static void
applet_app_switcher_item_set_vertical_setting (AppletAppSwitcherItem *self)
{
    AppletAppSwitcherItemPrivate *priv = self->priv;
    GList *children = clutter_actor_get_children (CLUTTER_ACTOR (priv->box));
    if (children != NULL)
    {
        g_object_ref (priv->app_container3);
        g_object_ref (priv->app_container2);
        g_object_ref (priv->app_container);
        _remove_actor (priv->box, priv->app_container3);
        _remove_actor (priv->box, priv->app_container2);
        _remove_actor (priv->box, priv->app_container);
    }

    st_box_layout_set_vertical (ST_BOX_LAYOUT (priv->box), TRUE);
    st_box_layout_set_vertical (ST_BOX_LAYOUT (priv->app_container), FALSE);
    st_box_layout_set_vertical (ST_BOX_LAYOUT (priv->app_container2), FALSE);
    st_box_layout_set_vertical (ST_BOX_LAYOUT (priv->app_container3), FALSE);

    _add_actor (priv->box, priv->app_container3);
    _add_actor (priv->box, priv->app_container2);
    _add_actor (priv->box, priv->app_container);

    g_list_free (children);
}

static AppletAppSwitcherItemAppThumbnail *
applet_app_switcher_item_get_app_thumbnail_with_window (GSList *app_thumbnails, MetaWindow *window)
{
    if (app_thumbnails == NULL || window == NULL)
    {
        return NULL;
    }
    GSList *thumbs = app_thumbnails;
    while (thumbs)
    {
        AppletAppSwitcherItemAppThumbnail *it = thumbs->data;
        if (it->meta_window == window)
        {
            return it;
        }
        thumbs = thumbs->next;
    }
    return NULL;
}

static GSList *
applet_app_switcher_item_get_app_windows (AppletAppSwitcherItem *self)
{
    AppletAppSwitcherItemPrivate *priv = self->priv;
    MetaWorkspace *meta_workspace = NULL;
    if (priv->meta_window != NULL)
    {
        meta_workspace = meta_window_get_workspace (priv->meta_window);
    }
    GSList *app_windows = cdos_app_get_windows (priv->app);
    gint len = 0;
    GSList *windows = NULL;
    while (app_windows)
    {
        MetaWindow *win = app_windows->data;
        MetaWindowType meta_window_type = meta_window_get_window_type (win);
        if (meta_window_type == META_WINDOW_NORMAL && meta_workspace == meta_window_get_workspace (win))
        {
            windows = g_slist_prepend (windows, win);
            len++;
        }
        app_windows = app_windows->next;
    }
    priv->app_windows_length = len;
    applet_app_switcher_item_reset_page_index (self);
    return windows;
}

static void
applet_app_switcher_item_set_button_sensitive (StWidget *button, gboolean sensitive)
{
    // clutter_actor_set_reactive ((ClutterActor *)button, TRUE);
    st_widget_remove_style_pseudo_class (button, "insensitive");
    if (sensitive)
    {
        st_widget_set_track_hover (button, TRUE);
    }
    else
    {
        st_widget_remove_style_pseudo_class (button, "hover");
        st_widget_add_style_pseudo_class (button, "insensitive");
        st_widget_set_track_hover (button, FALSE);
    }
}

static gboolean
applet_app_switcher_item_page_up_on_clicked (StButton *button, gint clicked_button, AppletAppSwitcherItem *self)
{
    // AppletAppSwitcherItemPrivate *priv = self->priv;
    if (clicked_button == 1)
    {
        applet_app_switcher_item_page_up (self);
    }
    return FALSE;
}

static gboolean
applet_app_switcher_item_page_down_on_clicked (StButton *button, gint clicked_button, AppletAppSwitcherItem *self)
{
    // AppletAppSwitcherItemPrivate *priv = self->priv;
    if (clicked_button == 1)
    {
        applet_app_switcher_item_page_down (self);
    }
    return FALSE;
}

static gboolean
applet_app_switcher_item_page_on_scroll (ClutterActor *actor, ClutterEvent *event, AppletAppSwitcherItem *self)
{
    // AppletAppSwitcherItemPrivate *priv = self->priv;

    ClutterScrollDirection direction = clutter_event_get_scroll_direction (event);
    if (direction == CLUTTER_SCROLL_UP)
    {
        applet_app_switcher_item_page_up (self);
    }
    else if (direction == CLUTTER_SCROLL_DOWN)
    {
        applet_app_switcher_item_page_down (self);
    }

    return FALSE;
}

static void
applet_app_switcher_item_page_up (AppletAppSwitcherItem *self)
{
    AppletAppSwitcherItemPrivate *priv = self->priv;
    if (priv->show_start_index > 0)
    {
        priv->show_last_index = priv->show_last_index - 1;
        priv->show_start_index = priv->show_start_index - 1;
        applet_app_switcher_item_refresh_vertical_list (self);
    }
}

static void
applet_app_switcher_item_page_down (AppletAppSwitcherItem *self)
{
    AppletAppSwitcherItemPrivate *priv = self->priv;

    if (priv->show_last_index < priv->app_windows_length - 1)
    {
        priv->show_last_index = priv->show_last_index + 1;
        priv->show_start_index = priv->show_start_index + 1;
        applet_app_switcher_item_refresh_vertical_list (self);
    }
}

static void
applet_app_switcher_item_page_connect_events (AppletAppSwitcherItem *self)
{
    AppletAppSwitcherItemPrivate *priv = self->priv;
    AppletAppSwitcherItemSignalIds *ids = priv->signal_ids;
    if (ids->page_up_clicked_id == 0)
    {
        ids->page_up_clicked_id = g_signal_connect (ST_BUTTON (priv->page_up_button),
                                                    "clicked",
                                                    G_CALLBACK (applet_app_switcher_item_page_up_on_clicked),
                                                    self);
    }
    if (ids->page_down_clicked_id == 0)
    {
        ids->page_down_clicked_id = g_signal_connect (ST_BUTTON (priv->page_down_button),
                                                      "clicked",
                                                      G_CALLBACK (applet_app_switcher_item_page_down_on_clicked),
                                                      self);
    }
    if (ids->page_scroll_id == 0)
    {
        ids->page_scroll_id = g_signal_connect (
            CLUTTER_ACTOR (priv->box), "scroll-event", G_CALLBACK (applet_app_switcher_item_page_on_scroll), self);
    }
}

static void
applet_app_switcher_item_page_disconnect_events (AppletAppSwitcherItem *self)
{
    AppletAppSwitcherItemPrivate *priv = self->priv;
    AppletAppSwitcherItemSignalIds *ids = priv->signal_ids;
    if (ids->page_up_clicked_id > 0)
    {
        g_signal_handler_disconnect (priv->page_up_button, ids->page_up_clicked_id);
        ids->page_up_clicked_id = 0L;
    }
    if (ids->page_down_clicked_id > 0)
    {
        g_signal_handler_disconnect (priv->page_down_button, ids->page_down_clicked_id);
        ids->page_down_clicked_id = 0L;
    }
    if (ids->page_scroll_id > 0)
    {
        g_signal_handler_disconnect (priv->box, ids->page_scroll_id);
        ids->page_scroll_id = 0L;
    }
}

static void
applet_app_switcher_item_refresh_vertical_list (AppletAppSwitcherItem *self)
{
    AppletAppSwitcherItemPrivate *priv = self->priv;
    GSList *actors = priv->thumbnail_actors;

    if (priv->app_windows_length > priv->page_size)
    {
        clutter_actor_show (CLUTTER_ACTOR (priv->app_container2));
        clutter_actor_show (CLUTTER_ACTOR (priv->app_container3));
    }
    else
    {
        clutter_actor_hide (CLUTTER_ACTOR (priv->app_container2));
        clutter_actor_hide (CLUTTER_ACTOR (priv->app_container3));
    }
    gint i = 0;
    while (actors)
    {
        StWidget *actor = actors->data;
        if (i >= priv->show_start_index && i <= priv->show_last_index)
        {
            clutter_actor_show (CLUTTER_ACTOR (actor));
        }
        else
        {
            clutter_actor_hide (CLUTTER_ACTOR (actor));
        }
        i++;
        actors = actors->next;
    }

    applet_app_switcher_item_set_button_sensitive (priv->page_up_button, TRUE);
    applet_app_switcher_item_set_button_sensitive (priv->page_down_button, TRUE);

    if (priv->show_last_index == priv->app_windows_length - 1)
    {
        applet_app_switcher_item_set_button_sensitive (priv->page_down_button, FALSE);
    }
    if (priv->show_start_index == 0)
    {
        applet_app_switcher_item_set_button_sensitive (priv->page_up_button, FALSE);
    }
}

static void
applet_app_switcher_item_reset_page_index (AppletAppSwitcherItem *self)
{
    AppletAppSwitcherItemPrivate *priv = self->priv;
    priv->show_start_index = 0;
    priv->show_last_index =
        priv->app_windows_length > priv->page_size ? priv->page_size - 1 : priv->app_windows_length - 1;
}

AppletAppSwitcherItem *
applet_app_switcher_item_new (CdosApp *app, gboolean is_favorite, MetaWindow *meta_window)
{
    AppletAppSwitcherItem *self;
    self = g_object_new (APPLET_TYPE_APP_SWITCHER_ITEM, NULL);
    applet_app_switcher_item_setup (self, app, is_favorite, meta_window);
    return self;
}

void
applet_app_switcher_item_set_meta_window (AppletAppSwitcherItem *self, MetaWindow *meta_window)
{
    AppletAppSwitcherItemPrivate *priv = self->priv;
    priv->meta_window = meta_window;
    applet_app_switcher_item_refresh (self);
}

static void
applet_app_switcher_item_add_new_windows (AppletAppSwitcherItem *self, GSList *windows)
{
    AppletAppSwitcherItemPrivate *priv = self->priv;

    gint windows_length = g_slist_length (windows);

    // notice:此值为近似值，并不是该控件的实际值
    //按照默认的布局进行计算；
    // monitor_size = priv->monitor_width;
    // thumbnail_size = LIKELY_THUMBNAIL_WIDTH;

    gfloat first_loop, last_loop;

    first_loop = priv->thumbnails_space;

    if (windows_length < first_loop)
    {
        first_loop = windows_length;
    }
    // gint loop_step = (int)first_loop;
    // notice:将原来的默认显示3排，改为显示一排
    // last_loop = windows_length - next_loop > loop_step ? (int)next_loop + (int)loop_step : windows_length;
    last_loop = first_loop;
    //判断是否需要更换布局；
    if (windows_length > last_loop && priv->vertical_thumbs == FALSE)
    {
        priv->vertical_thumbs = TRUE;
        applet_app_switcher_item_on_thumbnails_layout_changed (self);
        return;
    }
    else if (windows_length <= last_loop && priv->vertical_thumbs == TRUE)
    {
        priv->vertical_thumbs = FALSE;
        applet_app_switcher_item_on_thumbnails_layout_changed (self);
        return;
    }

    applet_app_switcher_item_add_windows_loop (self, 0, windows_length, priv->app_container, windows, 1);
    if (windows_length >= first_loop && priv->vertical_thumbs == TRUE)
    {
        applet_app_switcher_item_refresh_vertical_list (self);
    }
}

static void
applet_app_switcher_item_add_windows_loop (AppletAppSwitcherItem *self,
                                           gint i,
                                           gint win_length,
                                           StWidget *actor,
                                           GSList *windows,
                                           gint container_num)
{
    AppletAppSwitcherItemPrivate *priv = self->priv;
    // gint windows_length = g_slist_length (windows);
    for (i = 0; i < win_length; i++)
    {
        MetaWindow *window = g_slist_nth_data (windows, i);
        AppletAppSwitcherItemAppThumbnail *thumb =
            applet_app_switcher_item_get_app_thumbnail_with_window (priv->app_thumbnails, window);

        if (thumb != NULL)
        {
            applet_window_thumbnail_is_favorite (thumb->thumbnail, priv->is_favorite_app);
        }
        else
        {
            AppletWindowThumbnail *thumbnail = applet_window_thumbnail_new (priv->app, priv->is_favorite_app, window);
            thumb = g_new0 (AppletAppSwitcherItemAppThumbnail, 1);
            thumb->meta_window = window;
            thumb->thumbnail = thumbnail;
            thumb->container_num = container_num;
            priv->app_thumbnails = g_slist_append (priv->app_thumbnails, thumb);
        }

        if (window == priv->meta_window && win_length > 1)
        {
            st_widget_add_style_class_name (thumb->thumbnail, "item-box-last-focused");
        } else {
            st_widget_remove_style_class_name (thumb->thumbnail, "item-box-last-focused");
        }


        gboolean no_thumbnail = priv->vertical_thumbs == TRUE ? TRUE : FALSE;
        applet_window_thumbnail_set_no_thumbnail (thumb->thumbnail, no_thumbnail);

        StWidget *thumb_actor = applet_window_thumbnail_get_actor (thumb->thumbnail);
        ClutterActor *parent = clutter_actor_get_parent ((ClutterActor *)thumb_actor);

        if (parent != CLUTTER_ACTOR (actor))
        {
            g_object_ref (thumb_actor);
            // clutter_container_remove_actor (CLUTTER_CONTAINER (parent), CLUTTER_ACTOR (thumb_actor));

            if (parent != NULL)
            {
                _remove_actor (parent, thumb_actor);
            }
            _add_actor (actor, thumb_actor);
            g_object_set (thumb_actor, 
                          "x-expand", TRUE, 
                          "x-align", CLUTTER_ACTOR_ALIGN_FILL, 
                          NULL);

            //记录列表控件序列
            if (g_slist_index (priv->thumbnail_actors, thumb_actor) > -1)
            {
                priv->thumbnail_actors = g_slist_remove (priv->thumbnail_actors, thumb_actor);
            }
            priv->thumbnail_actors = g_slist_append (priv->thumbnail_actors, thumb_actor);
        }
        applet_window_thumbnail_set_scale (thumb->thumbnail, 0);

        if (priv->show_window)
        {
            if (priv->show_window != window)
            {
                clutter_actor_hide (CLUTTER_ACTOR (thumb_actor));
            }
            else
            {
                clutter_actor_show (CLUTTER_ACTOR (thumb_actor));
            }
        }
        else
        {
            clutter_actor_show (CLUTTER_ACTOR (thumb_actor));
        }
    }
    clutter_actor_show (CLUTTER_ACTOR (actor));
}

static void
applet_app_switcher_item_remove_old_windows (AppletAppSwitcherItem *self, GSList *windows)
{
    AppletAppSwitcherItemPrivate *priv = self->priv;
    GSList *app_thumbnails = priv->app_thumbnails;
    while (app_thumbnails != NULL)
    {
        AppletAppSwitcherItemAppThumbnail *thumbnail = app_thumbnails->data;
        if (g_slist_find (windows, thumbnail->meta_window) == NULL)
        {
            StWidget *thumb_actor = applet_window_thumbnail_get_actor (thumbnail->thumbnail);
            g_clear_pointer ((ClutterActor **)&thumbnail->thumbnail, clutter_actor_destroy);
            thumbnail->thumbnail = NULL;
            priv->app_thumbnails = g_slist_remove (priv->app_thumbnails, thumbnail);
            priv->thumbnail_actors = g_slist_remove (priv->thumbnail_actors, thumb_actor);

            g_free (thumbnail);
        }
        app_thumbnails = app_thumbnails->next;
    }

    GList *children = clutter_actor_get_children (CLUTTER_ACTOR (priv->app_container));
    gint children_length = g_list_length (children);

    GList *children2 = clutter_actor_get_children (CLUTTER_ACTOR (priv->app_container2));
    gint children_length2 = g_list_length (children2);

    GList *children3 = clutter_actor_get_children (CLUTTER_ACTOR (priv->app_container3));
    gint children_length3 = g_list_length (children3);

    if (priv->vertical_thumbs == FALSE)
    {
        gfloat thumb_space = priv->thumbnails_space;
        if (children_length < thumb_space && children_length2 > 0)
        {
            ClutterActor *actor = g_list_nth_data (children2, 0);
            g_object_ref (actor);
            _remove_actor (priv->app_container2, actor);
            _add_actor (priv->app_container, actor);
            g_list_free (children);
            children = clutter_actor_get_children (CLUTTER_ACTOR (priv->app_container));
            children_length = g_list_length (children);
            g_list_free (children2);
            children2 = clutter_actor_get_children (CLUTTER_ACTOR (priv->app_container2));
            children_length2 = g_list_length (children2);
        }
        if (children_length2 <= 0)
        {
            clutter_actor_hide (CLUTTER_ACTOR (priv->app_container2));
        }

        if (children_length2 < thumb_space && children_length3 > 0)
        {
            ClutterActor *actor = g_list_nth_data (children3, 0);
            g_object_ref (actor);
            _remove_actor (priv->app_container3, actor);
            _add_actor (priv->app_container2, actor);
            g_list_free (children2);
            children2 = clutter_actor_get_children (CLUTTER_ACTOR (priv->app_container2));
            g_list_free (children3);
            children3 = clutter_actor_get_children (CLUTTER_ACTOR (priv->app_container3));
            children_length3 = g_list_length (children3);
        }

        if (children_length3 <= 0)
        {
            clutter_actor_hide (CLUTTER_ACTOR (priv->app_container3));
        }
    }
    g_list_free (children);
    g_list_free (children2);
    g_list_free (children3);
}

void
applet_app_switcher_item_set_is_favorite (AppletAppSwitcherItem *self, gboolean is_added)
{
    self->priv->is_favorite_app = is_added;
}

void
applet_app_switcher_item_refresh (AppletAppSwitcherItem *self)
{
    g_return_if_fail (APPLET_IS_APP_SWITCHER_ITEM (self));
    AppletAppSwitcherItemPrivate *priv = self->priv;

    CdosLayoutManager *layout_manager = cdos_layout_manager_get_default ();
    const MetaRectangle *primary_monitor = cdos_layout_manager_get_primary_monitor (layout_manager);
    if (!primary_monitor)
        return;
    priv->monitor_width = primary_monitor->width;
    priv->monitor_height = primary_monitor->height;
    priv->thumbnails_space = ceil (priv->monitor_width / LIKELY_THUMBNAIL_WIDTH);
    priv->page_size = (int)ceil (priv->monitor_height * 0.8 / LIKELY_THUMBNAIL_HEIGHT_V);
    priv->show_last_index = priv->page_size - 1;

    if (priv->meta_window_thumbnail != NULL && !applet_window_thumbnail_need_reflush (priv->meta_window_thumbnail))
    {
        g_clear_pointer ((ClutterActor **)&priv->meta_window_thumbnail, clutter_actor_destroy);
    }
    MetaWindow *thumbnail_meta_window = NULL;
    if (priv->meta_window_thumbnail)
    {
        thumbnail_meta_window = applet_window_thumbnail_get_meta_window (priv->meta_window_thumbnail);
    }
    if (priv->meta_window_thumbnail != NULL && thumbnail_meta_window == priv->meta_window)
    {
        applet_window_thumbnail_is_favorite (priv->meta_window_thumbnail, priv->is_favorite_app);
    }
    else
    {
        if (priv->meta_window_thumbnail != NULL)
        {
            g_clear_object (&priv->meta_window_thumbnail);
        }
        if (priv->is_favorite_app)
        {
            priv->meta_window_thumbnail =
                applet_window_thumbnail_new (priv->app, priv->is_favorite_app, priv->meta_window);
            StWidget *thumbnail_actor = applet_window_thumbnail_get_actor (priv->meta_window_thumbnail);
            _add_actor (priv->app_container, thumbnail_actor);
        }
    }
    GSList *windows = applet_app_switcher_item_get_app_windows (self);

    applet_app_switcher_item_remove_old_windows (self, windows);
    applet_app_switcher_item_add_new_windows (self, windows);
    g_slist_free (windows);
}

MetaWindow *
applet_app_switcher_item_get_meta_window (AppletAppSwitcherItem *self)
{
    return self->priv->meta_window;
}

CdosApp *
applet_app_switcher_item_get_app (AppletAppSwitcherItem *self)
{
    return self->priv->app;
}

StWidget *
applet_app_switcher_item_get_box (AppletAppSwitcherItem *self)
{
    return self->priv->box;
}

void
applet_app_switcher_item_show_one (AppletAppSwitcherItem *self, MetaWindow *meta_window)
{
    g_return_if_fail (APPLET_IS_APP_SWITCHER_ITEM (self));
    AppletAppSwitcherItemPrivate *priv = self->priv;
    if (priv->show_window != meta_window)
    {
        priv->show_window = meta_window;
        applet_app_switcher_item_refresh (self);
    }
}
