// Created by zhangtao on 16-4-16.

#include "applet-app-thumbnail-hover-menu.h"
#include "applet-window-list.h"
#include "cdos-ui-plugin.h"
#include "cdos-debug.h"


struct _AppletAppThumbnailHoverMenuPrivate
{
    gboolean is_favorite_app;
    gboolean should_open;
    gint hover_time;
    MetaWindow *meta_window;
    CdosApp *app;
    ClutterActor *source_actor;
    AppletAppSwitcherItem *app_switcher_item;
    AppletAppContextMenu *context_menu;
    gboolean is_keep_menu;

    guint menu_open_id;
    guint menu_close_id;
    guint actor_leave_id;
};

G_DEFINE_TYPE_WITH_PRIVATE (AppletAppThumbnailHoverMenu, applet_app_thumbnail_hover_menu, CDOS_TYPE_POPUP_MENU);

static void applet_app_thumbnail_hover_menu_dispose (GObject *object);
static void applet_app_thumbnail_hover_menu_finalize (GObject *object);
static void applet_app_thumbnail_hover_menu_setup (AppletAppThumbnailHoverMenu *self);
static gboolean applet_app_thumbnail_hover_menu_actor_on_enter (ClutterActor *actor,
                                                                ClutterEvent *event,
                                                                AppletAppThumbnailHoverMenu *self);
static gboolean applet_app_thumbnail_hover_menu_actor_on_leave (ClutterActor *actor,
                                                                ClutterEvent *event,
                                                                AppletAppThumbnailHoverMenu *self);
static gboolean applet_app_thumbnail_hover_menu_actor_on_button_release (ClutterActor *actor,
                                                                         ClutterEvent *event,
                                                                         AppletAppThumbnailHoverMenu *self);
static gboolean menu_main_loop_hover_close (AppletAppThumbnailHoverMenu *self);
static gboolean menu_main_loop_hover_open (AppletAppThumbnailHoverMenu *self);

static void
applet_app_thumbnail_hover_menu_init (AppletAppThumbnailHoverMenu *self)
{
    AppletAppThumbnailHoverMenuPrivate *priv;
    self->priv = applet_app_thumbnail_hover_menu_get_instance_private (self);
    priv = self->priv;
    priv->context_menu = NULL;
    priv->is_keep_menu = TRUE;
    priv->meta_window = NULL;
    priv->is_favorite_app = FALSE;
    //控制弹窗是否需要打开或关闭；
    priv->should_open = TRUE;
}

static void
applet_app_thumbnail_hover_menu_class_init (AppletAppThumbnailHoverMenuClass *klass)
{
    GObjectClass *gobject_class = G_OBJECT_CLASS (klass);

    gobject_class->dispose = applet_app_thumbnail_hover_menu_dispose;
    gobject_class->finalize = applet_app_thumbnail_hover_menu_finalize;
}

static void
applet_app_thumbnail_hover_menu_dispose (GObject *object)
{
    AppletAppThumbnailHoverMenu *self = APPLET_APP_THUMBNAIL_HOVER_MENU (object);
    AppletAppThumbnailHoverMenuPrivate *priv = self->priv;

    applet_app_thumbnail_hover_menu_close (self, TRUE);

    if (priv->menu_open_id)
    {
        g_source_remove (priv->menu_open_id);
        priv->menu_open_id = 0;
    }
    if (priv->menu_close_id)
    {
        g_source_remove (priv->menu_close_id);
        priv->menu_close_id = 0;
    }
    if (priv->actor_leave_id)
    {
        g_source_remove (priv->actor_leave_id);
        priv->actor_leave_id = 0;
    }

    G_OBJECT_CLASS (applet_app_thumbnail_hover_menu_parent_class)->dispose (object);
}

static void
applet_app_thumbnail_hover_menu_finalize (GObject *object)
{
    // AppletAppThumbnailHoverMenu *self = APPLET_APP_THUMBNAIL_HOVER_MENU (object);
    // AppletAppThumbnailHoverMenuPrivate *priv = self->priv;

    G_OBJECT_CLASS (applet_app_thumbnail_hover_menu_parent_class)->finalize (object);
}

static void
applet_app_thumbnail_hover_menu_setup (AppletAppThumbnailHoverMenu *self)
{
    AppletAppThumbnailHoverMenuPrivate *priv = self->priv;

    StWidget *actor = ST_WIDGET (CDOS_POPUP_MENU (self)->actor);
    StWidget *box = (StWidget *)cdos_popup_menu_base_get_box (CDOS_POPUP_MENU_BASE (self));
    st_widget_set_style_class_name ((StWidget *)self, "window-list-hover-menu");

    priv->app_switcher_item = applet_app_switcher_item_new (priv->app, priv->is_favorite_app, priv->meta_window);

    CdosPopupMenuBase *menu = CDOS_POPUP_MENU_BASE (self);
    cdos_popup_menu_base_add_menu_item (menu, CDOS_POPUP_BASE_MENU_ITEM (priv->app_switcher_item), -1);
    clutter_actor_hide (CLUTTER_ACTOR (actor));
    clutter_actor_hide (CLUTTER_ACTOR (box));

    ClutterActor *ui_group = cdos_ui_plugin_get_ui_group ();
    clutter_actor_add_child (ui_group, CLUTTER_ACTOR (actor));

    g_signal_connect (actor, "enter-event", G_CALLBACK (applet_app_thumbnail_hover_menu_actor_on_enter), self);
    g_signal_connect (actor, "leave-event", G_CALLBACK (applet_app_thumbnail_hover_menu_actor_on_leave), self);
    g_signal_connect (actor,
                      "button-release-event",
                      G_CALLBACK (applet_app_thumbnail_hover_menu_actor_on_button_release),
                      self);

    gint hover_time = 250;
    priv->hover_time = hover_time;
}

static gboolean
applet_app_thumbnail_hover_menu_actor_on_enter (ClutterActor *actor,
                                                ClutterEvent *event,
                                                AppletAppThumbnailHoverMenu *self)
{
    AppletAppThumbnailHoverMenuPrivate *priv = self->priv;
    priv->should_open = TRUE;
    g_timeout_add (priv->hover_time, (GSourceFunc)menu_main_loop_hover_open, self);
    return FALSE;
}

static gboolean
applet_app_thumbnail_hover_menu_actor_on_leave (ClutterActor *actor,
                                                ClutterEvent *event,
                                                AppletAppThumbnailHoverMenu *self)
{
    AppletAppThumbnailHoverMenuPrivate *priv = self->priv;
    priv->should_open = FALSE;
    priv->actor_leave_id = g_timeout_add (priv->hover_time, (GSourceFunc)menu_main_loop_hover_close, self);
    return FALSE;
}

static gboolean
applet_app_thumbnail_hover_menu_actor_on_button_release (ClutterActor *actor,
                                                         ClutterEvent *event,
                                                         AppletAppThumbnailHoverMenu *self)
{
    AppletAppThumbnailHoverMenuPrivate *priv = self->priv;
    gboolean on_click_thumbs = FALSE;

    if (priv->source_actor == (gpointer)actor && on_click_thumbs)
    {
        return FALSE;
    }
    priv->should_open = FALSE;

    g_timeout_add (priv->hover_time, (GSourceFunc)menu_main_loop_hover_close, self);
    return FALSE;
}

static gboolean
menu_main_loop_hover_open (AppletAppThumbnailHoverMenu *self)
{
    AppletAppThumbnailHoverMenuPrivate *priv = self->priv;
    if (priv->should_open)
    {
        // the source actor may be changed;
        CdosBoxPointer *box_pointer = cdos_popup_menu_get_box_pointer ((CdosPopupMenu *)self);
        gfloat arrow = cdos_popup_menu_get_arrow_alignment ((CdosPopupMenu *)self);
        ClutterActor *source_actor = cdos_popup_menu_base_get_source_actor ((CdosPopupMenuBase *)self);
        cdos_box_pointer_set_position (box_pointer, source_actor, arrow);
        cdos_box_pointer_set_source_alignment (box_pointer, 0.5);
        applet_app_thumbnail_hover_menu_open (self, TRUE);
    }

    priv->menu_open_id = 0;
    return FALSE;
}

static gboolean
menu_main_loop_hover_close (AppletAppThumbnailHoverMenu *self)
{
    AppletAppThumbnailHoverMenuPrivate *priv = self->priv;
    if (!priv->should_open)
        applet_app_thumbnail_hover_menu_close (self, TRUE);

    priv->actor_leave_id = 0;
    priv->menu_close_id = 0;

    return FALSE;
}

AppletAppThumbnailHoverMenu *
applet_app_thumbnail_hover_menu_new (CdosApp *app, gboolean is_favorite, ClutterActor *source_actor, StSide arrow_side)
{
    AppletAppThumbnailHoverMenu *self;

    self = g_object_new (APPLET_TYPE_APP_THUMBNAIL_HOVER_MENU,
                         "source-actor", source_actor,
                         "is-popup", TRUE,
                         "arrow-side", arrow_side,
                         "arrow-alignment", 0.5,
                         NULL);
    AppletAppThumbnailHoverMenuPrivate *priv = self->priv;
    priv->app = app;
    priv->is_favorite_app = is_favorite;
    priv->source_actor = source_actor;

    applet_app_thumbnail_hover_menu_setup (self);

    return self;
}

void
applet_app_thumbnail_hover_menu_open (AppletAppThumbnailHoverMenu *self, gboolean animate)
{
    g_return_if_fail (APPLET_IS_APP_THUMBNAIL_HOVER_MENU (self));
    AppletAppThumbnailHoverMenuPrivate *priv = self->priv;
    if (!priv->should_open)
    {
        priv->should_open = TRUE;
    }
    //applet_app_switcher_item_refresh (APPLET_APP_SWITCHER_ITEM (priv->app_switcher_item));
    ClutterActor *actor = CLUTTER_ACTOR (priv->app_switcher_item);
    StBoxLayout *box = cdos_popup_menu_base_get_box (CDOS_POPUP_MENU_BASE (self));
    clutter_actor_show (actor);
    clutter_actor_show (CLUTTER_ACTOR (box));
    cdos_popup_menu_open (CDOS_POPUP_MENU_BASE (self), animate);
}

void
applet_app_thumbnail_hover_menu_close (AppletAppThumbnailHoverMenu *self, gboolean animate)
{
    g_return_if_fail (APPLET_IS_APP_THUMBNAIL_HOVER_MENU (self));
    AppletAppThumbnailHoverMenuPrivate *priv = self->priv;
    if (priv->should_open)
    {
        priv->should_open = FALSE;
    }
    cdos_popup_menu_close (CDOS_POPUP_MENU_BASE (self), animate);
}

void
applet_app_thumbnail_hover_menu_hover_close (AppletAppThumbnailHoverMenu *self)
{
    g_return_if_fail (APPLET_IS_APP_THUMBNAIL_HOVER_MENU (self));
    AppletAppThumbnailHoverMenuPrivate *priv = self->priv;
    if (priv->menu_close_id)
        g_source_remove (priv->menu_close_id);
    if (priv->should_open)
        return;
    priv->menu_close_id = g_timeout_add (priv->hover_time, (GSourceFunc)menu_main_loop_hover_close, self);
    // menu_main_loop_hover_close (self);
}

void
applet_app_thumbnail_hover_menu_hover_open (AppletAppThumbnailHoverMenu *self)
{
    g_return_if_fail (APPLET_IS_APP_THUMBNAIL_HOVER_MENU (self));
    AppletAppThumbnailHoverMenuPrivate *priv = self->priv;
    g_clear_handle_id (&priv->menu_open_id, g_source_remove);
    if (!priv->should_open)
        return;
    priv->menu_open_id = g_timeout_add (priv->hover_time, (GSourceFunc)menu_main_loop_hover_open, self);
    // menu_main_loop_hover_open (self);
}

void
applet_app_thumbnail_hover_menu_set_meta_window (AppletAppThumbnailHoverMenu *self, MetaWindow *meta_window)
{
    g_return_if_fail (APPLET_IS_APP_THUMBNAIL_HOVER_MENU (self));
    AppletAppThumbnailHoverMenuPrivate *priv = self->priv;
    priv->meta_window = meta_window;
    applet_app_switcher_item_set_meta_window (priv->app_switcher_item, meta_window);
}

MetaWindow *
applet_app_thumbnail_hover_menu_get_meta_window (AppletAppThumbnailHoverMenu *self)
{
    g_return_val_if_fail (APPLET_IS_APP_THUMBNAIL_HOVER_MENU (self), NULL);
    AppletAppThumbnailHoverMenuPrivate *priv = self->priv;
    return priv->meta_window;
}

AppletAppSwitcherItem *
applet_app_thumbnail_hover_menu_get_app_switcher_item (AppletAppThumbnailHoverMenu *self)
{
    g_return_val_if_fail (APPLET_IS_APP_THUMBNAIL_HOVER_MENU (self), NULL);
    AppletAppThumbnailHoverMenuPrivate *priv = self->priv;
    return priv->app_switcher_item;
}

void
applet_app_thumbnail_hover_menu_set_should_open (AppletAppThumbnailHoverMenu *self, gboolean should_open)
{
    g_return_if_fail (APPLET_IS_APP_THUMBNAIL_HOVER_MENU (self));
    AppletAppThumbnailHoverMenuPrivate *priv = self->priv;
    priv->should_open = should_open;
}

void
applet_app_thumbnail_hover_menu_refresh (AppletAppThumbnailHoverMenu *self)
{
    g_return_if_fail (APPLET_IS_APP_THUMBNAIL_HOVER_MENU (self));
    AppletAppThumbnailHoverMenuPrivate *priv = self->priv;
    if (priv->is_keep_menu == FALSE)
    {
        priv->should_open = FALSE;
    }
    else
    {
        priv->should_open = TRUE;
    }
    menu_main_loop_hover_close (self);
    applet_app_switcher_item_refresh (APPLET_APP_SWITCHER_ITEM (priv->app_switcher_item));
}

void
applet_app_thumbnail_hover_menu_set_context_menu (AppletAppThumbnailHoverMenu *self, AppletAppContextMenu *context_menu)
{
    g_return_if_fail (APPLET_IS_APP_THUMBNAIL_HOVER_MENU (self));
    AppletAppThumbnailHoverMenuPrivate *priv = self->priv;
    priv->context_menu = context_menu;
}

void
applet_app_thumbnail_hover_menu_set_is_keep_menu (AppletAppThumbnailHoverMenu *self, gboolean is_keep_menu)
{
    g_return_if_fail (APPLET_IS_APP_THUMBNAIL_HOVER_MENU (self));
    AppletAppThumbnailHoverMenuPrivate *priv = self->priv;
    priv->is_keep_menu = is_keep_menu;
}

void
applet_app_thumbnail_hover_menu_toggle (AppletAppThumbnailHoverMenu *self)
{
    g_return_if_fail (APPLET_IS_APP_THUMBNAIL_HOVER_MENU (self));
    AppletAppThumbnailHoverMenuPrivate *priv = self->priv;

    priv->should_open = !priv->should_open;

    if (priv->should_open)
    {
        applet_app_thumbnail_hover_menu_hover_open (self);
    }
    else
    {
        applet_app_thumbnail_hover_menu_hover_close (self);
    }
}
