// Created by zhangtao on 16-4-16.
#include "cdos-global.h"
#include "applet-button-box.h"

enum
{
    CONDENSE_CHANGED,
    LAST_SIGNAL
};

struct _AppletButtonBoxPrivate
{
    // if the buttons is too much, they will be condensed into one;
    gboolean buttons_condense;
    GList *buttons;
    MetaWindow *last_focused_window;

    // if condense_button is NULL, show all;
    AppletIconLabelButton *condense_button;
};

G_DEFINE_TYPE_WITH_PRIVATE (AppletButtonBox, applet_button_box, ST_TYPE_WIDGET);

static gint applet_button_box_signals[LAST_SIGNAL] = {0};

static void applet_button_box_condense (AppletButtonBox *self);
static void applet_button_box_scatter (AppletButtonBox *self);

static void
applet_button_box_dispose (GObject *object)
{
    AppletButtonBox *self = APPLET_BUTTON_BOX (object);
    AppletButtonBoxPrivate *priv = self->priv;

    if (priv->buttons)
    {
        g_list_free (priv->buttons);
        priv->buttons = NULL;
    }

    G_OBJECT_CLASS (applet_button_box_parent_class)->dispose (object);
}

static void
applet_button_box_finalize (GObject *object)
{
    // AppletButtonBox *self = APPLET_BUTTON_BOX (object);
    // AppletButtonBoxPrivate *priv = self->priv;

    G_OBJECT_CLASS (applet_button_box_parent_class)->finalize (object);
}

static void
applet_button_box_init (AppletButtonBox *self)
{
    AppletButtonBoxPrivate *priv;
    self->priv = applet_button_box_get_instance_private (self);

    priv = self->priv;
    priv->last_focused_window = NULL;
    priv->buttons_condense = FALSE;
    priv->condense_button = NULL;

    ClutterLayoutManager *box_layout = clutter_box_layout_new ();
    clutter_actor_set_layout_manager ((ClutterActor *)self, box_layout);
}

static void
applet_button_box_class_init (AppletButtonBoxClass *klass)
{
    GObjectClass *gobject_class = G_OBJECT_CLASS (klass);

    gobject_class->dispose = applet_button_box_dispose;
    gobject_class->finalize = applet_button_box_finalize;

    applet_button_box_signals[CONDENSE_CHANGED] = g_signal_new ("condense-changed",
                                                                G_TYPE_FROM_CLASS (gobject_class),
                                                                G_SIGNAL_RUN_LAST,
                                                                0,
                                                                NULL,
                                                                NULL,
                                                                g_cclosure_marshal_VOID__BOOLEAN,
                                                                G_TYPE_NONE,
                                                                1,
                                                                G_TYPE_BOOLEAN);
}

static void
applet_button_box_update_window_num (AppletButtonBox *self)
{
    AppletButtonBoxPrivate *priv = self->priv;
    if (priv->condense_button)
    {
        gint window_len = g_list_length (priv->buttons);
        applet_icon_label_button_set_window_num (priv->condense_button, window_len);
        applet_icon_label_button_show_window_num (priv->condense_button, TRUE);
    }
}

static gboolean
applet_button_box_window_has_focus (AppletButtonBox *self)
{
    AppletButtonBoxPrivate *priv = self->priv;
    MetaWindow *window = priv->last_focused_window;
    if (window == NULL)
    {
        return FALSE;
    }

    if (meta_window_appears_focused (window) || meta_window_has_focus (window))
    {
        return TRUE;
    }
    return FALSE;
}

static void
applet_button_box_focus_window_changed (AppletButtonBox *self)
{
    AppletButtonBoxPrivate *priv = self->priv;
    if (priv->buttons_condense)
    {
        applet_button_box_condense (self);
    }
    else
    {
        applet_button_box_scatter (self);
    }
}

static void
applet_button_box_condense (AppletButtonBox *self)
{
    AppletButtonBoxPrivate *priv = self->priv;
    priv->buttons_condense = TRUE;
    GList *buttons = priv->buttons;
    gboolean has_focus = applet_button_box_window_has_focus (self);

    while (buttons != NULL)
    {
        AppletWindowButton *button = buttons->data;
        MetaWindow *window = applet_icon_label_button_get_window (APPLET_ICON_LABEL_BUTTON (button));
        const gchar *pseudo_class = NULL;
        if (window == priv->last_focused_window)
        {
            pseudo_class = has_focus ? "focus" : NULL;
            priv->condense_button = APPLET_ICON_LABEL_BUTTON (button);
            applet_button_box_update_window_num (self);
            applet_icon_label_button_show (APPLET_ICON_LABEL_BUTTON (button), TRUE);
        }
        else
        {
            applet_icon_label_button_show_window_num (APPLET_ICON_LABEL_BUTTON (button), FALSE);
            applet_icon_label_button_hide (APPLET_ICON_LABEL_BUTTON (button), TRUE);
        }
        st_widget_set_style_pseudo_class (ST_WIDGET (button), pseudo_class);
        buttons = buttons->next;
    }
}

static void
applet_button_box_scatter (AppletButtonBox *self)
{
    AppletButtonBoxPrivate *priv = self->priv;
    priv->buttons_condense = FALSE;
    GList *buttons = priv->buttons;
    priv->condense_button = NULL;
    gboolean has_focus = applet_button_box_window_has_focus (self);
    while (buttons != NULL)
    {
        AppletWindowButton *button = buttons->data;
        MetaWindow *window = applet_icon_label_button_get_window (APPLET_ICON_LABEL_BUTTON (button));
        const gchar *pseudo_class = NULL;
        if (window == priv->last_focused_window)
        {
            pseudo_class = has_focus ? "focus" : NULL;
        }
        st_widget_set_style_pseudo_class (ST_WIDGET (button), pseudo_class);
        applet_icon_label_button_show_window_num (APPLET_ICON_LABEL_BUTTON (button), FALSE);
        applet_icon_label_button_show (APPLET_ICON_LABEL_BUTTON (button), TRUE);
        buttons = buttons->next;
    }
}

static void
applet_button_box_remove_button (AppletButtonBox *self, AppletWindowButton *button)
{
    AppletButtonBoxPrivate *priv = self->priv;

    if (priv->condense_button == (AppletIconLabelButton *)button)
    {
        priv->condense_button = NULL;
        MetaDisplay *display = cdos_global_get_display (cdos_global_get ());
        MetaWindow *focus_window = meta_display_get_focus_window (display);
        for (GList *iter = priv->buttons; iter; iter = iter->next)
        {
            AppletIconLabelButton *button = iter->data;
            MetaWindow *window = applet_icon_label_button_get_window (button);
            if (window == focus_window)
            {
                priv->last_focused_window = window;
                priv->condense_button = button;
                clutter_actor_show ((ClutterActor *)button);
            }
            else
                clutter_actor_hide ((ClutterActor *)button);
        }
        if (!priv->condense_button && priv->buttons)
            priv->condense_button = priv->buttons->data;
    }
    applet_button_box_update_window_num (self);
}

static void
button_on_destroy (ClutterActor *actor, gpointer user_data)
{
    AppletButtonBox *self = APPLET_BUTTON_BOX (user_data);
    AppletButtonBoxPrivate *priv = self->priv;

    priv->buttons = g_list_remove (priv->buttons, actor);
    gint buttons_len = g_list_length (priv->buttons);

    if (priv->buttons_condense)
    {
        if (buttons_len == 1)
            applet_button_box_set_condense (self, FALSE);
        else if (buttons_len > 1)
            applet_button_box_remove_button (self, APPLET_WINDOW_BUTTON (actor));
    }
}

AppletButtonBox *
applet_button_box_new (void)
{
    AppletButtonBox *self;
    self = g_object_new (APPLET_TYPE_BUTTON_BOX, "style-class", "window-list-button-box", NULL);
    return self;
}

void
applet_button_box_add_button (AppletButtonBox *self, AppletWindowButton *button)
{
    g_return_if_fail (APPLET_IS_BUTTON_BOX (self));
    g_return_if_fail (APPLET_WINDOW_BUTTON (button));
    AppletButtonBoxPrivate *priv = self->priv;
    priv->buttons = g_list_append (priv->buttons, button);
    clutter_actor_add_child (CLUTTER_ACTOR (self), CLUTTER_ACTOR (button));
    applet_button_box_update_window_num (self);
    g_signal_connect (button, "destroy", G_CALLBACK (button_on_destroy), self);
}

void
applet_button_box_clear (AppletButtonBox *self)
{
    g_return_if_fail (APPLET_IS_BUTTON_BOX (self));
    clutter_actor_remove_all_children (CLUTTER_ACTOR (self));
}

void
applet_button_box_set_condense (AppletButtonBox *self, gboolean condense)
{
    g_return_if_fail (APPLET_IS_BUTTON_BOX (self));
    AppletButtonBoxPrivate *priv = self->priv;
    if (priv->buttons_condense == condense)
    {
        return;
    }
    if (condense)
    {
        applet_button_box_condense (self);
    }
    else
    {
        applet_button_box_scatter (self);
    }
    g_signal_emit (self, applet_button_box_signals[CONDENSE_CHANGED], 0, priv->buttons_condense);
}

gboolean
applet_button_box_get_condense (AppletButtonBox *self)
{
    g_return_val_if_fail (APPLET_IS_BUTTON_BOX (self), FALSE);
    AppletButtonBoxPrivate *priv = self->priv;
    return priv->buttons_condense;
}

void
applet_button_box_set_last_focused_window (AppletButtonBox *self, MetaWindow *window)
{
    g_return_if_fail (APPLET_IS_BUTTON_BOX (self));
    AppletButtonBoxPrivate *priv = self->priv;

    priv->last_focused_window = window;
    applet_button_box_focus_window_changed (self);
}

GList *
applet_button_box_get_buttons (AppletButtonBox *self)
{
    AppletButtonBoxPrivate *priv = self->priv;
    return priv->buttons;
}
