// Created by zhangtao on 16-4-16.

#include <math.h>
#include "applet-icon-label-button.h"
#include "applet-window-list.h"
#include "cdos-applet.h"
#include "cdos-tooltip.h"
#include "cdos-transition.h"
#include "applet-panel-launchers-manager.h"

struct _AppletIconLabelButtonPrivate
{
    StTextureCache *texture_cache;
    ClutterActor *icon;
    gdouble icon_size;
    gfloat spacing;
    gboolean icon_sinked;

    StWidget *label;
    gint app_window_num;
    gboolean show_window_num;

    CdosApp *app;
    gboolean is_favorite;

    MetaWindow *window;
    CdosTooltip *tooltip;
    gulong icon_theme_changed_id;
};

enum
{
    PROP_0,
    PROP_IS_FAVORITE,
    PROP_APP
};

G_DEFINE_ABSTRACT_TYPE_WITH_PRIVATE (AppletIconLabelButton, applet_icon_label_button, ST_TYPE_WIDGET);

static void applet_icon_label_button_icon_sink (AppletIconLabelButton *self, gint size);
static void applet_icon_label_button_icon_recovery (AppletIconLabelButton *self);

static void applet_icon_label_button_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec);
static void applet_icon_label_button_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec);

static void applet_icon_label_button_set_app (AppletIconLabelButton *self, CdosApp *app);

static void
applet_icon_label_button_style_changed (StWidget *widget)
{
    AppletIconLabelButton *self = APPLET_ICON_LABEL_BUTTON (widget);
    AppletIconLabelButtonPrivate *priv = self->priv;

    StThemeNode *theme_node = st_widget_get_theme_node (ST_WIDGET (self));

    gfloat icon_size = 0.5 + st_theme_node_get_length (theme_node, "icon-size");
    if (priv->icon_size != icon_size)
    {
        clutter_actor_set_size (priv->icon, icon_size, icon_size);
        priv->icon_size = icon_size;
    }

    priv->spacing = st_theme_node_get_length (theme_node, "spacing");

    ST_WIDGET_CLASS (applet_icon_label_button_parent_class)->style_changed (widget);
}

static gboolean
button_on_press_event (ClutterActor *actor, ClutterEvent *event)
{
    AppletIconLabelButton *self = APPLET_ICON_LABEL_BUTTON (actor);
    // AppletIconLabelButtonPrivate *priv = self->priv;

    guint32 button = clutter_event_get_button (event);
    if (button == 1 || button == 2)
    {
        applet_icon_label_button_icon_sink (self, 2);
        st_widget_add_style_pseudo_class (ST_WIDGET (actor), "click");
    }
    return FALSE;
}

static gboolean
button_on_release_event (ClutterActor *actor, ClutterEvent *event)
{
    applet_icon_label_button_icon_recovery (APPLET_ICON_LABEL_BUTTON (actor));
    st_widget_remove_style_pseudo_class (ST_WIDGET (actor), "click");

    return FALSE;
}

static gboolean
button_on_leave_event (ClutterActor *actor, ClutterEvent *event)
{
    applet_icon_label_button_icon_recovery (APPLET_ICON_LABEL_BUTTON (actor));
    st_widget_remove_style_pseudo_class (ST_WIDGET (actor), "click");

    return FALSE;
}

static void
applet_icon_label_button_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
{
    AppletIconLabelButton *self = APPLET_ICON_LABEL_BUTTON (object);
    AppletIconLabelButtonPrivate *priv = self->priv;
    CdosApp *app;
    switch (prop_id)
    {
        case PROP_APP:
            app = g_value_get_object (value);
            applet_icon_label_button_set_app (self, app);
            break;
        case PROP_IS_FAVORITE:
            priv->is_favorite = g_value_get_boolean (value);
            break;
        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
            break;
    }
}

static void
applet_icon_label_button_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec)
{
    AppletIconLabelButton *self = APPLET_ICON_LABEL_BUTTON (object);
    AppletIconLabelButtonPrivate *priv = self->priv;
    switch (prop_id)
    {
        case PROP_APP:
            g_value_set_object (value, priv->app);
            break;
        case PROP_IS_FAVORITE:
            g_value_set_boolean (value, priv->is_favorite);
            break;
        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
            break;
    }
}

static void
applet_icon_label_button_dispose (GObject *object)
{
    AppletIconLabelButton *self = APPLET_ICON_LABEL_BUTTON (object);
    AppletIconLabelButtonPrivate *priv = self->priv;

    if (priv->window)
    {
        g_signal_handlers_disconnect_by_data (priv->window, self);
        priv->window = NULL;
    }
    if (priv->icon_theme_changed_id)
    {
        g_signal_handler_disconnect (priv->texture_cache, priv->icon_theme_changed_id);
        priv->icon_theme_changed_id = 0;
    }

    G_OBJECT_CLASS (applet_icon_label_button_parent_class)->dispose (object);
}

static void
applet_icon_label_button_finalize (GObject *object)
{
    // AppletIconLabelButton *self = APPLET_ICON_LABEL_BUTTON (object);
    // AppletIconLabelButtonPrivate *priv = self->priv;

    G_OBJECT_CLASS (applet_icon_label_button_parent_class)->finalize (object);
}

static void
applet_icon_label_button_get_preferred_width (ClutterActor *actor,
                                      gfloat for_height,
                                      gfloat *min_width_p,
                                      gfloat *natural_width_p)
{
    AppletIconLabelButton *self = APPLET_ICON_LABEL_BUTTON (actor);
    AppletIconLabelButtonPrivate *priv = self->priv;

    StThemeNode *theme_node = st_widget_get_theme_node ((StWidget *)actor);
    st_theme_node_adjust_for_height (theme_node, &for_height);

    if (min_width_p)
        *min_width_p = (float)priv->icon_size;
    if (natural_width_p)
        *natural_width_p = (float)(priv->icon_size + priv->spacing * 2);

    st_theme_node_adjust_preferred_width (theme_node, min_width_p, natural_width_p);
}

static void
applet_icon_label_button_get_preferred_height (ClutterActor *actor,
                                       gfloat for_width,
                                       gfloat *min_height_p,
                                       gfloat *natural_height_p)
{
    AppletIconLabelButton *self = APPLET_ICON_LABEL_BUTTON (actor);
    AppletIconLabelButtonPrivate *priv = self->priv;
    StThemeNode *theme_node = st_widget_get_theme_node ((StWidget *)actor);
    st_theme_node_adjust_for_width (theme_node, &for_width);

    if (min_height_p)
        *min_height_p = (float)priv->icon_size;
    if (natural_height_p)
        *natural_height_p = (float)(priv->icon_size + priv->spacing * 2);
    st_theme_node_adjust_preferred_height (theme_node, min_height_p, natural_height_p);
}

static void
applet_icon_label_button_allocate (ClutterActor *actor,
                                   const ClutterActorBox *box,
                                   ClutterAllocationFlags flags)
{
    AppletIconLabelButton *self = APPLET_ICON_LABEL_BUTTON (actor);
    AppletIconLabelButtonPrivate *priv = self->priv;

    StThemeNode *theme_node = st_widget_get_theme_node ((StWidget *)actor);

    clutter_actor_set_allocation (actor, box, flags);

    ClutterActorBox content_box, child_box = {0};
    st_theme_node_get_content_box (theme_node, box, &content_box);

    gfloat avail_height, label_height;
    gfloat spacing = 0;
    ClutterActor *label;

    avail_height = content_box.y2 - content_box.y1;

    if (clutter_actor_is_visible (priv->icon))
    {
        child_box.x1 = content_box.x1;
        child_box.x2 = child_box.x1 + priv->icon_size;
        child_box.y1 = content_box.y1 + (avail_height - priv->icon_size) / 2;
        child_box.y2 = child_box.y1 + priv->icon_size;

        clutter_actor_allocate (priv->icon, &child_box, flags);
    }
    label = CLUTTER_ACTOR (priv->label);
    if (clutter_actor_is_visible (label))
    {
        if (clutter_actor_is_visible (priv->icon))
            spacing = priv->spacing;
        child_box.x1 = (int)(0.5 + child_box.x2 + spacing);
        child_box.x2 = content_box.x2;
        clutter_actor_get_preferred_height (label, child_box.x2 - child_box.x1, NULL, &label_height);
        child_box.y1 = (int)(0.5 + content_box.y1 + (avail_height - label_height) / 2);
        child_box.y2 = child_box.y1 + label_height;

        clutter_actor_allocate (label, &child_box, flags);
    }
}

static void
applet_icon_label_button_init (AppletIconLabelButton *self)
{
    AppletIconLabelButtonPrivate *priv;
    self->priv = applet_icon_label_button_get_instance_private (self);
    priv = self->priv;
    priv->app = NULL;
    priv->window = NULL;
    priv->is_favorite = FALSE;
    priv->app_window_num = 0;
    priv->show_window_num = FALSE;

    priv->icon_size = applet_window_list_get_panel_height ();
    priv->spacing = 0;

    st_widget_set_can_focus (ST_WIDGET (self), TRUE);
    st_widget_set_track_hover (ST_WIDGET (self), TRUE);
    clutter_actor_set_reactive (CLUTTER_ACTOR (self), TRUE);

    ClutterLayoutManager *box_layout = clutter_box_layout_new ();
    clutter_actor_set_layout_manager ((ClutterActor *)self, box_layout);

    priv->texture_cache = st_texture_cache_get_default ();

    priv->label = st_label_new (NULL);
    st_widget_set_style_class_name (priv->label, "window-list-button-label");
    clutter_actor_add_child ((ClutterActor *)self, CLUTTER_ACTOR (priv->label));

    g_signal_connect_swapped (self, "button-press-event", G_CALLBACK (button_on_press_event), self);
    g_signal_connect_swapped (self, "button-release-event", G_CALLBACK (button_on_release_event), self);
    g_signal_connect_swapped (self, "leave-event", G_CALLBACK (button_on_leave_event), self);
    priv->tooltip = cdos_tooltip_new (CLUTTER_ACTOR (self), "");
    applet_icon_label_button_enable_tooltip (self, FALSE);
}

static void
applet_icon_label_button_class_init (AppletIconLabelButtonClass *klass)
{
    GParamSpec *pspec;
    GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
    StWidgetClass *widget_class = ST_WIDGET_CLASS (klass);

    gobject_class->dispose = applet_icon_label_button_dispose;
    gobject_class->finalize = applet_icon_label_button_finalize;
    gobject_class->set_property = applet_icon_label_button_set_property;
    gobject_class->get_property = applet_icon_label_button_get_property;

    widget_class->style_changed = applet_icon_label_button_style_changed;

    ClutterActorClass *actor_class = CLUTTER_ACTOR_CLASS (klass);
    actor_class->allocate = applet_icon_label_button_allocate;
    actor_class->get_preferred_width = applet_icon_label_button_get_preferred_width;
    actor_class->get_preferred_height = applet_icon_label_button_get_preferred_height;

    pspec = g_param_spec_boolean ("is-favorite",
                                  "Is Favorite",
                                  "The app is favorite app",
                                  FALSE,
                                  G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
    g_object_class_install_property (gobject_class, PROP_IS_FAVORITE, pspec);

    pspec = g_param_spec_object ("app",
                                 "App",
                                 "The app of this button",
                                 CDOS_TYPE_APP,
                                 G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
    g_object_class_install_property (gobject_class, PROP_APP, pspec);
}

static void
on_icon_theme_changed (StTextureCache *texture_cache,
                       AppletIconLabelButton *self)
{
    AppletIconLabelButtonPrivate *priv = self->priv;
    if (!priv->app)
        return;

    clutter_actor_destroy (priv->icon);
    priv->icon = cdos_app_create_icon_texture (priv->app, (int)priv->icon_size);
    clutter_actor_insert_child_below ((ClutterActor *)self, priv->icon, NULL);
}

static void
update_icon_label (AppletIconLabelButton *self)
{
    AppletIconLabelButtonPrivate *priv = self->priv;
    gchar *app_name = NULL;
    if (priv->is_favorite)
    {
        app_name = applet_panel_launchers_get_display_name (priv->app);
        priv->icon = applet_panel_launchers_create_icon_texture (priv->app, (int)priv->icon_size);
    }
    if (!app_name)
        app_name = g_strdup (cdos_app_get_name (priv->app));
    if (!priv->icon)
        priv->icon = cdos_app_create_icon_texture (priv->app, (int)priv->icon_size);
    clutter_actor_insert_child_below ((ClutterActor *)self, priv->icon, NULL);
    st_label_set_text (ST_LABEL (priv->label), app_name);
    g_free (app_name);
}

static void
applet_icon_label_button_set_app (AppletIconLabelButton *self, CdosApp *app)
{
    AppletIconLabelButtonPrivate *priv = self->priv;
    if (!app)
        return;

    priv->app = app;
    update_icon_label (self);
    if (!priv->icon_theme_changed_id)
    {
        priv->icon_theme_changed_id = g_signal_connect (priv->texture_cache,
                                                        "icon-theme-changed",
                                                        G_CALLBACK (on_icon_theme_changed),
                                                        self);
    }
}

static void
applet_icon_label_button_update_label (AppletIconLabelButton *self)
{
    AppletIconLabelButtonPrivate *priv = self->priv;
    const gchar *title = NULL; 
    gchar *app_name = NULL;

    if (!priv->app)
        return;

    if (!priv->window)
    {
        if (priv->is_favorite)
            app_name = applet_panel_launchers_get_display_name (priv->app);
        if (!app_name)
            app_name = g_strdup (cdos_app_get_name (priv->app));
        st_label_set_text (ST_LABEL (priv->label), app_name);
        g_free (app_name);
        return;
    }

    title = meta_window_get_title (priv->window);
    if (priv->show_window_num)
    {
        gchar *new_title = NULL;
        new_title = g_strdup_printf ("(%d) %s", priv->app_window_num, title);
        st_label_set_text (ST_LABEL (priv->label), new_title);
        g_free (new_title);
    }
    else if (title)
    {
        st_label_set_text (ST_LABEL (priv->label), title);
    }
}

CdosApp *
applet_icon_label_button_get_app (AppletIconLabelButton *self)
{
    g_return_val_if_fail (APPLET_IS_ICON_LABEL_BUTTON (self), NULL);
    AppletIconLabelButtonPrivate *priv = self->priv;
    return priv->app;
}

gboolean
applet_icon_label_button_is_favorite_app (AppletIconLabelButton *self)
{
    g_return_val_if_fail (APPLET_IS_ICON_LABEL_BUTTON (self), FALSE);
    AppletIconLabelButtonPrivate *priv = self->priv;
    return priv->is_favorite;
}

static void
applet_icon_label_button_icon_sink (AppletIconLabelButton *self, gint size)
{
    AppletIconLabelButtonPrivate *priv = self->priv;
    if (priv->icon_sinked)
    {
        return;
    }

    clutter_actor_set_pivot_point (priv->icon, 0.5, 0.5);
    cdos_transition_simple (priv->icon,
                            CLUTTER_EASE_IN,
                            250,
                            "scale-x", 0.9,
                            "scale-y", 0.9,
                            NULL);
    priv->icon_sinked = TRUE;
}

static void
applet_icon_label_button_icon_recovery (AppletIconLabelButton *self)
{
    AppletIconLabelButtonPrivate *priv = self->priv;
    if (!priv->icon_sinked)
    {
        return;
    }
    cdos_transition_simple (priv->icon,
                            CLUTTER_EASE_OUT,
                            250,
                            "scale-x", 1.0,
                            "scale-y", 1.0,
                            NULL);
    priv->icon_sinked = FALSE;
}

void
applet_icon_label_button_show (AppletIconLabelButton *self, gboolean animate)
{
    g_return_if_fail (APPLET_IS_ICON_LABEL_BUTTON (self));

    clutter_actor_show (CLUTTER_ACTOR (self));
    // notice:此处需要添加tweener动画；
}

void
applet_icon_label_button_hide (AppletIconLabelButton *self, gboolean animate)
{
    g_return_if_fail (APPLET_IS_ICON_LABEL_BUTTON (self));
    // AppletIconLabelButtonPrivate *priv = self->priv;
    clutter_actor_hide (CLUTTER_ACTOR (self));
    // notice:此处需要添加tweener动画；
}

void
applet_icon_label_button_show_label (AppletIconLabelButton *self)
{
    g_return_if_fail (APPLET_IS_ICON_LABEL_BUTTON (self));
    AppletIconLabelButtonPrivate *priv = self->priv;
    clutter_actor_show (CLUTTER_ACTOR (priv->label));
}

void
applet_icon_label_button_hide_label (AppletIconLabelButton *self)
{
    g_return_if_fail (APPLET_IS_ICON_LABEL_BUTTON (self));
    AppletIconLabelButtonPrivate *priv = self->priv;
    clutter_actor_hide (CLUTTER_ACTOR (priv->label));
}

MetaWindow *
applet_icon_label_button_get_window (AppletIconLabelButton *self)
{
    g_return_val_if_fail (APPLET_IS_ICON_LABEL_BUTTON (self), NULL);
    AppletIconLabelButtonPrivate *priv = self->priv;
    return priv->window;
}

static void
meta_window_on_notify_title (MetaWindow *meta_window, GParamSpec *pspec, gpointer user_data)
{
    AppletIconLabelButton *self = APPLET_ICON_LABEL_BUTTON (user_data);
    applet_icon_label_button_update_label (self);
}

void
applet_icon_label_button_set_window (AppletIconLabelButton *self, MetaWindow *window)
{
    g_return_if_fail (APPLET_IS_ICON_LABEL_BUTTON (self));
    AppletIconLabelButtonPrivate *priv = self->priv;
    if (priv->window == window)
    {
        return;
    }

    if (priv->window)
    {
        g_signal_handlers_disconnect_by_data (priv->window, self);
    }

    priv->window = window;
    if (priv->window)
    {
        g_signal_connect (priv->window, "notify::title", G_CALLBACK (meta_window_on_notify_title), self);
        applet_icon_label_button_update_label (self);
    }
}

void
applet_icon_label_button_set_window_num (AppletIconLabelButton *self, gint window_num)
{
    g_return_if_fail (APPLET_IS_ICON_LABEL_BUTTON (self));
    AppletIconLabelButtonPrivate *priv = self->priv;
    if (priv->app_window_num != window_num)
    {
        priv->app_window_num = window_num;
        applet_icon_label_button_update_label (self);
    }
}

void
applet_icon_label_button_show_window_num (AppletIconLabelButton *self, gboolean visible)
{
    g_return_if_fail (APPLET_IS_ICON_LABEL_BUTTON (self));
    AppletIconLabelButtonPrivate *priv = self->priv;
    if (priv->show_window_num != visible)
    {
        priv->show_window_num = visible;
        applet_icon_label_button_update_label (self);
    }
}

void
applet_icon_label_button_enable_tooltip (AppletIconLabelButton *self, gboolean enable)
{
    g_return_if_fail (APPLET_IS_ICON_LABEL_BUTTON (self));
    AppletIconLabelButtonPrivate *priv = self->priv;
    cdos_tooltip_set_prevent_show (priv->tooltip, !enable);
}

void
applet_icon_label_button_set_tooltip_text (AppletIconLabelButton *self, const gchar *text)
{
    g_return_if_fail (APPLET_IS_ICON_LABEL_BUTTON (self));
    AppletIconLabelButtonPrivate *priv = self->priv;
    cdos_tooltip_set_text (priv->tooltip, text);
}
