/**
 * Copyright (c) 2016-2018 CPU and Fundamental Software Research Center, CAS
 *
 * This software is published by the license of CPU-OS License，you can use and
 * distibuted this software under this License. See CPU-OS Liense for more detail.
 *
 * You should have receive a copy of CPU-OS License. If not, please contact us
 * by email <support_os@cpu-os.ac.cn>
 *
 **/

// Created by zhangtao on 16-4-16.

#include "common.h"

#include "cdos-global.h"
#include "cdos-app-system.h"
#include "cdos-app.h"
#include "cdos-popup-menu.h"
#include "cdos-popup-menu-manager.h"
#include "cdos-utils-private.h"
#include "applet-panel-launchers-manager.h"

#include "applet-pinned-favs.h"
#include "applet-window-list.h"
#include "applet-app-button.h"

struct _AppletPinnedFavsPrivate
{
    GList *app_buttons;
    // the number of shown, if -1, show all;
    gint show_num;
    StWidget *expand_button;
    StWidget *expand_table;
    ClutterGridLayout *grid_layout;

    // if sleep, do nothing;
    gboolean sleep;
    // whether the button of context_menu is in the expand menu;
    gboolean context_menu_in_expand;

    CdosPopupMenu *expand_menu;
    CdosPopupMenuManager *expand_menu_manager;

    CdosPopupMenu *context_menu;
    CdosPopupMenuManager *context_menu_manager;

    StWidget *separator;
    CdosPopupMenuItem *move_favorite_item;

    GSettings *global_settings;
    GSettings *settings;
};

G_DEFINE_TYPE_WITH_PRIVATE  (AppletPinnedFavs, applet_pinned_favs, ST_TYPE_WIDGET);

static void applet_pinned_favs_dispose (GObject *object);
static void applet_pinned_favs_finalize (GObject *object);

static void
applet_pinned_favs_clear (AppletPinnedFavs *self)
{
    AppletPinnedFavsPrivate *priv = self->priv;
    if (priv->context_menu)
    {
        cdos_popup_menu_close (CDOS_POPUP_MENU_BASE (priv->context_menu), FALSE);
        cdos_popup_menu_base_set_source_actor ((CdosPopupMenuBase *)priv->context_menu, CLUTTER_ACTOR (self));
    }
    if (priv->app_buttons)
    {
        GList *buttons = priv->app_buttons;
        while (buttons != NULL)
        {
            AppletAppButton *button = buttons->data;
            ClutterActor *parent = clutter_actor_get_parent (CLUTTER_ACTOR (button));
            if (parent == CLUTTER_ACTOR (self))
            {
                _remove_actor (self, button);
            }
            else if (parent == CLUTTER_ACTOR (priv->expand_table))
            {
                _remove_actor (priv->expand_table, button);
            }
            else
            {
                clutter_actor_destroy ((ClutterActor *)button);
            }
            buttons = buttons->next;
        }
        g_list_free (priv->app_buttons);
        priv->app_buttons = NULL;
    }
}

static gboolean
button_on_release_event (ClutterActor *actor, ClutterEvent *event, gpointer user_data)
{
    AppletPinnedFavs *self = APPLET_PINNED_FAVS (user_data);
    AppletPinnedFavsPrivate *priv = self->priv;

    ClutterModifierType state = clutter_event_get_state (event);
    guint32 button = clutter_event_get_button (event);

    CdosApp *app = applet_icon_label_button_get_app (APPLET_ICON_LABEL_BUTTON (actor));
    if ((state & CLUTTER_BUTTON1_MASK || button == 1) || (state & CLUTTER_BUTTON2_MASK || button == 2))
    {
        cdos_popup_menu_close (CDOS_POPUP_MENU_BASE (priv->context_menu), FALSE);
        cdos_app_open_new_window (app, -1);
    }
    else if ((state & CLUTTER_BUTTON3_MASK || button == 3))
    {
        cdos_popup_menu_base_set_source_actor ((CdosPopupMenuBase *)priv->context_menu, (ClutterActor *)actor);
        cdos_popup_menu_base_toggle (CDOS_POPUP_MENU_BASE (priv->context_menu));
        if (CDOS_POPUP_MENU_BASE (priv->context_menu)->is_open)
        {
            ClutterActor *parent = clutter_actor_get_parent (CLUTTER_ACTOR (actor));
            priv->context_menu_in_expand = parent == CLUTTER_ACTOR (priv->expand_table) ? TRUE : FALSE;
        }
    }
    return TRUE;
}

static void
applet_pinned_favs_expand_menu_add_child (AppletPinnedFavs *self, ClutterActor *actor, guint index)
{
    AppletPinnedFavsPrivate *priv = self->priv;

    gint i = index - priv->show_num;
    gint row_size = 3, row, col;

    col = i % row_size;
    row = (int)(i / row_size);

    clutter_grid_layout_attach (priv->grid_layout, actor, col, row, 1, 1);
}

static void
applet_pinned_favs_add_app_button (AppletPinnedFavs *self, CdosApp *app, guint index)
{
    AppletPinnedFavsPrivate *priv = self->priv;
    gchar *app_name = NULL;
    if (app)
    {
        // create new button;
        AppletAppButton *button = applet_app_button_new (app, TRUE, NULL);
        app_name = applet_panel_launchers_get_display_name (app);
        if (!app_name)
            app_name = g_strdup (cdos_app_get_name (app));

        applet_icon_label_button_enable_tooltip (APPLET_ICON_LABEL_BUTTON (button), TRUE);
        applet_icon_label_button_set_tooltip_text (APPLET_ICON_LABEL_BUTTON (button), app_name);

        st_widget_add_style_class_name (ST_WIDGET (button), "window-list-pinned-button");
        priv->app_buttons = g_list_append (priv->app_buttons, button);

        g_signal_connect (button, "button-release-event", G_CALLBACK (button_on_release_event), self);
        if (priv->show_num == -1 || index < priv->show_num)
        {
            clutter_actor_insert_child_at_index (CLUTTER_ACTOR (self),
                                                 CLUTTER_ACTOR (button),
                                                 0);
        }
        else
        {
            applet_pinned_favs_expand_menu_add_child (self, CLUTTER_ACTOR (button), index);
        }
        g_free (app_name);
    }
}

static void
panel_launchers_on_changed (GSettings *settings, gchar *key, gpointer user_data)
{
    AppletPinnedFavs *self = APPLET_PINNED_FAVS (user_data);
    AppletPinnedFavsPrivate *priv = self->priv;
    if (priv->sleep)
    {
        return;
    }
    applet_panel_launchers_update_files ();
    applet_pinned_favs_reload (self);
}

static void
pinned_show_number_on_changed (GSettings *settings, gchar *key, gpointer user_data)
{
    AppletPinnedFavs *self = APPLET_PINNED_FAVS (user_data);
    AppletPinnedFavsPrivate *priv = self->priv;
    if (priv->sleep)
        return;
    // show_num may be changed outside;
    gint old_show_num = priv->show_num;
    priv->show_num = g_settings_get_int (settings, "pinned-show-number");
    if (old_show_num == priv->show_num)
        return;
    applet_pinned_favs_reload (self);
}

// target_position is only used to move action;
static void
applet_pinned_favs_remove_or_move_launcher (AppletPinnedFavs *self, gboolean move, gint target_position)
{
    AppletPinnedFavsPrivate *priv = self->priv;
    CdosApp *app = NULL;
    const gchar *app_id = NULL;
    GSettings *settings = NULL;
    gchar **desktop_files = NULL, **desktop_files_new = NULL, **desktop_files_new1 = NULL;
    ClutterActor *source_actor;

    // cdos_popup_menu_close (CDOS_POPUP_MENU_BASE (priv->context_menu), FALSE);
    source_actor = cdos_popup_menu_base_get_source_actor (CDOS_POPUP_MENU_BASE (priv->context_menu));
    if (!APPLET_IS_APP_BUTTON (source_actor))
    {
        return;
    }

    app = applet_icon_label_button_get_app (APPLET_ICON_LABEL_BUTTON (source_actor));
    app_id = cdos_app_get_id (app);

    settings = priv->global_settings;
    desktop_files = g_settings_get_strv (settings, "panel-launchers");

    desktop_files_new = cdos_strv_elem_del_by_str (desktop_files, app_id);

    if (move)
    {
        desktop_files_new1 = cdos_strv_elem_insert ((const gchar **)desktop_files_new, app_id, target_position);
        g_strfreev (desktop_files_new);
        desktop_files_new = desktop_files_new1;
    }

    g_settings_set_strv (settings, "panel-launchers", (const gchar *const *)desktop_files_new);

    if (desktop_files_new != NULL)
    {
        g_free (desktop_files_new);
        g_strfreev (desktop_files);
    }
}

static void
menu_item_on_remove_favorite (CdosPopupBaseMenuItem *item, ClutterEvent *event, gboolean keep_menu, gpointer user_data)
{
    AppletPinnedFavs *self = APPLET_PINNED_FAVS (user_data);
    AppletPinnedFavsPrivate *priv = self->priv;
    cdos_popup_menu_close (CDOS_POPUP_MENU_BASE (priv->context_menu), FALSE);
    cdos_popup_menu_close (CDOS_POPUP_MENU_BASE (priv->expand_menu), FALSE);

    applet_pinned_favs_remove_or_move_launcher (self, FALSE, -1);
}

static void
menu_item_on_move_favorite (CdosPopupBaseMenuItem *item, ClutterEvent *event, gboolean keep_menu, gpointer user_data)
{
    AppletPinnedFavs *self = APPLET_PINNED_FAVS (user_data);
    AppletPinnedFavsPrivate *priv = self->priv;

    gint new_show_num = priv->show_num - 1;
    if (priv->show_num == -1)
    {
        gchar **launchers = g_settings_get_strv (priv->global_settings, "panel-launchers");
        guint len = g_strv_length (launchers);
        new_show_num = len - 1;
        if (launchers)
        {
            g_strfreev (launchers);
        }
    }

    gint pos = -1; // from panel to expand menu:default move to end;
    cdos_popup_menu_close (CDOS_POPUP_MENU_BASE (priv->context_menu), FALSE);
    cdos_popup_menu_close (CDOS_POPUP_MENU_BASE (priv->expand_menu), FALSE);

    if (priv->context_menu_in_expand)
    {
        // from expand menu to panel: move to the position of show_num;
        pos = priv->show_num;
        new_show_num = priv->show_num + 1;
    }
    applet_pinned_favs_remove_or_move_launcher (self, TRUE, pos);

    // update the show number;
    g_settings_set_int (priv->settings, "pinned-show-number", new_show_num);
}

static void
menu_item_on_show_properties_window (CdosPopupBaseMenuItem *item, ClutterEvent *event, gboolean keep_menu, gpointer user_data)
{
    AppletPinnedFavs *self = APPLET_PINNED_FAVS (user_data);
    AppletPinnedFavsPrivate *priv = self->priv;
    const gchar * app_id = NULL;
    ClutterActor *source_actor;
    CdosApp *app;

    //cdos_popup_menu_close (CDOS_POPUP_MENU_BASE (priv->context_menu), FALSE);
    source_actor = cdos_popup_menu_base_get_source_actor (CDOS_POPUP_MENU_BASE (priv->context_menu));
    if (!APPLET_IS_APP_BUTTON (source_actor))
    {
        return;
    }

    app = applet_icon_label_button_get_app (APPLET_ICON_LABEL_BUTTON (source_actor));
    app_id = cdos_app_get_id (app);

    applet_panel_launchers_edit_properties (app_id);
}

static CdosPopupMenuItem *
applet_pinned_favs_create_menu_item (AppletPinnedFavs *self, const gchar *label_text, GCallback callback)
{
    AppletPinnedFavsPrivate *priv = self->priv;
    CdosPopupMenuItem *item = cdos_popup_menu_item_new ();
    cdos_popup_menu_item_set_label (item, label_text);
    cdos_popup_menu_base_add_menu_item ((CdosPopupMenuBase *)priv->context_menu, CDOS_POPUP_BASE_MENU_ITEM (item), -1);
    g_signal_connect (CDOS_POPUP_BASE_MENU_ITEM (item), "activate", callback, self);
    return item;
}

static void
context_menu_on_open_state_changed (CdosPopupMenuBase *popup_menu, gboolean state, gpointer user_data)
{
    AppletPinnedFavs *self = APPLET_PINNED_FAVS (user_data);
    AppletPinnedFavsPrivate *priv = self->priv;
    ClutterActor *source_actor;
    source_actor = cdos_popup_menu_base_get_source_actor (CDOS_POPUP_MENU_BASE (priv->context_menu));
    if (!APPLET_IS_APP_BUTTON (source_actor))
    {
        return;
    }

    if (state)
    {
        st_widget_add_style_pseudo_class (ST_WIDGET (source_actor), "focus");
        CDOS_POPUP_MENU_BASE (priv->expand_menu)->pass_events = TRUE;
    }
    else
    {
        st_widget_remove_style_pseudo_class (ST_WIDGET (source_actor), "focus");
        CDOS_POPUP_MENU_BASE (priv->expand_menu)->pass_events = FALSE;
    }
}

static void
applet_pinned_favs_create_menu (AppletPinnedFavs *self)
{
    AppletPinnedFavsPrivate *priv = self->priv;
    CdosPopupMenuItem *menu_item = NULL;
    priv->context_menu = cdos_popup_menu_new_with_params (CLUTTER_ACTOR (self), 0, 2);
    priv->context_menu_manager = cdos_popup_menu_manager_new (CLUTTER_ACTOR (self));
    cdos_popup_menu_manager_add_menu (priv->context_menu_manager, CDOS_POPUP_MENU_BASE (priv->context_menu), -1);
    menu_item = applet_pinned_favs_create_menu_item (self, _("Remove App"), G_CALLBACK (menu_item_on_remove_favorite));
    //"Move to expand menu" and "Move to panel"
    menu_item =
        applet_pinned_favs_create_menu_item (self, _("Toggle shaded state"), G_CALLBACK (menu_item_on_move_favorite));
    priv->move_favorite_item = menu_item;

    menu_item = applet_pinned_favs_create_menu_item (self, _ ("Properties"), G_CALLBACK (menu_item_on_show_properties_window));

    g_signal_connect (priv->context_menu,
                      "open-state-changed",
                      G_CALLBACK (context_menu_on_open_state_changed),
                      self);
    // expand menu
    priv->expand_menu = cdos_popup_menu_new_with_params (CLUTTER_ACTOR (priv->expand_button), 1.0, 2);
    priv->expand_menu_manager = cdos_popup_menu_manager_new (CLUTTER_ACTOR (priv->expand_button));
    cdos_popup_menu_manager_add_menu (priv->expand_menu_manager, CDOS_POPUP_MENU_BASE (priv->expand_menu), -1);

    priv->expand_table = g_object_new (ST_TYPE_WIDGET, NULL);
    priv->grid_layout = (ClutterGridLayout *)clutter_grid_layout_new ();
    clutter_actor_set_layout_manager ((ClutterActor *)priv->expand_table, (ClutterLayoutManager *)priv->grid_layout);
    clutter_grid_layout_set_row_homogeneous (priv->grid_layout, TRUE);
    clutter_grid_layout_set_column_homogeneous (priv->grid_layout, FALSE);
    cdos_popup_menu_base_add_actor (CDOS_POPUP_MENU_BASE (priv->expand_menu), CLUTTER_ACTOR (priv->expand_table));
    // clutter_container_add_actor((ClutterContainer*) priv->box, CLUTTER_ACTOR(priv->table));
}

static void
expand_button_on_clicked (StButton *button, gint clicked_button, gpointer user_data)
{
    AppletPinnedFavs *self = APPLET_PINNED_FAVS (user_data);
    AppletPinnedFavsPrivate *priv = self->priv;
    if (clicked_button == 1)
    {
        cdos_popup_menu_base_toggle (CDOS_POPUP_MENU_BASE (priv->expand_menu));
    }
}

static void
applet_pinned_favs_init (AppletPinnedFavs *self)
{
    AppletPinnedFavsPrivate *priv;
    self->priv = applet_pinned_favs_get_instance_private (self);
    priv = self->priv;

    priv->sleep = TRUE;
    priv->context_menu_in_expand = FALSE;

    ClutterLayoutManager *box_layout = clutter_box_layout_new ();
    clutter_actor_set_layout_manager ((ClutterActor *)self, box_layout);

    priv->settings = applet_window_list_get_settings ();
    priv->show_num = g_settings_get_int (priv->settings, "pinned-show-number");
    g_signal_connect (priv->settings, "changed::pinned-show-number", G_CALLBACK (pinned_show_number_on_changed), self);

    priv->expand_button = st_button_new ();
    st_widget_set_style_class_name (priv->expand_button, "window-list-pinned-expand");
    clutter_actor_set_reactive (CLUTTER_ACTOR (priv->expand_button), TRUE);
    _add_actor (self, priv->expand_button);
    g_signal_connect (priv->expand_button, "clicked", G_CALLBACK (expand_button_on_clicked), self);
    clutter_actor_hide (CLUTTER_ACTOR (priv->expand_button));

    priv->separator = st_label_new ("|");
    st_widget_set_style_class_name (priv->separator, "window-list-pinned-separator");
    _add_actor (self, priv->separator);

    applet_pinned_favs_create_menu (self);

    priv->global_settings = cdos_global_get_settings (cdos_global_get ());
    g_signal_connect (priv->global_settings, "changed::panel-launchers", G_CALLBACK (panel_launchers_on_changed), self);
}

static void
applet_pinned_favs_class_init (AppletPinnedFavsClass *klass)
{
    GObjectClass *gobject_class = G_OBJECT_CLASS (klass);

    gobject_class->dispose = applet_pinned_favs_dispose;
    gobject_class->finalize = applet_pinned_favs_finalize;
}

static void
applet_pinned_favs_dispose (GObject *object)
{
    AppletPinnedFavs *self = APPLET_PINNED_FAVS (object);
    AppletPinnedFavsPrivate *priv = self->priv;

    if (priv->global_settings)
    {
        g_signal_handlers_disconnect_by_data (priv->global_settings, self);
        priv->global_settings = NULL;
    }

    if (priv->settings)
    {
        g_signal_handlers_disconnect_by_data (priv->settings, self);
        priv->settings = NULL;
    }

    g_clear_pointer (&priv->app_buttons, g_list_free);

    if (priv->context_menu)
    {
        cdos_popup_menu_base_destroy (CDOS_POPUP_MENU_BASE (priv->context_menu));
        priv->context_menu = NULL;
    }
    g_clear_object (&priv->context_menu_manager);

    if (priv->expand_menu)
    {
        cdos_popup_menu_base_destroy (CDOS_POPUP_MENU_BASE (priv->expand_menu));
        priv->expand_menu = NULL;
    }
    g_clear_object (&priv->expand_menu_manager);

    G_OBJECT_CLASS (applet_pinned_favs_parent_class)->dispose (object);
}

static void
applet_pinned_favs_finalize (GObject *object)
{
    G_OBJECT_CLASS (applet_pinned_favs_parent_class)->finalize (object);
}

AppletPinnedFavs *
applet_pinned_favs_new (void)
{
    AppletPinnedFavs *self;
    self = g_object_new (APPLET_TYPE_PINNED_FAVS, "style-class", "window-list-pinned-favs", NULL);
    return self;
}

void
applet_pinned_favs_reload (AppletPinnedFavs *self)
{
    g_return_if_fail (APPLET_PINNED_FAVS (self));
    AppletPinnedFavsPrivate *priv = self->priv;
    priv->sleep = FALSE;
    applet_pinned_favs_clear (self);

    gchar **launchers = NULL;
    CdosAppSystem *app_system = cdos_app_system_get_default ();
    launchers = g_settings_get_strv (priv->global_settings, "panel-launchers");

    guint i = 0;
    gchar *launcher;

    while (launchers[i])
    {
        CdosApp *app = NULL;
        launcher = launchers[i];
        app = cdos_app_system_lookup_app (app_system, launcher);

        applet_pinned_favs_add_app_button (self, app, i);
        ++i;
    }
    if (launchers)
    {
        g_strfreev (launchers);
    }
    if (priv->show_num >= 0 && i > priv->show_num)
    {
        clutter_actor_show (CLUTTER_ACTOR (priv->expand_button));
    }
    else
    {
        clutter_actor_hide (CLUTTER_ACTOR (priv->expand_button));
    }
    if (i > 0)
    {
        clutter_actor_show (CLUTTER_ACTOR (self));
    }
    else
    {
        clutter_actor_hide (CLUTTER_ACTOR (self));
    }
}

void
applet_pinned_favs_unload (AppletPinnedFavs *self)
{
    g_return_if_fail (APPLET_PINNED_FAVS (self));
    AppletPinnedFavsPrivate *priv = self->priv;
    priv->sleep = TRUE;
    applet_pinned_favs_clear (self);
    clutter_actor_hide (CLUTTER_ACTOR (self));
}
