// Created by zhangtao on 16-4-16.
/**
 * SECTION: applet-window-list
 * @short_description: 任务栏中的应用窗口列表管理器。
 * @title: AppletWindowList
 * @include: applet-window-list.h
 *
 * #AppletWindowList是为桌面任务栏提供应用窗口列表的相关管理，为用户提供应用的相关打开，关闭，应用切换，窗口预览等操作，同时提供部分应用的快捷启动操作。
 * 应用窗口列表管理器托管在任务栏中部区域，默认靠左侧对齐，功能分为两个部分：1.快捷启动应用列表；2.应用窗口列表。
 * 应用窗口列表管理器在当前工作区的任务栏中唯一存在，生命周期同当前工作区。
 *
 */
#include <stdlib.h>
#include <meta/workspace.h>
#include <meta/meta-workspace-manager.h>

#include "cdos-global.h"
#include "cdos-popup-switch-menu-item.h"
#include "cdos-ui-plugin.h"
#include "cdos-overview.h"
#include "applet-window-list.h"
#include "applet-app-list.h"
#include "applet-app-group.h"


struct _AppletWindowListPrivate
{
    StSide orientation;
    GSettings *settings;
    StWidget *actor;
    ClutterActor *box;
    AppletAppList *app_list;

    MetaDisplay *display;
    CdosOverview *main_overview;
};

G_DEFINE_TYPE_WITH_PRIVATE (AppletWindowList, applet_window_list, CDOS_TYPE_APPLET);

static AppletWindowList *window_list_default = NULL;

static void applet_window_list_dispose (GObject *object);
static void applet_window_list_finalize (GObject *object);

static AppletWindowList *
applet_window_list_new (MetaData *metadata, StSide orientation, gint panel_height, gchar *instance_id);

static void applet_window_list_on_overview_show (CdosOverview *overview, AppletWindowList *self);
static void applet_window_list_on_overview_hide (CdosOverview *overview, AppletWindowList *self);

static void
applet_window_list_on_orientation_changed (CdosApplet *applet, StSide orientation)
{
    AppletWindowList *self = APPLET_WINDOW_LIST (applet);
    AppletWindowListPrivate *priv = self->priv;
    priv->orientation = orientation;
    if (priv->app_list)
        applet_app_list_set_orientation (priv->app_list, orientation);
}

static void
applet_window_list_allocate (ClutterActor *actor,
                             const ClutterActorBox *box,
                             ClutterAllocationFlags flags)
{
    AppletWindowList *self = APPLET_WINDOW_LIST (actor);
    AppletWindowListPrivate *priv = self->priv;

    StThemeNode *theme_node = st_widget_get_theme_node ((StWidget *)actor);

    clutter_actor_set_allocation (actor, box, flags);

    ClutterActorBox content_box, child_box = {0};
    st_theme_node_get_content_box (theme_node, box, &content_box);

    child_box = content_box;

    applet_app_list_set_max_width (priv->app_list, box->x2 - box->x1);
    clutter_actor_allocate ((ClutterActor *)priv->app_list, &child_box, flags);
}

static void
applet_window_list_init (AppletWindowList *self)
{
    AppletWindowListPrivate *priv;
    priv = self->priv = applet_window_list_get_instance_private (self);

    CdosGlobal *global = cdos_global_get ();

    priv->actor = ST_WIDGET (self);
    clutter_actor_set_reactive (CLUTTER_ACTOR (priv->actor), FALSE);
    //clutter_actor_set_x_expand ((ClutterActor *)priv->actor, TRUE);
    clutter_actor_set_x_align ((ClutterActor *)priv->actor, CLUTTER_ACTOR_ALIGN_FILL);
    st_widget_set_track_hover (priv->actor, FALSE);

    priv->settings = g_settings_new (CDOS_WINDOW_LIST_SCHEMA);

    priv->display = cdos_global_get_display (global);

    priv->main_overview = cdos_overview_get ();
    g_signal_connect (priv->main_overview, "showing", G_CALLBACK (applet_window_list_on_overview_show), self);
    g_signal_connect (priv->main_overview, "hiding", G_CALLBACK (applet_window_list_on_overview_hide), self);

    /* CdosPanel *panel = cdos_ui_plugin_get_panel ();
    cdos_panel_add_separator_item (panel);

    CdosPopupSwitchMenuItem *item;
    item = cdos_popup_switch_menu_item_new_with_params (_("Group app buttons"), FALSE, NULL);
    cdos_popup_base_menu_item_set_keep_menu (CDOS_POPUP_BASE_MENU_ITEM (item), FALSE);
    cdos_panel_add_menu_item (panel, (CdosPopupBaseMenuItem *)item);
    g_settings_bind (priv->settings, "group-apps", item, "state", G_SETTINGS_BIND_DEFAULT); */

    /*
    item = cdos_popup_switch_menu_item_new_with_params (_("Enable hover peek"), FALSE, NULL);
    cdos_panel_add_menu_item (panel, (CdosPopupBaseMenuItem *)item);
    g_settings_bind (priv->settings, "enable-hover-peek", item, "state", G_SETTINGS_BIND_DEFAULT);
    */
}

static void
applet_window_list_class_init (AppletWindowListClass *klass)
{
    GObjectClass *gobject_class = G_OBJECT_CLASS (klass);

    gobject_class->dispose = applet_window_list_dispose;
    gobject_class->finalize = applet_window_list_finalize;

    ClutterActorClass *actor_class = CLUTTER_ACTOR_CLASS (klass);
    actor_class->allocate = applet_window_list_allocate;

    CdosAppletClass *applet_class = CDOS_APPLET_CLASS (klass);
    applet_class->on_orientation_changed = applet_window_list_on_orientation_changed;
}

static void
applet_window_list_dispose (GObject *object)
{
    AppletWindowList *self = APPLET_WINDOW_LIST (object);
    AppletWindowListPrivate *priv = self->priv;
    CdosBaseWM *global_wm = NULL;
    g_object_get (cdos_global_get (), "window-manager", &global_wm, NULL);
    if (global_wm)
    {
        g_signal_handlers_disconnect_by_data (global_wm, self);
    }

    if (priv->main_overview)
    {
        g_signal_handlers_disconnect_by_data (priv->main_overview, self);
        priv->main_overview = NULL;
    }

    _remove_actor (priv->actor, priv->app_list);

    g_clear_object (&priv->settings);

    G_OBJECT_CLASS (applet_window_list_parent_class)->dispose (object);
}

static void
applet_window_list_finalize (GObject *object)
{
    G_OBJECT_CLASS (applet_window_list_parent_class)->finalize (object);
}

static void
app_list_new_for_workspace (CdosBaseWM *cdos_base_wm,
                            gint previous_workspace_index,
                            gint current_workspace_index,
                            MetaMotionDirection direction,
                            AppletWindowList *self)
{
    AppletWindowListPrivate *priv = self->priv;
    MetaWorkspaceManager *wsm = meta_display_get_workspace_manager (priv->display);
    MetaWorkspace *meta_workspace = meta_workspace_manager_get_workspace_by_index (wsm, current_workspace_index);
    AppletAppList *app_list;

    app_list = applet_app_list_new (meta_workspace);
    _add_actor (priv->actor, (ClutterActor *)app_list);
    priv->app_list = app_list;
}

static void
applet_window_list_on_overview_show (CdosOverview *overview, AppletWindowList *self)
{
    AppletWindowListPrivate *priv = self->priv;
    clutter_actor_hide (CLUTTER_ACTOR (priv->actor));
}

static void
applet_window_list_on_overview_hide (CdosOverview *overview, AppletWindowList *self)
{
    AppletWindowListPrivate *priv = self->priv;
    clutter_actor_show (CLUTTER_ACTOR (priv->actor));
}

static AppletWindowList *
applet_window_list_new (MetaData *metadata, StSide orientation, gint panel_height, gchar *instance_id)
{
    AppletWindowList *self;
    self = g_object_new (APPLET_TYPE_WINDOW_LIST, "orientation", (int)orientation,
                         NULL);

    window_list_default = self;

    // cdos_applet_constructor (CDOS_APPLET(self), orientation, panel_height,instance_id);
    AppletWindowListPrivate *priv;
    priv = self->priv;
    priv->orientation = orientation;

    MetaWorkspaceManager *wsm = meta_display_get_workspace_manager (priv->display);
    gint active_workspace_index = meta_workspace_manager_get_active_workspace_index (wsm);
    CdosBaseWM *global_wm;
    global_wm = cdos_global_get_window_manager (cdos_global_get ());
    app_list_new_for_workspace (global_wm, 0, active_workspace_index, META_MOTION_RIGHT, self);
    st_widget_add_style_class_name ((StWidget *)self, "applet-box-window-list");
    return self;
}

gfloat
applet_window_list_get_panel_height (void)
{
    gfloat panel_height = cdos_applet_get_panel_height (CDOS_APPLET (window_list_default));
    return panel_height;
}

/**
 * applets_window_list_main:
 * @metadata: window-list基本信息，由配置文件读取
 * @orientation: 对齐方式，StSide类型成员
 * @panel_height: 面板高度
 * @instance_id: 当前实例在面板中的位置序号
 *
 * 创建一个应用窗口列表管理器。
 *
 */
CdosApplet *
applets_window_list_main (MetaData *metadata, StSide orientation, gfloat panel_height, gchar *instance_id)
{
    // gint applet_id = atoi(instance_id);
    if ((int)panel_height == 0)
    {
        panel_height = 40.0;
    }

    AppletWindowList *applet = applet_window_list_new (metadata, orientation, (int)panel_height, instance_id);
    return (CdosApplet *)applet;
}

/**
 * applet_window_list_get_settings:
 * @self: 当前实例对象
 *
 * 获取当前实例对象的配置实例对象，通过该实例对象可以获取，修改配置项，并监听配置文件的变化。
 *
 * Returns: #CdosAppletSettings类型的实例对象。
 */
GSettings *
applet_window_list_get_settings (void)
{
    AppletWindowListPrivate *priv = window_list_default->priv;
    return priv->settings;
}

AppletWindowList *
app_window_list_get_default (void)
{
    return window_list_default;
}
