// Created by zhangtao on 16-4-16.

#include <math.h>
#include <meta/meta-window-actor.h>
#include "applet-window-thumbnail.h"
#include "applet-window-list.h"
#include "cdos-global.h"
#include "common.h"
#include "applet-panel-launchers-manager.h"

struct _AppletWindowThumbnailPrivate
{
    MetaWindow *meta_window;
    CdosApp *app;
    gboolean is_favorite_app;
    gboolean was_minimized;
    StWidget *actor;
    StWidget *thumbnail_actor;
    StWidget *container1;
    ClutterActor *icon;
    StWidget *label;
    StWidget *container2;
    StWidget *button;
    gint thumbnail_width;
    gint thumbnail_height;
    gboolean stop_click;
    ClutterActor *thumbnail;
    StWidget *error_icon_title;
    ClutterActor *error_icon_thumbnail;
    gfloat scale;
    gboolean no_thumbnail;
    gboolean enable_hover_peek;
    GSettings *settings;
};
G_DEFINE_TYPE_WITH_PRIVATE (AppletWindowThumbnail, applet_window_thumbnail, ST_TYPE_WIDGET);

static void applet_window_thumbnail_dispose (GObject *object);
static void applet_window_thumbnail_finalize (GObject *object);

static void applet_window_thumbnail_setup (AppletWindowThumbnail *self,
                                           CdosApp *app,
                                           gboolean is_favorite,
                                           MetaWindow *meta_window);
static void meta_window_on_notify_title (MetaWindow *meta_window, GParamSpec *pspec, gpointer user_data);
static gboolean thumbnail_actor_on_enter (ClutterActor *actor, ClutterEvent *event, gpointer user_data);
static gboolean thumbnail_actor_on_leave (ClutterActor *actor, ClutterEvent *event, gpointer user_data);
static gboolean thumbnail_actor_on_release (ClutterActor *actor, ClutterEvent *event, gpointer user_data);
static gboolean thumbnail_button_on_release (ClutterActor *actor, ClutterEvent *event, gpointer user_data);

static void applet_window_thumbnail_hover_peek (AppletWindowThumbnail *self, gboolean make_above);

static ClutterActor *applet_window_thumbnail_get_thumbnail_private (AppletWindowThumbnail *self);

static void applet_window_thumbnail_refresh (AppletWindowThumbnail *self);

static void applet_window_thumbnail_window_missing (AppletWindowThumbnail *self);

static void
settings_on_changed (GSettings *settings, gchar *key, gpointer user_data)
{
    AppletWindowThumbnail *self = APPLET_WINDOW_THUMBNAIL (user_data);
    AppletWindowThumbnailPrivate *priv = self->priv;

    if (g_strcmp0 (key, CDOS_WINDOW_LIST_HOVER_PEEK) == 0)
    {
        priv->enable_hover_peek = g_settings_get_boolean (settings, CDOS_WINDOW_LIST_HOVER_PEEK);
    }
}

static void
applet_window_thumbnail_init (AppletWindowThumbnail *self)
{
    AppletWindowThumbnailPrivate *priv;
    self->priv = applet_window_thumbnail_get_instance_private (self);
    priv = self->priv;

    priv->was_minimized = FALSE;
    priv->actor = ST_WIDGET (self);
    ClutterLayoutManager *layout = clutter_box_layout_new ();
    clutter_box_layout_set_orientation ((ClutterBoxLayout *)layout, CLUTTER_ORIENTATION_VERTICAL);
    clutter_actor_set_layout_manager ((ClutterActor *)self, layout);

    priv->no_thumbnail = FALSE;

    priv->settings = applet_window_list_get_settings ();
    priv->enable_hover_peek = g_settings_get_boolean (priv->settings, CDOS_WINDOW_LIST_HOVER_PEEK);
    g_signal_connect (priv->settings, "changed", G_CALLBACK (settings_on_changed), self);

    st_widget_set_style_class_name (priv->actor, "item-box");
    clutter_actor_set_reactive (CLUTTER_ACTOR (priv->actor), TRUE);
    st_widget_set_track_hover (priv->actor, TRUE);

    priv->thumbnail_actor = st_bin_new ();
    clutter_actor_hide (CLUTTER_ACTOR (priv->thumbnail_actor));
    st_widget_set_style_class_name (priv->thumbnail_actor, "thumbnail");

    priv->container1 = st_box_layout_new ();
    priv->label = st_label_new (NULL);
    st_widget_set_style_class_name (priv->label, "thumbnail-title-label");
    st_widget_set_style_class_name (priv->container1, "thumbnail-title-box");

    priv->container2 = st_box_layout_new ();
    priv->button = st_box_layout_new ();
    st_widget_set_style_class_name (priv->button, "window-close-cs");
    clutter_actor_set_reactive (CLUTTER_ACTOR (priv->button), TRUE);
    st_widget_set_track_hover (priv->button, TRUE);
    clutter_actor_hide (CLUTTER_ACTOR (priv->button));

    _add_actor (priv->container2, priv->container1);
    _add_actor (priv->container2, priv->button);

    g_object_set (priv->container1, "x-expand", TRUE, "x-align", CLUTTER_ACTOR_ALIGN_START, NULL);

    _add_actor (priv->actor, priv->container2);
    _add_actor (priv->actor, priv->thumbnail_actor);
}

static void
applet_window_thumbnail_class_init (AppletWindowThumbnailClass *klass)
{
    GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
    gobject_class->dispose = applet_window_thumbnail_dispose;
    gobject_class->finalize = applet_window_thumbnail_finalize;
}

static void
applet_window_thumbnail_dispose (GObject *object)
{
    AppletWindowThumbnail *self = APPLET_WINDOW_THUMBNAIL (object);
    AppletWindowThumbnailPrivate *priv = self->priv;

    if (priv->settings)
    {
        g_signal_handlers_disconnect_by_data (priv->settings, self);
        priv->settings = NULL;
    }

    if (priv->meta_window)
    {
        g_signal_handlers_disconnect_by_data (priv->meta_window, self);
        priv->meta_window = NULL;
    }

    if (priv->thumbnail)
    {
        clutter_actor_destroy (CLUTTER_ACTOR (priv->thumbnail));
        priv->thumbnail = NULL;
    }

    G_OBJECT_CLASS (applet_window_thumbnail_parent_class)->dispose (object);
}

static void
applet_window_thumbnail_finalize (GObject *object)
{
    // AppletWindowThumbnail *self = APPLET_WINDOW_THUMBNAIL (object);
    // AppletWindowThumbnailPrivate *priv = self->priv;

    G_OBJECT_CLASS (applet_window_thumbnail_parent_class)->finalize (object);
}

static void
applet_window_thumbnail_setup (AppletWindowThumbnail *self, CdosApp *app, gboolean is_favorite, MetaWindow *meta_window)
{
    AppletWindowThumbnailPrivate *priv = self->priv;
    priv->meta_window = meta_window;
    priv->app = app;
    priv->is_favorite_app = is_favorite;
    
    if (priv->is_favorite_app)
        priv->icon = applet_panel_launchers_create_icon_texture (priv->app, APPLET_THUMBNAIL_ICON_SIZE);
    if (!priv->icon)
        priv->icon = cdos_app_create_icon_texture (priv->app, APPLET_THUMBNAIL_ICON_SIZE);

    _add_actor (priv->container1, priv->icon);
    _add_actor (priv->container1, priv->label);

    applet_window_thumbnail_is_favorite (self, priv->is_favorite_app);
    if (priv->meta_window != NULL)
    {
        g_signal_connect (priv->meta_window, "notify::title", G_CALLBACK (meta_window_on_notify_title), self);
    }
    g_signal_connect (priv->actor, "enter-event", G_CALLBACK (thumbnail_actor_on_enter), self);
    g_signal_connect (priv->actor, "leave-event", G_CALLBACK (thumbnail_actor_on_leave), self);
    g_signal_connect (
        priv->actor, "button-release-event", G_CALLBACK (thumbnail_actor_on_release), self);
    g_signal_connect (
        priv->button, "button-release-event", G_CALLBACK (thumbnail_button_on_release), self);
}

static void
meta_window_on_notify_title (MetaWindow *meta_window, GParamSpec *pspec, gpointer user_data)
{
    AppletWindowThumbnail *self = APPLET_WINDOW_THUMBNAIL (user_data);
    if (meta_window != self->priv->meta_window)
    {
        return;
    }
    AppletWindowThumbnailPrivate *priv = self->priv;
    const gchar *title = meta_window_get_title (meta_window);
    if (priv->label)
    {
        st_label_set_text (ST_LABEL (priv->label), title);
    }
}

static gboolean
thumbnail_actor_on_enter (ClutterActor *actor, ClutterEvent *event, gpointer user_data)
{
    AppletWindowThumbnail *self = APPLET_WINDOW_THUMBNAIL (user_data);
    AppletWindowThumbnailPrivate *priv = self->priv;
    applet_window_thumbnail_hover_peek (self, TRUE);
    if (!priv->is_favorite_app)
    {
        st_widget_add_style_pseudo_class (priv->actor, "outlined");
        st_widget_add_style_pseudo_class (priv->actor, "selected");
        gboolean show_thumbs = TRUE;

        if (show_thumbs)
        {
            clutter_actor_show (CLUTTER_ACTOR (priv->button));
            gboolean minimized;

            g_object_get (priv->meta_window, "minimized", &minimized, NULL);
            if (minimized && priv->enable_hover_peek)
            {
                meta_window_unminimize (priv->meta_window);
                priv->was_minimized = TRUE;
            }
            else
            {
                priv->was_minimized = FALSE;
            }
        }
    }
    return FALSE;
}

static gboolean
thumbnail_actor_on_leave (ClutterActor *actor, ClutterEvent *event, gpointer user_data)
{
    AppletWindowThumbnail *self = APPLET_WINDOW_THUMBNAIL (user_data);
    AppletWindowThumbnailPrivate *priv = self->priv;
    applet_window_thumbnail_hover_peek (self, FALSE);
    if (!priv->is_favorite_app)
    {
        st_widget_remove_style_pseudo_class (priv->actor, "outlined");
        st_widget_remove_style_pseudo_class (priv->actor, "selected");
        clutter_actor_hide (CLUTTER_ACTOR (priv->button));

        if (priv->was_minimized)
        {
            meta_window_minimize (priv->meta_window);
        }
    }
    return FALSE;
}

static gboolean
thumbnail_actor_on_release (ClutterActor *actor, ClutterEvent *event, gpointer user_data)
{
    AppletWindowThumbnail *self = APPLET_WINDOW_THUMBNAIL (user_data);
    AppletWindowThumbnailPrivate *priv = self->priv;
    priv->was_minimized = FALSE;
    ClutterModifierType state = clutter_event_get_state (event);
    guint32 current_time = cdos_global_get_current_time (cdos_global_get ());
    guint32 button = clutter_event_get_button (event);
    if ((state & CLUTTER_BUTTON1_MASK || button == 1) && !priv->stop_click && !priv->is_favorite_app)
    {
        meta_window_activate (priv->meta_window, current_time);
    }
    else if ((state & CLUTTER_BUTTON2_MASK || button == 2) && !priv->stop_click)
    {
        meta_window_delete (priv->meta_window, current_time);
    }
    priv->stop_click = FALSE;
    return FALSE;
}

static gboolean
thumbnail_button_on_release (ClutterActor *actor, ClutterEvent *event, gpointer user_data)
{
    AppletWindowThumbnail *self = APPLET_WINDOW_THUMBNAIL (user_data);
    AppletWindowThumbnailPrivate *priv = self->priv;
    ClutterModifierType state = clutter_event_get_state (event);
    guint32 button = clutter_event_get_button (event);
    if ((state & CLUTTER_BUTTON1_MASK || button == 1) && actor == CLUTTER_ACTOR (priv->button))
    {
        applet_window_thumbnail_hover_peek (self, TRUE);
        guint32 current_time = cdos_global_get_current_time (cdos_global_get ());
        meta_window_delete (priv->meta_window, current_time);
        priv->stop_click = TRUE;
        return TRUE;
    }
    return FALSE;
}

static void
applet_window_thumbnail_hover_peek (AppletWindowThumbnail *self, gboolean make_above)
{
    AppletWindowThumbnailPrivate *priv = self->priv;
    if (!priv->enable_hover_peek)
    {
        return;
    }
    if (make_above)
    {
    }
    else
    {
    }
}

static ClutterActor *
applet_window_thumbnail_get_thumbnail_private (AppletWindowThumbnail *self)
{
    AppletWindowThumbnailPrivate *priv = self->priv;
    ClutterActor *thumbnail = NULL;
    GObject *meta_window_actor = NULL;
    meta_window_actor = meta_window_get_compositor_private (priv->meta_window);
    if (meta_window_actor)
    {
        gfloat width, height;
        MetaShapedTexture *tex;
        tex = meta_window_actor_get_texture (META_WINDOW_ACTOR (meta_window_actor));
        thumbnail = clutter_actor_new ();
        clutter_actor_get_size (thumbnail, &width, &height);
        priv->thumbnail_width = width;
        priv->thumbnail_height = height;
        clutter_actor_set_reactive (thumbnail, TRUE);
        clutter_actor_set_content (thumbnail, (ClutterContent *)tex);
    }
    return thumbnail;
}

static void
applet_window_thumbnail_refresh (AppletWindowThumbnail *self)
{
    AppletWindowThumbnailPrivate *priv = self->priv;
    priv->is_favorite_app = FALSE;
    const gchar *title = meta_window_get_title (priv->meta_window);
    st_label_set_text (ST_LABEL (priv->label), title);
    gboolean show_thumbs = !priv->no_thumbnail;

    if (META_IS_WINDOW (priv->meta_window))
    {
        if (show_thumbs && priv->meta_window)
        {
            priv->thumbnail = applet_window_thumbnail_get_thumbnail_private (self);
            st_bin_set_child (ST_BIN (priv->thumbnail_actor), priv->thumbnail);
            clutter_actor_show (CLUTTER_ACTOR (priv->thumbnail_actor));
        }
        if (show_thumbs == FALSE)
        {
            if (priv->thumbnail)
                clutter_actor_destroy (CLUTTER_ACTOR (priv->thumbnail));
            st_bin_set_child (ST_BIN (priv->thumbnail_actor), NULL);
            clutter_actor_hide (CLUTTER_ACTOR (priv->thumbnail_actor));
            priv->thumbnail = NULL;
        }
    }
    else
    {
        applet_window_thumbnail_window_missing (self);
    }
}

static void
applet_window_thumbnail_window_missing (AppletWindowThumbnail *self)
{
    AppletWindowThumbnailPrivate *priv = self->priv;
    st_label_set_text ((StLabel *)priv->label, _("The window is missing, maybe these was something wrong with the program!"));
    if (priv->thumbnail)
    {
        clutter_actor_destroy (priv->thumbnail);
        priv->thumbnail = NULL;
    }
    priv->error_icon_thumbnail = st_icon_new ();
    // priv->error_icon_title = st_icon_new ();
    st_icon_set_icon_name (ST_ICON (priv->error_icon_thumbnail), "error");
    // st_icon_set_name (ST_ICON (priv->error_icon_title), "error");
    // st_icon_set_icon_size (ST_ICON (priv->error_icon_title), 16);

    st_bin_set_child (ST_BIN (priv->thumbnail_actor), priv->error_icon_thumbnail);
    clutter_actor_show (CLUTTER_ACTOR (priv->thumbnail_actor));
    clutter_actor_hide (CLUTTER_ACTOR (priv->button));
}

AppletWindowThumbnail *
applet_window_thumbnail_new (CdosApp *app, gboolean is_favorite, MetaWindow *meta_window)
{
    if (!is_favorite && !meta_window)
    {
        return NULL;
    }
    AppletWindowThumbnail *self;
    self = g_object_new (APPLET_TYPE_WINDOW_THUMBNAIL, NULL);
    applet_window_thumbnail_setup (self, app, is_favorite, meta_window);
    return self;
}

gboolean
applet_window_thumbnail_need_reflush (AppletWindowThumbnail *self)
{
    g_return_val_if_fail (APPLET_IS_WINDOW_THUMBNAIL (self), FALSE);
    AppletWindowThumbnailPrivate *priv = self->priv;
    return priv->thumbnail == NULL ? FALSE : TRUE;
}

void
applet_window_thumbnail_is_favorite (AppletWindowThumbnail *self, gboolean is_added)
{
    g_return_if_fail (APPLET_IS_WINDOW_THUMBNAIL (self));
    AppletWindowThumbnailPrivate *priv = self->priv;
    gchar *app_text = NULL;
    if (is_added)
    {
        st_bin_set_child (ST_BIN (priv->thumbnail_actor), NULL);
        if (priv->is_favorite_app)
            app_text = applet_panel_launchers_get_display_name (priv->app);
        if (!app_text)
            app_text = g_strdup (cdos_app_get_name (priv->app));
        priv->thumbnail_width = APPLET_THUMBNAIL_ICON_SIZE + (int)ceil (sizeof (app_text) * 12);
        st_label_set_text (ST_LABEL (priv->label), app_text);
        priv->is_favorite_app = TRUE;
        clutter_actor_set_width (CLUTTER_ACTOR (priv->container1), priv->thumbnail_width);
        g_free (app_text);
    }
    else
    {
        applet_window_thumbnail_refresh (self);
    }
}

MetaWindow *
applet_window_thumbnail_get_meta_window (AppletWindowThumbnail *self)
{
    g_return_val_if_fail (APPLET_IS_WINDOW_THUMBNAIL (self), NULL);
    AppletWindowThumbnailPrivate *priv = self->priv;
    return priv->meta_window;
}

CdosApp *
applet_window_thumbnail_get_app (AppletWindowThumbnail *self)
{
    g_return_val_if_fail (APPLET_IS_WINDOW_THUMBNAIL (self), NULL);
    AppletWindowThumbnailPrivate *priv = self->priv;
    return priv->app;
}

StWidget *
applet_window_thumbnail_get_actor (AppletWindowThumbnail *self)
{
    g_return_val_if_fail (APPLET_IS_WINDOW_THUMBNAIL (self), NULL);
    AppletWindowThumbnailPrivate *priv = self->priv;
    return priv->actor;
}

void
applet_window_thumbnail_set_scale (AppletWindowThumbnail *self, gfloat scale)
{
    g_return_if_fail (APPLET_IS_WINDOW_THUMBNAIL (self));
    AppletWindowThumbnailPrivate *priv = self->priv;
    if (priv->thumbnail == NULL)
    {
        return;
    }
    if (scale <= 0)
    {
        gfloat w, h;
        clutter_actor_get_size (CLUTTER_ACTOR (priv->thumbnail_actor), &w, &h);
        scale = MIN (1.0, MIN (w / priv->thumbnail_width, h / priv->thumbnail_height));
    }
    priv->scale = scale;
    clutter_actor_set_width (priv->thumbnail, priv->thumbnail_width * (priv->scale));
    clutter_actor_set_height (priv->thumbnail, priv->thumbnail_height * (priv->scale));
}

gfloat
applet_window_thumbnail_get_scale (AppletWindowThumbnail *self)
{
    g_return_val_if_fail (APPLET_IS_WINDOW_THUMBNAIL (self), 0);
    AppletWindowThumbnailPrivate *priv = self->priv;
    if (priv->thumbnail == NULL)
    {
        return 0;
    }
    return priv->scale;
}

void
applet_window_thumbnail_set_no_thumbnail (AppletWindowThumbnail *self, gboolean no_thumbnail)
{
    g_return_if_fail (APPLET_IS_WINDOW_THUMBNAIL (self));
    AppletWindowThumbnailPrivate *priv = self->priv;
    priv->no_thumbnail = no_thumbnail;
    applet_window_thumbnail_refresh (self);
}
