/* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
#ifndef __CDOS_GLOBAL_H__
#define __CDOS_GLOBAL_H__

#include <clutter/clutter.h>
#include <glib-object.h>
#include <glib/gi18n.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <gtk/gtk.h>
#include <meta/meta-plugin.h>

#include "cdos-wm.h"
#include "st.h"
G_BEGIN_DECLS

#define CDOS_TYPE_GLOBAL (cdos_global_get_type ())
G_DECLARE_FINAL_TYPE (CdosGlobal, cdos_global, CDOS, GLOBAL, GObject)

CdosGlobal   *cdos_global_get                       (void);

ClutterStage  *cdos_global_get_stage                 (CdosGlobal *global);
MetaDisplay   *cdos_global_get_display               (CdosGlobal *global);
GList         *cdos_global_get_window_actors         (CdosGlobal *global);
GSettings     *cdos_global_get_settings              (CdosGlobal *global);
GSettings     *cdos_global_get_overrides_settings    (CdosGlobal *global);
guint32        cdos_global_get_current_time          (CdosGlobal *global);
MetaWorkspaceManager   *cdos_global_get_workspace_manager (CdosGlobal *global);

typedef enum {
  CDOS_ACTION_MODE_NONE          = 0,
  CDOS_ACTION_MODE_NORMAL        = 1 << 0,
  CDOS_ACTION_MODE_OVERVIEW      = 1 << 1,
  CDOS_ACTION_MODE_LOCK_SCREEN   = 1 << 2,
  CDOS_ACTION_MODE_UNLOCK_SCREEN = 1 << 3,
  CDOS_ACTION_MODE_SYSTEM_MODAL  = 1 << 5,
  CDOS_ACTION_MODE_POPUP         = 1 << 7,

  CDOS_ACTION_MODE_ALL = ~0,
} CdosActionMode;

/* Input/event handling */
gboolean cdos_global_begin_modal            (CdosGlobal         *global,
                                             guint32             timestamp,
                                             MetaModalOptions    options);
void     cdos_global_end_modal              (CdosGlobal         *global,
                                             guint32              timestamp);

void     cdos_global_set_stage_input_region (CdosGlobal         *global,
                                             GSList              *rectangles);

void    cdos_global_get_pointer             (CdosGlobal         *global,
                                             int                 *x,
                                             int                 *y,
                                             ClutterModifierType *mods);

typedef struct {
  guint glibc_uordblks;
} CdosMemoryInfo;

/* Run-at-leisure API */
void cdos_global_begin_work     (CdosGlobal          *global);
void cdos_global_end_work       (CdosGlobal          *global);

typedef void (*CdosLeisureFunction) (gpointer data);

void cdos_global_run_at_leisure (CdosGlobal          *global,
                                 CdosLeisureFunction  func,
                                 gpointer              user_data,
                                 GDestroyNotify        notify);


/* Misc utilities / Cdos API */
void     cdos_global_sync_pointer              (CdosGlobal  *global);

GAppLaunchContext *cdos_global_create_app_launch_context (CdosGlobal  *global,
                                                          guint32       timestamp,
                                                          int           workspace);

void     cdos_global_play_theme_sound          (CdosGlobal *global,
                                                guint        id,
                                                const char   *name,
                                                const char   *description,
                                                ClutterEvent *for_event);
void     cdos_global_play_theme_sound_full     (CdosGlobal  *global,
                                                guint         id,
                                                const char   *name,
                                                const char   *description,
                                                ClutterEvent *for_event,
                                                const char   *application_id,
                                                const char   *application_name);
void     cdos_global_play_sound_file           (CdosGlobal  *global,
                                                guint         id,
                                                const char   *file_name,
                                                const char   *description,
                                                ClutterEvent *for_event);
void     cdos_global_play_sound_file_full      (CdosGlobal  *global,
                                                guint         id,
                                                const char   *file_name,
                                                const char   *description,
                                                ClutterEvent *for_event,
                                                const char   *application_id,
                                                const char   *application_name);

void     cdos_global_cancel_theme_sound        (CdosGlobal  *global,
                                                guint         id);

void     cdos_global_init_xdnd                 (CdosGlobal  *global);

void     cdos_global_reexec_self               (CdosGlobal  *global);

void     cdos_global_log_structured            (const char *message,
                                                 const char *const *keys);

const char *     cdos_global_get_session_mode  (CdosGlobal  *global);

void     cdos_global_set_runtime_state         (CdosGlobal  *global,
                                                 const char   *property_name,
                                                 GVariant     *variant);
GVariant * cdos_global_get_runtime_state       (CdosGlobal  *global,
                                                 const char   *property_type,
                                                 const char   *property_name);

void     cdos_global_set_persistent_state      (CdosGlobal  *global,
                                                 const char   *property_name,
                                                 GVariant     *variant);
GVariant * cdos_global_get_persistent_state    (CdosGlobal  *global,
                                                 const char   *property_type,
                                                 const char   *property_name);
guint cdos_global_get_ui_scale (CdosGlobal *global);

ClutterActor *cdos_global_get_window_group (CdosGlobal *global);
ClutterActor *cdos_global_get_top_window_group (CdosGlobal *global);

CdosBaseWM  *cdos_global_get_window_manager (CdosGlobal *global);
StFocusManager *cdos_global_get_focus_manager (CdosGlobal *global);

void cdos_global_get_screen_size (CdosGlobal *global, gint *width, gint *height);

void display_current_time (const char* custom_msg);

gchar *cdos_global_get_md5_for_string (const gchar *string);

GdkWindow *cdos_global_get_im_window (CdosGlobal *global);

G_END_DECLS

#endif /* __CDOS_GLOBAL_H__ */
