#include "cdos-global.h"

#include "cdos-input-method.h"
#include "cdos-input-method-fcitx.h"

enum
{
    FOCUS_IN,
    FOCUS_OUT,
    RESET,
    N_SIGNALS
};

static guint signals[N_SIGNALS] = { 0 };

typedef struct _CdosInputMethod
{
    ClutterInputMethod parent;
    ClutterInputContentPurpose purpose;
    ClutterInputContentHintFlags hint;
}CdosInputMethod;

G_DEFINE_TYPE (CdosInputMethod, cdos_input_method, CLUTTER_TYPE_INPUT_METHOD)

static void
cdos_input_method_dispose (GObject *gobject)
{
    G_OBJECT_CLASS (cdos_input_method_parent_class)->dispose (gobject);
}

static void
cdos_input_method_focus_in (ClutterInputMethod *im, ClutterInputFocus *actor)
{
    g_signal_emit (im, signals[FOCUS_IN], 0);
}

static void
cdos_input_method_focus_out (ClutterInputMethod *im)
{
    g_signal_emit (im, signals[FOCUS_OUT], 0);
}

static void
cdos_input_method_reset (ClutterInputMethod *im)
{
    g_debug ("%s", __func__);
    g_signal_emit (im, signals[RESET], 0);
}

static void
cdos_input_method_set_cursor_location (ClutterInputMethod *im,
                                       const graphene_rect_t *rect)
{
    g_debug ("%s", __func__);

}

static void
cdos_input_method_set_surrounding (ClutterInputMethod *im,
                                   const gchar        *text,
                                   guint               cursor,
                                   guint               anchor)
{
    g_debug ("%s", __func__);
}

static void
cdos_input_method_update_content_hints (ClutterInputMethod *im,
                                        ClutterInputContentHintFlags hint)
{
    g_debug ("%s", __func__);

    CdosInputMethod *self = CDOS_INPUT_METHOD (im);
    self->hint = hint;
}

static void
cdos_input_method_update_content_purpose (ClutterInputMethod         *im,
                                          ClutterInputContentPurpose  purpose)
{
    g_debug ("%s", __func__);
    CdosInputMethod *self = CDOS_INPUT_METHOD (im);
    self->purpose = purpose;
}

static gboolean
cdos_input_method_filter_key_event (ClutterInputMethod *im,
                                    const ClutterEvent *key)
{
    g_debug ("%s", __func__);
    gboolean result = FALSE;

    /*
    CdosInputMethodPrivate *priv = cdos_input_method_get_instance_private (CDOS_INPUT_METHOD (im));
    ClutterEventType type = clutter_event_type (key);
    old_position = clutter_input_method_get_cursor_position (clutter_text);

    if (type == CLUTTER_KEY_PRESS && 
        clutter_text_get_cursor_position (clutter_text) != old_position)
    */

    return result;
}

static void
cdos_input_method_init (CdosInputMethod *self)
{
}

static void
cdos_input_method_class_init (CdosInputMethodClass *klass)
{
    GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
    gobject_class->dispose = cdos_input_method_dispose;

    ClutterInputMethodClass *parent_class = CLUTTER_INPUT_METHOD_CLASS (klass);
    parent_class->focus_in = cdos_input_method_focus_in;
    parent_class->focus_out = cdos_input_method_focus_out;
    parent_class->reset = cdos_input_method_reset;
    parent_class->set_cursor_location = cdos_input_method_set_cursor_location;
    parent_class->set_surrounding = cdos_input_method_set_surrounding;
    parent_class->update_content_hints = cdos_input_method_update_content_hints;
    parent_class->update_content_purpose = cdos_input_method_update_content_purpose;
    parent_class->filter_key_event = cdos_input_method_filter_key_event;

    signals[FOCUS_IN] =
        g_signal_new ("focus-in",
                      G_TYPE_FROM_CLASS (gobject_class),
                      G_SIGNAL_RUN_LAST,
                      0, NULL, NULL, NULL,
                      G_TYPE_NONE, 0);
    signals[FOCUS_OUT] =
        g_signal_new ("focus-out",
                      G_TYPE_FROM_CLASS (gobject_class),
                      G_SIGNAL_RUN_LAST,
                      0, NULL, NULL, NULL,
                      G_TYPE_NONE, 0);
    signals[RESET] =
        g_signal_new ("reset",
                      G_TYPE_FROM_CLASS (gobject_class),
                      G_SIGNAL_RUN_LAST,
                      0, NULL, NULL, NULL,
                      G_TYPE_NONE, 0);
}

ClutterInputMethod *
cdos_input_method_new (void)
{
    static ClutterInputMethod *im = NULL;

    if (im)
        return im;

    const char *im_module = g_getenv ("CLUTTER_IM_MODULE");
    if (!im_module)
        im_module = g_getenv ("GTK_IM_MODULE");
    if (G_UNLIKELY (!im_module))
    {
        g_warning ("CLUTTER_IM_MODULE or GTK_IM_MODULE is empty. Fallback to fcitx");
        im_module = "fcitx";
    }
    if (g_strcmp0 (im_module, "fcitx") == 0)
    {
        im = cdos_input_method_fcitx_new ();
    }
    else
        im = g_object_new (CDOS_TYPE_INPUT_METHOD, NULL);
    g_object_add_weak_pointer ((GObject *)im, (gpointer *)&im);
    return im;
}

ClutterInputMethod *
cdos_input_method_get (void)
{
    return cdos_input_method_new ();
}
