/* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */

#include "config.h"

#include <string.h>

#include <meta/meta-enum-types.h>
#include <meta/keybindings.h>

#include "cdos-wm-private.h"
#include "cdos-global.h"
#include "cdos-marshal.h"

struct _CdosBaseWM
{
    GObject parent;

    MetaPlugin *plugin;
};

/* Signals */
enum
{
    MINIMIZE,
    UNMINIMIZE,
    SIZE_CHANGE,
    MAP,
    DESTROY,
    SWITCH_WORKSPACE,
    KILL_SWITCH_WORKSPACE,
    KILL_WINDOW_EFFECTS,
    SHOW_TILE_PREVIEW,
    HIDE_TILE_PREVIEW,
    SHOW_WINDOW_MENU,
    FILTER_KEYBINDING,
    CONFIRM_DISPLAY_CHANGE,

    LAST_SIGNAL
};

G_DEFINE_TYPE (CdosBaseWM, cdos_base_wm, G_TYPE_OBJECT);

static guint cdos_base_wm_signals[LAST_SIGNAL] = {0};

static void
cdos_base_wm_init (CdosBaseWM *wm)
{
}

static void
cdos_base_wm_finalize (GObject *object)
{
    G_OBJECT_CLASS (cdos_base_wm_parent_class)->finalize (object);
}

static void
cdos_base_wm_class_init (CdosBaseWMClass *klass)
{
    GObjectClass *gobject_class = G_OBJECT_CLASS (klass);

    gobject_class->finalize = cdos_base_wm_finalize;

    cdos_base_wm_signals[MINIMIZE] = g_signal_new ("minimize",
                                                   G_TYPE_FROM_CLASS (klass),
                                                   G_SIGNAL_RUN_LAST,
                                                   0,
                                                   NULL,
                                                   NULL,
                                                   g_cclosure_marshal_VOID__OBJECT,
                                                   G_TYPE_NONE,
                                                   1,
                                                   META_TYPE_WINDOW_ACTOR);
    cdos_base_wm_signals[UNMINIMIZE] = g_signal_new ("unminimize",
                                                     G_TYPE_FROM_CLASS (klass),
                                                     G_SIGNAL_RUN_LAST,
                                                     0,
                                                     NULL,
                                                     NULL,
                                                     _cdos_marshal_VOID__OBJECT_INT_INT_INT_INT,
                                                     G_TYPE_NONE,
                                                     5,
                                                     META_TYPE_WINDOW_ACTOR,
                                                     G_TYPE_INT,
                                                     G_TYPE_INT,
                                                     G_TYPE_INT,
                                                     G_TYPE_INT);
    cdos_base_wm_signals[SIZE_CHANGE] = g_signal_new ("size-change",
                                                      G_TYPE_FROM_CLASS (klass),
                                                      G_SIGNAL_RUN_LAST,
                                                      0,
                                                      NULL,
                                                      NULL,
                                                      _cdos_marshal_VOID__OBJECT_INT_INT_INT_INT,
                                                      G_TYPE_NONE,
                                                      5,
                                                      META_TYPE_WINDOW_ACTOR,
                                                      G_TYPE_INT,
                                                      G_TYPE_INT,
                                                      G_TYPE_INT,
                                                      G_TYPE_INT);

    cdos_base_wm_signals[MAP] = g_signal_new ("map",
                                              G_TYPE_FROM_CLASS (klass),
                                              G_SIGNAL_RUN_LAST,
                                              0,
                                              NULL,
                                              NULL,
                                              g_cclosure_marshal_VOID__OBJECT,
                                              G_TYPE_NONE,
                                              1,
                                              META_TYPE_WINDOW_ACTOR);
    cdos_base_wm_signals[DESTROY] = g_signal_new ("destroy",
                                                  G_TYPE_FROM_CLASS (klass),
                                                  G_SIGNAL_RUN_LAST,
                                                  0,
                                                  NULL,
                                                  NULL,
                                                  g_cclosure_marshal_VOID__OBJECT,
                                                  G_TYPE_NONE,
                                                  1,
                                                  META_TYPE_WINDOW_ACTOR);
    cdos_base_wm_signals[SWITCH_WORKSPACE] = g_signal_new ("switch-workspace",
                                                           G_TYPE_FROM_CLASS (klass),
                                                           G_SIGNAL_RUN_LAST,
                                                           0,
                                                           NULL,
                                                           NULL,
                                                           _cdos_marshal_VOID__INT_INT_INT,
                                                           G_TYPE_NONE,
                                                           3,
                                                           G_TYPE_INT,
                                                           G_TYPE_INT,
                                                           G_TYPE_INT);
    cdos_base_wm_signals[KILL_SWITCH_WORKSPACE] = g_signal_new ("kill-switch-workspace",
                                                                G_TYPE_FROM_CLASS (klass),
                                                                G_SIGNAL_RUN_LAST,
                                                                0,
                                                                NULL,
                                                                NULL,
                                                                g_cclosure_marshal_VOID__VOID,
                                                                G_TYPE_NONE,
                                                                0);
    cdos_base_wm_signals[KILL_WINDOW_EFFECTS] = g_signal_new ("kill-window-effects",
                                                              G_TYPE_FROM_CLASS (klass),
                                                              G_SIGNAL_RUN_LAST,
                                                              0,
                                                              NULL,
                                                              NULL,
                                                              g_cclosure_marshal_VOID__OBJECT,
                                                              G_TYPE_NONE,
                                                              1,
                                                              META_TYPE_WINDOW_ACTOR);

    cdos_base_wm_signals[SHOW_TILE_PREVIEW] = g_signal_new ("show-tile-preview",
                                                            G_TYPE_FROM_CLASS (klass),
                                                            G_SIGNAL_RUN_LAST,
                                                            0,
                                                            NULL,
                                                            NULL,
                                                            NULL,
                                                            G_TYPE_NONE,
                                                            3,
                                                            META_TYPE_WINDOW,
                                                            META_TYPE_RECTANGLE,
                                                            G_TYPE_INT);
    cdos_base_wm_signals[HIDE_TILE_PREVIEW] = g_signal_new (
        "hide-tile-preview", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST, 0, NULL, NULL, NULL, G_TYPE_NONE, 0);
    cdos_base_wm_signals[SHOW_WINDOW_MENU] = g_signal_new ("show-window-menu",
                                                           G_TYPE_FROM_CLASS (klass),
                                                           G_SIGNAL_RUN_LAST,
                                                           0,
                                                           NULL,
                                                           NULL,
                                                           NULL,
                                                           G_TYPE_NONE,
                                                           3,
                                                           META_TYPE_WINDOW,
                                                           G_TYPE_INT,
                                                           META_TYPE_RECTANGLE);
    cdos_base_wm_signals[FILTER_KEYBINDING] = g_signal_new ("filter-keybinding",
                                                            G_TYPE_FROM_CLASS (klass),
                                                            G_SIGNAL_RUN_LAST,
                                                            0,
                                                            g_signal_accumulator_true_handled,
                                                            NULL,
                                                            NULL,
                                                            G_TYPE_BOOLEAN,
                                                            1,
                                                            META_TYPE_KEY_BINDING);
    cdos_base_wm_signals[CONFIRM_DISPLAY_CHANGE] = g_signal_new (
        "confirm-display-change", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST, 0, NULL, NULL, NULL, G_TYPE_NONE, 0);
}

void
_cdos_base_wm_switch_workspace (CdosBaseWM *wm, gint from, gint to, MetaMotionDirection direction)
{
    g_signal_emit (wm, cdos_base_wm_signals[SWITCH_WORKSPACE], 0, from, to, direction);
}

/**
 * cdos_base_wm_completed_switch_workspace:
 * @wm: CdosBaseWM对象
 *
 * 当完成交换工作区后调用该插件
 **/
void
cdos_base_wm_completed_switch_workspace (CdosBaseWM *wm)
{
    meta_plugin_switch_workspace_completed (wm->plugin);
}

/**
 * cdos_base_wm_completed_minimize:
 * @wm: CdosBaseWM对象
 * @actor: MetaWindowActor对象
 *
 * 当实现窗口最小化功能后调用该插件
 **/
void
cdos_base_wm_completed_minimize (CdosBaseWM *wm, MetaWindowActor *actor)
{
    meta_plugin_minimize_completed (wm->plugin, actor);
}

void
cdos_base_wm_completed_unminimize (CdosBaseWM *wm, MetaWindowActor *actor)
{
    meta_plugin_unminimize_completed (wm->plugin, actor);
}

void
cdos_base_wm_completed_size_change (CdosBaseWM *wm, MetaWindowActor *actor)
{
    meta_plugin_size_change_completed (wm->plugin, actor);
}

/**
 * cdos_base_wm_completed_tile:
 * @wm: CdosBaseWM对象
 * @actor: MetaWindowActor对象
 *
 * 当实现窗口标题功能后调用该插件
 **/

/*
void
cdos_base_wm_completed_tile  (CdosBaseWM         *wm,
                             MetaWindowActor *actor)
{
  meta_plugin_tile_completed (wm->plugin, actor);
}
*/

/**
 * cdos_base_wm_completed_map:
 * @wm: CdosBaseWM对象
 * @actor: MetaWindowActor对象
 *
 * 当完成窗口影射功能后调用该插件
 **/
void
cdos_base_wm_completed_map (CdosBaseWM *wm, MetaWindowActor *actor)
{
    meta_plugin_map_completed (wm->plugin, actor);
}

/**
 * cdos_base_wm_completed_destroy:
 * @wm: CdosBaseWM对象
 * @actor: MetaWindowActor对象
 *
 * 当完成窗口释放对象后调用该插件
 **/
void
cdos_base_wm_completed_destroy (CdosBaseWM *wm, MetaWindowActor *actor)
{
    meta_plugin_destroy_completed (wm->plugin, actor);
}

void
cdos_base_wm_completed_display_change (CdosBaseWM *wm, gboolean ok)
{
    meta_plugin_complete_display_change (wm->plugin, ok);
}

void
_cdos_base_wm_kill_switch_workspace (CdosBaseWM *wm)
{
    g_signal_emit (wm, cdos_base_wm_signals[KILL_SWITCH_WORKSPACE], 0);
}

void
_cdos_base_wm_kill_window_effects (CdosBaseWM *wm, MetaWindowActor *actor)
{
    g_signal_emit (wm, cdos_base_wm_signals[KILL_WINDOW_EFFECTS], 0, actor);
}

void
_cdos_base_wm_show_tile_preview (CdosBaseWM *wm, MetaWindow *window, MetaRectangle *tile_rect, int tile_monitor)
{
    g_signal_emit (wm, cdos_base_wm_signals[SHOW_TILE_PREVIEW], 0, window, tile_rect, tile_monitor);
}

void
_cdos_base_wm_minimize (CdosBaseWM *wm, MetaWindowActor *actor)
{
    g_signal_emit (wm, cdos_base_wm_signals[MINIMIZE], 0, actor);
}

void
_cdos_base_wm_hide_tile_preview (CdosBaseWM *wm)
{
    g_signal_emit (wm, cdos_base_wm_signals[HIDE_TILE_PREVIEW], 0);
}

void
_cdos_base_wm_show_window_menu (CdosBaseWM *wm, MetaWindow *window, MetaWindowMenuType menu, int x, int y)
{
    MetaRectangle rect;

    rect.x = x;
    rect.y = y;
    rect.width = rect.height = 0;

    _cdos_base_wm_show_window_menu_for_rect (wm, window, menu, &rect);
}

void
_cdos_base_wm_show_window_menu_for_rect (CdosBaseWM *wm,
                                         MetaWindow *window,
                                         MetaWindowMenuType menu,
                                         MetaRectangle *rect)
{
    g_signal_emit (wm, cdos_base_wm_signals[SHOW_WINDOW_MENU], 0, window, menu, rect);
}

void
_cdos_base_wm_unminimize (CdosBaseWM *wm, MetaWindowActor *actor)
{
    g_signal_emit (wm, cdos_base_wm_signals[UNMINIMIZE], 0, actor);
}

void
_cdos_base_wm_size_change (CdosBaseWM *wm,
                           MetaWindowActor *actor,
                           MetaSizeChange which_change,
                           MetaRectangle *old_frame_rect,
                           MetaRectangle *old_buffer_rect)
{
    g_signal_emit (wm, cdos_base_wm_signals[SIZE_CHANGE], 0, actor, which_change, old_frame_rect, old_buffer_rect);
}

void
_cdos_base_wm_map (CdosBaseWM *wm, MetaWindowActor *actor)
{
    g_signal_emit (wm, cdos_base_wm_signals[MAP], 0, actor);
}

void
_cdos_base_wm_destroy (CdosBaseWM *wm, MetaWindowActor *actor)
{
    g_signal_emit (wm, cdos_base_wm_signals[DESTROY], 0, actor);
}

gboolean
_cdos_base_wm_filter_keybinding (CdosBaseWM *wm, MetaKeyBinding *binding)
{
    gboolean rv;

    g_signal_emit (wm, cdos_base_wm_signals[FILTER_KEYBINDING], 0, binding, &rv);

    return rv;
}

void
_cdos_base_wm_confirm_display_change (CdosBaseWM *wm)
{
    g_signal_emit (wm, cdos_base_wm_signals[CONFIRM_DISPLAY_CHANGE], 0);
}

MetaPlugin *
cdos_base_wm_get_plugin (CdosBaseWM *wm)
{
    return wm->plugin;
}

/**
 * cdos_base_wm_new:
 * @plugin: MetaPlugin对象
 *
 * 创建新的窗口管理接口
 *
 * Return value: 新的窗口管理接口
 **/
CdosBaseWM *
cdos_base_wm_new (MetaPlugin *plugin)
{
    CdosBaseWM *wm;

    wm = g_object_new (CDOS_TYPE_BASE_WM, NULL);
    wm->plugin = plugin;

    return wm;
}
