/* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */

#include "config.h"

#ifdef HAVE_MALLINFO
#include <malloc.h>
#endif
#include <signal.h>
#include <stdlib.h>
#include <string.h>
#include <X11/Xlib.h>
#include <clutter/clutter.h>
#include <clutter/x11/clutter-x11.h>
#include <gdk/gdk.h>
#include <gdk/gdkx.h>
#include <gtk/gtk.h>
#include <meta/main.h>
#include <meta/util.h>
#include <meta/meta-plugin.h>
#include <meta/prefs.h>
#include <atk-bridge.h>

#include "cdos-debug.h"
#include "cdos-global.h"
#include "cdos-global-private.h"
#include "st.h"

extern GType cdos_plugin_get_type (void);
extern int debug_mode;
extern int debug_call;

#define CDOS_DBUS_SERVICE "org.freedesktop.Desktop"

#define WM_NAME "Cdos"
#define GNOME_WM_KEYBINDINGS "Gnome"

static guint dbus_identifier = 0;

/*static void
on_handle_exit_none (Xunli *skeleton,
			GDBusMethodInvocation *invocation,
			guint test){
	GVariant *value;
	
	xunli_complete_exit_none (skeleton, invocation);
}


static void
on_name_acquired (GDBusConnection *connection,
			const gchar *name,
			gpointer data){
	Xunli  *skeleton;

	skeleton = xunli_skeleton_new ();

	g_signal_connect (skeleton,
			   "handle-exit-none",
			   G_CALLBACK (on_handle_exit_none),
			   NULL);

	g_dbus_interface_skeleton_export (G_DBUS_INTERFACE_SKELETON (skeleton),
					   connection,
					   "/org/Cdos/Xunli",
					   NULL);

} */
static void
on_cdos_dbus_lost (GDBusConnection *connection,
                   const gchar *name,
                   gpointer data)
{
  g_bus_unown_name (dbus_identifier);
}

static void
cdos_dbus_init (gboolean replace)
{
  GDBusConnection *session;
  GDBusProxy *bus;
  GBusNameOwnerFlags flags;

  session = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, NULL);

  bus = g_dbus_proxy_new_sync (session,
                               0,
                               NULL,
                               "org.freedesktop.DBus",
                               "/org/freedesktop/DBus",
                               "org.freedesktop.DBus",
                               NULL, NULL);

  if (replace)
    flags = G_BUS_NAME_OWNER_FLAGS_ALLOW_REPLACEMENT |
            G_BUS_NAME_OWNER_FLAGS_REPLACE;
  dbus_identifier = g_bus_own_name_on_connection (session,
                                                  CDOS_DBUS_SERVICE,
                                                  flags,
                                                  NULL,
                                                  on_cdos_dbus_lost,
                                                  NULL,
                                                  NULL);
  if (!dbus_identifier)
    {
      g_warning ("failed to acquire org.Cdos");
      g_printerr ("%s already exists on bus and --replace not specified\n",
                  CDOS_DBUS_SERVICE);
      exit (1);
    }
  /* ...and the org.freedesktop.Notifications service; we always
   * specify REPLACE_EXISTING to ensure we kill off
   * notification-daemon if it was running.
   */
  dbus_identifier = g_bus_own_name_on_connection (session,
                                                  "org.freedesktop.Notifications",
                                                  G_BUS_NAME_OWNER_FLAGS_REPLACE,
                                                  NULL,
                                                  NULL,
                                                  NULL,
                                                  NULL);
  if (!dbus_identifier)
    {
      g_warning ("failed to acquire org.freedesktop.Notifications");
    }
  /* ...and the on-screen keyboard service 
  if (!dbus_g_proxy_call (bus, "RequestName", &error,
                          G_TYPE_STRING, "org.cdos.Caribou.Keyboard",
                          G_TYPE_UINT, DBUS_NAME_FLAG_REPLACE_EXISTING,
                          G_TYPE_INVALID,
                          G_TYPE_UINT, &request_name_result,
                          G_TYPE_INVALID))
    {
      g_print ("failed to acquire org.cdos.Caribou.Keyboard: %s\n", error->message);
    }
  */
  g_object_unref (bus);
}

static void 
sigroutine (int dunno)
{
	meta_quit(META_EXIT_SUCCESS);
}

static void
muted_log_handler (const char     *log_domain,
                   GLogLevelFlags  log_level,
                   const char     *message,
                   gpointer        data)
{
  /* Intentionally empty to discard message */
}

static gboolean
print_version (const gchar    *option_name,
               const gchar    *value,
               gpointer        data,
               GError        **error)
{
  g_print ("Cdos %s\n", VERSION);
  exit (0);
}

static gboolean
set_debug_mode (const gchar    *option_name,
                const gchar    *value,
                gpointer        data,
                GError        **error)
{
    g_setenv ("GALLIUM_HUD", ".x1600.y220GTT-usage,VRAM-usage,fps", TRUE);
    debug_mode = 1;
    return 1;
}

GOptionEntry gnome_cdos_options[] = {
  {
    "version", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK,
    print_version,
    N_("Print version"),
    NULL
  },
  {
    "debug-backtrace", 'b', G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK,
    set_debug_mode,
    N_("print verbose debug message"),
    NULL
  },
  {
    "debug-function-call", 'f', 0, G_OPTION_ARG_INT,
    &debug_call,
    N_("Print function call stack when crashed."),
    NULL
  },
  { NULL }
};

static void
cdos_a11y_init (void)
{
    cally_accessibility_init ();
    if (clutter_get_accessibility_enabled () == FALSE)
    {
        g_warning ("Accessibility: clutter has no accessibility enabled"
                   " skipping the atk-bridge load");
    }
    else
        atk_bridge_adaptor_init (NULL, NULL);
}

static void 
backtrace_symbols (int signo)
{
    debug_mode = 1;
    cdos_backtrace ();
    debug_mode = 0;
    exit (0);
}

/* Copy from gmessage.c */
static const gchar *
log_level_to_color (GLogLevelFlags log_level,
                    gboolean       use_color)
{
  /* we may not call _any_ GLib functions here */

  if (!use_color)
    return "";

  if (log_level & G_LOG_LEVEL_ERROR)
    return "\033[1;31m";
  else if (log_level & G_LOG_LEVEL_CRITICAL)
    return "\033[1;35m";
  else if (log_level & G_LOG_LEVEL_WARNING)
    return "\033[1;33m";
  else if (log_level & G_LOG_LEVEL_MESSAGE)
    return "\033[1;32m";
  else if (log_level & G_LOG_LEVEL_INFO)
    return "\033[1;32m";
  else if (log_level & G_LOG_LEVEL_DEBUG)
    return "\033[1;32m";

  /* No color for custom log levels. */
  return "";
}

static const gchar *
reset_color (gboolean use_color)
{
    if (!use_color)
        return "";
    else
        return "\033[0m";
}

static void
log_handler (const gchar *domain,
             GLogLevelFlags log_level,
             const gchar *msg,
             gpointer user_data)
{
    if (!debug_mode)
    {
        g_log_default_handler (domain, log_level, msg, NULL);
        return;
    }
    /* log flags */
    /*
       G_LOG_FLAG_RECURSION          = 1 << 0,
       G_LOG_FLAG_FATAL              = 1 << 1,

       G_LOG_LEVEL_ERROR             = 1 << 2,
       G_LOG_LEVEL_CRITICAL          = 1 << 3,
       G_LOG_LEVEL_WARNING           = 1 << 4,
       G_LOG_LEVEL_MESSAGE           = 1 << 5,
       G_LOG_LEVEL_INFO              = 1 << 6,
       G_LOG_LEVEL_DEBUG             = 1 << 7,
    */
    const gchar *msg_level[] = {
        "ERROR",
        "CRITICAL",
        "WARNING",
        "MESSAGE",
        "INFO",
        "DEBUG"
    };
    const char *level = msg_level[2];
    GLogLevelFlags l_level = log_level;
    if (l_level >= G_LOG_LEVEL_ERROR)
    {
        guint i = 0;
        l_level >>= 2;
        while (l_level >>= 1)
            i++;
        level = msg_level [i];
    }

    fprintf (stdout, "\n%s-%s%s%s **: %s\n", 
            domain, log_level_to_color(log_level,1),level, reset_color(1),  msg);

    //debug_mode = log_level & (G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_WARNING);

    if (log_level & (G_LOG_LEVEL_WARNING | G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_ERROR))
    {
        g_log_default_handler (domain, log_level, msg, NULL);
        cdos_backtrace ();
    }
    //debug_mode = 0;
}

static void
set_log_handler (void)
{
    const gchar *domains[] = {
        "GLib", "GLib-GObject", "GLib-GIO", "Gtk", "Gdk",
        "GThread", "Pango",
        "St", "Clutter",
        "Mutter",
        "libnm",
        "Desktop"
    };
    guint i;
    for (i = 0; i < G_N_ELEMENTS(domains); i++)
    {
        g_log_set_handler (domains[i], G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL |
            G_LOG_FLAG_RECURSION, log_handler, NULL);
    }
}

int
main (int argc, char **argv)
{
  GOptionContext *ctx;
  GError *error = NULL;
  int ecode;

  bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
  textdomain (GETTEXT_PACKAGE);

  g_setenv ("GDK_SCALE", "1", TRUE);

  ctx = meta_get_option_context ();
  g_option_context_add_main_entries (ctx, gnome_cdos_options, GETTEXT_PACKAGE);
  if (!g_option_context_parse (ctx, &argc, &argv, &error))
    {
      g_printerr ("%s: %s\n", argv[0], error->message);
      exit (1);
    }

  g_option_context_free (ctx);

  if (debug_mode || debug_call >= 1)
    signal (SIGSEGV, backtrace_symbols);

  set_log_handler ();
  meta_plugin_manager_set_plugin_type (cdos_plugin_get_type ());
  meta_set_wm_name (WM_NAME);
  meta_set_gnome_wm_keybindings (GNOME_WM_KEYBINDINGS);

  g_setenv ("CDOS_VERSION", VERSION, TRUE);

  /* Prevent meta_init() from causing gtk to load gail and at-bridge */
  g_setenv ("NO_AT_BRIDGE", "1", TRUE);
  meta_init ();
  g_unsetenv ("NO_AT_BRIDGE");

  cdos_dbus_init (meta_get_replace_current_wm ());

  cdos_a11y_init ();

  /* Disable debug spew from various libraries */
  g_log_set_handler ("Gvc", G_LOG_LEVEL_DEBUG,
                     muted_log_handler, NULL);
  g_log_set_handler ("AccountsService", G_LOG_LEVEL_DEBUG,
                     muted_log_handler, NULL);
  g_log_set_handler ("Bluetooth", G_LOG_LEVEL_DEBUG | G_LOG_LEVEL_MESSAGE,
                     muted_log_handler, NULL);

  /* Initialize the global object */
  _cdos_global_init (NULL);

  g_unsetenv ("GDK_SCALE");

  // add a siganl to exit the desktop 
  signal(SIGQUIT, sigroutine);

  ecode = meta_run ();

//  XDestroyImage(img);

  if (g_getenv ("CDOS_ENABLE_CLEANUP"))
    {
      g_printerr ("Doing final cleanup...\n");
      g_object_unref (cdos_global_get ());
    }

  return ecode;
}
