/* st-blur-effect.h
 *
 * Copyright 2019 Georges Basile Stavracas Neto <georges.stavracas@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 */

#pragma once

#include <clutter/clutter.h>
#include <cogl/cogl.h>

G_BEGIN_DECLS

/**
 * StBlurMode:
 * @ST_BLUR_MODE_ACTOR: blur the actor contents, and its children
 * @ST_BLUR_MODE_BACKGROUND: blur what's beneath the actor
 *
 * The mode of blurring of the effect.
 */
typedef enum
{
  ST_BLUR_MODE_ACTOR,
  ST_BLUR_MODE_BACKGROUND,
} StBlurMode;

#define ST_TYPE_BLUR_EFFECT (st_blur_effect_get_type())
G_DECLARE_FINAL_TYPE (StBlurEffect, st_blur_effect, ST, BLUR_EFFECT, ClutterEffect)

StBlurEffect *st_blur_effect_new (void);

int st_blur_effect_get_sigma (StBlurEffect *self);
void st_blur_effect_set_sigma (StBlurEffect *self, int sigma);

float st_blur_effect_get_brightness (StBlurEffect *self);
void st_blur_effect_set_brightness (StBlurEffect *self, float brightness);

StBlurMode st_blur_effect_get_mode (StBlurEffect *self);
void st_blur_effect_set_mode (StBlurEffect *self, StBlurMode mode);

G_END_DECLS