#ifndef __CDOS_APPLET_H__
#define __CDOS_APPLET_H__

#include <clutter/clutter.h>
#include "cdos-global.h"
#include "cdos-popup-menu.h"
#include "cdos-popup-menu-manager.h"

#include "st.h"
#include "cdos-extension.h"

GType cdos_applet_get_type(void);

#define CDOS_TYPE_APPLET                (cdos_applet_get_type ())
#define CDOS_APPLET(obj)                (G_TYPE_CHECK_INSTANCE_CAST ((obj), CDOS_TYPE_APPLET, CdosApplet))
#define CDOS_IS_APPLET(obj)             (G_TYPE_CHECK_INSTANCE_TYPE ((obj), CDOS_TYPE_APPLET))
#define CDOS_APPLET_CLASS(klass)        (G_TYPE_CHECK_CLASS_CAST ((klass), CDOS_TYPE_APPLET, CdosAppletClass))
#define CDOS_IS_APPLET_CLASS(klass)     (G_TYPE_CHECK_CLASS_TYPE ((klass), CDOS_TYPE_APPLET))
#define CDOS_APPLET_GET_CLASS(obj)      (G_TYPE_INSTANCE_GET_CLASS ((obj), CDOS_TYPE_APPLET, CdosAppletClass))

typedef struct _CdosApplet              CdosApplet;
typedef struct _CdosAppletPrivate       CdosAppletPrivate;
typedef struct _CdosAppletClass         CdosAppletClass;

struct _CdosApplet
{
    StWidget 	parent_instance;
    CdosAppletPrivate *priv;
    guint		timeout_id;
};

struct _CdosAppletClass
{
    StWidgetClass parent_class;
    gboolean (*on_applet_clicked) (CdosApplet *self);
    gboolean (*on_applet_button_press_cb) (CdosApplet *self, ClutterEvent *event, gpointer data);
    gboolean (*on_applet_added_to_panel) (CdosApplet *self, gboolean applets_loaded);
    gboolean (*on_applet_removed_from_panel) (CdosApplet *self);
    gboolean (*on_panel_height_changed) (void);
    void (*on_orientation_changed) (CdosApplet *self, StSide orientation);
};

StWidget *
cdos_applet_get_actor (CdosApplet *applet);

ClutterActor *cdos_applet_get_panel_location (CdosApplet *applet);
void cdos_applet_set_panel_location (CdosApplet *self, ClutterActor *panellocation);

ClutterActor *cdos_applet_get_new_panel_location (CdosApplet *applet);
void cdos_applet_set_new_panel_location (CdosApplet *applet, ClutterActor *new_panel_location);

gint cdos_applet_get_order (CdosApplet *applet);
void cdos_applet_set_order (CdosApplet *self, guint order);

gint cdos_applet_get_new_order (CdosApplet *applet);
void cdos_applet_set_new_order (CdosApplet *applet, gint new_order);

void cdos_applet_set_extension (CdosApplet *applet, CdosExtension *extension);
CdosExtension *cdos_applet_get_extension (CdosApplet *applet);

gchar *cdos_applet_get_uuid (CdosApplet *applet);
void cdos_applet_set_uuid (CdosApplet *applet, gchar *uuid);

void cdos_applet_set_applet_tooltip (CdosApplet *self, const gchar *text);

void cdos_applet_set_orientation (CdosApplet *applet, StSide orientation);

gfloat cdos_applet_get_panel_height (CdosApplet *applet);

void cdos_applet_set_panel_height (CdosApplet *applet, gfloat panel_height);

gboolean cdos_applet_get_scalemode (CdosApplet *self);
void cdos_applet_set_scalemode (CdosApplet *self, gboolean scalemode);

void cdos_applet_set_popup_menu_active (CdosApplet *self, gboolean active);
gboolean cdos_applet_get_popup_menu_active (CdosApplet *self);

CdosPopupMenu *
cdos_applet_get_popup_menu (CdosApplet *self);

CdosPopupMenuManager *cdos_applet_get_menu_manager (CdosApplet *self);

void 
cdos_applet_set_context_menu_active (CdosApplet *self, gboolean active);
gboolean 
cdos_applet_get_context_menu_active (CdosApplet *self);
CdosPopupMenu *
cdos_applet_get_context_menu (CdosApplet *self);

void cdos_applet_show (CdosApplet *self);
void cdos_applet_hide (CdosApplet *self);

#endif
