#ifndef __APP_SWITCHER_PRIVATE_H__
#define __APP_SWITCHER_PRIVATE_H__

#include "cdos-appswitcher.h"

#include "cdos-global.h"
#include "cdos-window-tracker.h"
#include "cdos-start.h"

#include <string.h>

typedef gboolean (*ShowFunc) (AppSwitcher *self);
typedef void (*HideFunc) (AppSwitcher *self);
typedef void (*OnDestroyFunc) (AppSwitcher *self);
typedef void (*CreateListFunc) (AppSwitcher *self);
typedef void (*UpdateListFunc) (AppSwitcher *self, int index);
typedef void (*SelectNextFunc) (AppSwitcher *self);
typedef void (*SelectPreviousFunc) (AppSwitcher *self);
typedef void (*OnWorkspaceSelectedFunc) (AppSwitcher *self, MetaWorkspace *workspace);
typedef gint (*CheckSwitchTimeFunc) (AppSwitcher *self);
typedef void (*SetCurrentWindowFunc) (AppSwitcher *self, MetaWindow *window);
typedef void (*DestroyActorsFunc) (AppSwitcher *self);

struct _AppSwitcher
{
    ClutterActor *actor; // 由子类实现
    gulong initial_delay_timeout_id;
    MetaKeyBinding *binding;
    GList *windows; // MetaWindow
    GList *previews; // preview actor
    gboolean have_modal;
    gulong motion_timeout_id;
    gulong check_destroyed_timeout_id;
    GList *current_index;
    guint modifier_mask;
    CdosWindowTracker *tracker;
    CdosBaseWM *window_manager;
    gulong dcid;
    gulong mcid;

    gboolean enforce_primary_monitor;
    const MetaRectangle *active_monitor;

    gboolean mouse_active;
    gint mouse_timeout;

    ShowFunc show;
    HideFunc hide;
    OnDestroyFunc on_destroy;
    CreateListFunc create_list;
    UpdateListFunc update_list;
    SelectNextFunc select_next;
    SelectPreviousFunc select_previous;
    OnWorkspaceSelectedFunc on_workspace_selected;
    CheckSwitchTimeFunc check_switch_time;
    SetCurrentWindowFunc set_current_window;
    DestroyActorsFunc destroy_actors;
};

void appswitcher_init (AppSwitcher *switcher, MetaKeyBinding *binding);
void cdos_appswitcher_destroy (AppSwitcher *switcher);

void window_destroyed (CdosBaseWM *wm, MetaWindowActor *actor, AppSwitcher *switcher);
void appswitcher_activate_selected (CdosBaseWM *wm, MetaWindowActor *actor, AppSwitcher *switcher);
gboolean appswitcher_destroy (ClutterActor *actor, ClutterEvent *event, AppSwitcher *switcher);

gboolean appswitcher_setup_modal (AppSwitcher *switcher);

void appswitcher_next (AppSwitcher *switcher);
void appswitcher_previous (AppSwitcher *switcher);
#endif // __APP_SWITCHER_PRIVATE_H__
