/**
 * SECTION: cdos-auto-mount-manager
 * @short_description: 管理自动挂载
 * @include: cdos-auto-mount-manager.h
 *
 * CdosAutoMountManagerClass 类用来管理自动挂载
 */
#include "cdos-auto-mount-manager.h"
#include "cdos-mount-operation.h"
#include "cdos-sound-manager.h"

G_DEFINE_TYPE (CdosAutoMountManager, cdos_auto_mount_manager, G_TYPE_OBJECT);


#define SETTINGS_SCHEMA "org.cdos.desktop.media-handling"
#define SETTING_ENABLE_AUTOMOUNT "automount"

#define AUTORUN_EXPIRE_TIMEOUT_SECS 10

static void
allowAutorun (GVolume *volume)
{
    // volume.allowAutorun = true;
}

static gboolean
vol_timeout_cb (GVolume *volume)
{
    // volume.allowAutorun = false;
    return FALSE;
}


static void
allowAutorunExpire (GVolume *volume)
{
    g_timeout_add_seconds (AUTORUN_EXPIRE_TIMEOUT_SECS, (GSourceFunc)vol_timeout_cb, volume);
}

// callback of mountVolume
static void
onVolumeMounted_cb (GObject *source_object, GAsyncResult *res, gpointer user_data)
{
    GError *error = NULL;

    GVolume *volume = G_VOLUME (source_object);

    allowAutorunExpire (volume);

    if (!g_volume_mount_finish (volume, res, &error))
    {
        // FIXME: needs proper error code handling instead of this
        // See https://bugzilla.gnome.org/show_bug.cgi?id=591480
        /*if (string.indexOf('No key available with this passphrase') != -1)
                        reaskPassword (volume);
                else
                        g_warning ("Unable to mount volume %s", g_volume_get_name (volume) )
                */
        g_warning ("Unable to mount volume %s: %s", g_volume_get_name (volume), error->message);
        g_error_free (error);
    }
}

static void
mountVolume (GVolume *volume, GMountOperation *operation)
{
    allowAutorun (volume);

    g_volume_mount (volume, 0, operation, NULL, onVolumeMounted_cb, NULL);
}

static void
checkAndMountVolume (CdosAutoMountManager *self, GVolume *volume, gboolean checkSession, gboolean useMountOp)
{
    if (checkSession == -1)
        checkSession = TRUE;
    if (useMountOp == -1)
        useMountOp = TRUE;

    if (checkSession)
    {
        if (self->is_active)
        {
            if (g_list_index (self->volumeQueue, volume) == -1)
                self->volumeQueue = g_list_append (self->volumeQueue, volume);

            return;
        }
    }
    // Volume is already mounted, don't bother.
    if (g_volume_get_mount (volume))
        return;

    if (!g_settings_get_boolean (self->settings, SETTING_ENABLE_AUTOMOUNT) ||
        !g_volume_should_automount (volume) ||
        !g_volume_can_mount (volume))
    {
        // allow the autorun to run anyway; this can happen if the
        // mount gets added programmatically later, even if
        // should_automount() or can_mount() are false, like for
        // blank optical media.
        allowAutorun (volume);
        allowAutorunExpire (volume);

        return;
    }
    // mountVolume (volume, NULL);
    // notice:暂时关闭掉自动挂载打弹窗，由文件管理器入口处进行相关的检测和处理;
    // if (useMountOp) {
    // //operation = new CdosBaseMountOperation.CdosBaseMountOperation(volume);
    //     	//mountVolume ( volume, operation.mountOp);
    //     CdosMountOperation *operation = cdos_mount_operation_new (volume, -1);
    // 	mountVolume (volume, G_MOUNT_OPERATION(operation));
    // 	g_object_unref(operation);
    // }
    // else
    // {
    //    g_print ("auto mount %d\n", __LINE__);
    //    	mountVolume (volume, NULL);
    // }
    mountVolume (volume, NULL);
}


static void
vol_iter_func (gpointer data, gpointer user_data)
{
    CdosAutoMountManager *self = CDOS_AUTO_MOUNT_MANAGER (user_data);

    GVolume *volume = G_VOLUME (data);

    checkAndMountVolume (self, volume, 0, 1);
}


static void
screenSaverActiveChanged (GObject *gobject, gboolean isActive, gpointer user_data)
{
    CdosAutoMountManager *self = CDOS_AUTO_MOUNT_MANAGER (user_data);

    if (!isActive)
    {
        g_list_foreach (self->volumeQueue, (GFunc)vol_iter_func, self);
    }
    g_list_free (self->volumeQueue);
    // clear the queue anyway
    self->volumeQueue = NULL;
}

static void
onVolumeAdded (GObject *gobject, GVolume *volume, gpointer user_data)
{
    CdosAutoMountManager *self = CDOS_AUTO_MOUNT_MANAGER (user_data);

    // GVolumeMonitor *monitor = G_VOLUME_MONITOR (gobject);
    g_debug ("%s %s: %s", __FILE__, __func__, g_volume_get_name (volume));

    checkAndMountVolume (self, volume, -1, 1);
}

static void
onVolumeRemoved (GObject *gobject, GVolume *volume, gpointer user_data)
{
    // GVolumeMonitor *monitor = G_VOLUME_MONITOR (gobject);
    CdosAutoMountManager *self = CDOS_AUTO_MOUNT_MANAGER (user_data);
    GList *temp = NULL;
    GList *vq = self->volumeQueue;
    if (vq)
    {
        while (vq != NULL)
        {
            GVolume *v = vq->data;
            if (v != volume)
            {
                temp = g_list_append (temp, v);
            }
            vq = vq->next;
        }
        g_list_free (vq);
    }
    self->volumeQueue = temp;
}


static void
onDriveConnected (GObject *gobject, GDrive *drive, gpointer user_data)
{
    CdosAutoMountManager *self = CDOS_AUTO_MOUNT_MANAGER (user_data);
    if (self->is_active)
        return;

    cdos_sound_manager_play ("plug");
}


static void
onDriveDisconnected (GObject *gobject, GDrive *drive, gpointer user_data)
{
    CdosAutoMountManager *self = CDOS_AUTO_MOUNT_MANAGER (user_data);

    if (self->is_active)
        return;

    cdos_sound_manager_play ("unplug");
}


static void
drive_stop_cb (GObject *source_object, GAsyncResult *res, gpointer user_data)
{
    GDrive *drive = G_DRIVE (source_object);

    if (!g_drive_stop_finish (drive, res, NULL))
        g_warning ("Unable to stop the drive after drive-eject-button");
}


static void
drive_eject_cb (GObject *source_object, GAsyncResult *res, gpointer user_data)
{
    GDrive *drive = G_DRIVE (source_object);

    if (!g_drive_stop_finish (drive, res, NULL))
        g_warning ("Unable to eject the drive after drive-eject-button");
}


static void
onDriveEjectButton (GObject *gobject, GDrive *drive, gpointer user_data)
{
    // TODO: this code path is not tested, as the GVfs volume monitor
    // doesn't emit this signal just yet.

    // we force stop/eject in this case, so we don't have to pass a
    // mount operation object
    if (g_drive_can_stop (drive))
    {
        g_drive_stop (drive, G_MOUNT_UNMOUNT_FORCE, NULL, NULL, drive_stop_cb, NULL);
    }
    else if (g_drive_can_eject (drive))
    {
        g_drive_eject_with_operation (drive, G_MOUNT_UNMOUNT_FORCE, NULL, NULL, drive_eject_cb, NULL);
    }
}

static void
iter_func_mountall (gpointer data, gpointer user_data)
{
    CdosAutoMountManager *self = CDOS_AUTO_MOUNT_MANAGER (user_data);

    GVolume *volume = G_VOLUME (data);

    // this._checkAndMountVolume(volume, { checkSession: false,
    //                                        useMountOp: false });
    checkAndMountVolume (self, volume, 0, 0);
}


static gboolean
startupMountAll (gpointer user_data)
{
    CdosAutoMountManager *self = CDOS_AUTO_MOUNT_MANAGER (user_data);

    GList *volumes = g_volume_monitor_get_volumes (self->volumeMonitor);

    g_list_foreach (volumes, (GFunc)iter_func_mountall, self);
    g_list_free (volumes);
    return FALSE;
}

static void
cdos_auto_mount_manager_dispose (GObject *gobject)
{
    CdosAutoMountManager *self = CDOS_AUTO_MOUNT_MANAGER (gobject);
    g_clear_object (&self->settings);
    g_clear_object (&self->ss_proxy);
    g_clear_object (&self->volumeMonitor);

    if (self->volumeQueue != NULL)
    {
        GList *v = self->volumeQueue;
        while (v != NULL)
        {
            if (v->data)
            {
                g_object_unref (v->data);
            }
            v = v->next;
        }
        g_list_free (self->volumeQueue);
    }
    G_OBJECT_CLASS (cdos_auto_mount_manager_parent_class)->dispose (gobject);
}

static void
cdos_auto_mount_manager_finalize (GObject *gobject)
{
    G_OBJECT_CLASS (cdos_auto_mount_manager_parent_class)->finalize (gobject);
}



static void
cdos_auto_mount_manager_class_init (CdosAutoMountManagerClass *klass)
{
    GObjectClass *gobject_class = G_OBJECT_CLASS (klass);

    gobject_class->finalize = cdos_auto_mount_manager_finalize;
    gobject_class->dispose = cdos_auto_mount_manager_dispose;
}


static void
cdos_auto_mount_manager_init (CdosAutoMountManager *self)
{
    self->settings = g_settings_new (SETTINGS_SCHEMA);

    self->volumeQueue = NULL;

    self->ss_proxy = cdos_screen_saver_get ();
    if (self->ss_proxy)
        g_signal_connect (self->ss_proxy, "active-changed", G_CALLBACK (screenSaverActiveChanged), self);

    self->volumeMonitor = g_volume_monitor_get ();

    g_signal_connect (self->volumeMonitor, "volume-added", G_CALLBACK (onVolumeAdded), self);
    g_signal_connect (self->volumeMonitor, "volume-removed", G_CALLBACK (onVolumeRemoved), self);
    g_signal_connect (self->volumeMonitor, "drive-connected", G_CALLBACK (onDriveConnected), self);
    g_signal_connect (self->volumeMonitor, "drive-disconnected", G_CALLBACK (onDriveDisconnected), self);
    g_signal_connect (self->volumeMonitor, "drive-eject-button", G_CALLBACK (onDriveEjectButton), self);
    g_idle_add ((GSourceFunc)startupMountAll, self);
}

/**
 * cdos_auto_mount_manager_new:
 *
 * 新建一个 #CdosAutoMountManager 类的实例。
 *
 * Returns: 返回一个 #CdosAutoMountManager 的实例。
 */
CdosAutoMountManager *
cdos_auto_mount_manager_new (void)
{
    return g_object_new (CDOS_TYPE_AUTO_MOUNT_MANAGER, NULL);
}
