/*
 * SECTION: cdos-drag-action
 * @Title: cdos-drag-action
 * @Short_Description:
 */

#include <stdlib.h>
#include "st.h"
#include "cdos-marshal.h"
#include "cdos-drag-drop.h"
#include "cdos-start.h"

gboolean drag_drop_successfull = FALSE;

static ClutterDragAction *current_drag_action = NULL;

ClutterDragAction *
cdos_get_current_drag_action (void)
{
    return current_drag_action;
}

void
cdos_set_current_drag_action (ClutterDragAction *action)
{
    current_drag_action = action;
}

static void
on_drag_begin (ClutterDragAction   *action,
               ClutterActor        *actor,
               gfloat               event_x,
               gfloat               event_y,
               ClutterModifierType  modifiers,
               gpointer             user_data)
{
    current_drag_action = action;
    gfloat pos_x, pos_y;
    gfloat width, height;
    clutter_actor_get_transformed_position (actor, &pos_x, &pos_y);

    ClutterActor *stage = clutter_actor_get_stage (actor);
    ClutterActor *handle = clutter_drag_action_get_drag_handle (action);
    if (!handle)
    {
        ClutterActor *clone = clutter_clone_new (actor);
        handle = (ClutterActor *)st_bin_new ();
        st_bin_set_child ((StBin *)handle, clone);
        st_bin_set_fill ((StBin *)handle, TRUE, TRUE);
        /*
        handle = clutter_actor_new ();
        clutter_actor_add_child (handle, clone);
        */
        clutter_actor_get_transformed_size (actor, &width, &height);
        clutter_actor_set_size (handle, width, height);
        clutter_actor_set_position (handle, pos_x, pos_y);
        clutter_actor_add_child (stage, handle);
        clutter_drag_action_set_drag_handle ((ClutterDragAction *)action, handle);
    }
    else
        clutter_actor_set_position (handle, event_x, event_y);
    if (clutter_actor_get_parent (handle) != stage)
    {
        clutter_actor_add_child (stage, handle);
    }
    clutter_actor_show (handle);
    clutter_actor_set_child_above_sibling (stage, handle, NULL);
    g_signal_handlers_disconnect_by_func (handle, clutter_actor_hide, NULL);

    clutter_actor_save_easing_state (handle);
    clutter_actor_set_easing_mode (handle, CLUTTER_LINEAR);
    clutter_actor_set_opacity (handle, 128);
    clutter_actor_restore_easing_state (handle);

    cdos_push_modal (actor, 0, 0, CDOS_ACTION_MODE_NONE);
    drag_drop_successfull = FALSE;
}

static void
on_drag_end (ClutterDragAction   *action,
             ClutterActor        *actor,
             gfloat               event_x,
             gfloat               event_y,
             ClutterModifierType  modifiers,
             gpointer             user_data)
{
    cdos_pop_modal (actor, 0);

    ClutterActor *handle = clutter_drag_action_get_drag_handle (action);
    clutter_actor_save_easing_state (handle);

    if (!drag_drop_successfull)
    {
        gfloat pos_x, pos_y;
        clutter_actor_get_transformed_position (actor, &pos_x, &pos_y);
        clutter_actor_save_easing_state (handle);
        clutter_actor_set_easing_mode (handle, CLUTTER_LINEAR);
        clutter_actor_set_opacity (handle, 255);
        clutter_actor_set_position (handle, pos_x, pos_y);
        clutter_actor_restore_easing_state (handle);
    }
    else
    {
        clutter_actor_save_easing_state (handle);
        clutter_actor_set_easing_mode (handle, CLUTTER_LINEAR);
        clutter_actor_set_opacity (handle, 255);
        clutter_actor_restore_easing_state (handle);
    }

    g_signal_connect (handle, "transitions-completed",
                      G_CALLBACK (clutter_actor_hide), NULL);
    current_drag_action = NULL;
}

ClutterAction *
cdos_drag_action_new (ClutterActor *actor)
{
    g_return_val_if_fail (CLUTTER_IS_ACTOR (actor), NULL);

    ClutterAction *action = clutter_drag_action_new ();
    clutter_drag_action_set_drag_threshold ((ClutterDragAction *)action, 3, 3);
    g_signal_connect (action, "drag-begin", G_CALLBACK (on_drag_begin), actor);
    g_signal_connect (action, "drag-end", G_CALLBACK (on_drag_end), actor);

    clutter_actor_set_reactive (actor, TRUE);
    clutter_actor_add_action (actor, action);

    return action;
}

/*
 * CdosDropAction:
 *
 */

/*
 * 复制自 clutter-drop-action.c, on_stage_capture 做了一些修改。
 */

struct _CdosDropActionPrivate
{
    ClutterActor *stage;
    ClutterActor *actor;
    gulong mapped_id;
};

G_DEFINE_TYPE_WITH_PRIVATE (CdosDropAction, cdos_drop_action, CLUTTER_TYPE_ACTION);

static ClutterActor *
_cdos_stage_get_pointer_drag_actor (ClutterStage       *stage,
                                    ClutterInputDevice *device)
{
  GHashTable *drag_actors;

  drag_actors = g_object_get_data (G_OBJECT (stage),
                                   "__clutter_stage_pointer_drag_actors");
  if (drag_actors == NULL)
    return NULL;

  return g_hash_table_lookup (drag_actors, device);
}

static ClutterActor *
_cdos_stage_get_touch_drag_actor (ClutterStage         *stage,
                                  ClutterEventSequence *sequence)
{
  GHashTable *drag_actors;

  drag_actors = g_object_get_data (G_OBJECT (stage),
                                   "__clutter_stage_touch_drag_actors");
  if (drag_actors == NULL)
    return NULL;

  return g_hash_table_lookup (drag_actors, sequence);
}

typedef struct _DropTarget
{
  ClutterActor *stage;
  gulong capture_id;
  GHashTable *actions;
  CdosDropAction *last_action;
}DropTarget;

enum
{
  CAN_DROP,
  OVER_IN,
  OVER_OUT,
  DROP,
  DROP_CANCEL,

  LAST_SIGNAL
};

static guint drop_signals[LAST_SIGNAL] = { 0, };

static void
drop_target_free (gpointer _data)
{
  DropTarget *data = _data;

  g_signal_handler_disconnect (data->stage, data->capture_id);
  g_hash_table_destroy (data->actions);
  g_free (data);
}

static gboolean
on_stage_capture (ClutterStage *stage,
                  ClutterEvent *event,
                  gpointer      user_data)
{
  DropTarget *data = user_data;
  gfloat event_x, event_y;
  ClutterActor *actor, *drag_actor;
  CdosDropAction *drop_action;
  ClutterInputDevice *device;
  gboolean was_reactive;
  device = clutter_event_get_device (event);
  drag_actor = _cdos_stage_get_pointer_drag_actor (stage, device);
  switch (clutter_event_type (event))
    {
    case CLUTTER_MOTION:
    case CLUTTER_BUTTON_RELEASE:
      if (clutter_event_type (event) == CLUTTER_MOTION &&
          !(clutter_event_get_state (event) & CLUTTER_BUTTON1_MASK))
        return CLUTTER_EVENT_PROPAGATE;

      if (clutter_event_type (event) == CLUTTER_BUTTON_RELEASE &&
          clutter_event_get_button (event) != CLUTTER_BUTTON_PRIMARY)
        return CLUTTER_EVENT_PROPAGATE;

      device = clutter_event_get_device (event);
      drag_actor = _cdos_stage_get_pointer_drag_actor (stage, device);
      if (drag_actor == NULL)
        return CLUTTER_EVENT_PROPAGATE;
      break;

    case CLUTTER_TOUCH_UPDATE:
    case CLUTTER_TOUCH_END:
      drag_actor = _cdos_stage_get_touch_drag_actor (stage,
                                                     clutter_event_get_event_sequence (event));
      if (drag_actor == NULL)
        return CLUTTER_EVENT_PROPAGATE;
      break;

    default:
      return CLUTTER_EVENT_PROPAGATE;
    }

  clutter_event_get_coords (event, &event_x, &event_y);

  /* get the actor under the cursor, excluding the dragged actor; we
   * use reactivity because it won't cause any scene invalidation
   */
  was_reactive = clutter_actor_get_reactive (drag_actor);
  clutter_actor_set_reactive (drag_actor, FALSE);

  actor = clutter_stage_get_actor_at_pos (stage, CLUTTER_PICK_REACTIVE,
                                          event_x,
                                          event_y);
  if (actor == NULL || actor == CLUTTER_ACTOR (stage))
    {
      if (data->last_action != NULL)
        {
          ClutterActorMeta *meta = CLUTTER_ACTOR_META (data->last_action);

          g_signal_emit (data->last_action, drop_signals[OVER_OUT], 0,
                         clutter_actor_meta_get_actor (meta));

          data->last_action = NULL;
        }

      goto out;
    }

  drop_action = g_hash_table_lookup (data->actions, actor);

  if (drop_action == NULL)
    {
      if (data->last_action != NULL)
        {
          ClutterActorMeta *meta = CLUTTER_ACTOR_META (data->last_action);
          ClutterActor *last_actor = clutter_actor_meta_get_actor (meta);

          if (!clutter_actor_contains (last_actor, actor))
            {
              g_signal_emit (data->last_action, drop_signals[OVER_OUT], 0, last_actor);
              data->last_action = NULL;
            }
        }
      else
        {
          ClutterActor *parent = actor;
          while ((parent = clutter_actor_get_parent (parent)) != (ClutterActor *)stage)
            {
              drop_action = g_hash_table_lookup (data->actions, parent);
              if (drop_action)
                {
                  g_signal_emit (drop_action, drop_signals[OVER_IN], 0, parent);
                  data->last_action = drop_action;
                  break;
                }
            }
        }

      goto out;
    }
  else
    {
      if (data->last_action != drop_action)
        {
          ClutterActorMeta *meta;

          if (data->last_action != NULL)
            {
              meta = CLUTTER_ACTOR_META (data->last_action);

              g_signal_emit (data->last_action, drop_signals[OVER_OUT], 0,
                             clutter_actor_meta_get_actor (meta));
            }

          meta = CLUTTER_ACTOR_META (drop_action);

          g_signal_emit (drop_action, drop_signals[OVER_IN], 0,
                         clutter_actor_meta_get_actor (meta));
        }

      data->last_action = drop_action;
    }

out:
  if (clutter_event_type (event) == CLUTTER_BUTTON_RELEASE ||
      clutter_event_type (event) == CLUTTER_TOUCH_END)
    {
      if (data->last_action != NULL)
        {
          ClutterActorMeta *meta = CLUTTER_ACTOR_META (data->last_action);
          gboolean can_drop = FALSE;

          g_signal_emit (data->last_action, drop_signals[CAN_DROP], 0,
                         clutter_actor_meta_get_actor (meta),
                         event_x, event_y,
                         &can_drop);

          if (can_drop)
            {
              g_signal_emit (data->last_action, drop_signals[DROP], 0,
                             clutter_actor_meta_get_actor (meta),
                             event_x, event_y);
            }
          else
            {
              g_signal_emit (data->last_action, drop_signals[DROP_CANCEL], 0,
                             clutter_actor_meta_get_actor (meta),
                             event_x, event_y);
            }

        }

      data->last_action = NULL;
    }

  if (drag_actor != NULL)
    clutter_actor_set_reactive (drag_actor, was_reactive);

  return CLUTTER_EVENT_PROPAGATE;
}

static void
drop_action_register (CdosDropAction *self)
{
  CdosDropActionPrivate *priv = self->priv;
  DropTarget *data;

  g_assert (priv->stage != NULL);

  data = g_object_get_data (G_OBJECT (priv->stage), "__cdos_drop_targets");
  if (data == NULL)
    {
      data = g_new0 (DropTarget, 1);

      data->stage = priv->stage;
      data->actions = g_hash_table_new (NULL, NULL);
      data->capture_id = g_signal_connect (priv->stage, "captured-event",
                                           G_CALLBACK (on_stage_capture),
                                           data);
      g_object_set_data_full (G_OBJECT (priv->stage), "__cdos_drop_targets",
                              data,
                              drop_target_free);
    }

  g_hash_table_replace (data->actions, priv->actor, self);
}

static void
drop_action_unregister (CdosDropAction *self)
{
  CdosDropActionPrivate *priv = self->priv;
  DropTarget *data = NULL;

  if (priv->stage != NULL)
    data = g_object_get_data (G_OBJECT (priv->stage), "__cdos_drop_targets");

  if (data == NULL)
    return;

  g_hash_table_remove (data->actions, priv->actor);
  if (g_hash_table_size (data->actions) == 0)
    g_object_set_data (G_OBJECT (data->stage), "__cdos_drop_targets", NULL);
}

static void
on_actor_mapped (ClutterActor      *actor,
                 GParamSpec        *pspec,
                 CdosDropAction *self)
{
  if (clutter_actor_is_mapped (actor))
    {
      if (self->priv->stage == NULL)
        self->priv->stage = clutter_actor_get_stage (actor);

      drop_action_register (self);
    }
  else
    drop_action_unregister (self);
}

static void
cdos_drop_action_set_actor (ClutterActorMeta *meta,
                            ClutterActor     *actor)
{
  CdosDropActionPrivate *priv = CDOS_DROP_ACTION (meta)->priv;

  if (priv->actor != NULL)
    {
      drop_action_unregister (CDOS_DROP_ACTION (meta));

      if (priv->mapped_id != 0)
        g_signal_handler_disconnect (priv->actor, priv->mapped_id);

      priv->stage = NULL;
      priv->actor = NULL;
      priv->mapped_id = 0;
    }

  priv->actor = actor;

  if (priv->actor != NULL)
    {
      priv->stage = clutter_actor_get_stage (actor);
      priv->mapped_id = g_signal_connect (actor, "notify::mapped",
                                          G_CALLBACK (on_actor_mapped),
                                          meta);

      if (priv->stage != NULL)
        drop_action_register (CDOS_DROP_ACTION (meta));
    }
  CLUTTER_ACTOR_META_CLASS (cdos_drop_action_parent_class)->set_actor (meta, actor);
}

static gboolean
signal_accumulator (GSignalInvocationHint *ihint,
                    GValue                *return_accu,
                    const GValue          *handler_return,
                    gpointer               user_data)
{
  gboolean continue_emission;

  continue_emission = g_value_get_boolean (handler_return);
  g_value_set_boolean (return_accu, continue_emission);

  return continue_emission;
}

static gboolean
cdos_drop_action_real_can_drop (CdosDropAction *action,
                                ClutterActor      *actor,
                                gfloat             event_x,
                                gfloat             event_y)
{
  return TRUE;
}

static void
cdos_drop_action_init (CdosDropAction *self)
{
    self->priv = cdos_drop_action_get_instance_private (self);
}

static void
cdos_drop_action_class_init (CdosDropActionClass *klass)
{
  ClutterActorMetaClass *meta_class = CLUTTER_ACTOR_META_CLASS (klass);

  meta_class->set_actor = cdos_drop_action_set_actor;

  klass->can_drop = cdos_drop_action_real_can_drop;

  /**
   * CdosDropAction::can-drop:
   * @action: the #CdosDropAction that emitted the signal
   * @actor: the #ClutterActor attached to the @action
   * @event_x: the X coordinate (in stage space) of the drop event
   * @event_y: the Y coordinate (in stage space) of the drop event
   *
   * The ::can-drop signal is emitted when the dragged actor is dropped
   * on @actor. The return value of the ::can-drop signal will determine
   * whether or not the #CdosDropAction::drop signal is going to be
   * emitted on @action.
   *
   * The default implementation of #CdosDropAction returns %TRUE for
   * this signal.
   *
   * Return value: %TRUE if the drop is accepted, and %FALSE otherwise
   *
   * Since: 1.8
   */
  drop_signals[CAN_DROP] =
    g_signal_new ("can-drop",
                  G_TYPE_FROM_CLASS (klass),
                  G_SIGNAL_RUN_LAST,
                  G_STRUCT_OFFSET (CdosDropActionClass, can_drop),
                  signal_accumulator, NULL,
                  _cdos_marshal_BOOLEAN__OBJECT_FLOAT_FLOAT,
                  G_TYPE_BOOLEAN, 3,
                  CLUTTER_TYPE_ACTOR,
                  G_TYPE_FLOAT,
                  G_TYPE_FLOAT);

  /**
   * CdosDropAction::over-in:
   * @action: the #CdosDropAction that emitted the signal
   * @actor: the #ClutterActor attached to the @action
   *
   * The ::over-in signal is emitted when the dragged actor crosses
   * into @actor.
   *
   * Since: 1.8
   */
  drop_signals[OVER_IN] =
    g_signal_new ("over-in",
                  G_TYPE_FROM_CLASS (klass),
                  G_SIGNAL_RUN_LAST,
                  G_STRUCT_OFFSET (CdosDropActionClass, over_in),
                  NULL, NULL,
                  NULL,
                  G_TYPE_NONE, 1,
                  CLUTTER_TYPE_ACTOR);

  /**
   * CdosDropAction::over-out:
   * @action: the #CdosDropAction that emitted the signal
   * @actor: the #ClutterActor attached to the @action
   *
   * The ::over-out signal is emitted when the dragged actor crosses
   * outside @actor.
   *
   * Since: 1.8
   */
  drop_signals[OVER_OUT] =
    g_signal_new ("over-out",
                  G_TYPE_FROM_CLASS (klass),
                  G_SIGNAL_RUN_LAST,
                  G_STRUCT_OFFSET (CdosDropActionClass, over_out),
                  NULL, NULL,
                  NULL,
                  G_TYPE_NONE, 1,
                  CLUTTER_TYPE_ACTOR);

  /**
   * CdosDropAction::drop:
   * @action: the #CdosDropAction that emitted the signal
   * @actor: the #ClutterActor attached to the @action
   * @event_x: the X coordinate (in stage space) of the drop event
   * @event_y: the Y coordinate (in stage space) of the drop event
   *
   * The ::drop signal is emitted when the dragged actor is dropped
   * on @actor. This signal is only emitted if at least an handler of
   * #CdosDropAction::can-drop returns %TRUE.
   *
   * Since: 1.8
   */
  drop_signals[DROP] =
    g_signal_new ("drop",
                  G_TYPE_FROM_CLASS (klass),
                  G_SIGNAL_RUN_LAST,
                  G_STRUCT_OFFSET (CdosDropActionClass, drop),
                  NULL, NULL,
                  _cdos_marshal_VOID__OBJECT_FLOAT_FLOAT,
                  G_TYPE_NONE, 3,
                  CLUTTER_TYPE_ACTOR,
                  G_TYPE_FLOAT,
                  G_TYPE_FLOAT);


  /**
   * CdosDropAction::drop-cancel:
   * @action: the #CdosDropAction that emitted the signal
   * @actor: the #ClutterActor attached to the @action
   * @event_x: the X coordinate (in stage space) of the drop event
   * @event_y: the Y coordinate (in stage space) of the drop event
   *
   * The ::drop-cancel signal is emitted when the drop is refused
   * by an emission of the #CdosDropAction::can-drop signal.
   *
   * After the ::drop-cancel signal is fired the active drag is
   * terminated.
   *
   * Since: 1.12
   */
  drop_signals[DROP_CANCEL] =
    g_signal_new ("drop-cancel",
                  G_TYPE_FROM_CLASS (klass),
                  G_SIGNAL_RUN_LAST,
                  G_STRUCT_OFFSET (CdosDropActionClass, drop),
                  NULL, NULL,
                  _cdos_marshal_VOID__OBJECT_FLOAT_FLOAT,
                  G_TYPE_NONE, 3,
                  CLUTTER_TYPE_ACTOR,
                  G_TYPE_FLOAT,
                  G_TYPE_FLOAT);
}

ClutterAction *
cdos_drag_action_new_with_handle (ClutterActor *actor, ClutterActor *handle)
{
    g_return_val_if_fail (CLUTTER_IS_ACTOR (actor), NULL);
    g_return_val_if_fail (CLUTTER_IS_ACTOR (handle), NULL);

    ClutterAction *action = cdos_drag_action_new (actor);
    if (action)
    {
        clutter_drag_action_set_drag_handle ((ClutterDragAction *)action, handle);
    }
    return action;
}

ClutterAction *
cdos_drop_action_new (ClutterActor *actor)
{
    g_return_val_if_fail (CLUTTER_IS_ACTOR (actor), NULL);

    ClutterAction *action;
    action = g_object_new (CDOS_TYPE_DROP_ACTION, NULL);
    clutter_actor_set_reactive (actor, TRUE);
    clutter_actor_add_action (actor, action);

    return action;
}
