#ifndef __CDOS_EXTENSION_H__
#define __CDOS_EXTENSION_H__

#include <glib.h>
#include <glib-object.h>

#include "st.h"

G_BEGIN_DECLS

GType cdos_extension_get_type (void);

#define CDOS_TYPE_EXTENSION (cdos_extension_get_type ())
#define CDOS_EXTENSION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), CDOS_TYPE_EXTENSION, CdosExtension))
#define CDOS_IS_EXTENSION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), CDOS_TYPE_EXTENSION))
#define CDOS_EXTENSION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), CDOS_TYPE_EXTENSION, CdosExtensionClass))

typedef struct _CdosExtension CdosExtension;
typedef struct _CdosExtensionClass CdosExtensionClass;


enum
{
    CDOS_EXTENSION_MODULE_MENU = 1,
    CDOS_EXTENSION_MODULE_SCALE,
    CDOS_EXTENSION_MODULE_WINDOW_LIST,
    CDOS_EXTENSION_MODULE_NOTIFICATIONS,
    CDOS_EXTENSION_MODULE_NOTIFICATIONS2,
    CDOS_EXTENSION_MODULE_WORKSPACE,
    CDOS_EXTENSION_MODULE_REMOVABLE_DRIVES,
    CDOS_EXTENSION_MODULE_KEYBOARD,
    CDOS_EXTENSION_MODULE_BLUETOOTH,
    CDOS_EXTENSION_MODULE_NETWORK,
    CDOS_EXTENSION_MODULE_SOUND,
    CDOS_EXTENSION_MODULE_POWER,
    CDOS_EXTENSION_MODULE_SYSTRAY,
    CDOS_EXTENSION_MODULE_CALENDAR,
    CDOS_EXTENSION_MODULE_SHOW_DESKTOP,
    CDOS_EXTENSION_MODULE_FEEDBACK,

    CDOS_EXTENSION_MODULE_PANEL_LAUNCHERS,
    CDOS_EXTENSION_MODULE_SETTINGS,
    CDOS_EXTENSION_MODULE_EXPO,
    CDOS_EXTENSION_MODULE_GROUPED_APPLETS,
    NUM_CDOS_EXTENSION_MODULE
};

typedef struct _CdosApplet CdosApplet;
typedef struct _MetaData MetaData;

struct _MetaData
{
    gchar *name;
    gchar *uuid;
    gchar *role;
    gchar *description;
    guint state;
    gchar *path;
    gint max_instances;
    gchar *error;
};

struct _CdosExtension
{
    GObject parent_instance;

    gchar *uuid;
    gchar *lower_type;
    gint module; //代表一个具体的applet， enum CDOS_EXTENSION_MODULE_*
    GFile *dir;
    time_t start_time;
    CdosApplet *role_provider;
    GHashTable *loaded_definitions;
    MetaData *meta;
    StTheme *theme;
    gchar *style_sheet;
    gboolean loaded_menu;
};

struct _CdosExtensionClass
{
    GObjectClass parent_class;
    void (*status_menu_loaded) (CdosExtension *extension, CdosApplet *applet);
};


CdosExtension *cdos_extension_new (const gchar *uuid);

CdosExtension *cdos_extension_load_extension (const gchar *uuid);

void cdos_extension_unload_extension (const gchar *uuid);

CdosApplet *
cdos_extension_module_main (CdosExtension *extension, StSide orientation, gfloat panel_height, gchar *applet_id);

gint cdos_extension_get_max_instances (const gchar *uuid);

MetaData *cdos_extension_get_meta (CdosExtension *extension);

/* Maps uuid -> metadata object */
GHashTable *cdos_extension_get_extension_meta (void);


GFile *cdos_extension_get_dir (CdosExtension *extenison);

gchar *cdos_extension_get_uuid (CdosExtension *extension);

CdosExtension *cdos_extension_get_object_by_uuid (const gchar *uuid);

GHashTable *cdos_extension_get_loaded_definitions (CdosExtension *extension);
void cdos_extension_set_loaded_definitions (CdosExtension *extension, GHashTable *loaded_definitions);

G_END_DECLS

#endif /* __CDOS_EXTENSION_H__ */
