#include <clutter/clutter.h>
#include <gio/gio.h>
#include "cdos-icon-applet.h"
#include "st.h"
#include "cdos-start.h"

/**
 * SECTION:cdos-icon-applet
 * @Title: CdosIconApplet
 * @Short_description: 加载和显示图标
 *
 * CdosIconApplet类是作为其他类的基类,功能为加载和显示图标
 */

struct _CdosIconAppletPrivate
{
    ClutterActor  *applet_icon;
    const gchar   *icon_name;
    ClutterActor  *badge;
};

G_DEFINE_TYPE_WITH_PRIVATE (CdosIconApplet, cdos_iconapplet, CDOS_TYPE_APPLET);

/**
 * cdos_iconapplet_set_icon_size:
 * @self: #CdosIconApplet,需要图标的applet
 * @icon_name: 图标的大小
 *
 * 设置applet图标的大小
 *
 * Returns: void
 */
void cdos_iconapplet_set_icon_size (CdosIconApplet    *self,const gint     size)
{
    g_return_if_fail (CDOS_IS_APPLET (self));
    CdosIconAppletPrivate *priv;
    priv = self->priv;
    if (priv->applet_icon)
    {
        st_icon_set_icon_size (ST_ICON (priv->applet_icon), size);
    }
}

/**
 * cdos_iconapplet_set_icon_name:
 * @self: #CdosIconApplet,需要图标的applet
 * @icon_name: 图标的名称
 *
 * 设置applet图标的名称以及类型
 *
 * Returns: void
 */
void
cdos_iconapplet_set_icon_name (CdosIconApplet *self, const gchar *icon_name)
{
    g_return_if_fail (CDOS_IS_ICONAPPLET (self));
    CdosIconAppletPrivate *priv ;
    priv = self->priv ;
    if (priv->applet_icon)
    {
        st_icon_set_icon_name (ST_ICON (priv->applet_icon), icon_name);
    }
    else
    {
        priv->applet_icon = g_object_new(ST_TYPE_ICON,
                                         "icon_name", icon_name,
                                         "icon-size", 24,
                                         "reactive", TRUE,
                                         "track_hover", TRUE,
                                         "style_class", "applet-icon",
                                         NULL);
        st_bin_set_child (self->applet_icon_box, priv->applet_icon);
    }
    priv->icon_name = icon_name;
}


/**
 * cdos_iconapplet_set_applet_icon_gion:
 * @self: #CdosIconApplet,需要图标的applet
 * @icon: 图标
 *
 * 设置applet图标的名称以及类型
 *
 * Returns: void
 */
void
cdos_iconapplet_set_applet_icon_gion (CdosIconApplet *self, GIcon *gicon)
{
    g_return_if_fail(CDOS_IS_ICONAPPLET(self));
    CdosIconAppletPrivate *priv ;
    priv = self->priv ;
    if(priv->applet_icon)
    {
        st_icon_set_gicon ((StIcon *)priv->applet_icon, gicon);
    }
}

/**
 * cdos_iconapplet_set_icon_path:
 * @self: #CdosIconApplet,需要图标的applet
 * @icon_path: 图标的路径
 *
 * 设置applet图标的路径以及类型
 *
 * Returns: void
 */
void
cdos_iconapplet_set_icon_path (CdosIconApplet   *self,
                               const char     *icon_path)
{
    g_return_if_fail(CDOS_IS_ICONAPPLET(self));
    CdosIconAppletPrivate *priv ;
    priv = self->priv;
    //CdosGlobal  *global;
    //guint ui_scale;
    global = cdos_global_get();
    // global->ui_scale = 1;
    if (icon_path)
    {
        GFile *file;
        GIcon *gicon;
        file = g_file_new_for_path (icon_path);
        //StIconType type = ST_ICON_FULLCOLOR;
        gicon = g_file_icon_new (file);
        if(priv->applet_icon)
        {
            st_icon_set_gicon (ST_ICON (priv->applet_icon), gicon);
        }
        else
        {
            priv->applet_icon = g_object_new ( ST_TYPE_ICON,
                                               "gicon", gicon,
                                               "icon-size", 24,
                                               "reactive", TRUE,
                                               "track_hover", TRUE,
                                               "style_class", "applet-icon",
                                               NULL);

        }
        st_bin_set_child (self->applet_icon_box, priv->applet_icon);
        g_object_unref (gicon);
        g_object_unref (file);
    }
    priv->icon_name = icon_path;
}

static void
on_panel_height_changed (CdosIconApplet *self)
{
    g_return_if_fail (CDOS_IS_ICONAPPLET (self));
    CdosIconAppletPrivate *priv;
    priv = self->priv;
    // StBin       *applet_icon_box;
    global = cdos_global_get();

    if(st_bin_get_child(self->applet_icon_box))
    {
        clutter_actor_destroy (st_bin_get_child (self->applet_icon_box));
    }

    cdos_iconapplet_set_icon_name (self, priv->icon_name);
}


/**
 * cdos_iconapplet_set_icon_symbolic_path:
 * @self: #CdosIconApplet,需要图标的applet
 * @icon_path: 图标的路径
 *
 * 设置applet图标的系统路径以及类型
 *
 * Returns: void
 */
void
cdos_iconapplet_set_icon_symbolic_path (CdosIconApplet    *self,
                                        const char      *icon_path)
{
    g_return_if_fail (CDOS_IS_ICONAPPLET (self));
    CdosIconAppletPrivate *priv ;
    priv = self->priv;

    //CdosGlobal  *global;
    //guint           ui_scale;
    global = cdos_global_get();

    if(icon_path)
    {
        GFile *file;
        GIcon *gicon;
        file = g_file_new_for_path(icon_path);
        gicon = g_file_icon_new(file);
        if(priv->applet_icon)
        {
            st_icon_set_gicon(ST_ICON(priv->applet_icon),gicon);
        }
        else
        {
            priv->applet_icon = g_object_new ( ST_TYPE_ICON,
                                               "gicon",gicon,
                                               "icon-size", 18,
                                               "reactive",TRUE,
                                               "track_hover",TRUE,
                                               "style_class","applet-icon",
                                               NULL);
            st_bin_set_child(self->applet_icon_box,priv->applet_icon) ;
        }
        g_object_unref(gicon);
        g_object_unref(file);
    }
    priv->icon_name = icon_path;
    on_panel_height_changed(self);
}

static void
cdos_iconapplet_get_preferred_width (ClutterActor *actor,
                                     gfloat        for_height,
                                     gfloat       *min_width_p,
                                     gfloat       *natural_width_p)
{
    StThemeNode *theme_node = st_widget_get_theme_node ((StWidget *)actor);
    st_theme_node_adjust_for_height (theme_node, &for_height);

    gfloat icon_min_w, icon_nat_w;
    gfloat badge_min_w = 0, badge_nat_w = 0;
    gfloat badge_min_h = 0, badge_nat_h = 0;

    CdosIconApplet *self = CDOS_ICONAPPLET (actor);
    CdosIconAppletPrivate *priv = self->priv;
    clutter_actor_get_preferred_width ((ClutterActor *)self->applet_icon_box, for_height, &icon_min_w, &icon_nat_w);
    if (priv->badge && clutter_actor_is_visible (priv->badge))
    {
        clutter_actor_get_preferred_width (priv->badge, for_height, &badge_min_w, &badge_nat_w);
        clutter_actor_get_preferred_height (priv->badge, -1, &badge_min_h, &badge_nat_h);

        if (badge_min_w < badge_min_h)
            badge_min_w = badge_min_h;
        if (badge_nat_w < badge_nat_h)
            badge_nat_w = badge_nat_h;
    }
    *min_width_p = icon_min_w + badge_min_w;
    *natural_width_p = icon_nat_w + badge_nat_w;

    st_theme_node_adjust_preferred_width (theme_node, min_width_p, natural_width_p);
}

static void
cdos_iconapplet_allocate (ClutterActor *actor,
                          const ClutterActorBox *box,
                          ClutterAllocationFlags flags)
{
    StThemeNode *theme_node = st_widget_get_theme_node ((StWidget *)actor);
    ClutterActorBox content_box;

    clutter_actor_set_allocation (actor, box, flags);

    st_theme_node_get_content_box (theme_node, box, &content_box);

    CdosIconApplet *self = CDOS_ICONAPPLET (actor);
    CdosIconAppletPrivate *priv = self->priv;

    ClutterActorBox icon_box;
    ClutterActor *child = CLUTTER_ACTOR (self->applet_icon_box);
    gfloat width = content_box.x2 - content_box.x1;
    gfloat height = content_box.y2 - content_box.y1;
    {
        gfloat min_w, nat_w;
        gfloat min_h, nat_h;
        clutter_actor_get_preferred_size (child, &min_w, &min_h, &nat_w, &nat_h);
        icon_box.x1 = content_box.x1 + (int)(width - min_w) / 2;
        icon_box.x2 = icon_box.x1 + min_w;
        icon_box.y1 = content_box.y1 + (int)(height - min_h) / 2;
        icon_box.y2 = icon_box.y1 + min_h;
        clutter_actor_allocate (child, &icon_box, flags);
    }

    if (priv->applet_icon && priv->badge && clutter_actor_is_visible (priv->badge))
    {
        ClutterActorBox badge_box;
        gfloat min_w, min_h, nat_w, nat_h;
        //clutter_actor_get_preferred_size (priv->badge, &min_w, &min_h, &nat_w, &nat_h);
        clutter_actor_get_preferred_width (priv->badge, -1, &min_w, &nat_w);
        clutter_actor_get_preferred_height (priv->badge, -1, &min_h, &nat_h);
        if (min_w < min_h)
            min_w = min_h;
        badge_box.x1 = icon_box.x2 - (int)(min_w / 2);
        badge_box.x2 = badge_box.x1 + min_w;
        badge_box.y1 = icon_box.y1 - (int)(min_h / 2);
        badge_box.y2 = badge_box.y1 + min_h;
        clutter_actor_allocate (priv->badge, &badge_box, flags);
    }
}

static void
cdos_iconapplet_class_init (CdosIconAppletClass *klass)
{
    /*
    GObjectClass *object_class = G_OBJECT_CLASS (klass);
    object_class->finalize = cdos_iconapplet_finalize;
    object_class->dispose = cdos_iconapplet_dispose;
    */

    ClutterActorClass *actor_class = CLUTTER_ACTOR_CLASS (klass);
    actor_class->get_preferred_width = cdos_iconapplet_get_preferred_width;
    actor_class->allocate = cdos_iconapplet_allocate;
}

static void
cdos_iconapplet_init (CdosIconApplet *self)
{
    CdosIconAppletPrivate *priv;
    priv = self->priv = cdos_iconapplet_get_instance_private (self);
    self->applet_icon_box = g_object_new (ST_TYPE_BIN, NULL);
    clutter_actor_add_child ((ClutterActor *)self, CLUTTER_ACTOR (self->applet_icon_box));
    priv->applet_icon = NULL;
}

/**
 * cdos_iconapplet_get_applet_icon_box:
 * @self: #CdosIconApplet,需要图标的applet
 *
 * 获取类CdosIconApplet的变量applet_icon_box
 * Returns: applet_icon_box
 */
StBin *
cdos_iconapplet_get_applet_icon_box (CdosIconApplet *self)
{
    g_return_val_if_fail(CDOS_IS_ICONAPPLET(self),NULL);
    //CdosIconAppletPrivate *priv;

    //priv = CDOS_ICONAPPLET_GET_PRIVATE(self);
    return self->applet_icon_box;
}


/**
 * cdos_iconapplet_new:
 * @guint: panel的高度
 *
 * 根据panel的高度创建新的iconapplet
 * Returns: （CdosIconApplet）iconapplet
 */
CdosIconApplet *
cdos_iconapplet_new(guint panel_height)
{
    CdosIconApplet *iconapplet;
    iconapplet = g_object_new(CDOS_TYPE_ICONAPPLET,NULL);

    return iconapplet;
}

void
cdos_iconapplet_reset_for_systray (CdosIconApplet *self)
{
    g_return_if_fail (CDOS_IS_ICONAPPLET (self));
    CdosIconAppletPrivate *priv = self->priv;
    st_widget_set_track_hover (ST_WIDGET (self), FALSE);
    st_widget_set_track_hover (ST_WIDGET (self->applet_icon_box),TRUE);
    clutter_actor_set_reactive (CLUTTER_ACTOR (self->applet_icon_box), TRUE);

    st_widget_add_style_class_name(ST_WIDGET(self),"applet-systray");
    st_widget_add_style_class_name(ST_WIDGET(self->applet_icon_box),"applet-systray-box");
    if(priv->applet_icon){
        st_widget_add_style_class_name(ST_WIDGET(priv->applet_icon),"applet-systray-icon");
    }
}

ClutterActor *
cdos_iconapplet_get_applet_icon (CdosIconApplet *self)
{
    g_return_val_if_fail (CDOS_IS_ICONAPPLET (self), NULL);
    CdosIconAppletPrivate *priv = self->priv;
    return priv->applet_icon;
}

void
cdos_iconapplet_set_badge (CdosIconApplet *self, const char *badge)
{
    g_return_if_fail (CDOS_IS_ICONAPPLET (self));

    CdosIconAppletPrivate *priv = self->priv;
    if (!priv->badge && badge)
    {
        StWidget *bin = st_bin_new ();
        StWidget *actor = st_label_new (badge);
        ClutterText *text = (ClutterText *)st_label_get_clutter_text (ST_LABEL (actor));
        clutter_text_set_ellipsize (text, PANGO_ELLIPSIZE_NONE);
        priv->badge = CLUTTER_ACTOR (bin);
        st_bin_set_child (ST_BIN (bin), (ClutterActor *)actor);
        st_widget_add_style_class_name (bin, "applet-badge");
        clutter_actor_add_child (CLUTTER_ACTOR (self), priv->badge);
    }
    else if (priv->badge && badge)
    {
        st_label_set_text (ST_LABEL (st_bin_get_child ((StBin *)priv->badge)), badge);
        clutter_actor_show (priv->badge);
    }

    if (!badge && priv->badge)
        clutter_actor_hide (priv->badge);
}

StWidget *
cdos_iconapplet_get_badge (CdosIconApplet *self)
{
    g_return_val_if_fail (CDOS_IS_ICONAPPLET (self), NULL);

    return (StWidget *)self->priv->badge;
}
