#include "config.h"
#include "cdos-icon-dispatcher.h"
#include "cdos-marshal.h"
#include "cdos-tray-manager.h"
#include "cdos-ui-plugin.h"

/**
 * SECTION:cdos-icon-dispatcher
 * @Title: CdosIconDispatcher
 * @Short_description: 监听 CdosTrayManager 的添加移除信号
 * @image:enenen.png
 *
 * CdosIconDispatcher 负责信号的转发，用于面板右下角第三方应用的图标显示
 * 在信号转发之前，它会将这些小应用按角色分类
 */
G_DEFINE_TYPE (CdosIconDispatcher, cdos_icon_dispatcher, G_TYPE_OBJECT);

enum
{
    STATUS_ICON_ADDED,
    STATUS_ICON_REMOVED,
    MESSAGE_ICON_ADDED,
    MESSAGE_ICON_REMOVED,
    BEFORE_REDISPLAY, // zhangdj
    AFTER_REDISPLAY,
    LAST_SIGNAL
};

static guint cdos_icon_dispatcher_signals[LAST_SIGNAL] = {0};

static void on_tray_icon_added (CdosTrayManager *traymanager, ClutterActor *icon, gpointer user_data);
static void on_tray_icon_removed (CdosTrayManager *traymanager, ClutterActor *icon, gpointer user_data);

static void
cdos_icon_dispatcher_dispose (GObject *gobject)
{
    CdosIconDispatcher *self = CDOS_ICON_DISPATCHER (gobject);

    g_clear_object (&self->traymanager);

    G_OBJECT_CLASS (cdos_icon_dispatcher_parent_class)->dispose (gobject);
}

static void
cdos_icon_dispatcher_class_init (CdosIconDispatcherClass *klass)
{
    GObjectClass *gobject_class = G_OBJECT_CLASS (klass);

    gobject_class->dispose = cdos_icon_dispatcher_dispose;

    /**
        * CdosIconDispatcher::status-icon-added:
        * @actor: 发出这个信号的对象
        *
        * 当 CdosTrayManager 对象发出 tray-icon-added 信号之后，
        * @actor 监听到这个信号，转发出 ::status-icon-added 信号
        */
    cdos_icon_dispatcher_signals[STATUS_ICON_ADDED] =
        g_signal_new ("status-icon-added",
                      G_TYPE_FROM_CLASS (klass),
                      G_SIGNAL_RUN_LAST,
                      G_STRUCT_OFFSET (CdosIconDispatcherClass, status_icon_added),
                      NULL,
                      NULL,
                      _cdos_marshal_VOID__OBJECT_STRING,
                      G_TYPE_NONE,
                      2,
                      CLUTTER_TYPE_ACTOR,
                      G_TYPE_STRING);

    cdos_icon_dispatcher_signals[STATUS_ICON_REMOVED] =
        g_signal_new ("status-icon-removed",
                      G_TYPE_FROM_CLASS (klass),
                      G_SIGNAL_RUN_LAST,
                      G_STRUCT_OFFSET (CdosIconDispatcherClass, status_icon_removed),
                      NULL,
                      NULL,
                      g_cclosure_marshal_VOID__OBJECT,
                      G_TYPE_NONE,
                      1,
                      CLUTTER_TYPE_ACTOR);

    cdos_icon_dispatcher_signals[MESSAGE_ICON_ADDED] =
        g_signal_new ("message-icon-added",
                      G_TYPE_FROM_CLASS (klass),
                      G_SIGNAL_RUN_LAST,
                      G_STRUCT_OFFSET (CdosIconDispatcherClass, message_icon_added),
                      NULL,
                      NULL,
                      g_cclosure_marshal_VOID__OBJECT,
                      G_TYPE_NONE,
                      1,
                      CLUTTER_TYPE_ACTOR);
    cdos_icon_dispatcher_signals[MESSAGE_ICON_REMOVED] =
        g_signal_new ("message-icon-removed",
                      G_TYPE_FROM_CLASS (klass),
                      G_SIGNAL_RUN_LAST,
                      G_STRUCT_OFFSET (CdosIconDispatcherClass, message_icon_removed),
                      NULL,
                      NULL,
                      g_cclosure_marshal_VOID__OBJECT,
                      G_TYPE_NONE,
                      1,
                      CLUTTER_TYPE_ACTOR);

    // zhangdj
    cdos_icon_dispatcher_signals[BEFORE_REDISPLAY] =
        g_signal_new ("before-redisplay",
                      G_TYPE_FROM_CLASS (klass),
                      G_SIGNAL_RUN_LAST,
                      G_STRUCT_OFFSET (CdosIconDispatcherClass, before_redisplay),
                      NULL,
                      NULL,
                      g_cclosure_marshal_VOID__VOID,
                      G_TYPE_NONE,
                      0);

    cdos_icon_dispatcher_signals[AFTER_REDISPLAY] =
        g_signal_new ("after-redisplay",
                      G_TYPE_FROM_CLASS (klass),
                      G_SIGNAL_RUN_LAST,
                      G_STRUCT_OFFSET (CdosIconDispatcherClass, after_redisplay),
                      NULL,
                      NULL,
                      g_cclosure_marshal_VOID__VOID,
                      G_TYPE_NONE,
                      0);
}

static void
cdos_icon_dispatcher_init (CdosIconDispatcher *self)
{
    // CdosIconDispatcherPrivate *priv;

    // priv = self->priv = CDOS_ICON_DISPATCHER_GET_PRIVATE (self);
    CdosTrayManager *traymanager = g_object_new (CDOS_TYPE_TRAY_MANAGER, NULL);

    self->traymanager = traymanager;
    g_signal_connect (self->traymanager, "tray-icon-added", G_CALLBACK (on_tray_icon_added), self);
    g_signal_connect (self->traymanager, "tray-icon-removed", G_CALLBACK (on_tray_icon_removed), self);
    /*
     * 系统里面已经没有indicator-application-service了。而且执行该命令比较
     * 耗时，注释掉
     */
    //cdos_util_killall ("indicator-application-service");
}
static const gchar *
find_role (const gchar *wm_class)
{
    const gchar *wm_array[] = {"bluetooth-applet",
                               "gnome-volume-control-applet",
                               "gnome-sound-applet",
                               "nm-applet",
                               "gnome-power-manager",
                               "keyboard",
                               "a11y-keyboard",
                               "kbd-scrolllock",
                               "kbd-numlock",
                               "kbd-capslock",
                               "ibus-ui-gtk"};

    const gchar *roles[] = {"bluetooth",
                            "volume",
                            "volume",
                            "network",
                            "battery",
                            "keyboard",
                            "a11y",
                            "keyboard",
                            "keyboard",
                            "keyboard",
                            "input-method"};

    int len = (int)(sizeof (wm_array) / sizeof (wm_array[0]));
    int i = 0, findops = -1;
    for (i = 0; i < len; i++)
    {
        if (g_ascii_strcasecmp (wm_class, wm_array[i]) == 0)
        {
            findops = i;
        }
    }
    if (findops != -1)
    {
        return roles[findops];
    }
    else
    {
        return NULL;
    }
}
/*
 * dispatcher 这个对象监听 traymanager 的 tray-icon -added 信号
 * 根据收到的 icon 获取 wm_class
 * 取到对应的 role 之后，发出 status-icon-added 信号
 * systray 这个manager 会监听这个信号并作出处理
 * dispatcher 只做信号的统一转发
 */
static void
on_tray_icon_added (CdosTrayManager *traymanager, ClutterActor *icon, gpointer user_data)
{
    gchar *wm_class;
    CdosTrayIcon *trayIcon;
    // CdosTrayIconPrivate *priv;
    CdosIconDispatcher *dispatcher;
    const gchar *role;
    trayIcon = CDOS_TRAY_ICON (icon);

    dispatcher = CDOS_ICON_DISPATCHER (user_data);
    //	priv = CDOS_TRAY_ICON_GET_PRIVATE(trayIcon);
    //	wm_class = priv->wm_class;
    g_object_get (trayIcon, "wm-class", &wm_class, NULL);

    if (!wm_class)
    {
        wm_class = g_strdup ("unknown");
    }
    role = find_role (wm_class);
    if (!role)
    {
        role = wm_class;
    }
    g_signal_emit (dispatcher, cdos_icon_dispatcher_signals[STATUS_ICON_ADDED], 0, icon, role);
    g_free (wm_class);
}

static void
on_tray_icon_removed (CdosTrayManager *traymanager, ClutterActor *icon, gpointer user_data)
{
    CdosTrayIcon *trayIcon;
    CdosIconDispatcher *dispatcher;

    dispatcher = CDOS_ICON_DISPATCHER (user_data);
    trayIcon = CDOS_TRAY_ICON (icon);

    g_signal_emit (dispatcher, cdos_icon_dispatcher_signals[STATUS_ICON_REMOVED], 0, trayIcon);
}

/**
 * cdos_icon_dispatcher_start:
 * @self: 参数一， #CdosIconDispatcher 对象
 * @panel: 参数二， #CdosPanel 对象
 *
 * 主程序中调用，初始化并开始监听第三方程序的信号并转发信号。
 *
 * Returns: 空.
 */
void
cdos_icon_dispatcher_start (CdosIconDispatcher *self, CdosPanel *panel)
{
    g_return_if_fail (CDOS_IS_ICON_DISPATCHER (self));

    if (self->traymanager != NULL)
    {
        cdos_tray_manager_manage_screen (self->traymanager, (StWidget *)panel);
    }
}

/**
 * cdos_icon_dispatcher_redisplay:
 * @dispatcher: #CdosIconDispatcher 对象
 *
 * 刷新面板上第三方小应用程序的显示
 *
 * Returns: 空
 */
void
cdos_icon_dispatcher_redisplay (CdosIconDispatcher *dispatcher)
{
    g_return_if_fail (CDOS_IS_ICON_DISPATCHER (dispatcher));
    g_signal_emit_by_name (dispatcher, "before-redisplay");
    cdos_tray_manager_redisplay (dispatcher->traymanager);
    g_signal_emit_by_name (dispatcher, "after-redisplay");
}

/**
 * cdos_icon_dispatcher_new:
 * @traymanager: 传递给 CdosIconDispatcher 的参数，#CdosTrayManager 类型对象
 *
 * 新建对象 #CdosIconDispatcher.
 * dispatcher 这个对象监听 traymanager 的 tray-icon -added 信号
 * 根据收到的 icon 获取 wm_class
 * 取到对应的 role 之后，发出 status-icon-added 信号
 * systray 这个manager 会监听这个信号并作出处理
 * dispatcher 只做信号的统一转发
 *
 * Return value:  #CdosIconDispatcher 对象.
 */
CdosIconDispatcher *
cdos_icon_dispatcher_new (void)
{
    CdosIconDispatcher *icond;
    icond = g_object_new (CDOS_TYPE_ICON_DISPATCHER, NULL);
    CdosPanel *panel = cdos_ui_plugin_get_panel ();
    cdos_icon_dispatcher_start (icond, panel);
    return icond;
}


CdosIconDispatcher *
cdos_icon_dispatcher_get (void)
{
    static CdosIconDispatcher *icond = NULL;
    if (!icond)
        icond = cdos_icon_dispatcher_new ();
    return icond;
}
