/*
 * Created by zhangtao on 16-7-04.
 *
 * Modified by leit on 2021/08/17
 */

#include "cdos-ui-plugin.h"
#include "cdos-layout-manager.h"
#include "cdos-keyboard-manager.h"
#include "cdos-global.h"
#include "cdos-start.h"
#include "cdos-key.h"

typedef struct _CdosKey
{
    StButton parent;

    CdosKeyboardManager *keyboard_manager;

    guint keyval;
    gboolean grabbed;
    CaribouKeyModel *key;
    CaribouKeyModel** extended_keys;
    gint extended_keys_length;
    StWidget *extended_keyboard;
    CdosBoxPointer *box_pointer;
    gboolean show_extended_keys;

    gdouble key_width;

    gulong key_pressed_id;
    gulong key_released_id;
    gulong key_notify_show_subkeys_id;
    gulong stage_captured_id;
}CdosKey;

G_DEFINE_TYPE (CdosKey, cdos_key, ST_TYPE_BUTTON)

enum {
    SHOW_SUBKEYS,
    HIDE_SUBKEYS,
    LAST_SIGNAL
};
static gint cdos_key_signals[LAST_SIGNAL] = { 0 };

static void cdos_key_get_extended_keys (CdosKey *self);

static void cdos_key_ungrab (CdosKey *self);

static gboolean cdos_key_stage_on_captured (ClutterActor *actor, ClutterEvent *event, CdosKey *self);

static void cdos_key_key_on_notify_show_subkeys (GObject *object, GParamSpec *pspec, CdosKey *self);

static void
cdos_key_dispose (GObject *object)
{
    G_OBJECT_CLASS (cdos_key_parent_class)->dispose (object);
}

static void
cdos_key_finalize (GObject *object)
{
    G_OBJECT_CLASS (cdos_key_parent_class)->finalize (object);
}

/*
static gboolean
cdos_key_key_on_pressed (CaribouIKeyboardObject *ikey, CaribouKeyModel *key, CdosKey *self)
{
    st_button_set_checked (ST_BUTTON (self), TRUE);
    return FALSE;
}

static gboolean
cdos_key_key_on_released (CaribouIKeyboardObject *ikey, CaribouKeyModel *key, CdosKey *self)
{
    st_button_set_checked (ST_BUTTON (self), FALSE);
    return FALSE;
}
*/

static gboolean
cdos_key_button_on_press (ClutterActor *actor, ClutterEvent *event, CdosKey *self)
{
    if ((self->keyval >= 10 && self->keyval <= 127) ||
        self->keyval == 163 ||
        self->keyval == 8364)
        cdos_keyboard_manager_keyval_press (self->keyboard_manager, self->keyval);
    else
        caribou_key_model_press (self->key);
    return CLUTTER_ACTOR_CLASS (cdos_key_parent_class)->button_press_event (actor,
                                                                              (ClutterButtonEvent *)event);
}

static gboolean
cdos_key_button_on_release (ClutterActor *actor, ClutterEvent *event, CdosKey *self)
{
    if ((self->keyval >= 10 && self->keyval <= 127) ||
        self->keyval == 163 ||
        self->keyval == 8364)
        cdos_keyboard_manager_keyval_release (self->keyboard_manager, self->keyval);
    else
        caribou_key_model_release (self->key);
    return CLUTTER_ACTOR_CLASS (cdos_key_parent_class)->button_release_event (actor,
                                                                              (ClutterButtonEvent *)event);
}

static gboolean
cdos_extended_key_on_press (ClutterActor *actor, ClutterEvent *event, CaribouKeyModel *key)
{
    caribou_key_model_press (key);
    return TRUE;
}

static gboolean
cdos_extended_key_on_release (ClutterActor *actor, ClutterEvent *event, CaribouKeyModel *key)
{
    g_debug ("%s", __func__);
    caribou_key_model_release (key);
    return TRUE;
}

static StWidget*
cdos_key_make_key (CdosKey *self)
{
    const gchar *label;
    gchar *new_label;
    StWidget *button = NULL;
    button = ST_WIDGET (self);
    label = caribou_key_model_get_label (self->key);
    new_label = g_markup_escape_text (label, -1);
    gdouble key_width = caribou_key_model_get_width (self->key);
    self->key_width = key_width;

    clutter_actor_set_name (CLUTTER_ACTOR (button), "cdos-keyboard-key");
    clutter_actor_set_reactive (CLUTTER_ACTOR (button), TRUE);
    st_button_set_label (ST_BUTTON (button), new_label);
    st_widget_set_style_class_name (button, "keyboard-key");
    st_widget_set_track_hover (button, TRUE);
    g_signal_connect (button,
                      "button-press-event",
                      G_CALLBACK (cdos_key_button_on_press),
                      self);
    g_signal_connect (button,
                      "button-release-event",
                      G_CALLBACK (cdos_key_button_on_release),
                      self);
    return button;
}

static void
cdos_key_constructor (CdosKey *self, CaribouKeyModel *key)
{
    self->key = key;
    cdos_key_make_key (self);

    const gchar *key_name = caribou_key_model_get_name (self->key);
    //当切换到大写时，输入完毕，则不再切换到小写；
    guint keyval = caribou_key_model_get_keyval (self->key);
    self->keyval = keyval;
    if ((keyval >= 21 && keyval <= 127) || g_strcmp0 (key_name, "Return") == 0)
    {
        caribou_key_model_set_repeatable (self->key, TRUE);
    }
    else if (g_strcmp0 (key_name, "Control_L") == 0 ||
             g_strcmp0 (key_name, "Alt_L") == 0)
    {
        //what is 'latch'
    }

    self->extended_keys = caribou_key_model_get_extended_keys (self->key, &self->extended_keys_length);
    self->extended_keyboard = NULL;
    self->show_extended_keys = caribou_key_model_get_show_subkeys (self->key);

    if (self->extended_keys_length > 0 && self->show_extended_keys)
    {
        self->grabbed = FALSE;
        self->key_notify_show_subkeys_id =
                g_signal_connect (self->key,
                                  "notify::show-subkeys",
                                  G_CALLBACK (cdos_key_key_on_notify_show_subkeys),
                                  self);

        self->box_pointer = cdos_box_pointer_new (ST_SIDE_BOTTOM);
        StBin *box_pointer_bin = ST_BIN (self->box_pointer->bin);
        StWidget *box_pointer_actor = (StWidget *)self->box_pointer;

        st_bin_set_fill (box_pointer_bin, TRUE, TRUE);
        st_widget_add_style_class_name (box_pointer_actor, "keyboard-subkeys");
        cdos_key_get_extended_keys (self);

        clutter_actor_hide (CLUTTER_ACTOR (box_pointer_actor));
    }
}

static void
cdos_key_init (CdosKey *self)
{
    self->show_extended_keys = FALSE;
    self->keyboard_manager = cdos_keyboard_manager_get ();
}

static void
cdos_key_class_init (CdosKeyClass *klass)
{
    GObjectClass *gobject_class = G_OBJECT_CLASS (klass);

    gobject_class->dispose = cdos_key_dispose;
    gobject_class->finalize = cdos_key_finalize;
    /**
	 * CdosKey::show-subkeys:
	 * @key:当前按键的实例对象
	 *
	 * 显示子键时触发该信号
	 */
    cdos_key_signals[SHOW_SUBKEYS] =
                    g_signal_new ("show-subkeys",
                    CDOS_TYPE_KEY,
                    G_SIGNAL_RUN_LAST,
                    0,
                    NULL, NULL,
                    g_cclosure_marshal_VOID__VOID,
                    G_TYPE_NONE, 0);
    /**
	 * CdosKey::hide-subkeys:
	 * @key:当前按键的实例对象
	 *
	 * 隐藏子键时触发该信号
	 */
    cdos_key_signals[HIDE_SUBKEYS] =
                    g_signal_new ("hide-subkeys",
                    CDOS_TYPE_KEY,
                    G_SIGNAL_RUN_LAST,
                    0,
                    NULL, NULL,
                    g_cclosure_marshal_VOID__VOID,
                    G_TYPE_NONE, 0);

}
/*
static const gchar*
cdos_key_get_unichar (CdosKey *self, CaribouKeyModel *key)
{
    guint keyval = caribou_key_model_get_keyval (key);
    guint32 unichar = clutter_keysym_to_unicode (keyval);
    if (unichar)
    {
        gchar *new_unichar = (char *)g_malloc (6);
        g_unichar_to_utf8 (unichar, new_unichar);
        return new_unichar;
    }
}
*/

static void
cdos_key_get_extended_keys (CdosKey *self)
{
    self->extended_keyboard = st_box_layout_new ();
    st_widget_set_style_class_name (self->extended_keyboard, "keyboard-layout");
    st_box_layout_set_vertical (ST_BOX_LAYOUT (self->extended_keyboard), FALSE);

    guint i;
    for (i = 0; i < self->extended_keys_length; i++)
    {
        CaribouKeyModel *extended_key = self->extended_keys[i];
        const gchar *key_label = caribou_key_model_get_label (extended_key);
        gchar *label = g_markup_escape_text (key_label, -1);

        StWidget *key = st_button_new ();
        //g_object_set_data ((GObject *)key, "extended-key", extended_key);
        st_button_set_label (ST_BUTTON (key), label);
        st_widget_set_style_class_name (key, "keyboard-key");

        g_signal_connect (key,
                          "button-press-event",
                          G_CALLBACK (cdos_extended_key_on_press),
                          key);
        g_signal_connect (key,
                          "button-release-event",
                          G_CALLBACK (cdos_extended_key_on_release),
                          key);
        clutter_actor_add_child (CLUTTER_ACTOR (self->extended_keyboard), CLUTTER_ACTOR (key));
    }
    st_bin_set_child (self->box_pointer->bin, CLUTTER_ACTOR (self->extended_keyboard));
}

static void
cdos_key_ungrab (CdosKey *self)
{
    if (self->stage_captured_id)
    {
        ClutterStage *stage = cdos_global_get_stage (cdos_global_get ());
        g_signal_handler_disconnect (stage, self->stage_captured_id);
        self->stage_captured_id = 0;
    }
    self->grabbed = FALSE;
    cdos_pop_modal ((ClutterActor *)self, cdos_global_get_current_time (global));
}

static gboolean
cdos_key_stage_on_captured (ClutterActor *actor, ClutterEvent *event, CdosKey *self)
{
    ClutterActor *source = clutter_event_get_source (event);
    ClutterEventType type = clutter_event_type (event);
    if ((type == CLUTTER_BUTTON_PRESS || type == CLUTTER_BUTTON_RELEASE) &&
        clutter_actor_contains (CLUTTER_ACTOR (self->extended_keyboard), source))
    {
        CaribouKeyModel *extended_key = g_object_get_data ((GObject *)source, "extended-key");
        caribou_key_model_press (extended_key);
        caribou_key_model_release (extended_key);
    }
    if (type == CLUTTER_BUTTON_PRESS)
    {
        clutter_actor_hide (CLUTTER_ACTOR (self->box_pointer));
        cdos_key_ungrab (self);
        return TRUE;
    }
    return FALSE;
}

static void
cdos_key_key_on_notify_show_subkeys (GObject *object, GParamSpec *pspec, CdosKey *self)
{
    if (caribou_key_model_get_show_subkeys (self->key))
    {
        st_button_fake_release (ST_BUTTON (self));
        cdos_box_pointer_set_position (self->box_pointer, (ClutterActor *)self, 0.5);
        cdos_box_pointer_show (self->box_pointer, TRUE);
        st_widget_set_hover (ST_WIDGET (self), FALSE);
        g_signal_emit (self, cdos_key_signals[SHOW_SUBKEYS], 0);
        if (!self->grabbed)
        {
            ClutterStage *global_stage = cdos_global_get_stage (global);
            cdos_pop_modal ((ClutterActor *)self, cdos_global_get_current_time (global));
            self->stage_captured_id =
                    g_signal_connect (global_stage,
                                      "captured-event",
                                      G_CALLBACK (cdos_key_stage_on_captured),
                                      self);
            self->grabbed = TRUE;
        }
        caribou_key_model_release (self->key);
    }
    else
    {
        if (self->grabbed)
        {
            cdos_key_ungrab (self);
        }
        cdos_box_pointer_hide (self->box_pointer, TRUE);
        g_signal_emit (self, cdos_key_signals[HIDE_SUBKEYS], 0);
    }
}

CdosBoxPointer*
cdos_key_get_subkeys_box (CdosKey *self)
{
    return self->box_pointer;
}

/**
 * cdos_key_new:
 * @key: CaribouKeyMdoule * key
 *
 * 新建一个 #CdosKey 类的实例。
 *
 * Returns: 返回一个 #CdosKey 的实例。
 */
CdosKey*
cdos_key_new (CaribouKeyModel *key)
{
    CdosKey *self;
    self = g_object_new (CDOS_TYPE_KEY, NULL);
    cdos_key_constructor (self, key);
    return self;
}

/**
 * cdos_key_get_actor_extended_keyboard:
 * @actor: #CdosKey 对象
 *
 * get actor extended keys.
 *
 * Returns: a #StWidget instance
 */
StWidget *
cdos_key_get_actor_extended_keyboard (CdosKey *self)
{
    g_return_val_if_fail (CDOS_IS_KEY (self), NULL);
    return self->extended_keyboard;
}

/**
 * cdos_key_get_key_width:
 * @actor: #CdosKey 对象
 *
 * Get key width of self->key.
 *
 * Returns: key width
 */
gfloat
cdos_key_get_key_width (CdosKey *self)
{
    g_return_val_if_fail (CDOS_IS_KEY (self), 0);
    return self->key_width;
}
