#ifndef __CDOS_LAYOUT_MANAGER_H__
#define __CDOS_LAYOUT_MANAGER_H__

#include <meta/display.h>
#include <meta/util.h>
#include <meta/boxes.h>
#include "st.h"
#include "cdos-global.h"

#define CDOS_TYPE_LAYOUT_MANAGER                (cdos_layout_manager_get_type ())
#define CDOS_LAYOUT_MANAGER(obj)                (G_TYPE_CHECK_INSTANCE_CAST ((obj), CDOS_TYPE_LAYOUT_MANAGER, CdosLayoutManager))
#define CDOS_IS_LAYOUT_MANAGER(obj)             (G_TYPE_CHECK_INSTANCE_TYPE ((obj), CDOS_TYPE_LAYOUT_MANAGER))
#define CDOS_LAYOUT_MANAGER_CLASS(klass)        (G_TYPE_CHECK_CLASS_CAST ((klass), CDOS_TYPE_LAYOUT_MANAGER, CdosLayoutManagerClass))
#define CDOS_IS_LAYOUT_MANAGER_CLASS(klass)     (G_TYPE_CHECK_CLASS_TYPE ((klass), CDOS_TYPE_LAYOUT_MANAGER))
#define CDOS_LAYOUT_MANAGER_GET_CLASS(obj)      (G_TYPE_INSTANCE_GET_CLASS ((obj), CDOS_TYPE_LAYOUT_MANAGER, CdosLayoutManagerClass))

typedef struct _CdosLayoutManager CdosLayoutManager;
typedef struct _CdosLayoutManagerPrivate CdosLayoutManagerPrivate;
typedef struct _CdosLayoutManagerClass CdosLayoutManagerClass;

struct _CdosLayoutManager
{
    GObject parent_instance;

    CdosLayoutManagerPrivate *priv;
};

struct _CdosLayoutManagerClass
{
    GObjectClass parent_class;
};

/* Flags of add_chrome() */
#define TRACK_FULLSCREEN 1
#define AFFECT_STRUTS 2       /* 影响 workarea */
#define AFFECT_INPUT_REGION 4 /* 影响输入区域 */
#define ABOVE_TOP_WINDOW_GROUP 8

GType cdos_layout_manager_get_type (void);

gint cdos_layout_manager_get_index (void);

const MetaRectangle *cdos_layout_manager_get_primary_monitor (CdosLayoutManager *self);

const MetaRectangle *
cdos_layout_manager_find_monitor_for_rect (CdosLayoutManager *self, int x, int y, int width, int height);

const MetaRectangle *cdos_layout_manager_find_monitor_for_actor (CdosLayoutManager *self, ClutterActor *actor);

const MetaRectangle *cdos_layout_manager_get_monitor_geometry (CdosLayoutManager *self, gint monitor_index);
StWidget *cdos_layout_manager_get_panelbox (CdosLayoutManager *self);

StBoxLayout *cdos_layout_manager_get_keyboardbox (CdosLayoutManager *self);

void cdos_layout_manager_on_window_restacked (MetaDisplay *display, CdosLayoutManager *self);

gboolean cdos_layout_manager_update_regions (CdosLayoutManager *self);

void cdos_layout_manager_update_boxes (CdosLayoutManager *self);

void cdos_layout_manager_add_chrome (CdosLayoutManager *self, ClutterActor *actor, gint flags);

void cdos_layout_manager_remove_chrome (CdosLayoutManager *self, ClutterActor *actor);

GPtrArray *cdos_layout_manager_get_monitors (CdosLayoutManager *self);

gint cdos_layout_manager_get_focus_index (CdosLayoutManager *self);

const MetaRectangle *cdos_layout_manager_get_focus_monitor (CdosLayoutManager *self);

void cdos_layout_manager_set_keyboard_index (CdosLayoutManager *self, gint index);

gint cdos_layout_manager_get_keyboard_index (CdosLayoutManager *self);

void cdos_layout_manager_show_keyboard (CdosLayoutManager *self);

void cdos_layout_manager_hide_keyboard (CdosLayoutManager *self, gboolean animate);

CdosLayoutManager *cdos_layout_manager_get_default (void);

#endif
