#ifndef __CDOS_LIGHTBOX_H__
#define __CDOS_LIGHTBOX_H__

#include "st.h"

/* 
 * CdosLightBox:
 * @container: parent ClutterActor
 *
 * lightbox 创建一个 黑色半透明的“阴影”actor 遮盖住 @container 的内容，
 * 可以通过改变 actors 在 @container 中的顺序使特定 actor 高亮。在活动状
 * 态时，lightbox 会跟踪 @container 中添加或删除的 actors,保證在 lightbox
 * 被移除时，actors 仍然保持正确顺序。
 *
 */
GType cdos_lightbox_get_type (void);

#define CDOS_TYPE_LIGHTBOX (cdos_lightbox_get_type())
#define CDOS_LIGHTBOX(obj) G_TYPE_CHECK_INSTANCE_CAST((obj), cdos_lightbox_get_type(), CdosLightbox)
#define CDOS_LIGHTBOX_CLASS(klass) G_TYPE_CHECK_CLASS_CAST((klass), cdos_lightbox_get_type(), CdosLightboxClass)
#define CDOS_LIGHTBOX_GET_CLASS(obj) G_TYPE_INSTANCE_GET_CLASS((obj), cdos_lightbox_get_type(), CdosLightboxClass)

typedef struct _CdosLightbox CdosLightbox;
typedef struct _CdosLightboxClass CdosLightboxClass;

struct _CdosLightbox
{
    StWidget         parent;
/* <private> */
    ClutterActor    *actor;
    gfloat           width;
    gfloat           height;
    gboolean         inhibit_events;

    ClutterActor    *container;
    GList           *children;
    gfloat           fade_time;
    gpointer         highlighted;
    gulong           actor_added_handler;
    gulong           actor_removed_handler;
    gulong           allocation_changed_handler;
};

struct _CdosLightboxClass
{
    StWidgetClass parent_class;
};

CdosLightbox *cdos_lightbox_new (ClutterActor *container,  gfloat width, gfloat height, gboolean inhibit_events);

void    cdos_lightbox_set_fade_time (CdosLightbox *box, guint msec);
void    cdos_lightbox_highlight (CdosLightbox *lightbox, ClutterActor *actor);
void    cdos_lightbox_show (CdosLightbox *lightbox);
void    cdos_lightbox_hide (CdosLightbox *lightbox);
#endif
