/**
 * The MIT License 
 * 
 * Copyright (c) 2019 Illusion Lee
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy 
 * of this software and associated documentation files (the "Software"), to deal 
 * in the Software without restriction, including without limitation the rights 
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
 * copies of the Software, and to permit persons to whom the Software is 
 * furnished to do so, subject to the following conditions: 
 * 
 * The above copyright notice and this permission notice shall be included in all 
 * copies or substantial portions of the Software. 
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
 * SOFTWARE. 
 */

#include <stdio.h>
#include <inttypes.h>
#include <glib.h>
#include "cdos-lunar.h"
#include "cdos-lunar-data.h"

static const char *tiangan_dizhi[] =
{
    "甲子", "乙丑", "丙寅", "丁卯", "戊辰", "己巳",
    "庚午", "辛未", "壬申", "癸酉", "甲戌", "乙亥",
    "丙子", "丁丑", "戊寅", "己卯", "庚辰", "辛巳",
    "壬午", "癸未", "甲申", "乙酉", "丙戌", "丁亥",
    "戊子", "己丑", "庚寅", "辛卯", "壬辰", "癸巳",
    "甲午", "乙未", "丙申", "丁酉", "戊戌", "己亥",
    "庚子", "辛丑", "壬寅", "癸卯", "甲辰", "乙巳",
    "丙午", "丁未", "戊申", "己酉", "庚戌", "辛亥",
    "壬子", "癸丑", "甲寅", "乙卯", "丙辰", "丁巳",
    "戊午", "己未", "庚申", "辛酉", "壬戌", "癸亥"
};

static const char *lunar_day[] =
{
    "混沌", "初一", "初二", "初三", "初四", "初五", "初六", "初七", "初八", "初九", "初十",
    "十一", "十二", "十三", "十四", "十五", "十六", "十七", "十八", "十九", "二十",
    "廿一", "廿二", "廿三", "廿四", "廿五", "廿六", "廿七", "廿八", "廿九", "三十"
};

static const char *lunar_month[] =
{
    "虚", "正月", "二月", "三月", "四月", "五月", "六月",
    "七月", "八月", "九月", "十月", "冬月", "腊月"
};
static const char *lunar_leap_month[] =
{
    "虚", "闰正月", "闰二月", "闰三月", "闰四月", "闰五月", "闰六月",
    "闰七月", "闰八月", "闰九月", "闰十月", "闰冬月", "闰腊月"
};

static unsigned int lunar_calendar_table[199] =
{
    /*1901-1910*/
    0x04AE53, 0x0A5748, 0x5526BD, 0x0D2650, 0x0D9544, 0x46AAB9, 0x056A4D, 0x09AD42, 0x24AEB6, 0x04AE4A,
    /*1911-1920*/
    0x6A4DBE, 0x0A4D52, 0x0D2546, 0x5D52BA, 0x0B544E, 0x0D6A43, 0x296D37, 0x095B4B, 0x749BC1, 0x049754,
    /*1921-1930*/
    0x0A4B48, 0x5B25BC, 0x06A550, 0x06D445, 0x4ADAB8, 0x02B64D, 0x095742, 0x2497B7, 0x04974A, 0x664B3E,
    /*1931-1940*/
    0x0D4A51, 0x0EA546, 0x56D4BA, 0x05AD4E, 0x02B644, 0x393738, 0x092E4B, 0x7C96BF, 0x0C9553, 0x0D4A48,
    /*1941-1950*/
    0x6DA53B, 0x0B554F, 0x056A45, 0x4AADB9, 0x025D4D, 0x092D42, 0x2C95B6, 0x0A954A, 0x7B4ABD, 0x06CA51,
    /*1951-1960*/
    0x0B5546, 0x555ABB, 0x04DA4E, 0x0A5B43, 0x352BB8, 0x052B4C, 0x8A953F, 0x0E9552, 0x06AA48, 0x6AD53C,
    /*1961-1970*/
    0x0AB54F, 0x04B645, 0x4A5739, 0x0A574D, 0x052642, 0x3E9335, 0x0D9549, 0x75AABE, 0x056A51, 0x096D46,
    /*1971-1980*/
    0x54AEBB, 0x04AD4F, 0x0A4D43, 0x4D26B7, 0x0D254B, 0x8D52BF, 0x0B5452, 0x0B6A47, 0x696D3C, 0x095B50,
    /*1981-1990*/
    0x049B45, 0x4A4BB9, 0x0A4B4D, 0xAB25C2, 0x06A554, 0x06D449, 0x6ADA3D, 0x0AB651, 0x093746, 0x5497BB,
    /*1991-2000*/
    0x04974F, 0x064B44, 0x36A537, 0x0EA54A, 0x86B2BF, 0x05AC53, 0x0AB647, 0x5936BC, 0x092E50, 0x0C9645,
    /*2001-2010*/
    0x4D4AB8, 0x0D4A4C, 0x0DA541, 0x25AAB6, 0x056A49, 0x7AADBD, 0x025D52, 0x092D47, 0x5C95BA, 0x0A954E,
    /*2011-2020*/
    0x0B4A43, 0x4B5537, 0x0AD54A, 0x955ABF, 0x04BA53, 0x0A5B48, 0x652BBC, 0x052B50, 0x0A9345, 0x474AB9,
    /*2021-2030*/
    0x06AA4C, 0x0AD541, 0x24DAB6, 0x04B64A, 0x69573D, 0x0A4E51, 0x0D2646, 0x5E933A, 0x0D534D, 0x05AA43,
    /*2031-2040*/
    0x36B537, 0x096D4B, 0xB4AEBF, 0x04AD53, 0x0A4D48, 0x6D25BC, 0x0D254F, 0x0D5244, 0x5DAA38, 0x0B5A4C,
    /*2041-2050*/
    0x056D41, 0x24ADB6, 0x049B4A, 0x7A4BBE, 0x0A4B51, 0x0AA546, 0x5B52BA, 0x06D24E, 0x0ADA42, 0x355B37,
    /*2051-2060*/
    0x09374B, 0x8497C1, 0x049753, 0x064B48, 0x66A53C, 0x0EA54F, 0x06B244, 0x4AB638, 0x0AAE4C, 0x092E42,
    /*2061-2070*/
    0x3C9735, 0x0C9649, 0x7D4ABD, 0x0D4A51, 0x0DA545, 0x55AABA, 0x056A4E, 0x0A6D43, 0x452EB7, 0x052D4B,
    /*2071-2080*/
    0x8A95BF, 0x0A9553, 0x0B4A47, 0x6B553B, 0x0AD54F, 0x055A45, 0x4A5D38, 0x0A5B4C, 0x052B42, 0x3A93B6,
    /*2081-2090*/
    0x069349, 0x7729BD, 0x06AA51, 0x0AD546, 0x54DABA, 0x04B64E, 0x0A5743, 0x452738, 0x0D264A, 0x8E933E,
    /*2091-2099*/
    0x0D5252, 0x0DAA47, 0x66B53B, 0x056D4F, 0x04AE45, 0x4A4EB9, 0x0A4D4C, 0x0D1541, 0x2D92B5
};

static unsigned int month_sum[12] = {0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334};

/* 二十四节气 */
static const gchar *solar_terms[] =
{
    "冬至",
    "小寒", "大寒", "立春", "雨水", "惊蛰", "春分",
    "清明", "谷雨", "立夏", "小满", "芒种", "夏至",
    "小暑", "大暑", "立秋", "处暑", "白露", "秋分",
    "寒露", "霜降", "立冬", "小雪", "大雪", "冬至"
};

static const int days_of_year[] =
{
    0, 31, 59, 90, 120, 151,
    181, 212, 243, 273, 304, 334, 365
};

static int *solar_terms_arr = NULL;
static int *solar_terms_arrs[200] = {NULL};

/* 农历节日日期: (月 << 6)|日 */
static guint lunar_festival_date[] =
{
    0x41, 0x4f, 0x145, 0x1c7, 0x1cf, 0x20F, 0x249, 0x28F, 0x308, 0x40
};
static const gchar *lunar_festival[] =
{
    "春节",
    "元宵节",
    "端午节",
    "七夕",
    "中元节",
    "中秋节",
    "重阳节",
    "下元",
    "腊八节",
    "除夕"
};
static guint lunar_festivals[0x31e] = {0};

/* 公历节日 */
static guint solar_festival_date[] =
{
    0x21, 0x4E, 0x68, 0x06C,
    0xA1, 0xC1, 0x101, 0x12A, 0x141
};
static const gchar *solar_festival[] =
{
    "元旦",
    "情人节",
    "妇女节",
    "植树节",
    "劳动节",
    "儿童节",
    "建军节",
    "教师节",
    "国庆节"
};
static guint solar_festivals[0x1A0] = {0};

/* 当月是否为闰月 */
static gint
cdos_lunar_calendar (int year, int month, int day, uint32_t *lunar_calendar_day, int32_t *days_of_month)
{
    int lunar_newyear;      /* 春节离当年元旦的天数 */
    int sun_newyear;        /* 阳历日离当年元旦的天数 */
    // int days_of_month;   /* 大小月的天数 29或30 */
    int index;              /* 从哪个月开始来计算 */
    int flag;               /* 对闰月特殊处理 */
    int leap_month = 0;

    if (((lunar_calendar_table[year - 1901] & 0x0060) >> 5) == 1)
        lunar_newyear = (lunar_calendar_table[year - 1901] & 0x001F) - 1;
    else
        lunar_newyear = (lunar_calendar_table[year - 1901] & 0x001F) - 1 + 31;
    sun_newyear = month_sum[month - 1] + day - 1;
    if ( (!(year % 4)) && (month > 2))
        sun_newyear++;

    /* 判断阳历日在春节前还是春节后 */
    if (sun_newyear >= lunar_newyear) /* 阳历日在春节后（含春节那天） */
    {
        sun_newyear -= lunar_newyear;
        month = 1;
        index = 1;
        flag = 0;
        if ((lunar_calendar_table[year - 1901] & (0x80000 >> (index - 1))) == 0)
            *days_of_month = 29;
        else
            *days_of_month = 30;
        while (sun_newyear >= *days_of_month)
        {
            sun_newyear -= *days_of_month;
            index++;
            if (month == ((lunar_calendar_table[year - 1901] & 0xF00000) >> 20))
            {
                leap_month = 1;
                flag = ~flag;
                if (flag == 0)
                    month++;
            }
            else
            {
                month++;
            }
            if ((lunar_calendar_table[year - 1901] & (0x80000 >> (index - 1)) ) == 0)
                *days_of_month = 29;
            else
                *days_of_month = 30;
        }
        day = sun_newyear + 1;
    }
    else /* 阳历日在春节前 */
    {
        lunar_newyear -= sun_newyear;
        year--;
        month = 12;
        if (((lunar_calendar_table[year - 1901] & 0xF00000) >> 20) == 0)
        {
            index = 12;
        }
        else
        {
            index = 13;
        }
        flag = 0;
        if ((lunar_calendar_table[year - 1901] & (0x80000 >> (index - 1)) ) == 0)
        {
            *days_of_month = 29;
        }
        else
        {
            *days_of_month = 30;
        }
        while (lunar_newyear > *days_of_month)
        {
            lunar_newyear -= *days_of_month;
            index--;
            if (flag == 0)
            {
                month--;
            }
            if (month == ((lunar_calendar_table[year - 1901] & 0xF00000) >> 20))
            {
                g_debug ("FIXME: %s %d %d", __func__, flag, __LINE__);
                leap_month = 1;
                flag = ~flag;
            }
            if ((lunar_calendar_table[year - 1901] & (0x80000 >> (index - 1)) ) == 0)
            {
                *days_of_month = 29;
            }
            else
            {
                *days_of_month = 30;
            }
        }
        day = *days_of_month - lunar_newyear + 1;
    }
    /* *lunar_calendar_day = day; */
    *lunar_calendar_day = (month << 6) | day;
    if (month == ((lunar_calendar_table[year - 1901] & 0xF00000) >> 20))
        return leap_month;
    else
        return 0;
}

/* 
 * 返回 year 年第n个节气的日期
 * 支持范围 1900 - 2100 年
 **/
static gboolean
cdos_nth_jieqi (int year, int n, int *ret_month, int *ret_day, int *ret_jieqi)
{
    g_assert (n > 0);
    /*
    if (n < 0)
    {
        year -= 1;
        n += 24;
    }
    */

    if (year < 1900 || year >= 2100)
    {
        g_warn_if_reached ();
        return FALSE;
    }
    if (n < 1 || n > 24)
    {
        g_warn_if_reached ();
        return FALSE;
    }

    if (ret_month)
        *ret_month = (1 + n) / 2;
    if (!ret_day)
        return FALSE;

    int offset = (n - 1) % 6;
    int term = term_info[(year - 1900) * 4 + (n - 1) / 6];
    int day = 0;
    switch (offset)
    {
        case 0:
            day = term >> 23;
            break;
        case 1:
            day = term >> 18 & 0x1f;
            break;
        case 2:
            day = term >> 14 & 0x0f;
            break;
        case 3:
            day = term >> 9 & 0x1f;
            break;
        case 4:
            day = term >> 5 & 0x0f;
            break;
        case 5:
            day = term & 0x1f;
            break;
        default:
            break;
    }
    *ret_day = day;
    *ret_jieqi = n;

    return TRUE;
}

static void
cdos_lunar_init (int year, int month, int day)
{
    static gboolean inited = FALSE;
    gboolean is_show_jieqi = TRUE;
    if (!inited)
    {
        for (int i = 0; i < sizeof (lunar_festival_date) / sizeof (guint); i++)
            lunar_festivals[lunar_festival_date[i]] = i + 1;

        for (int i = 0; i < sizeof (solar_festival_date) / sizeof (guint); i++)
            solar_festivals[solar_festival_date[i]] = i + 1;

        inited = TRUE;
    }

    static int _y = 0, _m = 0;

    int index = year - 1900;

    if (_y != year || month != _m)
    {
        if (_y == 0 || _y != year)
        {
            if (!solar_terms_arrs[index])
            {
                solar_terms_arrs[index] = g_malloc0(sizeof(int) * 366);
            }
        }
        solar_terms_arr = solar_terms_arrs[index];
        /* 月份为month，计算前后2个节气日期 */
        int y, m;
        int ret_month=0, ret_day=0;
        int ret_jieqi = 0;
        y = year;
        m = month;
        if (month - _m > 1)
        {
            // year不在 1900 <=  year < 2100 范围内时，ret_month -1 会导致 days_of_year 数组访问下标为 -1 的内存，这里加个返回值判断。
            is_show_jieqi = cdos_nth_jieqi (y, 2 * m - 3, &ret_month, &ret_day, &ret_jieqi);
            if (is_show_jieqi)
                solar_terms_arr[days_of_year[ret_month - 1] + ret_day] = ret_jieqi;

            is_show_jieqi = cdos_nth_jieqi (y, 2 * m - 2, &ret_month, &ret_day, &ret_jieqi);
            if (is_show_jieqi)
                solar_terms_arr[days_of_year[ret_month - 1] + ret_day] = ret_jieqi;
        }

        is_show_jieqi = cdos_nth_jieqi (y, 2 * m - 1, &ret_month, &ret_day, &ret_jieqi);
        if (is_show_jieqi)
            solar_terms_arr[days_of_year[ret_month - 1] + ret_day] = ret_jieqi;

        is_show_jieqi = cdos_nth_jieqi (y, 2 * m, &ret_month, &ret_day, &ret_jieqi);
        if (is_show_jieqi)
            solar_terms_arr[days_of_year[ret_month - 1] + ret_day] = ret_jieqi;

        if (m < 12)
        {
            is_show_jieqi = cdos_nth_jieqi (y, 2 * m + 1, &ret_month, &ret_day, &ret_jieqi);
            if (is_show_jieqi)
                solar_terms_arr[days_of_year[ret_month - 1] + ret_day] = ret_jieqi;
        }

        _y = year;
        _m = month;
    }
}

int
cdos_sun_to_lunar (int year, int month, int day,
                   const gchar **ret_lunar_year,
                   const gchar **ret_lunar_month,
                   const gchar **ret_lunar_day,
                   const gchar **ret_festival)
{
    cdos_lunar_init (year, month, day);

    uint32_t lunar_calendar_day  = 0;
    int32_t days_of_month = 0;
    gboolean is_leap = FALSE; /* 是否为闰月 */

    is_leap = cdos_lunar_calendar (year, month, day, &lunar_calendar_day, &days_of_month);

    /* 除夕有大小年及闰月，单独判断 */
    if (ret_festival)
    {
        if (lunar_calendar_day == 0x31e || lunar_calendar_day == 0x31d)
        {
            int ty, tm, td;
            uint32_t lcd = 0;
            int32_t dom;
            GDate *gdate = g_date_new_dmy (day, month, year);
            g_date_add_days (gdate, 1);
            ty = g_date_get_year (gdate);
            tm = g_date_get_month (gdate);
            td = g_date_get_day (gdate);
            cdos_lunar_calendar (ty, tm, td, &lcd, &dom);
            if (lcd == 0x41 &&
                ((days_of_month == 30 && lunar_calendar_day == 0x31e)
                 || (days_of_month == 29 && lunar_calendar_day == 0x31d)))
            {
                *ret_festival = lunar_festival[9];
            }
        }
        else if (!is_leap && lunar_festivals[lunar_calendar_day])
        {
            *ret_festival = lunar_festival[lunar_festivals[lunar_calendar_day] - 1];
        }
        // year不在 1900 <=  year < 2100 范围内时，会导致 solar_terms_arrs 数组越界，这里加个年份过滤。
        else if ( 1900 <=  year  && year < 2100 && solar_terms_arrs[year - 1900][days_of_year[month - 1] + day] )
            *ret_festival = solar_terms[solar_terms_arrs[year - 1900][days_of_year[month - 1] + day]];
        else if (!*ret_festival)
        {
            if (solar_festivals[(month << 5) + day])
                *ret_festival = solar_festival[solar_festivals[(month << 5) + day] - 1];
        }
    }

    if (ret_lunar_day)
    {
        *ret_lunar_day = lunar_day[lunar_calendar_day & 0x3F];
    }
    if (ret_lunar_month)
    {
        if (is_leap)
            *ret_lunar_month = lunar_leap_month[(lunar_calendar_day & 0x3C0) >> 6];
        else
            *ret_lunar_month = lunar_month[(lunar_calendar_day & 0x3C0) >> 6];
    }

    if (ret_lunar_year)
        *ret_lunar_year = NULL;

    return (lunar_calendar_day & 0x3F) == 1;
}

const char *
cdos_sun_to_lunar_festival (int year, int month, int day)
{
    const gchar *ret_lunar_month = NULL, *ret_lunar_day = NULL, *ret_festival = NULL;
    int is_chuyi = cdos_sun_to_lunar (year, month, day, NULL, &ret_lunar_month, &ret_lunar_day, &ret_festival);

    if (ret_festival)
        return ret_festival;
    else if (is_chuyi)
        return ret_lunar_month;
    else
        return ret_lunar_day;
}

const char *
cdos_get_ganzhi_of_year (int year, int month, int day)
{
    int gz = (year - 1804) % 60;

    uint32_t lunar_calendar_day  = 0;
    int32_t days_of_month = 0;

    /* 农历干支纪年以春节为界限 */
    if (month <= 2)
    {
        cdos_lunar_calendar (year, month, day, &lunar_calendar_day, &days_of_month);
        int lunar_month = (lunar_calendar_day & 0x3C0) >> 6;
        if (lunar_month >= 11)
            --gz;
    }
    while (gz < 0)
        gz += 60;

    return tiangan_dizhi[gz];
}

#if 0
int main (void)
{
    for (int i = 0; i < sizeof (lunar_festival_date) / sizeof (guint); i++)
        lunar_festivals[lunar_festival_date[i]] = i + 1;

    for (int i = 0; i < 31; i++)
        cdos_sun_to_lunar_festival (2020, 2, i + 1);

    /*
    static const char *tiangan[] =
    {
        "甲", "乙", "丙", "丁", "戊", "己", "庚", "辛", "壬", "癸"
    };

    static const char *dizhi[] =
    {
        "子", "丑", "寅", "卯", "辰", "巳", "午", "未", "申", "酉", "戌", "亥"
    };
    for (int i = 0; i < 6; i += 1)
    {
        for (int j = 0; j < 10; j++)
        {
            printf("\"%s%s\", ", tiangan[j], dizhi[(j + i*10)%12]);
        }
    }
    */
    printf("%s\n", cdos_get_ganzhi_of_year(2020, 1, 26));

    return 0;
}
#endif
