#include "common.h"
#include "cdos-message-dialog.h"
#include "cdos-start.h"

enum
{
    RESPONSE,
    LAST_SIGNAL
};

static guint signals[LAST_SIGNAL] = {0};

struct _CdosMessageDialog
{
    StBoxLayout parent;

    GCallback callback;
    gpointer user_data;

    ClutterActor *title_label;
    ClutterActor *close_button;
    ClutterActor *icon;
    ClutterActor *message_label;

    ClutterActor *top;
    ClutterActor *center;

    ClutterActor *button_box;

    guint spacing;
};

G_DEFINE_TYPE (CdosMessageDialog, cdos_message_dialog, ST_TYPE_BOX_LAYOUT);

static void
on_button_clicked (StButton *button,
                   guint event,
                   CdosMessageDialog *self)
{
    guint response = GPOINTER_TO_UINT (g_object_get_data ((GObject *)button, "response-id"));
    g_signal_emit (self, signals[RESPONSE], 0, response);
    clutter_actor_hide ((ClutterActor *)self);
}

static void
on_apply_clicked (StButton *button,
                  guint event,
                  CdosMessageDialog *self)
{
    if (self->callback)
        ((void (*)(void *))self->callback) (self->user_data);
    else
    {
        guint response = GPOINTER_TO_UINT (g_object_get_data ((GObject *)button, "response-id"));
        g_signal_emit (self, signals[RESPONSE], 0, response);
        clutter_actor_hide ((ClutterActor *)self);
    }
}

static void
cdos_message_dialog_style_changed (StWidget *widget)
{
    CdosMessageDialog *self = CDOS_MESSAGE_DIALOG (widget);

    StThemeNode *theme_node = st_widget_get_theme_node (widget);
    self->spacing = st_theme_node_get_length (theme_node, "spacing");

    ST_WIDGET_CLASS (cdos_message_dialog_parent_class)->style_changed (widget);
}
/*
static void
cdos_message_dialog_get_preferred_width (ClutterActor *actor,
                                         gfloat for_height,
                                         gfloat   *min_width,
                                         gfloat   *nat_width)
{
    StThemeNode *theme_node = st_widget_get_theme_node ((StWidget *)actor);

    CdosMessageDialog *self = CDOS_MESSAGE_DIALOG (actor);
    st_theme_node_adjust_preferred_height (theme_node, min_width, nat_width);
}

static void
cdos_message_dialog_get_preferred_height (ClutterActor *actor,
                                          gfloat for_width,
                                          gfloat   *min_height,
                                          gfloat   *nat_height)
{
    StThemeNode *theme_node = st_widget_get_theme_node ((StWidget *)actor);
    CdosMessageDialog *self = CDOS_MESSAGE_DIALOG (actor);
    gfloat height;

    *min_height = height;
    *nat_height = height;
    st_theme_node_adjust_preferred_height (theme_node, min_height, nat_height);
}

static void
cdos_message_dialog_allocate (ClutterActor *actor,
                              const ClutterActorBox *box,
                              ClutterAllocationFlags flags)
{
    StThemeNode *theme_node = st_widget_get_theme_node ((StWidget *)actor);

    clutter_actor_set_allocation (actor, box, flags);

    ClutterActorBox content_box, child_box;
    st_theme_node_get_content_box (theme_node, box, &content_box);
}
*/

static void
cdos_message_dialog_on_show (ClutterActor *actor)
{
    CLUTTER_ACTOR_CLASS (cdos_message_dialog_parent_class)->show (actor);
    cdos_push_modal (actor, 0, 0, CDOS_ACTION_MODE_SYSTEM_MODAL);
}

static void
cdos_message_dialog_on_hide (ClutterActor *actor)
{
    cdos_pop_modal (actor, 0);
    CLUTTER_ACTOR_CLASS (cdos_message_dialog_parent_class)->hide (actor);
}

static gboolean
cdos_message_dialog_on_key_release (ClutterActor *actor,
                                    ClutterKeyEvent *key_event)
{
    return TRUE;
}

static gboolean
cdos_message_dialog_on_key_press (ClutterActor *actor,
                                  ClutterKeyEvent *key_event)
{
    ClutterEvent *event = (ClutterEvent *)key_event;
    guint key = clutter_event_get_key_symbol (event);

    if (key == CLUTTER_KEY_Escape)
    {
        clutter_actor_hide (actor);
        return TRUE;
    }
    if (key == CLUTTER_KEY_Left ||
        key == CLUTTER_KEY_Right ||
        key == CLUTTER_KEY_Up ||
        key == CLUTTER_KEY_Down)
    {
        st_widget_navigate_focus ((StWidget *)actor, NULL, GTK_DIR_TAB_FORWARD, FALSE);
    }
    return TRUE;
}

static void
cdos_message_dialog_init (CdosMessageDialog *self)
{
    st_widget_add_style_class_name ((StWidget *)self, "msg-dialog");
    clutter_actor_set_reactive ((ClutterActor *)self, TRUE);

    self->top = (ClutterActor *)st_box_layout_new ();
    clutter_actor_set_x_expand (self->top, TRUE);
    st_widget_add_style_class_name ((StWidget *)self->top, "msg-dialog-header");

    self->title_label = (ClutterActor *)st_label_new (NULL);
    clutter_actor_set_x_expand (self->title_label, TRUE);
    st_widget_add_style_class_name ((StWidget *)self->title_label, "msg-title");
    self->close_button = (ClutterActor *)st_button_new ();
    clutter_actor_set_x_align (self->close_button, CLUTTER_ACTOR_ALIGN_END);
    st_widget_add_style_class_name ((StWidget *)self->close_button, "msg-close-button");
    ClutterActor *icon = g_object_new (ST_TYPE_ICON,
                                       "icon-name", "window-close",
                                       "icon-size", 16,
                                       "style-class", "msg-close",
                                       NULL);
    st_bin_set_child ((StBin *)self->close_button, icon);
    g_object_set_data ((GObject *)self->close_button, "response-id", GUINT_TO_POINTER(CDOS_RESPONSE_CLOSE));
    g_signal_connect (self->close_button, "clicked", G_CALLBACK (on_button_clicked), self);

    _add_actor (self->top, self->title_label);
    _add_actor (self->top, self->close_button);

    self->center = (ClutterActor *)st_box_layout_new ();
    clutter_actor_set_x_expand (self->center, TRUE);
    clutter_actor_set_y_expand (self->center, TRUE);
    st_widget_add_style_class_name ((StWidget *)self->center, "msg-dialog-center");
    self->icon = (ClutterActor *)g_object_new (ST_TYPE_ICON,
                                               "style-class", "msg-warning",
                                               "icon-name", "dialog-information",
                                               NULL);
    self->message_label = (ClutterActor *)st_label_new (NULL);
    clutter_actor_set_y_align (self->message_label, CLUTTER_ACTOR_ALIGN_CENTER);
    clutter_text_set_ellipsize ((ClutterText *)st_label_get_clutter_text ((StLabel *)self->message_label),
                                PANGO_ELLIPSIZE_NONE);
    _add_actor (self->center, self->icon);
    _add_actor (self->center, self->message_label);

    self->button_box = (ClutterActor *)g_object_new (ST_TYPE_BOX_LAYOUT,
                                                     "style-class", "msg-dialog-button-box",
                                                     NULL);
    clutter_actor_set_x_expand (self->button_box, TRUE);
    clutter_actor_set_x_align (self->button_box, CLUTTER_ACTOR_ALIGN_FILL);

    _add_actor (self, self->top);
    _add_actor (self, self->center);
    //extra = clutter_actor_new ();
    //clutter_actor_set_y_expand (extra, TRUE);
    //_add_actor (self, extra);
    _add_actor (self, self->button_box);

    st_box_layout_set_vertical (ST_BOX_LAYOUT (self), TRUE);
}

static void
cdos_message_dialog_class_init (CdosMessageDialogClass *klass)
{
    ClutterActorClass *actor_class = CLUTTER_ACTOR_CLASS (klass);
    /*
    actor_class->allocate = cdos_message_dialog_allocate;
    actor_class->get_preferred_width = cdos_message_dialog_get_preferred_width;
    actor_class->get_preferred_height = cdos_message_dialog_get_preferred_height;
    */
    actor_class->show = cdos_message_dialog_on_show;
    actor_class->hide = cdos_message_dialog_on_hide;
    actor_class->key_press_event = cdos_message_dialog_on_key_press;
    actor_class->key_release_event = cdos_message_dialog_on_key_release;

    StWidgetClass *widget_class = ST_WIDGET_CLASS (klass);
    widget_class->style_changed = cdos_message_dialog_style_changed;

    /**
     * CdosMessageDialog::response:
     * @widget: #CdosMessageDialog 对象
     * @id: response id of button
     *
     * 按钮点击时时发送该信号
     */
    signals[RESPONSE] =
        g_signal_new ("response",
                      G_TYPE_FROM_CLASS (klass),
                      G_SIGNAL_RUN_LAST,
                      0,
                      NULL, NULL, NULL,
                      G_TYPE_NONE, 1, G_TYPE_UINT);
}

void
cdos_message_dialog_set_title (CdosMessageDialog *self,
                               const gchar *title)
{
    g_return_if_fail (CDOS_IS_MESSAGE_DIALOG (self));
    st_label_set_text ((StLabel *)self->title_label, title);
}

void
cdos_message_dialog_set_message (CdosMessageDialog *self,
                                 const gchar *message)
{
    g_return_if_fail (CDOS_IS_MESSAGE_DIALOG (self));
    st_label_set_text ((StLabel *)self->message_label, message);
}

void
cdos_message_dialog_set_callback (CdosMessageDialog *self,
                                  GCallback callback,
                                  gpointer user_data)
{
    g_return_if_fail (CDOS_IS_MESSAGE_DIALOG (self));

    self->callback = callback;
    self->user_data = user_data;
}

void
cdos_message_dialog_set_icon_name (CdosMessageDialog *self,
                                   const gchar *icon)
{
    g_return_if_fail (CDOS_IS_MESSAGE_DIALOG (self));
    st_icon_set_icon_name ((StIcon *)self->icon, icon);
}

CdosMessageDialog *
cdos_message_dialog_new_with_buttons (const gchar *title,
                                      const gchar *message,
                                      GCallback callback,
                                      gpointer user_data)
{
    CdosMessageDialog *self = g_object_new (CDOS_TYPE_MESSAGE_DIALOG,
                                            NULL);
    if (title)
        cdos_message_dialog_set_title (self, title);
    if (message)
        cdos_message_dialog_set_message (self, message);

    ClutterActor *cancel = (ClutterActor *)st_button_new_with_label (_("Cancel"));
    st_widget_add_style_class_name ((StWidget *)cancel, "msg-button-cancel");
    st_widget_add_style_class_name ((StWidget *)cancel, "msg-dialog-button");
    g_object_set_data ((GObject *)cancel, "response-id", GUINT_TO_POINTER(CDOS_RESPONSE_CANCEL));

    ClutterActor *apply = (ClutterActor *)st_button_new_with_label (_("Yes"));
    st_widget_add_style_class_name ((StWidget *)apply, "msg-button-apply");
    st_widget_add_style_class_name ((StWidget *)apply, "msg-dialog-button");
    g_object_set_data ((GObject *)apply, "response-id", GUINT_TO_POINTER(CDOS_RESPONSE_APPLY));

    _add_actor (self->button_box, cancel);
    _add_actor (self->button_box, apply);

    clutter_actor_set_x_expand (cancel, TRUE);
    clutter_actor_set_x_align (cancel, CLUTTER_ACTOR_ALIGN_END);
    clutter_actor_set_x_align (apply, CLUTTER_ACTOR_ALIGN_END);

    g_signal_connect (cancel, "clicked", G_CALLBACK (on_button_clicked), self);
    g_signal_connect (apply, "clicked", G_CALLBACK (on_apply_clicked), self);

    self->callback = callback;
    self->user_data = user_data;

    return self;
}

/**
* cdos_message_dialog_new:
* @title: Title of #CdosMessageDialog.
* @message: Message of #CdosMessageDialog.
*
* Create a new #CdosMessageDialog
*
* Returns: a new #CdosMessageDialog
*/

CdosMessageDialog *
cdos_message_dialog_new (const gchar *title,
                         const gchar *message)
{
    CdosMessageDialog *self = g_object_new (CDOS_TYPE_MESSAGE_DIALOG,
                                            NULL);
    if (title)
        cdos_message_dialog_set_title (self, title);
    if (message)
        cdos_message_dialog_set_message (self, message);

    return self;
}

/**
* cdos_message_dialog_add_button:
* @label: text label of button
* @response_id: response id of button
*
* Add a new #StButton with the specified label and response id
*
* Returns: a new #StButton
*/

StWidget *
cdos_message_dialog_add_button (CdosMessageDialog *self,
                                const char *label,
                                CdosResponseType response_id)
{
    ClutterActor *button = (ClutterActor *)st_button_new_with_label (label);
    st_widget_add_style_class_name ((StWidget *)button, "msg-dialog-button");
    g_object_set_data ((GObject *)button, "response-id", GUINT_TO_POINTER(response_id));
    clutter_actor_set_x_align (button, CLUTTER_ACTOR_ALIGN_END);

    _add_actor (self->button_box, button);
    g_signal_connect (button, "clicked", G_CALLBACK (on_button_clicked), self);

    return ST_WIDGET (button);
}
