#ifndef __MODAL_DIALOG_H__
#define __MODAL_DIALOG_H__

#include "st.h"
#include "cdos-lightbox.h"

/**
 * ModalState:
 * @MODAL_CLOSED: 关闭
 * @MODAL_CLOSING: 正在关闭
 * @MODAL_OPENED: 打开
 * @MODAL_OPENING: 正在打开
 * @MODAL_FADED_OUT: 渐隐
 *
 * #ModalDialog 的当前状态
 *
 */
typedef enum {
    MODAL_CLOSED,
    MODAL_OPENED,
    MODAL_OPENING,
    MODAL_CLOSING,
    MODAL_FADED_OUT
}ModalState;

/**
 * MAX_KEY_SYM
 *
 * #ModalDialog 上支持的最大快捷键数目
 *
 */
#define MAX_KEY_SYM 256

/**
 * KeyPressAction:
 * @userdata: user data
 * @modal: #ModalDialog
 *
 * 按键回调函数
 *
 */
typedef void (*KeyPressAction) (void *userdata, void *modal);

/**
 * ButtonCallback:
 * @button: StWidget
 * @button_event: 鼠标事件
 * @modal: @ModalDialog
 *
 * button 被点击时的回调函数
 *
 */
typedef void (*ButtonCallback) (void *button, gint button_event, void *modal);
typedef struct _ModalDialog ModalDialog;
struct _ModalDialog{
    ModalState state;
    gboolean has_modal;
    gboolean cdos_reactive;
    StWidget *group;
    ClutterConstraint *constraint;
    StWidget *background_bin;
    StWidget *dialog_layout;
    CdosLightbox *lightbox;
    ClutterActor *event_blocker;
    StWidget *content_layout;
    StWidget *button_layout;
    GList *buttons;
    StWidget *close_button;

    /* <private> */
    StWidget *initial_key_focus;
    StWidget *saved_key_focus;
    ClutterActor *stack;

    KeyPressAction action_keys[MAX_KEY_SYM];
    void     *action_data[MAX_KEY_SYM];
};

ModalDialog *cdos_modal_dialog_init (ModalDialog *modal,
                                     gboolean cdos_reactive,
                                     const gchar    *style_class);

void cdos_modal_dialog_destroy (ModalDialog *modal);
void cdos_modal_dialog_close (ModalDialog *modal, guint32 timestamp);
gboolean cdos_modal_dialog_open (ModalDialog *modal, guint32 timestamp);


/**
 * ButtonInfo:
 * @label: 在该按钮上显示的字符串
 * @action: 按键回调函数
 * @callback: 按钮被点击时的回调函数
 * @key: 快捷键
 * @focused: 是否有焦点
 * @userdata: @callback 回调函数的参数
 *
 */
typedef struct _ButtonInfo ButtonInfo;
struct _ButtonInfo {
    char *label;
    KeyPressAction action;
    ButtonCallback callback;
    int   key;
    int   focused;
    void  *userdata;
};

void cdos_modal_dialog_set_buttons (ModalDialog *modal,
                                    GList       *buttons);

gboolean
modal_dialog_pop_modal (ModalDialog *modal_dialog,
                        guint32       timestamp);
gboolean
modal_dialog_push_modal (ModalDialog *modal_dialog,
                         guint32       timestamp);
void
modal_dialog_set_initial_key_focus (ModalDialog *modal_dialog, 
                                    ClutterActor *actor);

#endif // __MODAL_DIALOG_H
