
//该模块在src/ui/cdos-auto-mount-manager.c中被使用，但是cdos-auto-mount-manager暂时没有被使用。
/**
 * SECTION: cdos-mount-operation
 * @short_description: 处理挂载操作的相关类型和方法
 * @include: cdos-mount-operation.h
 *
 * 包括ProcessesDialog类, MountQuestionDialog类,
 * CdosMountOperation类和MountPasswordNotification类,
 * 用来处理挂载操作
 */
#include <clutter/clutter.h>
#include <gio/gio.h>

#include "cdos-mount-operation.h"
#include "cdos-message-tray.h"
#include "cdos-start.h"

#include "st.h"
#include "cdos-source.h"
#include "cdos-app.h"
#include "cdos-notification.h"

typedef struct _MountPasswordNotification MountPasswordNotification;
typedef struct _ListItem ListItem;
typedef struct _ProcessesDialog ProcessesDialog;
typedef struct _MountPasswordSource MountPasswordSource;
typedef struct _MountQuestionDialog MountQuestionDialog;

struct _CdosMountOperationPrivate
{
    gboolean notification_showing;
    ClutterActor *icon;
    ModalDialog *dialog;
    void (*dialog_destroy) (ModalDialog *dialog);
    ProcessesDialog *processes_dialog;
    MountPasswordSource *source;
    gboolean reask_password;
    GArray *processes;
    gchar **choices;
    gchar *message;

    gulong source_destroy_id;
    gulong source_password_ready_id;
};

G_DEFINE_TYPE_WITH_PRIVATE (CdosMountOperation, cdos_mount_operation, CDOS_TYPE_BASE_MOUNT_OPERATION);

struct _ListItem
{
    CdosApp *app;
    StWidget *layout;
    StWidget *actor;
    StIcon *icon;
    StBin *icon_bin;
    StWidget *label_name;
    StBin *label_bin;
};

struct _MountPasswordNotification
{
    CdosNotification *notification;
    StWidget *label;
    StBoxLayout *response_entry;
};

struct _MountQuestionDialog
{
    ModalDialog modal;
    StWidget *main_content_layout;
    StWidget *message_layout;
    StLabel *subject_label;
    StLabel *description_label;
};

struct _ProcessesDialog
{
    /*ModalDialog    modal;*/
    MountQuestionDialog question_dialog;
    StScrollView *scroll_view;
    StWidget *application_list;
    GList *list_items;
};

struct _MountPasswordSource
{
    CdosSource *source;
    gchar **strings;
    MountPasswordNotification *notification;
};

static void cdos_mount_operation_on_ask_question (GMountOperation *base_mount_operation,
                                                  gchar *message,
                                                  GStrv choices,
                                                  gpointer user_data);

static void
cdos_mount_operation_on_ask_password (GMountOperation *base_mount_operation, gchar *message, CdosMountOperation *self);

static void cdos_mount_operation_on_show_processes2 (CdosBaseMountOperation *base_mount_operation,
                                                     CdosMountOperation *self);

static void cdos_mount_operation_on_aborted (GMountOperation *base_mount_operation, CdosMountOperation *self);

static ProcessesDialog *cdos_mount_operation_create_processes_dialog (StIcon *icon);

static void cdos_mount_operation_destroy_processes_dialog (ModalDialog *modal);

static ListItem *cdos_mount_operation_create_list_item (CdosApp *app);

static void cdos_mount_operation_destroy_list_item (ListItem *list_item);

static void cdos_mount_operation_destroy_question_dialog (ModalDialog *modal);

static void cdos_mount_question_dialog_update (ModalDialog *dialog, gchar *message, GStrv choices);

static void cdos_mount_operation_set_label_text (StLabel *label, const gchar *text);

static void cdos_mount_operation_set_button_for_choices (ModalDialog *dialog, gchar **choices);

static void cdos_mount_operation_set_label_for_messages (MountQuestionDialog *dialog, gchar *message);

static MountPasswordSource *cdos_mount_operation_create_password_source (gchar *message, gboolean reask_password);

static void cdos_mount_operation_destroy_password_source (MountPasswordSource *password_source);

static MountPasswordNotification *
cdos_mount_operation_create_password_notification (CdosSource *source, gchar **strings, gboolean reask_password);

static void cdos_mount_operation_destroy_password_notification (MountPasswordNotification *notification);

static void cdos_mount_operation_destroy_processes_dialog_list_items (ProcessesDialog *processes_dialog);

static ProcessesDialog *
cdos_mount_operation_create_processes_dialog (StIcon *icon)
{
    MountQuestionDialog *question_dialog;
    ProcessesDialog *processes_dialog;

    processes_dialog = g_new0 (ProcessesDialog, 1);
    question_dialog = (MountQuestionDialog *)processes_dialog;

    ModalDialog *modal = (ModalDialog *)processes_dialog;
    cdos_modal_dialog_init (modal, FALSE, NULL);

    question_dialog->main_content_layout = st_box_layout_new ();
    clutter_actor_add_child ((ClutterActor *)modal->content_layout,
                            (ClutterActor *)question_dialog->main_content_layout);

    question_dialog->message_layout = g_object_new (ST_TYPE_BOX_LAYOUT, "vertical", TRUE, NULL);
    clutter_actor_add_child ((ClutterActor *)question_dialog->main_content_layout,
                            (ClutterActor *)question_dialog->message_layout);

    question_dialog->subject_label = g_object_new (ST_TYPE_LABEL, "style-class", "mount-question-dialog-subject", NULL);
    clutter_actor_add_child ((ClutterActor *)question_dialog->message_layout,
                            (ClutterActor *)question_dialog->subject_label);

    question_dialog->description_label =
        g_object_new (ST_TYPE_LABEL, "style-class", "mount-question-dialog-description", NULL);
    clutter_text_set_ellipsize (CLUTTER_TEXT (question_dialog->description_label), PANGO_ELLIPSIZE_NONE);
    clutter_text_set_line_wrap (CLUTTER_TEXT (question_dialog->description_label), TRUE);
    clutter_actor_add_child ((ClutterActor *)question_dialog->message_layout,
                            (ClutterActor *)question_dialog->description_label);

    processes_dialog->scroll_view =
        g_object_new (ST_TYPE_SCROLL_VIEW, "style-class", "show-processes-dialog-app-list", NULL);
    st_scroll_view_set_policy (processes_dialog->scroll_view, GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
    clutter_actor_add_child ((ClutterActor *)modal->content_layout, (ClutterActor *)processes_dialog->scroll_view);
    clutter_actor_hide (CLUTTER_ACTOR (processes_dialog->scroll_view));

    processes_dialog->application_list = g_object_new (ST_TYPE_BOX_LAYOUT, "vertical", TRUE, NULL);
    st_scroll_view_add_child (processes_dialog->scroll_view,
                            (ClutterActor *)processes_dialog->application_list);

    return processes_dialog;
}

static void
cdos_mount_operation_destroy_processes_dialog (ModalDialog *modal)
{
    ProcessesDialog *processes_dialog = (ProcessesDialog *)modal;

    cdos_mount_operation_destroy_processes_dialog_list_items (processes_dialog);

    if (processes_dialog->scroll_view)
    {
        clutter_actor_destroy (CLUTTER_ACTOR (processes_dialog->scroll_view));
    }

    cdos_mount_operation_destroy_question_dialog (modal);
    g_free (processes_dialog);
}

static ListItem *
cdos_mount_operation_create_list_item (CdosApp *app)
{
    ListItem *item;
    item = g_new0 (ListItem, 1);
    item->app = app;
    item->layout = g_object_new (ST_TYPE_BOX_LAYOUT, "vertical", FALSE, NULL);
    /*item->actor = g_object_new(ST_TYPE_BUTTON,
                                "style-class","show-processes-dialog-app-list-item"
                                "can-focus",TRUE,
                                "child",item->layout,
                                "reactive",TRUE,
                                "x-align",ST_ALIGN_START,
                                "x-fill",TRUE,
                                NULL);*/

    item->icon = (StIcon *)cdos_app_create_icon_texture (item->app, LIST_ITEM_ICON_SIZE);

    item->icon_bin = g_object_new (ST_TYPE_BIN,
                                   "style-class", "show-processes-dialog-app-list-item-icon",
                                   "child", item->icon,
                                   NULL);

    clutter_actor_add_child ((ClutterActor *)item->layout, (ClutterActor *)item->icon_bin);

    const char *name = cdos_app_get_name (item->app);

    item->label_name = g_object_new (ST_TYPE_BIN,
                                     "style-class", "show-processes-dialog-app-list-item-name",
                                     "text", name,
                                     NULL);

    item->label_bin = g_object_new (ST_TYPE_BIN, "y-align", ST_ALIGN_MIDDLE, "child", item->label_name, NULL);

    clutter_actor_add_child ((ClutterActor *)item->layout, (ClutterActor *)item->label_bin);

    return item;
}
static void
cdos_mount_operation_destroy_list_item (ListItem *list_item)
{
    clutter_actor_destroy (CLUTTER_ACTOR (list_item->layout));
    g_free (list_item);
}

static void
cdos_mount_operation_destroy_processes_dialog_list_items (ProcessesDialog *processes_dialog)
{
    if (processes_dialog->list_items)
    {
        GList *list_items = processes_dialog->list_items;
        while (list_items)
        {
            ListItem *item = list_items->data;
            if (item)
            {
                cdos_mount_operation_destroy_list_item (item);
                list_items->data = NULL;
            }
            list_items = list_items->next;
        }
        g_list_free (processes_dialog->list_items);
    }
}

static void
cdos_mount_operation_list_item_on_activate (ListItem *item)
{
    // g_signal_emit(item,"response",-1);
}

static void
cdos_set_apps_for_pids (ProcessesDialog *processes, GArray *pids)
{
    clutter_actor_destroy_all_children (CLUTTER_ACTOR (processes->application_list));
    cdos_mount_operation_destroy_processes_dialog_list_items (processes);

    int i;
    CdosWindowTracker *tracker;
    tracker = cdos_window_tracker_get_default ();
    for (i = 0; i < pids->len; i++)
    {
        int pid;
        pid = g_array_index (pids, int, i);
        CdosApp *app;
        app = cdos_window_tracker_get_app_from_pid (tracker, pid);

        if (!app)
            continue;
        ListItem *item = cdos_mount_operation_create_list_item (app);
        clutter_actor_add_child ((ClutterActor *)processes->application_list, (ClutterActor *)item->layout);
        g_signal_connect(item->layout,
                         "button-press-release",
                         G_CALLBACK(cdos_mount_operation_list_item_on_activate),
                         NULL);
        processes->list_items = g_list_append (processes->list_items, item);
    }
}


static void
cdos_processes_dialog_update (ProcessesDialog *dialog, gchar *message, GArray *processes, GStrv choices)
{
    cdos_set_apps_for_pids (dialog, processes);

    cdos_mount_operation_set_label_for_messages ((MountQuestionDialog *)dialog, message);

    cdos_mount_operation_set_button_for_choices ((ModalDialog *)dialog, choices);
}

static MountQuestionDialog *
cdos_mount_operation_create_question_dialog (void)
{
    MountQuestionDialog *question_dialog;
    question_dialog = g_new0 (MountQuestionDialog, 1);

    ModalDialog *modal = (ModalDialog *)question_dialog;
    cdos_modal_dialog_init (modal, FALSE, NULL);

    question_dialog->main_content_layout = st_box_layout_new ();
    clutter_actor_add_child ((ClutterActor *)modal->content_layout,
                             (ClutterActor *)question_dialog->main_content_layout);

    /*ListItem *item;
    item = g_new0(ListItem ,1);
    item ->icon_bin = g_object_new(ST_TYPE_BIN, "child",icon, NULL);
    clutter_actor_add_child((ClutterContainer *)question_dialog->main_content_layout, (ClutterActor *)item->icon_bin);*/

    question_dialog->message_layout = g_object_new (ST_TYPE_BOX_LAYOUT, "vertical", TRUE, NULL);
    clutter_actor_add_child ((ClutterActor *)question_dialog->main_content_layout,
                             (ClutterActor *)question_dialog->message_layout);

    question_dialog->subject_label = g_object_new (ST_TYPE_LABEL, "style-class", "mount-question-dialog-subject", NULL);
    clutter_actor_add_child ((ClutterActor *)question_dialog->message_layout,
                             (ClutterActor *)question_dialog->subject_label);

    question_dialog->description_label =
        g_object_new (ST_TYPE_LABEL, "style-class", "mount-question-dialog-description", NULL);
    clutter_text_set_ellipsize (CLUTTER_TEXT (question_dialog->description_label), PANGO_ELLIPSIZE_NONE);
    clutter_text_set_line_wrap (CLUTTER_TEXT (question_dialog->description_label), TRUE);
    clutter_actor_add_child ((ClutterActor *)question_dialog->message_layout,
                             (ClutterActor *)question_dialog->description_label);

    return question_dialog;
}

static void
cdos_mount_operation_destroy_question_dialog (ModalDialog *modal)
{
    MountQuestionDialog *question_dialog = (MountQuestionDialog *)modal;
    if (question_dialog->main_content_layout)
    {
        clutter_actor_destroy (CLUTTER_ACTOR (question_dialog->main_content_layout));
        question_dialog->main_content_layout = NULL;
    }
    cdos_modal_dialog_destroy (modal);
    g_free (question_dialog);
}

static void
cdos_mount_question_dialog_update (ModalDialog *dialog, gchar *message, gchar **choices)
{
    cdos_mount_operation_set_label_for_messages ((MountQuestionDialog *)dialog, message);

    cdos_mount_operation_set_button_for_choices (dialog, choices);
}

/**
 * cdos_mount_operation_set_label_text:
 * @label: A #StLabel
 * @text: 设置标签
 *
 * 设置选项的标签
 *
 */
static void
cdos_mount_operation_set_label_text (StLabel *label, const gchar *text)
{
    if (text)
    {
        st_label_set_text (label, text);
        clutter_actor_show ((ClutterActor *)label);
    }
    else
    {
        st_label_set_text (label, "");
        clutter_actor_hide (CLUTTER_ACTOR (label));
    }
}
static void
cdos_mount_operation_button_on_activate (gpointer button, int clicked_button, gpointer dialog)
{
    guint32 time = cdos_global_get_current_time (global);
    cdos_modal_dialog_close (dialog, time);
}

static void
cdos_mount_operation_set_button_for_choices (ModalDialog *dialog, GStrv choices)
{
    GList *buttons = NULL;
    int i;
    for (i = 0; choices[i] != NULL; i++)
    {
        ButtonInfo *info = g_new0 (ButtonInfo, 1);
        info->userdata = dialog;
        info->label = choices[i];
        info->callback = cdos_mount_operation_button_on_activate;
        buttons = g_list_prepend (buttons, info);
    }
    cdos_modal_dialog_set_buttons (dialog, buttons);
}

static void
cdos_mount_operation_set_label_for_messages (MountQuestionDialog *dialog, gchar *message)
{
    gchar **labels = g_strsplit (message, "\n", -1);
    cdos_mount_operation_set_label_text (dialog->subject_label, labels[0]);

    if (labels[0] != NULL && labels[1] != NULL)
    {
        cdos_mount_operation_set_label_text (dialog->description_label, labels[1]);
    }
}

static void
cdos_mount_operation_dialog_on_response (ModalDialog *dialog, int choice, CdosMountOperation *self)
{
    // g_print ("choice is %d\n", choice);
    CdosMountOperationPrivate *priv = self->priv;
    g_mount_operation_set_choice (G_MOUNT_OPERATION (self), choice);
    g_mount_operation_reply (G_MOUNT_OPERATION (self), G_MOUNT_OPERATION_HANDLED);

    CdosGlobal *global = cdos_global_get ();
    guint32 time = cdos_global_get_current_time (global);

    cdos_modal_dialog_close (priv->dialog, time);
    cdos_mount_operation_destroy_question_dialog (priv->dialog);
    priv->dialog = NULL;
}


static void
cdos_mount_operation_on_ask_question (GMountOperation *base_mount_operation,
                                      gchar *message,
                                      GStrv choices,
                                      gpointer user_data)
{
    // g_print("\n%s:=====>> message = %s;\n",__func__,message);

    guint i = 0;
    for (i = 0;; i++)
    {
        // g_print ("choices is %s\n", choices[i]);
        if (choices[i] == NULL)
        {
            break;
        }
    }
    CdosMountOperation *self = (CdosMountOperation *)user_data;
    CdosMountOperationPrivate *priv = self->priv;
    priv->dialog = (ModalDialog *)cdos_mount_operation_create_question_dialog ();
    priv->dialog_destroy = cdos_mount_operation_destroy_question_dialog;

    g_signal_connect (priv->dialog, "response", G_CALLBACK (cdos_mount_operation_dialog_on_response), self);
    cdos_mount_question_dialog_update (priv->dialog, message, choices);
    CdosGlobal *global = cdos_global_get ();
    cdos_modal_dialog_open (priv->dialog, cdos_global_get_current_time (global));
}

static void
cdos_mount_operation_source_on_password_ready (CdosMountOperation *self, gchar *password)
{
    CdosMountOperationPrivate *priv = self->priv;
    g_mount_operation_set_password (G_MOUNT_OPERATION (self), password);
    g_mount_operation_reply (G_MOUNT_OPERATION (self), G_MOUNT_OPERATION_HANDLED);

    priv->notification_showing = FALSE;
    clutter_actor_destroy ((ClutterActor *)(priv->source));
}

static void
cdos_mount_operation_source_on_destroy (CdosMountOperation *self)
{
    CdosMountOperationPrivate *priv = self->priv;
    if (!(priv->notification_showing))
        return;

    priv->notification_showing = FALSE;
    g_mount_operation_reply (G_MOUNT_OPERATION (self), G_MOUNT_OPERATION_ABORTED);
}


static void
cdos_mount_operation_on_ask_password (GMountOperation *base_mount_operation, gchar *message, CdosMountOperation *self)
{
    CdosMountOperationPrivate *priv = self->priv;
    priv->notification_showing = 1;

    if (priv->source)
    {
        cdos_mount_operation_destroy_password_source (priv->source);
    }

    priv->source = cdos_mount_operation_create_password_source (message, priv->reask_password);
    g_signal_connect(priv->source,
                     "password-ready",
                     G_CALLBACK(cdos_mount_operation_source_on_password_ready),
                     NULL);

    g_signal_connect(priv->source,
                     "destroy",
                     G_CALLBACK(cdos_mount_operation_source_on_destroy),
                     NULL);
}


static void
cdos_mount_operation_on_aborted (GMountOperation *base_mount_operation, CdosMountOperation *self)
{
    CdosMountOperationPrivate *priv = self->priv;
    if (!(priv->dialog))
        return;

    CdosGlobal *global = cdos_global_get ();
    cdos_modal_dialog_close (priv->dialog, cdos_global_get_current_time (global));

    priv->dialog = NULL;
}

static void
cdos_mount_operation_processes_dialog_on_response (CdosMountOperation *self,
                                                   ProcessesDialog *processes_dialog,
                                                   int choice)
{
    // g_print ("%d %s\n", __func__, __LINE__);
    CdosMountOperationPrivate *priv = self->priv;
    if (choice == -1)
    {
        g_mount_operation_reply (G_MOUNT_OPERATION (self), G_MOUNT_OPERATION_ABORTED);
    }
    else
    {
        g_mount_operation_set_choice (G_MOUNT_OPERATION (self), choice);
        g_mount_operation_reply (G_MOUNT_OPERATION (self), G_MOUNT_OPERATION_HANDLED);
    }

    CdosGlobal *global = cdos_global_get ();
    cdos_modal_dialog_close ((ModalDialog *)priv->processes_dialog, cdos_global_get_current_time (global));

    priv->dialog = NULL;
}

static void
cdos_mount_operation_on_show_processes2 (CdosBaseMountOperation *base_mount_operation, CdosMountOperation *self)
{
    CdosMountOperationPrivate *priv = self->priv;
    priv->processes = cdos_base_mount_operation_get_show_processes_pids ((CdosBaseMountOperation *)self);
    priv->choices = cdos_base_mount_operation_get_show_processes_choices ((CdosBaseMountOperation *)self);
    priv->message = cdos_base_mount_operation_get_show_processes_message ((CdosBaseMountOperation *)self);

    if (!(priv->processes_dialog))
    {
        priv->processes_dialog = cdos_mount_operation_create_processes_dialog ((StIcon *)priv->icon);
        priv->dialog = (ModalDialog *)priv->processes_dialog;
        priv->dialog_destroy = cdos_mount_operation_destroy_processes_dialog;
        g_signal_connect (
            priv->processes_dialog, "response", G_CALLBACK (cdos_mount_operation_processes_dialog_on_response), NULL);
        CdosGlobal *global = cdos_global_get ();
        cdos_modal_dialog_open ((ModalDialog *)priv->processes_dialog, cdos_global_get_current_time (global));
    }
    cdos_processes_dialog_update ((ProcessesDialog *)self, priv->message, priv->processes, priv->choices);
}

/**
 * cdos_mount_operation_new:
 * @source: #CdosSource
 * @reask_password: 是否有请求密码
 *
 * 新建一个 #CdosMountOperation 类的实例。
 * Notice:该模块的代码未进行全面的调试和优化，如有使用，请重新调试和优化。
 * Returns: 返回一个 #CdosMountOperation 的实例。
 */
CdosMountOperation *
cdos_mount_operation_new (GVolume *source, gboolean reask_password)
{
    g_return_val_if_fail (G_IS_VOLUME (source), NULL);

    CdosMountOperation *mount_operation = NULL;
    mount_operation = g_object_new (CDOS_TYPE_MOUNT_OPERATION, NULL);
    CdosMountOperationPrivate *priv = mount_operation->priv;
    GIcon *gicon = g_volume_get_icon (source);
    st_icon_set_gicon (ST_ICON (priv->icon), gicon);
    priv->reask_password = reask_password;

    return mount_operation;
}

static MountPasswordSource *
cdos_mount_operation_create_password_source (gchar *message, gboolean reask_password)
{
    MountPasswordSource *password;
    password = g_new0 (MountPasswordSource, 1);
    password->strings = g_strsplit (message, "\n", -1);
    CdosSource *source = cdos_source_new (password->strings[0]);
    password->source = source;
    password->notification =
        cdos_mount_operation_create_password_notification (source, password->strings, reask_password);
    CdosMessageTray *message_tray = cdos_message_tray_get_default ();

    if (message_tray)
        clutter_actor_add_child ((ClutterActor *)message_tray, (ClutterActor *)password);
    cdos_source_notify (CDOS_SOURCE (password), (CdosNotification *)password->notification);

    return password;
}
static void
cdos_mount_operation_destroy_password_source (MountPasswordSource *password_source)
{
    cdos_mount_operation_destroy_password_notification (password_source->notification);
    g_object_unref (password_source->source);
    g_free (password_source->strings);
    g_free (password_source);
}


static void
cdos_on_entry_activated (MountPasswordNotification *pnotification, CdosSource *source)
{
    const gchar *text = st_entry_get_text ((StEntry *)pnotification->response_entry);
    if (g_strcmp0 (text, "") == 0)
        return;
    // g_signal_emit(source,"password-ready",text,NULL);
}

static MountPasswordNotification *
cdos_mount_operation_create_password_notification (CdosSource *source, gchar **strings, gboolean reask_password)
{
    CdosNotificationParams *params = g_new0 (CdosNotificationParams, 1);

    MountPasswordNotification *password_notification;
    password_notification = g_new0 (MountPasswordNotification, 1);

    CdosNotification *notification = cdos_notification_new (source, strings[0], NULL, params);
    g_free (params);

    cdos_notification_set_transient (notification, FALSE);
    cdos_notification_set_urgency (notification, CDOS_NOTIFICATION_CRITICAL);

    if (reask_password)
    {
        password_notification->label = g_object_new (
            ST_TYPE_LABEL, "style-class", "mount-password-reask", "text", "Wrong password, please try again", NULL);
        // clutter_actor_add_child((ClutterContainer *)password_notification, (ClutterActor
        // *)password_notification->label);
    }

    password_notification->response_entry = ST_BOX_LAYOUT (st_box_layout_new ());

    /*
    StWidget *entry = g_object_new (ST_TYPE_ENTRY, "style-class", "mount-password-entry", "can-focus", TRUE, NULL);

    cdos_notification_set_action_area(notification,
                                      password_notification->response_entry,
                                      entry);
    */

    ClutterActor *clutter_text = st_entry_get_clutter_text ((StEntry *)password_notification->response_entry);
    g_signal_connect (clutter_text, "activate", G_CALLBACK (cdos_on_entry_activated), password_notification);

    clutter_text_set_password_char ((ClutterText *)clutter_text, 0x25cf);
    clutter_actor_grab_key_focus ((ClutterActor *)password_notification->response_entry);

    password_notification->notification = notification;

    return password_notification;
}
static void
cdos_mount_operation_destroy_password_notification (MountPasswordNotification *password_notification)
{
    g_free (password_notification);
}

static void
cdos_mount_operation_init (CdosMountOperation *self)
{
    CdosMountOperationPrivate *priv;
    priv = self->priv = cdos_mount_operation_get_instance_private (self);

    priv->dialog = NULL;
    priv->processes_dialog = NULL;

    g_signal_connect (self, "ask-question", G_CALLBACK (cdos_mount_operation_on_ask_question), self);
    g_signal_connect (self, "ask-password", G_CALLBACK (cdos_mount_operation_on_ask_password), self);
    g_signal_connect (self, "show-processes-2", G_CALLBACK (cdos_mount_operation_on_show_processes2), self);
    g_signal_connect (self, "aborted", G_CALLBACK (cdos_mount_operation_on_aborted), self);

    priv->icon = st_icon_new ();
    st_widget_set_style_class_name ((StWidget *)priv->icon, "mount-operation-icon");
}

static void
cdos_mount_operation_class_init (CdosMountOperationClass *klass)
{
}

