/** 
 * Copyright (c) 2016-2018 CPU and Fundamental Software Research Center, CAS
 *  
 * This software is published by the license of CPU-OS License，you can use and 
 * distibuted this software under this License. See CPU-OS Liense for more detail.
 *
 * You should have receive a copy of CPU-OS License. If not, please contact us 
 * by email <support_os@cpu-os.ac.cn>  
 * 
 **/
/*
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *
 */
/*
 * 参考 http://github.com/matiasdelellis/libmpris2client.git
 */

#ifndef __CDOS_MPRIS2_PLAYER_H__
#define __CDOS_MPRIS2_PLAYER_H__

#include <glib-object.h>

G_BEGIN_DECLS

typedef enum
{
    CDOS_MPRIS2_NONE = 0,
    CDOS_MPRIS2_CAN_SEEK = 1,
    CDOS_MPRIS2_CAN_GO_PREVIOUS = 2,
    CDOS_MPRIS2_CAN_GO_NEXT = 4,
    CDOS_MPRIS2_CAN_PLAY = 8,
    CDOS_MPRIS2_CAN_PAUSE = 16,
    CDOS_MPRIS2_CAN_CONTROL = 32
}CdosMpris2PropsFlags;

typedef enum
{
    CDOS_MPRIS2_PLAYING,
    CDOS_MPRIS2_PAUSED,
    CDOS_MPRIS2_STOPPED
}CdosMpris2PlaybackStatus;

typedef enum
{
    CDOS_MPRIS2_LOOP_NONE,
    CDOS_MPRIS2_LOOP_TRACK,
    CDOS_MPRIS2_LOOP_PLAYLIST
}CdosMpris2LoopStatus;

typedef struct _CdosMpris2Metadata CdosMpris2Metadata;

#define CDOS_TYPE_MPRIS2_PLAYER (cdos_mpris2_player_get_type ())

G_DECLARE_FINAL_TYPE (CdosMpris2Player, cdos_mpris2_player, CDOS, MPRIS2_PLAYER, GObject)

CdosMpris2Player *cdos_mpris2_player_new (void);

void cdos_mpris2_player_set_player       (CdosMpris2Player *mpris2, GDBusConnection *connection, const gchar *dbus_name);

const gchar *cdos_mpris2_player_get_player (CdosMpris2Player *self);

gboolean cdos_mpris2_player_is_connected (CdosMpris2Player *mpris2);

/*
 * Interface MediaPlayer2 Methods.
 */
void cdos_mpris2_player_raise_player (CdosMpris2Player *mpris2);
void cdos_mpris2_player_quit_player  (CdosMpris2Player *mpris2);

/*
 * Interface MediaPlayer2 Properties.
 */
gboolean        cdos_mpris2_player_can_quit                  (CdosMpris2Player *mpris2);
gboolean        cdos_mpris2_player_can_raise                 (CdosMpris2Player *mpris2);
gboolean        cdos_mpris2_player_has_tracklist_support     (CdosMpris2Player *mpris2);
const gchar    *cdos_mpris2_player_get_player_identity       (CdosMpris2Player *mpris2);
gchar         **cdos_mpris2_player_get_supported_uri_schemes (CdosMpris2Player *mpris2);
gchar         **cdos_mpris2_player_get_supported_mime_types  (CdosMpris2Player *mpris2);

/*
 * Optional Properties
 *
 * Fullscreen
 * CanSetFullscreen
 * DesktopEntry
 */
const gchar *cdos_mpris2_player_get_player_desktop_entry  (CdosMpris2Player *mpris2);

/*
 * MediaPlayer2.Player
 * https://specfications.freedesktop.org/mpris-spec/latest/Player_Interface.html
 */
/*
 * Methods
 */
void cdos_mpris2_player_next            (CdosMpris2Player *mpris2);
void cdos_mpris2_player_previous        (CdosMpris2Player *mpris2);
void cdos_mpris2_player_pause           (CdosMpris2Player *mpris2);
void cdos_mpris2_player_play_pause      (CdosMpris2Player *mpris2);
void cdos_mpris2_player_stop            (CdosMpris2Player *mpris2);
void cdos_mpris2_player_play            (CdosMpris2Player *mpris2);
void cdos_mpris2_player_seek            (CdosMpris2Player *mpris2, gint64 offset);
//void cdos_mpris2_player_set_position    (CdosMpris2Player *mpris2, const gchar *track_id, gint position);
//void cdos_mpris2_player_open_uri        (CdosMpris2Player *mpris2, const gchar *uri);

/*
 * Properties
 */
CdosMpris2PlaybackStatus  cdos_mpris2_player_get_playback_status (CdosMpris2Player *mpris2);

gdouble  cdos_mpris2_player_get_playback_rate         (CdosMpris2Player *mpris2);

CdosMpris2Metadata *cdos_mpris2_player_get_metadata   (CdosMpris2Player *mpris2);

gdouble  cdos_mpris2_player_get_volume                (CdosMpris2Player *mpris2);
void     cdos_mpris2_player_set_volume                (CdosMpris2Player *mpris2, gdouble volume);

gint64   cdos_mpris2_player_get_position              (CdosMpris2Player *mpris2);
gint64   cdos_mpris2_player_get_accurate_position     (CdosMpris2Player *mpris2);

gdouble  cdos_mpris2_player_get_minimum_rate          (CdosMpris2Player *mpris2);
gdouble  cdos_mpris2_player_get_maximum_rate          (CdosMpris2Player *mpris2);

gboolean cdos_mpris2_player_get_can_go_next           (CdosMpris2Player *mpris2);
gboolean cdos_mpris2_player_get_can_go_previous       (CdosMpris2Player *mpris2);

gboolean cdos_mpris2_player_get_can_play              (CdosMpris2Player *mpris2);
gboolean cdos_mpris2_player_get_can_pause             (CdosMpris2Player *mpris2);

gboolean cdos_mpris2_player_get_can_seek              (CdosMpris2Player *mpris2);

gboolean cdos_mpris2_player_get_can_control           (CdosMpris2Player *mpris2);

/*
 * Optional Properties
 *
 * LoopStatus
 * Shuffle
 */

/*
 * Metadata
 */
const gchar  *cdos_mpris2_metadata_get_title   (CdosMpris2Metadata *metadata);
gchar        *cdos_mpris2_metadata_get_artists (CdosMpris2Metadata *metadata);
const gchar  *cdos_mpris2_metadata_get_album   (CdosMpris2Metadata *metadata);
const gchar  *cdos_mpris2_metadata_get_arturl  (CdosMpris2Metadata *metadata);
const gchar  *cdos_mpris2_metadata_get_url     (CdosMpris2Metadata *metadata);
const gchar  *cdos_mpris2_metadata_get_trackid (CdosMpris2Metadata *metadata);
guint   cdos_mpris2_metadata_get_track_no      (CdosMpris2Metadata *metadata);
gint64   cdos_mpris2_metadata_get_length       (CdosMpris2Metadata *metadata);

#endif
