/** 
 * Copyright (c) 2016-2018 CPU and Fundamental Software Research Center, CAS
 *  
 * This software is published by the license of CPU-OS License，you can use and 
 * distibuted this software under this License. See CPU-OS Liense for more detail.
 *
 * You should have receive a copy of CPU-OS License. If not, please contact us 
 * by email <support_os@cpu-os.ac.cn>  
 * 
 **/
#ifndef __CDOS_NOTIFICATION_DAEMON_SOURCE_H__
#define __CDOS_NOTIFICATION_DAEMON_SOURCE_H__

#include <string.h>
#include <glib.h>
#include <glib-object.h>
#include "cdos-source.h"
#include "cdos-global.h"
#include "cdos-app.h"
#include "cdos-window-tracker.h"

#define CDOS_TYPE_NOTIFICATION_DAEMON_SOURCE            (cdos_notification_daemon_source_get_type ())

#define CDOS_NOTIFICATION_DAEMON_SOURCE(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), CDOS_TYPE_NOTIFICATION_DAEMON_SOURCE, CdosNotificationDaemonSource))

#define CDOS_IS_NOTIFICATION_DAEMON_SOURCE(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), CDOS_TYPE_NOTIFICATION_DAEMON_SOURCE))

#define CDOS_NOTIFICATION_DAEMON_SOURCE_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), CDOS_TYPE_NOTIFICATION_DAEMON_SOURCE, CdosNotificationDaemonSourceClass))

#define CDOS_IS_NOTIFICATION_DAEMON_SOURCE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), CDOS_TYPE_NOTIFICATION_DAEMON_SOURCE))

#define CDOS_NOTIFICATION_DAEMON_SOURCE_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), CDOS_TYPE_NOTIFICATION_DAEMON_SOURCE, CdosNotificationDaemonSourceClass))

typedef struct _CdosNotificationDaemonSource CdosNotificationDaemonSource;
typedef struct _CdosNotificationDaemonSourceClass CdosNotificationDaemonSourceClass;
typedef struct _CdosNotificationDaemonSourcePrivate CdosNotificationDaemonSourcePrivate;

/**
 * NotificationDaemonSourceLevel: 
 * @CLEAN: 无横幅无声音
 * @DND: Do not Disturb, 勿扰模式
 * @SILENT: 静音模式
 * @DEFAULT: 默认级别，表示有提示音和提示横幅
 * 
 * CdosNotification 来源的应用在通知中的级别
 */
typedef enum _CdosNotificationDaemonSourceLevel
{
    CDOS_NOTIFICATION_DAEMON_SOURCE_BLOCK,
    CDOS_NOTIFICATION_DAEMON_SOURCE_CLEAN,
    CDOS_NOTIFICATION_DAEMON_SOURCE_DND,
    CDOS_NOTIFICATION_DAEMON_SOURCE_SILENT,
    CDOS_NOTIFICATION_DAEMON_SOURCE_UNDEFINED_RESERVED
}CdosNotificationDaemonSourceLevel;

struct _CdosNotificationDaemonSource
{
    CdosSource parent_instance;

    CdosApp *app;
    gboolean useNotificationIcon;

    /*<private>*/
    CdosNotificationDaemonSourcePrivate *priv;
};

struct _CdosNotificationDaemonSourceClass
{
    CdosSourceClass parent_class;

    /*<private>*/
};

GType cdos_notification_daemon_source_get_type (void);

/*接口*/
//gboolean cdos_notification_daemon_source_handle_summary_click(CdosNotificationDaemonSource *self);
void cdos_notification_daemon_source_process_notification (CdosNotificationDaemonSource *self, CdosNotification *notification);

/* property */
void cdos_notification_daemon_source_set_pid (CdosNotificationDaemonSource *self, guint pid);
void cdos_notification_daemon_source_set_sender (CdosNotificationDaemonSource *self, const gchar *sender);

gchar * cdos_notification_daemon_source_get_sender (CdosNotificationDaemonSource *self);
guint cdos_notification_daemon_source_get_pid (CdosNotificationDaemonSource *self);

guint cdos_notification_daemon_source_get_level (CdosNotificationDaemonSource *self);
void cdos_notification_daemon_source_set_level (CdosNotificationDaemonSource *self, guint level);

CdosNotificationDaemonSource * cdos_notification_daemon_source_new (const gchar *name, guint pid, const gchar *sender);

#endif
