/** 
 * Copyright (c) 2016-2018 CPU and Fundamental Software Research Center, CAS
 *  
 * This software is published by the license of CPU-OS License，you can use and 
 * distibuted this software under this License. See CPU-OS Liense for more detail.
 *
 * You should have receive a copy of CPU-OS License. If not, please contact us 
 * by email <support_os@cpu-os.ac.cn>  
 * 
 **/
#ifndef __CDOS_NOTIFICATION_H__
#define __CDOS_NOTIFICATION_H__

#include "cdos-url-highlighter.h"
#include "st.h"

GType cdos_notification_get_type (void);

#define CDOS_TYPE_NOTIFICATION        (cdos_notification_get_type ())
#define CDOS_NOTIFICATION(obj)              (G_TYPE_CHECK_INSTANCE_CAST ((obj), CDOS_TYPE_NOTIFICATION, CdosNotification))
#define CDOS_IS_NOTIFICATION(obj)           (G_TYPE_CHECK_INSTANCE_TYPE ((obj), CDOS_TYPE_NOTIFICATION))
#define CDOS_NOTIFICATION_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST ((klass), CDOS_TYPE_NOTIFICATION, CdosNotificationClass))
#define CDOS_IS_NOTIFICATION_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE ((klass), CDOS_TYPE_NOTIFICATION))
#define CDOS_NOTIFICATION_GET_CLASS(obj)   (G_TYPE_INSTANCE_GET_CLASS ((obj), CDOS_TYPE_NOTIFICATION, CdosNotificationClass))

typedef struct _CdosNotification            CdosNotification;
typedef struct _CdosNotificationPrivate     CdosNotificationPrivate;
typedef struct _CdosNotificationClass       CdosNotificationClass;

typedef struct _CdosSource CdosSource;

/**
 * NotificationDestroyedReason: 
 * @EXPIRED: 表示使用者还没有与通知交互
 * @DISMISSED: 表示通知被使用者关闭
 * @SOURCE_CLOSED: 表示被其他相关的source要求关闭
 * 
 * CdosNotification 关闭原因的枚举变量
 */
typedef enum _CdosNotificationClosedReason
{
    CDOS_NOTIFICATION_EXPIRED,
    CDOS_NOTIFICATION_DISMISSED,
    CDOS_NOTIFICATION_SOURCE_CLOSED,
    CDOS_NOTIFICATION_UNDEFINED_RESERVED
}CdosNotificationClosedReason;

/**
 * Urgency:
 * @LOW: 紧急程度低
 * @NORMAL: 紧急程度一般
 * @HIGH: 紧急程度高
 * @CRITICAL: 崩溃
 *
 * 表示从notification daemon接收到通知的紧急程度的枚举变量
 */
typedef enum _CdosNofificationUrgency
{
    CDOS_NOTIFICATION_NORMAL,
    CDOS_NOTIFICATION_LOW,
    CDOS_NOTIFICATION_CRITICAL
}CdosNotificationUrgency;

struct _CdosNotification
{
    StWidget parent_instance;

    gchar *category;
    gchar *sound_file;
    gchar *sound_name;
    gchar *title;
    gchar *body;
    CdosNotificationUrgency urgency;
    gboolean resident;
    gboolean is_transient;
    CdosSource *source;
    GDateTime *timestamp;
    StLabel *time_label;
    StLabel *title_label;
    StLabel *body_label;
    ClutterActor *image;
    ClutterActor *action_area;
    gboolean unshown;

    gboolean in_notification_center;
    guint closed_reason;
    guint replaces_id;

    CdosNotificationPrivate *priv;
};

struct _CdosNotificationClass
{
    StWidgetClass parent_class;
};

/**
 * #Params
 * @customContent: 自定义内容
 * @body: 消息体
 * @icon: 图标
 * @titleMarkup: 标题是否允许标记
 * @bannerMarkup: 消息体是可以标记
 * @bodyMarkup: C版本里与bannerMarkup相同
 * @clear: 是否可清除,默认 %FALSE
 *
 */
typedef struct _CdosNotificationParams  CdosNotificationParams;
struct _CdosNotificationParams
{
    ClutterActor *icon;
    gboolean clear;
    guint replaces_id;
    const char *category;
    const char *sound_file;
    const char *sound_name;
    GDateTime *timestamp;
};

void cdos_notification_update (CdosNotification *self, const gchar *title, const gchar *banner, CdosNotificationParams *params );

void cdos_notification_set_icon_visible (CdosNotification *self, gboolean visible);

void cdos_notification_set_urgency (CdosNotification *self, guint urgency);

void cdos_notification_set_resident (CdosNotification *self, gboolean resident);

void cdos_notification_set_transient (CdosNotification *self, gboolean is_transient);
gboolean cdos_notification_get_transient (CdosNotification *self);

void cdos_notification_set_use_action_icons (CdosNotification *self, gboolean value);

void cdos_notification_set_actions (CdosNotification *self, const gchar **actions);

void cdos_notification_expand (CdosNotification *self , gboolean animate);
void cdos_notification_collapse (CdosNotification *self);

void cdos_notification_destroy (CdosNotification *self, CdosNotificationClosedReason reason);

gchar *cdos_notification_timeify (CdosNotification *self, const char *format);

CdosNotification *cdos_notification_new (CdosSource *source,
                                         const gchar *title,
                                         const gchar *body,
                                         CdosNotificationParams *params);

#endif
