#define CLUTTER_DISABLE_DEPRECATION_WARNINGS

#include <math.h>
#include <meta/window.h>
#include <meta/workspace.h>

#include "glib-2.62.h"

#include "cdos-overview-window-clone.h"
#include "cdos-overview-thumbnails-box.h"
#include "cdos-overview-workspace-thumbnail.h"
#include "cdos-window-manager.h"
#include "cdos-layout-manager.h"
#include "cdos-window-overlay.h"
#include "cdos-base-util.h"
#include "cdos-workspace.h"

#include "cdos-global.h"


/**
 * SECTION:cdos-overview-thumbnails-box
 * @Title: CdosOverviewThumbnailsBox
 * @Short_description: 存放工作区缩略图的容器
 *
 * CdosOverviewThumbnailsBox 用于工作区的展示方式，目前工作区默认按横排显示，
 * 通过修改 org.cdos.workspace-overview-view-as-grid 的值，可以网格显示工作区缩略图
 * 成员 thumnbnails 保存系统中所有的 CdosOverviewWorkspaceThumbnail 对象，默认有四个工作区，也就是四个工作区缩略图
 */
typedef struct
{
    gpointer param1;
    gpointer param2;

} Params_overview;

enum
{
    PROP_0,
    PROP_SCALE
};

// The maximum size of a thumbnail is 1/8 the width and height of the screen
#define MAX_THUMBNAIL_SCALE 0.9

#define POINTER_LEAVE_MILLISECONDS_GRACE 500
#define POINTER_ENTER_MILLISECONDS_GRACE 150

#define RESCALE_ANIMATION_TIME 200

struct _CdosOverviewThumbnailsBox
{
    StWidget parent;

    GPtrArray *thumbnails;        //类型為CdosOverviewWorkspaceThumbnail
    ClutterActorBox box;
    MetaRectangle porthole;

    StWidget *add_workspace_button_box;
    StWidget *add_workspace_button;
    StWidget *add_workspace_title_box;
    StWidget *add_workspace_title;

    CdosOverviewWorkspaceThumbnail *lastActiveWorkspace;

    CdosOverviewWorkspaceThumbnail *lastHovered;

    gint kb_thumbnail_index;
    guint allocId;

    gulong switchWorkspaceNotifyId;
    gulong workspaceAddedId;
    gulong workspaceRemovedId;
};

struct _CdosOverviewThumbnailsBoxClass
{
    StWidgetClass parent_class;
};

G_DEFINE_TYPE  (CdosOverviewThumbnailsBox, cdos_overview_thumbnails_box, ST_TYPE_WIDGET);

static void
getPreferredWidth (CdosOverviewThumbnailsBox *self,
                   gfloat *forHeight,
                   gfloat *min_size,
                   gfloat *natural_size)
{
    if (self->thumbnails == NULL)
        return;

    gdouble spacing;
    spacing = st_theme_node_get_length (st_widget_get_theme_node (ST_WIDGET (self)), "spacing" );

    guint nWorkspaces;
    MetaDisplay *display = cdos_global_get_display (cdos_global_get());
    MetaWorkspaceManager *wsm = meta_display_get_workspace_manager (display);
    nWorkspaces = meta_workspace_manager_get_n_workspaces (wsm);
    gfloat totalSpacing;
    totalSpacing = (nWorkspaces - 1) * spacing;

    CdosLayoutManager *layout_manager = cdos_layout_manager_get_default ();
    const MetaRectangle *rect;
    rect = cdos_layout_manager_get_primary_monitor (layout_manager);

    gfloat width;
    width = rect->width * thumbnail_box_scale * cdos_global_get_ui_scale (global);

    *min_size = totalSpacing;
    *natural_size = width * nWorkspaces + totalSpacing;
}

static void
getPreferredHeight (CdosOverviewThumbnailsBox *self,
                    gfloat *forWidth,
                    gfloat *min_size,
                    gfloat *natural_size)
{
    if (self->thumbnails == NULL)
        return;

    gdouble spacing = st_theme_node_get_length (st_widget_get_theme_node (ST_WIDGET (self)), "spacing" );

    guint nWorkspaces;
    MetaDisplay *display = cdos_global_get_display (cdos_global_get());
    MetaWorkspaceManager *wsm = meta_display_get_workspace_manager (display);
    nWorkspaces = meta_workspace_manager_get_n_workspaces (wsm);
    gfloat totalSpacing;
    totalSpacing = (nWorkspaces - 1) * spacing;

    CdosLayoutManager *layout_manager = cdos_layout_manager_get_default ();
    const MetaRectangle *rect = cdos_layout_manager_get_primary_monitor (layout_manager);

    gfloat avail;
    avail = *forWidth - totalSpacing;

    gfloat scale;
    scale = (avail / nWorkspaces) / rect->width;
    scale = min (scale, thumbnail_box_scale);

    gfloat height = rect->height * scale;

    *min_size = height;
    *natural_size = height;
}

static void
allocate (CdosOverviewThumbnailsBox *self, ClutterActorBox *box, ClutterAllocationFlags flags)
{
    if (self->thumbnails == NULL) // not visible
        return;

    CdosLayoutManager *layout = cdos_layout_manager_get_default ();
    const MetaRectangle *primary = cdos_layout_manager_get_primary_monitor (layout);

    self->box = *box;

    gdouble spacing = st_theme_node_get_length (st_widget_get_theme_node (ST_WIDGET (self)), "spacing" );
    spacing = max (spacing, 30.0);

    guint nWorkspaces = self->thumbnails->len;

    guint nColumns = nWorkspaces;

    gfloat totalSpacingX = (nColumns - 1) * spacing;
    gfloat availX = (self->box.x2 - self->box.x1) - totalSpacingX - spacing * 2 ;
    gfloat availY = (self->box.y2 - self->box.y1) - 2 * spacing;
    //gfloat scaleY = availY / (box->y2 - box->y1);
    //gfloat thumbnailHeight = box->y2 - box->y1 - 2 * spacing; //availY * thumbnail_box_scale;

    gfloat scale = 1.0 / nWorkspaces;
    if (thumbnail_box_scale > scale)
        thumbnail_box_scale = scale;

    gfloat thumbnailWidth  = availX * thumbnail_box_scale;

    gfloat x = 0.f;//spacing + (availX - nColumns * thumbnailHeight) / 2.0;
    gfloat y = (box->y2 - box->y1 - availY) / 2.0;
    gfloat neededX = (thumbnailWidth * nColumns) + totalSpacingX + (spacing * 2);
    gfloat extraSpaceX = (self->box.x2 - self->box.x1) - neededX;
    x = spacing + extraSpaceX / 2.0;
    gfloat title_box_w, title_box_h;
    for (int i = 0; i < nWorkspaces; i++)
    {
        CdosOverviewWorkspaceThumbnail *thumbnail = self->thumbnails->pdata[i];

        ClutterActorBox childBox;
        childBox.x1 = x;
        childBox.x2 = x + primary->width;
        childBox.y1 = y;
        childBox.y2 = y + primary->height;

        clutter_actor_set_scale (CLUTTER_ACTOR (thumbnail), thumbnail_box_scale, thumbnail_box_scale);

        clutter_actor_allocate (CLUTTER_ACTOR (thumbnail), &childBox, flags);


        // 设置每个缩略图的 title 
        clutter_actor_get_size (CLUTTER_ACTOR (thumbnail->workspace_title_box), &title_box_w, &title_box_h); 
        gchar * workspace_name = g_strdup_printf ("%s%d",_("Workspace"), i+1);
        st_label_set_text (thumbnail->workspace_title, workspace_name);
        // g_free (workspace_name);

        ClutterActorBox childBox_title_box;
        childBox_title_box.x1 = x;
        childBox_title_box.x2 = x + (primary->width * thumbnail_box_scale);
        childBox_title_box.y1 = y - title_box_h;
        childBox_title_box.y2 = y;
        clutter_actor_allocate (CLUTTER_ACTOR (thumbnail->workspace_title_box), &childBox_title_box, flags);


        // 设置每个缩略图的 close_button
        ClutterActorBox childBox_close_button;
        childBox_close_button.x1 = x + (primary->width * thumbnail_box_scale) - 17 ;
        childBox_close_button.x2 = x + (primary->width * thumbnail_box_scale) + 17;
        childBox_close_button.y1 = y - 17;
        childBox_close_button.y2 = y + 17;
        clutter_actor_allocate (CLUTTER_ACTOR (thumbnail->workspace_close_button), &childBox_close_button, flags);

        // 只有一个工作区时，隐藏关闭按钮。
        if (nWorkspaces==1)
            clutter_actor_hide (CLUTTER_ACTOR (thumbnail->workspace_close_button));
        else
            clutter_actor_show (CLUTTER_ACTOR (thumbnail->workspace_close_button));

        //y = y + spacing + thumbnailHeight;
        x = x + spacing + thumbnailWidth;
    }

    // 设置 增加工作区的标题
    ClutterActorBox childBox_add_workspace_title_box;
    childBox_add_workspace_title_box.x1 = x;
    childBox_add_workspace_title_box.x2 = x + (primary->width * thumbnail_box_scale); 
    childBox_add_workspace_title_box.y1 = y - title_box_h;
    childBox_add_workspace_title_box.y2 = y; 
    clutter_actor_allocate (CLUTTER_ACTOR (self->add_workspace_title_box), &childBox_add_workspace_title_box, flags);


    // 设置 增加工作区的按钮
    ClutterActorBox childBox_add_button;
    childBox_add_button.x1 = x;
    childBox_add_button.x2 = x + (primary->width * thumbnail_box_scale);
    childBox_add_button.y1 = y;
    childBox_add_button.y2 = y + (primary->height * thumbnail_box_scale);
    clutter_actor_allocate (CLUTTER_ACTOR (self->add_workspace_button), &childBox_add_button, flags);

    if (nWorkspaces >= 4)
    {
        clutter_actor_hide (CLUTTER_ACTOR (self->add_workspace_title_box));
        clutter_actor_hide (CLUTTER_ACTOR (self->add_workspace_button_box));
    }
    else
    {
        clutter_actor_show (CLUTTER_ACTOR (self->add_workspace_title_box));
        clutter_actor_show (CLUTTER_ACTOR (self->add_workspace_button_box));
    }
}

//尚未完成 source类型未知
/*
static gboolean background_acceptDrop (CdosOverviewThumbnailsBox *self, void *source, ClutterActor *actor, gint x, gint y, guint time) {
    MetaWindow *metaWindow = source->metaWindow;

    //这句不懂
    if (background_handleDragOver (self, source, actor, x, y, time) == DRAG_MOTION_RESULT_MOVE_DROP) {
                CdosDraggable *draggable = source._draggable;

                ClutterActor *parent = clutter_actor_get_parent (actor);
        clutter_actor_remove_child (parent, actor);

                draggable._dragOrigParent.add_actor(actor);
        clutter_actor_add_child ();

                clutter_actor_set_opacity (actor, draggable._dragOrigOpacity);

                meta_window_stick (metaWindow);

                return TRUE;
    }
    return FALSE;
}
*/

static void
onScrollEvent (GObject *gobject, ClutterEvent *event, gpointer user_data)
{
    switch (clutter_event_get_scroll_direction (event))
    {
        case CLUTTER_SCROLL_UP:
            cdos_wm_action_move_workspace_up ();
            break;

        case CLUTTER_SCROLL_DOWN:
            cdos_wm_action_move_workspace_up ();
            break;
        default:
            break;
    }
}

static void
activateSelectedWorkspace (CdosOverviewThumbnailsBox *self)
{
    CdosOverviewWorkspaceThumbnail *thumbnail;
    thumbnail = self->thumbnails->pdata[self->kb_thumbnail_index];
    cdos_overview_workspace_thumbnail_activate_workspace (thumbnail);
}

static gboolean
selectNextWorkspace (CdosOverviewThumbnailsBox *self, guint symbol)
{
    gint prev_index = self->kb_thumbnail_index;

    guint n_columns;
    MetaDisplay *display = cdos_global_get_display (cdos_global_get());
    MetaWorkspaceManager *wsm = meta_display_get_workspace_manager (display);
    n_columns = meta_workspace_manager_get_n_workspaces (wsm);

    gint next_index = prev_index;
    switch (symbol)
    {
        case CLUTTER_KEY_Right:
        case CLUTTER_KEY_Down:
            next_index = prev_index + 1;
            if (next_index == n_columns)
                next_index = 0;
            break;
        case CLUTTER_KEY_Left:
        case CLUTTER_KEY_Up:
            next_index = prev_index - 1;
            if (next_index < 0)
                next_index = n_columns - 1;
            break;
        default:
            break;
    }

    if (next_index >= 0)
    {
        self->kb_thumbnail_index = next_index;
    }
    CdosOverviewWorkspaceThumbnail *thumbnail;
    thumbnail = self->thumbnails->pdata[self->kb_thumbnail_index];
    clutter_actor_grab_key_focus ((ClutterActor *)thumbnail);

    return FALSE;
}

static void
activeWorkspaceChanged (CdosBaseWM *cdos_base_wm,
                        gint from,
                        gint to,
                        gint direction,
                        CdosOverviewThumbnailsBox *self)
{
    MetaDisplay *display = cdos_global_get_display (cdos_global_get());
    MetaWorkspaceManager *wsm = meta_display_get_workspace_manager (display);

    MetaWorkspace *activeWorkspace = meta_workspace_manager_get_active_workspace (wsm);
    MetaWorkspace *lastActiveWorkspace = meta_workspace_manager_get_workspace_by_index (wsm, from);

    for (int i = 0; i < self->thumbnails->len; i++)
    {
        CdosOverviewWorkspaceThumbnail *th = self->thumbnails->pdata[i];
        if (th->metaWorkspace == activeWorkspace)
            th->isActive = TRUE;

        else if (th->metaWorkspace == lastActiveWorkspace)
        {
            th->isActive = FALSE;
            cdos_overview_workspace_thumbnail_shade (th, TRUE);
        }
        gint m = max (to, from);
        if (i >= m)
            break;
        i++;
    }

    self->kb_thumbnail_index = to;
}

//判断是否為内部事件
static gboolean
isInternalEvent (CdosOverviewWorkspaceThumbnail *thumbnail, ClutterActor *actor, ClutterEvent *event)
{

    ClutterActor *relatedActor = clutter_event_get_related (event);

    return actor == relatedActor || clutter_actor_contains (CLUTTER_ACTOR (thumbnail), relatedActor);
}

static gboolean
onThumbnailMotionEvent (ClutterActor *actor,
                        ClutterEvent *event,
                        CdosOverviewThumbnailsBox *self)
{
    CdosOverviewWorkspaceThumbnail *thumbnail = CDOS_OVERVIEW_WORKSPACE_THUMBNAIL (actor);

    if (!thumbnail->hovering)
    {
        thumbnail->hovering = TRUE;
        self->lastHovered = thumbnail;
        cdos_overview_workspace_thumbnail_highlight (thumbnail);
        for (int i = 0; i < self->thumbnails->len; i++)
        {
            CdosOverviewWorkspaceThumbnail *th = self->thumbnails->pdata[i];
            if (th == thumbnail)
            {
                self->kb_thumbnail_index = i;
                break;
            }
        }
    }
    return TRUE;
}

static gboolean
onThumbnailLeaveEvent (ClutterActor *actor,
                       ClutterEvent *event,
                       CdosOverviewThumbnailsBox *self)
{
    CdosOverviewWorkspaceThumbnail *thumbnail = CDOS_OVERVIEW_WORKSPACE_THUMBNAIL (actor);

    if (thumbnail->hovering && !isInternalEvent (thumbnail, actor, event))
    {
        thumbnail->hovering = FALSE;
        if (thumbnail->isActive == FALSE)
            cdos_overview_workspace_thumbnail_shade (thumbnail, TRUE);
    }
    return FALSE;
}

static void
on_thumbnail_key_focus_in (ClutterActor *actor,
                           CdosOverviewThumbnailsBox *self)
{
    CdosOverviewWorkspaceThumbnail *thumbnail = CDOS_OVERVIEW_WORKSPACE_THUMBNAIL (actor);

    if (!thumbnail->hovering)
    {
        thumbnail->hovering = TRUE;
        self->lastHovered = thumbnail;
        cdos_overview_workspace_thumbnail_highlight (thumbnail);
        for (int i = 0; i < self->thumbnails->len; i++)
        {
            CdosOverviewWorkspaceThumbnail *th = self->thumbnails->pdata[i];
            if (th == thumbnail)
            {
                self->kb_thumbnail_index = i;
                break;
            }
        }
    }
}

static void
on_thumbnail_key_focus_out (ClutterActor *actor,
                            CdosOverviewThumbnailsBox *self)
{
    CdosOverviewWorkspaceThumbnail *thumbnail = CDOS_OVERVIEW_WORKSPACE_THUMBNAIL (actor);

    if (thumbnail->hovering)
    {
        thumbnail->hovering = FALSE;
        if (thumbnail->isActive == FALSE)
            cdos_overview_workspace_thumbnail_shade (thumbnail, TRUE);
    }
}

static void
addThumbnails (CdosOverviewThumbnailsBox *self, gint start, gint count)
{
    MetaDisplay *display = cdos_global_get_display (cdos_global_get());
    MetaWorkspaceManager *wsm = meta_display_get_workspace_manager (display);

    gint k;
    for (k = start; k < start + count; k++)
    {
        MetaWorkspace *metaWorkspace = meta_workspace_manager_get_workspace_by_index (wsm, k);
        CdosOverviewWorkspaceThumbnail *thumbnail = cdos_overview_workspace_thumbnail_new (metaWorkspace);

        g_ptr_array_add (self->thumbnails, thumbnail);

        if (metaWorkspace == meta_workspace_manager_get_active_workspace (wsm))
        {
            self->lastActiveWorkspace = thumbnail;
            //cdos_overview_workspace_thumbnail_set_active (thumbnail, TRUE);
        }

        _add_actor (self, thumbnail);
        _add_actor (self, thumbnail->workspace_title_box);
        _add_actor (self, thumbnail->workspace_close_button);

        //        g_signal_connect (thumbnail, "drag-over", G_CALLBACK (onThumbnailDragOver), self);

        // We want to ignore spurious events caused by animations
        // (when the contents are moving and not the pointer).
        //thumbnail->pointerTracker = cdos_pointer_tracker_new ();

        g_signal_connect (thumbnail, "motion-event", G_CALLBACK (onThumbnailMotionEvent), self);
        g_signal_connect (thumbnail, "leave-event", G_CALLBACK (onThumbnailLeaveEvent), self);
        g_signal_connect (thumbnail, "key-focus-in", G_CALLBACK (on_thumbnail_key_focus_in), self);
        g_signal_connect (thumbnail, "key-focus-out", G_CALLBACK (on_thumbnail_key_focus_out), self);
    }
}

static void
onWorkspaceAdded (CdosBaseWM *cdos_base_wm,  gint index, CdosOverviewThumbnailsBox *self)
{
    addThumbnails (self, index, 1);

    GSettings *settings = g_settings_new ("org.cdos");
    gint number_workspaces = g_settings_get_int (settings, "number-workspaces");
    number_workspaces ++;
    g_settings_set_int (settings, "number-workspaces", number_workspaces);
}

static void
onWorkspaceRemoved (CdosBaseWM *wm, gint index, CdosOverviewThumbnailsBox *self)
{
    CdosOverviewWorkspaceThumbnail *thumbnail = self->thumbnails->pdata[index];

    // 处理 tile 和 close-button
    clutter_actor_destroy (thumbnail->workspace_title_box);
    thumbnail->workspace_title_box = NULL;
    clutter_actor_destroy (thumbnail->workspace_close_button);
    thumbnail->workspace_close_button = NULL;

    clutter_actor_destroy (self->thumbnails->pdata[index]);
    g_ptr_array_remove_index (self->thumbnails, index);

    GSettings *settings = g_settings_new ("org.cdos");
    gint number_workspaces = g_settings_get_int (settings, "number-workspaces");
    number_workspaces --;
    g_settings_set_int (settings, "number-workspaces", number_workspaces);
}

static void
thumbnails_box_disconnect_signals (CdosOverviewThumbnailsBox *self)
{
    CdosBaseWM *cdos_base_wm = cdos_global_get_window_manager (cdos_global_get ());
    MetaDisplay *display = cdos_global_get_display (cdos_global_get());
    MetaWorkspaceManager *wsm = meta_display_get_workspace_manager (display);

    g_clear_signal_handler(&self->switchWorkspaceNotifyId, cdos_base_wm);

    g_clear_signal_handler (&self->workspaceAddedId, wsm);
    g_clear_signal_handler (&self->workspaceRemovedId, wsm);
}

static void
cdos_overview_thumbnails_box_dispose (GObject *object)
{
    CdosOverviewThumbnailsBox *self = CDOS_OVERVIEW_THUMBNAILS_BOX (object);
    thumbnails_box_disconnect_signals (self);

    if(self->thumbnails)
    {
        g_ptr_array_foreach (self->thumbnails, (GFunc)clutter_actor_destroy, NULL);
        g_ptr_array_free (self->thumbnails, TRUE);
        self->thumbnails = NULL;
    }

    G_OBJECT_CLASS (cdos_overview_thumbnails_box_parent_class)->dispose (object);
}

static void
cdos_overview_thumbnails_box_finalize (GObject *object)
{
    G_OBJECT_CLASS (cdos_overview_thumbnails_box_parent_class)->finalize (object);
}

static void
cdos_overview_thumbnails_box_allocate (ClutterActor           *self,
                                       const ClutterActorBox  *box,
                                       ClutterAllocationFlags  flags)
{
  StThemeNode *theme_node;
  ClutterActorBox content_box;
  clutter_actor_set_allocation (self, box, flags);

  theme_node = st_widget_get_theme_node (ST_WIDGET (self));
  st_theme_node_get_content_box (theme_node, box, &content_box);

  allocate ((CdosOverviewThumbnailsBox *)self, &content_box, flags);
}

static void
cdos_overview_thumbnails_box_get_preferred_width (ClutterActor *actor,
                                                  gfloat        for_height,
                                                  gfloat       *min_width_p,
                                                  gfloat       *natural_width_p)
{
  StThemeNode *theme_node = st_widget_get_theme_node (ST_WIDGET (actor));

  st_theme_node_adjust_for_height (theme_node, &for_height);

  getPreferredWidth ((CdosOverviewThumbnailsBox *)actor, &for_height, min_width_p, natural_width_p);

  st_theme_node_adjust_preferred_width (theme_node, min_width_p, natural_width_p);
}

static void
cdos_overview_thumbnails_box_get_preferred_height (ClutterActor *actor,
                                              gfloat        for_width,
                                              gfloat       *min_height_p,
                                              gfloat       *natural_height_p)
{
  StThemeNode *theme_node = st_widget_get_theme_node (ST_WIDGET (actor));

  st_theme_node_adjust_for_width (theme_node, &for_width);

  getPreferredHeight ((CdosOverviewThumbnailsBox *)actor, &for_width, min_height_p, natural_height_p);

  st_theme_node_adjust_preferred_height (theme_node, min_height_p, natural_height_p);
}

static void
on_workspace_add_button_clicked (StButton *button, gint clicked_button, CdosOverviewThumbnailsBox *self)
{
    MetaDisplay *display = cdos_global_get_display (cdos_global_get());
    MetaWorkspaceManager *wsm = meta_display_get_workspace_manager (display);

    meta_workspace_manager_append_new_workspace (wsm, FALSE, cdos_global_get_current_time (global));    // 触发信号，去跑 onWorkspaceAdded
}

static void 
cdos_overview_thumbnails_box_add_workspace_button (CdosOverviewThumbnailsBox *self)
{
    ClutterActor *add_workspace_button_icon;
    self->add_workspace_button_box = g_object_new (ST_TYPE_BOX_LAYOUT,  
                                                                    NULL);
    self->add_workspace_button = g_object_new (ST_TYPE_BUTTON,
                                                            "visible", TRUE,
                                                            "x-expand", TRUE,
                                                            "y-expand", TRUE, 
                                                            "x-align", CLUTTER_ACTOR_ALIGN_FILL, 
                                                            "y-align", CLUTTER_ACTOR_ALIGN_FILL, 
                                                            NULL);
    st_widget_add_style_class_name (self->add_workspace_button, "workspace-thumbnail-background");                                                       
    add_workspace_button_icon = g_object_new (ST_TYPE_ICON,
                               "icon-name", "workspace-add-button",
                               NULL);

    st_bin_set_child ((StBin *)self->add_workspace_button, add_workspace_button_icon);
    _add_actor (self->add_workspace_button_box, self->add_workspace_button);
    _add_actor (self, self->add_workspace_button_box);
    g_signal_connect (self->add_workspace_button, "clicked", G_CALLBACK (on_workspace_add_button_clicked), self);


    self->add_workspace_title_box = g_object_new (ST_TYPE_BOX_LAYOUT, 
                                                        NULL);

    gchar * add_workspace_name = g_strdup (_("New Workspace"));
    self->add_workspace_title = g_object_new (ST_TYPE_LABEL,
                                                "text", add_workspace_name,
                                                "visible", TRUE,
                                                "x-expand", TRUE,
                                                "y-expand", TRUE, 
                                                "x-align", CLUTTER_ACTOR_ALIGN_CENTER, 
                                                "y-align", CLUTTER_ACTOR_ALIGN_CENTER, 
                                                NULL);
    st_widget_add_style_class_name (self->add_workspace_title, "window-caption");
    _add_actor (self->add_workspace_title_box, self->add_workspace_title);
    _add_actor (self, self->add_workspace_title_box);
} 

static void
cdos_overview_thumbnails_box_show (ClutterActor *actor)
{
    CdosOverviewThumbnailsBox *self = (gpointer)actor;
    CdosBaseWM *cdos_base_wm = cdos_global_get_window_manager (cdos_global_get ());
    MetaDisplay *display = cdos_global_get_display (cdos_global_get());
    MetaWorkspaceManager *wsm = meta_display_get_workspace_manager (display);
    self->switchWorkspaceNotifyId = g_signal_connect (cdos_base_wm, "switch-workspace", G_CALLBACK (activeWorkspaceChanged), self);
    self->workspaceAddedId = g_signal_connect (wsm, "workspace-added", G_CALLBACK (onWorkspaceAdded), self);
    self->workspaceRemovedId = g_signal_connect (wsm, "workspace-removed", G_CALLBACK (onWorkspaceRemoved), self);

    CLUTTER_ACTOR_CLASS (cdos_overview_thumbnails_box_parent_class)->show (actor);
    addThumbnails (self, 0, meta_workspace_manager_get_n_workspaces (wsm));

    // 增加工作区的按钮
    cdos_overview_thumbnails_box_add_workspace_button (self);
}

/*
static void
cdos_overview_thumbnails_box_hide (ClutterActor *actor)
{
    CdosOverviewThumbnailsBox *self = (gpointer)actor;
    g_return_if_fail(CDOS_IS_OVERVIEW_THUMBNAILS_BOX(self));
    thumbnails_box_disconnect_signals (self);
    GList *iter;
    for (iter = self->thumbnails; iter; iter = iter->next)
    {
        CdosOverviewWorkspaceThumbnail *thumbnail = iter->data;
        //cdos_overview_workspace_thumbnail_destroy (thumbnail_index);
        clutter_actor_destroy ((ClutterActor *)thumbnail);
    }
    g_list_free(self->thumbnails);
    self->thumbnails = NULL;
    CLUTTER_ACTOR_CLASS (cdos_overview_thumbnails_box_parent_class)->hide (actor);
}
*/

gboolean
cdos_overview_thumbnails_box_key_press_event (CdosOverviewThumbnailsBox *self, ClutterEvent *event)
{
    ClutterModifierType modifiers = cdos_base_util_get_event_state (event);

    ClutterModifierType ctrlAltMask = CLUTTER_CONTROL_MASK | CLUTTER_MOD1_MASK;

    guint symbol = clutter_event_get_key_symbol (event);

    if (symbol == CLUTTER_KEY_Return || symbol == CLUTTER_KEY_space || symbol == CLUTTER_KEY_KP_Enter)
    {
        activateSelectedWorkspace (self);
        return TRUE;
    }

    if (modifiers & ctrlAltMask)
        return FALSE;

    return selectNextWorkspace (self, symbol);
}

static void
cdos_overview_thumbnails_box_class_init (CdosOverviewThumbnailsBoxClass *klass)
{
    GObjectClass *gobject_class = G_OBJECT_CLASS (klass);

    gobject_class->finalize = cdos_overview_thumbnails_box_finalize;
    gobject_class->dispose = cdos_overview_thumbnails_box_dispose;

    ClutterActorClass *parent_class = CLUTTER_ACTOR_CLASS (klass);

    parent_class->get_preferred_width = cdos_overview_thumbnails_box_get_preferred_width;
    parent_class->get_preferred_height = cdos_overview_thumbnails_box_get_preferred_height;
    parent_class->allocate = cdos_overview_thumbnails_box_allocate;
    parent_class->show = cdos_overview_thumbnails_box_show;
    //parent_class->hide = cdos_overview_thumbnails_box_hide;
    //parent_class->leave_event = cdos_overview_thumbnails_box_leave;
}

static void
cdos_overview_thumbnails_box_init (CdosOverviewThumbnailsBox *self)
{
    MetaDisplay *display = cdos_global_get_display (cdos_global_get());
    MetaWorkspaceManager *wsm = meta_display_get_workspace_manager (display);

    self->thumbnails = g_ptr_array_new ();

    // The "porthole" is the portion of the screen that we show in the workspaces
    self->porthole.x = 0;
    self->porthole.y = 0;
    meta_display_get_size (display, &self->porthole.width, &self->porthole.height);

    self->kb_thumbnail_index = meta_workspace_manager_get_active_workspace_index (wsm);
}

static void
cdos_overview_thumbnails_box_post_init (CdosOverviewThumbnailsBox *self)
{
    st_widget_add_style_class_name (ST_WIDGET (self), "workspace-thumbnails");
    clutter_actor_set_reactive ((ClutterActor *)self, TRUE);
    clutter_actor_set_request_mode ((ClutterActor *)self, CLUTTER_REQUEST_HEIGHT_FOR_WIDTH /* 横向排列工作区缩略图 */);

    g_signal_connect (self, "scroll-event", G_CALLBACK (onScrollEvent), self);
}

CdosOverviewThumbnailsBox *
cdos_overview_thumbnails_box_new (void)
{
    CdosOverviewThumbnailsBox *instance = g_object_new (CDOS_TYPE_OVERVIEW_THUMBNAILS_BOX, NULL);

    cdos_overview_thumbnails_box_post_init (instance);

    return instance;
}

