#define CLUTTER_DISABLE_DEPRECATION_WARNINGS

#include <math.h>
#include "cdos-overview-window-clone.h"

#include "cdos-global.h"
#include "cdos-window-tracker.h"
#include "cdos-debug.h"
#include "cdos-base-util.h"
#include "cdos-start.h"
#include "cdos-drag-drop.h"

/**
 * CdosFixedLayout
 */
G_DEFINE_TYPE (CdosFixedLayout, cdos_fixed_layout, CLUTTER_TYPE_LAYOUT_MANAGER);

static void
cdos_fixed_layout_get_preferred_width (ClutterLayoutManager *manager,
                                       ClutterContainer *container,
                                       gfloat for_height,
                                       gfloat *min_width_p,
                                       gfloat *nat_width_p)
{
    CdosFixedLayout *self = (CdosFixedLayout *)manager;
    clutter_actor_get_preferred_width (self->primary_actor, for_height, min_width_p, nat_width_p);
}

static void
cdos_fixed_layout_get_preferred_height (ClutterLayoutManager *manager,
                                        ClutterContainer *container,
                                        gfloat for_width,
                                        gfloat *min_height_p,
                                        gfloat *nat_height_p)
{
    CdosFixedLayout *self = (CdosFixedLayout *)manager;
    clutter_actor_get_preferred_height (self->primary_actor, for_width, min_height_p, nat_height_p);
}

static void
cdos_fixed_layout_allocate (ClutterLayoutManager *manager,
                            ClutterContainer *container,
                            const ClutterActorBox *allocation,
                            ClutterAllocationFlags flags)
{
    ClutterActor *child;
    for (child = clutter_actor_get_first_child (CLUTTER_ACTOR (container));
         child;
         child = clutter_actor_get_next_sibling (child))
    {
        clutter_actor_allocate_preferred_size (child, flags);
    }
}
static void
cdos_fixed_layout_init (CdosFixedLayout *self)
{
}

static void
cdos_fixed_layout_class_init (CdosFixedLayoutClass *klass)
{
    ClutterLayoutManagerClass *parent_class = CLUTTER_LAYOUT_MANAGER_CLASS (klass);
    parent_class->get_preferred_width = cdos_fixed_layout_get_preferred_width;
    parent_class->get_preferred_height = cdos_fixed_layout_get_preferred_height;
    parent_class->allocate = cdos_fixed_layout_allocate;
}

ClutterLayoutManager *
cdos_fixed_layout_new (ClutterActor *actor)
{
    CdosFixedLayout *layout = g_object_new (CDOS_TYPE_FIXED_LAYOUT, NULL);
    layout->primary_actor = actor;
    return (ClutterLayoutManager *)layout;
}

/**
 * SECTION:cdos-overview-window-clone
 * @Title: CdosOverviewWindowClone
 * @Short_description: 工作区中打开窗口的克隆窗口
 *
 * CdosOverviewWindowClone 就是每个工作区内，窗口的克隆体。我们通过组合键进入工作区缩略图模式，
 * 除了每个工作区的缩略图，还需要显示这个工作区内所有打开的窗口的缩略图（同 CdosOverview 的工作，不过是缩小版的），
 * 这个对象就是每个窗口的克隆体，由于是克隆体，关闭这个窗口时，需要将这个克隆对象销毁，
 * 更改窗口大小时，需要刷新 clone 的大小坐标鼠标等。
 */
#define ICON_OPACITY 230 /* round(255 * 0.9) */
#define DRAGGING_WINDOW_OPACITY 204 /* round(255 * 0.8) */
#define WINDOW_DND_SIZE 256
#define ICON_SIZE 128

#define DEMANDS_ATTENTION_CLASS_NAME "window-list-item-demands-attention"


/*signal*/

enum
{
    WORKSPACE_CHANGED,
    DRAG_BEGIN,
    DRAG_CANCELLED,
    DRAG_END,
    LAST_SIGNAL
};

static guint cdos_overview_window_clone_signals[LAST_SIGNAL] = {0};

G_DEFINE_TYPE (CdosOverviewWindowClone, cdos_overview_window_clone, CLUTTER_TYPE_ACTOR);

static void
onPositionChanged (MetaWindowActor *rw, gpointer user_data)
{
    CdosOverviewWindowClone *self = CDOS_OVERVIEW_WINDOW_CLONE (user_data);
    gfloat width, height;

    self->origX = clutter_actor_get_x (CLUTTER_ACTOR (self->realWindow));
    self->origY = clutter_actor_get_y (CLUTTER_ACTOR (self->realWindow));

    clutter_actor_set_position ((ClutterActor *)self, self->origX, self->origY);

    width = clutter_actor_get_width (CLUTTER_ACTOR (self->realWindow));
    height = clutter_actor_get_height (CLUTTER_ACTOR (self->realWindow));

    clutter_actor_set_size ((ClutterActor *)self, width, height);
}

static void
onRealWindowDestroyed (GObject *gobject, CdosOverviewWindowClone *self)
{
    clutter_actor_destroy ((ClutterActor *)self);
}

static void
disconnect_signals (CdosOverviewWindowClone *self)
{
    if (self->workspaceChangedId > 0)
    {
        g_signal_handler_disconnect (self->metaWindow, self->workspaceChangedId);
        self->workspaceChangedId = 0L;
    }
    /*
    if(self->sizeChangedId > 0){
        g_signal_handler_disconnect (self->realWindow, self->sizeChangedId);
        self->sizeChangedId = 0L;
    }
    */
    if (self->positionChangedId > 0)
    {
        g_signal_handler_disconnect (self->metaWindow, self->positionChangedId);
        self->positionChangedId = 0L;
    }
    if (self->realWindowDestroyedId > 0)
    {
        g_signal_handler_disconnect (self->realWindow, self->realWindowDestroyedId);
        self->realWindowDestroyedId = 0L;
    }
}

/*
static gboolean
onButtonPressEvent (GObject *gobject, ClutterEvent *event, gpointer user_data)
{
    CdosOverviewWindowClone *self = CDOS_OVERVIEW_WINDOW_CLONE (user_data);
    ClutterActor *actor = CLUTTER_ACTOR (gobject);

    self->lastButtonPressActor = actor;
    self->lastButtonPressTime = clutter_event_get_time (event);
    return TRUE;
}
*/

static void
cdos_overview_window_clone_dispose (GObject *object)
{
    /*
     * dispose() 为什么会被调用两次?
    cdos_debug ("%p", object);
    */
    CdosOverviewWindowClone *self = CDOS_OVERVIEW_WINDOW_CLONE (object);
    disconnect_signals (self);

    G_OBJECT_CLASS (cdos_overview_window_clone_parent_class)->dispose (object);
}

static void
cdos_overview_window_clone_class_init (CdosOverviewWindowCloneClass *klass)
{
    GObjectClass *gobject_class = G_OBJECT_CLASS (klass);

    gobject_class->dispose = cdos_overview_window_clone_dispose;

    cdos_overview_window_clone_signals[WORKSPACE_CHANGED] = g_signal_new ("workspace-changed",
                                                                          G_TYPE_FROM_CLASS (klass),
                                                                          G_SIGNAL_RUN_LAST,
                                                                          0,
                                                                          NULL,
                                                                          NULL,
                                                                          g_cclosure_marshal_VOID__INT,
                                                                          G_TYPE_NONE,
                                                                          1,
                                                                          G_TYPE_INT);
}

/* 实例初始化 */
static void
cdos_overview_window_clone_init (CdosOverviewWindowClone *self)
{
    ClutterLayoutManager *layout_manager = cdos_fixed_layout_new ((ClutterActor *)self);
    clutter_actor_set_layout_manager ((ClutterActor *)self, layout_manager);
    clutter_actor_set_reactive ((ClutterActor *)self, TRUE);
}

static gboolean
foreach_transient (MetaWindow *window, ClutterActor *parent)
{
    ClutterActor *actor = (ClutterActor *)meta_window_get_compositor_private (window);
    if (!actor)
        return FALSE;
    if (!meta_window_is_attached_dialog (window))
        return FALSE;
    ClutterActor *clone = clutter_clone_new (actor);
    clutter_actor_add_child (parent, clone);
    MetaRectangle parent_rect, rect;
    MetaWindow *parent_window = meta_window_actor_get_meta_window ((MetaWindowActor *)parent);
    meta_window_get_frame_rect (parent_window, &parent_rect);
    meta_window_get_frame_rect (window, &rect);
    clutter_actor_set_position (
        clone, (parent_rect.width - rect.width) / 2.0, (parent_rect.height - rect.height) / 2.0);
    meta_window_foreach_transient (window, (MetaWindowForeachFunc)foreach_transient, clone);

    return TRUE;
}

static void
cdos_overview_window_clone_post_init (CdosOverviewWindowClone *self, MetaWindowActor *realWindow)
{
    self->realWindow = realWindow;
    self->metaWindow = meta_window_actor_get_meta_window (self->realWindow);

    if (self->clone)
        clutter_actor_destroy (self->clone);

    self->clone = clutter_clone_new ((ClutterActor *)self->realWindow);
    clutter_actor_add_child ((ClutterActor *)self, self->clone);

    self->positionChangedId =
        g_signal_connect (self->metaWindow, "position-changed", G_CALLBACK (onPositionChanged), self);
    self->realWindowDestroyedId =
        g_signal_connect (self->realWindow, "destroy", G_CALLBACK (onRealWindowDestroyed), self);

    meta_window_foreach_transient (self->metaWindow, (MetaWindowForeachFunc)foreach_transient, self->clone);

    onPositionChanged (self->realWindow, self);
}

/**
 * cdos_overview_window_clone_new:
 * @windowActor: #MetaWindowActor 对象
 *
 * 根据传递过来的 #MetaWindowActor，也就是实际的窗口对象，创建克隆体
 *
 * Return value: #CdosOverviewWindowClone 对象
 */
CdosOverviewWindowClone *
cdos_overview_window_clone_new (MetaWindowActor *windowActor)
{
    CdosOverviewWindowClone *instance = g_object_new (CDOS_TYPE_OVERVIEW_WINDOW_CLONE, NULL);
    cdos_overview_window_clone_post_init (instance, windowActor);
    return instance;
}

/**
 * cdos_overview_window_clone_set_stack_above:
 * @self: #CdosOverviewWindowClone 对象
 * @actor: 需要放在窗口栈顶的 #ClutterActor 对象
 *
 * 将 actor 置顶（同一 parent 的所有 actor 控件）
 */
void
cdos_overview_window_clone_set_stack_above (CdosOverviewWindowClone *self, ClutterActor *actor)
{
    g_return_if_fail (CDOS_IS_OVERVIEW_WINDOW_CLONE (self));
    if (clutter_actor_get_parent (actor) != clutter_actor_get_parent ((ClutterActor *)self))
        return;

    self->stackAbove = actor;

    if (self->stackAbove == NULL)
        //this.actor.lower_bottom();
        clutter_actor_set_child_below_sibling (clutter_actor_get_parent ((ClutterActor *)self), (ClutterActor *)self, NULL);
    else
        //this.actor.raise(this.stackAbove);
        clutter_actor_set_child_above_sibling (clutter_actor_get_parent ((ClutterActor *)self), (ClutterActor *)self, self->stackAbove);
}
