#include "cdos-layout-manager.h"
#include "cdos-start.h"
#include "cdos-panel-item-tooltip.h"

G_DEFINE_TYPE (CdosPanelItemTooltip, cdos_panel_item_tooltip, CDOS_TYPE_TOOLTIP);

static void
panel_item_tooltip_set_position (CdosTooltip *tooltip)
{
    CdosPanelItemTooltip *self = CDOS_PANEL_ITEM_TOOLTIP (tooltip);

    CdosLayoutManager *layout_manager;
    const MetaRectangle *monitor;
    gfloat tooltip_top;
    ClutterActorBox item_box;
    gfloat tooltip_height;
    gfloat tooltip_width;

    CdosPanel *panel = cdos_ui_plugin_get_panel ();
    guint panel_height = cdos_panel_get_height (panel);

    clutter_actor_get_preferred_size ((ClutterActor *)tooltip, NULL, NULL, &tooltip_width, &tooltip_height);

    clutter_actor_get_allocation_box (tooltip->item, &item_box);
    layout_manager = cdos_layout_manager_get_default ();
    monitor = cdos_layout_manager_get_primary_monitor (layout_manager);

    /* FIXME 当 applets 在 group-applet 中时，tooltip 位置不正确 */
    if (self->orientation == ST_SIDE_BOTTOM)
    {
        tooltip_top = (float)(monitor->y + monitor->height - tooltip_height - panel_height);
    } else {
        tooltip_top = (float)(monitor->y + item_box.y2);
    }

    gint tooltip_left = tooltip->mousePositionX - tooltip_width / 2;
    if (tooltip_left < monitor->x)
        tooltip_left = monitor->x;
    if (tooltip_left + tooltip_width > monitor->x + monitor->width)
        tooltip_left = monitor->x + monitor->width - tooltip_width;

    clutter_actor_set_position ((ClutterActor *)tooltip, tooltip_left, tooltip_top);
}

static void
cdos_panel_item_tooltip_class_init (CdosPanelItemTooltipClass *klass)
{
    CdosTooltipClass *parent_class = CDOS_TOOLTIP_CLASS (klass);
    //parent_class->show = panel_item_tooltip_show;
    parent_class->set_position = panel_item_tooltip_set_position;
}

static void
cdos_panel_item_tooltip_init (CdosPanelItemTooltip *self)
{

}

CdosPanelItemTooltip *
cdos_panel_item_tooltip_new (ClutterActor *item, const gchar *init_title, StSide orientation)
{
    CdosPanelItemTooltip *tooltip;
    tooltip = g_object_new (CDOS_TYPE_PANEL_ITEM_TOOLTIP, 
                            "item", item,
                            "text", init_title, 
                            NULL);
    tooltip->orientation = orientation;
    return tooltip;
}
