#ifndef __CDOS_POPUP_BASE_MENU_ITEM_H__
#define __CDOS_POPUP_BASE_MENU_ITEM_H__

#include "st.h"
#include <glib.h>

G_BEGIN_DECLS

#define CDOS_TYPE_POPUP_BASE_MENU_ITEM  (cdos_popup_base_menu_item_get_type())
G_DECLARE_DERIVABLE_TYPE (CdosPopupBaseMenuItem, cdos_popup_base_menu_item,
                          CDOS, POPUP_BASE_MENU_ITEM, StWidget)

struct _CdosPopupBaseMenuItemClass
{
    StWidgetClass parent_class;
    /*signals*/
    void (*activate) (CdosPopupBaseMenuItem * menuitem, gpointer event, gboolean keepMenu);
    void (*active_changed) (CdosPopupBaseMenuItem * menuitem, gboolean active);
    void (*sensitive_changed) (CdosPopupBaseMenuItem * menuitem, gboolean sensitive);
    void (*value_changed) (CdosPopupBaseMenuItem *menuItem, gfloat value);
};


typedef struct  
{
    gboolean     reactive ;
    gboolean     activate;
    gboolean     hover;
    gboolean     sensitive;
    const gchar *style_class;
    gboolean     focusOnHover;
}PopupBaseMenuItemInit;

typedef struct 
{
    gboolean        expand;
    ClutterActorAlign    align;
    ClutterActor   *actor;
}ActorChild;

typedef enum
{
    CDOS_ORNAMENT_DOT,
    CDOS_ORNAMENT_CHECK,
    CDOS_ORNAMENT_CLOSE, /* \u2715 \u2717  */
    CDOS_ORNAMENT_SQUARE, /* \u25A2 white square with rounded corners */
    CDOS_ORNAMENT_UMBRELLA, /* \u2602 */
    CDOS_ORNAMENT_CLOUD, /* \u2601 */
    CDOS_ORNAMENT_SUN, /* \u2600 */
    CDOS_ORNAMENT_SPACE,
    CDOS_ORNAMENT_NONE
}CdosOrnament;

CdosPopupBaseMenuItem *cdos_popup_base_menu_item_new (void);

CdosPopupBaseMenuItem *cdos_popup_base_menu_item_new_with_params (PopupBaseMenuItemInit *params);

void  cdos_popup_base_menu_item_set_params (CdosPopupBaseMenuItem *item, PopupBaseMenuItemInit *params);

void  cdos_popup_base_menu_item_activate (CdosPopupBaseMenuItem *item, ClutterEvent *event, gboolean keepMenu);


void  cdos_popup_base_menu_item_set_sensitive (CdosPopupBaseMenuItem *item, gboolean sensitive);

void cdos_popup_base_menu_item_destroy (CdosPopupBaseMenuItem *item);

void cdos_popup_base_menu_item_add_actor (CdosPopupBaseMenuItem *item, ClutterActor *child, ActorChild *params);

gint cdos_popup_base_menu_item_get_spacing (CdosPopupBaseMenuItem *item);

void cdos_popup_base_menu_item_set_spacing (CdosPopupBaseMenuItem *item, gint spacing);

gboolean cdos_popup_base_menu_item_get_active (CdosPopupBaseMenuItem *item);

void cdos_popup_base_menu_item_set_active (CdosPopupBaseMenuItem *item, gboolean active);

gboolean  cdos_popup_base_menu_item_get_activatable (CdosPopupBaseMenuItem *item);

void cdos_popup_base_menu_item_set_activatable (CdosPopupBaseMenuItem *item, gboolean  activatable);

PopupBaseMenuItemInit *cdos_popup_base_menu_item_get_init_params (PopupBaseMenuItemInit *params);

ActorChild *cdos_popup_base_menu_item_get_actor_child (ActorChild *child);

void cdos_popup_base_menu_item_set_keep_menu (CdosPopupBaseMenuItem *item,gboolean keep_menu);

gboolean cdos_popup_base_menu_item_get_keep_menu (CdosPopupBaseMenuItem *item);

void cdos_popup_base_menu_item_set_ornament (CdosPopupBaseMenuItem *item, CdosOrnament ornament);

G_END_DECLS

#endif
