#ifndef __POPUP_MENU_BASE_H__
#define __POPUP_MENU_BASE_H__

#include <glib.h>
#include <gtk/gtk.h>
#include "st.h"
#include "cdos-global.h"
#include "cdos-box-pointer.h"
#include "cdos-popup-base-menu-item.h"
#include "cdos-popup-menu-item.h"

//extern CdosPopupBaseMenuItem;

GType
cdos_popup_menu_base_get_type (void);

#define CDOS_TYPE_POPUP_MENU_BASE   (cdos_popup_menu_base_get_type ())

#define CDOS_POPUP_MENU_BASE(obj)  (G_TYPE_CHECK_INSTANCE_CAST ((obj), CDOS_TYPE_POPUP_MENU_BASE, CdosPopupMenuBase))

#define CDOS_IS_POPUP_MENU_BASE(obj)  (G_TYPE_CHECK_INSTANCE_TYPE ((obj), CDOS_TYPE_POPUP_MENU_BASE))

#define CDOS_POPUP_MENU_BASE_CLASS(klass)  (G_TYPE_CHECK_CLASS_CAST ((klass), CDOS_TYPE_POPUP_MENU_BASE, CdosPopupMenuBaseClass))

#define CDOS_IS_POPUP_MENU_BASE_CLASS(klass)  (G_TYPE_CHECK_CLASS_TYPE ((klass), CDOS_TYPE_POPUP_MENU_BASE))

#define CDOS_POPUP_MENU_BASE_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), CDOS_TYPE_POPUP_MENU_BASE, CdosPopupMenuBaseClass))

gpointer _obj_get_delegate (GObject *obj);

ClutterActor * _menu_item_get_actor(GObject *obj);

#define menu_item_get_actor(obj) _menu_item_get_actor((GObject *)obj)

#define ADD_MENU_ITEM(menu, item) cdos_popup_menu_base_add_menu_item((CdosPopupMenuBase *)menu, (CdosPopupBaseMenuItem *)item, -1)

typedef struct _CdosPopupMenuBasePrivate CdosPopupMenuBasePrivate;
typedef struct _CdosPopupMenuBase        CdosPopupMenuBase;
typedef struct _CdosPopupMenuBaseClass   CdosPopupMenuBaseClass;

struct _CdosPopupMenuBase
{
    CdosBoxPointer             parent_instance;
    StBoxLayout               *box;
    guint                      length;
    gboolean                   is_open;
    gboolean                   block_source_events;
    gboolean                   pass_events;
    CdosPopupBaseMenuItem     *active_menu_item;
    CdosPopupMenuBasePrivate  *priv;
};

struct _CdosPopupMenuBaseClass
{
    CdosBoxPointerClass parent_class;
    //vfuncs
    ClutterActor *(*get_actor)(CdosPopupMenuBase *obj);
    void (*open) (CdosPopupMenuBase *obj, gboolean animate);
    void (*close) (CdosPopupMenuBase *obj, gboolean animate);
    //signals
    void (*active_changed) (CdosPopupMenuBase *obj);
    void (*child_menu_added) (CdosPopupMenuBase *obj, CdosPopupMenuBase *child_menu);
    void (*child_menu_removed) (CdosPopupMenuBase *obj, CdosPopupMenuBase *child_menu);
    void (*menu_item_add) (CdosPopupMenuBase *obj);
    void (*open_state_changed) (CdosPopupMenuBase *obj, gboolean open);
    void (*destroy) (CdosPopupMenuBase *obj);
};

CdosPopupMenuBase *cdos_popup_menu_base_new (void);

ClutterActor *cdos_popup_menu_base_get_source_actor (CdosPopupMenuBase *self);

void cdos_popup_menu_base_set_source_actor (CdosPopupMenuBase *self,
                                            ClutterActor *actor);

const gchar *
cdos_popup_menu_base_get_style_class (CdosPopupMenuBase *self);

void
cdos_popup_menu_base_set_style_class (CdosPopupMenuBase *self, const gchar *text);

StBoxLayout *
cdos_popup_menu_base_get_box (CdosPopupMenuBase *self);

/**/
CdosPopupMenuItem *
cdos_popup_menu_base_add_action (CdosPopupMenuBase *self, 
                                 const gchar *title,
                                 GCallback callback,
                                 gpointer user_data);

CdosPopupMenuItem *
cdos_popup_menu_base_add_commandline_action (CdosPopupMenuBase *self, 
                                             const gchar *title,
                                             const gchar *cmd);

gboolean cdos_popup_menu_base_is_child_menu (CdosPopupMenuBase *self, CdosPopupMenuBase *menu);

void cdos_popup_menu_base_add_child_menu (CdosPopupMenuBase *self, CdosPopupMenuBase *menu);

void cdos_popup_menu_base_remove_child_menu (CdosPopupMenuBase *self, CdosPopupMenuBase *menu);

void cdos_popup_menu_base_set_vertical_separator (CdosPopupMenuBase *self);

GList *cdos_popup_menu_base_get_menu_items (CdosPopupMenuBase * self);

void cdos_popup_menu_base_add_menu_item (CdosPopupMenuBase *self,
                                         CdosPopupBaseMenuItem *menu_item, 
                                         gint position);

void cdos_popup_menu_base_add_actor (CdosPopupMenuBase *self, ClutterActor *actor);

void cdos_popup_menu_base_remove_all (CdosPopupMenuBase *self);

void cdos_popup_menu_base_toggle (CdosPopupMenuBase *self);

void cdos_popup_menu_base_toggle_with_options (CdosPopupMenuBase *self, gboolean animate, void *onComplete);

void cdos_popup_menu_base_destroy (CdosPopupMenuBase *self);

void cdos_popup_menu_base_remove_menu_item (CdosPopupMenuBase *self,CdosPopupBaseMenuItem *item);
#endif
