#include "cdos-popup-menu-item.h"
#include "st.h"

/**
 * SECTION: cdos-popup-menu-item.c
 * @Title: CdosPopupMenuItem
 * @short_description: CdosPopupMenuItem继承自CdosPopupBaseMenuItem, 此控件为我们默认往item
 *　 里面添加了一个text。
 *
 * CdosPopupMenuItem 为我们提供了一个默认添加label的item。
 */


struct _CdosPopupMenuItemPrivate
{
    StLabel  *label;
};

G_DEFINE_TYPE_WITH_PRIVATE (CdosPopupMenuItem, cdos_popup_menu_item, CDOS_TYPE_POPUP_BASE_MENU_ITEM);

static void
cdos_popup_menu_item_class_init (CdosPopupMenuItemClass *klass)
{
}

static void
cdos_popup_menu_item_init (CdosPopupMenuItem *item)
{
    CdosPopupMenuItemPrivate *priv;
    priv = item->priv = cdos_popup_menu_item_get_instance_private (item);
    priv->label = ST_LABEL (st_label_new (""));
    ActorChild actor;
    actor.align = CLUTTER_ACTOR_ALIGN_START;
    actor.expand = TRUE;
    cdos_popup_base_menu_item_add_actor ((CdosPopupBaseMenuItem *)item, (ClutterActor *)priv->label, &actor);
}

/**
 * cdos_popup_menu_item_new:
 *
 * 新建一个 #CdosPopupMenuItem 类的实例。
 *
 * Returns: 返回一个 #CdosPopupMenuItem 的实例。
 */
CdosPopupMenuItem *
cdos_popup_menu_item_new (void)
{
    return g_object_new (CDOS_TYPE_POPUP_MENU_ITEM, NULL);
}

/**
 * cdos_popup_menu_item_new_with_params:
 * @text: text
 * @params: 初始化默认的值
 *
 * 新建一个 #CdosPopupMenuItem 类的实例。
 *
 * Returns: 返回一个 #CdosPopupMenuItem 的实例。
 */
CdosPopupMenuItem *
cdos_popup_menu_item_new_with_params (const gchar *text,
                                      PopupBaseMenuItemInit *params)
{
    CdosPopupMenuItem *gobject;

    gobject = g_object_new(CDOS_TYPE_POPUP_MENU_ITEM, NULL);

    cdos_popup_base_menu_item_set_params((CdosPopupBaseMenuItem *)gobject, params);

    cdos_popup_menu_item_set_label(gobject, text);

    return gobject;
}

/**
 * cdos_popup_menu_item_set_label:
 * @item: A #CdosPopupMenuItem
 * @label: 需要显示的text
 *
 * 设置item的label显示的文本
 *
 */
void
cdos_popup_menu_item_set_label(CdosPopupMenuItem *item, const gchar *label)
{
    CdosPopupMenuItemPrivate *priv;
    g_return_if_fail (CDOS_IS_POPUP_MENU_ITEM (item));

    priv = item->priv;

    if (priv->label)
    {
        st_label_set_text (priv->label, label);
    }
}

/**
 * cdos_popup_menu_item_get_label:
 * @item: A #CdosPopupMenuItem
 *
 * 获取item的label
 *
 * Returns: A #StLabel
 */
StLabel *
cdos_popup_menu_item_get_label(CdosPopupMenuItem *item)
{
    CdosPopupMenuItemPrivate *priv;
    g_return_val_if_fail(CDOS_IS_POPUP_MENU_ITEM(item), NULL);

    priv = item->priv;
    return priv->label;
}

/**
 * cdos_popup_menu_item_set_label_align:
 * @item: A #Cdos{type}
 * @align: ＃ClutterActorAlign　
 * @margin: margind的大小
 *
 * 设置item的label中文本的对其方式
 *
 */
void
cdos_popup_menu_item_set_label_align(CdosPopupMenuItem *item, StAlign align, guint margin)
{
    CdosPopupMenuItemPrivate *priv;
    g_return_if_fail(CDOS_IS_POPUP_MENU_ITEM(item));
    priv = item->priv;
    if(align == ST_ALIGN_END)
    {
        clutter_actor_set_margin_right(CLUTTER_ACTOR(priv->label), margin);
    }
    else if(align == ST_ALIGN_START)
    {
        clutter_actor_set_margin_left (CLUTTER_ACTOR(priv->label), margin);
    }
    if (align == ST_ALIGN_MIDDLE)
        g_object_set (priv->label, "x-expand", TRUE, "x-align", CLUTTER_ACTOR_ALIGN_CENTER, NULL);
    else if (align == ST_ALIGN_START)
        g_object_set (priv->label, "x-align", CLUTTER_ACTOR_ALIGN_START, NULL);
    else if (align == ST_ALIGN_END)
        g_object_set (priv->label, "x-align", CLUTTER_ACTOR_ALIGN_END, NULL);
}
