/*
 * cdos-popup-menu-manager.c
 *
 *  Created on: 2016年4月1日
 *      Author: wj
 */

#include "cdos-popup-menu-manager.h"
#include "cdos-global.h"
#include "cdos-start.h"
#include "cdos-drag-drop.h"
#include "cdos-popup-menu.h"
#include "cdos-keyboard-manager.h"

static CdosPopupMenuManager *current_active_manager = NULL;
static GList *event_captured_menus = NULL;
static gulong event_capture_id = 0l;

/**
 * SECTION: cdos-popup-menu-manager.c
 * @Title: CdosPopupMenuManager
 * @short_description: 为menu提供一个管理器
 *
 *
 * CdosPopupMenuManager 管理menu
 */

struct _CdosPopupMenuManagerPrivate
{
    ClutterActor       *owner;
    gulong              event_capture_id;
    gulong              enter_event_id;
    gulong              leave_event_id;
    gulong              key_focus_notify_id;
    CdosPopupMenuBase  *active_menu;
    GList              *menus;
    GList              *menu_stack;
    //CdosStageInputMode  pre_grab_input_mode;
    gboolean            grabbed_from_keynav;
    gboolean            did_pop;
    gboolean            block_all_events;

    gboolean (*on_event_capture)(ClutterActor *actor, ClutterEvent *event, gpointer user_data);
};

G_DEFINE_TYPE_WITH_PRIVATE (CdosPopupMenuManager, cdos_popup_menu_manager, G_TYPE_OBJECT);

typedef struct
{
    CdosPopupMenuBase *menu;
    gulong             open_state_change_id;
    gulong             child_menu_added_id;
    gulong             child_menu_removed_id;
    gulong             destroy_id;
    gulong             enter_id;
    gulong             focus_in_id;
}MenuData;

typedef struct
{
    CdosPopupMenuBase    *menu;
    CdosPopupMenuManager *self;
}UserData;

enum
{
    PROP_0,
    PROP_OWNER
};

static void
cdos_popup_menu_manager_finalize (GObject *gobject)
{
    CdosPopupMenuManager *self = CDOS_POPUP_MENU_MANAGER (gobject);
    CdosPopupMenuManagerPrivate *priv = self->priv;
    g_list_free (priv->menu_stack);
    g_list_free (priv->menus);

    G_OBJECT_CLASS (cdos_popup_menu_manager_parent_class)->finalize (gobject);
}

static void
cdos_popup_menu_manager_dispose (GObject *gobject)
{
    CdosPopupMenuManager *self = CDOS_POPUP_MENU_MANAGER (gobject);
    CdosPopupMenuManagerPrivate *priv = self->priv;
    CdosGlobal *global = cdos_global_get ();
    ClutterStage *stage = cdos_global_get_stage (global);
    if (priv->enter_event_id > 0)
    {
        g_signal_handler_disconnect (stage, priv->enter_event_id);
        priv->enter_event_id = 0;
    }
    if (priv->event_capture_id > 0)
    {
        g_signal_handler_disconnect (stage, priv->event_capture_id);
        priv->event_capture_id = 0;
    }
    if (priv->leave_event_id > 0)
    {
        g_signal_handler_disconnect (stage, priv->leave_event_id);
        priv->leave_event_id = 0;
    }
    if (priv->key_focus_notify_id > 0)
    {
        g_signal_handler_disconnect (stage, priv->key_focus_notify_id);
        priv->key_focus_notify_id = 0;
    }

    GList *menus = priv->menus;
    while (menus)
    {
        MenuData *menudata = menus->data;
        menus = menus->next;
        if(menudata)
        {
            cdos_popup_menu_manager_remove_menu (self, menudata->menu);
        }
    }
    //g_object_unref(priv->owner);
    G_OBJECT_CLASS (cdos_popup_menu_manager_parent_class)->dispose (gobject);
}

static void
cdos_popup_menu_manager_set_property (GObject      *gobject,
                                      guint         prop_id,
                                      const GValue *value,
                                      GParamSpec   *pspec)
{
  CdosPopupMenuManager *obj = CDOS_POPUP_MENU_MANAGER (gobject);

  switch (prop_id)
  {
    case PROP_OWNER:
      cdos_popup_menu_manager_set_owner (obj, g_value_get_object (value));
      break;
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
      break;
  }
}

static void
cdos_popup_menu_manager_get_property (GObject    *gobject,
                                      guint       prop_id,
                                      GValue     *value,
                                      GParamSpec *pspec)
{
  CdosPopupMenuManagerPrivate * priv = CDOS_POPUP_MENU_MANAGER (gobject)->priv;
  switch (prop_id)
  {
    case PROP_OWNER:
      g_value_set_object(value, priv->owner);
      break;
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID(gobject, prop_id, pspec);
      break;
  }
}

static gboolean
cdos_popup_menu_manager_active_menu_contains (CdosPopupMenuManager *self,
                                              ClutterActor         *actor)
{
    CdosPopupMenuManagerPrivate * priv = self->priv;

    if (priv->active_menu != NULL)
    {
        if (clutter_actor_contains ((ClutterActor *)priv->active_menu, actor))
            return TRUE;

        ClutterActor *source_actor;
        source_actor = (ClutterActor *)cdos_popup_menu_base_get_source_actor (priv->active_menu);
        return clutter_actor_contains (source_actor, actor);
    }
    return FALSE;
}

static gboolean
cdos_popup_menu_manager_event_is_on_active_menu (CdosPopupMenuManager *self,
                                                 ClutterEvent         *event)
{
    return cdos_popup_menu_manager_active_menu_contains (self, clutter_event_get_source (event));
}

static void
cdos_popup_menu_manager_close_menu(CdosPopupMenuManager *self)
{
    CdosPopupMenuManagerPrivate * priv = self->priv;
    CdosPopupMenuBaseClass *menu_class;

    if (priv->active_menu)
    {
        menu_class = CDOS_POPUP_MENU_BASE_GET_CLASS(priv->active_menu);
        menu_class->close(priv->active_menu, TRUE);
    }
}

static gboolean
cdos_popup_menu_manager_on_event_capture (ClutterActor *actor,
                                          ClutterEvent *event,
                                          gpointer      user_data)
{
    CdosPopupMenuManager *self = CDOS_POPUP_MENU_MANAGER (user_data);
    CdosPopupMenuManagerPrivate * priv = self->priv;
    gboolean active_menu_contains;
    ClutterEventType event_type;

    if (!self->grabbed)
        return FALSE;

    if (priv->active_menu != NULL && priv->active_menu->pass_events)
        return FALSE;

    /*
    if (priv->did_pop)
    {
        priv->did_pop = FALSE;
        return TRUE;
    }
    */

    event_type = clutter_event_type (event);

    if (((event_type == CLUTTER_MOTION || event_type == CLUTTER_BUTTON_RELEASE)
         || (event_type == CLUTTER_TOUCH_UPDATE || event_type == CLUTTER_TOUCH_END))
        && st_scroll_get_scrolling ())
        return FALSE;

    /* ClutterKeyEvent */
    if (event->type == CLUTTER_KEY_PRESS && clutter_event_get_key_symbol (event) == CLUTTER_KEY_Escape)
    {
        //cdos_popup_menu_manager_close_menu (self);
        return FALSE;
    }

    active_menu_contains = cdos_popup_menu_manager_event_is_on_active_menu (self, event);
    if (active_menu_contains)
    {
        return FALSE;
    }
    else if (event_type == CLUTTER_BUTTON_PRESS || event_type == CLUTTER_TOUCH_BEGIN)
    {
        cdos_popup_menu_manager_close_menu (self);
        return TRUE;
    }
    if (!priv->block_all_events)
        return FALSE;
    return TRUE;
}

static gboolean
cdos_popup_menu_managers_on_event_capture (ClutterActor *actor,
                                           ClutterEvent *event,
                                           gpointer      user_data)
{
    CdosKeyboardManager *kb_manager;

    if (cdos_get_current_drag_action ())
        return FALSE;

    kb_manager = cdos_keyboard_manager_get ();
    if (cdos_keyboard_manager_should_take_event (kb_manager, event))
    {
        return FALSE;
    }

    for (GList *iter = event_captured_menus; iter; iter = iter->next)
    {
        return cdos_popup_menu_manager_on_event_capture (actor, event, iter->data);
    }
    return FALSE;
}

static gint
cdos_popup_menu_manager_find_menu (CdosPopupMenuManager *self,
                                   CdosPopupMenuBase    *item)
{
    CdosPopupMenuManagerPrivate * priv = self->priv;
    GList * l;
    MenuData *menudata;
    gint index = -1;
    for (l = priv->menus; l != NULL; l = l->next)
    {
        index++ ;
        menudata = l->data;
        if (menudata !=NULL && item == (CdosPopupMenuBase *)menudata->menu)
            return index;
    }
    return -1;
}

static void
cdos_popup_menu_manager_grab (CdosPopupMenuManager *self)
{
    CdosGlobal *global = cdos_global_get ();
    ClutterStage *stage = cdos_global_get_stage (global);
    if (!cdos_push_modal (self->priv->owner, 0, 0, CDOS_ACTION_MODE_POPUP))
        return;

    if (!event_capture_id)
    {
        event_capture_id = g_signal_connect (stage,
                                             "captured-event",
                                             G_CALLBACK(cdos_popup_menu_managers_on_event_capture),
                                             self);
    }
    event_captured_menus = g_list_prepend (event_captured_menus, self);

    self->grabbed = TRUE;
}

static void
cdos_popup_menu_manager_ungrab (CdosPopupMenuManager *self)
{
    CdosPopupMenuManagerPrivate * priv = self->priv;
    CdosGlobal *global = cdos_global_get ();
    ClutterStage *stage = cdos_global_get_stage (global);

    if(!self->grabbed)
        return;

    // g_signal_handler_disconnect (stage, priv->event_capture_id);
    event_captured_menus = g_list_remove (event_captured_menus, self);
    if (!event_captured_menus)
    {
        if (event_capture_id)
            g_signal_handler_disconnect (stage, event_capture_id);
        event_capture_id = 0;
    }

    if (priv->enter_event_id)
    {
        g_signal_handler_disconnect (stage, priv->enter_event_id);
        priv->enter_event_id = 0;
    }

    if (priv->leave_event_id)
    {
        g_signal_handler_disconnect (stage, priv->leave_event_id);
        priv->leave_event_id = 0;
    }

    if (priv->key_focus_notify_id)
    {
        g_signal_handler_disconnect(stage, priv->key_focus_notify_id);
        priv->key_focus_notify_id = 0;
    }

    self->grabbed = FALSE;

    cdos_pop_modal (self->priv->owner, 0);
}

static void
cdos_popup_menu_manager_on_menu_open_state (CdosPopupMenuBase *menu,
                                            gboolean           open,
                                            CdosPopupMenuManager *self)
{
    CdosPopupMenuManagerPrivate * priv = self->priv;
    CdosGlobal *global = cdos_global_get ();
    ClutterStage *stage = cdos_global_get_stage (global);
    ClutterActor *focus = NULL;
    gboolean had_focus;
    //CdosStageInputMode *stage_input_mode;
    ClutterActor *source_actor;

    source_actor = cdos_popup_menu_base_get_source_actor (menu);
    if (open)
    {
        if (ST_IS_WIDGET (source_actor))
            st_widget_add_style_pseudo_class ((StWidget *)source_actor, "selected");
        if (priv->active_menu
            && cdos_popup_menu_base_is_child_menu (priv->active_menu, menu))
        {
            priv->menu_stack = g_list_append (priv->menu_stack, priv->active_menu);
            clutter_actor_grab_key_focus ((ClutterActor *)menu);
        }
        priv->active_menu = menu;
        current_active_manager = self;
    }
    else
    {
        if (ST_IS_WIDGET (source_actor))
            st_widget_remove_style_pseudo_class ((StWidget *)source_actor, "selected");
        for (GList *it = priv->menus; it; it = it->next)
        {
            CdosPopupMenuBase *child_menu;
            MenuData *data = it->data;
            child_menu = data->menu;
            if (cdos_popup_menu_base_is_child_menu (menu, child_menu))
            {
                cdos_popup_menu_close (child_menu, FALSE);
            }
        }
        if (priv->menu_stack && priv->menu_stack->data)
        {
            priv->active_menu = (CdosPopupMenuBase *)g_list_last (priv->menu_stack)->data;
            priv->menu_stack = g_list_remove (priv->menu_stack, g_list_last (priv->menu_stack)->data);
            if (source_actor)
            {
                clutter_actor_grab_key_focus (source_actor);
                priv->did_pop = TRUE;
            }
        }
    }

    // Check what the focus was before calling pushModal/popModal
    focus = clutter_stage_get_key_focus (stage);
    had_focus = focus && cdos_popup_menu_manager_active_menu_contains (self, focus);
    if (open)
    {
        if (!self->grabbed)
        {
            //priv->pre_grab_input_mode = cdos_global_get_stage_input_mode(global);
            priv->grabbed_from_keynav = had_focus;
            cdos_popup_menu_manager_grab (self);

            if (had_focus)
                clutter_stage_set_key_focus (stage, focus);
            else
            {
                clutter_stage_set_key_focus (stage, (ClutterActor *)menu);
            }
        }
    }
    else if (priv->active_menu == menu)
    {
        if (self->grabbed)
            cdos_popup_menu_manager_ungrab (self);

        priv->active_menu = NULL;
        if (priv->grabbed_from_keynav)
        {
            /*
            if (priv->pre_grab_input_mode
                    == CDOS_STAGE_INPUT_MODE_FOCUSED)
            {
                cdos_global_set_stage_input_mode(global,
                        CDOS_STAGE_INPUT_MODE_FOCUSED);
            }
            */
            if (had_focus && source_actor)
            {
                clutter_actor_grab_key_focus (source_actor);
            }
            else if (focus)
            {
                clutter_actor_grab_key_focus (focus);
            }
        }
    }
}

static void
cdos_popup_menu_manager_on_child_menu_added (CdosPopupMenuBase *menu,
                                             CdosPopupMenuBase *child_menu,
                                             gpointer           data)
{
    CdosPopupMenuManager *self = data;
    cdos_popup_menu_manager_add_menu (self, child_menu, -1);
}

static void
cdos_popup_menu_manager_on_child_menu_removed (CdosPopupMenuBase *menu,
                                               CdosPopupMenuBase *child_menu,
                                               gpointer           data)
{
    CdosPopupMenuManager *self = data;
    cdos_popup_menu_manager_remove_menu (self, child_menu);
}

static void
cdos_popup_menu_manager_on_menu_destroy (CdosPopupMenuBase *menu,
                                         gpointer           data)
{
    CdosPopupMenuManager *self = data;
    CdosPopupMenuManagerPrivate *priv = self->priv;
    MenuData *menudata;
    ClutterActor *source;

    gint position = cdos_popup_menu_manager_find_menu (self, menu);

    if (position == -1)
        return;

    menudata = g_list_nth_data (priv->menus, (guint)position);

    source = cdos_popup_menu_base_get_source_actor (menu);
    g_signal_handlers_disconnect_by_data (menu, source);
    if (menudata->enter_id > 0)
    {
        //g_signal_handler_disconnect (source, menudata->enter_id);
        menudata->enter_id = 0L;
    }
    if (menudata->focus_in_id > 0)
    {
        //g_signal_handler_disconnect (source, menudata->focus_in_id);
        menudata->focus_in_id = 0L;
    }

    priv->menus = g_list_remove (priv->menus, menudata);
    g_free (menudata);
}

static void
cdos_popup_menu_manager_class_init (CdosPopupMenuManagerClass *klass)
{
    GObjectClass * gobject_class = G_OBJECT_CLASS (klass);
    GParamSpec * pspec;

    gobject_class->finalize = cdos_popup_menu_manager_finalize;
    gobject_class->dispose = cdos_popup_menu_manager_dispose;
    gobject_class->set_property = cdos_popup_menu_manager_set_property;
    gobject_class->get_property = cdos_popup_menu_manager_get_property;

    pspec = g_param_spec_object ("owner",
                                 "Owner",
                                 "Owner of the object",
                                 CLUTTER_TYPE_ACTOR,
                                 G_PARAM_READWRITE);

    g_object_class_install_property (gobject_class, PROP_OWNER, pspec);
}

static void
cdos_popup_menu_manager_init (CdosPopupMenuManager *self)
{
    CdosPopupMenuManagerPrivate * priv;
    priv = self->priv = cdos_popup_menu_manager_get_instance_private (self);

    self->grabbed = FALSE;
    priv->event_capture_id = 0;
    priv->enter_event_id = 0;
    priv->leave_event_id = 0;
    priv->key_focus_notify_id = 0;
    priv->active_menu = NULL;
    priv->menus = NULL;
    priv->menu_stack = NULL;
    //priv->pre_grab_input_mode = CDOS_STAGE_INPUT_MODE_NORMAL;
    priv->grabbed_from_keynav = FALSE;
    priv->block_all_events = TRUE;
}

/**
 * cdos_popup_menu_manager_new:
 * @owner: menuManager的拥有者
 *
 * 新建一个 #CdosPopupMenuManager类的实例。
 *
 * Returns: 返回一个 #CdosPopupMenuManager的实例。
 */

CdosPopupMenuManager *
cdos_popup_menu_manager_new (ClutterActor *owner)
{
    CdosPopupMenuManager *object;
    object = g_object_new (CDOS_TYPE_POPUP_MENU_MANAGER, "owner", owner, NULL);
    return object;
}

/**
 * cdos_popup_menu_manager_get_owner:
 * @self: A #CdosPopupMenuManager
 *
 * 获取self的owner成员
 *
 * Returns: ＃ClutterActor　
 */
ClutterActor *
cdos_popup_menu_manager_get_owner (CdosPopupMenuManager *self)
{
    g_return_val_if_fail (CDOS_IS_POPUP_MENU_MANAGER (self), NULL);
    return self->priv->owner;
}

/**
 * cdos_popup_menu_manager_set_owner:
 * @self: A #CdosPopupMenuManager
 * @owner:　CdosPopupMenuManager的拥有对象
 *
 * 设置self的owner属性
 *
 */
void
cdos_popup_menu_manager_set_owner (CdosPopupMenuManager *self, ClutterActor *owner)
{
    g_return_if_fail (CDOS_IS_POPUP_MENU_MANAGER (self));

    CdosPopupMenuManagerPrivate *priv;
    priv = self->priv;
    if (priv->owner)
    {
        //g_object_unref(priv->owner);
    }

    if(owner)
    {
        //g_object_ref (owner);
        priv->owner = owner;
    }
    else
    {
        priv->owner = NULL;
    }
}

/**
 * cdos_popup_menu_manager_add_menu:
 * @self: A #CdosPopupMenuManager
 * @menu: 要添加的menu
 * @position: 添加menu的位置，-1为缺省设置
 *
 * 添加menu到CdosPopupMenuManager中
 *
 */
void
cdos_popup_menu_manager_add_menu (CdosPopupMenuManager *self,
                                  CdosPopupMenuBase    *menu,
                                  gint                  position)
{
    g_return_if_fail (CDOS_IS_POPUP_MENU_MANAGER (self));
    g_return_if_fail (CDOS_IS_POPUP_MENU_BASE (menu));

    CdosPopupMenuManagerPrivate * priv = self->priv;
    ClutterActor *source;


    gint pos = cdos_popup_menu_manager_find_menu (self, menu);
    if (pos >= 0)
        return;

    MenuData *menudata = g_malloc (sizeof(MenuData));
    UserData *user_data = g_malloc (sizeof(UserData));

    user_data->menu = menu;
    user_data->self = self;

    menudata->menu = menu;//g_object_ref(menu);
    menudata->open_state_change_id = g_signal_connect (menu,
                                                       "open-state-changed",
                                                       G_CALLBACK(cdos_popup_menu_manager_on_menu_open_state),
                                                       self);

    menudata->child_menu_added_id = g_signal_connect (menu,
                                                      "child-menu-added",
                                                      G_CALLBACK(cdos_popup_menu_manager_on_child_menu_added),
                                                      self);

    menudata->child_menu_removed_id = g_signal_connect (menu,
                                                        "child-menu-removed",
                                                        G_CALLBACK(cdos_popup_menu_manager_on_child_menu_removed),
                                                        self);

    menudata->destroy_id = g_signal_connect (menu,
                                             "destroy",
                                             G_CALLBACK(cdos_popup_menu_manager_on_menu_destroy),
                                             self);

    menudata->enter_id = 0;
    menudata->focus_in_id = 0;
    source = cdos_popup_menu_base_get_source_actor (menu);
    g_object_set_data (G_OBJECT (menu),"source_user_data", user_data);

    if (NULL != source)
    {
        /*
        menudata->enter_id = g_signal_connect (source,
                                               "enter-event",
                                               G_CALLBACK(source_actor_on_enter_event_callback),
                                               user_data);

        menudata->focus_in_id = g_signal_connect (source,
                                                  "key-focus-in",
                                                  G_CALLBACK(source_actor_on_key_focus_in_callback),
                                                  user_data);
        */
    }

    if (position < 0)
    {
        priv->menus = g_list_append (priv->menus, menudata);
    }
    else
    {
        /*
        MenuData *menudata = g_list_nth_data (priv->menus, position);
        if (menudata)
        {
            cdos_popup_menu_manager_remove_menu (self,menudata->menu);
        }
        */
        //priv->menus = g_list_remove(priv->menus, menudata);
        priv->menus = g_list_insert (priv->menus, menudata, position);
    }
}

/**
 * cdos_popup_menu_manager_remove_menu:
 * @self: A #CdosPopupMenuManager
 * @menu: 要移除的menu
 *
 * 移除CdosPopupMenuManager中的某个menu
 *
 */
void
cdos_popup_menu_manager_remove_menu (CdosPopupMenuManager *self,
                                     CdosPopupMenuBase    *menu)
{
    g_return_if_fail (CDOS_IS_POPUP_MENU_MANAGER (self));
    g_return_if_fail (CDOS_IS_POPUP_MENU_BASE (menu));

    CdosPopupMenuManagerPrivate * priv = self->priv;
    MenuData *menudata;
    UserData *user_data = NULL;
    ClutterActor *source;

    if (menu == priv->active_menu)
        cdos_popup_menu_manager_close_menu (self);

    gint position = cdos_popup_menu_manager_find_menu (self, menu);

    if (position == -1)
        return;

    menudata = g_list_nth_data (priv->menus, (guint)position);
    g_signal_handler_disconnect (menu, menudata->open_state_change_id);
    g_signal_handler_disconnect (menu, menudata->child_menu_added_id);
    g_signal_handler_disconnect (menu, menudata->child_menu_removed_id);
    g_signal_handler_disconnect (menu, menudata->destroy_id);

    source = cdos_popup_menu_base_get_source_actor (menu);
    if (menudata->enter_id > 0)
    {
        g_signal_handler_disconnect (source, menudata->enter_id);
        menudata->enter_id = 0L;
    }
    if (menudata->focus_in_id > 0)
    {
        g_signal_handler_disconnect (source, menudata->focus_in_id);
        menudata->focus_in_id = 0L;
    }

    user_data = g_object_get_data (G_OBJECT (menu), "source_user_data");

    if (NULL != user_data)
    {
        g_free (user_data);
        g_object_set_data (G_OBJECT (menu), "source_user_data", NULL);
    }
    // g_object_unref (menudata->menu);
    priv->menus = g_list_remove (priv->menus, menudata);
    g_free (menudata);
}

void
cdos_popup_menu_manager_block_all_events (CdosPopupMenuManager *self,
                                          gboolean block_all_events)
{
    g_return_if_fail (CDOS_IS_POPUP_MENU_MANAGER (self));
    self->priv->block_all_events = block_all_events;
}

/**
 * cdos_popup_menu_manager_set_event_capture:
 * @manager: A #CdosPopupMenuManager
 * @fun: 回调函数
 *
 * 设置self的on_event_capture。捕捉到事件的回调函数
 *
 */
void
cdos_popup_menu_manager_set_event_capture(CdosPopupMenuManager *manager, 
      gboolean (*fun )(ClutterActor *actor, ClutterEvent *event, gpointer user_data))
{
    g_return_if_fail(CDOS_IS_POPUP_MENU_MANAGER(manager));
	  CdosPopupMenuManagerPrivate * priv = manager->priv;
    if (priv->on_event_capture == fun)
    { 
      return ;
    }
    cdos_popup_menu_manager_ungrab (manager);
	  if(fun)
		  priv->on_event_capture = fun;
	  else
		  priv->on_event_capture = cdos_popup_menu_manager_on_event_capture;
    cdos_popup_menu_manager_grab (manager);  
}

