#include "cdos-popup-menu-section.h"

/**
 * SECTION: cdos-popup-menu-section.c
 * @Title: CdosPopupMenuSection
 * @short_description: 提供一个section(部分，将几个menu组合成一个section)
 *　
 *
 * CdosPopupMenuSection 提供一个可以分区域的menu
 */

struct _CdosPopupMenuSectionPrivate
{
    gchar *text;
};

G_DEFINE_TYPE_WITH_PRIVATE (CdosPopupMenuSection, cdos_popup_menu_section,CDOS_TYPE_POPUP_MENU_BASE);

enum
{
    PROP_0,
    PROP_TEXT
};

static void
cdos_popup_menu_section_finalize (GObject * gobject)
{
    CdosPopupMenuSection *section = CDOS_POPUP_MENU_SECTION (gobject);
    CdosPopupMenuSectionPrivate * priv = section->priv;

    if (priv->text)
    {
        g_free (priv->text);
    }
    G_OBJECT_CLASS (cdos_popup_menu_section_parent_class)->finalize (gobject);
}

static void
cdos_popup_menu_section_dispose (GObject * gobject)
{
    //CdosPopupMenuSection *section = CDOS_POPUP_MENU_SECTION (gobject);
    //CdosPopupMenuSectionPrivate * priv = section->priv;

    G_OBJECT_CLASS (cdos_popup_menu_section_parent_class)->dispose (gobject);
}

static void
cdos_popup_menu_section_set_property (GObject       *gobject,
                                      guint          prop_id,
                                              const GValue  *value,
                                      GParamSpec    *pspec)
{
    CdosPopupMenuSection * newobj = CDOS_POPUP_MENU_SECTION (gobject);

    switch (prop_id)

    {
        case PROP_TEXT:
            cdos_popup_menu_section_set_text (newobj, g_value_get_string (value));
            break;
        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
            break;
    }
}

static void
cdos_popup_menu_section_get_property (GObject    *gobject,
                                      guint       prop_id,
                                              GValue     *value,
                                      GParamSpec *pspec)
{
    CdosPopupMenuSectionPrivate *priv = CDOS_POPUP_MENU_SECTION (gobject)->priv;

    switch (prop_id)
    {
        case PROP_TEXT:
            g_value_set_string (value, priv->text);
            break;
        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
            break;
    }
}

static ClutterActor *
cdos_popup_menu_section_get_actor (CdosPopupMenuBase *self)
{
    return CLUTTER_ACTOR (self);
}

static void
cdos_popup_menu_section_class_init (CdosPopupMenuSectionClass *klass)
{
    CdosPopupMenuBaseClass *parent_class = CDOS_POPUP_MENU_BASE_CLASS (klass);
    GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
    GParamSpec *pspec;
    parent_class->open = cdos_popup_menu_section_open;
    parent_class->close = cdos_popup_menu_section_close;
    parent_class->get_actor = cdos_popup_menu_section_get_actor;
    gobject_class->finalize = cdos_popup_menu_section_finalize;
    gobject_class->dispose = cdos_popup_menu_section_dispose;
    gobject_class->set_property = cdos_popup_menu_section_set_property;
    gobject_class->get_property = cdos_popup_menu_section_get_property;

    pspec = g_param_spec_string ("text",
                                 "Text",
                                 "Text of the object",
                                 NULL,
                                 G_PARAM_READWRITE);

    g_object_class_install_property (gobject_class, PROP_TEXT, pspec);
}

static void
cdos_popup_menu_section_init (CdosPopupMenuSection *self)
{
    self->priv = cdos_popup_menu_section_get_instance_private (self);
    CDOS_POPUP_MENU_BASE (self)->is_open = FALSE;
}

/**
 * cdos_popup_menu_section_set_text:
 * @self: A #CdosPopupMenuSection
 * @text: 设置text
 *
 * 设置self的text
 *
 */
void
cdos_popup_menu_section_set_text (CdosPopupMenuSection *self, const gchar *text)
{
    CdosPopupMenuSectionPrivate * priv;
    g_return_if_fail (CDOS_IS_POPUP_MENU_SECTION (self));
    priv = self->priv;

    if (priv->text)
    {
        g_free (priv->text);
    }

    if (text)
        priv->text = g_strdup (text);
    else
        priv->text = NULL;
}

/**
 * cdos_popup_menu_section_get_text:
 * @self: A #CdosPopupMenuSection
 *
 * 获取self的text属性
 *
 * Returns: gchar*
 */
const gchar *
cdos_popup_menu_section_get_text (CdosPopupMenuSection * self)
{
    g_return_val_if_fail (CDOS_IS_POPUP_MENU_SECTION (self), NULL);
    return self->priv->text;
}

/**
 * cdos_popup_menu_section_new:
 *
 * 新建一个 #CdosPopupMenuSection 类的实例。
 *
 * Returns: 返回一个 #CdosPopupMenuSection 的实例。
 */
CdosPopupMenuSection *
cdos_popup_menu_section_new (void)
{
    return g_object_new (CDOS_TYPE_POPUP_MENU_SECTION, NULL);
}

/**
 * cdos_popup_menu_section_open:
 * @self: A #CdosPopupMenuSection
 * @animate: 是否开启过渡动画
 *
 * 打开CdosPopupMenuSection
 *
 */
void
cdos_popup_menu_section_open (CdosPopupMenuBase *self, gboolean animate)
{
  /*do nothing*/
}

/**
 * cdos_popup_menu_section_close:
 * @self: A #CdosPopupMenuSection
 * @animate: 是否开启过渡动画
 *
 * 关闭CdosPopupMenuSection
 *
 */
void
cdos_popup_menu_section_close (CdosPopupMenuBase *self, gboolean animate)
{
  /*do nothing*/
}
