#include "st.h"
#include "cdos-popup-separator-menu-item.h"

/**
 * SECTION: cdos-popup-separator-menu-item.c
 * @Title: CdosPopupSeparatorMenuItem
 * @short_description: 继承自 #CdosPopupBaseMenuItem, 这个控件里是一根线，
 * 你可以选择线的方向当在一个menu中添加多个item时，而且item是不同的类型，
 * 就可以使用这个控件。
 *
 * #CdosPopupSeparatorMenuItem 为我们提供了一个有一条线的item
 */
typedef struct _CdosPopupSeparatorMenuItem
{
    CdosPopupBaseMenuItem parent;
    StWidget *separator;
}CdosPopupSeparatorMenuItem;

G_DEFINE_TYPE (CdosPopupSeparatorMenuItem, cdos_popup_separator_menu_item, CDOS_TYPE_POPUP_BASE_MENU_ITEM);

static void
cdos_popup_separator_menu_item_init (CdosPopupSeparatorMenuItem *item)
{
    ActorChild child;
    PopupBaseMenuItemInit  params;
    StWidget *label;

    ClutterActor *actor = (ClutterActor *)item;
    clutter_actor_set_reactive (actor, FALSE);
    clutter_actor_set_x_expand (actor, TRUE);
    st_widget_set_can_focus ((StWidget *)actor, FALSE);

    cdos_popup_base_menu_item_get_init_params (&params);
    params.reactive = FALSE;
    cdos_popup_base_menu_item_set_params (CDOS_POPUP_BASE_MENU_ITEM (item), &params);

    st_widget_set_style_class_name (ST_WIDGET (item), "popup-separator-menu-item");

    label = st_label_new ("");
    child.expand = FALSE;
    cdos_popup_base_menu_item_add_actor (CDOS_POPUP_BASE_MENU_ITEM (item),
                                         (ClutterActor *)label,
                                         &child);

    StWidget *separator;
    child.expand = TRUE;
    child.align = CLUTTER_ACTOR_ALIGN_CENTER;
    separator = g_object_new (ST_TYPE_WIDGET,
                              "style-class", "popup-separator-menu-item-separator",
                              "x-expand", TRUE,
                              "y-expand", FALSE,
                              NULL);
    cdos_popup_base_menu_item_add_actor (CDOS_POPUP_BASE_MENU_ITEM (item),
                                         (ClutterActor *)separator,
                                         &child);
    item->separator = separator;
}

static void
cdos_popup_separator_menu_item_class_init (CdosPopupSeparatorMenuItemClass *klass)
{
}

/**
 * cdos_popup_separator_menu_item_new:
 *
 * 新建一个 #CdosPopupSeparatorMenuItem 类的实例。
 *
 * Returns: 返回一个 #CdosPopupSeparatorMenuItem 的实例。
 */
CdosPopupSeparatorMenuItem *
cdos_popup_separator_menu_item_new (void)
{
    CdosPopupSeparatorMenuItem *gobject;

    gobject = g_object_new (CDOS_TYPE_POPUP_SEPARATOR_MENU_ITEM, NULL);

    return gobject;
}

StWidget *
cdos_popup_separator_menu_item_get_separator (CdosPopupSeparatorMenuItem *self)
{
    g_return_val_if_fail (CDOS_IS_POPUP_SEPARATOR_MENU_ITEM (self), NULL);

    return self->separator;
}
