 #include "cdos-popup-submenu-menu-item.h"
#include "cdos-popup-sub-menu.h"
#include <gtk/gtk.h>
#include "cdos-start.h"

/**
 * SECTION: cdos-popup-submenu-menu-item.c
 * @Title: CdosPopupSubmenuMenuItem
 * @short_description:　具备多个子菜单项的item
 *　
 *
 * CdosPopupSubmenuMenuItem  具备多个子菜单项的item
 */


enum {
    PROP_0,
    PROP_LABEL,
    PROP_HIDE_EXPANDER
};

struct  _CdosPopupSubmenuMenuItemPrivate
{
    ClutterActor   *triangle;
    gboolean        hide_expander;
    StLabel        *label;
};

G_DEFINE_TYPE_WITH_PRIVATE (CdosPopupSubmenuMenuItem, cdos_popup_submenu_menu_item, CDOS_TYPE_POPUP_BASE_MENU_ITEM);

static void
cdos_popup_submenu_menu_item_open_state_changed(CdosPopupMenuBase        *menu,
                                                gboolean                  open,
                                                CdosPopupSubmenuMenuItem *item)
{
    CdosPopupBaseMenuItem *self = (void *)item;
    StWidget *actor = (StWidget *)self;
    if (open)
        st_widget_add_style_pseudo_class (actor, "open");
    else
        st_widget_remove_style_pseudo_class (actor, "open");
}

static gboolean
cdos_popup_submenu_menu_item_on_key_press_event(ClutterActor          *actor,
                                                ClutterKeyEvent          *event);

static void
cdos_popup_submenu_menu_item_dispose(GObject *gobject)
{
    CdosPopupSubmenuMenuItem *self = CDOS_POPUP_SUBMENU_MENU_ITEM(gobject);

    if (self->menu)
    {
        clutter_actor_destroy ((ClutterActor *)self->menu);
        self->menu = NULL;
    }

    G_OBJECT_CLASS(cdos_popup_submenu_menu_item_parent_class)->dispose (gobject);
}

static void
cdos_popup_submenu_menu_item_finalize(GObject *gobject)
{
    //CdosPopupSubmenuMenuItem *self = CDOS_POPUP_SUBMENU_MENU_ITEM(gobject);
    //CdosPopupSubmenuMenuItemPrivate         *priv = self->priv;

    G_OBJECT_CLASS(cdos_popup_submenu_menu_item_parent_class)->finalize (gobject);
}

static void
cdos_popup_submenu_menu_item_init (CdosPopupSubmenuMenuItem *item)
{
    CdosPopupSubmenuMenuItemPrivate  *priv;
    ClutterActor *actor;

    priv = item->priv = cdos_popup_submenu_menu_item_get_instance_private (item);

    actor = (ClutterActor *)item;

    st_widget_add_style_class_name (ST_WIDGET(actor), "popup-submenu-menu-item");

    priv->label = (StLabel *)st_label_new(NULL);

    clutter_actor_add_child (actor, (ClutterActor *)priv->label);
}

static void
cdos_popup_submenu_menu_item_activate (CdosPopupBaseMenuItem *item,
                                       gpointer               event,
                                       gboolean               keep_menu)
{
    CdosPopupSubmenuMenuItem *self = CDOS_POPUP_SUBMENU_MENU_ITEM (item);
    //CdosPopupSubmenuMenuItemPrivate *priv = self->priv;
    //cdos_popup_sub_menu_open((CdosPopupMenuBase *)priv->menu, TRUE);
    cdos_popup_menu_base_toggle ((CdosPopupMenuBase *)self->menu);
}

static void
set_hide_expander (CdosPopupSubmenuMenuItem *item, gboolean hide_expander)
{
    CdosPopupSubmenuMenuItemPrivate *priv = item->priv;

    if (!hide_expander)
    {
        if (!priv->triangle)
        {
            priv->triangle = g_object_new (ST_TYPE_ICON,
                                           "icon-name", "media-playback-start",
                                           "style-class", "popup-menu-icon",
                                           NULL);
        }
        clutter_actor_insert_child_above ((ClutterActor *)item, priv->triangle, NULL);
        clutter_actor_set_x_align (priv->triangle, CLUTTER_ACTOR_ALIGN_END);
        clutter_actor_set_x_expand ((ClutterActor *)priv->label, TRUE);
    }
    else
    {
        if (priv->triangle)
        {
            clutter_actor_destroy (priv->triangle);
            priv->triangle = NULL;
        }
    }
}

static void
set_property (GObject *gobject,
              guint prop_id,
              const GValue *value,
              GParamSpec *pspec)
{
    CdosPopupSubmenuMenuItem *item = CDOS_POPUP_SUBMENU_MENU_ITEM (gobject);
    CdosPopupSubmenuMenuItemPrivate *priv = item->priv;
    switch (prop_id)
    {
        case PROP_LABEL:
          {
            const char *text = g_value_get_string (value);
            st_label_set_text (priv->label, text);
            break;
          }
        case PROP_HIDE_EXPANDER:
            priv->hide_expander = g_value_get_boolean (value);
            set_hide_expander (item, priv->hide_expander);
            break;
        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
            break;
    }
}

static void
get_property (GObject *gobject,
              guint prop_id,
              GValue *value,
              GParamSpec *pspec)
{
    CdosPopupSubmenuMenuItem *item = CDOS_POPUP_SUBMENU_MENU_ITEM (gobject);
    CdosPopupSubmenuMenuItemPrivate *priv = item->priv;
    switch (prop_id)
    {
        case PROP_LABEL:
            g_value_set_string (value, st_label_get_text (priv->label));
            break;
        case PROP_HIDE_EXPANDER:
            g_value_set_boolean (value, priv->hide_expander);
            break;
        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
            break;
    }
}

static void
cdos_popup_submenu_menu_item_class_init(CdosPopupSubmenuMenuItemClass *klass)
{
    GParamSpec     *pspec;
    GObjectClass     *gobject_class = G_OBJECT_CLASS(klass);
    gobject_class->set_property = set_property;
    gobject_class->get_property = get_property;

    gobject_class->finalize = cdos_popup_submenu_menu_item_finalize;
    gobject_class->dispose = cdos_popup_submenu_menu_item_dispose;

    CdosPopupBaseMenuItemClass *base_class = CDOS_POPUP_BASE_MENU_ITEM_CLASS (klass);
    base_class->activate = cdos_popup_submenu_menu_item_activate;

    ClutterActorClass *actor_class = CLUTTER_ACTOR_CLASS (klass);
    actor_class->key_press_event = cdos_popup_submenu_menu_item_on_key_press_event;

    pspec = g_param_spec_string ("label",
                                 "menu label",
                                 "menu lable text",
                                 NULL,
                                 G_PARAM_READWRITE);
    g_object_class_install_property (gobject_class, PROP_LABEL, pspec);

    pspec = g_param_spec_boolean ("hide-expander",
                                  "Hide Expander",
                                  "Whether show triangle",
                                  FALSE,
                                  G_PARAM_READWRITE);
    g_object_class_install_property (gobject_class, PROP_HIDE_EXPANDER, pspec);

}

/**
 * cdos_popup_submenu_menu_item_new:
 * @text: 显示的text
 * @hide_expander: ＃gboolean
 *
 * 新建一个 #CdosPopupSubmenuMenuItem 类的实例。
 *
 * Returns: 返回一个 #CdosPopupSubmenuMenuItem 的实例。
 */
CdosPopupSubmenuMenuItem *
cdos_popup_submenu_menu_item_new (const gchar *text,
                                  gboolean hide_expander)
{
    CdosPopupSubmenuMenuItem *object;
    object = g_object_new (CDOS_TYPE_POPUP_SUBMENU_MENU_ITEM,
                           "label", text,
                           "hide-expander", hide_expander,
                           NULL);

    PopupBaseMenuItemInit params;
    cdos_popup_base_menu_item_get_init_params(&params);
    cdos_popup_base_menu_item_set_params((CdosPopupBaseMenuItem *)object, &params);

    //cdos_popup_submenu_menu_item_set_text(object, text, hide_expander);

    CdosPopupSubmenuMenuItemPrivate *priv = object->priv;
    ClutterActor *actor = CLUTTER_ACTOR (object);
    object->menu = cdos_popup_sub_menu_new (actor, priv->triangle);

    g_signal_connect (object->menu, "open-state-changed",
                      G_CALLBACK(cdos_popup_submenu_menu_item_open_state_changed),
                      object);

    return object;
}
/*
static void
cdos_popup_submenu_menu_item_set_text(CdosPopupSubmenuMenuItem *item,
                      const  gchar *text,
                      gboolean     hide_expander)
{

    CdosPopupSubmenuMenuItemPrivate      *priv;
    priv = item->priv;

    if (hide_expander != priv->hide_expander)
    {
        priv->hide_expander = hide_expander;
        set_hide_expander (item, priv->hide_expander);
    }
    st_label_set_text (priv->label, text);
}
*/

static gboolean
cdos_popup_submenu_menu_item_on_key_press_event (ClutterActor *actor,
                                                 ClutterKeyEvent *event)
{
    CdosPopupSubmenuMenuItem *self = CDOS_POPUP_SUBMENU_MENU_ITEM (actor);
    guint symbol;

    symbol = clutter_event_get_key_symbol ((ClutterEvent *)event);
    if (self->menu)
    {
        if (symbol == CLUTTER_KEY_Right)
        {
            cdos_popup_sub_menu_open (CDOS_POPUP_MENU_BASE (self->menu), TRUE);
            st_widget_navigate_focus (ST_WIDGET (self->menu), NULL, GTK_DIR_DOWN, FALSE);
            return TRUE;
        }
        else if (symbol == CLUTTER_KEY_Left && CDOS_POPUP_MENU_BASE(self->menu)->is_open)
        {
            cdos_popup_sub_menu_close (CDOS_POPUP_MENU_BASE (self->menu), FALSE);
            return TRUE;
        }
    }
    return CLUTTER_ACTOR_CLASS (cdos_popup_submenu_menu_item_parent_class)->key_press_event (actor, event);
}

/**
 * cdos_popup_submenu_menu_item_get_menu:
 * @item: A #CdosPopupSubmenuMenuItem
 *
 * 获取item的menu成员
 *
 * Returns: #CdosPopupSubMenu
 */
CdosPopupSubMenu *
cdos_popup_submenu_menu_item_get_menu (CdosPopupSubmenuMenuItem *item)
{
    g_return_val_if_fail (CDOS_IS_POPUP_SUBMENU_MENU_ITEM (item), NULL);
    return item->menu;
}
