#include "cdos-popup-switch-menu-item.h"
#include "cdos-switchs.h"

/**
 * SECTION: cdos-popup-switch-menu-item.c
 * @Title: CdosPopupSwitchMenuItem
 * @short_description: 继承自CdosPopupBaseMenuItem, 这个控件提供了将一个
 *　　开关按钮放入一个item中。如果你需要在菜单中添加一个开关按钮，那么这个就是为你准备的。
 *
 * CdosPopupSwitchMenuItem  为你提供一个具备开关按钮的item。
 */

struct _CdosPopupSwitchMenuItemPrivate
{
    StLabel       *label;
    CdosSwitchs   *m_switch;
    StBin         *bin_status;
    //StLabel       *label_status;
};

G_DEFINE_TYPE_WITH_PRIVATE (CdosPopupSwitchMenuItem, cdos_popup_switch_menu_item, CDOS_TYPE_POPUP_BASE_MENU_ITEM);

enum
{
    PROP_0,
    PROP_STATE
};

enum
{
    TOGGLED,
    LAST_SIGNAL
};

static guint cdos_popup_switch_menu_item_signals[LAST_SIGNAL] =  { 0, };

static void
cdos_popup_switch_menu_item_set_property (GObject *gobject,
                                          guint prop_id,
                                          const GValue *value,
                                          GParamSpec *pspec)
{
    CdosPopupSwitchMenuItem *self = CDOS_POPUP_SWITCH_MENU_ITEM (gobject);
    switch (prop_id)
    {
        case PROP_STATE:
            cdos_switchs_set_toggle_state (self->priv->m_switch, g_value_get_boolean (value));
            break;
        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID (self, prop_id, pspec);
    }
}

static void
cdos_popup_switch_menu_item_get_property (GObject *gobject,
                                          guint prop_id,
                                          GValue *value,
                                          GParamSpec *pspec)
{
    CdosPopupSwitchMenuItem *self = CDOS_POPUP_SWITCH_MENU_ITEM (gobject);

    switch (prop_id)
    {
        case PROP_STATE:
            g_value_set_boolean (value, cdos_switchs_get_state (self->priv->m_switch));
            break;
        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
    }
}

/**
 * cdos_popup_switch_menu_item_new:
 *
 * 新建一个 #CdosPopupSwitchMenuItem 类的实例。
 *
 * Returns: 返回一个 #CdosPopupSwitchMenuItem 的实例。
 */
CdosPopupSwitchMenuItem *
cdos_popup_switch_menu_item_new(void)
{
    return g_object_new(CDOS_TYPE_POPUP_SWITCH_MENU_ITEM, NULL);
}

/**
 * cdos_popup_switch_menu_item_new_with_params:
 *
 * 新建一个 #CdosPopupSwitchMenuItem 类的实例。
 *
 * Returns: 返回一个 #CdosPopupSwitchMenuItem 的实例。
 */
CdosPopupSwitchMenuItem *
cdos_popup_switch_menu_item_new_with_params (const gchar *text,
                                             gboolean active,
                                             PopupBaseMenuItemInit *params)
{
    CdosPopupSwitchMenuItem *gobject;
    gobject = g_object_new (CDOS_TYPE_POPUP_SWITCH_MENU_ITEM, NULL);
    cdos_popup_base_menu_item_set_params (CDOS_POPUP_BASE_MENU_ITEM(gobject), params);

    cdos_popup_switch_menu_item_set_text (CDOS_POPUP_SWITCH_MENU_ITEM(gobject), text);
    cdos_popup_switch_menu_item_set_active (CDOS_POPUP_SWITCH_MENU_ITEM(gobject), active);

    return gobject;
}

static void
cdos_popup_switch_menu_item_class_init (CdosPopupSwitchMenuItemClass *klass)
{
    GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
    gobject_class->set_property = cdos_popup_switch_menu_item_set_property;
    gobject_class->get_property = cdos_popup_switch_menu_item_get_property;

    GParamSpec *pspec;
    pspec = g_param_spec_boolean ("state", "State", "The state of the item", FALSE, G_PARAM_READWRITE);
    g_object_class_install_property (gobject_class, PROP_STATE, pspec);

    cdos_popup_switch_menu_item_signals[TOGGLED] = g_signal_new (
            "toggled",
            G_TYPE_FROM_CLASS(klass),
            G_SIGNAL_RUN_LAST,
            G_STRUCT_OFFSET(CdosPopupSwitchMenuItemClass, toggled),
            NULL,
            NULL,
            g_cclosure_marshal_VOID__BOOLEAN,
            G_TYPE_NONE,
               1,
            G_TYPE_BOOLEAN);
}


static void
cdos_popup_switch_menu_item_init (CdosPopupSwitchMenuItem *item)
{
    ActorChild  child;

    item->priv = cdos_popup_switch_menu_item_get_instance_private (item);

    item->priv->label = ST_LABEL(st_label_new(NULL));
    item->priv->m_switch = cdos_switchs_new (0);
    cdos_popup_base_menu_item_set_keep_menu (CDOS_POPUP_BASE_MENU_ITEM(item), TRUE);
    child.expand = FALSE;
    child.align = CLUTTER_ACTOR_ALIGN_START;

    item->priv->bin_status = ST_BIN (st_bin_new ());

    cdos_popup_base_menu_item_add_actor (CDOS_POPUP_BASE_MENU_ITEM (item),
                                         CLUTTER_ACTOR (item->priv->bin_status), &child);

    child.expand = TRUE;
    child.align = CLUTTER_ACTOR_ALIGN_START;
    cdos_popup_base_menu_item_add_actor (CDOS_POPUP_BASE_MENU_ITEM (item),
                                         CLUTTER_ACTOR (item->priv->label), &child);
    /*attention error*/
    /*
    item->priv->label_status = g_object_new(ST_TYPE_LABEL, "text", " ",
                    "style-class", "popup-inactive-menu-item",
                    NULL);

    clutter_container_add_actor(CLUTTER_CONTAINER(item->priv->bin_status),
                CLUTTER_ACTOR(item->priv->m_switch));
    clutter_container_add_actor(CLUTTER_CONTAINER(item->priv->bin_status),
                CLUTTER_ACTOR(item->priv->label));
    */
    st_bin_set_child (item->priv->bin_status, CLUTTER_ACTOR (item->priv->m_switch));

    g_signal_connect(item, "activate", G_CALLBACK(cdos_popup_switch_menu_item_activate), item);

}
/**
 * cdos_popup_switch_menu_item_set_status:
 * @item: A #CdosPopupSwitchMenuItem
 * @text: 显示的text
 *
 * 设置item显示的text
 *
 */
/*
void
cdos_popup_switch_menu_item_set_status(CdosPopupSwitchMenuItem  *item,
                       const char               *text)
{
    CdosPopupSwitchMenuItemPrivate             *priv;
    StLabel                        *label_status;
    StBoxLayout                    *bin_status;
    CdosSwitchs                     *m_switch;

    g_return_if_fail(CDOS_IS_POPUP_SWITCH_MENU_ITEM(item));
    priv = item->priv;
    label_status = priv->label_status;
    bin_status = priv->bin_status;
    m_switch = priv->m_switch;
    if (text){
        g_object_set(label_status, "text", text, NULL);
        g_object_set(bin_status, "child", label_status, NULL);
        g_object_set(item, "reactive", FALSE, NULL);
        g_object_set(item, "can-focus", FALSE, NULL);

    }else{
        g_object_set(bin_status, "child", cdos_switchs_get_actor(m_switch), NULL);
        g_object_set(item, "reactive", TRUE, NULL);
        g_object_set(item, "can-focus", TRUE, NULL);
    }
}
*/

/**
 * cdos_popup_switch_menu_item_activate:
 * @item: A #CdosPopupSwitchMenuItem
 * @status: CdosPopupSwitchMenuItem的状态
 *
 * 设置CdosPopupSwitchMenuItem的activate
 *
 */
void
cdos_popup_switch_menu_item_activate(CdosPopupSwitchMenuItem *item, ClutterEvent *event,gboolean keep_menu)
{
    g_return_if_fail(CDOS_IS_POPUP_SWITCH_MENU_ITEM(item));
    CdosPopupSwitchMenuItemPrivate *priv;
    CdosSwitchs *m_switch;
    priv = item->priv;
    m_switch = priv->m_switch;

    if (clutter_actor_is_mapped (CLUTTER_ACTOR (m_switch)))
        cdos_popup_switch_menu_item_toggle(item);
}

/**
 * cdos_popup_switch_menu_item_toggle:
 * @item: A #CdosPopupSwitchMenuItem
 * @status: CdosPopupSwitchMenuItem的状态
 *
 * 设置CdosPopupSwitchMenuItem的状态
 *
 */
void
cdos_popup_switch_menu_item_toggle(CdosPopupSwitchMenuItem *item)
{
    CdosSwitchs *m_switch;
    CdosPopupSwitchMenuItemPrivate         *priv;
    gboolean                 state;
    g_return_if_fail(CDOS_IS_POPUP_SWITCH_MENU_ITEM(item));

    priv = item->priv;
    m_switch = priv->m_switch;
    cdos_switchs_toggle(m_switch);
    state = cdos_switchs_get_state(m_switch);
    g_signal_emit_by_name(item, "toggled", state);
    g_object_notify ((GObject *)item, "state");
}

/**
 * cdos_popup_switch_menu_item_set_toggle_status:
 * @item: A #CdosPopupSwitchMenuItem
 * @status: CdosPopupSwitchMenuItem的开关状态
 *
 * 设置CdosPopupSwitchMenuItem的开关状态
 *
 */
void
cdos_popup_switch_menu_item_set_toggle_status(CdosPopupSwitchMenuItem *item, gboolean status)
{
    CdosPopupSwitchMenuItemPrivate  *priv;
    CdosSwitchs *m_switch;
    g_return_if_fail(CDOS_IS_POPUP_SWITCH_MENU_ITEM(item));

    priv = item->priv;
    m_switch = priv->m_switch;
    cdos_switchs_set_toggle_state(m_switch, status);
    g_object_notify ((GObject *)item, "state");
}

/**
 * cdos_popup_switch_menu_item_get_status:
 * @item: A #CdosPopupSwitchMenuItem
 *
 * 获取item的state属性
 *
 * Returns: CdosPopupSwitchMenuItem的状态
 */
gboolean
cdos_popup_switch_menu_item_get_status(CdosPopupSwitchMenuItem *item)
{
    g_return_val_if_fail(CDOS_IS_POPUP_SWITCH_MENU_ITEM(item), FALSE);

    CdosPopupSwitchMenuItemPrivate *priv;
    CdosSwitchs *m_switch;
    priv = item->priv;
    m_switch = priv->m_switch;
    return cdos_switchs_get_state(m_switch);
}

/**
 * cdos_popup_switch_menu_item_set_text:
 * @item: A #CdosPopupSwitchMenuItem
 * @text:  CdosPopupSwitchMenuItem显示的text
 *
 * 设置CdosPopupSwitchMenuItem显示的text
 *
 */
void
cdos_popup_switch_menu_item_set_text (CdosPopupSwitchMenuItem *item,
                                      const gchar *text)
{
    g_return_if_fail(CDOS_IS_POPUP_SWITCH_MENU_ITEM(item));
    CdosPopupSwitchMenuItemPrivate    *priv;
    priv = item->priv;
    st_label_set_text(priv->label, text);
}

/**
 * cdos_popup_switch_menu_item_set_active:
 * @item: A #CdosPopupSwitchMenuItem
 * @active:  CdosPopupSwitchMenuItem的activate状态
 *
 * 设置CdosPopupSwitchMenuItem的activate状态
 *
 */
void
cdos_popup_switch_menu_item_set_active (CdosPopupSwitchMenuItem *item,
                                        gboolean                 active)
{
    CdosPopupSwitchMenuItemPrivate *priv;
    gboolean m_active;

    g_return_if_fail (CDOS_IS_POPUP_SWITCH_MENU_ITEM (item));

    priv = item->priv;
    m_active = cdos_switchs_get_state (priv->m_switch);

    if (active == m_active)
        return;

    cdos_switchs_set_state (priv->m_switch, active);

    g_object_notify ((GObject *)item, "state");
}
