/**
 * Copyright (c) 2016-2018 CPU and Fundamental Software Research Center, CAS
 *
 * This software is published by the license of CPU-OS License，you can use and
 * distibuted this software under this License. See CPU-OS Liense for more detail.
 *
 * You should have receive a copy of CPU-OS License. If not, please contact us
 * by email <support_os@cpu-os.ac.cn>
 *
 **/
#include <math.h>

#include "cdos-progress-bar.h"

struct _CdosProgressBarPrivate
{
    ClutterTextDirection direction;
    gfloat pulse_step;
    gfloat fraction;
    gfloat fill;

    gboolean use_markup;
    gboolean draw_value;
    gboolean paint_shadow;

    StDrawingArea *trough;
    GList *markups;

    gdouble trough_width;
    ClutterColor trough_normal_color;
    ClutterColor trough_active_color;
    ClutterColor trough_fill_color; //表示音乐播放器下载缓冲进度等

    ClutterInputDevice *device;
    ClutterEventSequence *sequence;
};

G_DEFINE_TYPE_WITH_PRIVATE (CdosProgressBar, cdos_progress_bar, ST_TYPE_WIDGET);

enum
{
    VALUE_CHANGED,
    LAST_SIGNAL
};

/*
static void
cdos_progress_bar_dispose (GObject *gobject)
{
    G_OBJECT_CLASS (cdos_progress_bar_parent_class)->dispose (gobject);
}

static void
cdos_progress_bar_finalize (GObject *object)
{
    G_OBJECT_CLASS (cdos_progress_bar_parent_class)->finalize (gobject);
}
*/

static void
cdos_progress_bar_get_preferred_width (ClutterActor *actor, gfloat for_height, gfloat *min_width_p, gfloat *nat_width_p)
{
    StThemeNode *theme_node;
    theme_node = st_widget_get_theme_node ((StWidget *)actor);

    *min_width_p = 1;
    *nat_width_p = 50;
    st_theme_node_adjust_preferred_width (theme_node, min_width_p, nat_width_p);
}

static void
cdos_progress_bar_get_preferred_height (ClutterActor *actor,
                                        gfloat for_width,
                                        gfloat *min_height_p,
                                        gfloat *nat_height_p)
{
    StThemeNode *theme_node;
    theme_node = st_widget_get_theme_node ((StWidget *)actor);

    *min_height_p = 1;
    *nat_height_p = 5;
    st_theme_node_adjust_preferred_height (theme_node, min_height_p, nat_height_p);
}

static void
cdos_progress_bar_allocate (ClutterActor *actor, const ClutterActorBox *box, ClutterAllocationFlags flags)
{
    CdosProgressBarPrivate *priv;
    StThemeNode *theme_node;
    ClutterActorBox content_box;
    ClutterActorBox child_box;

    clutter_actor_set_allocation (actor, box, flags);

    theme_node = st_widget_get_theme_node (ST_WIDGET (actor));
    st_theme_node_get_content_box (theme_node, box, &content_box);

    CdosProgressBar *self = CDOS_PROGRESS_BAR (actor);
    priv = self->priv;

    gfloat x = content_box.x1;
    gfloat y = content_box.y1;
    gfloat h = content_box.y2 - content_box.y1;

    y = content_box.y1 + h / 2;
    child_box.x1 = x;
    child_box.x2 = content_box.x2;
    child_box.y1 = y - priv->trough_width / 2.0;
    child_box.y2 = MAX (1.0, child_box.y1 + priv->trough_width);
    clutter_actor_allocate ((ClutterActor *)priv->trough, &child_box, flags);
}

static void
on_trough_repaint (StDrawingArea *trough, CdosProgressBar *self)
{
    CdosProgressBarPrivate *priv = self->priv;
    gdouble radius;
    guint width, height;

    gfloat value = CLAMP (priv->fraction, 0, 1);
    if (priv->pulse_step > 0)
    {
        gint n = value / priv->pulse_step;
        value = n * priv->pulse_step;
    }

    cairo_t *cr = st_drawing_area_get_context (priv->trough);
    st_drawing_area_get_surface_size (priv->trough, &width, &height);


    ClutterTextDirection dir = priv->direction;
    gboolean is_rtl = dir == CLUTTER_TEXT_DIRECTION_RTL;

    radius = height / 2.0;
    if (!is_rtl)
        value *= width;
    else
        value = width * (1 - value);

    cairo_arc (cr, radius, radius, radius, M_PI / 2.0, M_PI * 3.0 / 2.0);
    cairo_line_to (cr, value, 0);
    cairo_line_to (cr, value, height);
    cairo_line_to (cr, radius, height);

#define FILL_WITH_COLOR(color)                                                                                         \
    do                                                                                                                 \
    {                                                                                                                  \
        cairo_set_source_rgba (cr, color.red / 255.0, color.green / 255.0, color.blue / 255.0, color.alpha / 255.0);   \
        cairo_fill_preserve (cr);                                                                                      \
        cairo_set_line_width (cr, 0);                                                                                  \
        cairo_stroke (cr);                                                                                             \
    } while (0)

    if (!is_rtl)
        FILL_WITH_COLOR (priv->trough_active_color);
    else
        FILL_WITH_COLOR (priv->trough_normal_color);

    /* draw fill */
    if (priv->fill > priv->fraction)
    {
        /* TODO */
        cairo_set_source_rgba (cr,
                               priv->trough_fill_color.red / 255.0,
                               priv->trough_fill_color.green / 255.0,
                               priv->trough_fill_color.blue / 255.0,
                               priv->trough_fill_color.alpha / 255.0);
        cairo_fill_preserve (cr);
        cairo_set_line_width (cr, 0);
        cairo_stroke (cr);
    }
    /* draw normal */
    cairo_arc (cr, width - radius, radius, radius, M_PI * 3.0 / 2.0, M_PI / 2.0);
    cairo_line_to (cr, value, height);
    cairo_line_to (cr, value, 0);
    cairo_line_to (cr, width - radius, 0);
    if (!is_rtl)
        FILL_WITH_COLOR (priv->trough_normal_color);
    else
        FILL_WITH_COLOR (priv->trough_active_color);
#undef FILL_WITH_COLOR
}

static void
cdos_progress_bar_paint (ClutterActor *actor, ClutterPaintContext *ctx)
{
    CdosProgressBar *self = CDOS_PROGRESS_BAR (actor);
    CdosProgressBarPrivate *priv = self->priv;

    CLUTTER_ACTOR_CLASS (cdos_progress_bar_parent_class)->paint (actor, ctx);
    // st_drawing_area_queue_repaint (priv->trough);
    clutter_actor_paint ((ClutterActor *)priv->trough, ctx);
}

static gboolean
set_fraction (CdosProgressBar *self, gfloat value)
{
    CdosProgressBarPrivate *priv = self->priv;
    if (value == priv->fraction)
        return FALSE;

    if (value >= 1.0)
        value = 1.0;
    else if (value <= 0)
        value = 0;
    if (priv->pulse_step > 0)
    {
        guint n = (value - 0) / priv->pulse_step;
        value = n * priv->pulse_step;
        value = MIN (value, 1.0);
    }
    if (value == priv->fraction)
        return FALSE;

    priv->fraction = value;

    clutter_actor_queue_relayout ((ClutterActor *)self);
    st_drawing_area_queue_repaint (priv->trough);
    return TRUE;
}

static void
cdos_progress_bar_style_changed (StWidget *widget)
{
    CdosProgressBar *self = CDOS_PROGRESS_BAR (widget);
    CdosProgressBarPrivate *priv = self->priv;

    StThemeNode *theme_node;
    theme_node = st_widget_get_theme_node (widget);

    if (!st_theme_node_lookup_length (theme_node, "-trough-width", FALSE, &priv->trough_width))
        priv->trough_width = 5;

    if (!st_theme_node_lookup_color (theme_node, "-trough-normal-color", FALSE, &priv->trough_normal_color))
    {
        priv->trough_normal_color.red = 200;
        priv->trough_normal_color.green = 200;
        priv->trough_normal_color.blue = 200;
        priv->trough_normal_color.alpha = 255;
    }
    if (!st_theme_node_lookup_color (theme_node, "-trough-active-color", FALSE, &priv->trough_active_color))
    {
        priv->trough_active_color.red = 0;
        priv->trough_active_color.green = 90;
        priv->trough_active_color.blue = 255;
        priv->trough_active_color.alpha = 255;
    }
    if (!st_theme_node_lookup_color (theme_node, "-trough-fill-color", FALSE, &priv->trough_fill_color))
    {
        priv->trough_fill_color.red = 128;
        priv->trough_fill_color.green = 128;
        priv->trough_fill_color.blue = 128;
        priv->trough_fill_color.alpha = 255;
    }

    st_widget_style_changed ((StWidget *)priv->trough);
}

static void
cdos_progress_bar_init (CdosProgressBar *self)
{
    CdosProgressBarPrivate *priv;
    priv = self->priv =  cdos_progress_bar_get_instance_private (self);

    priv->direction = CLUTTER_TEXT_DIRECTION_LTR;

    priv->pulse_step = 0;
    priv->fraction = 0;
    priv->fill = 0;

    priv->trough = g_object_new (ST_TYPE_DRAWING_AREA, NULL);
    g_signal_connect (priv->trough, "repaint", G_CALLBACK (on_trough_repaint), self);
    clutter_actor_add_child ((ClutterActor *)self, (ClutterActor *)priv->trough);

    clutter_actor_set_reactive ((ClutterActor *)self, FALSE);
}

static void
cdos_progress_bar_class_init (CdosProgressBarClass *klass)
{
    // GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
    // gobject_class->dispose = cdos_progress_bar_dispose;
    // gobject_class->finalize = cdos_progress_bar_finalize;

    ClutterActorClass *actor_class = CLUTTER_ACTOR_CLASS (klass);
    actor_class->get_preferred_width = cdos_progress_bar_get_preferred_width;
    actor_class->get_preferred_height = cdos_progress_bar_get_preferred_height;
    actor_class->allocate = cdos_progress_bar_allocate;
    actor_class->paint = cdos_progress_bar_paint;

    StWidgetClass *widget_class = ST_WIDGET_CLASS (klass);
    widget_class->style_changed = cdos_progress_bar_style_changed;
}

CdosProgressBar *
cdos_progress_bar_new (void)
{
    CdosProgressBar *self = g_object_new (CDOS_TYPE_PROGRESS_BAR, "style-class", "progress-bar", NULL);

    return self;
}

void
cdos_progress_bar_set_fraction (CdosProgressBar *self, gfloat fraction)
{
    g_return_if_fail (CDOS_IS_PROGRESS_BAR (self));
    set_fraction (self, fraction);
}

gfloat
cdos_progress_bar_get_fraction (CdosProgressBar *self)
{
    g_return_val_if_fail (CDOS_IS_PROGRESS_BAR (self), 0);
    return self->priv->fraction;
}

void
cdos_progress_bar_set_direction (CdosProgressBar *self, ClutterTextDirection direction)
{
    g_return_if_fail (CDOS_IS_PROGRESS_BAR (self));
    if (self->priv->direction != direction)
    {
        self->priv->direction = direction;
        if (clutter_actor_is_visible ((ClutterActor *)self))
        {
            clutter_actor_queue_relayout ((ClutterActor *)self);
            st_drawing_area_queue_repaint (self->priv->trough);
        }
    }
}

void
cdos_progress_bar_set_pulse_step (CdosProgressBar *self, gfloat pulse_step)
{
    g_return_if_fail (CDOS_IS_PROGRESS_BAR (self));
    CdosProgressBarPrivate *priv;
    priv = self->priv;
    priv->pulse_step = pulse_step;
}
