#include <math.h>

#include "cdos-scale.h"

struct _CdosScalePrivate
{
    ClutterOrientation orientation;
    ClutterTextDirection direction;
    gfloat min;
    gfloat max;
    gfloat step;
    gfloat value;
    gfloat fill;

    gboolean use_markup;
    gboolean draw_value;
    gboolean paint_shadow;

    StDrawingArea *trough;
    ClutterActor *slider;
    GList *markups;

    gfloat trough_width;
    ClutterColor trough_normal_color;
    ClutterColor trough_active_color;
    ClutterColor trough_fill_color; //表示音乐播放器下载缓冲进度等

    gboolean dragging;
    ClutterInputDevice *device;
    ClutterEventSequence *sequence;
};

G_DEFINE_TYPE_WITH_PRIVATE (CdosScale, cdos_scale, ST_TYPE_WIDGET);

enum {
    PROP_0,
    PROP_ORIENTATION
};

enum {
    VALUE_CHANGED,
    LAST_SIGNAL
};

static guint cdos_scale_signals [LAST_SIGNAL] = {0};

/*
static void
cdos_scale_dispose (GObject *gobject)
{
    G_OBJECT_CLASS (cdos_scale_parent_class)->dispose (gobject);
}

static void
cdos_scale_finalize (GObject *object)
{
    G_OBJECT_CLASS (cdos_scale_parent_class)->finalize (gobject);
}
*/

static void
cdos_scale_get_property (GObject    *gobject,
                         guint       prop_id,
                         GValue     *value,
                         GParamSpec *pspec)
{
    CdosScale *self = CDOS_SCALE (gobject);
    CdosScalePrivate *priv = self->priv;

    switch (prop_id)
    {
        case PROP_ORIENTATION:
            g_value_set_uint (value, priv->orientation);
            break;
        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
            break;
    }
}

static void
cdos_scale_set_property (GObject       *gobject,
                         guint          prop_id,
                         const GValue  *value,
                         GParamSpec    *pspec)
{
    CdosScale *self = CDOS_SCALE (gobject);
    CdosScalePrivate *priv = self->priv;

    switch (prop_id)
    {
        case PROP_ORIENTATION:
            priv->orientation = g_value_get_uint (value);
            break;
        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
            break;
    }
}

static void
cdos_scale_get_preferred_width (ClutterActor *actor,
                                gfloat        for_height,
                                gfloat       *min_width_p,
                                gfloat       *nat_width_p)
{
    CdosScale *self = CDOS_SCALE (actor);
    CdosScalePrivate *priv = self->priv;

    StThemeNode *theme_node;
    theme_node = st_widget_get_theme_node ((StWidget *)actor);

    gfloat min, nat;
    clutter_actor_get_preferred_width (priv->slider, for_height, &min, &nat);

    if (priv->orientation == CLUTTER_ORIENTATION_HORIZONTAL)
    {
        *min_width_p = min;
        *nat_width_p = nat + 50;
    }
    else
    {
        *min_width_p = min;
        *nat_width_p = nat;
    }
    st_theme_node_adjust_preferred_width (theme_node, min_width_p, nat_width_p);
}

static void
cdos_scale_get_preferred_height (ClutterActor *actor,
                                 gfloat        for_width,
                                 gfloat       *min_height_p,
                                 gfloat       *nat_height_p)
{
    CdosScale *self = CDOS_SCALE (actor);
    CdosScalePrivate *priv = self->priv;

    StThemeNode *theme_node;
    theme_node = st_widget_get_theme_node ((StWidget *)actor);

    gfloat min, nat;
    clutter_actor_get_preferred_height (priv->slider, for_width, &min, &nat);

    if (priv->orientation == CLUTTER_ORIENTATION_HORIZONTAL)
    {
        *min_height_p = min;
        *nat_height_p = nat;
    }
    else
    {
        *min_height_p = min;
        *nat_height_p = nat + 50;
    }
    st_theme_node_adjust_preferred_height (theme_node, min_height_p, nat_height_p);
}

static void
cdos_scale_allocate (ClutterActor *actor,
                     const ClutterActorBox *box,
                     ClutterAllocationFlags flags)
{
    CdosScalePrivate *priv;
    StThemeNode *theme_node;
    ClutterActorBox content_box;
    ClutterActorBox child_box;
    gfloat min_h, nat_h, min_w, nat_w;
    gfloat position;
    gfloat range;

    clutter_actor_set_allocation (actor, box, flags);

    theme_node = st_widget_get_theme_node (ST_WIDGET (actor));
    st_theme_node_get_content_box (theme_node, box, &content_box);

    CdosScale *self = CDOS_SCALE (actor);
    priv = self->priv;

    gboolean is_rtl = priv->direction == CLUTTER_TEXT_DIRECTION_RTL;
    gfloat x = content_box.x1;
    gfloat y = content_box.y1;
    gfloat w = content_box.x2 - content_box.x1;
    gfloat h = content_box.y2 - content_box.y1;
    range = priv->max - priv->min;
    if (range == 0)
        range = G_MAXUINT;
    gfloat value = CLAMP (priv->value, priv->min, priv->max);
    value -= priv->min;

    clutter_actor_get_preferred_width (priv->slider, h, &min_w, &nat_w);
    clutter_actor_get_preferred_height (priv->slider, w, &min_h, &nat_h);
    nat_w = MAX(nat_w, 1.0);
    nat_h = MAX(nat_h, 1.0);

    if (priv->orientation == CLUTTER_ORIENTATION_HORIZONTAL)
    {
        y = content_box.y1 + h / 2 - nat_h / 2;
        child_box.x1 = x;
        child_box.x2 = content_box.x2;
        child_box.y1 = y + nat_h / 2 - priv->trough_width / 2.0;
        child_box.y2 = MAX (1.0, child_box.y1 + priv->trough_width);
        clutter_actor_allocate ((ClutterActor *)priv->trough, &child_box, flags);

        if (!is_rtl)
            position = value / range * w;
        else
            position = (1 - value / range) * w;

        child_box.x1 = MAX (0, position - nat_w / 2);
        child_box.x2 = child_box.x1 + nat_w;
        if (child_box.x2 > content_box.x2)
        {
            child_box.x2 = content_box.x2;
            child_box.x1 = child_box.x2 - nat_w;
        }
        child_box.y1 = y;
        child_box.y2 = child_box.y1 + nat_h;
        clutter_actor_allocate (priv->slider, &child_box, flags);
    }
    else
    {
        x = content_box.x1 + w / 2.0 - nat_w / 2.0;
        child_box.x1 = x + nat_w / 2 - priv->trough_width / 2.0;
        child_box.x2 = MAX (1.0, child_box.x1 + priv->trough_width);
        child_box.y1 = y;
        child_box.y2 = content_box.y2;
        clutter_actor_allocate ((ClutterActor *)priv->trough, &child_box, flags);

        if (!is_rtl)
            position = value / range * h;
        else
            position = (1 - value / range) * h;
        child_box.x1 = x;
        child_box.x2 = child_box.x1 + nat_w;
        child_box.y1 = MAX (0, position - nat_h / 2);
        child_box.y2 = child_box.y1 + nat_h;
        if (child_box.y2 > content_box.y2)
        {
            child_box.y2 = content_box.y2;
            child_box.y1 = child_box.y2 - nat_h;
        }
        clutter_actor_allocate (priv->slider, &child_box, flags);
    }
}

static void
on_trough_repaint (StDrawingArea *trough, CdosScale *self)
{
    CdosScalePrivate *priv = self->priv;
    gdouble radius;
    guint width, height;

    gfloat range = priv->max - priv->min;
    gfloat value = CLAMP (priv->value, priv->min, priv->max);
    if (priv->step > 0)
    {
        gint n = (value - priv->min) / priv->step;
        value = n * priv->step;
    }
    value = (value - priv->min) / range;

    cairo_t *cr = st_drawing_area_get_context (priv->trough);
    st_drawing_area_get_surface_size (priv->trough, &width, &height);

    ClutterTextDirection dir = priv->direction;
    gboolean is_rtl = dir == CLUTTER_TEXT_DIRECTION_RTL;

    if (priv->orientation == CLUTTER_ORIENTATION_HORIZONTAL)
    {
        radius = height / 2.0;
        if (!is_rtl)
            value *= width;
        else
            value = width * (1 - value);
    }
    else
    {
        radius = width / 2.0;
        if (!is_rtl)
            value *= height;
        else
            value = height * (1 - value);
    }

    if (priv->orientation == CLUTTER_ORIENTATION_HORIZONTAL)
    {
        cairo_arc (cr, radius, radius, radius, M_PI / 2.0, M_PI * 3.0 / 2.0);
        cairo_line_to (cr, value, 0);
        cairo_line_to (cr, value, height);
        cairo_line_to (cr, radius, height);
    }
    else
    {
        cairo_arc (cr, radius, radius, radius, M_PI, 0);
        cairo_line_to (cr, width, value);
        cairo_line_to (cr, 0, value);
        cairo_line_to (cr, 0, radius);
    }
#define FILL_WITH_COLOR(color) do{ \
    cairo_set_source_rgba (cr, color.red / 255.0, \
                           color.green / 255.0, \
                           color.blue / 255.0, \
                           color.alpha / 255.0); \
    cairo_fill_preserve (cr); \
    cairo_set_line_width (cr, 0); \
    cairo_stroke (cr);} \
    while(0)

    if (!is_rtl)
        FILL_WITH_COLOR(priv->trough_active_color);
    else
        FILL_WITH_COLOR(priv->trough_normal_color);

    /* draw fill */
    if (priv->fill > priv->value)
    {
        /* TODO */
        cairo_set_source_rgba (cr, priv->trough_fill_color.red / 255.0,
                priv->trough_fill_color.green / 255.0,
                priv->trough_fill_color.blue / 255.0,
                priv->trough_fill_color.alpha / 255.0);
        cairo_fill_preserve (cr);
        cairo_set_line_width (cr, 0);
        cairo_stroke (cr);
    }
    /* draw normal */
    if (priv->orientation == CLUTTER_ORIENTATION_HORIZONTAL)
    {
        cairo_arc (cr, width - radius, radius, radius, M_PI * 3.0 / 2.0, M_PI / 2.0);
        cairo_line_to (cr, value, height);
        cairo_line_to (cr, value, 0);
        cairo_line_to (cr, width - radius, 0);
    }
    else
    {
        cairo_arc (cr, radius, height - radius, radius, 0, M_PI);
        cairo_line_to (cr, 0, value);
        cairo_line_to (cr, width, value);
        cairo_line_to (cr, width, height - radius);
    }
    if (!is_rtl)
        FILL_WITH_COLOR(priv->trough_normal_color);
    else
        FILL_WITH_COLOR(priv->trough_active_color);
#undef FILL_WITH_COLOR
}

static void
cdos_scale_paint (ClutterActor *actor, ClutterPaintContext *context)
{
    CdosScale *self = CDOS_SCALE (actor);
    CdosScalePrivate *priv = self->priv;

    //st_drawing_area_queue_repaint (priv->trough);
    clutter_actor_paint ((ClutterActor *)priv->trough, context);
    clutter_actor_paint (priv->slider, context);
}

static gboolean
set_value (CdosScale *self, gfloat value)
{
    CdosScalePrivate *priv = self->priv;
    if (value == priv->value)
        return FALSE;

    if (value >= priv->max)
        value = priv->max;
    else if (value <= priv->min)
        value = priv->min;
    else if (priv->step > 0)
    {
        guint n = (value - priv->min) / priv->step;
        value = n * priv->step + priv->min;
        value = MIN (value, priv->max);
    }
    if (value == priv->value)
        return FALSE;

    priv->value = value;

    clutter_actor_queue_relayout ((ClutterActor *)self);
    st_drawing_area_queue_repaint (priv->trough);
    return TRUE;
}

static void
change_value (CdosScale *self, gfloat value)
{
    if (set_value (self, value))
        g_signal_emit (self, cdos_scale_signals[VALUE_CHANGED], 0, self->priv->value);
}

static void
cdos_scale_move_slider (CdosScale *self, graphene_point_t *point)
{
    gfloat value;
    gfloat x, y;
    gfloat real_x, real_y;
    CdosScalePrivate *priv = self->priv;

    gfloat range = priv->max - priv->min;

    clutter_actor_get_transformed_position ((ClutterActor *)self, &x, &y);
    ClutterActorBox box;
    clutter_actor_get_allocation_box ((ClutterActor *)self, &box);
    if (priv->orientation != CLUTTER_ORIENTATION_VERTICAL)
    {
        gfloat width = box.x2 - box.x1; //clutter_actor_get_width ((ClutterActor *)self);
        real_x = CLAMP (point->x - x, 0, width);
        if (priv->direction == CLUTTER_TEXT_DIRECTION_LTR)
            value = real_x / width * range + priv->min;
        else
            value = (width - real_x) / width * range + priv->min;
    }
    else
    {
        gfloat height = box.y2 - box.y1; //clutter_actor_get_height ((ClutterActor *)self);
        real_y = CLAMP (point->y - y, 0, height);
        if (priv->direction == CLUTTER_TEXT_DIRECTION_LTR)
            value = real_y / height * range + priv->min;
        else
            value = (height - real_y) / height * range + priv->min;
    }
    change_value (self, value);
}

static gboolean
start_dragging (CdosScale *self,
                ClutterEvent *event)
{
    CdosScalePrivate *priv = self->priv;
    ClutterActor *actor = CLUTTER_ACTOR (self);
    ClutterInputDevice *device;
    ClutterEventSequence *sequence;
    graphene_point_t position;

    priv->dragging = TRUE;

    device = clutter_event_get_device (event);
    sequence = clutter_event_get_event_sequence (event);
    if (sequence)
        clutter_input_device_sequence_grab (device, sequence, actor);
    else
        clutter_input_device_grab (device, actor);
    priv->device = device;
    priv->sequence = sequence;

    clutter_event_get_position (event, &position);
    cdos_scale_move_slider (self, &position);
    return TRUE;
}

static void
end_dragging (CdosScale *self)
{
    CdosScalePrivate *priv = self->priv;

    if (priv->dragging)
    {
        if (priv->sequence)
            clutter_input_device_sequence_ungrab (priv->device, priv->sequence);
        else
            clutter_input_device_ungrab (priv->device);
        priv->sequence = NULL;
        priv->device = NULL;
        priv->dragging = FALSE;
    }
}

static gboolean
cdos_scale_button_press_event (ClutterActor *actor,
                               ClutterButtonEvent *button)
{
    CdosScale *self = CDOS_SCALE (actor);
    CdosScalePrivate *priv = self->priv;

    if (priv->dragging)
        return TRUE;

    clutter_stage_set_key_focus ((ClutterStage *)clutter_actor_get_stage (actor), actor);
    ClutterEvent *event = (ClutterEvent *)button;
    return start_dragging (self, event);
}

static gboolean
cdos_scale_button_release_event (ClutterActor *actor,
                                 ClutterButtonEvent *button)
{
    CdosScale *self = CDOS_SCALE (actor);
    CdosScalePrivate *priv = self->priv;

    if (priv->dragging)
    {
        end_dragging (self);
    }
    return TRUE;
}

static gboolean
cdos_scale_motion_event (ClutterActor *actor,
                         ClutterMotionEvent *event)
{
    CdosScale *self = CDOS_SCALE (actor);
    CdosScalePrivate *priv = self->priv;

    if (priv->dragging)
    {
        graphene_point_t position;
        clutter_event_get_position ((ClutterEvent *)event, &position);
        cdos_scale_move_slider (self, &position);
    }
    return TRUE;
}

#define SCROLL_STEP 0.02

static gboolean
cdos_scale_scroll_event (ClutterActor *actor,
                         ClutterScrollEvent *event)
{
    CdosScale *self = CDOS_SCALE (actor);
    CdosScalePrivate *priv = self->priv;
    ClutterScrollDirection direction;
    gfloat delta;

    if (clutter_event_is_pointer_emulated ((ClutterEvent *)event))
        return FALSE;

    if (priv->step)
        delta = priv->step;
    else
        delta = SCROLL_STEP * (priv->max - priv->min);

    direction = clutter_event_get_scroll_direction ((ClutterEvent *)event);
    if (direction == CLUTTER_SCROLL_DOWN)
        delta = -delta;
    else if (direction == CLUTTER_SCROLL_UP)
        delta = delta;
    else if (direction == CLUTTER_SCROLL_SMOOTH)
    {
        gdouble dx, dy;
        clutter_event_get_scroll_delta ((ClutterEvent *)event, &dx, &dy);
        delta = -dy * delta;
    }
    else
        return TRUE;

    gfloat value = priv->value + delta;
    change_value (self, value);
    return TRUE;
}

static gboolean
cdos_scale_key_press_event (ClutterActor *actor,
                            ClutterKeyEvent *event)
{
    CdosScale *self = CDOS_SCALE (actor);
    CdosScalePrivate *priv = self->priv;
    gfloat delta;
    gint ltr = 1;

    if (priv->step)
        delta = priv->step;
    else
        delta = SCROLL_STEP * (priv->max - priv->min);

    if ((priv->orientation == CLUTTER_ORIENTATION_HORIZONTAL && priv->direction == CLUTTER_TEXT_DIRECTION_RTL)
        || (priv->orientation == CLUTTER_ORIENTATION_VERTICAL && priv->direction == CLUTTER_TEXT_DIRECTION_LTR))
        ltr = -1;

    guint key = clutter_event_get_key_symbol ((ClutterEvent *)event);
    if (key == CLUTTER_KEY_Left || key == CLUTTER_KEY_Down)
        delta = -delta * ltr;
    else if (key != CLUTTER_KEY_Right && key != CLUTTER_KEY_Up)
        return TRUE;

    gfloat value = priv->value + delta;
    change_value (self, value);
    return TRUE;
}

static void
cdos_scale_style_changed (StWidget *widget)
{
    CdosScale *self = CDOS_SCALE (widget);
    CdosScalePrivate *priv = self->priv;

    StThemeNode *theme_node;
    theme_node = st_widget_get_theme_node (widget);

    priv->trough_width = st_theme_node_get_length (theme_node, "-trough-width");
     if (!st_theme_node_lookup_color (theme_node, "-trough-normal-color", FALSE, &priv->trough_normal_color))
    {
        priv->trough_normal_color.red = 200;
        priv->trough_normal_color.green = 200;
        priv->trough_normal_color.blue = 200;
        priv->trough_normal_color.alpha = 255;
    }
    if (!st_theme_node_lookup_color (theme_node, "-trough-active-color", FALSE, &priv->trough_active_color))
    {
        //use '-gtk-background-color' replace '-trough-active-color'
        st_theme_node_get_background_color (theme_node, &priv->trough_active_color);
        /* priv->trough_active_color.red = 0;
        priv->trough_active_color.green = 90;
        priv->trough_active_color.blue = 255;
        priv->trough_active_color.alpha = 255; */
    }
    if (!st_theme_node_lookup_color (theme_node, "-trough-fill-color", FALSE, &priv->trough_fill_color))
    {
        priv->trough_fill_color.red = 128;
        priv->trough_fill_color.green = 128;
        priv->trough_fill_color.blue = 128;
        priv->trough_fill_color.alpha = 255;
    } 

    st_widget_style_changed ((StWidget *)priv->trough);
    st_widget_style_changed ((StWidget *)priv->slider);
}

static void
cdos_scale_init (CdosScale *self)
{
    CdosScalePrivate *priv;
    priv = self->priv = cdos_scale_get_instance_private (self);

    priv->orientation = CLUTTER_ORIENTATION_HORIZONTAL;
    priv->direction = CLUTTER_TEXT_DIRECTION_LTR;
    priv->dragging = FALSE;

    priv->min = 0;
    priv->max = 1.0;
    priv->step = 0;
    priv->value = 0;
    priv->fill = priv->min;

    priv->trough = g_object_new (ST_TYPE_DRAWING_AREA, NULL);
    g_signal_connect (priv->trough, "repaint", G_CALLBACK (on_trough_repaint), self);
    priv->slider = g_object_new (ST_TYPE_WIDGET,
                                 "style-class", "slider",
                                 "reactive", TRUE,
                                 "track-hover", TRUE,
                                 NULL);
    clutter_actor_add_child ((ClutterActor *)self, (ClutterActor *)priv->trough);
    clutter_actor_add_child ((ClutterActor *)self, (ClutterActor *)priv->slider);

    clutter_actor_set_reactive ((ClutterActor *)self, TRUE);
}

static void
cdos_scale_class_init (CdosScaleClass *klass)
{
    GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
    gobject_class->get_property = cdos_scale_get_property;
    gobject_class->set_property = cdos_scale_set_property;
    //gobject_class->dispose = cdos_scale_dispose;
    //gobject_class->finalize = cdos_scale_finalize;

    ClutterActorClass *actor_class = CLUTTER_ACTOR_CLASS (klass);
    actor_class->get_preferred_width = cdos_scale_get_preferred_width;
    actor_class->get_preferred_height = cdos_scale_get_preferred_height;
    actor_class->allocate = cdos_scale_allocate;
    actor_class->paint = cdos_scale_paint;
    actor_class->button_press_event = cdos_scale_button_press_event;
    actor_class->button_release_event = cdos_scale_button_release_event;
    actor_class->motion_event = cdos_scale_motion_event;
    actor_class->scroll_event = cdos_scale_scroll_event;
    actor_class->key_press_event = cdos_scale_key_press_event;

    StWidgetClass *widget_class = ST_WIDGET_CLASS (klass);
    widget_class->style_changed = cdos_scale_style_changed;

    GParamSpec *pspec;
    pspec = g_param_spec_uint ("orientation",
                               "Orientation",
                               "The orientation of scale",
                               CLUTTER_ORIENTATION_HORIZONTAL,
                               CLUTTER_ORIENTATION_VERTICAL,
                               CLUTTER_ORIENTATION_HORIZONTAL,
                               G_PARAM_READWRITE);
    g_object_class_install_property (gobject_class, PROP_ORIENTATION, pspec);

    cdos_scale_signals[VALUE_CHANGED] =
        g_signal_new ("value-changed",
                      CDOS_TYPE_SCALE,
                      G_SIGNAL_RUN_LAST,
                      0,
                      NULL, NULL,
                      g_cclosure_marshal_VOID__FLOAT,
                      G_TYPE_NONE, 1,
                      G_TYPE_FLOAT);
}

CdosScale *
cdos_scale_new (void)
{
    CdosScale *self = g_object_new (CDOS_TYPE_SCALE,
                                     "style-class", "scale",
                                     NULL);
    return self;
}

CdosScale *
cdos_scale_new_with_range (ClutterOrientation orientation, gfloat min, gfloat max, gfloat step)
{
    CdosScale *self = g_object_new (CDOS_TYPE_SCALE,
                                    "style-class", "scale",
                                    "orientation", orientation,
                                    NULL);

    CdosScalePrivate *priv;
    priv = self->priv;
    priv->min = min;
    priv->max = max;
    priv->step = step;

    return self;
}

void
cdos_scale_set_value (CdosScale *self, gfloat value)
{
    g_return_if_fail (CDOS_IS_SCALE (self));
    set_value (self, value);
}

gfloat
cdos_scale_get_value (CdosScale *self)
{
    g_return_val_if_fail (CDOS_IS_SCALE (self), 0);
    return self->priv->value;
}

void
cdos_scale_set_direction (CdosScale *self, ClutterTextDirection direction)
{
    g_return_if_fail (CDOS_IS_SCALE (self));
    if (self->priv->direction != direction)
    {
        self->priv->direction = direction;
        if (clutter_actor_is_visible ((ClutterActor *)self))
        {
            clutter_actor_queue_relayout ((ClutterActor *)self);
            st_drawing_area_queue_repaint (self->priv->trough);
        }
    }
}

void
cdos_scale_set_range (CdosScale *self, gfloat min, gfloat max, gfloat step)
{
    g_return_if_fail (CDOS_IS_SCALE (self));
    CdosScalePrivate *priv;
    priv = self->priv;
    priv->min = min;
    priv->max = max;
    priv->step = step;
}
