/*
*
* This program is free software; you can redistribute it and/or modify it
* under the terms and conditions of the GNU Lesser General Public License,
* version 2.1, as published by the Free Software Foundation.
*
* This program is distributed in the hope it will be useful, but WITHOUT ANY
* WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
* FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
* more details.
*
* You should have received a copy of the GNU Lesser General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

#include <glib.h>
#include <gio/gio.h>

#include "cdos-search-provider.h"

G_DEFINE_TYPE (CdosSearchProvider, cdos_search_provider, G_TYPE_OBJECT)

enum
{
    FINISHED,
    LAST_SIGNAL
}; 

static guint signals[LAST_SIGNAL] = { 0, };

static void
cdos_search_provider_dispose (GObject *object)
{
    G_OBJECT_CLASS (cdos_search_provider_parent_class)->dispose (object);
}

static void
cdos_search_provider_init (CdosSearchProvider *self)
{
}

static void
cdos_search_provider_class_init (CdosSearchProviderClass *klass)
{
    GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
    gobject_class->dispose = cdos_search_provider_dispose;

    /**
     * CdosSearchProvider::finished:
     * @self: the #CdosSearchProvider
     *
     * Emitted when the search have finished.
     */
    signals[FINISHED] =
        g_signal_new ("finished",
                      G_TYPE_FROM_CLASS (klass),
                      G_SIGNAL_RUN_LAST,
                      0,
                      NULL, NULL, NULL,
                      G_TYPE_NONE,
                      1, G_TYPE_POINTER);
}

void
cdos_search_provider_search (CdosSearchProvider *self,
                             const char *str,
                             gint count)
{
    if (CDOS_SEARCH_PROVIDER_CLASS (G_OBJECT_GET_CLASS (self))->search)
    {
        CDOS_SEARCH_PROVIDER_CLASS (G_OBJECT_GET_CLASS (self))->search (self, str, count);
    }
}

void
cdos_search_provider_cancel (CdosSearchProvider *self)
{
    if (CDOS_SEARCH_PROVIDER_CLASS (G_OBJECT_GET_CLASS (self))->cancel)
    {
        CDOS_SEARCH_PROVIDER_CLASS (G_OBJECT_GET_CLASS (self))->cancel (self);
    }
}

void
cdos_search_results_free (GPtrArray *results)
{
    for (gint i = 0; i < results->len; i++)
    {
        CdosSearchResult *result = g_ptr_array_index (results, i);
        g_free (result->name);
        g_free (result->description);
        g_object_unref (result->icon);
    }
    g_ptr_array_free (results, TRUE);
}
