#include "config.h"
#include <clutter/clutter.h>
#include <gio/gio.h>
#include <meta/meta-workspace-manager.h>

#include "cdos-icon-applet.h"
#include "cdos-showdesktop.h"

/**
 * SECTION:cdos-showdesktop
 * @title: CdosShowdesktop
 * @short_description: 显示桌面
 *
 * CdosShowdesktop 在桌面上有窗口显示时,点击显示桌面,桌面就可以把窗口隐藏,只显示桌面
 */

struct _CdosShowdesktopPrivate
{
     StBin     *applet_icon_box;
     gboolean   show;
}; 

G_DEFINE_TYPE_WITH_PRIVATE (CdosShowdesktop, cdos_showdesktop, CDOS_TYPE_ICONAPPLET);

static gboolean 
cdos_showdesktop_on_applet_clicked (ClutterActor *actor, ClutterEvent *event, CdosApplet *self)
{
#ifdef HAVE_META_WORKSPACE_MANAGER_SHOW_DESKTOP
    //CdosShowdesktop *me = CDOS_SHOWDESKTOP (self);

    CdosGlobal *global = cdos_global_get ();
    MetaWorkspaceManager *wsm = cdos_global_get_workspace_manager (global);
    guint32 time = cdos_global_get_current_time (global);

    MetaWorkspace *workspace = meta_workspace_manager_get_active_workspace (wsm);
    gboolean is_showing_desktop = meta_workspace_get_showing_desktop (workspace);
    if (!is_showing_desktop)
    {
        meta_workspace_manager_show_desktop (wsm, time);
    }
    else
    {
        meta_workspace_manager_unshow_desktop (wsm);
    }
#endif
    return TRUE;
}

static void 
cdos_showdesktop_class_init( CdosShowdesktopClass *kclass)
{
}

static void
cdos_showdesktop_init(CdosShowdesktop *self)
{
    self->priv = cdos_showdesktop_get_instance_private (self);
    cdos_applet_set_popup_menu_active ((CdosApplet *)self, FALSE);
}


static CdosShowdesktop *
cdos_showdesktop_new (guint orientation, guint panel_height)
{
    CdosShowdesktop *showdesktop;
    showdesktop = g_object_new(CDOS_TYPE_SHOWDESKTOP, "orientation", orientation, NULL);
    CdosShowdesktopPrivate *priv = showdesktop->priv;

    st_widget_remove_style_class_name (ST_WIDGET (showdesktop), "applet-box");
    st_widget_set_style_class_name (ST_WIDGET (showdesktop), "show-desktop");

    priv->applet_icon_box = cdos_iconapplet_get_applet_icon_box (CDOS_ICONAPPLET (showdesktop));

    //cdos_iconapplet_set_icon_name (CDOS_ICONAPPLET (showdesktop), "show-desktop");
    cdos_applet_set_applet_tooltip((CdosApplet *)showdesktop, _("show-desktop"));
    st_widget_set_style_class_name ((StWidget *)priv->applet_icon_box,"show-desktop-box");
    clutter_actor_set_reactive(CLUTTER_ACTOR(priv->applet_icon_box),TRUE);
    st_widget_set_track_hover((StWidget *)priv->applet_icon_box,TRUE);

    g_signal_connect (showdesktop, "button-press-event",
                      G_CALLBACK(cdos_showdesktop_on_applet_clicked),
                      showdesktop);

    return showdesktop;
}   

/**
 * cdos_showdesktop_main:
 * @meta: a #MetaData
 * @orientation: 桌面显示的方向
 * @panel_height: panel的高度
 *
 * 新建一个showdesktop,并实现显示桌面的功能
 *
 * Returns: showdesktop
 */
CdosApplet * 
cdos_showdesktop_main (MetaData *meta,guint orientation, guint panel_height)
{
    static CdosShowdesktop *showdesktop = NULL;

    if(showdesktop == NULL){
        showdesktop = cdos_showdesktop_new (orientation, panel_height);
    }
    return CDOS_APPLET (showdesktop);
}
