/** 
 * Copyright (c) 2016-2018 CPU and Fundamental Software Research Center, CAS
 *  
 * This software is published by the license of CPU-OS License，you can use and 
 * distibuted this software under this License. See CPU-OS Liense for more detail.
 *
 * You should have receive a copy of CPU-OS License. If not, please contact us 
 * by email <support_os@cpu-os.ac.cn>  
 * 
 **/
#ifndef __CDOS_SOURCE_H__
#define __CDOS_SOURCE_H__

#include "st.h"
#include "cdos-notification.h"

GType cdos_source_get_type (void);

#define CDOS_TYPE_SOURCE        (cdos_source_get_type ())
#define CDOS_SOURCE(obj)              (G_TYPE_CHECK_INSTANCE_CAST ((obj), CDOS_TYPE_SOURCE, CdosSource))
#define CDOS_IS_SOURCE(obj)           (G_TYPE_CHECK_INSTANCE_TYPE ((obj), CDOS_TYPE_SOURCE))
#define CDOS_SOURCE_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST ((klass), CDOS_TYPE_SOURCE, CdosSourceClass))
#define CDOS_IS_SOURCE_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE ((klass), CDOS_TYPE_SOURCE))
#define CDOS_SOURCE_GET_CLASS(obj)   (G_TYPE_INSTANCE_GET_CLASS ((obj), CDOS_TYPE_SOURCE, CdosSourceClass))

typedef struct _CdosSource        CdosSource;
typedef struct _CdosSourceClass   CdosSourceClass;

#define SOURCE_ICON_SIZE 16
#define DEFAULT_ANIMATE_DURATION 150
#define DEFAULT_ACCELERATION_SPEED 0.8

struct _CdosSourceClass
{
    StWidgetClass parent_class;

    //signal
    void (*notification_added) (CdosSource *self, CdosNotification *notification);
    void (*notify) (CdosSource *self, CdosNotification *notification);

    void (*open) (CdosSource *self);
    void (*remove_last)(CdosSource *self);
    ClutterActor *(*create_icon) (CdosSource *self);
};

struct _CdosSource
{
    StWidget parent_instance;

    gchar *name;
    gchar *app_name;

    gboolean is_transient;
    char *category;

    GList *notifications; //指向CdosNotification

    /*<private>*/
    ClutterActor *icon_bin;
    ClutterActor *icon;
    StLabel *name_label;
    ClutterActor *time;
    StWidget *close;
};

/* Source 公有函数 */

void cdos_source_set_transient (CdosSource *self, gboolean isTransient);

void cdos_source_create_notification (CdosSource *self);

void cdos_source_push_notification (CdosSource *self, CdosNotification *notification);

void cdos_source_notify (CdosSource *self, CdosNotification *notification);

void cdos_source_destroy (CdosSource *self, guint reason);

void cdos_source_open (CdosSource *self);

void cdos_source_destroy_non_resident_notifications (CdosSource *self);

void cdos_source_set_name (CdosSource *self, const char *name);

const gchar *cdos_source_get_name (CdosSource *self);

void cdos_source_set_app_name (CdosSource *self, const char *app_name);

const gchar *cdos_source_get_app_name (CdosSource *self);

ClutterActor *cdos_source_create_icon (CdosSource *self);

void cdos_source_set_summary_icon (CdosSource *self, ClutterActor *icon);

CdosSource *cdos_source_new (const gchar *name);

void cdos_source_clear_all (CdosSource *self);

#endif
