#include "config.h"

#include <meta/meta-background-group.h>
#include <meta/main.h>

#include "cdos-debug.h"
#include "cdos-start.h"
#include "cdos-window-manager.h"
#include "cdos-layout-manager.h"
#include "cdos-indicator-manager.h"
#include "cdos-input-method.h"
#include "cdos-ui-plugin.h"
#include "cdos-global.h"
#include "cdos-run-dialog.h"
#include "cdos-background.h"
#include "cdos-notification-daemon.h"
#include "cdos-applet-manager.h"
#include "cdos-auto-mount-manager.h"
#include "cdos-message-tray.h"
#include "cdos-sound-manager.h"
#include "cdos-theme-manager.h"
#include "cdos-login-manager.h"
#include "cdos-keyboard-manager.h"
#include "cdos-magnifier.h"

extern int debug_call;
extern int debug_mode;

ClutterActor *cdos_panel = NULL;
CdosGlobal *global = NULL;
gchar *desktop_layout = NULL;
guint menu_stack_depth = 0;

static gboolean software_rendering = FALSE;
static CdosWindowTracker *tracker = NULL;
static ClutterStage *stage = NULL;
static unsigned int modal_count = 0;
static GList *modal_actor_focus_stack = NULL;
static gchar *css_style_sheet = NULL;
static const gchar *default_css_style_sheet = "resource:///org/cdos/desktop/theme/style.css";
static GSettings *cdos_settings = NULL;
static GSettings *desktop_settings = NULL;

// 工作区补充 liting
static gboolean dynamicWorkspaces = FALSE;
static GList *workspaces = NULL;
static guint checkWorkspacesId;
static guint n_workspaces = 0;
#define WORKSPACE_NAME_LENGTH 32
#define WORKSPACE_NUM 4
#define BACKGROUND_SCHEMA "org.cdos.desktop.background"

static gchar **workspace_names = NULL;

typedef struct _CdosUIGroup
{
    StWidget parent;
}CdosUIGroup;

typedef struct _CdosUIGroupClass
{
    StWidgetClass parent_class;
}CdosUIGroupClass;

GType cdos_ui_group_get_type (void);
#define CDOS_TYPE_UI_GROUP cdos_ui_group_get_type()

G_DEFINE_TYPE (CdosUIGroup, cdos_ui_group, ST_TYPE_WIDGET);

static void
cdos_ui_group_allocate (ClutterActor *self,
                        const ClutterActorBox *box,
                        ClutterAllocationFlags flags)
{
    clutter_actor_set_allocation (self, box, flags);

    ClutterActorIter iter;
    ClutterActor *actor;
    clutter_actor_iter_init (&iter, self);

    while (clutter_actor_iter_next (&iter, &actor))
    {
        if (clutter_actor_is_visible (actor))
            clutter_actor_allocate_preferred_size (actor, (ClutterAllocationFlags)flags);
    }
}

static void
cdos_ui_group_get_preferred_width (ClutterActor *self,
                                   gfloat for_width,
                                   gfloat *min_width,
                                   gfloat *nat_width)
{
    gfloat width = clutter_actor_get_width ((ClutterActor *)stage);
    *min_width = width;
    *nat_width = width;
}

static void
cdos_ui_group_get_preferred_height (ClutterActor *self,
                                    gfloat for_height,
                                    gfloat *min_height,
                                    gfloat *nat_height)
{
    gfloat height = clutter_actor_get_height ((ClutterActor *)stage);
    *min_height = height;
    *nat_height = height;
}

static void
cdos_ui_group_init (CdosUIGroup *group)
{
}

static void
cdos_ui_group_class_init (CdosUIGroupClass *klass)
{
    ClutterActorClass *actor_class = CLUTTER_ACTOR_CLASS (klass);
    actor_class->allocate = cdos_ui_group_allocate;
    actor_class->get_preferred_width = cdos_ui_group_get_preferred_width;
    actor_class->get_preferred_height = cdos_ui_group_get_preferred_height;
}

CdosNotification *
cdos_notify_info (ClutterActor *actor, const char *msg, const char *details)
{
    CdosSource *source = cdos_source_new (_("Desktop"));
    CdosNotificationParams *params = g_new0 (CdosNotificationParams, 1);

    if (!actor)
    {
        actor = st_icon_new ();

        st_icon_set_icon_name (ST_ICON (actor), "gtk-dialog-info");
        st_icon_set_icon_size (ST_ICON (actor), (gint) 48);
    }
    params->icon = actor;

    CdosMessageTray *messageTray = cdos_message_tray_get_default ();
    cdos_message_tray_add (messageTray, source);
    CdosNotification *notification = cdos_notification_new (source, msg, details, params);
    cdos_notification_set_transient (notification, FALSE);
    cdos_notification_set_urgency (notification, CDOS_NOTIFICATION_NORMAL);
    cdos_source_notify (source, notification);
    g_free (params);
    return notification;
}

CdosNotification *
cdos_notify_error (const char *msg, const char *details)
{
    g_debug ("global ERROR: %s\n%s\n", msg, details);
    CdosSource *source = cdos_source_new (_("Desktop"));
    CdosNotificationParams *params = g_new0 (CdosNotificationParams, 1);

    const gchar *stock_icon = NULL;
    /*
    gint hints_urgency = CDOS_NOTIFICATION_CRITICAL;
    switch (hints_urgency)
    {
        case CDOS_NOTIFICATION_LOW :
        case CDOS_NOTIFICATION_NORMAL:
            stock_icon = "gtk-dialog-info";
            break;
        case CDOS_NOTIFICATION_CRITICAL :
            stock_icon = "gtk-dialog-error";
            break;
        default:
            stock_icon = "gtk-dialog-info";
            break;
    }
    */
    stock_icon = "gtk-dialog-error";
    ClutterActor *actor = st_icon_new ();

    st_icon_set_icon_name (ST_ICON (actor), stock_icon);
    st_icon_set_icon_size (ST_ICON (actor), (gint) 48);
    params->icon = actor;

    CdosMessageTray *messageTray = cdos_message_tray_get_default ();
    cdos_message_tray_add (messageTray, source);
    CdosNotification *notification = cdos_notification_new (source, msg, details, params);
    cdos_notification_set_urgency (notification, CDOS_NOTIFICATION_CRITICAL);
    cdos_source_notify (source, notification);
    g_free (params);
    return notification;
}

typedef struct _ModalRecord
{
    ClutterActor *actor;
    ClutterActor *prev_focus;
    gulong actor_destroy_id;
    gulong prev_focus_destroy_id;
    CdosActionMode action_mode;
} ModalRecord;

static gint
find_modal (ClutterActor *actor)
{
    gint index = 0;
    GList *focus_stack = NULL;
    focus_stack = modal_actor_focus_stack;
    ModalRecord *record = NULL;

    while (focus_stack != NULL)
    {
        GList *next = focus_stack->next;
        record = focus_stack->data;
        if (record && record->actor == actor)
        {
            return index;
        }
        index++;
        focus_stack = next;
    }
    return -1;
}

static void
on_focus_destroy (ClutterActor *actor, ModalRecord *record)
{
    guint index = find_modal (actor);

    if (index > 0)
    {
        record->prev_focus = NULL;
        g_signal_handler_disconnect (record->prev_focus, record->prev_focus_destroy_id);
        record->prev_focus_destroy_id = 0;
    }
}

static void
_cdos_pop_modal (ClutterActor *actor)
{
    guint focus_index = find_modal (actor);
    if (focus_index >= 0)
        cdos_pop_modal (actor, 0);
}

gint
cdos_get_modal_count (void)
{
    return modal_count;
}

gboolean
cdos_push_modal (ClutterActor *actor,
                 guint timestamp,
                 MetaModalOptions options,
                 CdosActionMode mode)
{
    g_return_val_if_fail (CLUTTER_IS_ACTOR (actor), FALSE);

    MetaDisplay *display = cdos_global_get_display (global);
    if (!timestamp)
        timestamp = cdos_global_get_current_time (global);
    if (modal_count == 0)
    {
        if (!cdos_global_begin_modal (global, timestamp, options))
        {
            g_critical ("%s: %p", __func__, actor);
            return FALSE;
        }
        meta_disable_unredirect_for_display (display);
    }
    ClutterStage *stage = cdos_global_get_stage (global);

    modal_count += 1;
    gulong actor_destroy_id = g_signal_connect (actor, "destroy", G_CALLBACK (_cdos_pop_modal), NULL);

    ModalRecord *record = g_new0 (ModalRecord, 1);
    record->actor = actor;
    record->prev_focus = clutter_stage_get_key_focus (stage);
    record->actor_destroy_id = actor_destroy_id;
    record->action_mode = mode;

    if (record->prev_focus != NULL)
    {
        record->prev_focus_destroy_id =
            g_signal_connect (record->prev_focus, "destroy", G_CALLBACK (on_focus_destroy), record);
    }
    modal_actor_focus_stack = g_list_append (modal_actor_focus_stack, record);
    cdos_set_action_mode (mode);
    clutter_stage_set_key_focus (stage, actor);
    return TRUE;
}

void
cdos_pop_modal (ClutterActor *actor, guint timestamp)
{
    g_return_if_fail (CLUTTER_IS_ACTOR (actor));
    gint focus_index = 0;

    if (!timestamp)
        timestamp = cdos_global_get_current_time (global);
    focus_index = find_modal (actor);

    ClutterStage *stage = cdos_global_get_stage (global);
    if (focus_index < 0)
    {
        clutter_stage_set_key_focus (stage, NULL);
        cdos_global_end_modal (global, timestamp);
        cdos_set_action_mode (CDOS_ACTION_MODE_NORMAL);
        g_debug ("%s %p", __func__, actor);
        return;
    }
    modal_count -= 1;
    ModalRecord *record = g_list_nth_data (modal_actor_focus_stack, focus_index);
    g_signal_handler_disconnect (record->actor, record->actor_destroy_id);

    guint focus_length = g_list_length (modal_actor_focus_stack);
    if (focus_index == focus_length - 1)
    {
        if (record->prev_focus != NULL)
            g_signal_handler_disconnect (record->prev_focus, record->prev_focus_destroy_id);
        cdos_set_action_mode (record->action_mode);
        clutter_stage_set_key_focus (stage, record->prev_focus);
    }
    else
    {
        GList *last_item = g_list_last (modal_actor_focus_stack);
        if (last_item)
        {
            ModalRecord *t = last_item->data;
            if (t->prev_focus)
                g_signal_handler_disconnect (t->prev_focus, t->prev_focus_destroy_id);
        }
        guint i = 0;
        for (i = focus_length - 1; i > focus_index; i--)
        {
            ModalRecord *m = g_list_nth_data (modal_actor_focus_stack, i);
            ModalRecord *n = g_list_nth_data (modal_actor_focus_stack, i - 1);
            m->prev_focus = n->prev_focus;
            m->prev_focus_destroy_id = n->prev_focus_destroy_id;
            m->action_mode = n->action_mode;
        }
    }
    // delete
    modal_actor_focus_stack = g_list_remove (modal_actor_focus_stack, record); // FIXME, only remove first one
    if (modal_count > 0)
        return;
    cdos_global_end_modal (global, timestamp);

    CdosLayoutManager *layout_manager = cdos_layout_manager_get_default ();
    cdos_layout_manager_update_regions (layout_manager);
    MetaDisplay *display = cdos_global_get_display (global);
    meta_enable_unredirect_for_display (display);
    cdos_set_action_mode (CDOS_ACTION_MODE_NORMAL);
}

const gchar *
cdos_get_theme_style_sheet (void)
{
    return css_style_sheet;
}

void
cdos_set_theme_style_sheet (const gchar *css)
{
    g_clear_pointer (&css_style_sheet, g_free);
    if (css == NULL)
        return;
    css_style_sheet = g_strdup (css);
}

void
cdos_load_theme (void)
{
    StThemeContext *context = st_theme_context_get_for_stage (cdos_global_get_stage (global));
    StTheme *theme = st_theme_new (NULL, NULL, NULL);
    gboolean loaded = TRUE;
    GError *err = NULL;
    if (css_style_sheet)
    {
        if (g_file_test (css_style_sheet, G_FILE_TEST_IS_DIR))
        {
            GDir *dir = g_dir_open (css_style_sheet, 0, &err);
            if (err)
            {
                g_warning ("%s %s", __func__, err->message);
                g_clear_error (&err);
            }
            else
            {
                const gchar *entry;
                while ((entry = g_dir_read_name (dir)))
                {
                    if (g_str_has_suffix (entry, ".css"))
                    {
                        gchar *css_file_path = g_strconcat (css_style_sheet, "/", entry, NULL);
                        GFile *css_file = g_file_new_for_path (css_file_path);
                        loaded &= st_theme_load_stylesheet (theme, css_file, NULL);
                        if (!loaded)
                        {
                            g_warning ("There was some problem parsing the theme: %s. Will fallback to the default theme.", css_file_path);
                        }
                        g_free (css_file_path);
                    }
                }
                g_dir_close (dir);
            }
        }
        else
        {
            GFile *css_file = g_file_new_for_path (css_style_sheet);
            loaded = st_theme_load_stylesheet (theme, css_file, NULL);
        }
    }
    if (!loaded || !css_style_sheet)
    {
        static GResource *theme_resource = NULL;
        g_message ("There is not other theme stylesheet files loaded, default theme will be used.");
        if (theme_resource)
        {
            g_resources_unregister (theme_resource);
            g_clear_pointer (&theme_resource, g_resource_unref);
        }
        g_autofree gchar *filename = NULL;
        filename = g_strconcat (CDOS_DATADIR, "/cdos-desktop-theme.gresource", NULL);
        theme_resource = g_resource_load (filename, &err);
        if (err)
        {
            g_warning ("%s %s", __func__, err->message);
            g_clear_error (&err);
        }
        if (theme_resource)
        {
            g_resources_register (theme_resource);
        }
        GFile *css_file = g_file_new_for_uri (default_css_style_sheet);
        st_theme_load_stylesheet (theme, css_file, NULL);
    }
    st_theme_context_set_theme (context, theme);
}

gboolean
cdos_is_software_rendering (void)
{
    return software_rendering;
}

gboolean
cdos_is_interesting (MetaWindow *window)
{
    const char *title = meta_window_get_title (window);
    if (title != NULL)
        if (strcmp (title, "JavaEmbeddedFrame") == 0)
            return false;

    if (cdos_window_tracker_is_window_interesting (window))
        return true;

    if (cdos_window_tracker_get_window_app (tracker, window))
        return false;

    MetaWindowType type = meta_window_get_window_type (window);
    return (type == META_WINDOW_DIALOG) || (type == META_WINDOW_MODAL_DIALOG);
}

static gboolean
win_activate (MetaWindow *window, void *data)
{
    guint32 time = cdos_global_get_current_time (global);

    meta_window_activate (window, time);
    return true;
}
static int
win_foreach_transient (MetaWindow *win)
{
    meta_window_foreach_transient (win, win_activate, 0);
    return 0;
}

void
cdos_activate_window (MetaWindow *window, guint32 time, int workspace_num)
{
    g_return_if_fail (META_IS_WINDOW (window));
    MetaDisplay *display = cdos_global_get_display (global);
    MetaWorkspaceManager *wsm = meta_display_get_workspace_manager (display);

    int active_wks_num = meta_workspace_manager_get_active_workspace_index (wsm);
    int window_wks_num;
    if (workspace_num >= 0)
        window_wks_num = workspace_num;
    else
    {
        MetaWorkspace *wks = meta_window_get_workspace (window);
        window_wks_num = meta_workspace_index (wks);
    }
    if (!time)
        time = cdos_global_get_current_time (global);

    if (window_wks_num != active_wks_num)
    {
        MetaWorkspace *workspace = meta_workspace_manager_get_workspace_by_index (wsm, window_wks_num);
        meta_workspace_activate_with_focus (workspace, window, time);
    }
    else
    {
        meta_window_activate (window, time);
        g_idle_add ((GSourceFunc)win_foreach_transient, window);
    }

    CdosOverview *overview = cdos_overview_get ();
    clutter_actor_hide ((ClutterActor *)overview);
}

GList *
cdos_get_tab_list (MetaWorkspace *workspace_opt, MetaDisplay *display_opt)
{
    MetaDisplay *display = display_opt ? display_opt : cdos_global_get_display (global);
    MetaWorkspaceManager *wsm = meta_display_get_workspace_manager (display);
    MetaWorkspace *workspace = workspace_opt ? workspace_opt : meta_workspace_manager_get_active_workspace (wsm);
    GList *windows = NULL;
    GList *all_windows = meta_display_get_tab_list (display, META_TAB_LIST_NORMAL_ALL, workspace);
    GList *iter = NULL;
    for (iter = all_windows; iter; iter = iter->next)
    {
        MetaWindow *w = iter->data;
        if (cdos_is_interesting (w))
        {
            if (g_list_find (windows, iter->data) == NULL)
            {
                windows = g_list_append (windows, iter->data);
            }
        }
    }
    return windows;
}

GList *
get_window_actors_for_workspace (MetaWorkspace *workspace)
{
    CdosGlobal *global = cdos_global_get ();

    GList *window_actors = NULL;
    GList *windowfilter = NULL;

    window_actors = cdos_global_get_window_actors (global);

    while (window_actors != NULL)
    {
        GList *next = window_actors->next;
        MetaWindowActor *win = window_actors->data;
        MetaWindow *mwin = meta_window_actor_get_meta_window (win);

        if (meta_window_located_on_workspace (mwin, workspace))
        {
            windowfilter = g_list_prepend (windowfilter, win);
        }
        window_actors = next;
    }
    windowfilter = g_list_reverse (windowfilter);
    return windowfilter;
}
// end

// add by liting for workspace :(
typedef struct _WrapWorkspace
{
    MetaWorkspace *workspace;
    MetaWindow *lastRemovedWindow;
    guint windowAddedId;
    guint windowRemovedId;
} WrapWorkspace;

static gboolean
cdos_start_check_workspaces (void)
{
    if (!dynamicWorkspaces)
    {
        return FALSE;
    }
    guint len = g_list_length (workspaces);
    guint i = 0;
    gboolean *empty_workspaces = {FALSE};
    empty_workspaces = g_malloc0 (sizeof (gboolean) * len);
    GList *ws = workspaces;
    while (ws != NULL)
    {
        WrapWorkspace *wworkspace = ws->data;
        MetaWindow *lastRemoved = wworkspace->lastRemovedWindow;
        MetaWindowType type = meta_window_get_window_type (lastRemoved);
        if (lastRemoved
            && (type == META_WINDOW_SPLASHSCREEN || type == META_WINDOW_DIALOG || type == META_WINDOW_MODAL_DIALOG))
        {
            empty_workspaces[i] = FALSE;
        }
        else
        {
            empty_workspaces[i] = TRUE;
        }
        ws = ws->next;
    }

    GList *win_actors = cdos_global_get_window_actors (global);
    MetaDisplay *display = cdos_global_get_display (global);
    MetaWorkspaceManager *wsm = meta_display_get_workspace_manager (display);
    while (win_actors != NULL)
    {
        MetaWindowActor *win = win_actors->data;

        MetaWindow *mwin = meta_window_actor_get_meta_window (win);
        if (meta_window_is_on_all_workspaces (mwin))
        {
            continue;
        }
        guint workspace_index = meta_workspace_manager_get_active_workspace_index (wsm);
        empty_workspaces[workspace_index] = FALSE;
        win_actors = win_actors->next;
    }


    return FALSE;
    // FIXME 写了一半，撒手不管先，先 staticWorkspaces
}

static void
notify_cdos_2d (void)
{

    gchar *buffer = g_strconcat (_("Cdos is currently running without video hardware acceleration and, as a result, you may observe much higher than normal CPU usage.\n\n"),
                                 _("There could be a problem with your drivers or some other issue.  For the best experience, it is recommended that you only use this mode for"),
                                 _(" troubleshooting purposes."),
                                 NULL);
    cdos_notify_error (_("Desktop"), buffer);
    g_free (buffer);
}

static gboolean
queue_check_workspaces (void)
{
    if (!dynamicWorkspaces)
    {
        return FALSE;
    }
    if (checkWorkspacesId == 0)
    {
        checkWorkspacesId =
            meta_later_add (META_LATER_BEFORE_REDRAW, (GSourceFunc)cdos_start_check_workspaces, NULL, NULL);
    }
    return TRUE;
}

typedef struct _TransferWorkspaceWindow
{
    WrapWorkspace *wworkspace;
    MetaWindow *mwin;
} TransferWorkspaceWindow;

static void
onTimeoutRemoved (TransferWorkspaceWindow *transfer)
{
    WrapWorkspace *wworkspace = transfer->wworkspace;

    if (wworkspace->lastRemovedWindow == transfer->mwin)
    {
        wworkspace->lastRemovedWindow = NULL;
        queue_check_workspaces ();
    }
    g_free (transfer);
}

static void
on_window_removed (MetaWorkspace *workspace, MetaWindow *mwin, gpointer data)
{
    WrapWorkspace *wworkspace = (WrapWorkspace *)data;
    wworkspace->lastRemovedWindow = mwin;
    queue_check_workspaces ();

    TransferWorkspaceWindow *transfer = g_malloc0 (sizeof (TransferWorkspaceWindow));
    transfer->wworkspace = wworkspace;
    transfer->mwin = mwin;
    // timeout add
    g_timeout_add (1000, (GSourceFunc)onTimeoutRemoved, transfer);
}
static gboolean
cdos_start_n_workspaces_changed (void)
{
    if (!dynamicWorkspaces)
    {
        return FALSE;
    }
    MetaDisplay *display = cdos_global_get_display (cdos_global_get ());
    MetaWorkspaceManager *wsm = meta_display_get_workspace_manager (display);
    guint oldNumWorkspaces = g_list_length (workspaces);
    guint newNumWorkspaces = meta_workspace_manager_get_n_workspaces (wsm);

    if (oldNumWorkspaces == newNumWorkspaces)
    {
        return FALSE;
    }
    // GList *lostWorkspaces = NULL;
    guint windowAddedId = 0;
    guint windowRemovedId = 0; // 这俩会挪到封装结构体中
    if (newNumWorkspaces > oldNumWorkspaces)
    {
        guint wnum = 0;
        for (wnum = oldNumWorkspaces; wnum < newNumWorkspaces; wnum++)
        {
            workspaces = g_list_append (workspaces, meta_workspace_manager_get_workspace_by_index (wsm, wnum));
            MetaWorkspace *workspace = g_list_nth_data (workspaces, wnum);
            windowAddedId = g_signal_connect (workspace, "window-added", G_CALLBACK (queue_check_workspaces), NULL);

            windowRemovedId = g_signal_connect (workspace, "window-removed", G_CALLBACK (on_window_removed), NULL);
        }
    }
    else
    {
        guint removedIndex = 0;
        guint removedNum = oldNumWorkspaces - newNumWorkspaces;
        guint w = 0;
        for (w = 0; w < oldNumWorkspaces; w++)
        {
            MetaWorkspace *workspace = meta_workspace_manager_get_workspace_by_index (wsm, w);

            if (g_list_nth_data (workspaces, w) != workspace)
            {
                removedIndex = w;
                break;
            }
        }
        for (w = 0; w < removedNum; w++)
        {
            MetaWorkspace *remove = g_list_nth_data (workspaces, removedIndex + w);
            workspaces = g_list_remove (workspaces, remove);
            // lostWorkspaces = g_list_append (lostWorkspaces, remove);
            g_signal_handler_disconnect (remove, windowAddedId);
            g_signal_handler_disconnect (remove, windowRemovedId);
        }
    }
    queue_check_workspaces ();
    return FALSE;
}

static gboolean
cdos_start_static_workspaced (void)
{
    MetaDisplay *display = cdos_global_get_display (cdos_global_get ());
    MetaWorkspaceManager *wsm = meta_display_get_workspace_manager (display);
    gint dif = n_workspaces - meta_workspace_manager_get_n_workspaces (wsm);

    if (dif > 0)
    {
        guint i = 0;
        for (i = 0; i < dif; i++)
        {
            meta_workspace_manager_append_new_workspace (wsm, FALSE, cdos_global_get_current_time (global));
        }
    }
    else
    {
        gint i = 0;
        if (n_workspaces == 0)
        {
            return FALSE;
        }
        for (i = 0; i > dif; i--)
        {
            guint removeWorkspaceIndex = meta_workspace_manager_get_n_workspaces (wsm) - 1;
            MetaWorkspace *removeW = meta_workspace_manager_get_workspace_by_index (wsm, removeWorkspaceIndex);
            meta_workspace_manager_remove_workspace (wsm, removeW, cdos_global_get_current_time (global));
        }
    }
    return TRUE;
}

static gchar *
make_default_workspace_name (int index)
{
    return g_strdup_printf ("%s %d", _("Workspace"), index + 1);
}

static void
cdos_trim_workspace_names (int index)
{
    guint len = g_strv_length (workspace_names);
    gint i = 0;

    guint new_len = len;

    for (i = len - 1; i >= 0 && (i >= n_workspaces || !strlen (workspace_names[i])); i--)
    {
        g_free (workspace_names[i]);
        new_len--;
    }
    workspace_names = g_renew (gchar *, workspace_names, new_len + 1);
    workspace_names[new_len] = NULL;

    cdos_debug ("len is %d, new_len: %d\n", len, new_len);
}

static gchar *
c_strdup (const char *str)
{
    if (str == NULL)
        return NULL;
    if (strlen (str) > 0)
        return g_strdup (str);
    char *new_str = g_malloc0 (1);
    return new_str;
}

// fill 本是为了填充空位， overrides 值为 "" 时，显示默认名
static void
cdos_fill_workspace_names (int index)
{
    guint len = g_strv_length (workspace_names);
    cdos_debug ("len in fill : %d, index: %d\n", len, index);

    if (index >= len)
    {
        if (workspace_names == NULL)
            workspace_names = g_new0 (gchar *, index + 2);
        else
        {
            // gchar **temp = NULL;
            // temp = g_renew (gchar *, workspace_names, index + 2);
            // g_strfreev(workspace_names);
            // workspace_names = temp;
            workspace_names = g_renew (gchar *, workspace_names, index + 2);
        }
        workspace_names[index + 1] = NULL;
        gint i = 0;
        for (i = index; i >= len && i >= 0; --i)
        {
            workspace_names[i] = c_strdup ("");
        }
    }
}

void
cdos_set_workspace_name (guint index, const gchar *name)
{
    // trim name
    const gchar *ws_name = cdos_get_workspace_name (index);
    if (g_strcmp0 (name, ws_name) != 0)
    {
        // g_free (ws_name);
        cdos_fill_workspace_names (index);

        gchar *default_name = make_default_workspace_name (index);
        if (g_strcmp0 (name, default_name) != 0)
        {
            workspace_names[index] = g_strdup (name);
        }
        else
        {
            workspace_names[index] = c_strdup ("");
        }
        // g_free (default_name);

        cdos_trim_workspace_names (index);
        GSettings *settings = cdos_global_get_settings (global);

        cdos_debug ("%d %d", index, g_strv_length (workspace_names));

        g_settings_set_strv (settings, "workspace-name-overrides", (const char *const *)workspace_names);
    }
}

/**
 *
 *
 */
const gchar *
cdos_get_workspace_name (guint index)
{
    guint len = g_strv_length (workspace_names);
    gchar *wsName = index < len ? workspace_names[index] : NULL;

    return (wsName && strlen (wsName) > 0) ? g_strdup (wsName) : make_default_workspace_name (index);
}

gboolean
cdos_start_add_workspace (void)
{
    if (dynamicWorkspaces || (n_workspaces >= 12))
    {
        if (n_workspaces >= 12)
        {
            // comfirm
            //
        }
        return FALSE;
    }
    n_workspaces++;
    GSettings *settings = cdos_global_get_settings (global);
    g_settings_set_int (settings, "number-workspaces", n_workspaces);
    cdos_start_static_workspaced ();
    return TRUE;
}

static void
cdos_start_init_recorder (void)
{
    // TODO
    g_debug ("%s: TODO", __func__);
}

static void
cdos_start_init_user_session (void)
{
    cdos_start_init_recorder ();
    MetaDisplay *display = cdos_global_get_display (cdos_global_get ());
    MetaWorkspaceManager *wsm = meta_display_get_workspace_manager (display);
    meta_workspace_manager_override_workspace_layout (wsm, META_DISPLAY_TOPLEFT, FALSE, 1, -1);
    CdosSystrayManager *systray_manager = cdos_systray_manager_new ();
    cdos_ui_plugin_set_systray_manager (systray_manager);
}

static void
hide_panel_in_fullscreen (MetaDisplay *display, gpointer data)
{
    gint monitor = cdos_layout_manager_get_index ();
    gint current_monitor = meta_display_get_current_monitor (display);
    gint primary_monitor = meta_display_get_primary_monitor (display);

    if (meta_display_get_monitor_in_fullscreen (display, monitor))
    {
        if (primary_monitor == current_monitor)
        {
            cdos_start_disable_panels ();
            return;
        }
    }
    cdos_start_enable_panels ();
}

static void
on_desktop_layout_changed (GSettings *settings,
                           GParamSpec *psepc,
                           gpointer data)
{
    desktop_layout = g_settings_get_string (settings, "desktop-layout");
    cdos_ui_plugin_set_desktop_layout (desktop_layout);

    StSide side = ST_SIDE_BOTTOM;
    CdosPanel *panel = cdos_ui_plugin_get_panel ();

    if (g_strcmp0 (desktop_layout, LAYOUT_FLIPPED) == 0)
    {
        cdos_ui_plugin_set_applet_side (ST_SIDE_TOP);
        st_widget_remove_style_class_name ((StWidget *)panel, "panel-bottom");
        st_widget_add_style_class_name ((StWidget *)panel, "panel-top");
        cdos_panel_set_bottom_position (panel, FALSE);
        side = ST_SIDE_TOP;
    }
    else /* if (g_strcmp0 (desktop_layout, LAYOUT_CLASSIC) == 0) */
    {
        cdos_ui_plugin_set_applet_side (ST_SIDE_BOTTOM);
        st_widget_remove_style_class_name ((StWidget *)panel, "panel-top");
        st_widget_add_style_class_name ((StWidget *)panel, "panel-bottom");
        cdos_panel_set_bottom_position (panel, TRUE);
    }
    cdos_applet_manager_set_applets_orientation (side);
    cdos_layout_manager_update_boxes (cdos_layout_manager_get_default ());
}

static void
update_scale_factor (void)
{
    cdos_layout_manager_update_boxes (cdos_layout_manager_get_default ());
}


void
cdos_start_center_pointer_on_screen (CdosLayoutManager *layout)
{
    ClutterBackend *backend = clutter_get_default_backend ();
    ClutterSeat *seat = clutter_backend_get_default_seat (backend);
    const MetaRectangle *primary = cdos_layout_manager_get_primary_monitor (layout);
    if (primary && seat)
        clutter_seat_warp_pointer (seat, primary->x + primary->width / 2, primary->y + primary->height / 2);
}

static gboolean 
start_ui_delay (void)
{
    display_current_time ("GUI start delay");
    cdos_sound_manager_new ();
    cdos_message_tray_get_default ();
    cdos_overview_new ();

    /* TODO: auto_mount_manager: 50ms*/
    cdos_auto_mount_manager_new ();

    cdos_start_init_user_session ();

    cdos_screen_saver_new ();
    cdos_power_dialog_new ();

    cdos_keyboard_manager_new ();
    cdos_magnifier_new();

    cdos_login_manager_new ();
    cdos_session_new ();

    return FALSE;
}

int
cdos_plugin_start (CdosGlobal *main_plugin_global)
{
    display_current_time ("GUI start");
    CdosPanel *panel = NULL;

    bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
    textdomain (GETTEXT_PACKAGE);

    g_debug ("%s %s", GETTEXT_PACKAGE, LOCALEDIR);

    cdos_ui_plugin_set_applet_side (ST_SIDE_BOTTOM);

    global = cdos_global_get ();

    stage = cdos_global_get_stage (global);
    clutter_actor_show ((ClutterActor *)stage);

    g_setenv ("XDG_CURRENT_DESKTOP", "CDOS", TRUE);

    MetaDisplay *display = cdos_global_get_display (global);
    tracker = cdos_window_tracker_get_default ();
    cdos_app_system_get_default ();

    GSettings *settings = cdos_global_get_settings (global);

    g_object_new (CDOS_TYPE_THEME_MANAGER, NULL);

    //stage 各层
    ClutterActor *ui_group = g_object_new (CDOS_TYPE_UI_GROUP, NULL);
    clutter_actor_set_flags (ui_group, CLUTTER_ACTOR_NO_LAYOUT);
    st_set_ui_root ((ClutterStage *)stage, ui_group);
    cdos_ui_plugin_set_ui_group (ui_group);
    clutter_actor_add_child ((ClutterActor *)stage, (ClutterActor *)ui_group);
    clutter_actor_set_child_above_sibling ((ClutterActor *)stage, ui_group, NULL);

    ClutterActor *window_group = NULL;
    window_group = cdos_global_get_window_group (global);
    g_object_ref (window_group);
    clutter_actor_remove_child ((ClutterActor *)stage, window_group);
    clutter_actor_add_child (ui_group, window_group);

    ClutterActor *background_group = meta_background_group_new ();
    clutter_actor_add_child (window_group, background_group);
    clutter_actor_set_child_below_sibling (window_group, background_group, NULL);
    cdos_background_manager_new (background_group, 0, 0, FALSE, BACKGROUND_SCHEMA);

    CdosLayoutManager *layout_manager = cdos_layout_manager_get_default ();

    StWidget *panelbox = cdos_layout_manager_get_panelbox (layout_manager);
    clutter_actor_set_reactive ((ClutterActor *)panelbox, TRUE);

    ClutterInputMethod *input_method = cdos_input_method_new ();
    clutter_backend_set_input_method (clutter_get_default_backend (), input_method);

    panel = cdos_panel_new (TRUE, TRUE);
    cdos_ui_plugin_set_panel (panel);

    cdos_panel = (ClutterActor *)panel;
    clutter_actor_set_reactive (cdos_panel, TRUE);
    clutter_actor_add_child ((ClutterActor *)panelbox, (ClutterActor *)cdos_panel);
    g_signal_connect (display, "in-fullscreen-changed", G_CALLBACK (hide_panel_in_fullscreen), layout_manager);
    g_signal_connect (cdos_panel, "button-release-event", G_CALLBACK (cdos_panel_on_button_press), NULL);

    g_signal_connect (settings, "changed::desktop-layout", G_CALLBACK(on_desktop_layout_changed), NULL);
    on_desktop_layout_changed (settings, NULL, NULL);

    g_signal_connect (gtk_settings_get_default (), "notify::gtk-xft-dpi",
                      G_CALLBACK (update_scale_factor), global);

    ClutterActor *feedback_group = NULL;
    feedback_group = meta_get_feedback_group_for_display (display);
    g_object_ref (feedback_group);
    clutter_actor_remove_child ((ClutterActor *)stage, feedback_group);
    clutter_actor_add_child (ui_group, feedback_group);

    //右键菜单，包括输入法菜单等位于 top window group
    ClutterActor *top_window_group = NULL;
    top_window_group = cdos_global_get_top_window_group (global);
    g_object_ref (top_window_group);
    clutter_actor_remove_child ((ClutterActor *)stage, top_window_group);
    clutter_actor_add_child (ui_group, top_window_group);

    //cdos_indicator_manager_get_default ();
    CdosSystrayManager *systray = cdos_systray_manager_new ();
    cdos_ui_plugin_set_systray_manager (systray);
    cdos_icon_dispatcher_get ();

    g_object_new (CDOS_TYPE_NOTIFICATION_DAEMON, NULL);

    cdos_layout_manager_update_regions (layout_manager);

    CdosBaseWM *cdos_base_wm = NULL;
    cdos_base_wm = cdos_global_get_window_manager (global);
    cdos_ui_plugin_set_cdos_base_wm (cdos_base_wm);

    cdos_wm_new (cdos_base_wm);

    cdos_set_action_mode (CDOS_ACTION_MODE_NORMAL);

    // 工作区
    n_workspaces = g_settings_get_int (settings, "number-workspaces");
    dynamicWorkspaces = FALSE;

    if (!dynamicWorkspaces)
    {
        cdos_start_static_workspaced ();
    }

    workspace_names = g_settings_get_strv (settings, "workspace-name-overrides");

    MetaWorkspaceManager *wsm = meta_display_get_workspace_manager (display);
    g_signal_connect (wsm, "notify::n-workspaces", G_CALLBACK (cdos_start_n_workspaces_changed), NULL);

    cdos_start_n_workspaces_changed ();

    meta_register_with_session ();

    cdos_applet_manager_init ();

    /*
    clutter_actor_add_child (CLUTTER_ACTOR (stage), CLUTTER_ACTOR (screen_saver));
    */

    /*
    clutter_actor_set_opacity (ui_group, 128);
    clutter_actor_set_pivot_point (ui_group, 0.5, 0.5);
    clutter_actor_set_scale (ui_group, 0, 0);
    ClutterTransition *transition;
    transition = cdos_transition_simple (ui_group,
                                         CLUTTER_EASE_OUT_QUAD,
                                         300,
                                         "opacity", 255,
                                         "scale-x", 1.0,
                                         "scale-y", 1.0,
                                         NULL);
    g_signal_connect (transition, "completed", G_CALLBACK (cdos_ui_group_show_done), layout_manager);
    */

    g_idle_add (start_ui_delay, NULL);

    if (g_getenv ("CDOS_SOFTWARE_RENDERING"))
    {
        g_debug ("Cdos Software Rendering mode enabled");
        software_rendering = true;
        notify_cdos_2d ();
    }
    display_current_time ("GUI end");
    return 0;
}

void
cdos_start_enable_panels (void)
{
    CdosPanel *panel;
    StWidget *dock;

    panel = cdos_ui_plugin_get_panel ();
    dock = cdos_ui_plugin_get_dock ();

    if (panel)
    {
        cdos_panel_enable (panel, 250);
    }
    if (dock)
    {
        /* TODO */
    }
}

void
cdos_start_disable_panels (void)
{
    CdosPanel *panel;
    StWidget *dock;

    panel = cdos_ui_plugin_get_panel ();
    dock = cdos_ui_plugin_get_dock ();
    if (panel)
    {
        cdos_panel_disable (panel, 0);
    }
    if (dock)
    {
        /* TODO */
    }
}

GSettings *
cdos_get_cdos_settings (void)
{
    if (!cdos_settings)
    {
        cdos_settings = g_settings_new ("org.cdos");
    }
    return cdos_settings;
}

GSettings *
cdos_get_desktop_settings (void)
{
    if (!desktop_settings)
    {
        desktop_settings = g_settings_new ("org.cdos.desktop");
    }
    return desktop_settings;
}
