#ifndef __CDOS_START_H__
#define __CDOS_START_H__

#include <stdlib.h>
//#include <libintl.h>
#include <stdio.h>
#include <stdint.h>

#include <meta/util.h>
#include <meta/display.h>
#include <meta/workspace.h>
#include <meta/meta-workspace-manager.h>

#include "cdos-global.h"
#include "cdos-wm.h"
#include "cdos-window-tracker.h"
#include "cdos-overview.h"
#include "cdos-ui-plugin.h"
#include "cdos-transition.h"
#include "cdos-notification.h"
#include "cdos-layout-manager.h"

extern CdosGlobal *global;
extern ClutterActor *ui_group;
extern ClutterActor *cdos_panel;
extern gchar *desktop_layout;
extern guint menu_stack_depth;

#define DEFAULT_BACKGROUND_COLOR clutter_color_new (0x22, 0x66, 0xbb, 0xff)
#define LAYOUT_TRADITIONAL "traditional"
#define LAYOUT_FLIPPED "flipped"
#define LAYOUT_CLASSIC "classic"

int cdos_plugin_start (CdosGlobal *cdos_plugin_global);

void cdos_load_theme (void);
const gchar *cdos_get_theme_style_sheet (void);
void cdos_set_theme_style_sheet (const gchar *css);

void cdos_windows_restacked (void);
void cdos_window_entered_monitor (MetaDisplay *display, int monitor_index, MetaWindow *meta_win);
void cdos_window_left_monitor (MetaDisplay *display, int monitor_index, MetaWindow *meta_win);

gint cdos_get_modal_count (void);

gboolean cdos_push_modal (ClutterActor *actor,
                          guint timestamp,
                          MetaModalOptions options,
                          CdosActionMode mode);

void cdos_pop_modal (ClutterActor *actor, guint timestamp);

gboolean cdos_is_software_rendering (void);

gboolean cdos_is_interesting (MetaWindow *window);

GList *get_window_actors_for_workspace (MetaWorkspace *workspace);

void cdos_activate_window (MetaWindow *window, guint32 time, int workspace_num);

GList *cdos_get_tab_list (MetaWorkspace *workspace_opt, MetaDisplay *display);

const gchar *cdos_get_workspace_name (guint index);
void cdos_set_workspace_name (guint index, const gchar *name);

gboolean cdos_start_add_workspace (void);

void cdos_start_enable_panels (void);
void cdos_start_disable_panels (void);

CdosNotification *cdos_notify_info (ClutterActor *actor, const char *msg, const char *details);
CdosNotification *cdos_notify_error (const char *msg, const char *details);

GSettings *cdos_get_cdos_settings (void);
GSettings *cdos_get_desktop_settings (void);

void cdos_start_center_pointer_on_screen (CdosLayoutManager *layout);

#endif // __CDOS_STRAT_H__
