#include <glib.h>
#include "cdos-switchs.h"

G_DEFINE_TYPE (CdosSwitchs, cdos_switchs, ST_TYPE_BIN);

enum
{
    PROP_0,
    PROP_STATE
};

static void
cdos_switchs_set_property (GObject *self, guint prop_id, const GValue *value, GParamSpec *pspec)
{
    switch (prop_id)
    {
        case PROP_STATE:
            cdos_switchs_set_toggle_state ((CdosSwitchs *)self, g_value_get_boolean (value));
            break;
        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID (self, prop_id, pspec);
    }
}

static void
cdos_switchs_get_property (GObject *gobject, guint prop_id, GValue *value, GParamSpec *pspec)
{
    CdosSwitchs *self = CDOS_SWITCHS (gobject);

    switch (prop_id)
    {
        case PROP_STATE:
            g_value_set_boolean (value, self->state);
            break;
        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
    }
}

/**
 * cdos_switchs_class_init:
 *
 * init the switchs class.
 *
 * Return value: None
 */
static void
cdos_switchs_class_init (CdosSwitchsClass *klass)
{
    GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
    gobject_class->set_property = cdos_switchs_set_property;
    gobject_class->get_property = cdos_switchs_get_property;
    GParamSpec *pspec;
    pspec = g_param_spec_boolean ("state", "State", "The state of the switch", FALSE, G_PARAM_READWRITE);
    g_object_class_install_property (gobject_class, PROP_STATE, pspec);
}

/**
 * cdos_switchs_init:
 *
 * init the switchs instance.
 *
 * Return value: None
 */
static void
cdos_switchs_init (CdosSwitchs *self)
{
    st_widget_set_style_class_name (ST_WIDGET (self), "toggle-switch");
    st_widget_add_style_class_name (ST_WIDGET (self), "toggle-switch-us");
    st_widget_set_track_hover (ST_WIDGET (self), TRUE);
    clutter_actor_set_reactive (CLUTTER_ACTOR (self), TRUE);
}

/**
 * cdos_switchs_new:
 *
 * 新建一个 #CdosSwitchs 类的实例。
 *
 * Returns: 返回一个 #GObject 的实例。
 */
CdosSwitchs *
cdos_switchs_new (gboolean state)
{
    return g_object_new (CDOS_TYPE_SWITCHS, "state", state, NULL);
}

/**
 * cdos_switchs_set_toggle_state:
 * @m_switch: A #CdosSwitchs
 * @state: switch的状态
 *
 * 设置switch的切换状态
 *
 */
void
cdos_switchs_set_toggle_state (CdosSwitchs *self, gboolean state)
{
    g_return_if_fail (CDOS_IS_SWITCHS (self));

    if (self->state == state)
        return;

    if (state)
    {
        st_widget_add_style_pseudo_class (ST_WIDGET (self), "checked");
    }
    else
    {
        st_widget_remove_style_pseudo_class (ST_WIDGET (self), "checked");
    }
    self->state = state;
}

/**
 * cdos_switchs_toggle:
 * @m_switch: A #CdosSwitchs
 *
 * 切换switch的状态
 *
 */
void
cdos_switchs_toggle (CdosSwitchs *m_switch)
{
    g_return_if_fail (CDOS_IS_SWITCHS (m_switch));
    cdos_switchs_set_toggle_state (m_switch, !m_switch->state);
}

/**
 * cdos_switchs_set_state:
 * @m_switch: A #CdosSwitchs
 * @state: switch的状态
 *
 * 设置switch的状态
 *
 */
void
cdos_switchs_set_state (CdosSwitchs *m_switch, gboolean state)
{
    g_return_if_fail (CDOS_IS_SWITCHS (m_switch));
    cdos_switchs_toggle (m_switch);
}

/**
 * cdos_switchs_get_state:
 * @m_switch: A #CdosSwitchs
 *
 * 获取switch的state
 *
 * Returns: A #gboolean
 */
gboolean
cdos_switchs_get_state (CdosSwitchs *m_switch)
{
    g_return_val_if_fail (CDOS_IS_SWITCHS (m_switch), FALSE);
    return m_switch->state;
}
