#include "cdos-text-icon-applet.h"
#include <clutter/clutter.h>

/**
 * SECTION:cdos-text-icon-applet
 * @Title: CdosTextIconApplet
 * @Short_description: 新建text_icon_applet及设置applet标签
 *
 * 新建一个text_icon_applet以及在书签中显示文本并把文本的actor设置到text中,隐藏applet的图标
 */
G_DEFINE_TYPE (CdosTextIconApplet, cdos_text_icon_applet, CDOS_TYPE_ICONAPPLET);

static void
cdos_text_icon_applet_init (CdosTextIconApplet *self)
{
    StLabel *applet_label = g_object_new (ST_TYPE_LABEL,
                                "reactive", TRUE,
                                "track-hover", TRUE,
                                "style-class", "applet-label",
                                NULL);
    self->applet_label = applet_label;
    ClutterActor *clutter_text = st_label_get_clutter_text (applet_label);
    clutter_text_set_ellipsize (CLUTTER_TEXT(clutter_text), PANGO_ELLIPSIZE_NONE);

    ClutterActor *applet_actor = CLUTTER_ACTOR (self);
    clutter_actor_add_child (CLUTTER_ACTOR(applet_actor), CLUTTER_ACTOR(applet_label));

    clutter_actor_queue_relayout (CLUTTER_ACTOR (self));
}

static void
cdos_text_icon_applet_class_init (CdosTextIconAppletClass *klass)
{
}

/**
 * cdos_text_icon_applet_hide_applet_icon
 * @text_icon_applet: 文本的applet
 *
 * 隐藏applet的图标
 *
 * Returns: void
 */
void
cdos_text_icon_applet_hide_applet_icon (CdosTextIconApplet *text_icon_applet)
{
  g_return_if_fail (CDOS_IS_TEXT_ICON_APPLET (text_icon_applet));
  StBin *bin = cdos_iconapplet_get_applet_icon_box (CDOS_ICONAPPLET (text_icon_applet));
  st_bin_set_child (bin, NULL);
}

/**
 * cdos_text_icon_applet_set_applet_label
 * @text_icon_applet: 文本的applet
 * @text: 文本
 *
 * 在书签中显示文本，设置文本的actor到文本中
 *
 * Returns： void
 */
void
cdos_text_icon_applet_set_applet_label (CdosTextIconApplet *text_icon_applet, const gchar *text)
{
    g_return_if_fail (CDOS_IS_TEXT_ICON_APPLET (text_icon_applet));
    st_label_set_text (text_icon_applet->applet_label, text);
    ClutterActor *child = st_bin_get_child (cdos_iconapplet_get_applet_icon_box (CDOS_ICONAPPLET (text_icon_applet)));
    if (text && !g_str_equal(text,"") && child != NULL){
        clutter_actor_set_margin_left (CLUTTER_ACTOR (text_icon_applet->applet_label), 6.0);
    } else {
        clutter_actor_set_margin_left (CLUTTER_ACTOR (text_icon_applet->applet_label), 0);
    }
}

/**
 * cdos_text_icon_applet_new
 * @orientation: 文本的方向
 * @panel_height：panel的方向
 * @instance_id: A #gchar
 *
 * 新建一个text_icon_applet
 *
 * Returns: text_icon_applet
 */
CdosTextIconApplet *
cdos_text_icon_applet_new (guint orientation,
			               gfloat panel_height,
                           const gchar *instance_id)
{
  CdosTextIconApplet *text_icon_applet = g_object_new (CDOS_TYPE_TEXT_ICON_APPLET,
				"orientation", orientation,
				"instance-id", instance_id,
				NULL);
  return text_icon_applet;
}
