/*
 * Created:
 *   2019年02月18日 10时19分58秒
 *
 * SECTION:  cdos-tile-preview.c
 * @Title: cdos-tile-preview.c
 * @Short_Description:
 */
#include <stdlib.h>
#include "cdos-tile-preview.h"
#include "cdos-global.h"
#include "cdos-layout-manager.h"
#include "cdos-transition.h"

struct _CdosTilePreviewPrivate
{
    gint monitor;
    MetaRectangle rect;
    gboolean showing;
};

G_DEFINE_TYPE_WITH_PRIVATE (CdosTilePreview, cdos_tile_preview, ST_TYPE_WIDGET);

static void
cdos_tile_preview_init (CdosTilePreview *self)
{
    self->priv = cdos_tile_preview_get_instance_private (self);

    self->priv->monitor = -1;
    self->priv->rect = (MetaRectangle){0, 0, 0, 0};
    self->priv->showing = FALSE;

    clutter_actor_hide ((ClutterActor *)self);
    ClutterActor *group = cdos_global_get_window_group (cdos_global_get ());
    clutter_actor_add_child (group, (ClutterActor *)self);
}

static void
cdos_tile_preview_class_init (CdosTilePreviewClass *klass)
{
}

void
cdos_tile_preview_show (CdosTilePreview *self, MetaWindow *window, MetaRectangle *rect, gint monitor)
{
    g_return_if_fail (CDOS_IS_TILE_PREVIEW (self));
    CdosTilePreviewPrivate *priv = self->priv;

    ClutterActor *window_actor = (ClutterActor *)meta_window_get_compositor_private (window);
    if (!window_actor)
        return;

    CdosGlobal *global = cdos_global_get ();
    ClutterActor *actor = CLUTTER_ACTOR (self);

    ClutterActor *window_group = cdos_global_get_window_group (global);
    clutter_actor_set_child_below_sibling (window_group, actor, window_actor);

    if (meta_rectangle_equal (&priv->rect, rect))
        return;
    priv->rect = *rect;

    gboolean change_monitor = (priv->monitor == -1) || (priv->monitor != monitor);
    priv->monitor = monitor;

    MetaRectangle layout_monitor;
    /*
    layout_monitor = cdos_layout_manager_get_monitors (cdos_layout_manager_get_default ())->pdata[monitor];
    */
    MetaDisplay *display = cdos_global_get_display (global);
    meta_display_get_monitor_geometry (display, monitor, &layout_monitor);

    if (!clutter_actor_is_visible (actor) || change_monitor)
    {
        MetaRectangle window_rect, dest;
        meta_window_get_frame_rect (window, &window_rect);
        gboolean exist = meta_rectangle_intersect (&layout_monitor, &window_rect, &dest);
        if (!exist)
            return;
        clutter_actor_set_size (actor, dest.width, dest.height);
        clutter_actor_set_position (actor, dest.x, dest.y);
        clutter_actor_set_opacity (actor, 0);
    }

    priv->showing = TRUE;
    clutter_actor_show (actor);
    /*
    ClutterColor color = (ClutterColor){0, 50, 255, 255};
    clutter_actor_set_background_color (actor, &color);
    */
    cdos_transition_simple (actor,
                            CLUTTER_EASE_OUT_QUAD,
                            250,
                            "x", (float)rect->x,
                            "y", (float)rect->y,
                            "width", (float)rect->width,
                            "height", (float)rect->height,
                            "opacity", 192,
                            NULL);
}

static void
on_hide_completed (ClutterTimeline *tml, CdosTilePreview *self)
{
    g_signal_handlers_disconnect_by_data (tml, self);
    CdosTilePreviewPrivate *priv = self->priv;
    if (priv->showing)
        return;

    ClutterActor *actor = CLUTTER_ACTOR (self);

    priv->monitor = -1;
    priv->rect = (MetaRectangle){0, 0, 0, 0};
    clutter_actor_hide (actor);
}

void
cdos_tile_preview_hide (CdosTilePreview *self)
{
    g_return_if_fail (CDOS_IS_TILE_PREVIEW (self));
    if (!clutter_actor_is_visible ((ClutterActor *)self))
        return;

    self->priv->showing = FALSE;
    ClutterTransition *transition =
        cdos_transition_simple ((ClutterActor *)self, CLUTTER_EASE_OUT_QUAD, 250, "opacity", 0, NULL);
    g_signal_connect (transition, "completed", G_CALLBACK (on_hide_completed), self);
}

ClutterActor *
cdos_tile_preview_new (void)
{
    return (ClutterActor *)g_object_new (CDOS_TYPE_TILE_PREVIEW, NULL);
}
