#include "cdos-start.h"
#include "cdos-layout-manager.h"
#include "cdos-ui-plugin.h"
#include "cdos-tooltip.h"

/**
 * SECTION:cdos-tooltip
 * @title: CdosTooltip
 * @short_description: 用于显示应用程序的描述信息
 *
 * CdosTooltip 当鼠标hover到应用程序时，弹出含有该应用描述信息的小框，这个小框就是tooltip。
 */

struct _CdosTooltipPrivate
{
    gboolean prevent_show;
    gulong actor_destroy_id;
    guint  show_timeout_id; 
};

G_DEFINE_TYPE_WITH_PRIVATE (CdosTooltip, cdos_tooltip, ST_TYPE_LABEL);

enum
{
    PROP_0,
    PROP_ITEM
};

static gboolean
cdos_tooltip_item_on_enter (ClutterActor *gobject, ClutterEvent *event, gpointer user_data)
{
    CdosTooltip *self = CDOS_TOOLTIP (user_data);
    clutter_event_get_coords (event, &self->mousePositionX, &self->mousePositionY);
    cdos_tooltip_show (self);
    return FALSE;
}

static gboolean
cdos_tooltip_item_on_leave (ClutterActor *gobject, ClutterEvent *event, gpointer user_data)
{
    CdosTooltip *self = CDOS_TOOLTIP (user_data);
    cdos_tooltip_hide (self);
    return FALSE;
}

static gboolean
cdos_tooltip_item_on_motion (ClutterActor *gobject, ClutterEvent *event, gpointer user_data)
{
    CdosTooltip *self = CDOS_TOOLTIP (user_data);
    if (!self->visible)
    {
        cdos_tooltip_show (self);
        clutter_event_get_coords (event, &self->mousePositionX, &self->mousePositionY);
    }
    return FALSE;
}

static gboolean
cdos_tooltip_item_on_button_release (ClutterActor *gobject, ClutterEvent *event, gpointer user_data)
{
    CdosTooltip *self = CDOS_TOOLTIP (user_data);
    cdos_tooltip_hide (self);
    return FALSE;
}

static gboolean
idle_cb (gpointer user_data)
{
    CdosTooltip *self = CDOS_TOOLTIP (user_data);
    cdos_tooltip_hide (self);
    return FALSE;
}

static void
cdos_tooltip_item_on_allocation_changed (ClutterActor *gobject,
                                         ClutterActorBox *box,
                                         ClutterAllocationFlags flags,
                                         gpointer user_data)
{
    CdosTooltip *self = CDOS_TOOLTIP (user_data);
    // An allocation change could mean that the actor has moved,
    // so hide, but wait until after the allocation cycle.
    if (0)
        g_idle_add ((GSourceFunc)idle_cb, self);
}

static gboolean
cdos_tooltip_item_on_button_press (ClutterActor *gobject, ClutterEvent *event, gpointer user_data)
{
    // CdosTooltip *self = CDOS_TOOLTIP (user_data);
    return FALSE;
}

static void
cdos_tooltip_item_on_destroy (ClutterActor *gobject, CdosTooltip *self)
{
    clutter_actor_destroy ((ClutterActor *)self);
}
//固定的函数

static void
cdos_tooltip_finalize (GObject *object)
{
    // CdosTooltip *self = CDOS_TOOLTIP(object);
    // CdosTooltipPrivate *priv = self->priv;
    G_OBJECT_CLASS (cdos_tooltip_parent_class)->finalize (object);
}

static void
cdos_tooltip_dispose (GObject *object)
{
    CdosTooltip *self = CDOS_TOOLTIP (object);
    // CdosTooltipPrivate *priv = self->priv;
    if (self->priv->show_timeout_id > 0)
    {
        g_source_remove (self->priv->show_timeout_id);
        self->priv->show_timeout_id = 0;
    }
    cdos_tooltip_hide (self);
    if (self->priv->actor_destroy_id)
    {
        g_signal_handler_disconnect (self->item, self->priv->actor_destroy_id);
        self->priv->actor_destroy_id = 0;
    }
    G_OBJECT_CLASS (cdos_tooltip_parent_class)->dispose (object);
}

static void
set_item (CdosTooltip *self, ClutterActor *item)
{
    self->item = item;
    g_signal_connect (item, "enter-event", G_CALLBACK (cdos_tooltip_item_on_enter), self);
    g_signal_connect (item, "leave-event", G_CALLBACK (cdos_tooltip_item_on_leave), self);
    //g_signal_connect (item, "motion-event", G_CALLBACK (cdos_tooltip_item_on_motion), self);
    g_signal_connect (item, "button-press-event", G_CALLBACK (cdos_tooltip_item_on_button_press), self);
    g_signal_connect (item, "button-release-event", G_CALLBACK (cdos_tooltip_item_on_button_release), self);
    self->priv->actor_destroy_id = g_signal_connect (item, "destroy", G_CALLBACK (cdos_tooltip_item_on_destroy), self);
    g_signal_connect (item, "allocation-changed", G_CALLBACK (cdos_tooltip_item_on_allocation_changed), self);
}

static void
cdos_tooltip_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
{
    CdosTooltip *tooltip = CDOS_TOOLTIP (object);
    switch (prop_id)
    {
        case PROP_ITEM:
        {
            ClutterActor *item = g_value_get_object (value);
            set_item (tooltip, item);
            break;
        }
        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
            break;
    }
}

static void
cdos_tooltip_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec)
{
    CdosTooltip *tooltip = CDOS_TOOLTIP (object);
    switch (prop_id)
    {
        case PROP_ITEM:
            g_value_set_object (value, tooltip->item);
            break;
        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
            break;
    }
}

static void
cdos_tooltip_set_position_real (CdosTooltip *self)
{
    ClutterActorBox box;
    clutter_actor_get_allocation_box (CLUTTER_ACTOR (self), &box);
    gfloat tooltipWidth = box.x2 - box.x1;
    gfloat tooltipHeight = box.y2 - box.y1;

    CdosLayoutManager *layout_manager = cdos_layout_manager_get_default ();
    //调用layout manager中的 findMonitorForActor, 尚没有此接口
    const MetaRectangle *monitor; // = Main.layoutManager.findMonitorForActor(this._item);
    monitor = cdos_layout_manager_get_primary_monitor(layout_manager);
    CdosPanel *panel = cdos_ui_plugin_get_panel ();
    guint panel_height = cdos_panel_get_height (panel);

    self->mousePositionY = monitor->height - panel_height - tooltipHeight;

    gfloat tooltipTop = self->mousePositionY; //[1];
    gfloat tooltipLeft = self->mousePositionX - tooltipWidth / 2; //[0];

    if (tooltipLeft < 0)
    {
        tooltipLeft = 0;
    }

    if (tooltipLeft + tooltipWidth > (float)monitor->x + (float)monitor->width)
    {
        tooltipLeft = (float)(monitor->x + monitor->width) - tooltipWidth;
    }
    clutter_actor_set_position (CLUTTER_ACTOR (self), (float)tooltipLeft, (float)tooltipTop);
}

static gboolean 
tooltip_show_delay (gpointer user_data)
{
    CdosTooltip *self = CDOS_TOOLTIP (user_data);
    CdosTooltipPrivate *priv = self->priv;
    clutter_actor_set_child_above_sibling (cdos_ui_plugin_get_ui_group (), (ClutterActor *)self, NULL);
    clutter_actor_remove_all_transitions ((ClutterActor *)self);
    clutter_actor_show (CLUTTER_ACTOR (self));
    self->visible = TRUE;
    priv->show_timeout_id = 0;
    return FALSE;
}

static void
cdos_tooltip_show_real (CdosTooltip *self)
{
    if (self->priv->show_timeout_id > 0)
    {
        g_source_remove (self->priv->show_timeout_id);
        self->priv->show_timeout_id = 0;
    }
    self->priv->show_timeout_id = g_timeout_add (1000, (GSourceFunc)tooltip_show_delay, self);
}

static void
cdos_tooltip_class_init (CdosTooltipClass *klass)
{
    GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
    gobject_class->finalize = cdos_tooltip_finalize;
    gobject_class->dispose = cdos_tooltip_dispose;
    gobject_class->set_property = cdos_tooltip_set_property;
    gobject_class->get_property = cdos_tooltip_get_property;
    klass->show = cdos_tooltip_show_real;
    klass->set_position = cdos_tooltip_set_position_real;

    GParamSpec *spec =
        g_param_spec_object ("item", "Item", "Tooltip's source item", CLUTTER_TYPE_ACTOR, G_PARAM_READWRITE);
    g_object_class_install_property (gobject_class, PROP_ITEM, spec);
}

static void
cdos_tooltip_init (CdosTooltip *self)
{
    // CdosTooltipPrivate *priv;
    self->priv = cdos_tooltip_get_instance_private (self);
    st_widget_set_style_class_name (CLUTTER_ACTOR(self),"tooltip");
    // this._tooltip.show_on_set_parent = false;
    // st_container_add_actor (CLUTTER_CONTAINER(ui_group), CLUTTER_ACTOR (self));
    self->visible = FALSE;
    self->preventShow = FALSE;

    ClutterActor *ui_group = cdos_ui_plugin_get_ui_group ();
    clutter_actor_add_child (ui_group, (ClutterActor *)self);
    clutter_actor_hide ((ClutterActor *)self);
}

//公有函数定义

/**
 * cdos_tooltip_new
 * @item: 一个#ClutterActor对象
 * @init_title: 需要显示的字符串
 *
 * 创建#CdosTooltip对象
 *
 * Returns: #CdosTooltip对象
 */

CdosTooltip *
cdos_tooltip_new (ClutterActor *item, const gchar *init_title)
{
    g_return_val_if_fail (CLUTTER_IS_ACTOR (item), NULL);
    g_return_val_if_fail (init_title != NULL, NULL);
    CdosTooltip *instance;
    instance = g_object_new (CDOS_TYPE_TOOLTIP,
                             "item", item, 
                             "text", init_title, 
                             NULL);
    return instance;
}

/**
 * cdos_tooltip_show
 * @self: 一个#CdosTooltip对象
 *
 * 显示#CdosTooltip
 *
 */

void
cdos_tooltip_show (CdosTooltip *self)
{
    g_return_if_fail (CDOS_IS_TOOLTIP (self));
    if (self->preventShow)
        return;
    if (st_label_get_text ((StLabel *)self) == NULL || g_strcmp0 (st_label_get_text ((StLabel *)self), "") == 0)
        return;
    ((CdosTooltipClass *)G_OBJECT_GET_CLASS (self))->set_position (self);
    ((CdosTooltipClass *)G_OBJECT_GET_CLASS (self))->show (self);
}

/**
 * cdos_tooltip_hide
 * @self: 一个#CdosTooltip对象
 *
 * 隐藏#CdosTooltip
 *
 */

void
cdos_tooltip_hide (CdosTooltip *self)
{
    g_return_if_fail (CDOS_IS_TOOLTIP (self));
    // Tweener.removeTweens(this);
    if (self->priv->show_timeout_id > 0)
    {
        g_source_remove (self->priv->show_timeout_id);
        self->priv->show_timeout_id = 0;
    }
    clutter_actor_hide (CLUTTER_ACTOR (self));
    self->visible = FALSE;
}

/**
 * cdos_tooltip_set_prevent_show
 * @self: 一个#CdosTooltip对象
 * @prevent_show: TRUE或FALSE。当prevent_show的值为TRUE时，将阻止#CdosTooltip显示
 *
 * 是否显示#CdosTooltip
 *
 */

void
cdos_tooltip_set_prevent_show (CdosTooltip *self, gboolean prevent_show)
{
    g_return_if_fail (CDOS_IS_TOOLTIP (self));
    self->preventShow = prevent_show;
}

/**
 * cdos_tooltip_set_position
 * @self: 一个#CdosTooltip对象
 * @px: x轴坐标
 * @py: y轴坐标
 *
 * 设置#CdosTooltip显示的位置
 *
 */

void
cdos_tooltip_set_position (CdosTooltip *self, gfloat px, gfloat py)
{
    g_return_if_fail (CDOS_IS_TOOLTIP (self));
    self->mousePositionX = px;
    self->mousePositionY = py;
}
