//
// Created by zhangtao on 16-3-17.
/**
 * SECTION: cdos-util
 * @short_description: 提供了部分辅助功能函数。
 * @title: CdosUtil
 * @include: cdos-util.h
 *
 * #CdosUtil提供了一组辅助功能函数，涉及url,命令行，进程操作等。
 */
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <libupower-glib/upower.h>
#include "cdos-util.h"
#include "cdos-global.h"

struct _MatchedUrl
{
    gint start_pos;
    gint end_pos;
    gchar *url;
};

static gchar *
cdos_util_get_url_pattern (void)
{
    static gchar *pattern = NULL;

    if (pattern)
    {
        return pattern;
    }

    /* https://gist.github.com/gruber/249502 */
    // gchar *argv[3];
    // argv[0] = g_strdup("pkill");
    // argv[1] = g_strdup("-f");
    // argv[2] = g_strconcat (temp1, process_name, temp2);

    const gchar *balanced_parens = "\\(([^\\s()<>]+|(\\(?:[^\\s()<>]+\\)))*\\)";
    const gchar *not_trailing_junk = "[^\\s`!()\\[\\]{};:\"\'.,<>?\u00AB\u00BB\u201C\u201D\u2018\u2019]";

    const gchar *reg1 = "(?i)\\b";
    const gchar *reg2 = "((?:[a-z0-9][\\w-]+(?::/{1,3}|@|:)|www\\d{0,3}[.]|[a-z0-9.\\-]+[.][a-z]{2,4}/)(?:[^\\s()<>]+|";
    //gchar *reg2 = ")((?:[a-z0-9][\\w-]+://|www\\d{0,3}[.]|[a-z0-9.\\-]+[.][a-z]{2,4}/)(?:[^\\s()<>]+|";
    const gchar *reg3 = ")+(?:";
    const gchar *reg4 = "|";
    const gchar *reg5 = "))";

    pattern = g_strconcat (
        reg1, reg2, balanced_parens, reg3, balanced_parens, reg4, not_trailing_junk, reg5, NULL);

    /*
    gint pattern_length = strlen(reg1) + strlen(reg2) + strlen(reg3) + strlen(reg4) + strlen(reg5) +
    strlen(balanced_parens) + strlen(leading_junk) + strlen(not_trailing_junk);

    pattern = (gchar *)malloc(pattern_length + 1);
    if(!pattern){
        printf("Error: the url pattern failed !");
        exit(EXIT_FAILURE);
    }
    strcpy(pattern,reg1);
    strcat(pattern,leading_junk);
    strcat(pattern,reg2);
    strcat(pattern,balanced_parens);
    strcat(pattern,reg3);
    strcat(pattern,balanced_parens);
    strcat(pattern,reg4);
    strcat(pattern,not_trailing_junk);
    strcat(pattern,reg5);
    */
    return pattern;
}
static GRegex *
cdos_util_get_url_regexp (void)
{
    gchar *pattern = cdos_util_get_url_pattern ();

    GRegex *regex = g_regex_new (pattern, 0, 0, NULL);
    return regex;
}

/**
 * cdos_util_find_urls:
 * @urls:需要查找的字符串
 *
 * 在给定的字符串中，查找包含有url格式的子字符串。
 *
 * Returns:返回全部url的链表。
 */
GList *
cdos_util_find_urls (gchar *urls)
{
    GRegex *regex;
    GList *res = NULL;
    GMatchInfo *match_info;

    regex = cdos_util_get_url_regexp ();

    g_regex_match (regex, urls, 0, &match_info);
    while (g_match_info_matches (match_info))
    {
        MatchedUrl *matchedUrl;
        gint start_pos, end_pos;
        gchar *url = g_match_info_fetch (match_info, 0);
        if (url)
        {
            matchedUrl = g_new (MatchedUrl, 1);
            matchedUrl->url = url;
            // g_print("\n %s:url=%s; \n",__func__,matchedUrl->url);
            if (g_match_info_fetch_pos (match_info, 0, &start_pos, &end_pos))
            {
                // g_print("\n %s:start_pos=%d; \n",__func__,start_pos);
                // g_print("\n %s:end_pos=%d; \n",__func__,end_pos);

                matchedUrl->start_pos = start_pos;
                matchedUrl->end_pos = end_pos;
            }
            res = g_list_append (res, matchedUrl);
        }

        g_match_info_next (match_info, NULL);
    }

    g_match_info_free (match_info);
    g_regex_unref (regex);

    return res;
}

/**
 * cdos_util_spawn:
 * @arg: 需要执行的命令字符串数组
 *
 * 在后台执行命令行操作命令，如果执行异常，则自行处理。
 *
 * Returns:返回对应的进程PID
 */
gint
cdos_util_spawn (const gchar **arg)
{
    gint pid;
    pid = cdos_util_try_spawn ((char **)arg);
    return pid;
}

/**
 * cdos_util_spawn_command_line:
 * @command_line: 需要执行的命令字符串
 *
 * 在后台执行命令行操作命令，如果执行异常，则自行处理。
 *
 * Returns:返回对应的进程PID
 */
gint
cdos_util_spawn_command_line (const gchar *command_line)
{
    GPid pid;
    GError *error = NULL;
    gint argcp;
    gchar **argvp;

    g_shell_parse_argv (command_line, &argcp, &argvp, &error);
    if (error)
    {
        g_warning ("Warnings:%s-->%s", __func__, error->message);
        g_error_free (error);
        return 0;
    }

    g_spawn_async (NULL, argvp, NULL, G_SPAWN_SEARCH_PATH |
                   G_SPAWN_STDOUT_TO_DEV_NULL | G_SPAWN_STDERR_TO_DEV_NULL,
                   NULL, NULL, &pid, &error);
    if(error)
    {
        g_warning ("Warnings:%s-->%s", __func__, error->message);
        g_error_free (error);
        return 0;
    }
    g_strfreev (argvp);
    return pid;
}

/**
 * cdos_util_try_spawn:
 * @argv: 需要执行的命令字符串数组
 *
 * 在后台执行命令行操作命令，如果执行失败则抛出错误异常。
 *
 * Returns:执行成功则返回对应的进程PID，否则返回0
 */
gint
cdos_util_try_spawn (gchar **argv)
{
    gint pid;
    GError *error = NULL;

    g_spawn_async(NULL, argv,NULL,(G_SPAWN_SEARCH_PATH | G_SPAWN_STDOUT_TO_DEV_NULL | G_SPAWN_STDERR_TO_DEV_NULL),NULL,NULL,&pid,&error);
    if(error)
    {
        g_warning("Warnings:%s-->%s",__func__,error->message);
        g_error_free(error);
        return 0;
    }
    else
    {
        return pid;
    }
}
/**
 * cdos_util_try_spawn_command_line:
 * @command_line: 需要执行的命令字符串指针地址
 *
 * 在后台执行命令行操作命令，如果执行失败则抛出错误异常。
 *
 * Returns:执行成功则返回对应的进程PID，否则返回0
 */
gint
cdos_util_try_spawn_command_line (gchar *command_line)
{
    gint pid = 0;
    GError *error;
    gchar **argvp;
    gint argcp;

    g_shell_parse_argv (command_line, &argcp, &argvp, &error);
    if (error)
    {
        g_warning ("Warnings:%s-->%s", __func__, error->message);
        g_error_free (error);
        return 0;
    }
    else
    {
        pid = cdos_util_try_spawn (argvp);
        if (pid)
            return pid;
        // cdos_util_handle_spawn_error(command_line,error->message);
        return 0;
    }
}

/**
 * cdos_util_killall:
 * @process_name: 进程名称
 *
 * 关闭给定名称的全部相关进程。
 */
void
cdos_util_killall (gchar *process_name)
{
    if (!process_name)
        return;

    const gchar *temp1 = "pkill -f '^([^ ]*/)?";
    const gchar *temp2 = "($| )'";
    gchar *cmd = g_new0 (gchar, strlen (temp1) + strlen (process_name) + strlen (temp2) + 1);

    GError *error = NULL;

    /*
    gchar *argv[4] = {NULL};
    argv[0] = g_strdup("pkill");
    argv[1] = g_strdup("-f");
    argv[2] = g_strconcat (temp1, process_name, temp2, NULL);

    result = g_spawn_sync(NULL, argv,NULL,G_SPAWN_SEARCH_PATH,NULL,NULL,&pid,&error);
    */
    sprintf (cmd, "%s%s%s", temp1, process_name, temp2);
    gint status;
    g_spawn_command_line_sync (cmd, NULL, NULL, &status, &error);

    if (error)
    {
        g_warning ("Failed to kill process!");
        g_error_free (error);
    }
    /*
    g_free (argv[0]);
    g_free (argv[1]);
    g_free (argv[2]);
    */
    g_free (cmd);
}

static const gchar *ignored_words[] = {
    "Semiconductor",
    "Components",
    "Corporation",
    "Communications",
    "Company",
    "Corp.",
    "Corp",
    "Co.",
    "Inc.",
    "Inc",
    "Incorporated",
    "Ltd.",
    "Limited.",
    "Intel",
    "chipset",
    "adapter",
    "[hex]",
    "NDIS",
    "Module"
};
static const gchar *ignored_phrases[] = {
    "Multiprotocol MAC/baseband processor",
    "Wireless LAN Controller",
    "Wireless LAN Adapter",
    "Wireless Adapter",
    "Network Connection",
    "Wireless Cardbus Adapter",
    "Wireless CardBus Adapter",
    "54 Mbps Wireless PC Card",
    "Wireless PC Card",
    "Wireless PC",
    "PC Card with XJACK(r) Antenna",
    "Wireless cardbus",
    "Wireless LAN PC Card",
    "Technology Group Ltd.",
    "Communication S.p.A.",
    "Business Mobile Networks BV",
    "Mobile Broadband Minicard Composite Device",
    "Mobile Communications AB",
    "(PC-Suite Mode)"
};

char *
cdos_util_fixup_pci_description (const gchar *desc)
{
    char *tmp = g_strdup (desc);
    int len = strlen (tmp);
    int i;
    for (i = 0; i < len; i++)
        if (tmp[i] == '_' || tmp[i] == ',')
            tmp[i] = ' ';
    for (i = 0; i < G_N_ELEMENTS (ignored_phrases); i++)
    {
        char *ptr = NULL;
        if ((ptr = strstr (tmp, ignored_phrases[i])) != NULL)
        {
            int len = strlen (ignored_phrases[i]);
            int j;
            for (j = 0; j < len; j++)
                ptr[j] = ' ';
        }
    }
    for (i = 0; i < G_N_ELEMENTS (ignored_words); i++)
    {
        char *ptr = NULL;
        if ((ptr = strstr (tmp, ignored_words[i])) != NULL)
        {
            int len = strlen (ignored_words[i]);
            int j;
            for (j = 0; j < len; j++)
                ptr[j] = ' ';
        }
    }
    char *res = g_new0 (char, len + 1);
    int j = 0;
    for (i = 0; tmp[i] == ' '; i++);
    for (; i < len - 1; i++)
    {
        if (tmp[i] == ' ' && tmp[i + 1] == ' ')
            continue;
        res[j] = tmp[i];
        j++;
    }
    if (tmp[i] != ' ')
        res[j] = tmp[i];
    g_free (tmp);
    return res;
}

/**
 * cdos_util_get_url_start_pos:
 * @url:当前匹配到的url数据信息对象
 *
 * 从匹配到的url数据信息中获取该url的起始位置索引。
 *
 * Returns:起始位置索引值。
 */
gint
cdos_util_get_url_start_pos (MatchedUrl *url)
{
    if (url == NULL)
    {
        return -1;
    }
    return url->start_pos;
}

/**
 * cdos_util_get_url_end_pos:
 * @url:当前匹配到的url数据信息对象
 *
 * 从匹配到的url数据信息中获取该url的终止位置索引。
 *
 * Returns:终止位置索引值。
 */
gint
cdos_util_get_url_end_pos (MatchedUrl *url)
{
    if (url == NULL)
    {
        return -1;
    }
    return url->end_pos;
}

/**
 * cdos_util_get_url:
 * @url:当前匹配到的url数据信息对象
 *
 * 从匹配到的url数据信息中获取url字符串。
 *
 * Returns:url字符串。
 */
gchar *
cdos_util_get_url (MatchedUrl *url)
{
    if (url == NULL)
    {
        return NULL;
    }
    return url->url;
}

/**
 * cdos_util_is_laptop:
 *
 * 判断是否是笔记本电脑。
 *
 * Returns:TRUE 是笔记本电脑，FALSE 不是笔记本电脑。
 */
gboolean
cdos_util_is_laptop (void)
{
    static UpClient *up_client = NULL;
    gboolean lid_is_present = FALSE;

    if (!up_client)
        up_client = up_client_new ();
    if (up_client)
        lid_is_present = up_client_get_lid_is_present (up_client);

    return lid_is_present;
}

static gboolean
can_suspend_or_hibernate (const gchar *method_name)
{
    g_autoptr(GDBusConnection) connection = NULL;
    g_autoptr(GVariant) variant = NULL;
    g_autoptr(GError) error = NULL;
    const char *s;

    connection = g_bus_get_sync (G_BUS_TYPE_SYSTEM, NULL, &error);
    if (!connection)
    {
        if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
            g_warning ("system bus not available: %s", error->message);
        return FALSE;
    }

    variant = g_dbus_connection_call_sync (connection,
                                         "org.freedesktop.login1",
                                         "/org/freedesktop/login1",
                                         "org.freedesktop.login1.Manager",
                                         method_name,
                                         NULL,
                                         NULL,
                                         G_DBUS_CALL_FLAGS_NONE,
                                         -1,
                                         NULL,
                                         &error);

    if (!variant)
    {
        if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
            g_debug ("Failed to call %s(): %s", method_name, error->message);
        return FALSE;
    }

    g_variant_get (variant, "(&s)", &s);
    return g_strcmp0 (s, "yes") == 0;
}

gboolean
check_acpi_supports (const gchar *action)
{
    gboolean has = TRUE;
    const char *file = "/etc/acpi/supports";
    char *check_content;
    FILE *stream = NULL;
    char buff[1024] = {0};
    const gchar *dbus_method;
    if (!g_file_test (file, G_FILE_TEST_EXISTS) || !action)
    {
        g_warning ("ACPI supports file is not found.");
        goto out;
    }
    check_content = g_strconcat ("head ", file, NULL);
    stream = popen (check_content, "r");
    g_free (check_content);
    if (stream == NULL)
    {
        goto out;
    }
    fread (buff, 1, sizeof (buff), stream);
    pclose (stream);
    if (buff != NULL)
    {
        if (strlen (buff) == 0 || strlen (buff) < strlen (action))
        {
            goto out;
        }
        if (strstr (buff, action) == NULL)
        {
            has = FALSE;
        }
    }
out:

    if (g_strcmp0 (action, "S3") == 0)
    {
        dbus_method = "CanSuspend";
    } 
    else if (g_strcmp0 (action, "S4") == 0)
    {
        dbus_method = "CanHibernate";
    }
    gboolean check_dbus = can_suspend_or_hibernate (dbus_method);
    g_message ("Check acpi action: %s = %d, acpi_support = %d.", dbus_method, check_dbus, has);
    return has && check_dbus;
}

gboolean
can_switch_user (void)
{
    const char *seat_path = g_getenv ("XDG_SEAT_PATH");
    gboolean cans_available = TRUE;
    if (seat_path == NULL)
    {
        g_warning ("Couldn't get environment XDG_SEAT_PATH.");
        return cans_available;
    }

    GDBusProxy *can_switch_user = g_dbus_proxy_new_for_bus_sync (G_BUS_TYPE_SYSTEM,
                              G_DBUS_PROXY_FLAGS_NONE,
                              NULL,
                              "org.freedesktop.DisplayManager",
                              seat_path,
                              "org.freedesktop.DisplayManager.Seat",
                              NULL,
                              NULL);
    if (can_switch_user)
    {
        GVariant *value = NULL;
        value = g_dbus_proxy_get_cached_property (G_DBUS_PROXY (can_switch_user), "CanSwitch");
        if (value)
        {
            cans_available = g_variant_get_boolean (value);
            g_variant_unref (value);
        }
        g_object_unref (can_switch_user);
    }
    return cans_available;
}

const gchar *
cdos_get_time_format (void)
{
    static GSettings *settings = NULL;
    if (!settings)
        settings = g_settings_new ("org.cdos.desktop.interface");
    gboolean use_24h = g_settings_get_boolean (settings, "clock-use-24h");
    const gchar *time_format = "%H:%M";
    if (!use_24h)
        time_format = "%p %l:%M";
    return time_format;
}

static void
drawing_area_on_repaint (StDrawingArea *area, gpointer user_data)
{
    cairo_surface_t *image = user_data;
    cairo_t         *cr;
    gint            w, h;
    gfloat          aw, ah, icon_size;

    if (image == NULL)
        return ;
    clutter_actor_get_size ((ClutterActor *)area, &aw, &ah);
    //get the small one as icon_size;
    icon_size = aw > ah ? ah : aw;

    w = cairo_image_surface_get_width (image);
    h = cairo_image_surface_get_height (image);

    cr = st_drawing_area_get_context (area);
    cairo_arc (cr, icon_size/2, icon_size/2, icon_size/2, 0, 2 * M_PI);
    cairo_clip (cr);
    cairo_scale (cr, icon_size/w, icon_size/h);
    //cairo_new_path (cr);
    cairo_set_source_surface (cr, image, 0, 0);
    cairo_paint (cr);
    cairo_fill (cr);
}

static void
drawing_area_on_destroy (ClutterActor *actor, gpointer user_data)
{
    cairo_surface_t *image = user_data;
    cairo_surface_destroy (image);
}

ClutterActor*
cdos_get_user_face_icon (ActUser *act_user, gint size)
{
    const gchar  *icon_path;
    GFile        *icon_file;
    GIcon        *gicon = NULL;
    StDrawingArea    *drawing_area = NULL;
    cairo_surface_t  *image;
    //gint    icon_width, icon_height, size;
    GtkIconInfo  *info = NULL;
    GtkIconTheme *icon_theme;

    icon_path = act_user_get_icon_file (act_user);
    if (icon_path && g_file_test (icon_path, G_FILE_TEST_EXISTS))
    {
        icon_file = g_file_new_for_path (icon_path);
        gicon = g_file_icon_new (icon_file);
        if (gicon)
        {
            //cogl_bitmap_get_size_from_file (icon_path, &icon_width, &icon_height);
            //size = icon_width >= icon_height ? icon_width : icon_height;
            icon_theme = gtk_icon_theme_get_default ();
            info = gtk_icon_theme_lookup_by_gicon (icon_theme, 
                                                  gicon, 
                                                  -1, 
                                                  GTK_ICON_LOOKUP_USE_BUILTIN);
            g_object_unref (gicon);
        }
        if (info)
        {
            image = gtk_icon_info_load_surface (info, NULL, NULL);
            if (image)
            {
                drawing_area = g_object_new (ST_TYPE_DRAWING_AREA, NULL);
                clutter_actor_set_size (drawing_area, (float)size, (float)size);
                g_signal_connect (drawing_area, 
                                 "repaint",
                                 G_CALLBACK (drawing_area_on_repaint), 
                                 image);
                g_signal_connect ((ClutterActor *)drawing_area, 
                                 "destroy",
                                 G_CALLBACK (drawing_area_on_destroy), 
                                 image);
            }
            g_object_unref (info);
        }
        g_object_unref (icon_file); 
    }
    return  (ClutterActor *)drawing_area;
}


const gchar *
cdos_user_get_display_name (ActUser *act_user)
{
    const gchar *user_name = act_user_get_real_name (act_user);
    if (!user_name)
        user_name = act_user_get_user_name (act_user);
    return user_name;
}

gboolean 
cdos_boot_is_live (void)
{
    static gint boot_flag = -1;
    const gchar *cmd = "cat /proc/cmdline";
    if (boot_flag == -1)
    {
        gchar *output;
        g_spawn_command_line_sync (cmd,
                                   &output,
                                   NULL,
                                   NULL,
                                   NULL);
        if (g_strrstr (output, "boot=live") != NULL)
            boot_flag = 1;
        else 
            boot_flag = 0;
    }
    return boot_flag == 1 ? TRUE : FALSE;
}

gboolean
cdos_enable_blur_effect (void)
{
    CdosGlobal *global = cdos_global_get ();
    GSettings *settings = cdos_global_get_settings (global);
    gboolean enabled = FALSE;
    if (settings)
    {
        enabled = g_settings_get_boolean (settings, "desktop-effects-blur");
    }
    return enabled;
}

gboolean
g_settings_has_key (GSettings *gsettings, const gchar *key)
{
  g_return_val_if_fail (G_IS_SETTINGS (gsettings), FALSE);
  g_return_val_if_fail (key != NULL, FALSE);
  gchar **keys;
  keys = g_settings_list_keys (gsettings);
  gint i;
  gboolean has = FALSE;
  for (i = 0; keys[i] != NULL; i++)
  {
    if (g_strcmp0 (keys[i], key) == 0)
    {
      has = TRUE;
      break;
    }
  }
  g_strfreev (keys);
  return has;
}