#include <glib.h>
#include "cdos-utils-private.h"

/**
 * cdos_utils_to_lower_case:
 * @str: 字符串
 *
 * 将当前字符串全部转换为小写。
 */
void
cdos_utils_to_lower_case (gchar *str)
{
    while (*str)
    {
        if (*str >= 'A' && *str <= 'Z')
        {
            *str = 'a' + (*str - 'A');
        }
        ++str;
    }
}

/**
 * cdos_strv_elem_del_by_nth:
 * @strv:字符串数组
 * @n:索引值
 *
 * 从字符串数组中，移除指定索引值的成员。
 *
 * Returns:返回一个新的字符串数组
 */
gchar **
cdos_strv_elem_del_by_nth (gchar **strv, gint n)
{
    int i;
    GPtrArray *p_a = g_ptr_array_new ();

    for (i = 0; strv[i] != NULL; i++)
    {
        if (i == n)
            continue;
        g_ptr_array_add (p_a, strv[i]);
    }
    g_ptr_array_add (p_a, NULL);


    return (gchar **)g_ptr_array_free (p_a, FALSE);
}

/**
 * cdos_strv_elem_del_by_str:
 * @strv:字符串数组
 * @str:待匹配的字符串
 *
 * 从字符串数组中，移除与指定字符串匹配的成员。
 *
 * Returns:返回一个新的字符串数组
 */
gchar **
cdos_strv_elem_del_by_str (gchar **strv, const gchar *str)
{
    gint len;
    gint i, j;
    gchar **result = NULL;

    len = g_strv_length (strv);
    result = g_new0 (gchar *, len + 1);

    for (i = 0, j = 0; i < len; i++)
    {
        if (g_strcmp0 (strv[i], str) != 0)
            result[j++] = g_strdup (strv[i]);
    }
    return result;
}

/**
 * cdos_strv_elem_add:
 * @strv:字符串数组
 * @str:待添加的字符串
 *
 * 从字符串数组中，添加新的字符串成员。
 *
 * Returns:返回一个新的字符串数组
 */
gchar **
cdos_strv_elem_add (const gchar **strv, const gchar *str)
{
    gint i;
    GPtrArray *p_a = g_ptr_array_new ();
    for (i = 0; strv[i]; i++)
    {
        g_ptr_array_add (p_a, g_strdup (strv[i]));
    }
    g_ptr_array_add (p_a, g_strdup (str));
    g_ptr_array_add (p_a, NULL);

    return (gchar **)g_ptr_array_free (p_a, FALSE);
}

gchar **
cdos_strv_elem_insert (const gchar **strv, const gchar *str, gint index)
{
    if (!strv || !str)
    {
        return NULL;
    }

    gint i;
    GPtrArray *p_a = g_ptr_array_new ();
    gint len = g_strv_length ((gchar **)strv);

    for (i = 0; i < len; i++)
    {
        g_ptr_array_add (p_a, g_strdup (strv[i]));
    }
    if (index >= len || index < 0)
    {
        index = -1;
    }
    g_ptr_array_insert (p_a, index, g_strdup (str));
    g_ptr_array_add (p_a, NULL);

    return (gchar **)g_ptr_array_free (p_a, FALSE);
}
