#include "st.h"

#include "cdos-popup-separator-menu-item.h"
#include "cdos-window-context-menu.h"
#include "cdos-layout-manager.h"
#include "cdos-ui-plugin.h"

/**
 * SECTION:cdos-window-context-menu
 * @Title: CdosWindowContextMenu
 * @Short_description: 右键菜单
 *
 * 缩略图模式下，右键某个克隆窗体，弹出的菜单，目测包含的功能有：移动到上一个或者下一个工作区、关闭这个窗口等，
 */
G_DEFINE_TYPE (CdosWindowContextMenu, cdos_window_context_menu, G_TYPE_OBJECT);


static gboolean
onSourceKeyPress (GObject *gobject, ClutterEvent *event, gpointer user_data)
{
    CdosWindowContextMenu *self = CDOS_WINDOW_CONTEXT_MENU (user_data);
    CdosPopupMenuBase *menu = CDOS_POPUP_MENU_BASE (self);
    CdosPopupMenuBaseClass *obj = CDOS_POPUP_MENU_BASE_CLASS (menu);

    guint symbol = clutter_event_get_key_symbol (event);

    if (symbol == CLUTTER_KEY_space || symbol == CLUTTER_KEY_Return)
    {
        //等同于this.menu.toggle();
        cdos_popup_menu_base_toggle (menu);

        return TRUE;
    }
    else if (symbol == CLUTTER_KEY_Escape && menu->is_open)
    {
        //等同于this.menu.close();
        obj->close (menu, FALSE);

        return TRUE;
    }
    else if (symbol == CLUTTER_KEY_Down)
    {
        if (!menu->is_open)
            cdos_popup_menu_base_toggle (menu);

        st_widget_navigate_focus (ST_WIDGET (gobject), CLUTTER_ACTOR (self), GTK_DIR_DOWN, FALSE);

        return TRUE;
    }
    else
    {
        return FALSE;
    }
}


static void
onToggled (GObject *gobject, gboolean opening, gpointer user_data)
{
    CdosWindowContextMenu *self = CDOS_WINDOW_CONTEXT_MENU (user_data);
    MetaWorkspace *mWorkspace;
    MetaWorkspace *neighbor_L, *neighbor_R;

    if (!opening)
    {
        clutter_actor_destroy (CLUTTER_ACTOR (self));
        return;
    }

    if (meta_window_is_on_all_workspaces (self->metaWindow))
        cdos_popup_menu_item_set_label (self->on_all_workspaces, _("Only on this workspace"));
    else
        cdos_popup_menu_item_set_label (self->on_all_workspaces, _("Visible on all workspaces"));


    mWorkspace = meta_window_get_workspace (self->metaWindow);
    neighbor_L = meta_workspace_get_neighbor (mWorkspace, META_MOTION_LEFT);

    if (mWorkspace != neighbor_L)
        clutter_actor_show (CLUTTER_ACTOR (self->move_to_left_workspace));
    else
        clutter_actor_hide (CLUTTER_ACTOR (self->move_to_left_workspace));

    neighbor_R = meta_workspace_get_neighbor (mWorkspace, META_MOTION_RIGHT);

    if (mWorkspace != neighbor_R)
        clutter_actor_show (CLUTTER_ACTOR (self->move_to_right_workspace));
    else
        clutter_actor_hide (CLUTTER_ACTOR (self->move_to_right_workspace));

    if (meta_window_get_maximized (self->metaWindow)) //水平為0, 垂直為1
        cdos_popup_menu_item_set_label (self->maximize_window, _("Unmaximize"));
    else
        cdos_popup_menu_item_set_label (self->maximize_window, _("Maximize"));
}


static void
onCloseWindowActivate (GObject *gobject, ClutterEvent *event, gpointer user_data)
{
    CdosWindowContextMenu *self = CDOS_WINDOW_CONTEXT_MENU (user_data);

    meta_window_delete (self->metaWindow, cdos_global_get_current_time (cdos_global_get ()));
}


static void
onMinimizeWindowActivate (GObject *gobject, ClutterEvent *event, gpointer user_data)
{
    CdosWindowContextMenu *self = CDOS_WINDOW_CONTEXT_MENU (user_data);

    gboolean isMinimized;

    g_object_get (self->metaWindow, "minimized", &isMinimized, NULL);

    if (isMinimized)
        meta_window_unminimize (self->metaWindow);
    else
        meta_window_minimize (self->metaWindow);
}


static void
onMaximizeWindowActivate (GObject *gobject, ClutterEvent *event, gpointer user_data)
{
    CdosWindowContextMenu *self = CDOS_WINDOW_CONTEXT_MENU (user_data);

    if (meta_window_get_maximized (self->metaWindow))
        meta_window_unmaximize (self->metaWindow, META_MAXIMIZE_HORIZONTAL | META_MAXIMIZE_VERTICAL);
    else
        meta_window_maximize (self->metaWindow, META_MAXIMIZE_HORIZONTAL | META_MAXIMIZE_VERTICAL);
}


static void
onMoveToLeftWorkspace (GObject *gobject, ClutterEvent *event, gpointer user_data)
{
}


static void
onMoveToRightWorkspace (GObject *gobject, ClutterEvent *event, gpointer user_data)
{
    CdosWindowContextMenu *self = CDOS_WINDOW_CONTEXT_MENU (user_data);

    MetaWorkspace *mWorkspace = meta_window_get_workspace (self->metaWindow);
    MetaWorkspace *neighbor_L = meta_workspace_get_neighbor (mWorkspace, META_MOTION_LEFT);

    if (neighbor_L)
        meta_window_change_workspace (self->metaWindow, neighbor_L);
}


static void
toggleOnAllWorkspaces (GObject *gobject, gpointer user_data)
{
    CdosWindowContextMenu *self = CDOS_WINDOW_CONTEXT_MENU (user_data);

    MetaWorkspace *mWorkspace = meta_window_get_workspace (self->metaWindow);
    MetaWorkspace *neighbor_R = meta_workspace_get_neighbor (mWorkspace, META_MOTION_RIGHT);

    if (neighbor_R)
        meta_window_change_workspace (self->metaWindow, neighbor_R);
}


static void
onMoveWindowToNewWorkspace (GObject *gobject, gpointer user_data)
{
    /* TODO */
}


static void
onAddMenuItem_func (gpointer data, gpointer user_data)
{
    CdosWindowContextMenu *self = CDOS_WINDOW_CONTEXT_MENU (user_data);

    cdos_popup_menu_base_add_menu_item (CDOS_POPUP_MENU_BASE (self), CDOS_POPUP_BASE_MENU_ITEM (data), -1);
}


static void
onItemChangeMonitor (GObject *gobject, gpointer user_data)
{
    CdosWindowContextMenu *self = CDOS_WINDOW_CONTEXT_MENU (user_data);

    meta_window_move_to_monitor (self->metaWindow, self->index);
}


static void
iter_func (gpointer data, gpointer user_data)
{
    CdosWindowContextMenu *self = CDOS_WINDOW_CONTEXT_MENU (user_data);

    if (self->index != meta_window_get_monitor (self->metaWindow))
    {
        gchar *string;
        CdosPopupMenuItem *itemChangeMonitor = cdos_popup_menu_item_new ();

        self->index++;

        string = g_strdup_printf (_("Move to monitor %d"), self->index);

        cdos_popup_menu_item_set_label (itemChangeMonitor, string);

        g_signal_connect (itemChangeMonitor, "activate", G_CALLBACK (onItemChangeMonitor), self);

        self->monitor_items = g_list_append (self->monitor_items, itemChangeMonitor);
    }
}


static void
cdos_window_context_menu_class_init (CdosWindowContextMenuClass *klass)
{
}



static void
cdos_window_context_menu_init (CdosWindowContextMenu *self)
{
}


static void
cdos_window_context_menu_post_init (CdosWindowContextMenu *self,
                                    ClutterActor *actor,
                                    MetaWindow *metaWindow)
{
    ClutterActor *uiGroup = cdos_ui_plugin_get_ui_group ();
    StSide orientation;
    // GList *monitorItems;
    GList *items = NULL;
    CdosPopupMenuItem *itemMoveToNewWorkspace;
    CdosLayoutManager *layout_manager = cdos_layout_manager_get_default ();
    gboolean isMinimized;


    // PopupMenu.PopupComboMenu.prototype._init.call(this, actor);
    //进行popupComboMenu继承动作

    clutter_actor_add_child (uiGroup, (ClutterActor *)self);

    clutter_actor_hide ((ClutterActor *)self);

    orientation = ST_SIDE_TOP;

    g_signal_connect (self, "key-press-event", G_CALLBACK (onSourceKeyPress), self);
    g_signal_connect (self, "open-state-changed", G_CALLBACK (onToggled), self);

    self->metaWindow = metaWindow;

    self->close_window = cdos_popup_menu_item_new ();
    cdos_popup_menu_item_set_label (self->close_window, _("Close"));
    g_signal_connect (self->close_window, "activate", G_CALLBACK (onCloseWindowActivate), self);

    g_object_get (self->metaWindow, "minimized", &isMinimized, NULL);
    self->minimize_window = cdos_popup_menu_item_new ();
    if (isMinimized)
    {
        cdos_popup_menu_item_set_label (self->minimize_window, _("Restore"));
    }
    else
    {
        cdos_popup_menu_item_set_label (self->minimize_window, _("Minimize"));
    }

    g_signal_connect (self->minimize_window, "activate", G_CALLBACK (onMinimizeWindowActivate), self);

    self->maximize_window = cdos_popup_menu_item_new ();
    cdos_popup_menu_item_set_label (self->maximize_window, _("Maximize"));
    g_signal_connect (self->maximize_window, "activate", G_CALLBACK (onMaximizeWindowActivate), self);

    self->move_to_left_workspace = cdos_popup_menu_item_new ();
    cdos_popup_menu_item_set_label (self->move_to_left_workspace, _("Move to left workspace"));
    g_signal_connect (self->move_to_left_workspace, "activate", G_CALLBACK (onMoveToLeftWorkspace), self);

    self->move_to_right_workspace = cdos_popup_menu_item_new ();
    cdos_popup_menu_item_set_label (self->move_to_right_workspace, _("Move to right workspace"));
    g_signal_connect (self->move_to_right_workspace, "activate", G_CALLBACK (onMoveToRightWorkspace), self);

    self->on_all_workspaces = cdos_popup_menu_item_new ();
    cdos_popup_menu_item_set_label (self->on_all_workspaces, _("Visible on all workspaces"));
    g_signal_connect (self->on_all_workspaces, "activate", G_CALLBACK (toggleOnAllWorkspaces), self);

    itemMoveToNewWorkspace = cdos_popup_menu_item_new ();
    cdos_popup_menu_item_set_label (itemMoveToNewWorkspace, _("Move to a new workspace"));
    g_signal_connect (itemMoveToNewWorkspace, "activate", G_CALLBACK (onMoveWindowToNewWorkspace), self);

    GPtrArray *monitors = cdos_layout_manager_get_monitors (layout_manager);

    if (monitors && monitors->len > 1)
    {
        g_ptr_array_foreach (monitors, (GFunc)iter_func, self);

        self->index = 0;
        self->monitor_items = g_list_append (self->monitor_items, cdos_popup_separator_menu_item_new ());
        self->index = 0;
    }

    items = g_list_append (items, itemMoveToNewWorkspace);
    items = g_list_append (items, self->on_all_workspaces);
    items = g_list_append (items, self->move_to_left_workspace);
    items = g_list_append (items, self->move_to_right_workspace);
    items = g_list_append (items, (gpointer)cdos_popup_separator_menu_item_new ());
    items = g_list_append (items, self->minimize_window);
    items = g_list_append (items, self->maximize_window);
    items = g_list_append (items, self->close_window);

    items = g_list_concat (self->monitor_items, items);

    if (orientation != ST_SIDE_BOTTOM)
    {
        items = g_list_reverse (items);
        g_list_foreach (items, (GFunc)onAddMenuItem_func, self);
    }
}

/**
 * cdos_window_context_menu_new:
 * @actor: 待定
 * @metaWindow: 待定
 * @onClose： 待定
 *
 * 待定
 */
CdosWindowContextMenu *
cdos_window_context_menu_new (ClutterActor *actor, MetaWindow *metaWindow)
{
    CdosWindowContextMenu *instance = g_object_new (CDOS_TYPE_WINDOW_CONTEXT_MENU, NULL);

    cdos_window_context_menu_post_init (instance, actor, metaWindow);

    return instance;
}
