#ifndef __CDOS_WORKSPACE_H__
#define __CDOS_WORKSPACE_H__

#include "cdos-global.h"
#include "st.h"
#include "cdos-workspaces-view.h"

#define CDOS_TYPE_WORKSPACE (cdos_workspace_get_type ())
#define CDOS_WORKSPACE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), CDOS_TYPE_WORKSPACE, CdosWorkspace))
#define CDOS_IS_WORKSPACE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), CDOS_TYPE_WORKSPACE))
#define CDOS_WORKSPACE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), CDOS_TYPE_WORKSPACE, CdosWorkspaceClass))
#define CDOS_IS_WORKSPACE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), CDOS_TYPE_WORKSPACE))
#define CDOS_WORKSPACE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), CDOS_TYPE_WORKSPACE, CdosWorkspaceClass))


#define CDOS_TYPE_WORKSPACE_MONITOR (cdos_workspace_monitor_get_type ())
#define CDOS_WORKSPACE_MONITOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), CDOS_TYPE_WORKSPACE_MONITOR, CdosWorkspaceMonitor))
#define CDOS_WORKSPACE_MONITOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), CDOS_TYPE_WORKSPACE_MONITOR, CdosWorkspaceMonitorClass))

typedef struct _CdosWorkspace CdosWorkspace;
typedef struct _CdosWorkspacePrivate CdosWorkspacePrivate;
typedef struct _CdosWorkspaceClass CdosWorkspaceClass;

typedef struct _CdosWorkspaceMonitor CdosWorkspaceMonitor;
typedef struct _CdosWorkspaceMonitorClass CdosWorkspaceMonitorClass;

struct _CdosWorkspace
{
    ClutterActor parent;
    MetaWorkspace *meta_workspace;

    CdosWorkspacePrivate *priv;
};

struct _CdosWorkspaceClass
{
    ClutterActorClass parent_class;
    /* signals */
};

struct _CdosWorkspaceMonitor
{
    ClutterActor   parent;
    CdosWorkspace *my_workspace;
    MetaWorkspace *meta_workspace;
    guint x;
    guint y;
    guint width;
    guint height;
    guint margin;
    guint slot_width;

    guint monitor_index;
    MetaRectangle *monitor;
    //ClutterActor *windowOverlaysGroup;
    ClutterActor *drop_rect;

    GList *windows; //成员类型为 CdosWindowClone

    GPtrArray *add_window_ids;
    gulong window_added_id;
    gulong window_removed_id;
    gulong window_entered_monitor_id;
    gulong window_left_monitor_id;
    //gulong stickyCallbackId;
    gulong overview_hidden_id; // zoomFromOverview 中
    gulong initial_position_idle_id;
    gboolean leaving_overview;

    guint kb_window_index;
    gboolean on_initial_position_windows;
    gboolean stacked_once;

    MetaWindow *mwindow; // 增加的，idle_add 的参数
};

struct _CdosWorkspaceMonitorClass
{
    ClutterActorClass parent_class;
};

typedef enum
{
    WINDOW_POSITION_INITIAL,
    WINDOW_POSITION_ANIMATE,
} WindowPositionFlags;

GType cdos_workspace_get_type (void);
GType cdos_workspace_monitor_get_type (void);

void cdos_workspace_show_windows_overlays (CdosWorkspace *self);

void cdos_workspace_hide_windows_overlays (CdosWorkspace *self);

gboolean cdos_workspace_is_empty (CdosWorkspace *self);

void cdos_workspace_zoom_to_overview (CdosWorkspace *self, guint avail_height);
void cdos_workspace_zoom_from_overview (CdosWorkspace *self);

gboolean cdos_workspace_on_key_press (CdosWorkspace *self, ClutterEvent *event);

void cdos_workspace_destroy (CdosWorkspace *self);

guint grid_navigator_next_index (guint itemCount, guint numCols, guint currentIndex, guint symbol);

void cdos_workspace_position_windows (CdosWorkspace *self, guint flag);

CdosWorkspace *cdos_workspace_new (MetaWorkspace *metaWorkspace, CdosWorkspacesView *view);

#endif /* __CDOS_WORKSPACE_H__ */
