/** 
 * Copyright (c) 2016-2018 CPU and Fundamental Software Research Center, CAS
 *  
 * This software is published by the license of CPU-OS License，you can use and 
 * distibuted this software under this License. See CPU-OS Liense for more detail.
 *
 * You should have receive a copy of CPU-OS License. If not, please contact us 
 * by email <support_os@cpu-os.ac.cn>  
 * 
 **/
#include <clutter/clutter.h>
#include <gio/gio.h>
#include "cdos-zoom-region.h"
#include "cdos-ui-plugin.h"
#include "cdos-start.h"

typedef struct 
{
    gfloat x_mag_factor;
    gfloat y_mag_factor;
    gint x_center;
    gint y_center;
    gboolean redo_cursor_tracking;

} CdosParams;

typedef struct
{
    gint x;
    gint y;
    gint width;
    gint height;
} RegionRect;

struct _CdosZoomRegionPrivate
{
    gboolean is_active;
    CdosGlobal *global;
    guint       allocate_id;
    guint       rectx;
    guint       recty;

};

G_DEFINE_TYPE_WITH_PRIVATE (CdosZoomRegion, cdos_zoom_region, G_TYPE_OBJECT);

// mouse-tracking 表示鼠标移动时 zoom region 跟随移动的方式
// push 到边缘才动

#define BACKGROUND_COLOR clutter_color_new(0x22, 0x66, 0xbb, 0xff)

static void
cdos_zoom_region_change_ROI (CdosZoomRegion *self, CdosParams *params);

static gboolean
cdos_zoom_region_on_settings_changed (GSettings *settings, gchar *key, gpointer data);

enum 
{
    MOUSE_TRACKING_MODE_NONE = 0,
};

static void
cdos_zoom_region_dispose (GObject *gobject)
{
    CdosZoomRegion *self = (CdosZoomRegion *)gobject;
    g_object_unref (self->mouse_actor);
    g_object_unref (self->ui_group_clone);

    G_OBJECT_CLASS (cdos_zoom_region_parent_class)->dispose (gobject);
}

static void
cdos_zoom_region_finalize (GObject *gobject)
{
    G_OBJECT_CLASS (cdos_zoom_region_parent_class)->finalize (gobject);
}

static void
cdos_zoom_region_class_init (CdosZoomRegionClass *klass)
{
    GObjectClass *gobject_class = G_OBJECT_CLASS (klass);// 自己的父类型是什么？

    gobject_class->finalize = cdos_zoom_region_finalize;
    gobject_class->dispose = cdos_zoom_region_dispose;

}

static void
cdos_zoom_region_init (CdosZoomRegion *self)
{
    CdosZoomRegionPrivate *priv;
    priv = self->priv = cdos_zoom_region_get_instance_private (self);

    priv->global = cdos_global_get ();
    self->mouse_tracking_mode = MOUSE_TRACKING_MODE_NONE;
    self->clamp_scrolling_at_edges = FALSE;

    MetaDisplay *display = cdos_global_get_display (cdos_global_get ());
    gint screen_width,screen_height;
    meta_display_get_size (display, &screen_width, &screen_height);

    self->mag_view = NULL;
    self->ui_group_clone = NULL;
    self->mouse_source_actor = NULL;
    self->mouse_actor = NULL;

    self->view_port_x = 0;
    self->view_port_y = 0;
    self->view_port_width = screen_width;
    self->view_port_height = screen_height;

    self->x_center = self->view_port_width / 2;
    self->y_center = self->view_port_height / 2;

    self->x_mag_factor = 1.0;
    self->y_mag_factor = 1.0;

    self->following_cursor = FALSE;
    self->settings = g_settings_new ("org.cdos.desktop.a11y.magnifier");

    g_signal_connect (self->settings, "changed::mag-factor", 
                                    G_CALLBACK (cdos_zoom_region_on_settings_changed), self);

}

static gboolean
cdos_zoom_region_on_settings_changed (GSettings *settings, gchar *key, gpointer data)
{
    CdosZoomRegion *self = (CdosZoomRegion *)data;

    if (g_strcmp0 (key, "mag-factor") == 0)
    {
        self->x_mag_factor = g_settings_get_double (settings, "mag-factor");
        self->y_mag_factor = g_settings_get_double (settings, "mag-factor");

        /*cdos_zoom_region_update_mouse_position (self);
        cdos_zoom_region_scroll_to_mouse_position (self);
        cdos_zoom_region_update_clone_geometry (self);
        clutter_actor_set_position (CLUTTER_ACTOR (self->ui_group_clone), 0, 0);
        clutter_actor_set_size (CLUTTER_ACTOR (self->ui_group_clone), self->view_port_width, self->view_port_height); */
        CdosParams *params = g_malloc0(sizeof (CdosParams));
        params->x_mag_factor = self->x_mag_factor;
        params->y_mag_factor = self->y_mag_factor;
        params->x_center = 0;
        params->y_center = 0;
        params->redo_cursor_tracking = TRUE;
        cdos_zoom_region_change_ROI (self, params);
    }
    return FALSE;
}

static gboolean
cdos_zoom_region_is_active (CdosZoomRegion *self)
{
    return self->mag_view != NULL;
}

static void
cdos_zoom_region_screen_to_viewport (CdosZoomRegion *self,
                                     gint x, gint y,
                                     gint64 *screen_x, gint64 *screen_y)
{
    gint relate_x = x - self->x_center;
    gint relate_y = y - self->y_center;
    gdouble f_screen_x = self->view_port_width / 2 + relate_x * self->x_mag_factor ;
    gdouble f_screen_y = self->view_port_height / 2 + relate_y * self->y_mag_factor ;
    /*if (x == 0 && y == 0) {
        g_debug ("screen %f %f\n temp: (x - self->x_center) %d (y - self->y_center) %d \n %lf %lf\n", 
                                f_screen_x, f_screen_y, (x - self->x_center), (y - self->y_center),
                                relate_x * self->x_mag_factor,
                                relate_y * self->y_mag_factor);
    }*/
    *screen_x = (gint64) f_screen_x;
    *screen_y = (gint64) f_screen_y;
}

static void
cdos_zoom_region_update_mag_view_geometry (CdosZoomRegion *self)
{
    if (!cdos_zoom_region_is_active (self))
    {
        return ;
    }

    clutter_actor_set_size (CLUTTER_ACTOR (self->mag_view), self->view_port_width, self->view_port_height);
    clutter_actor_set_position (CLUTTER_ACTOR (self->mag_view), self->view_port_x, self->view_port_y);

}

static void
cdos_zoom_region_update_mouse_position (CdosZoomRegion *self)
{
    if (!cdos_zoom_region_is_active (self)) return;

    gint64 x_mag_mouse, y_mag_mouse;

    //g_debug ("magnifier mouse: %d %d\n", self->magnifier->x_mouse, self->magnifier->y_mouse);
    cdos_zoom_region_screen_to_viewport (self, self->magnifier->x_mouse,
                                                    self->magnifier->y_mouse,
                                                    &x_mag_mouse, &y_mag_mouse);

    // round?
    //g_debug ("mouse position map: %d %d\n", x_mag_mouse, y_mag_mouse);
    // 配合 viewport 调整坐标
/*    gfloat max_height = self->view_port_height *(1- self->y_mag_factor);
    gfloat max_width = self->view_port_width *(1- self->x_mag_factor);

    {
        if ( x_mag_mouse < 0 ){
            x_mag_mouse = MAX (x_mag_mouse, (gint) max_width);
        } else if (x_mag_mouse > 1){
            x_mag_mouse = 0;
        }

        if ( y_mag_mouse < 0 ){
            y_mag_mouse = MAX (y_mag_mouse, (gint) max_height);
        }else if (y_mag_mouse > 1){
            y_mag_mouse = 0;
        }
    }*/
    clutter_actor_set_position (CLUTTER_ACTOR (self->mouse_actor), x_mag_mouse, y_mag_mouse);
}

static void
cdos_zoom_region_update_clone_geometry (CdosZoomRegion *self)
{
    if (!cdos_zoom_region_is_active (self))
    {
        return ;
    }


    //g_debug ("mag factor: %f %f\n", self->x_mag_factor, self->y_mag_factor);
    clutter_actor_set_scale (CLUTTER_ACTOR (self->ui_group_clone), self->x_mag_factor, self->y_mag_factor);
    clutter_actor_set_scale (CLUTTER_ACTOR (self->mouse_actor), self->x_mag_factor, self->y_mag_factor);
 
    gint64 x, y;
    cdos_zoom_region_screen_to_viewport (self, 0, 0, &x, &y);
    // viewport 这个 x，y 的坐标经常为负数，因为它是相对于屏幕的左上角顶点
    // // 如果放大的情况下，我把鼠标往右下角拖动，那么放大后的 uiGroup 左上角的点，相对于屏幕可见区来说就是负数
    // // 768 - （768 × 1.25） 就是高度最大可以的负值
    gfloat max_height = self->view_port_height *(1- self->y_mag_factor);
    gfloat max_width = self->view_port_width *(1- self->x_mag_factor);

    //g_debug ("viewport: %d %d\n, max is: %f %f\n", x, y, max_width, max_height);
    
    //if ((gint) max_width != 0 && (gint) max_height != 0)
    // 从大到小缩放的时候， ui_group_clone 变小，但是 viewport 的坐标没变，导致显示背景的蓝色
    // 这里调整 viewport 相对于显示器左上角的坐标值，如果相对负值超过了最大允许的负值，调整
    // 注意这里 viewport 调整后，还得调整鼠标的坐标，否则鼠标还受限于原先的坐标
    gint tempx = 0, deltax = 0;
    gint tempy = 0, deltay = 0;
    
    if ( x < 0 ){
        tempx = MAX (x, (gint) max_width);
    } else if (x > 1){
        tempx = 0;
    }

    if ( y < 0 ){
        tempy = MAX (y, (gint) max_height);
    }else if (y > 1){
        tempy = 0;
    }
    deltax = x - tempx;
    deltay = y - tempy;

    if (deltax != 0 || deltay != 0){
        clutter_actor_set_position (CLUTTER_ACTOR (self->ui_group_clone), tempx, tempy);
    } else { 
        clutter_actor_set_position (CLUTTER_ACTOR (self->ui_group_clone), x, y);
    }
    cdos_zoom_region_update_mouse_position (self);
    if (deltax != 0 || deltay != 0)
    {
        gfloat posx, posy;
        gint64 newx, newy;
        clutter_actor_get_position (self->mouse_actor, &posx, &posy);
        newx = (gint64)posx - deltax;
        newy = (gint64)posy - deltay;
 
        //g_debug ("delta x: %d, delta y: %d -- mx: %f, my: %f == newx: %d, newy: %d\n", 
        //                                deltax, deltay, posx, posy, newx, newy);
        clutter_actor_set_position (CLUTTER_ACTOR (self->mouse_actor), newx, newy);
    }
}

void cdos_zoom_region_set_mag_factor (CdosZoomRegion *self, gfloat mag_factor)
{
    self->x_mag_factor = mag_factor;
    self->y_mag_factor = mag_factor;
}

void cdos_zoom_region_set_magnifier (CdosZoomRegion *self, CdosMagnifier *magnifier)
{
    self->magnifier = magnifier;
}

void cdos_zoom_region_set_mouse_tracking_mode (CdosZoomRegion *self, CdosMouseTrackingMode mode)
{
    if (mode >= CDOS_MOUSE_TRACKING_MODE_NONE && mode <= CDOS_MOUSE_TRACKING_MODE_PUSH)
    {
        self->mouse_tracking_mode = mode;
    }
}

void cdos_zoom_region_set_mouse_source_actor (CdosZoomRegion *self, ClutterActor *actor)
{
    self->mouse_source_actor = actor;
}

void cdos_zoom_region_remove_mouse_source_actor (CdosZoomRegion *self)
{
    if (self->mouse_source_actor)
    {
        g_debug ("zoom region's mouse source not null\n");
    }
    if (self->mouse_actor)
    {
        g_debug ("zoom regions's mouse actor not null\n");
    }
}

static gboolean
cdos_zoom_region_is_mouse_over_region (CdosZoomRegion *self)
{
    gboolean mouse_is_over = FALSE;

    if (cdos_zoom_region_is_active (self))
    {
        gint x_mouse = self->magnifier->x_mouse;
        gint y_mouse = self->magnifier->y_mouse;

        gboolean x_over = x_mouse >= self->view_port_x && x_mouse < (self->view_port_x + self->view_port_width);
        gboolean y_over = y_mouse >= self->view_port_y && y_mouse < (self->view_port_y + self->view_port_height);
        mouse_is_over = x_over && y_over;
    }

    return mouse_is_over;
}

static RegionRect*
cdos_zoom_region_get_ROI (CdosZoomRegion *self)
{
    RegionRect *rect = g_malloc0 (sizeof (RegionRect));

    gfloat roi_width = self->view_port_width / self->x_mag_factor;
    gfloat roi_height = self->view_port_height / self->y_mag_factor;

    rect->x = self->x_center - (gint) (roi_width) / 2 ;
    rect->y = self->y_center - (gint) (roi_height) /2 ;
    rect->width = roi_width;
    rect->height = roi_height;

    return rect;
}

void cdos_zoom_region_set_clamp_scrolling_at_edges (CdosZoomRegion *self, gboolean clamp)
{
    self->clamp_scrolling_at_edges = clamp;

    if (clamp)
    {
        cdos_zoom_region_change_ROI (self, NULL);
    }
}

gboolean cdos_zoom_region_scroll_to_mouse_position (CdosZoomRegion *self)
{
    self->following_cursor = TRUE;

    CdosParams *params = g_malloc0(sizeof (CdosParams));
    params->x_mag_factor = self->x_mag_factor;
    params->y_mag_factor = self->y_mag_factor;
    params->x_center = 0;
    params->y_center = 0;
    params->redo_cursor_tracking = TRUE;

    if (self->mouse_tracking_mode != CDOS_MOUSE_TRACKING_MODE_NONE)
    {
        cdos_zoom_region_change_ROI (self, params);
    } else {
        cdos_zoom_region_update_mouse_position (self);
    }

    gboolean mouse_over_region;
    mouse_over_region = cdos_zoom_region_is_mouse_over_region (self);
    
    return mouse_over_region;
}

static void
cdos_zoom_region_create_actors (CdosZoomRegion *self)
{ 
    /*
    ClutterActor *top_window_group = NULL;
    top_window_group = cdos_global_get_top_window_group (cdos_global_get ());

    // top window group 需要 reparent，否则输入法的右键和 input 窗口都无法打开
    ClutterActor *ui_group = cdos_ui_plugin_get_ui_group ();
    cdos_reparent_actor (top_window_group, ui_group);
    */

    self->mag_view = g_object_new(ST_TYPE_BIN, 
                                    "style-class", "magnifier-zoom-region",
                                    "x-fill", TRUE, "y-fill", TRUE, NULL);
    //layout = clutter_bin_layout_new (CLUTTER_BIN_ALIGNMENT_CENTER,
                                              //CLUTTER_BIN_ALIGNMENT_CENTER);


    ClutterStage *stage = cdos_global_get_stage (cdos_global_get ());
    clutter_actor_add_child (CLUTTER_ACTOR (stage), CLUTTER_ACTOR (self->mag_view));

    // util_set_hidden_from_pick

    ClutterActor *main_group = clutter_actor_new ();
    //StWidget *box_group = st_box_layout_new ();
    //st_box_layout_set_vertical (box_group, TRUE);

    //g_object_set (main_group, "clip-to-allocation", TRUE, NULL);
    st_bin_set_child (self->mag_view, main_group);

    //clutter_actor_set_layout_manager (main_group, layout);
    ClutterActor *background = g_object_new (ST_TYPE_WIDGET,
                                             "style-class", "zoom-region-background",
                                             NULL);
    clutter_actor_add_child (main_group, (ClutterActor *)background);

    self->ui_group_clone = clutter_clone_new (cdos_ui_plugin_get_ui_group ());
    clutter_actor_add_child (main_group, self->ui_group_clone);
    //st_box_layout_insert_actor (box_group, self->ui_group_clone, -1);
    g_object_set (self->ui_group_clone, "clip-to-allocation", TRUE, NULL);

    MetaDisplay *display = cdos_global_get_display (cdos_global_get ());
    gint screen_width,screen_height;

    meta_display_get_size (display, &screen_width, &screen_height);
    //g_debug ("screen size: %d %d\n", screen_width, screen_height);

    self->view_port_width = screen_width;
    self->view_port_height = screen_height;
    clutter_actor_set_size (CLUTTER_ACTOR (cdos_ui_plugin_get_ui_group ()), screen_width, screen_height);
    clutter_actor_set_size (CLUTTER_ACTOR (background), screen_width, screen_height);
    clutter_actor_set_size (CLUTTER_ACTOR (main_group), screen_width, screen_height);

    if (clutter_actor_get_parent (self->mouse_source_actor) == NULL)
    {
        self->mouse_actor = self->mouse_source_actor;
    } else {
        // 如果 actor 有 parent 的话，不能再添加到 main-group 中，所以需要创建一个 clone
        self->mouse_actor = clutter_clone_new(self->mouse_source_actor);
    } 
    //if (self->mouse_actor != NULL) g_debug ("mouse actor not null in creating \n");

    clutter_actor_add_child (main_group, self->mouse_actor);
    //st_box_layout_insert_actor (box_group, self->mouse_actor, -1);
    //clutter_actor_add_child (background, self->mouse_actor);

    //clutter_actor_set_child_above_sibling (main_group, self->mouse_actor, self->ui_group_clone);
    //priv->allocate_id = g_signal_connect (main_group, "allocated", G_CALLBACK (on_main_allocated), self);
}

static void
cdos_zoom_region_destroy_actor (CdosZoomRegion *self)
{
    g_return_if_fail (self != NULL);
    
    /* FIXME cross hair
     *
     * */
    ClutterActor *main_group = st_bin_get_child (self->mag_view);
    if (main_group)
    {
        GList *children = clutter_actor_get_children (main_group);

        g_list_foreach (children, (GFunc)clutter_actor_destroy, NULL);
        clutter_actor_destroy (main_group);
    }
    clutter_actor_destroy ((ClutterActor *)self->mag_view);
    self->mag_view = NULL;
    //clutter_actor_destroy (self->ui_group_clone);
    self->ui_group_clone = NULL;
    self->mouse_actor = NULL;
    // FIXME self->cross_hairs_actor = NULL;
}

static void
cdos_zoom_region_center_from_mouse_push (CdosZoomRegion *self,
                                         gint x_mouse, gint64 y_mouse,
                                         gint *new_x, gint *new_y)
{
    RegionRect *rect = cdos_zoom_region_get_ROI (self);

    gfloat cursor_width, cursor_height;

    //g_debug ("moving, rect x: %d, y: %d, width: %d, height: %d\n", rect->x, rect->y, rect->width, rect->height);
    // 屏幕右边与下面多出一条蓝色背景区域，宽高与鼠标像素值相同（24）
    // right bottom 值正常， get_ROI 返回的 rect， 超出 screen size
    clutter_actor_get_size (self->mouse_source_actor, &cursor_width, &cursor_height);
    //clutter_actor_get_size (self->mouse_actor, &cursor_width, &cursor_height);
    gint xpos = rect->x + rect->width / 2 ;
    gint ypos = rect->y + rect->height / 2 ;

    //g_debug ("region of interest: %d %d %d %d\n", rect->x, rect->y, rect->width, rect->height);
    //gint64 x_roi_right = rect->x + rect->width - cursor_width;
    gint x_roi_right = rect->x + rect->width;
    //gint64 y_roi_bottom = rect->y + rect->height - cursor_height;
    gint y_roi_bottom = rect->y + rect->height;

    //g_debug ("right bottom: %d %d\n", x_roi_right, y_roi_bottom);

    if (x_mouse < rect->x)
        xpos -= (rect->x - x_mouse);
    else if (x_mouse > x_roi_right)
        xpos += (x_mouse - x_roi_right);

    if (y_mouse < rect->y)
        ypos -= (rect->y - y_mouse);
    else if (y_mouse > y_roi_bottom)
        ypos += (y_mouse - y_roi_bottom);

    *new_x = xpos;
    *new_y = ypos;
}

static void
cdos_zoom_region_center_from_mouse_position (CdosZoomRegion *self, 
                                             gint *new_x, gint *new_y)
{
    gint x_mouse = self->magnifier->x_mouse;
    gint y_mouse = self->magnifier->y_mouse;

    cdos_zoom_region_center_from_mouse_push (self, x_mouse, y_mouse, new_x, new_y);
}

static void
cdos_zoom_region_change_ROI (CdosZoomRegion *self, CdosParams *params)
{
    if (params == NULL)
    {
        CdosParams *params = g_malloc0(sizeof (CdosParams));
        params->x_mag_factor = self->x_mag_factor;
        params->y_mag_factor = self->y_mag_factor;
        params->x_center = self->x_center;
        params->y_center = self->y_center;
        params->redo_cursor_tracking = FALSE;
    }

    if (params->x_mag_factor <= 0)
    {
        params->x_mag_factor = self->x_mag_factor;
    }
    if (params->y_mag_factor <= 0)
    {
        params->y_mag_factor = self->y_mag_factor;
    }

    if (params->redo_cursor_tracking && 
                        self->mouse_tracking_mode != CDOS_MOUSE_TRACKING_MODE_NONE)
    {
        gint x_center, y_center;

        cdos_zoom_region_center_from_mouse_position (self, &x_center, &y_center);
        params->x_center = x_center;
        params->y_center = y_center;
    }

    if (self->clamp_scrolling_at_edges)
    {
        gfloat roi_width = (gfloat) (self->view_port_width) / self->x_mag_factor;
        gfloat roi_height = (gfloat) (self->view_port_height) / self->y_mag_factor;

        MetaDisplay *display = cdos_global_get_display (cdos_global_get ());
        gint screen_width, screen_height;
        meta_display_get_size (display, &screen_width, &screen_height);

        params->x_center = MIN (params->x_center, screen_width - roi_width / 2);
        params->x_center = MAX (params->x_center, roi_width / 2);
        params->y_center = MIN (params->y_center, screen_height - roi_height / 2);
        params->y_center = MAX (params->y_center, roi_height / 2);
    }

    self->x_center = params->x_center;
    self->y_center = params->y_center;

    cdos_zoom_region_update_clone_geometry (self);
    //cdos_zoom_region_update_mouse_position (self);
    if (params != NULL)
    {
        g_free (params);
    }
}

void
cdos_zoom_region_set_active (CdosZoomRegion *self, gboolean active)
{
    if (active == cdos_zoom_region_is_active (self))
        return;

    if (active )
    {
        cdos_zoom_region_create_actors (self);

        if (cdos_zoom_region_is_mouse_over_region (self))
        {
            //g_debug ("%s %d over region hide mouse\n", __func__, __LINE__);
            cdos_magnifier_hide_system_cursor (self->magnifier);
        }
        cdos_zoom_region_update_mag_view_geometry (self);
        cdos_zoom_region_update_clone_geometry (self);
        cdos_zoom_region_update_mouse_position (self);
        
    } else {
        cdos_zoom_region_destroy_actor (self);
    }
}

CdosZoomRegion *
cdos_zoom_region_new(CdosMagnifier *magnifier, ClutterActor *actor)
{
    CdosZoomRegion *self = NULL;
    self = g_object_new (CDOS_TYPE_ZOOM_REGION, NULL);
    return self;
}
