/** 
 * Copyright (c) 2016-2018 CPU and Fundamental Software Research Center, CAS
 *  
 * This software is published by the license of CPU-OS License，you can use and 
 * distibuted this software under this License. See CPU-OS Liense for more detail.
 *
 * You should have receive a copy of CPU-OS License. If not, please contact us 
 * by email <support_os@cpu-os.ac.cn>  
 * 
 **/
#ifndef __CDOS_ZOOM_REGION_H__
#define __CDOS_ZOOM_REGION_H__

#include "cdos-global.h"
#include "cdos-magnifier.h"

#define CDOS_TYPE_ZOOM_REGION                (cdos_zoom_region_get_type ())
#define CDOS_ZOOM_REGION(obj)                (G_TYPE_CHECK_INSTANCE_CAST ((obj), CDOS_TYPE_ZOOM_REGION, CdosZoomRegion))
#define CDOS_IS_ZOOM_REGION(obj)             (G_TYPE_CHECK_INSTANCE_TYPE ((obj), CDOS_TYPE_ZOOM_REGION))
#define CDOS_ZOOM_REGION_CLASS(klass)        (G_TYPE_CHECK_CLASS_CAST ((klass), CDOS_TYPE_ZOOM_REGION, CdosZoomRegionClass))
#define CDOS_IS_ZOOM_REGION_CLASS(klass)     (G_TYPE_CHECK_CLASS_TYPE ((klass), CDOS_TYPE_ZOOM_REGION))
#define CDOS_ZOOM_REGION_GET_CLASS(obj)      (G_TYPE_INSTANCE_GET_CLASS ((obj), CDOS_TYPE_ZOOM_REGION, CdosZoomRegionClass))

typedef struct _CdosZoomRegion              CdosZoomRegion;
typedef struct _CdosZoomRegionPrivate       CdosZoomRegionPrivate;
typedef struct _CdosZoomRegionClass         CdosZoomRegionClass;

typedef enum 
{
    NONE = 0,
    FULL_SCREEN,
    TOP_HALF,
    BOTTOM_HALF,
    LEFT_HALF,
    RIGHT_HARF,
    NUM_OF_POSITIONS
} ScreenPosition;

typedef enum
{
    CDOS_MOUSE_TRACKING_MODE_NONE = 0,
    CDOS_MOUSE_TRACKING_MODE_PUSH,
    NUM_OF_MODES
} CdosMouseTrackingMode;

struct _CdosZoomRegion
{
    GObject  *parent_instance;

    GSettings   *settings;
    CdosMagnifier *magnifier;
    ClutterActor  *mouse_source_actor;
    ClutterActor  *mouse_actor;
    ClutterActor  *main_group;

    guint screen_position;
    guint mouse_tracking_mode;

    ClutterActor *ui_group_clone;
    StBin *mag_view;

    guint view_port_x;
    guint view_port_y;
    guint view_port_width;
    guint view_port_height;
    guint x_center;
    guint y_center;

    gfloat x_mag_factor;
    gfloat y_mag_factor;
    
    gboolean following_cursor;
    gboolean clamp_scrolling_at_edges;

    CdosZoomRegionPrivate  *priv;
};

struct _CdosZoomRegionClass
{
    GObjectClass parent_class;
};

GType cdos_zoom_region_get_type (void);

CdosZoomRegion *
cdos_zoom_region_new(CdosMagnifier *magnifier, ClutterActor *actor);

void cdos_zoom_region_set_clamp_scrolling_at_edges (CdosZoomRegion *self, gboolean clamp);
void cdos_zoom_region_set_mouse_source_actor (CdosZoomRegion *self, ClutterActor *actor);

gboolean cdos_zoom_region_scroll_to_mouse_position (CdosZoomRegion *self);

void cdos_zoom_region_set_magnifier (CdosZoomRegion *self, CdosMagnifier *magnifier);
void cdos_zoom_region_set_mouse_tracking_mode (CdosZoomRegion *self, CdosMouseTrackingMode mode);
void cdos_zoom_region_set_active (CdosZoomRegion *self, gboolean active);
void cdos_zoom_region_remove_mouse_source_actor (CdosZoomRegion *self);
void cdos_zoom_region_set_mag_factor (CdosZoomRegion *self, gfloat mag_factor);

#endif /* __CDOS_ZOOM_REGION_H__ */
