#include <glib-object.h>
#include <string.h>
#include <gio/gio.h>

#define GS_DBUS_NAME                            "org.freedesktop.ScreenSaver"
#define GS_DBUS_PATH                            "/org/freedesktop/ScreenSaver"
#define GS_DBUS_INTERFACE                       "org.freedesktop.ScreenSaver"

static void
do_screensaver_lock (void)
{
    GDBusProxy *screensaver_proxy;
    GError *error = NULL;
    GVariant *v;
    screensaver_proxy = g_dbus_proxy_new_for_bus_sync (G_BUS_TYPE_SESSION,
                                                        G_DBUS_PROXY_FLAGS_DO_NOT_AUTO_START,
                                                        NULL,
                                                        GS_DBUS_NAME,
                                                        GS_DBUS_PATH,
                                                        GS_DBUS_INTERFACE,
                                                        NULL,
                                                        &error);
    if (error)
    {
        g_warning ("Connecte screensaver dbus failed:%s", error->message);
        g_error_free (error);
        return ;
    }
    v = g_dbus_proxy_call_sync (screensaver_proxy,
                                "Lock",
                                NULL,
                                G_DBUS_CALL_FLAGS_NONE, 
                                -1,
                                NULL, &error);
    if (error)
    {
        g_warning ("Call screensaver lock failed:%s", error->message);
        g_error_free (error);
    }
    if (v)
        g_variant_unref (v);
    g_object_unref (screensaver_proxy);
}

static void
do_screensaver_settings (gint action)
{
    GSettings *gsettings;
    gboolean enabled;
    const gchar *str;
    gsettings = g_settings_new ("org.cdos.desktop.screensaver");
    enabled = g_settings_get_boolean (gsettings, "lock-enabled");
    if (action == 0)
    {
        str = enabled == TRUE ? "true" : "false";
        g_print ("%s\n", str);
    }
    g_object_unref (gsettings);
}

int
main (int argc, char *argv[])
{
    GOptionContext *context;
    guint retval = 0;
    gboolean lock = FALSE;
    gboolean show_help = TRUE;
    gboolean show_enabled = FALSE;
    gchar *help;

    const GOptionEntry options[] = {
        { "lock", 'l', 0, G_OPTION_ARG_NONE, &lock,
           /* command line argument */
          "Make the found desktop screensaver lock", NULL },
        { "lock-enabled", 'e', 0, G_OPTION_ARG_NONE, &show_enabled,
           /* command line argument */
          "Wether the screensaver lock enabled", NULL },
        { "help", 'h', 0, G_OPTION_ARG_NONE, &show_help,
           /* command line argument */
          "Show help options", NULL },
        { NULL}
    };

    context = g_option_context_new (NULL);
    g_option_context_set_summary (context, "Control the found desktop screensaver");
    g_option_context_add_main_entries (context, options, NULL);
    g_option_context_set_help_enabled (context, TRUE);
    g_option_context_parse (context, &argc, &argv, NULL);

    if (lock)
    {
        do_screensaver_lock ();
        goto out;
    }
    if (show_enabled)
    {
        do_screensaver_settings (0);
        goto out;
    }
    if (show_help)
    {
        help = g_option_context_get_help (context, TRUE, NULL);
        g_print ("%s", help);
        g_free (help);
    }

out:
    g_option_context_free (context);
    return retval;
}

