/*
 * SECTION:  cdos-pinyin.c
 * @Title: cdos-pinyin.c
 * @Short_Description:  
 */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <locale.h>
#include <wchar.h>
#include <unistd.h>

#include <glib.h>

#include "cdos-pinyin.h"

#define HANZI_COUNT (0x9fa5-0x4e00 + 1)

const char *pinyin_table[] = {
    "a", "ai", "an", "ang", "ao",
    "ba", "bai", "ban", "bang", "bao", "bei", "ben", "beng", "bi",
    "bian", "biang", "biao", "bie", "bin", "bing", "bo", "bu",
    "ca", "cai", "can", "cang", "cao", "ce", "cen",
    "ceng", "cha", "chai", "chan", "chang", "chao", "che",
    "chen", "cheng", "chi", "chong", "chou", "chu",
    "chua", "chuai", "chuan", "chuang", "chui", "chun", "chuo",
    "ci", "cong", "cou", "cu", "cuan", "cui", "cun", "cuo",
    "da", "dai", "dan", "dang", "dao", "de",
    "dei", "den", "deng", "di", "dia", "dian", "diao", "die", "ding",
    "diu", "dong", "dou", "du", "duan", "dui", "dun", "duo",
    "e", "ei", "en", "eng", "er",
    "fa", "fan", "fang", "fei", "fen", "feng",
    "fiao", "fo", "fou", "fu",
    "ga", "gai", "gan", "gang", "gao", "ge", "gei",
    "gen", "geng", "gong", "gou",
    "gu", "gua", "guai", "guan", "guang", "gui", "gun", "guo",
    "ha", "hai", "han", "hang", "hao", "he", "hei",
    "hen", "heng", "hong", "hou", "hu", "hua", "huai",
    "huan", "huang", "hui", "hun", "huo",
    "ji", "jia", "jian", "jiang", "jiao", "jie", "jin",
    "jing", "jiong", "jiu", "ju", "juan", "jue", "jun",
    "ka", "kai", "kan", "kang", "kao", "ke", "kei",
    "ken", "keng", "kong", "kou", "ku", "kua", "kuai",
    "kuan", "kuang", "kui", "kun", "kuo",
    "la", "lai", "lan", "lang", "lao", "le", "lei", "leng",
    "li", "lia", "lian", "liang", "liao", "lie", "lin", "ling", "liu",
    "lo", "long", "lou", "lu", "luan", "lun", "luo", "lv", "lve",
    "m", "ma", "mai", "man", "mang", "mao", "me", "mei",
    "men", "meng", "mi", "mian", "miao", "mie", "min", "ming",
    "miu", "mo", "mou", "mu",
    "na", "nai", "nan", "nang", "nao", "ne", "nei", "nen", "neng", "ng",
    "ni", "nian", "niang", "niao", "nie",
    "nin", "ning", "niu", "nong", "nou", "nu", "nuan", "nun", "nuo",
    "nv", "nve",
    "o", "ong", "ou",
    "pa", "pai", "pan", "pang", "pao", "pei", "pen", "peng",
    "pi", "pian", "piao", "pie", "pin", "ping", "po", "pou", "pu",
    "qi", "qia", "qian", "qiang", "qiao", "qie", "qin", "qing",
    "qiong", "qiu", "qu", "quan", "que", "qun",
    "ran", "rang", "rao", "re", "ren", "reng", "ri", "rong", "rou",
    "ru", "ruan", "rui", "run", "ruo",
    "sa", "sai", "san", "sang", "sao",
    "se", "sen", "seng",
    "sha", "shai", "shan", "shang", "shao", "she",
    "shei", "shen", "sheng", "shi", "shou", "shu",
    "shua", "shuai", "shuan", "shuang", "shui", "shun", "shuo",
    "si", "song", "sou", "su", "suan", "sui", "sun", "suo",
    "ta", "tai", "tan", "tang", "tao", "te", "tei", "teng",
    "ti", "tian", "tiao", "tie", "ting", "tong", "tou",
    "tu", "tuan", "tui", "tun", "tuo",
    "wa", "wai", "wan", "wang", "wei", "wen", "weng", "wo", "wu",
    "xi", "xia", "xian", "xiang", "xiao", "xie", "xin", "xing",
    "xiong", "xiu", "xu", "xuan", "xue", "xun",
    "ya", "yai", "yan", "yang", "yao", "ye", "yi", "yin", "ying",
    "yo", "yong", "you", "yu", "yuan", "yue", "yun",
    "za", "zai", "zan", "zang", "zao", "ze", "zei", "zen", "zeng",
    "zha", "zhai", "zhan", "zhang", "zhao",
    "zhe", "zhei", "zhen", "zheng", "zhi", "zhong", "zhou",
    "zhu", "zhua", "zhuai", "zhuan", "zhuang", "zhui", "zhun",
    "zhuo", "zi", "zong", "zou", "zu", "zuan", "zui", "zun", "zuo"
};

/* 
 * 从0x4e00到0x9fa5的所有字符拼音索引
 * 去除了音调及多音字的重复音节
 * 不包括“嘸”的ḿ音,
 * 部分日韩文字拼音为none。
 **/
uint16_t pinyin_data[] = {
    370,71,397,151,262,376,258,297,351,116,343,209,
    392,288,297,351,258,133,21,93,376,203,
    96,40,40,404,263,143,249,303,303,
    267,19,369,50,73,313,37,72,
    267,177,72,375,177,366,19,289,
    112,142,100,364,243,399,133,138,
    90,109,162,44,32,180,408,401,5,
    343,59,345,401,140,174,143,252,
    94,370,133,370,213,349,142,142,340,394,
    198,368,209,193,370,370,398,349,389,125,85,
    171,378,371,254,244,262,125,108,37,302,
    37,302,370,118,370,370,205,226,142,258,369,
    350,353,96,142,351,125,305,74,
    303,133,215,134,143,303,197,125,
    194,187,0,281,362,366,94,294,
    212,260,320,376,401,394,260,97,398,187,
    111,260,187,180,370,145,171,178,100,193,
    376,397,303,303,84,41,376,163,
    376,379,125,258,349,140,313,318,
    102,102,364,355,320,364,258,390,364,133,258,
    335,344,349,150,57,137,115,370,32,
    122,248,211,369,353,140,333,177,353,
    140,369,264,265,20,375,355,59,176,
    79,345,200,276,276,133,133,344,370,
    301,303,276,171,71,385,139,257,40,267,
    5,392,139,138,19,277,50,410,92,
    139,288,188,19,25,409,381,303,321,392,
    94,352,352,340,30,79,123,334,276,260,
    97,116,370,100,20,58,181,370,34,33,392,
    286,33,370,211,200,276,86,34,204,
    367,3,260,399,249,348,349,135,134,138,
    368,92,90,25,276,344,89,18,66,87,
    399,258,246,376,69,78,346,370,
    356,150,370,133,1,349,133,258,94,
    85,359,360,139,371,249,59,94,324,399,
    375,132,130,160,376,54,379,288,345,
    44,404,35,143,364,260,297,33,188,25,36,
    363,356,345,401,38,352,361,232,20,6,5,
    106,222,222,355,7,360,181,400,
    301,268,313,49,12,313,303,263,134,95,249,370,
    313,370,38,397,68,330,116,97,194,59,401,
    21,268,13,393,298,49,345,66,401,
    416,375,367,329,329,11,391,68,119,13,340,
    299,376,370,70,92,94,13,20,416,105,157,228,334,
    222,352,268,374,341,260,375,147,
    9,246,130,127,100,170,353,100,367,
    6,85,207,134,84,213,19,133,121,
    132,111,269,331,137,49,370,303,
    357,301,340,149,398,96,167,370,
    38,159,104,174,371,303,202,401,
    360,375,2,186,210,84,188,73,334,
    30,38,363,104,400,370,281,55,135,
    351,313,58,190,321,137,368,396,27,385,390,
    262,160,31,228,230,139,349,124,
    141,37,333,396,416,118,264,190,143,
    305,74,333,301,340,338,20,214,354,14,250,
    338,376,350,52,80,267,360,110,
    158,349,146,370,94,177,412,262,354,
    174,374,131,140,260,288,246,316,
    94,350,174,94,254,9,376,305,313,258,
    351,356,301,359,376,66,35,143,40,398,
    366,175,174,167,313,135,359,94,
    132,143,354,242,135,16,41,329,88,
    90,364,2,10,376,356,13,125,38,
    33,398,19,142,368,54,412,175,343,
    167,25,410,100,109,10,330,305,
    305,200,61,323,145,46,357,248,
    324,33,124,370,258,329,97,177,140,138,
    318,33,138,87,398,156,144,410,
    143,260,222,188,408,348,345,189,314,
    173,131,73,409,11,349,143,213,
    23,135,390,369,398,294,265,228,
    372,37,36,260,366,282,399,334,47,134,
    133,138,345,376,19,285,329,345,250,
    366,90,324,60,348,80,355,35,302,
    149,66,416,30,333,10,355,129,
    368,391,40,262,2,375,135,360,389,
    49,94,13,398,410,203,133,370,
    355,363,23,313,76,27,385,390,396,240,335,
    335,10,380,190,185,138,345,89,33,
    163,111,315,398,313,316,351,94,377,279,
    174,232,379,136,105,193,8,68,324,
    118,138,350,296,260,135,270,145,25,36,41,
    288,10,354,279,368,321,323,320,367,
    85,19,134,58,381,324,106,18,
    41,235,24,172,54,374,384,26,410,
    248,314,4,44,404,376,390,258,51,297,
    45,140,38,294,116,392,265,366,
    66,355,190,185,10,251,16,139,176,186,
    195,260,352,323,372,73,404,353,
    296,262,141,338,415,257,350,170,
    33,110,178,258,37,65,391,404,32,345,133,
    20,130,44,332,135,59,137,368,142,293,
    89,352,376,143,80,349,137,135,334,405,180,
    20,106,352,316,352,136,206,369,
    139,134,138,262,249,90,400,1,287,
    370,146,230,32,323,296,370,60,140,361,
    160,135,41,59,137,294,381,24,
    18,2,281,322,40,31,168,222,370,
    139,260,201,349,228,266,222,33,
    179,172,190,162,9,376,16,382,
    398,313,375,118,265,36,174,328,
    345,184,41,32,273,353,305,130,355,174,
    189,382,235,324,366,172,215,84,
    349,379,382,377,358,39,393,358,
    352,110,77,283,378,152,77,283,378,203,336,33,392,
    84,77,283,378,84,264,336,313,366,366,
    303,303,152,60,260,74,89,197,301,
    74,6,152,140,174,129,281,344,218,
    269,177,376,305,5,104,182,186,350,116,
    168,104,330,109,357,19,258,133,143,
    68,409,49,89,367,135,304,133,370,
    133,32,141,197,272,218,212,377,197,
    98,272,27,141,27,381,107,141,
    197,400,197,209,105,360,203,202,279,
    371,375,355,149,146,230,370,202,303,
    109,201,399,414,377,207,157,180,
    94,355,202,19,73,322,98,90,254,
    19,125,39,145,364,162,369,173,
    243,85,206,73,352,179,259,135,
    140,37,315,199,336,258,106,407,314,
    140,37,177,265,69,181,73,97,135,
    371,51,1,174,25,207,407,54,
    313,79,139,180,180,228,350,75,
    133,86,86,86,90,143,41,397,
    90,211,398,94,90,254,90,148,
    129,148,97,65,254,149,260,358,160,
    336,4,341,41,133,60,116,116,384,
    61,69,61,276,276,45,89,263,
    370,133,149,260,55,41,346,133,
    59,357,126,343,145,174,378,179,
    182,385,98,45,94,41,268,69,
    296,206,181,399,243,17,138,138,
    245,9,174,296,17,32,140,107,103,
    61,45,163,158,79,84,398,306,
    269,361,30,294,49,152,138,111,49,111,
    148,79,133,329,140,74,189,385,
    377,56,354,362,153,152,166,260,30,294,45,
    107,135,56,174,329,88,256,32,
    258,45,409,98,343,9,20,133,79,
    265,366,296,75,408,135,133,9,20,366,143,
    132,302,135,79,398,76,349,107,94,249,
    302,135,100,57,389,148,45,44,32,
    337,404,186,142,174,248,296,251,157,137,34,
    107,262,145,126,389,408,176,143,
    249,182,111,137,34,111,135,135,324,
    132,133,135,370,135,398,32,413,
    209,174,401,174,364,269,7,104,
    134,349,194,179,139,140,155,355,179,398,
    73,401,41,232,138,268,298,370,401,
    204,174,139,140,170,170,144,157,367,
    341,354,210,162,138,179,119,303,
    152,139,140,99,20,10,206,38,169,374,
    374,203,152,363,144,265,186,21,
    201,38,171,172,148,203,73,360,360,
    149,349,370,363,347,367,302,170,211,
    186,251,303,133,264,136,137,34,269,
    353,370,262,86,144,334,73,143,59,
    355,194,363,363,190,174,35,273,353,
    269,9,298,379,142,9,105,349,
    379,0,358,96,96,9,50,370,
    358,248,143,325,368,100,257,80,245,
    94,104,57,142,104,13,126,10,
    216,38,303,87,142,370,380,136,150,
    136,162,125,351,268,86,111,263,
    383,25,162,88,125,376,111,163,111,130,
    59,163,111,176,176,317,75,142,145,
    350,249,268,240,370,152,259,366,14,222,
    268,240,303,363,260,223,286,412,302,
    349,130,7,303,350,343,126,355,
    343,10,412,52,408,355,59,296,32,194,214,212,
    59,133,20,307,190,21,20,109,162,14,21,
    391,259,147,186,375,186,350,107,348,
    355,138,138,345,367,3,266,398,197,
    371,345,298,133,270,187,38,144,
    355,360,139,270,349,133,80,265,
    350,288,33,2,116,345,366,80,333,174,394,390,
    116,2,174,364,364,366,299,66,389,390,
    244,364,263,364,398,303,27,196,329,
    174,299,124,333,414,56,88,377,
    27,377,353,366,174,145,294,351,68,
    41,142,139,4,111,366,313,174,
    33,260,168,174,366,366,377,313,210,104,123,
    180,204,280,267,268,268,0,172,75,352,361,
    404,288,24,301,28,288,24,301,28,288,24,301,28,288,24,301,28,288,1,370,58,
    375,30,133,375,309,86,304,108,
    5,85,285,174,305,408,370,174,145,268,304,
    14,360,134,243,315,133,345,376,315,
    70,283,50,157,106,143,105,181,107,
    325,61,157,398,137,393,298,5,71,152,
    322,38,303,375,267,255,369,355,118,
    313,323,38,171,69,133,0,123,
    205,360,376,196,38,100,361,314,368,409,
    119,100,133,69,74,55,334,207,124,174,
    336,353,389,351,119,369,190,364,0,193,
    240,132,370,146,40,180,339,371,
    88,249,13,264,264,138,100,21,93,249,5,
    78,89,80,126,116,333,117,155,311,258,
    123,398,409,371,301,349,349,34,212,217,362,48,145,
    350,46,74,281,346,124,124,123,240,349,376,99,
    364,146,190,80,100,346,58,258,
    37,349,99,94,137,123,38,302,
    212,217,339,330,94,370,58,240,174,10,6,
    377,379,341,107,113,126,258,261,349,80,303,144,
    247,346,206,222,217,192,181,272,375,66,
    400,303,400,332,35,350,370,258,398,254,
    409,49,107,106,409,49,345,360,124,105,119,0,152,216,351,
    246,370,354,118,301,125,207,57,59,268,
    143,414,352,97,380,340,79,256,245,13,
    94,367,119,380,385,389,119,132,125,115,142,374,
    94,57,400,341,147,106,147,95,416,
    21,184,73,228,340,313,352,132,
    258,84,80,110,389,70,350,370,179,
    409,205,202,398,368,133,350,259,400,147,189,100,
    305,363,380,354,152,115,130,159,127,303,325,
    352,80,2,361,359,341,107,113,366,369,170,370,
    1,253,301,334,123,358,79,341,
    114,381,375,70,66,242,353,1,102,121,
    162,364,57,354,13,378,130,223,126,
    357,160,79,89,133,138,390,230,210,373,
    118,377,379,184,256,196,100,238,80,38,
    298,174,212,218,217,412,119,158,354,352,
    170,255,5,20,394,389,177,169,5,205,179,190,
    318,94,21,116,122,103,48,378,100,134,
    375,366,106,106,10,6,116,320,47,
    370,1,134,260,336,336,59,352,366,343,174,350,
    324,416,267,35,349,384,364,74,
    258,66,264,193,0,123,104,74,0,
    170,177,320,384,128,173,294,133,
    412,348,345,90,139,371,125,351,258,304,345,
    306,33,84,341,174,261,2,138,385,133,373,
    223,376,330,167,294,350,340,125,
    1,400,331,105,154,408,408,393,297,66,
    122,168,180,0,23,261,407,339,353,77,349,346,
    54,258,294,138,70,329,106,258,258,325,59,59,
    378,341,409,49,13,336,54,48,43,119,364,258,
    394,88,177,352,249,294,166,385,
    265,372,107,241,385,303,291,404,226,113,
    189,366,66,269,323,32,340,20,71,169,
    354,0,324,38,329,2,142,59,
    147,374,345,214,296,376,394,166,
    138,124,116,70,389,400,31,342,235,275,
    132,113,360,371,380,368,238,348,203,125,379,
    44,130,128,128,377,361,119,350,119,369,133,163,
    399,45,345,294,360,129,79,389,366,361,
    177,376,289,38,262,137,366,59,296,32,247,11,
    24,319,258,174,373,389,30,345,204,372,247,
    21,163,350,376,138,185,158,384,262,
    125,329,368,119,354,125,294,0,359,261,291,
    374,316,123,104,355,370,1,320,193,30,
    115,152,321,57,289,36,281,315,341,106,
    133,12,244,349,352,260,263,303,100,409,138,
    170,347,341,313,38,118,320,134,188,
    115,120,320,264,226,119,409,287,221,82,
    100,212,67,1,261,334,13,4,
    4,176,414,318,394,401,209,315,315,323,
    66,258,137,39,137,61,148,100,323,296,24,
    26,134,1,354,251,185,95,106,
    354,137,125,130,113,240,352,385,33,
    360,303,255,62,63,193,193,125,172,171,75,
    95,324,369,12,372,287,137,202,
    354,126,194,272,416,248,170,354,
    133,401,34,393,163,414,354,313,118,
    94,178,262,350,41,360,304,323,32,59,323,120,209,
    363,80,415,86,20,38,130,382,45,
    52,380,119,59,145,339,165,29,137,369,350,
    258,118,176,360,303,65,130,371,257,
    145,264,363,226,186,313,366,372,
    57,391,238,400,139,230,378,130,355,
    258,80,384,370,303,137,262,38,377,1,
    374,145,362,160,376,247,61,95,356,121,
    78,60,356,287,249,249,371,414,
    228,66,168,321,132,238,281,118,119,351,
    366,79,359,249,400,40,133,138,390,139,118,329,
    33,363,198,22,30,329,186,130,20,245,9,
    375,226,371,125,199,198,209,123,394,174,
    182,355,115,215,354,209,366,174,186,
    184,255,59,36,253,249,353,132,
    198,350,79,158,366,32,372,273,
    68,166,321,354,137,145,48,128,132,
    404,226,354,380,22,174,32,31,174,
    370,189,215,380,382,24,316,350,388,135,
    366,380,226,401,168,226,215,168,189,345,113,
    130,371,267,313,227,135,214,130,356,
    371,214,337,337,78,339,150,377,141,
    250,379,50,125,130,377,80,113,
    164,50,345,334,336,345,188,113,271,
    278,181,106,113,322,113,336,375,
    113,371,131,257,376,116,377,188,
    269,144,376,265,113,44,46,345,377,269,144,
    158,257,377,377,364,337,336,336,
    337,191,130,370,128,377,187,187,336,
    364,336,333,302,257,186,160,364,
    381,345,360,100,376,407,349,111,249,370,
    66,62,260,316,260,396,408,60,259,351,
    296,162,33,258,371,226,209,133,134,
    398,398,7,363,370,264,199,89,146,
    279,155,339,78,87,11,89,11,323,149,127,249,246,
    416,155,13,140,66,168,140,133,160,
    66,140,135,323,174,5,349,89,
    406,255,243,7,324,164,268,323,398,
    340,97,254,68,107,222,322,249,
    141,367,92,4,186,267,211,152,
    105,362,85,66,38,35,181,401,94,
    125,398,46,166,184,184,186,4,
    58,245,206,357,73,334,133,119,190,
    49,38,172,96,371,124,77,393,
    94,110,368,79,79,111,30,367,
    371,85,105,377,70,355,154,297,
    304,80,19,68,123,364,159,57,
    147,60,148,117,216,2,357,352,
    377,128,8,256,94,5,370,371,116,360,
    46,28,103,1,12,90,66,87,270,145,374,
    146,351,134,66,194,195,169,144,37,366,296,
    264,139,394,179,179,257,21,37,126,21,
    303,363,113,141,369,223,66,376,
    21,364,269,318,316,249,265,397,343,143,
    188,397,37,156,39,297,73,58,323,2,
    23,41,335,12,352,149,398,79,370,303,398,
    370,246,133,407,258,290,143,222,
    158,152,324,164,222,135,77,139,
    98,376,80,248,12,106,336,173,181,87,
    364,260,382,135,164,2,301,79,130,216,336,
    37,371,128,13,176,113,70,404,
    124,9,21,257,9,376,66,197,210,349,138,282,
    80,1,369,103,149,410,376,129,80,368,
    366,9,133,199,33,75,340,371,
    90,399,138,139,90,98,44,135,
    254,172,136,129,173,76,343,361,
    133,133,160,372,321,37,374,148,
    316,316,303,202,321,347,37,336,
    324,270,399,174,248,8,287,291,383,
    77,330,349,397,363,100,396,1,
    104,366,352,330,396,377,346,355,182,
    115,169,33,248,12,36,186,186,
    240,267,260,382,135,199,209,404,337,309,305,185,
    38,195,16,140,258,305,398,66,392,
    149,374,68,36,398,408,350,113,261,
    139,66,297,211,54,366,321,388,
    260,261,177,345,406,262,388,360,
    296,296,85,257,160,338,337,73,86,262,270,
    209,78,78,415,78,66,302,79,130,79,
    323,65,349,89,129,323,57,369,
    401,135,4,261,133,262,4,154,370,336,
    249,13,68,136,369,374,362,20,145,323,
    168,143,127,60,273,260,363,352,168,
    350,119,1,364,61,118,282,139,
    172,162,186,366,323,345,127,184,
    184,283,174,180,273,32,363,366,
    172,5,343,303,276,288,405,405,
    302,370,194,152,262,401,405,125,125,
    164,370,125,360,164,304,196,55,
    304,370,398,399,106,372,41,41,136,353,90,244,10,
    390,14,318,271,181,94,56,351,
    358,361,355,216,351,163,350,342,377,343,
    197,343,316,79,79,369,265,342,105,
    105,258,201,201,371,132,36,57,58,322,
    27,330,322,94,108,368,367,117,11,
    99,303,325,11,322,335,366,325,13,370,
    159,134,95,351,79,126,162,379,134,95,351,5,
    82,176,128,66,329,366,245,144,258,133,
    213,90,355,89,68,269,144,163,411,
    128,258,263,355,148,299,38,389,11,370,136,325,
    383,405,11,350,129,88,69,363,406,12,
    68,4,299,347,255,114,322,4,376,349,4,376,
    136,176,79,379,136,303,89,132,
    13,187,79,35,236,281,232,71,330,213,260,
    135,321,138,142,233,30,118,352,86,
    133,312,281,88,246,344,123,405,94,
    193,59,276,94,375,140,366,115,138,346,
    399,241,75,133,155,117,399,368,139,
    379,204,93,246,249,38,378,145,405,229,366,
    212,214,356,89,13,376,340,90,343,377,
    87,349,376,111,75,5,222,400,
    408,393,57,222,213,377,335,352,361,360,398,370,
    80,199,209,258,13,301,263,80,
    119,360,85,397,206,7,211,94,
    181,409,409,303,272,296,367,195,
    138,106,313,357,345,409,143,296,
    253,276,368,73,136,305,133,96,
    353,126,132,144,137,354,105,75,211,170,135,135,
    370,223,398,396,133,352,122,110,
    146,363,159,125,366,207,179,246,80,364,375,
    366,30,301,352,371,303,111,269,409,
    314,345,123,341,185,364,274,137,
    187,254,352,298,174,37,302,355,196,
    94,320,349,211,345,152,48,167,48,333,
    224,357,214,376,212,235,256,13,218,318,144,
    301,398,116,66,405,80,253,338,
    195,203,349,376,366,349,350,1,366,376,
    313,376,341,174,352,143,268,406,310,
    258,352,408,73,33,186,1,80,80,
    80,185,203,50,256,246,21,143,255,23,
    181,343,16,354,305,258,130,94,86,
    348,348,323,88,88,138,330,222,
    269,144,140,131,140,260,139,68,357,125,
    343,341,167,13,371,400,40,48,216,94,140,
    188,237,168,131,164,371,364,143,174,
    68,352,126,126,372,32,301,333,
    60,367,368,349,214,285,48,134,335,376,360,
    376,345,66,329,280,199,59,282,219,264,
    130,348,260,47,204,94,138,76,
    370,246,399,199,129,203,2,372,361,
    138,345,199,377,397,267,329,355,
    340,79,176,197,272,313,250,345,341,
    52,125,4,138,9,360,335,376,111,
    41,411,368,249,350,377,372,279,281,
    38,182,199,243,4,193,105,163,
    264,301,134,290,396,377,138,320,279,207,
    372,133,316,225,352,325,244,169,
    216,16,1,249,253,370,251,376,
    172,361,195,370,392,150,374,222,
    174,66,111,366,139,404,33,385,
    116,214,219,170,209,394,125,125,4,
    219,261,193,252,106,349,262,340,
    391,204,352,352,209,178,176,126,
    111,65,398,360,370,126,350,163,
    274,350,366,32,137,199,86,86,
    352,366,139,370,130,137,94,303,13,296,
    318,261,176,128,361,266,356,225,73,370,
    24,1,224,228,209,331,40,139,
    49,376,253,279,281,213,366,322,
    372,260,225,378,372,203,13,209,
    301,357,222,75,182,377,168,366,
    309,181,137,224,168,352,260,372,309,
    355,130,128,269,202,174,187,366,401,48,168,
    409,138,145,145,156,379,409,193,409,
    55,319,363,94,10,409,354,356,201,
    313,322,9,133,106,232,362,375,229,
    404,115,187,319,363,216,205,50,260,
    305,32,24,364,409,222,94,409,174,
    362,20,281,213,226,226,372,187,
    203,228,401,279,321,111,390,266,376,
    304,2,336,134,314,343,280,368,123,
    370,140,407,202,94,401,60,123,410,
    109,400,71,343,377,370,9,303,303,
    39,301,152,361,303,375,128,370,
    331,303,352,104,37,271,104,354,
    381,389,9,303,115,366,354,134,138,301,
    36,279,129,202,157,161,18,316,359,
    23,382,133,377,133,371,202,157,
    265,119,396,135,94,228,19,128,
    199,264,116,376,303,228,139,264,228,
    398,376,9,161,228,264,209,30,
    143,185,107,264,125,349,178,303,228,
    390,301,345,355,161,130,178,146,
    128,370,370,9,264,39,9,90,
    55,77,313,363,61,191,189,77,304,
    255,90,404,94,299,369,370,153,152,136,136,
    404,345,376,415,363,305,401,77,61,354,
    138,133,298,84,84,84,95,135,305,
    36,297,297,209,95,33,178,352,
    352,131,375,344,375,178,178,368,
    184,196,201,244,344,344,344,95,368,79,163,
    399,142,97,106,97,338,97,97,
    303,371,38,35,151,222,139,345,370,225,318,
    143,249,29,350,13,143,138,330,
    268,329,138,349,69,303,303,254,19,
    133,355,396,350,222,391,350,345,
    195,80,185,254,19,329,88,305,401,355,329,
    336,190,190,350,29,190,143,355,
    143,145,178,145,305,401,350,35,26,339,407,
    222,133,296,341,352,174,2,130,130,
    123,184,370,258,276,349,116,2,301,376,
    41,318,258,148,276,378,7,368,3,
    364,349,138,80,133,260,89,343,
    258,28,260,258,30,138,268,98,
    352,4,168,61,5,416,416,367,
    143,98,152,105,362,255,174,331,
    143,143,366,94,359,134,181,340,249,
    4,58,162,378,268,125,255,206,
    2,331,181,66,254,73,391,163,
    359,197,334,362,370,14,119,152,5,
    189,80,94,226,363,70,186,82,84,
    96,269,334,73,370,211,303,2,345,
    128,398,303,202,174,85,334,345,375,
    259,351,174,368,137,262,397,187,137,
    80,80,376,355,369,21,262,271,90,
    90,216,174,375,352,279,61,301,
    37,336,103,146,99,351,371,349,
    169,149,170,167,352,270,156,39,
    39,321,180,126,143,167,258,206,
    164,164,412,54,106,54,364,364,98,
    188,188,181,173,145,79,397,113,371,
    73,116,397,345,354,249,13,366,314,
    138,12,412,145,73,391,32,106,371,
    409,385,129,376,342,345,367,60,90,267,
    367,329,370,398,303,303,70,381,368,80,
    401,149,391,190,366,199,116,133,133,
    128,333,302,199,260,149,349,197,376,410,
    168,152,138,366,366,345,410,30,318,
    279,152,264,376,258,185,336,54,
    350,347,25,60,324,279,372,138,148,1,182,
    349,314,151,262,409,345,12,68,56,
    264,260,374,226,56,133,303,285,314,
    410,136,178,150,32,70,66,28,71,
    336,185,392,391,32,391,32,4,26,268,
    261,345,414,61,61,350,376,249,246,
    184,353,29,20,264,137,366,170,
    391,180,178,178,264,65,340,415,
    137,262,145,111,368,137,368,145,391,296,370,
    362,216,369,369,370,226,352,133,
    355,138,152,138,111,350,144,66,4,414,345,370,
    279,61,181,138,376,378,371,281,
    138,174,179,111,350,144,184,184,68,372,123,350,
    143,32,372,163,366,345,216,269,
    34,53,187,68,68,226,366,366,
    366,163,366,44,160,44,400,129,
    140,357,363,34,34,179,104,416,262,
    143,104,0,349,106,106,30,31,49,267,
    267,133,370,313,5,398,393,353,117,
    370,139,363,144,5,363,139,94,
    380,13,303,21,71,307,86,226,
    303,89,241,398,350,125,59,345,
    392,324,232,58,209,341,246,241,332,94,
    176,398,400,20,398,66,209,370,
    370,254,259,144,281,307,58,396,
    310,262,396,303,271,350,8,58,
    111,40,61,254,392,135,288,343,58,345,
    33,294,263,258,133,385,113,197,401,124,
    396,397,202,345,348,94,370,8,
    254,70,104,243,129,325,202,134,
    328,130,399,296,262,301,195,211,16,113,
    385,211,8,392,140,32,94,398,
    125,86,45,405,13,13,392,202,262,
    32,89,201,8,40,61,205,41,138,
    352,168,97,254,223,135,19,19,
    357,97,368,128,375,375,133,110,2,
    249,333,385,110,405,209,193,198,265,13,
    264,78,339,45,111,364,6,333,138,360,
    186,349,405,158,372,66,62,245,68,
    364,204,103,49,94,334,244,88,
    353,370,398,331,398,359,75,79,416,
    354,336,111,158,196,201,244,333,375,21,
    19,254,37,167,10,133,56,2,305,150,
    374,340,314,305,265,376,376,204,
    315,27,353,88,142,80,111,345,130,182,
    294,351,176,169,315,398,21,265,142,
    142,139,264,4,165,185,371,178,58,
    186,370,41,32,336,313,356,204,
    33,349,88,110,158,160,13,261,291,
    355,180,180,178,186,133,372,352,
    333,374,174,333,371,363,366,333,
    66,255,242,135,130,213,130,104,223,
    148,14,370,258,230,184,89,143,366,
    370,383,13,370,370,84,288,303,
    84,303,303,104,69,371,125,94,
    123,349,338,38,136,5,301,66,329,338,
    392,145,392,325,94,66,202,352,125,
    34,232,140,396,370,202,144,269,343,
    298,285,361,377,140,69,392,136,261,136,
    248,59,323,261,136,13,13,299,59,323,135,
    105,100,85,13,157,135,17,354,
    59,323,113,261,136,123,202,113,343,145,
    133,362,133,111,60,186,186,337,130,
    398,130,130,370,370,370,370,132,
    132,296,352,357,346,334,366,366,376,
    38,23,16,69,18,248,8,374,251,251,
    392,372,38,38,408,20,340,370,133,244,87,
    399,370,344,35,13,66,181,94,
    344,397,52,344,140,58,350,363,
    121,367,127,190,124,344,134,341,37,397,398,
    360,140,336,50,50,167,50,62,63,62,
    242,350,73,133,33,398,50,410,400,
    167,376,355,138,135,303,329,134,351,14,
    129,94,363,345,244,368,345,350,
    397,251,329,38,62,398,397,398,397,17,62,
    399,39,35,137,368,130,137,130,199,184,
    353,9,268,143,356,356,13,370,171,
    276,61,71,333,96,133,276,276,32,
    323,326,326,338,97,116,370,258,303,322,55,398,
    344,196,350,179,86,372,330,206,346,206,346,
    399,39,349,133,349,350,134,375,
    343,50,314,399,160,376,305,14,398,258,303,
    54,36,322,339,407,78,260,264,223,131,358,
    229,162,344,352,356,150,117,125,148,350,89,
    127,322,314,349,240,33,45,143,
    370,9,34,206,200,246,416,389,387,367,
    157,143,7,232,216,229,397,241,21,332,391,
    125,106,125,52,143,412,57,176,313,287,375,40,66,
    58,370,336,70,375,94,133,248,357,
    377,222,108,94,350,13,375,368,263,
    361,50,19,129,360,362,41,13,249,305,
    350,305,323,374,410,77,202,398,370,
    303,219,227,363,303,350,170,122,162,
    210,398,355,176,331,368,129,70,118,
    156,111,122,350,258,360,354,137,305,313,125,159,
    267,367,130,130,38,134,370,358,
    108,180,130,409,360,38,297,236,
    121,82,152,73,330,104,269,404,350,
    259,378,248,154,62,130,80,349,267,
    334,366,148,27,216,379,196,374,
    374,377,144,249,249,164,262,378,376,305,336,
    138,152,350,394,180,329,116,118,137,263,
    329,21,370,260,130,350,10,195,200,
    370,122,314,269,37,163,174,349,349,
    375,174,177,128,50,370,223,378,174,
    227,216,80,270,361,260,349,206,
    50,88,10,62,54,33,200,174,
    133,109,109,357,61,258,156,330,
    188,350,149,112,222,265,40,78,
    113,391,140,343,377,343,139,133,168,180,
    376,360,132,119,144,269,323,59,329,329,223,
    344,48,46,125,131,200,350,33,356,345,
    130,80,349,320,283,410,135,374,68,143,
    24,37,62,10,263,24,59,57,109,
    79,216,379,353,406,70,129,47,
    266,275,357,27,14,206,410,329,303,
    262,40,10,361,345,100,260,345,
    376,376,335,13,361,128,206,13,370,
    203,374,258,148,60,297,324,367,371,80,36,356,59,197,
    152,259,152,376,1,263,366,235,97,
    379,50,314,287,173,89,372,163,163,
    270,104,123,379,316,316,312,258,368,314,
    129,133,106,143,45,222,355,148,
    397,374,26,363,301,20,148,350,377,
    350,355,131,374,367,174,26,290,325,371,
    49,360,41,260,263,322,129,379,301,207,
    104,123,299,26,50,251,211,211,113,38,
    24,24,24,54,206,326,392,334,
    4,309,195,109,270,384,142,130,
    148,176,240,314,264,139,371,190,297,
    345,337,195,260,299,374,265,150,
    66,38,398,394,185,190,144,258,258,376,254,
    178,50,375,39,398,334,37,258,
    268,248,10,17,266,137,388,38,
    176,254,163,130,262,37,65,397,371,371,
    350,350,59,57,323,79,77,77,78,339,316,
    145,27,354,137,86,89,170,170,39,
    116,258,352,206,140,178,349,24,
    145,52,352,323,302,249,370,41,
    352,216,215,59,323,140,314,116,137,133,
    345,361,128,73,264,264,143,26,290,154,
    355,372,4,197,370,180,291,146,
    127,200,168,1,180,366,113,351,
    38,376,371,58,201,1,370,222,201,77,
    258,133,209,168,352,200,40,398,235,235,
    366,367,20,398,162,162,375,94,
    182,205,37,130,32,201,168,127,
    361,273,32,133,143,128,299,370,
    176,214,202,209,324,145,98,405,98,405,98,405,
    100,378,349,135,360,305,279,350,125,
    37,348,138,100,135,261,132,261,
    391,73,258,134,70,386,134,133,
    398,149,133,163,96,65,391,261,
    100,135,138,376,135,366,186,350,125,
    391,350,125,350,125,48,58,268,125,125,
    125,80,303,329,197,125,174,87,
    320,14,250,68,141,297,141,370,370,296,
    125,88,366,304,304,23,380,389,267,
    171,174,23,257,5,241,57,57,277,86,94,281,381,
    340,392,69,66,378,174,150,98,376,349,376,349,158,116,301,
    30,340,38,370,106,350,100,138,157,349,64,260,398,
    276,165,200,290,367,229,7,35,
    274,350,30,258,260,264,7,134,376,94,5,4,
    350,394,249,398,398,319,149,80,64,393,37,
    133,366,162,344,381,14,34,143,346,125,106,
    378,145,5,264,59,301,397,379,343,
    217,222,283,212,370,305,402,256,335,74,150,
    394,299,256,94,94,245,5,4,229,385,337,
    157,188,261,37,379,125,9,19,398,390,
    248,214,21,257,249,322,368,325,396,389,
    367,9,119,259,222,369,66,38,249,246,
    134,209,193,199,36,364,40,268,206,
    401,134,364,94,13,389,401,59,31,22,211,
    223,166,94,245,7,243,242,180,212,
    108,260,143,340,321,398,5,340,340,4,229,
    143,105,408,243,253,86,393,6,6,66,222,
    143,165,184,135,259,374,168,228,
    20,385,390,260,121,165,107,303,138,134,397,
    227,104,104,269,308,55,415,380,382,151,
    370,38,115,355,27,291,48,130,253,403,369,303,299,212,
    6,38,107,398,165,110,79,79,79,398,
    263,2,230,396,100,137,159,158,73,
    281,212,331,179,389,190,70,299,341,145,
    179,143,398,187,364,402,348,321,355,134,
    216,60,137,397,133,130,352,376,
    1,340,310,235,56,20,103,329,396,
    37,320,294,320,294,155,260,199,230,143,8,248,
    135,370,333,296,285,343,355,134,30,
    248,137,158,349,146,142,334,164,132,38,
    336,305,30,408,256,94,189,190,5,116,298,226,
    144,385,305,314,315,369,376,145,408,21,343,21,257,398,
    415,369,390,190,315,340,310,170,319,
    8,135,128,61,345,343,376,264,248,
    299,179,206,200,94,21,6,143,61,
    348,189,1,144,269,378,410,36,46,138,
    336,11,212,223,226,285,345,275,416,348,351,258,
    352,37,68,290,188,265,260,98,79,
    304,69,256,66,392,131,133,325,
    259,258,242,305,260,343,181,369,364,
    145,397,177,107,222,226,370,132,360,296,366,397,71,
    191,23,323,35,19,138,329,156,
    338,366,56,411,400,40,143,330,260,154,
    6,241,138,186,113,207,138,398,
    59,296,201,32,352,24,296,290,109,248,377,235,
    135,397,155,142,375,135,376,366,163,214,
    123,280,249,35,345,287,410,23,411,361,204,
    329,66,226,30,303,410,314,396,370,363,
    129,374,14,367,128,366,382,413,2,360,143,
    364,348,152,259,43,337,406,133,329,66,166,166,
    37,148,142,142,336,138,258,130,102,
    39,73,354,299,70,369,355,377,260,135,369,30,
    389,10,368,345,12,168,346,264,
    32,100,185,410,102,137,105,264,
    279,270,40,411,43,391,319,319,20,
    41,279,215,8,248,56,290,152,80,368,61,
    398,232,235,231,166,355,352,135,315,267,99,352,
    312,289,139,205,80,46,235,296,
    321,138,394,324,243,7,255,7,57,174,325,
    125,398,213,341,126,260,346,261,37,330,301,
    396,80,355,212,235,269,30,389,100,
    349,82,299,98,299,226,305,6,368,
    18,315,323,286,294,295,32,319,320,142,182,178,137,216,39,
    45,113,19,90,248,307,66,336,398,258,133,30,315,314,
    390,176,37,38,109,186,189,185,
    410,96,350,125,41,389,261,324,126,54,
    398,213,209,193,136,261,111,372,398,4,262,398,
    226,35,195,32,24,157,41,291,202,316,337,137,34,
    209,209,394,32,352,24,296,155,260,16,136,368,
    105,260,178,133,372,145,252,252,
    170,78,352,282,111,382,387,260,370,352,
    37,37,286,216,123,313,116,122,110,
    57,415,223,180,397,37,130,345,405,137,
    133,26,59,59,296,35,20,35,145,
    354,315,178,11,94,262,20,56,416,408,
    404,345,340,257,264,78,223,126,355,
    186,137,53,321,116,262,368,133,402,348,135,
    97,374,172,215,186,296,408,385,390,
    257,48,133,60,291,26,265,265,140,
    128,138,264,160,59,355,259,134,369,249,20,
    20,6,4,143,369,80,201,315,202,
    133,322,408,61,357,168,22,143,
    369,281,369,369,222,132,138,18,
    228,100,398,398,138,165,209,135,355,
    179,166,323,6,315,186,174,189,378,274,329,398,390,
    243,367,172,22,286,305,382,223,352,
    146,246,132,174,189,166,167,128,372,186,189,184,
    260,260,382,53,260,168,352,135,372,199,
    273,32,347,53,355,299,226,189,146,
    202,209,38,382,53,187,323,413,174,295,68,
    341,60,137,145,168,174,189,215,398,
    111,111,111,258,370,133,363,257,257,304,151,
    375,96,370,104,97,116,7,87,397,
    255,68,157,206,349,210,106,119,27,
    354,202,41,304,100,113,80,66,360,137,206,
    36,142,301,79,77,376,38,4,6,
    360,137,79,77,176,226,13,33,68,
    79,270,370,97,288,152,366,78,77,258,370,133,
    335,354,362,79,270,137,140,367,351,206,
    305,312,1,401,262,1,401,397,66,36,94,
    305,312,178,268,358,361,370,137,296,137,
    408,401,370,75,176,13,174,322,354,354,346,
    362,258,258,390,18,145,137,390,169,
    88,7,7,168,376,399,168,345,200,74,
    302,178,134,125,355,134,376,396,
    137,348,109,335,331,74,139,38,371,398,94,
    261,391,268,408,391,76,408,313,
    356,408,408,264,180,408,41,76,
    401,87,32,138,117,376,349,303,246,182,375,
    199,244,8,258,391,197,190,246,249,13,
    182,94,87,361,140,140,222,412,
    393,370,182,298,135,376,370,258,
    398,86,251,86,391,160,318,376,
    349,133,133,133,132,278,59,142,
    398,384,355,331,363,360,95,166,
    97,116,116,322,372,66,66,62,360,32,303,162,
    367,303,344,206,206,339,407,47,349,
    379,10,3,385,7,138,164,302,
    125,87,118,111,33,361,207,131,
    89,264,125,370,350,356,366,385,
    87,323,301,143,367,382,19,357,
    372,361,255,396,181,47,118,199,
    416,209,14,360,131,393,410,303,
    303,376,88,70,370,197,222,33,346,
    73,1,19,3,400,184,352,162,
    331,34,303,129,129,129,361,163,360,159,
    137,139,398,139,297,334,123,366,
    96,353,295,354,369,379,130,116,
    116,146,343,352,164,400,350,302,37,
    302,21,394,394,349,343,130,118,
    36,343,330,408,414,400,257,140,372,
    350,296,222,350,265,258,75,140,111,
    397,370,398,2,366,343,180,177,37,
    344,354,382,88,361,361,370,351,
    379,130,360,206,163,369,372,305,75,
    345,305,265,197,214,135,168,233,2,
    367,47,368,320,257,207,137,148,
    118,347,33,258,118,366,174,1,
    133,133,200,382,355,118,211,211,
    50,222,392,130,9,257,116,361,44,
    178,352,323,140,252,180,339,350,
    370,133,129,58,369,369,174,323,
    334,354,88,301,393,118,370,353,
    357,301,137,9,140,366,1,369,
    281,305,201,363,368,257,9,174,36,
    162,70,178,366,132,186,350,279,
    184,215,189,187,295,324,366,401,
    378,378,268,369,103,369,125,119,
    305,26,26,302,195,388,29,388,29,329,
    414,24,260,135,360,130,160,371,263,119,89,13,249,
    378,375,282,248,89,7,94,181,88,158,
    268,360,47,329,236,95,331,312,396,169,169,
    144,414,207,129,196,344,344,339,393,34,133,258,133,
    372,410,344,334,45,169,170,201,184,
    211,253,65,345,209,11,389,305,401,305,401,
    0,401,305,276,5,257,251,79,79,61,331,211,
    174,267,111,133,142,13,359,37,49,
    294,281,380,269,260,376,349,97,349,
    30,294,363,86,349,409,174,357,
    23,55,276,84,298,16,340,394,66,79,392,196,
    38,370,106,96,104,75,370,174,79,340,258,305,
    98,331,138,203,343,167,142,196,
    367,193,204,313,398,350,377,343,117,88,10,10,
    138,73,99,368,352,41,47,241,
    305,77,126,356,229,40,401,40,314,7,
    314,133,348,378,139,105,133,197,249,
    249,202,344,3,87,19,89,370,94,214,
    350,125,66,364,74,356,396,368,180,
    283,80,199,393,113,398,258,50,410,379,
    126,302,305,384,66,79,174,186,135,
    37,314,261,90,391,354,352,396,158,
    254,313,322,350,398,108,354,134,134,
    143,105,9,94,209,370,355,369,369,303,226,
    13,340,79,370,79,174,181,19,222,38,166,119,
    243,7,86,399,58,49,367,372,94,6,20,
    210,97,258,272,280,197,298,314,
    394,351,375,301,111,143,340,416,389,214,
    228,374,66,38,398,70,389,412,30,389,59,106,
    21,257,142,4,94,135,5,94,246,20,17,79,416,349,152,
    213,401,13,17,182,31,296,313,401,
    10,246,303,88,108,30,389,368,37,142,303,
    398,182,199,174,279,389,296,303,27,384,16,
    391,398,184,73,186,302,174,378,168,
    374,305,363,308,258,263,36,258,350,174,
    370,353,396,174,291,107,330,149,11,19,
    276,354,137,6,276,19,409,40,370,355,
    49,360,401,135,415,414,84,84,375,376,
    85,104,151,170,391,179,371,367,
    119,125,102,398,370,303,100,381,187,94,
    138,122,117,111,325,110,345,162,281,
    2,2,144,370,329,408,158,398,266,
    334,289,289,128,138,143,142,362,79,
    46,376,210,380,382,0,372,138,182,391,
    364,274,216,396,60,258,262,126,111,130,
    136,405,363,320,294,36,396,10,333,372,
    107,140,20,11,86,94,283,334,145,
    350,169,182,90,258,346,146,97,
    316,371,177,267,333,375,199,8,184,
    248,405,66,361,144,355,336,30,384,4,375,106,
    13,66,116,409,398,276,84,10,103,
    135,128,343,235,134,331,133,354,
    190,161,298,290,36,89,314,201,349,
    174,313,258,74,264,372,320,143,329,
    355,164,408,305,32,366,86,345,140,
    174,18,19,351,92,40,325,61,398,167,176,
    135,408,181,174,258,19,188,50,314,
    260,203,258,258,23,112,131,32,62,394,
    88,242,10,246,8,8,256,10,131,410,37,384,
    133,174,179,248,376,376,106,146,73,
    324,98,344,66,58,329,270,86,37,391,
    258,377,366,376,269,144,370,292,276,301,
    46,173,181,258,408,94,316,152,167,411,315,
    411,393,408,109,89,89,36,301,265,222,
    343,113,186,118,138,263,370,40,400,69,143,
    143,37,302,412,54,177,261,156,398,406,46,364,
    143,10,137,408,409,18,248,71,
    41,33,200,126,135,111,350,75,
    260,61,111,68,189,398,269,144,269,207,
    94,103,248,296,370,340,292,79,44,
    369,94,345,130,345,76,134,410,135,116,
    370,396,301,350,364,366,44,135,47,
    376,119,389,30,348,250,13,368,113,159,
    360,285,367,166,366,11,130,163,
    138,163,313,90,355,340,133,398,135,
    211,197,41,158,125,125,176,173,333,
    214,376,375,199,314,50,361,377,361,367,372,
    396,250,70,369,133,138,369,41,311,78,
    376,51,411,345,199,66,303,133,138,148,138,
    267,372,280,129,185,171,378,269,353,
    253,303,96,323,168,346,379,376,36,
    190,143,301,41,13,249,355,134,370,
    391,223,396,94,20,235,202,169,279,106,135,139,
    143,321,368,396,8,294,355,377,409,
    207,316,134,368,138,129,97,88,
    389,260,193,319,377,355,279,303,
    398,54,346,333,182,279,324,270,
    390,313,302,321,152,350,106,258,
    99,99,319,243,325,100,47,68,
    231,133,312,105,46,261,30,260,176,352,
    127,199,360,98,99,408,340,262,
    367,68,396,134,135,149,414,61,184,18,19,
    401,289,350,70,133,111,176,130,279,374,260,
    113,96,96,337,308,269,126,258,291,292,54,398,
    248,375,34,125,136,125,128,111,226,
    370,99,150,111,111,26,195,343,139,
    66,405,171,378,368,170,169,36,50,410,174,38,359,
    265,297,86,334,109,385,316,172,
    186,177,202,185,34,137,316,152,41,
    324,16,186,142,178,394,389,305,392,
    195,209,211,225,211,367,331,248,401,294,355,
    350,269,122,135,50,133,366,261,
    362,372,84,363,398,262,414,50,
    257,305,126,111,396,415,378,296,
    350,47,68,85,88,97,209,349,262,
    274,216,180,182,262,352,284,86,391,135,
    340,178,379,311,338,78,37,324,37,201,
    143,37,316,267,145,145,323,68,130,133,
    235,353,340,228,283,401,334,45,388,29,
    89,88,266,272,366,122,260,106,182,170,
    99,41,350,302,409,382,133,74,
    140,186,352,52,41,377,321,305,262,136,
    323,180,230,371,350,130,296,414,
    361,37,97,143,414,370,264,257,
    366,172,90,130,60,133,318,20,
    254,20,37,41,402,111,130,133,138,134,
    265,390,303,336,135,261,61,370,16,314,
    299,180,174,30,201,371,40,325,61,322,
    203,258,337,18,19,132,133,260,176,222,202,
    228,370,99,135,149,371,231,282,281,265,366,
    258,202,393,111,47,133,163,255,
    65,41,100,203,375,398,129,113,106,260,
    172,172,286,186,174,53,190,41,205,199,
    130,240,190,398,99,75,377,174,378,
    88,408,401,315,176,136,41,265,401,
    186,366,174,401,36,145,133,80,316,
    127,111,226,376,184,166,167,262,352,111,
    143,354,181,372,135,371,375,372,
    353,230,20,32,391,168,143,309,299,
    345,414,50,269,268,25,142,376,189,
    174,53,187,60,268,366,168,168,
    401,172,174,5,215,376,181,109,
    260,49,128,356,376,376,370,260,352,240,
    360,34,41,268,350,258,148,1,370,371,145,350,148,
    360,119,376,160,169,161,312,315,350,
    81,1,258,258,360,42,38,43,264,161,149,260,
    161,149,152,44,294,107,366,371,356,355,
    376,260,354,369,100,349,323,139,271,
    240,125,329,128,360,247,350,354,
    360,350,299,296,176,116,41,370,80,376,
    48,128,398,397,49,21,349,258,
    21,21,342,143,260,398,38,291,38,
    291,294,399,318,318,174,385,376,174,
    111,58,80,313,135,394,209,346,209,
    368,209,52,367,330,302,58,297,
    360,363,305,24,140,251,259,267,
    316,265,140,379,176,370,93,20,398,303,369,366,
    24,131,199,59,133,70,396,379,346,
    40,18,329,139,297,371,38,142,
    163,130,53,370,59,75,136,176,18,
    75,135,135,305,240,76,401,366,371,
    265,155,302,370,294,152,262,152,262,354,368,363,68,
    130,130,106,262,133,370,240,130,
    76,370,354,349,109,343,211,199,199,
    1,138,75,58,376,13,13,13,249,
    249,13,32,197,118,23,13,179,
    134,391,287,211,340,363,84,279,
    352,143,211,118,267,74,235,294,323,
    246,143,79,54,13,288,288,197,
    287,318,305,305,340,119,135,321,288,
    190,211,197,334,279,33,257,186,
    391,290,391,201,186,268,70,303,398,
    66,206,145,201,196,258,252,213,258,
    61,352,44,89,367,278,218,218,94,
    301,73,265,258,371,350,115,367,
    2,364,152,265,364,73,59,190,
    265,367,379,379,310,310,397,37,19,
    374,60,310,171,222,339,86,111,142,
    333,398,267,18,241,385,203,53,130,
    69,116,30,408,270,44,343,86,322,57,
    350,340,196,267,258,296,253,116,
    260,349,349,363,313,281,104,136,
    38,349,336,142,324,297,398,133,398,260,
    202,106,376,344,140,140,283,146,123,
    322,322,133,14,14,97,116,28,346,200,399,
    87,244,358,145,125,129,229,375,258,89,360,
    360,264,370,348,379,377,117,366,
    301,36,36,59,375,78,125,132,258,
    211,236,229,199,209,321,57,203,202,349,39,123,244,
    13,294,398,246,243,406,409,380,105,
    242,199,209,385,90,240,174,188,25,
    90,345,125,209,199,305,143,380,
    340,79,340,340,79,119,174,202,174,370,38,85,
    88,375,330,398,393,106,391,366,
    313,162,141,143,355,370,267,370,70,134,
    399,269,20,255,130,202,13,11,385,41,299,
    171,375,4,106,123,97,85,197,313,
    125,248,254,49,86,398,316,228,37,
    181,245,20,258,313,222,143,378,286,
    401,302,172,361,145,362,94,243,206,
    322,367,133,374,109,12,362,184,309,
    186,59,189,255,355,255,385,303,140,371,
    243,138,369,130,130,381,37,371,
    345,124,135,367,179,313,133,84,
    357,94,286,350,291,258,409,398,371,349,350,352,
    151,401,136,189,189,2,366,80,73,370,
    313,172,370,202,269,139,255,345,
    354,355,123,360,316,312,162,325,263,138,
    143,84,400,281,254,363,358,398,
    110,128,207,132,341,259,242,349,
    268,182,370,134,140,260,135,136,137,
    396,303,408,27,85,160,130,133,182,
    32,131,125,360,230,363,139,179,267,
    345,394,146,363,116,8,196,408,375,66,
    350,20,74,128,123,370,257,37,372,
    168,118,169,116,174,103,94,349,
    174,47,90,123,370,376,334,170,115,
    139,134,39,141,199,318,218,37,246,
    352,301,336,164,254,226,116,140,
    354,299,223,336,374,39,354,352,333,
    80,316,339,379,144,28,329,174,310,
    313,172,310,325,75,170,167,176,
    345,348,113,379,128,66,122,284,135,
    392,291,94,109,357,304,325,308,364,
    48,392,369,156,215,343,348,377,116,340,73,
    119,348,143,299,177,131,321,408,
    68,263,133,62,144,409,350,354,258,
    106,113,109,366,180,324,33,400,248,118,
    33,305,258,87,398,186,216,48,408,143,
    325,50,172,394,254,248,88,314,330,
    249,246,59,376,360,222,376,186,97,202,
    140,37,181,188,371,54,268,127,376,
    223,301,301,16,125,47,407,125,377,167,131,
    265,366,260,330,204,398,371,20,
    11,377,346,206,285,275,189,88,265,377,152,
    133,299,377,291,186,409,75,74,370,
    135,203,302,242,350,376,377,301,301,
    280,128,401,135,233,376,267,349,333,
    268,143,75,90,389,20,348,348,113,329,66,
    345,346,281,355,27,345,119,98,136,
    366,123,361,202,152,197,372,366,
    375,123,265,204,302,199,381,131,213,
    111,38,80,242,199,176,258,258,
    199,330,51,345,24,337,203,130,206,360,
    255,360,133,247,135,135,125,90,
    353,370,371,391,303,138,396,129,
    323,376,13,206,131,303,336,302,374,
    143,73,337,233,267,137,267,137,267,366,371,324,297,
    184,132,377,214,7,243,375,269,405,131,
    177,32,352,47,226,409,343,303,
    195,372,166,163,130,90,123,135,360,185,
    345,96,20,372,255,139,366,111,324,
    377,320,377,176,223,352,368,201,407,37,
    152,322,57,321,341,182,105,290,207,
    389,303,370,188,193,257,345,174,
    381,349,350,346,261,385,303,316,
    1,396,264,315,379,359,371,279,131,
    316,320,222,225,321,303,281,1,243,
    41,360,41,244,347,25,205,100,68,
    118,362,129,258,350,355,409,66,398,357,372,94,
    138,126,100,409,325,328,318,13,
    137,130,112,371,385,118,184,398,366,
    299,195,372,47,190,168,187,368,
    18,323,376,359,125,13,16,398,
    136,157,301,297,66,202,4,186,
    125,360,125,375,32,86,374,112,195,
    265,376,251,133,364,34,258,350,
    133,186,185,184,139,113,50,314,185,
    398,96,261,174,366,26,137,50,
    47,337,404,240,328,369,350,202,324,
    209,297,116,176,168,341,38,97,
    90,248,361,370,195,409,196,150,189,321,
    11,248,305,392,392,39,405,360,128,132,165,
    135,366,309,178,182,54,329,367,136,
    50,410,372,123,359,305,109,372,354,
    50,410,164,360,176,398,345,249,252,376,
    137,262,255,60,353,130,138,349,241,133,
    243,345,316,260,260,350,364,186,350,
    363,78,129,110,206,284,316,170,178,396,
    50,410,370,398,343,323,296,323,34,363,
    163,130,369,298,336,389,401,288,286,120,13,
    296,32,32,305,334,257,180,345,
    291,291,37,141,37,65,126,137,170,
    35,97,55,140,313,305,401,248,116,
    379,182,123,104,94,118,119,352,135,
    296,350,4,376,186,168,228,376,180,
    203,302,384,60,128,385,303,355,376,174,
    303,362,181,343,195,409,374,160,130,24,
    176,68,369,4,128,396,32,195,
    97,59,323,370,318,249,143,321,264,
    133,408,176,230,113,348,139,89,247,291,
    133,294,318,130,132,41,321,314,71,333,291,
    401,167,18,176,202,222,303,305,202,
    228,372,372,201,139,258,13,249,133,133,
    118,281,54,414,348,325,371,371,77,
    49,132,125,265,168,146,363,1,148,152,257,408,393,
    345,18,106,260,372,18,165,88,
    25,198,135,345,77,189,255,382,53,190,174,
    375,367,186,313,398,372,75,74,344,
    130,355,243,301,16,32,205,209,182,
    135,257,9,291,37,65,106,18,132,352,
    186,264,116,372,279,174,140,354,
    372,318,345,77,355,127,342,362,401,184,309,
    167,77,86,125,167,305,176,372,
    202,133,176,135,415,372,89,180,370,
    135,378,32,58,273,215,135,168,86,
    309,377,408,137,385,90,299,172,168,50,
    268,374,260,85,109,145,366,118,
    372,286,382,53,187,366,174,202,296,
    323,60,324,137,32,372,118,5,401,
    168,168,215,343,187,363,269,352,366,
    97,366,376,132,132,16,205,110,65,
    130,354,354,130,123,181,384,404,
    142,389,376,355,38,408,381,381,24,
    367,258,399,89,11,229,141,111,346,257,
    370,186,46,249,148,243,366,366,
    244,90,211,34,178,270,150,78,110,
    356,398,110,110,345,261,14,57,
    351,397,401,152,393,94,5,355,
    355,181,408,41,361,143,323,9,245,141,
    245,93,322,322,19,367,334,260,296,401,
    389,68,345,303,176,38,129,400,
    125,312,168,333,137,368,360,122,269,
    179,128,367,354,359,352,371,349,
    400,368,303,345,334,73,205,381,148,
    123,170,189,351,401,361,397,255,366,
    130,110,35,130,151,143,86,298,
    369,130,0,324,139,275,179,350,
    94,245,141,355,35,257,333,408,333,343,
    115,248,169,366,360,90,38,279,
    125,350,305,119,363,131,158,144,369,354,
    350,366,116,405,268,146,66,355,35,133,258,
    349,366,190,116,366,128,200,143,
    61,10,89,180,164,131,339,350,
    54,349,123,34,143,94,348,1,137,410,50,
    90,254,266,285,350,370,266,356,408,34,
    366,366,370,145,376,98,272,249,
    358,372,98,302,33,298,358,372,223,103,
    268,36,119,163,399,76,351,130,379,363,
    90,176,361,357,129,137,262,135,13,
    372,401,345,337,260,296,350,370,233,233,
    32,366,141,141,376,199,294,345,
    369,389,139,266,280,199,128,360,393,
    345,86,267,318,367,179,401,138,
    384,107,9,125,379,214,303,132,
    14,105,338,324,34,296,82,379,20,
    129,355,350,349,350,379,119,119,354,
    350,379,358,358,296,266,368,363,
    202,176,372,349,279,104,366,261,
    182,350,13,16,50,410,186,4,135,305,
    370,185,248,90,318,54,370,328,145,410,
    376,125,370,398,4,345,182,116,272,
    240,275,141,195,164,297,53,388,
    135,350,350,350,370,354,38,129,
    32,59,369,323,272,366,363,262,146,
    65,78,301,137,262,145,408,89,313,178,376,
    180,334,73,298,89,86,366,363,168,
    199,324,370,141,200,401,137,372,
    376,370,362,168,322,179,384,24,318,
    350,270,410,176,130,401,355,181,
    345,370,355,393,130,57,230,168,
    360,352,119,363,139,40,61,368,
    119,168,16,279,372,174,179,209,9,285,
    190,166,179,4,363,162,129,312,178,174,
    186,145,178,366,363,350,355,184,369,
    24,273,378,168,50,145,39,109,
    268,35,202,324,168,401,168,181,
    53,376,393,402,393,402,241,397,245,37,36,
    377,1,345,116,145,145,94,369,
    5,70,369,368,412,309,84,243,
    45,152,383,70,261,374,261,250,
    7,243,34,135,242,75,45,376,
    389,14,203,70,8,20,45,375,375,374,
    75,364,37,229,229,253,142,171,210,211,
    321,211,170,276,196,87,197,211,
    98,349,366,100,267,10,313,135,106,
    375,40,152,302,211,66,260,269,269,
    409,326,350,196,155,260,349,106,
    350,174,174,256,133,98,398,326,11,
    269,47,75,143,134,135,260,90,250,
    152,143,151,41,350,10,189,138,
    193,288,345,197,174,78,334,262,136,
    350,174,75,179,242,251,9,350,
    40,345,163,40,269,269,269,5,86,
    267,133,31,408,9,116,2,100,405,110,
    193,375,150,98,246,88,124,364,371,128,86,
    405,379,162,229,66,162,399,211,
    10,249,143,370,269,38,302,357,245,351,340,370,
    125,181,88,249,222,368,375,105,
    362,143,59,20,158,352,228,128,
    121,137,119,209,393,138,363,296,321,303,
    279,304,334,73,170,75,351,303,160,
    397,376,319,376,13,196,74,350,303,144,
    174,351,371,317,169,10,398,366,
    294,174,116,352,140,242,88,354,
    6,249,258,222,16,371,167,179,135,366,
    261,164,366,113,410,202,33,370,
    398,397,364,345,201,23,52,299,179,
    68,189,125,410,125,345,90,348,
    377,357,401,197,345,44,352,337,
    364,134,263,216,355,119,100,115,134,124,14,250,375,375,
    199,30,368,319,20,255,207,126,377,
    315,193,128,58,376,303,118,261,
    370,396,25,118,99,195,140,136,209,
    392,32,4,4,118,320,89,145,
    13,13,129,257,180,360,334,368,354,
    178,312,350,354,304,78,137,100,179,355,144,
    75,130,160,352,355,321,352,363,
    228,14,250,132,231,281,201,179,216,375,110,
    304,186,321,352,202,273,128,216,375,
    189,352,258,145,361,204,409,307,190,
    186,376,316,344,267,95,71,171,
    5,133,123,66,44,97,142,376,
    258,376,33,367,193,123,349,94,206,346,
    138,364,18,89,14,8,378,145,200,379,
    145,343,135,260,199,59,253,345,128,
    352,261,25,181,58,370,2,97,254,68,
    94,361,352,350,20,49,105,134,298,
    255,49,152,272,302,301,370,322,412,143,
    134,206,296,182,13,396,396,145,
    85,184,139,137,135,174,110,352,
    400,104,366,359,367,360,189,316,
    401,264,371,154,363,9,84,353,368,
    351,122,111,39,360,7,246,170,
    60,372,131,130,346,80,37,66,329,349,
    349,37,146,199,10,333,352,41,
    116,361,266,366,267,361,169,174,359,
    94,182,364,350,181,174,139,176,
    320,320,90,343,68,253,19,391,54,291,
    206,376,143,36,167,206,302,345,376,
    330,305,408,416,12,246,37,125,258,80,
    164,33,258,12,343,186,50,109,
    366,69,10,180,264,249,241,270,
    408,264,85,139,266,75,138,131,130,
    376,197,199,47,361,329,357,58,
    280,206,135,345,282,128,355,138,44,
    135,404,33,367,176,269,351,76,377,
    369,216,125,372,376,129,283,291,
    182,303,279,320,368,346,349,396,
    139,372,193,325,182,324,174,169,
    111,330,396,261,25,56,145,393,368,1,
    18,250,336,305,33,164,404,50,139,370,
    54,50,258,174,140,384,320,267,361,
    4,176,200,392,371,369,372,398,
    186,349,65,359,388,363,268,60,
    180,178,266,145,316,129,111,257,140,
    86,139,182,133,130,140,1,13,
    24,268,384,60,137,109,323,130,160,
    128,291,318,330,41,376,139,186,94,
    18,250,305,346,414,168,350,133,409,361,
    282,348,96,172,75,174,398,280,
    174,382,266,329,111,318,166,184,
    186,174,382,168,372,202,350,353,266,345,
    109,61,382,128,369,364,107,20,70,20,245,
    125,398,125,251,7,273,174,341,0,
    353,123,260,341,7,247,87,59,347,240,
    0,0,341,125,181,370,254,49,
    0,144,33,38,0,60,341,21,
    406,254,14,400,396,0,49,372,
    258,352,185,66,240,201,404,12,
    180,388,349,249,59,347,372,366,
    97,58,301,330,330,116,33,302,
    265,301,32,32,283,302,316,301,
    374,307,186,94,374,12,12,228,
    330,375,134,301,375,389,68,94,214,
    68,330,302,254,333,71,126,333,71,396,381,409,201,
    13,13,258,211,363,182,33,211,379,
    86,94,103,330,138,138,269,345,
    94,13,330,211,0,243,136,341,57,94,
    214,182,11,396,360,41,211,211,27,133,
    381,409,96,13,57,398,40,303,191,258,191,
    86,243,370,86,243,126,299,376,299,211,146,
    370,182,299,70,40,126,60,406,
    133,343,136,37,33,337,172,133,
    30,182,70,337,180,136,136,261,40,
    249,70,70,249,364,305,138,263,59,305,305,
    398,66,370,222,217,213,71,13,138,178,
    98,100,370,142,400,351,296,360,237,368,
    174,167,367,36,375,5,138,145,362,258,
    364,351,54,13,370,174,410,45,90,
    401,245,249,97,152,49,362,398,
    59,396,85,14,398,328,143,133,88,
    105,296,68,134,361,389,19,226,397,
    374,140,269,328,39,334,370,138,345,375,376,
    130,323,303,367,398,398,121,364,199,
    74,140,354,334,336,196,249,354,
    317,257,174,398,56,79,349,294,
    170,304,128,352,370,12,248,392,109,
    323,88,193,193,180,38,133,330,68,2,369,80,
    38,13,13,206,106,77,152,80,345,
    376,54,364,401,52,59,301,399,
    398,38,376,124,90,166,367,36,336,
    376,113,346,128,158,134,351,371,370,
    185,290,145,38,350,109,370,346,
    133,45,7,130,172,182,31,56,304,237,368,
    68,36,57,17,323,392,16,301,52,
    189,370,410,40,392,390,315,291,
    270,69,185,185,209,264,371,372,
    129,94,178,184,262,137,182,170,352,
    88,59,59,371,119,1,7,352,109,
    111,345,230,376,345,370,374,249,172,
    174,167,305,59,180,68,180,167,17,
    133,38,367,361,138,397,201,174,
    132,167,133,68,361,372,371,268,
    374,323,68,189,187,187,20,20,
    111,5,85,65,85,6,6,263,
    133,13,384,384,197,62,66,241,5,138,129,
    111,49,181,99,118,209,133,137,248,
    99,368,1,80,118,116,13,343,40,
    260,350,1,354,118,129,118,385,
    54,118,354,369,255,118,137,1,
    357,129,174,189,20,251,119,137,249,97,
    245,400,146,267,55,270,389,106,
    146,146,400,389,52,106,393,391,59,75,206,
    258,372,376,10,69,399,247,119,
    372,119,370,20,343,119,3,391,
    366,135,119,376,163,86,96,100,119,61,
    243,94,267,302,37,61,186,391,201,
    174,139,360,135,243,109,2,186,
    360,400,40,60,2,106,174,211,71,
    97,360,196,196,344,398,258,377,352,330,
    353,78,356,350,243,243,90,78,206,
    207,302,357,303,379,131,203,243,87,204,
    59,199,197,149,352,157,303,367,372,
    397,368,4,301,132,57,396,162,143,360,157,
    301,370,38,302,199,209,205,401,396,396,
    203,303,377,70,329,222,409,409,34,
    389,361,19,87,244,243,184,111,318,334,202,
    70,398,66,217,207,361,311,363,38,162,144,
    210,396,331,367,366,209,399,209,
    408,393,394,397,199,320,262,298,354,116,128,66,
    37,56,403,144,80,203,352,350,164,
    167,135,296,330,112,343,173,303,
    266,174,364,140,397,174,167,318,414,
    144,310,130,318,75,13,13,249,211,131,
    222,186,370,385,99,138,23,400,376,131,
    193,351,357,130,131,381,47,135,
    199,75,124,361,329,163,99,283,
    197,360,85,348,204,40,111,345,163,202,
    347,157,133,60,36,152,315,351,266,128,
    209,207,195,200,89,385,392,370,69,74,
    157,209,311,50,185,190,38,195,200,251,
    37,111,201,343,284,311,252,350,262,
    257,401,65,301,311,178,35,352,135,
    149,369,362,334,349,202,180,111,163,135,
    369,1,130,391,135,106,393,268,143,
    345,40,290,228,37,363,368,132,378,201,
    203,253,203,172,162,113,145,361,203,
    132,186,201,184,109,269,195,350,41,
    324,149,401,197,139,264,109,139,264,109,376,360,145,312,
    385,145,303,370,301,398,124,301,
    372,143,400,137,56,76,1,137,
    388,378,5,303,59,71,258,133,409,
    97,349,394,158,98,261,156,350,86,162,
    60,193,294,59,145,174,94,206,
    80,360,126,150,398,258,263,149,138,253,18,89,
    80,364,249,394,366,318,404,35,
    78,341,366,139,90,85,209,389,
    143,376,152,189,340,340,66,390,396,
    80,94,88,211,401,174,166,14,232,254,
    248,181,245,171,255,20,255,301,
    380,1,174,184,334,374,174,162,
    41,155,269,401,162,110,111,80,216,
    259,186,345,111,1,189,100,154,352,112,371,357,366,
    73,248,254,350,170,123,312,303,351,262,
    265,345,1,262,370,155,354,270,152,158,32,
    169,123,376,354,351,196,8,189,184,374,334,
    35,35,348,182,372,196,270,366,
    294,164,376,38,126,186,36,135,
    237,314,408,155,248,366,406,46,79,156,
    37,258,410,50,265,180,146,20,71,
    206,69,135,391,119,186,182,1,318,270,350,
    173,10,371,77,349,258,188,343,
    68,216,98,10,258,36,282,366,70,
    71,400,340,138,364,372,14,152,13,
    345,312,303,396,76,351,60,329,66,216,
    248,135,66,323,30,330,258,78,
    90,361,270,270,262,193,104,223,316,355,
    80,49,182,313,329,324,8,244,126,152,106,249,
    163,345,289,172,56,330,351,259,364,350,176,260,
    243,345,1,96,379,77,394,152,166,404,
    368,112,404,32,258,4,262,248,182,
    186,149,45,36,371,172,16,258,
    209,258,401,54,410,265,48,188,133,
    296,170,189,268,388,65,135,350,180,
    71,68,129,243,20,133,299,262,66,174,
    135,137,350,392,262,78,135,376,
    406,119,262,152,132,385,172,138,41,369,
    270,125,60,370,136,249,249,376,253,
    80,258,1,152,135,376,282,201,245,
    49,20,367,205,22,352,356,162,172,
    172,398,174,174,86,270,245,372,
    174,184,184,209,20,309,109,135,
    22,366,303,303,174,277,299,378,
    313,258,321,193,355,368,352,398,258,
    258,398,12,87,77,399,276,370,303,
    375,398,331,94,94,202,13,412,398,
    317,199,416,268,125,401,301,318,
    49,31,202,190,376,353,349,331,
    251,401,111,351,398,133,390,99,396,
    99,310,172,139,301,96,164,66,61,
    132,325,258,106,109,414,181,186,
    19,139,61,398,186,32,296,13,249,41,
    130,375,350,371,409,132,396,94,
    377,360,352,297,367,329,398,370,199,313,
    66,10,408,396,372,133,99,324,
    313,193,321,94,361,258,376,350,
    133,313,296,32,59,111,318,174,230,
    202,61,174,273,378,329,382,172,
    280,376,376,240,174,355,264,119,336,
    359,313,276,336,409,30,212,97,370,398,
    352,19,223,267,267,399,39,89,118,197,
    379,152,204,398,140,13,398,376,
    202,13,158,7,249,222,174,375,412,
    249,20,181,209,37,223,264,367,
    416,398,66,305,143,409,132,165,133,
    37,36,334,303,398,132,165,132,371,409,398,
    138,276,75,370,401,130,230,94,257,
    350,99,169,94,363,385,310,190,164,
    97,140,329,37,336,305,298,310,364,
    188,186,106,416,276,407,8,6,
    133,258,398,398,164,173,181,248,152,19,
    40,414,412,316,376,316,191,353,370,350,263,
    14,133,94,249,13,235,138,399,410,
    360,37,36,61,346,352,135,176,409,142,376,133,
    360,396,398,61,134,133,258,99,99,
    106,279,318,279,133,150,211,24,296,28,
    200,199,398,133,186,316,133,372,346,
    267,291,119,370,129,263,133,318,
    354,274,257,137,408,20,334,399,416,186,318,
    230,291,130,273,235,376,253,133,
    338,346,37,36,132,162,190,16,245,291,
    273,408,145,174,53,382,362,341,142,266,
    350,266,156,376,301,140,368,44,
    407,336,170,263,390,368,14,9,
    368,19,341,401,158,137,178,182,262,69,349,
    341,111,368,398,45,368,331,368,137,45,
    141,354,37,157,53,348,59,158,
    152,408,132,316,109,163,74,408,
    371,363,348,341,364,369,376,143,266,368,
    368,331,34,376,330,68,366,69,143,178,
    350,349,163,45,34,152,161,53,161,53,184,
    37,54,178,384,53,262,266,74,
    384,184,263,174,41,303,94,260,
    41,258,123,258,118,302,89,305,204,
    268,157,391,401,181,184,19,140,140,
    392,6,313,146,123,334,314,140,396,
    69,370,305,140,268,138,254,76,
    174,404,29,388,65,55,342,140,149,
    140,401,401,75,171,139,248,376,38,97,
    196,401,343,75,133,137,5,317,
    133,264,393,319,364,406,283,377,125,
    117,354,28,139,116,249,13,13,135,370,73,
    296,302,57,351,212,66,401,212,38,106,
    174,263,206,9,331,313,94,27,
    11,85,57,409,66,181,416,385,232,
    94,88,105,86,134,100,86,303,197,
    255,329,135,266,184,206,14,189,
    111,268,38,371,368,352,13,266,
    165,65,137,139,269,319,281,85,
    162,401,334,133,57,117,27,399,
    157,167,13,295,60,397,27,94,
    379,146,336,241,174,169,143,109,135,
    116,334,351,398,37,317,303,401,
    416,354,298,333,27,366,99,160,
    97,40,162,98,379,238,260,354,
    135,256,21,94,257,167,411,242,10,13,13,100,
    322,38,108,58,376,135,393,61,106,38,397,
    265,140,294,400,186,20,133,180,317,
    146,271,94,389,106,156,260,269,146,
    46,109,343,377,27,412,255,385,263,
    340,189,59,354,285,135,361,14,
    319,353,352,254,396,357,125,303,370,
    401,378,368,48,47,190,349,73,312,354,262,133,
    138,129,357,199,86,44,404,250,
    90,401,123,263,124,267,204,260,
    106,163,370,185,379,119,324,378,
    40,99,88,285,397,105,226,260,
    354,53,104,97,184,248,244,75,174,13,408,132,
    41,295,38,401,261,25,184,168,135,
    21,174,130,13,401,66,50,366,248,
    28,382,24,404,413,317,249,251,16,74,376,205,337,404,
    385,295,113,111,370,125,32,157,52,
    254,384,133,111,316,185,27,133,186,
    223,320,53,69,320,171,76,401,
    354,20,202,205,295,313,60,178,59,68,
    94,135,206,163,58,137,65,129,
    319,404,170,382,354,186,303,382,258,
    242,258,242,97,143,186,186,366,
    20,60,287,402,105,260,176,21,20,
    400,167,303,168,163,376,378,118,
    396,135,322,329,226,40,133,370,258,
    328,404,400,86,243,14,315,305,400,260,408,
    328,186,186,135,340,372,376,167,
    184,263,176,168,260,378,399,268,
    176,14,76,413,174,295,189,372,
    378,408,376,202,66,86,301,394,
    301,236,119,172,352,409,222,55,
    392,260,390,13,7,349,294,34,150,140,
    280,89,13,54,371,394,202,321,
    125,5,174,97,143,255,376,52,
    391,400,38,316,331,174,350,316,
    123,334,409,49,27,291,378,400,376,180,405,
    6,170,89,84,268,119,177,352,
    94,177,24,140,174,378,186,143,
    258,54,6,392,180,410,140,113,
    126,288,301,301,324,14,280,203,124,
    360,410,125,135,382,49,174,355,
    94,235,10,106,105,359,99,324,267,
    134,26,405,324,202,199,288,301,89,384,
    150,136,209,288,301,288,235,350,177,
    136,160,20,128,305,410,352,235,
    337,226,174,416,66,226,331,168,
    202,313,313,142,350,133,104,397,142,104,
    133,30,400,363,378,368,123,104,376,119,100,
    343,276,346,346,267,212,409,335,
    229,93,133,138,305,47,249,13,396,294,
    123,398,133,89,379,276,59,139,
    316,87,320,54,142,389,380,114,139,
    94,398,258,409,40,123,389,380,172,
    350,94,355,301,20,13,401,268,181,
    401,298,97,367,94,340,396,330,58,
    41,303,399,352,412,141,7,268,
    209,305,414,162,140,276,117,355,
    138,401,40,107,159,6,209,145,162,125,
    49,128,103,103,325,355,138,158,137,269,308,
    96,1,189,170,361,12,19,248,352,94,101,133,334,73,
    279,331,69,61,371,172,355,144,360,96,115,
    70,334,313,136,353,130,145,398,
    135,144,38,398,203,346,195,343,396,190,37,267,
    305,8,334,354,128,343,264,352,103,360,
    329,359,355,123,350,94,333,318,
    77,164,94,140,125,398,366,352,141,
    90,133,360,276,410,388,180,36,79,174,179,
    190,140,40,269,298,258,258,407,
    133,258,343,260,265,397,352,304,345,265,258,325,
    343,98,344,12,406,23,113,54,
    188,109,182,258,391,13,48,34,181,203,
    258,133,330,323,32,410,112,411,350,409,
    357,177,139,88,283,206,376,410,
    86,190,186,360,372,297,409,360,353,
    135,152,352,282,203,133,258,76,39,399,
    66,206,204,197,377,355,369,9,313,267,
    14,128,103,410,203,345,94,345,
    335,360,305,105,204,355,176,410,14,250,112,379,
    371,329,107,348,398,379,346,37,32,58,
    355,377,410,360,302,345,103,361,
    372,139,370,406,222,8,106,125,243,
    400,135,49,56,320,269,309,379,346,351,54,318,307,
    350,279,325,94,379,396,99,281,
    125,381,388,328,352,361,316,396,410,325,
    129,23,13,90,52,174,320,316,366,371,
    350,410,172,404,144,260,195,398,190,
    211,209,251,176,202,361,410,133,296,
    318,86,255,190,12,370,290,210,208,204,211,178,368,375,400,
    261,302,352,133,410,359,272,361,
    318,262,388,416,398,296,288,180,
    143,145,86,178,48,415,135,274,32,
    283,359,130,126,413,350,261,346,
    57,302,130,350,133,291,135,136,128,
    262,290,50,355,137,408,13,59,323,32,370,230,
    318,370,294,281,133,18,260,168,
    257,94,363,413,409,248,368,174,209,172,
    355,413,162,375,360,172,352,32,
    137,186,32,372,23,353,273,352,414,
    413,189,174,350,174,286,61,168,172,176,313,
    142,376,123,104,400,352,260,119,100,378,368,133,
    343,162,133,276,345,379,123,47,
    249,13,294,98,212,276,410,188,109,89,
    398,346,346,87,401,396,229,305,352,
    97,355,94,176,412,301,350,398,
    399,400,7,94,41,298,370,140,
    58,8,279,138,158,274,70,117,
    130,101,133,361,136,189,170,145,137,334,
    103,354,144,359,350,318,325,133,
    329,133,360,181,372,360,258,88,
    48,34,297,112,302,345,203,304,12,
    40,325,182,269,410,388,391,343,190,186,
    406,409,152,353,135,203,168,329,
    204,133,258,379,346,130,313,79,76,14,250,
    352,105,406,128,66,190,14,206,
    377,139,94,281,396,90,54,318,307,99,
    32,174,370,135,18,251,195,172,
    372,320,316,210,208,204,211,178,290,355,178,296,388,
    136,260,262,290,128,137,408,413,93,355,
    98,93,270,93,270,20,254,353,
    393,98,372,372,265,351,109,415,
    323,37,258,347,372,172,323,186,
    109,344,344,344,344,116,344,189,
    94,301,85,106,401,143,197,106,
    206,98,5,249,107,329,144,94,301,
    366,393,414,108,107,408,376,398,2,
    85,168,305,313,249,193,182,5,249,
    85,174,34,345,13,133,388,39,
    182,133,144,202,393,189,249,133,
    133,187,367,353,202,261,57,199,367,353,
    181,375,89,5,99,367,106,261,
    383,199,99,181,370,350,401,66,359,261,
    370,352,279,271,271,261,128,320,
    352,370,375,261,156,260,352,366,376,103,138,
    324,377,350,86,296,89,296,176,
    172,103,231,261,32,376,123,104,370,
    39,347,89,123,38,38,54,94,
    351,11,370,166,370,249,13,255,181,182,
    398,268,376,350,355,353,350,350,152,
    262,262,130,130,354,294,123,136,66,390,
    54,88,61,400,294,38,401,135,361,
    38,250,410,343,130,124,119,119,
    116,4,251,370,176,124,268,4,180,
    247,262,4,86,370,130,361,61,
    368,170,170,151,197,394,258,303,105,
    105,105,70,70,84,306,282,235,84,213,
    213,76,404,172,333,409,103,34,118,
    379,5,241,249,313,38,313,268,41,134,143,
    132,41,170,188,133,138,324,240,185,
    231,136,244,389,385,185,133,170,132,
    375,209,127,84,370,71,369,57,
    314,264,379,372,38,59,59,123,103,
    398,243,226,59,396,35,181,397,
    375,341,338,408,178,184,398,228,331,84,236,
    364,332,394,113,360,176,118,302,179,
    253,140,143,13,66,398,113,346,360,
    254,50,71,222,333,143,50,163,
    176,163,50,176,347,163,176,176,
    50,4,302,314,333,163,226,398,
    59,228,263,222,135,333,333,184,376,
    376,393,313,316,370,316,313,393,
    393,280,370,172,171,133,267,154,26,
    100,20,66,128,129,38,276,354,281,
    400,377,75,98,279,36,97,31,348,
    33,106,398,264,116,94,88,7,246,
    243,135,87,407,47,375,212,3,154,
    272,104,376,346,368,258,249,13,13,260,
    350,350,88,154,140,322,301,399,
    392,355,301,345,400,70,59,88,13,
    5,20,268,330,10,10,107,322,409,88,
    88,158,398,222,254,248,409,94,400,243,396,
    352,416,246,134,302,398,9,211,
    268,125,259,38,371,360,367,184,
    73,147,186,140,232,366,244,159,
    370,110,115,100,73,38,137,358,
    358,84,2,122,250,220,213,409,111,163,
    397,331,398,54,199,355,54,355,
    194,194,209,133,355,227,160,286,383,
    258,216,202,230,187,343,20,346,
    343,359,137,140,280,122,56,179,
    296,333,199,47,301,134,0,144,
    52,359,356,340,245,37,218,94,
    74,340,225,216,249,106,189,174,
    176,392,54,138,177,310,249,16,
    188,250,113,144,46,59,330,218,
    140,213,166,350,369,0,366,276,301,406,
    94,94,143,88,261,343,73,249,
    113,410,71,348,199,282,404,38,
    51,189,240,66,2,357,216,305,
    308,214,379,399,280,80,287,336,
    368,135,345,137,376,134,76,13,
    33,94,352,222,203,341,328,338,
    8,260,190,341,304,324,316,406,
    100,370,20,178,133,249,355,99,
    190,18,240,33,186,16,113,244,43,
    16,136,94,324,209,350,44,47,404,190,
    137,372,190,398,362,55,180,334,
    248,222,43,178,54,163,354,328,
    86,243,398,137,296,125,349,54,284,353,
    318,89,372,296,59,402,59,160,230,
    339,176,13,10,374,145,41,370,144,
    166,100,176,290,339,106,258,54,18,
    363,216,348,378,383,352,16,357,161,
    166,366,186,132,380,189,268,383,
    187,222,187,380,36,260,352,348,110,141,383,25,
    180,110,141,409,137,226,40,359,133,99,
    40,203,14,226,398,398,100,135,70,398,
    398,139,359,322,396,142,352,376,30,
    368,376,39,350,350,142,376,376,
    357,143,142,356,299,299,299,142,
    303,323,305,303,330,323,257,257,
    109,126,330,44,311,351,349,400,
    61,44,296,370,86,241,322,86,
    7,44,117,87,7,13,186,399,
    135,25,181,401,385,79,20,352,
    100,44,351,186,266,244,350,159,
    94,384,90,174,298,376,169,333,
    376,345,20,201,223,143,129,304,
    152,14,211,70,61,8,30,370,
    315,25,26,185,58,362,368,39,
    65,60,261,186,370,133,135,132,
    201,258,186,186,32,309,102,177,
    135,135,291,366,94,254,366,366,
    26,4,370,171,71,137,267,1,213,
    331,267,138,248,343,370,30,203,
    202,97,260,376,376,298,358,75,
    125,351,258,196,409,130,125,318,398,353,
    13,249,94,339,47,345,349,398,258,296,
    346,260,276,94,157,138,96,186,360,401,
    133,264,258,377,366,89,5,283,356,
    133,126,188,126,87,349,125,145,105,398,
    379,264,4,41,197,364,88,94,277,
    117,50,32,371,375,14,370,263,345,
    174,249,80,352,33,25,401,316,
    66,329,377,272,181,322,331,298,66,204,
    265,174,133,374,152,119,211,10,9,105,
    206,370,370,143,268,252,285,275,158,401,228,
    222,241,20,19,296,359,368,352,11,
    123,372,416,389,73,143,30,70,226,97,
    125,254,248,199,94,302,283,106,13,345,
    94,408,197,86,263,197,197,5,
    409,209,409,398,38,133,140,184,
    50,225,377,362,372,266,100,207,
    174,279,371,102,260,31,36,376,
    118,409,179,349,133,111,49,135,
    49,124,110,196,30,137,137,94,
    376,401,409,136,130,371,30,85,
    279,281,39,196,334,399,260,401,
    363,128,94,269,96,57,140,357,
    44,26,140,84,2,262,38,276,
    135,370,329,129,254,174,139,170,305,
    405,57,134,274,13,27,262,130,
    133,60,409,279,131,357,372,189,372,
    260,363,139,319,371,194,123,400,368,
    75,345,174,74,94,276,371,119,
    13,21,379,66,336,318,318,37,
    36,349,17,350,103,174,257,401,
    209,174,405,416,340,267,320,294,320,
    36,248,90,143,199,201,357,140,35,
    301,356,146,366,333,375,56,109,109,343,116,
    375,56,134,344,316,375,229,298,354,352,
    169,177,94,251,80,209,211,346,343,203,138,214,211,
    149,167,176,303,348,336,352,176,132,
    375,372,372,104,47,196,196,49,
    343,379,140,66,268,73,135,411,41,106,
    166,186,143,345,146,226,276,164,119,
    257,409,381,99,113,94,188,33,40,
    314,46,391,200,23,5,174,336,
    20,116,9,264,144,350,264,66,
    138,294,257,60,139,262,393,322,398,38,103,126,
    106,181,88,264,139,2,344,12,400,
    366,412,135,180,193,323,305,330,61,
    125,258,119,54,325,47,13,33,
    128,88,167,258,201,254,345,59,
    294,128,366,370,331,258,343,27,
    213,396,340,142,332,189,13,370,
    243,20,245,71,372,372,372,354,
    286,267,152,353,343,376,376,94,
    176,361,361,214,27,348,47,298,
    376,14,197,2,80,166,189,170,372,165,
    165,136,203,416,416,412,9,280,
    350,369,2,268,135,94,190,140,
    247,90,123,123,124,357,336,401,408,394,
    409,353,301,100,259,265,202,129,
    301,257,96,73,400,260,345,20,
    345,241,133,125,383,134,76,368,
    146,50,269,345,396,163,333,131,
    350,303,258,168,410,368,377,199,
    379,305,66,404,109,272,362,32,
    148,163,160,126,136,185,345,242,374,
    315,371,303,47,303,379,396,169,
    281,212,201,174,270,317,377,128,174,143,
    350,8,41,360,305,336,182,132,68,
    260,412,143,255,56,377,41,376,160,
    243,257,257,212,312,350,89,379,
    397,135,133,285,25,82,202,118,
    319,396,207,315,360,182,350,106,
    169,279,347,96,100,119,56,303,324,189,
    281,320,361,10,368,408,111,13,410,
    112,416,331,27,246,168,59,133,
    174,301,169,376,181,372,209,69,331,66,
    331,197,334,401,248,2,176,50,
    350,254,267,360,94,139,47,138,345,338,
    26,376,370,409,143,178,186,13,186,360,
    21,392,172,261,195,366,181,133,
    16,112,116,66,316,186,299,297,
    66,205,131,195,343,20,66,56,394,
    301,361,345,125,4,202,185,52,
    399,23,255,136,202,50,225,130,
    144,371,135,223,305,371,113,36,
    125,294,157,260,193,383,385,261,
    74,176,180,157,1,13,174,345,
    133,260,363,302,86,201,240,32,68,
    363,137,283,283,172,376,262,401,
    126,135,194,379,9,375,268,186,
    274,130,80,329,88,145,414,85,
    281,89,163,311,283,364,360,94,
    145,60,349,73,313,354,350,286,
    379,298,258,135,379,319,181,376,
    351,347,133,123,313,230,172,361,
    379,376,350,354,118,9,20,118,1,345,
    130,130,133,49,409,353,343,187,205,370,
    173,136,24,301,261,291,176,152,377,
    57,329,324,362,13,391,319,352,176,
    86,71,355,106,355,305,135,118,151,
    123,286,356,363,368,6,315,305,
    363,77,253,377,345,228,40,400,194,348,281,
    251,322,133,258,384,36,396,84,222,
    372,99,50,354,118,258,85,135,360,376,
    163,138,133,14,69,408,202,168,139,25,383,
    204,266,258,352,178,240,352,316,
    190,370,360,355,174,370,166,172,
    137,66,398,10,328,368,209,128,
    16,245,86,315,323,338,266,262,345,
    182,130,240,99,379,9,174,305,
    401,41,1,180,384,361,264,167,132,
    340,349,283,283,258,122,186,316,
    338,196,379,253,254,376,363,133,141,
    361,209,267,316,141,248,226,20,
    273,370,352,376,143,176,176,371,
    261,372,184,335,126,378,181,268,
    368,86,202,168,111,168,133,60,
    195,172,172,130,90,398,345,163,
    391,127,174,133,202,172,127,189,
    133,163,186,135,286,328,172,269,
    354,370,187,200,17,125,125,186,
    237,190,313,354,260,41,125,360,
    56,94,360,360,186,125,376,118,
    137,143,113,9,366,391,391,163,
    18,350,305,39,267,69,133,267,
    71,303,351,145,394,299,376,116,
    409,123,130,201,100,318,351,31,
    303,370,193,353,87,8,80,5,38,
    260,346,346,283,8,12,249,378,378,
    146,258,334,371,258,398,24,377,343,145,270,
    130,264,260,258,399,364,118,211,344,
    89,89,117,104,399,384,94,272,138,
    94,38,74,9,352,222,58,62,267,
    375,389,254,38,375,152,116,143,
    174,94,272,389,105,268,360,249,249,20,352,
    401,69,17,19,106,391,268,299,370,
    332,181,106,59,339,372,174,37,
    268,210,100,189,49,130,130,196,8,94,
    367,341,179,401,370,352,165,137,
    174,370,360,254,138,100,114,299,370,344,
    209,266,263,222,111,266,398,195,170,
    394,134,216,313,258,357,138,267,
    354,374,134,338,35,10,80,370,116,
    305,361,90,301,301,94,352,394,
    349,94,174,169,13,41,377,375,
    138,59,366,333,68,338,130,348,
    398,399,88,143,202,258,258,376,
    146,166,201,261,313,350,188,174,
    70,331,325,164,116,116,376,8,
    88,249,345,78,370,377,320,269,
    260,283,222,265,345,177,113,343,
    73,80,7,66,344,24,367,372,
    113,32,71,166,152,133,355,333,
    197,360,203,376,138,303,361,129,
    366,14,280,345,94,377,199,345,
    94,281,355,375,267,197,351,372,
    303,39,324,401,410,66,94,377,
    163,201,166,58,125,267,70,174,
    348,379,268,214,185,47,279,372,
    136,7,169,244,313,350,49,350,258,
    377,347,176,315,7,279,279,133,
    349,359,116,264,370,13,249,126,324,
    370,75,213,220,119,351,125,111,130,193,207,
    370,346,372,328,399,25,290,258,
    195,61,297,303,394,26,38,66,4,
    186,345,70,398,324,36,251,268,143,249,
    376,32,135,189,185,264,399,371,136,
    307,346,354,343,394,394,193,209,193,
    113,182,197,350,50,174,195,354,
    32,392,196,201,353,209,414,313,267,
    326,398,248,248,137,268,17,178,
    243,111,350,133,404,129,88,11,170,178,
    145,145,130,371,363,32,137,296,216,
    354,349,39,363,313,41,37,60,
    174,355,296,370,140,57,32,258,
    49,353,299,189,264,372,31,174,
    386,361,176,401,385,355,196,355,
    258,279,135,201,118,281,132,408,
    138,253,119,205,86,172,138,166,
    206,174,47,174,267,226,186,75,
    354,401,184,174,184,90,369,249,
    215,106,144,372,305,350,24,268,
    269,75,24,195,268,138,401,408,
    355,362,129,236,246,236,356,399,194,
    84,152,205,350,117,357,366,149,377,
    268,181,361,305,352,334,353,138,
    352,364,125,345,61,39,345,61,
    407,122,268,370,370,21,97,376,
    16,30,370,296,36,94,112,89,
    307,54,138,212,399,59,278,399,399,
    138,398,355,272,398,276,264,139,
    146,377,199,31,4,225,130,272,
    134,340,181,58,9,245,245,368,416,
    13,298,323,143,138,119,152,362,359,396,
    370,241,94,66,341,94,112,398,
    398,272,243,370,197,340,212,145,105,
    361,394,268,10,249,376,350,202,20,
    20,94,38,235,38,258,79,235,158,276,248,134,138,
    135,415,20,209,138,84,100,281,401,111,107,
    371,23,179,147,117,405,60,360,
    164,154,225,305,134,164,37,174,
    144,301,256,100,138,370,376,396,182,
    267,271,133,370,21,405,310,294,
    271,174,176,176,158,135,9,32,
    13,249,164,325,377,181,38,33,40,61,
    79,16,177,33,297,246,246,88,377,112,
    189,113,366,2,75,350,329,398,143,370,
    258,113,107,154,258,329,329,94,
    39,355,14,70,164,76,359,359,
    119,377,9,9,94,376,337,366,
    130,10,401,190,245,59,379,321,
    105,57,127,279,377,281,213,141,
    320,7,338,38,289,225,372,138,
    260,127,158,176,168,174,394,303,
    190,370,70,355,352,345,16,26,
    133,261,292,9,353,13,94,135,
    404,135,54,133,59,380,86,20,
    353,356,17,274,195,168,4,385,
    111,26,318,230,32,176,13,139,
    60,305,323,13,168,94,281,398,
    77,305,341,303,6,355,20,36,
    167,184,350,352,168,394,58,143,
    382,303,135,243,370,168,364,350,
    364,368,90,323,264,94,91,94,5,255,
    119,133,133,135,352,109,14,366,111,
    145,137,250,197,202,202,252,205,303,313,
    32,396,145,137,202,331,176,368,398,
    146,350,296,345,350,330,376,168,
    80,75,264,265,244,133,207,372,105,
    268,391,139,109,65,135,14,189,187,268,
    135,345,145,137,268,189,168,301,66,
    109,135,352,109,366,111,202,303,32,
    168,145,137,133,350,66,330,376,105,
    139,268,137,145,267,139,52,145,398,
    34,133,106,59,409,414,66,297,126,355,
    269,100,303,138,355,111,104,41,138,355,
    131,267,357,316,222,133,258,145,398,
    389,13,357,125,297,104,398,362,125,
    41,350,370,174,186,145,350,366,350,
    366,366,71,94,267,267,137,123,
    133,86,363,69,123,31,325,360,
    138,59,276,363,371,296,258,340,
    133,363,371,80,89,364,368,314,
    301,371,356,145,354,217,36,375,
    398,358,87,356,34,299,366,286,
    407,360,370,370,316,38,119,301,
    119,360,396,401,397,105,409,409,
    391,106,94,135,70,181,66,367,
    174,216,243,400,97,370,143,368,
    389,340,370,268,393,254,13,358,
    268,5,57,412,325,401,49,394,
    374,360,363,370,129,119,303,30,
    354,303,121,30,105,111,269,130,
    138,126,96,353,345,301,40,334,
    202,391,207,189,130,366,358,107,
    84,19,331,69,370,38,38,172,401,162,159,
    349,376,328,133,398,276,52,169,
    80,162,81,350,303,333,59,10,20,32,
    375,155,262,264,306,2,376,354,
    37,138,352,349,349,99,314,21,
    130,140,312,310,378,396,312,310,378,75,126,33,
    310,300,138,152,268,145,50,354,318,344,
    352,88,38,167,321,370,222,212,371,69,331,
    249,10,408,32,36,407,133,258,323,
    406,345,143,265,73,397,385,416,389,56,411,
    260,408,177,135,41,133,351,118,188,301,
    16,126,14,376,70,360,250,303,66,
    361,303,131,126,107,80,399,66,355,
    94,257,333,135,258,376,409,404,
    350,295,1,130,371,2,352,214,36,90,
    401,367,366,129,361,100,235,360,
    210,369,345,357,328,400,296,135,
    20,163,129,132,100,372,202,354,
    202,350,261,36,362,329,316,8,
    38,260,303,136,377,355,119,325,
    368,368,186,376,16,50,265,174,
    209,209,297,394,208,135,385,138,
    176,185,24,240,112,350,408,4,
    4,139,394,370,125,136,195,34,
    116,126,32,360,388,291,350,389,
    77,397,216,168,80,372,145,133,
    415,137,20,130,404,349,387,389,
    303,262,323,135,257,302,361,384,
    323,60,318,352,133,137,140,391,
    230,370,1,391,249,130,126,370,
    370,296,273,280,260,77,321,125,
    400,118,1,372,135,376,135,130,
    75,394,144,361,382,172,301,345,32,
    174,370,338,14,394,366,80,40,345,
    40,368,32,273,371,168,36,355,
    226,128,382,370,60,391,366,75,
    366,133,71,94,276,133,138,123,
    325,273,296,258,340,363,370,363,
    133,276,136,130,240,143,364,217,
    360,125,80,188,358,314,90,299,87,
    145,397,106,119,254,412,303,398,358,
    389,316,396,66,400,49,268,393,
    13,370,370,58,162,172,303,107,303,
    138,133,130,37,401,301,126,59,105,
    269,111,363,370,397,96,353,367,30,
    131,360,400,40,138,349,376,262,349,
    99,375,130,162,312,310,378,314,81,350,265,
    401,411,235,75,74,408,88,152,345,
    376,310,301,331,69,400,32,177,407,318,
    323,301,370,210,36,70,129,135,
    355,362,369,345,80,376,361,32,
    409,2,366,66,202,250,360,209,
    60,316,355,368,8,303,260,202,
    139,195,394,135,208,323,387,262,
    168,257,145,366,260,391,36,106,
    260,123,351,133,123,116,123,350,
    350,132,126,178,116,75,184,74,136,
    258,38,174,65,343,13,305,352,
    90,398,398,366,366,303,41,130,
    339,370,339,370,135,5,124,80,
    41,353,128,135,366,154,96,143,94,
    350,18,118,376,401,134,89,350,
    125,346,128,18,66,410,89,370,
    398,9,31,2,249,212,249,105,
    212,375,69,209,313,359,128,154,164,
    119,209,119,118,209,209,2,197,174,222,13,
    376,134,337,197,249,350,370,143,185,
    209,41,323,128,145,10,396,377,379,
    94,23,104,326,370,117,343,253,
    132,86,323,109,385,390,398,84,401,
    303,13,409,84,111,250,14,194,
    58,326,302,162,88,332,370,38,197,
    119,13,11,186,180,130,96,250,409,
    134,106,360,386,137,96,383,135,372,
    146,396,299,18,18,267,299,44,
    383,400,167,382,49,36,297,330,
    246,103,352,194,135,318,94,59,
    50,50,398,133,392,75,139,358,206,
    47,379,9,381,167,90,25,133,
    302,1,404,413,94,105,287,385,178,
    370,6,36,343,404,398,406,16,379,
    388,59,382,366,257,296,343,372,
    139,97,352,383,13,75,305,366,
    297,361,184,97,383,10,396,94,
    377,379,104,23,385,352,6,392,132,
    398,86,323,253,14,105,401,109,
    84,135,13,11,303,332,111,162,58,
    197,88,119,370,386,398,106,134,130,
    409,180,186,383,409,96,139,267,
    396,167,299,94,75,133,305,297,
    49,13,400,103,246,59,167,90,
    406,94,404,287,385,366,382,379,
    388,296,372,97,38,350,299,214,
    334,350,37,119,37,394,351,324,
    411,411,174,142,94,393,97,258,
    296,266,371,352,409,145,264,38,
    49,36,36,70,336,263,143,34,66,350,
    391,145,378,268,52,133,138,268,41,107,132,
    362,409,331,79,179,97,320,52,
    350,393,316,371,143,135,270,258,133,324,
    48,408,54,186,268,52,60,267,409,329,
    268,52,38,129,262,262,137,384,329,378,
    84,382,382,412,241,9,20,159,349,152,
    78,145,111,94,36,135,87,244,398,321,
    378,5,245,258,258,378,261,340,322,370,
    135,36,181,199,5,70,158,340,134,
    49,245,259,401,143,68,332,70,398,94,
    243,7,143,263,296,20,222,143,174,189,102,
    370,133,58,79,38,352,137,79,401,269,
    159,403,111,266,163,353,70,186,
    250,12,398,138,331,325,23,135,57,262,
    13,352,79,133,143,133,305,40,336,
    48,140,226,354,21,362,271,211,
    305,177,374,137,40,262,210,321,
    135,133,348,345,48,138,133,226,
    143,226,188,186,173,127,143,38,
    343,269,329,20,412,263,258,52,
    410,23,410,248,398,397,68,398,
    376,79,78,44,374,399,66,394,
    36,43,135,107,324,143,94,52,
    70,250,280,235,329,30,338,135,
    61,56,350,321,261,223,68,329,
    133,226,243,182,382,13,39,186,
    178,52,324,58,316,350,163,133,
    398,261,66,243,410,176,12,384,
    223,17,338,143,65,29,352,86,
    41,399,78,20,52,52,145,145,
    180,321,262,262,257,178,78,53,
    109,384,321,13,13,401,143,41,
    262,78,40,133,349,378,223,180,
    179,398,174,189,398,32,41,76,345,
    184,180,352,345,413,168,355,273,
    286,355,226,321,268,133,53,413,350,
    163,145,180,301,104,59,89,268,
    329,79,79,104,169,276,189,1,
    133,143,324,156,170,366,199,150,
    268,185,170,79,398,366,329,61,
    372,376,35,143,364,389,95,111,146,345,378,
    356,352,58,361,86,111,276,296,162,305,
    339,36,58,80,212,258,197,282,
    162,260,404,123,125,268,162,66,
    181,58,4,396,86,162,367,248,
    10,106,106,245,401,279,80,5,
    400,398,368,152,370,70,265,303,254,
    84,104,143,137,110,186,148,269,
    400,381,398,299,177,376,298,375,
    343,371,394,343,94,265,400,222,
    181,394,116,177,409,130,344,48,
    113,149,370,248,260,112,223,254,
    109,10,188,242,177,282,280,133,
    367,352,44,51,47,100,375,123,
    305,94,409,94,346,86,391,376,
    346,325,106,396,351,377,186,137,
    34,404,345,131,362,394,137,391,
    21,170,89,86,180,100,291,149,
    128,370,133,77,84,376,135,123,
    172,246,174,174,186,180,35,143,364,
    111,361,58,276,404,403,80,188,282,
    123,106,152,186,400,398,370,125,
    396,174,368,265,303,381,398,137,
    400,269,186,137,394,94,177,223,
    10,130,112,344,177,48,409,51,
    94,133,346,305,246,377,351,391,223,
    186,394,180,356,106,49,49,13,249,
    414,14,166,166,49,362,7,14,
    14,14,362,14,7,49,14,14,
    36,281,230,230,396,48,48,370,
    277,14,61,14,303,376,178,57,32,
    97,260,376,376,258,363,370,113,
    194,258,380,344,162,336,407,372,57,
    379,139,117,364,86,349,57,80,
    128,115,394,395,57,139,377,345,176,38,
    35,38,331,398,174,370,141,134,36,
    58,84,66,255,242,401,344,70,385,325,
    305,370,268,140,130,73,375,202,
    12,133,213,370,138,406,77,179,363,
    338,314,303,325,244,124,222,78,
    141,361,363,21,375,354,267,335,
    401,267,66,66,336,140,329,74,
    370,394,334,110,349,303,37,316,
    384,271,90,176,320,130,174,106,
    167,11,56,401,12,128,58,186,
    375,400,139,376,48,163,345,329,
    370,57,377,189,13,235,376,60,
    318,78,318,366,44,38,66,329,376,
    303,396,375,379,80,14,113,80,
    351,129,267,61,57,345,214,370,
    105,368,40,182,363,321,66,38,
    377,316,321,260,193,368,109,392,
    4,303,22,38,316,384,394,78,
    66,185,38,56,180,415,274,260,
    361,376,370,80,178,143,303,13,
    368,194,355,318,128,115,391,328,84,
    204,14,14,166,174,38,377,368,189,
    174,370,333,65,258,374,296,116,
    376,196,281,266,350,162,94,150,117,
    18,87,357,212,218,356,301,8,377,
    55,132,355,364,369,376,360,8,349,143,375,116,
    322,267,13,249,19,298,10,341,
    66,411,369,180,162,111,401,303,
    158,376,96,115,119,263,350,398,133,363,128,
    124,357,137,350,111,212,169,134,
    160,397,169,379,366,37,74,350,
    190,94,349,94,99,118,169,134,
    103,146,372,20,350,10,174,398,379,
    21,354,4,258,249,265,113,400,323,
    411,254,167,222,36,375,21,353,
    59,143,374,262,370,75,74,366,199,
    285,10,80,305,144,376,379,124,
    163,353,353,315,324,207,350,281,
    41,409,411,370,349,353,379,118,
    374,13,197,34,94,178,371,404,
    125,262,366,392,195,262,360,65,
    13,363,13,388,345,397,197,296,
    180,255,59,201,369,26,160,90,
    201,411,162,176,382,32,375,258,
    366,32,382,181,128,350,90,382,
    174,375,71,267,408,246,400,370,
    97,376,142,366,414,197,59,360,
    74,396,89,377,94,379,322,330,
    259,340,416,116,106,316,255,40,
    381,207,170,48,40,375,334,398,
    352,136,37,371,336,137,199,158,
    317,172,257,414,115,366,303,224,223,
    345,186,168,366,325,246,391,47,
    323,59,414,406,52,164,329,352,75,
    125,360,357,323,267,40,47,379,255,85,
    152,315,202,269,40,56,379,374,
    3,389,115,324,136,251,32,36,376,
    174,384,170,370,136,21,137,350,
    323,255,85,230,370,303,174,143,366,176,352,370,
    224,281,363,40,366,181,202,202,
    224,356,137,303,202,366,14,23,
    303,375,303,303,174,399,39,369,177,
    174,350,139,139,95,370,178,61,393,
    71,255,267,119,94,396,398,5,
    187,94,213,69,296,262,137,157,44,
    409,86,126,376,126,349,116,98,258,196,
    278,276,135,66,58,313,350,370,31,303,370,336,
    350,236,260,267,278,276,135,249,393,369,364,139,
    5,87,36,357,74,378,260,94,
    21,212,356,80,145,78,105,371,
    260,7,286,276,34,229,89,379,
    370,264,249,113,123,371,146,69,
    370,399,350,96,278,132,322,150,
    377,186,80,264,79,409,222,336,
    303,206,106,152,181,19,313,106,
    20,249,376,313,416,21,375,68,
    134,396,303,303,332,143,413,303,
    321,361,393,9,119,13,302,41,
    303,20,401,38,380,255,334,260,
    94,390,197,260,94,174,378,249,
    367,7,20,138,105,305,397,211,
    350,350,66,134,211,323,301,370,
    313,162,147,10,135,334,357,123,
    137,38,84,100,19,254,303,197,114,
    371,146,400,39,353,141,334,209,172,
    133,376,313,360,130,276,415,398,266,296,312,
    38,174,352,350,357,269,249,332,401,124,353,
    207,159,69,331,368,352,165,330,107,352,359,146,30,
    170,133,249,281,202,370,371,110,
    2,72,375,291,151,260,187,313,
    1,69,116,283,303,398,155,267,354,
    394,226,359,383,329,56,352,165,330,107,123,104,399,374,
    335,336,74,190,199,201,169,343,135,356,379,10,
    349,316,376,32,333,71,20,116,134,
    123,144,135,53,90,32,343,398,313,340,361,144,
    126,349,349,331,162,408,48,191,357,140,264,
    301,116,191,369,41,388,143,352,
    80,196,257,174,243,283,37,99,
    174,326,19,401,396,336,182,414,226,
    143,33,377,343,135,98,69,325,297,
    188,152,181,249,186,174,265,246,
    144,206,414,248,2,249,352,364,
    406,172,0,156,321,164,75,218,
    46,409,397,11,226,50,47,323,
    71,258,260,406,133,376,139,109,
    197,33,330,350,176,69,106,56,
    305,396,186,201,186,126,16,95,
    167,154,87,21,213,343,382,125,
    62,352,250,132,177,85,200,148,
    367,38,176,113,352,75,336,345,
    410,94,280,133,80,146,36,329,
    389,125,367,76,351,376,155,302,
    129,345,94,393,30,263,303,123,
    163,235,210,262,262,124,335,50,
    128,369,206,135,76,135,313,163,
    125,361,394,138,396,14,399,409,
    359,369,199,242,1,138,260,199,
    56,30,57,321,8,351,176,320,291,148,182,
    368,412,369,321,100,231,347,279,324,320,261,37,
    100,174,312,46,20,243,57,13,249,289,
    98,409,349,372,129,331,182,148,
    319,294,315,343,135,99,118,396,396,169,
    370,377,324,226,350,134,100,193,
    144,314,412,320,351,90,346,212,
    186,320,240,412,48,337,359,109,361,
    176,304,315,4,195,209,189,13,345,
    182,66,288,262,24,50,370,186,4,4,155,
    261,54,258,297,324,195,374,32,
    90,140,16,305,185,359,50,184,
    382,135,382,26,174,351,350,150,309,
    12,392,260,397,186,126,133,257,
    130,318,283,261,255,180,291,359,288,352,286,37,
    111,313,182,216,129,252,318,86,
    262,269,350,324,353,145,137,415,
    178,258,170,77,356,382,133,135,
    399,65,364,372,77,145,231,382,
    257,332,86,37,71,296,148,135,
    88,318,186,144,130,376,176,408,
    262,135,408,172,13,332,128,369,
    79,113,60,37,143,89,57,10,370,
    1,410,363,69,401,122,406,133,
    226,119,132,265,18,372,111,228,
    360,135,135,260,30,398,205,174,
    172,133,413,162,297,248,166,75,
    312,48,190,16,9,186,352,161,
    184,80,186,356,135,168,20,135,
    378,32,353,135,350,109,25,226,
    172,53,268,243,189,413,187,384,
    226,145,324,401,168,139,95,370,
    396,71,393,255,178,336,260,44,
    296,286,350,86,69,200,236,367,31,
    357,96,21,322,143,78,34,399,
    212,10,98,7,260,378,368,264,146,
    349,105,150,87,132,74,229,5,241,
    376,260,397,260,106,20,152,255,
    21,20,378,413,211,323,134,68,330,
    375,332,20,181,312,260,366,197,9,
    303,361,321,340,13,222,249,79,357,
    151,170,84,196,364,375,37,134,
    369,216,398,60,37,334,190,69,371,
    148,389,401,352,350,333,71,72,352,165,330,107,126,
    269,294,114,152,69,331,368,100,207,397,291,
    137,370,32,39,324,2,371,281,
    401,170,257,349,167,326,176,155,
    354,320,174,388,41,113,99,80,
    359,56,191,90,356,182,148,135,
    283,329,169,264,143,0,261,394,
    235,56,197,11,258,62,152,164,
    33,350,106,189,46,406,139,398,
    352,144,132,246,323,71,135,143,
    201,409,263,372,148,261,313,80,
    30,262,399,76,315,129,128,1,
    75,199,185,409,88,199,209,396,
    20,100,226,324,144,226,212,182,
    99,8,370,134,18,279,16,324,
    195,189,12,374,140,66,412,361,
    182,356,145,178,257,186,77,168,
    257,53,261,65,132,172,128,408,
    176,370,30,16,166,32,353,33,392,
    33,142,4,70,138,178,202,33,392,
    200,193,308,296,132,296,200,366,13,
    116,13,13,296,148,150,12,123,284,
    288,352,352,135,135,206,351,310,74,
    389,216,391,248,351,152,181,14,109,13,
    284,119,109,100,119,100,85,41,123,353,
    111,206,291,350,164,169,190,333,294,
    143,378,378,32,268,180,33,294,
    164,366,346,366,80,366,131,376,346,
    123,9,123,144,353,268,368,346,7,243,2,
    345,371,165,270,168,75,299,269,90,
    330,226,321,148,119,270,45,109,
    74,258,163,324,33,109,251,149,116,350,291,321,
    130,32,249,60,128,321,346,321,
    200,308,296,366,116,13,13,346,45,
    284,345,352,123,135,206,150,200,
    389,216,111,346,321,206,190,148,
    85,100,119,164,142,378,169,75,299,
    376,366,33,350,346,131,366,80,
    32,168,268,130,165,270,119,330,
    321,270,149,128,94,94,171,77,
    356,260,349,370,340,371,367,74,
    80,302,7,246,155,379,282,398,
    249,140,87,367,371,396,138,37,
    80,268,66,412,416,68,180,0,80,
    340,340,10,249,255,19,94,133,186,184,
    36,357,79,185,209,136,353,305,79,
    352,84,111,376,96,296,146,262,
    357,47,349,13,351,296,302,398,
    257,74,377,396,41,352,61,226,
    379,352,246,88,411,258,77,188,
    371,143,46,36,249,181,325,352,
    186,302,352,371,401,367,277,351,
    39,366,371,376,368,305,66,376,184,345,
    345,226,77,406,318,79,2,129,138,318,
    371,258,96,1,366,130,79,100,379,349,345,163,
    1,350,324,133,392,61,4,350,
    371,286,274,180,338,65,249,318,
    318,4,376,352,89,222,84,133,61,
    350,371,398,130,79,184,350,174,58,174,58,
    174,58,406,54,345,125,119,398,319,146,144,214,235,370,
    270,262,366,264,135,358,364,133,106,
    128,398,105,146,144,49,374,143,41,
    125,380,189,376,40,69,318,116,
    132,309,109,128,41,380,374,133,111,350,
    40,182,174,214,235,376,380,40,133,
    376,376,362,212,93,291,350,211,346,
    89,244,379,174,38,367,181,172,
    2,9,349,201,68,60,125,349,69,
    360,133,211,36,354,389,333,396,
    246,199,181,258,400,132,294,88,
    123,391,371,222,305,339,180,181,
    73,372,349,181,309,181,351,123,
    371,194,194,379,182,201,18,349,
    345,165,371,350,370,1,59,328,
    352,376,185,186,184,58,133,244,367,
    5,249,345,90,350,133,194,201,
    201,172,174,132,1,88,58,184,
    181,1,90,174,9,119,119,119,
    19,265,265,140,177,330,397,140,37,
    265,140,140,68,140,330,88,88,
    151,202,203,203,245,369,203,130,
    369,100,71,30,135,276,66,75,
    349,276,264,139,362,229,5,371,
    286,212,209,412,57,7,355,368,
    325,10,138,123,245,367,19,371,
    100,321,286,325,138,133,355,2,2,121,104,
    259,57,262,333,195,200,14,372,318,331,
    262,298,361,144,156,12,321,297,392,19,249,13,10,165,
    143,166,355,70,280,8,83,267,267,
    119,262,211,210,143,135,14,66,135,
    346,379,325,105,321,10,355,243,100,
    13,19,165,324,185,111,262,362,133,
    135,136,32,57,132,352,260,75,
    341,135,168,345,276,94,199,341,269,
    100,345,262,116,33,165,280,379,
    299,355,345,100,6,325,105,379,99,
    13,345,318,75,341,75,345,276,
    94,116,345,379,346,325,142,142,352,
    355,352,133,371,380,379,298,171,
    248,129,372,379,248,2,371,353,
    125,369,71,265,267,353,311,116,
    360,370,360,80,314,163,258,117,
    376,343,7,78,66,59,243,255,
    181,35,140,172,119,262,80,80,
    345,138,165,301,370,370,152,77,
    376,254,172,94,134,335,130,163,
    134,189,333,37,372,146,125,116,
    140,338,338,18,167,338,409,409,
    46,71,167,323,116,260,152,54,
    141,264,370,287,329,80,80,366,
    346,149,374,404,366,352,356,370,
    377,289,68,68,136,163,172,170,
    251,342,195,52,368,118,262,106,
    363,366,130,32,281,201,18,352,
    253,186,168,226,269,369,71,265,
    116,353,311,360,360,343,106,78,
    258,7,314,117,376,186,181,255,
    140,103,138,355,134,134,333,119,100,372,141,152,
    370,253,18,130,338,116,372,372,152,
    329,374,80,404,366,80,226,195,
    68,289,118,172,32,391,281,253,269,
    90,16,72,107,94,351,391,16,286,
    5,94,322,179,107,361,354,143,16,
    313,345,367,368,315,148,290,315,86,
    182,350,182,178,251,251,182,16,16,
    16,178,16,291,90,359,90,367,
    391,16,286,143,313,315,368,182,
    251,16,16,88,86,88,88,303,313,370,
    303,24,133,71,313,340,391,319,
    353,339,276,376,367,144,38,371,86,
    86,319,371,401,335,370,313,416,385,389,13,138,
    325,9,49,332,313,9,303,79,
    115,276,330,137,119,19,368,334,
    49,353,367,144,84,366,171,350,
    24,20,218,80,21,146,74,316,
    376,303,368,131,113,303,135,48,
    19,352,21,369,59,88,392,345,
    109,80,233,379,125,129,332,130,
    135,124,1,357,89,345,106,30,
    314,324,20,99,350,163,182,315,
    325,369,346,209,324,195,13,376,
    359,139,288,163,404,296,350,59,
    370,133,274,37,374,325,345,353,
    391,89,115,201,366,209,32,353,215,
    189,382,215,303,71,133,340,357,
    339,350,276,376,38,86,371,135,
    303,9,313,79,370,84,274,353,
    119,100,171,137,350,19,20,74,80,
    376,218,146,113,131,352,109,30,
    163,106,315,32,369,209,20,182,
    359,139,195,288,404,215,304,163,
    113,353,89,20,222,13,20,336,
    116,88,135,2,1,94,352,379,348,
    356,89,253,356,193,376,90,254,116,
    66,340,79,340,394,38,363,401,398,303,246,
    356,139,278,286,379,346,398,59,186,
    375,20,9,145,160,340,79,370,268,346,
    268,141,255,393,377,248,400,143,
    401,232,143,249,383,134,181,396,
    322,58,94,367,303,13,340,340,313,
    182,193,250,325,398,279,328,73,
    363,361,269,301,141,84,115,20,401,
    371,189,400,59,115,182,143,314,
    264,196,177,169,116,336,361,338,146,
    80,37,357,313,186,406,400,299,
    250,164,325,167,410,152,258,258,
    366,88,290,366,100,368,349,250,
    50,250,260,88,129,260,132,376,
    329,269,351,410,163,280,313,107,
    340,111,315,260,37,398,182,248,
    328,350,26,75,366,377,411,290,
    296,258,398,309,186,350,189,392,
    209,4,24,251,50,268,13,398,
    376,360,126,20,316,354,180,391,
    78,182,340,29,68,137,332,366,
    189,391,140,370,369,340,253,400,
    366,184,190,328,353,133,309,143,
    350,128,174,16,193,376,340,363,
    38,268,278,20,190,383,303,313,
    94,143,411,401,340,232,134,370,
    322,354,193,371,137,126,189,115,
    250,16,174,37,366,357,264,146,
    258,258,152,406,410,316,24,250,
    398,163,290,349,4,182,260,296,
    251,16,189,50,32,400,133,309,353,
    106,345,345,345,376,97,370,3,
    335,138,9,10,209,49,329,66,158,
    115,262,354,124,159,100,338,103,250,
    13,152,259,376,318,185,20,354,
    8,20,145,49,161,18,209,178,185,
    354,75,383,318,329,18,161,186,
    99,99,262,151,262,170,290,16,
    164,164,66,87,359,272,197,59,
    164,18,85,331,249,409,85,272,
    329,9,13,255,197,201,94,84,84,268,
    104,359,165,378,133,248,402,298,294,
    329,174,18,410,329,248,314,397,
    269,410,311,135,79,125,166,142,
    258,176,396,18,248,193,288,195,
    195,293,360,179,260,260,230,128,
    165,228,18,179,273,74,74,216,
    123,350,74,149,74,74,142,33,
    376,376,100,174,366,94,388,111,410,
    182,111,297,376,111,199,133,258,
    95,163,131,5,255,199,360,366,
    354,177,376,338,258,344,177,345,
    97,38,251,13,209,133,360,40,
    366,391,376,61,276,133,5,123,
    340,69,133,376,80,133,294,117,
    339,209,138,301,7,377,249,186,
    346,125,186,380,87,89,212,375,
    250,209,119,351,268,116,249,181,
    340,5,267,254,94,13,49,133,345,
    143,69,20,5,375,112,249,223,357,
    322,9,94,389,143,106,303,73,
    40,58,321,138,305,124,353,84,2,
    345,393,401,371,179,189,100,334,370,
    370,19,345,137,158,111,355,341,163,352,100,
    130,170,94,151,359,340,146,329,
    203,298,389,320,264,376,218,394,
    112,103,316,349,267,296,257,21,128,
    331,174,294,294,151,201,37,174,
    411,350,374,301,409,258,265,353,
    218,47,133,69,263,106,400,73,
    167,88,222,370,313,164,186,142,33,
    140,188,181,411,174,201,410,398,
    223,125,376,66,303,301,128,329,
    124,357,401,166,410,133,14,14,
    128,269,386,345,345,376,47,280,
    70,129,176,366,267,267,135,13,
    80,367,94,287,135,351,340,125,
    303,285,361,346,135,118,349,244,
    290,182,193,303,303,109,409,328,
    321,368,80,374,260,258,346,285,
    301,176,4,171,130,206,133,331,
    268,135,301,195,350,267,251,133,
    133,401,136,359,404,374,392,150,
    362,17,376,268,353,20,137,363,
    316,129,415,296,296,86,111,180,
    363,368,350,388,353,89,109,124,
    160,386,290,391,97,111,372,174,
    33,172,305,1,281,133,360,125,
    305,174,179,171,205,396,353,80,
    186,109,174,352,376,61,133,375,
    339,186,87,5,119,100,5,254,223,
    186,375,389,94,20,5,9,124,249,
    322,111,355,138,151,345,84,334,386,
    124,160,133,137,352,389,353,363,
    103,174,176,135,174,303,331,112,
    294,128,146,133,374,265,181,258,
    411,88,164,33,106,222,223,69,
    140,301,303,409,89,70,13,33,
    329,346,345,287,350,80,267,94,129,
    269,136,14,290,4,258,321,109,
    368,244,135,171,16,362,17,195,
    206,374,345,350,111,145,296,180,415,
    125,97,174,391,296,109,225,69,370,94,
    174,142,21,366,94,69,393,133,90,
    281,97,116,366,303,90,207,9,377,398,
    125,264,94,111,7,89,346,135,260,391,303,376,
    93,368,145,145,249,128,396,9,
    366,364,397,87,90,346,240,58,
    134,281,181,205,94,340,206,174,
    14,398,100,377,49,268,354,38,
    59,143,368,106,399,376,367,376,
    364,70,376,330,372,77,349,84,
    107,1,398,366,122,354,134,179,
    401,367,370,123,186,281,210,100,
    276,137,359,400,38,189,122,223,
    80,187,134,133,336,128,340,21,
    349,135,376,20,146,146,13,350,
    146,143,336,140,329,80,80,162,
    125,349,301,167,382,243,186,249,
    305,94,2,408,248,264,260,10,
    69,186,270,135,143,336,364,377,
    258,174,369,406,156,79,164,302,
    258,140,370,370,140,409,167,73,
    258,47,103,143,268,370,415,133,
    305,372,38,204,280,2,267,329,38,
    125,329,38,80,138,197,94,13,47,336,
    366,119,138,377,250,14,164,199,125,372,
    44,398,349,143,73,25,261,87,119,125,372,
    377,352,347,303,119,41,324,351,
    285,182,133,106,125,135,319,363,116,49,
    49,370,368,366,133,174,330,157,
    329,329,370,336,193,354,99,330,
    36,133,337,394,4,368,370,240,
    38,398,182,374,185,190,13,309,408,
    376,349,145,371,329,313,137,370,
    126,13,372,316,129,86,137,178,
    366,99,142,352,352,336,194,415,
    376,372,186,337,352,362,370,249,
    401,189,350,370,133,385,376,391,
    369,367,249,228,125,202,372,201,
    66,378,376,172,21,186,119,184,
    309,378,372,109,268,174,187,225,
    142,133,377,207,303,240,364,25,
    9,396,106,73,186,364,354,367,
    181,38,268,377,362,340,313,398,
    84,107,359,122,400,100,187,123,
    349,20,174,144,125,80,376,352,
    329,349,270,204,2,164,10,248,
    260,47,103,377,316,125,119,80,
    106,267,49,199,349,370,368,347,
    182,133,370,135,119,370,372,394,
    182,178,137,142,376,186,128,391,
    372,125,201,109,309,186,139,181,
    135,352,56,135,135,366,56,186,
    375,52,133,245,16,52,245,401,52,146,271,
    401,135,202,202,376,182,36,146,
    180,222,258,186,142,146,140,174,
    353,352,134,202,174,299,392,180,
    140,258,181,366,52,194,194,119,
    34,94,203,203,94,245,268,268,
    210,94,352,167,268,203,38,90,
    94,268,203,193,209,198,209,198,130,202,
    411,234,89,129,129,139,110,330,
    335,123,126,162,123,305,174,223,
    38,120,120,370,260,59,350,339,
    209,209,260,58,41,375,68,370,
    351,366,268,199,366,265,378,174,
    60,75,24,366,366,366,59,301,2,
    396,366,58,24,370,199,59,391,366,75,
    186,398,89,94,94,206,203,201,206,203,201,377,
    52,268,34,341,401,398,201,4,
    17,340,13,377,34,340,71,202,
    213,71,409,106,106,73,89,325,
    377,249,33,99,26,377,324,328,
    305,305,89,88,346,5,69,340,
    399,268,302,303,375,303,333,349,
    143,140,131,143,366,336,313,350,
    352,366,172,13,368,267,116,349,
    349,124,355,80,389,359,347,389,
    230,215,258,390,390,133,409,133,133,
    258,133,409,390,133,38,36,36,119,364,371,
    355,9,385,355,409,38,366,143,
    331,181,181,41,269,355,371,226,
    142,368,48,379,376,41,370,222,
    385,411,268,379,366,376,80,348,
    370,49,411,68,41,139,364,38,
    36,119,371,154,143,181,9,331,409,
    371,154,376,48,268,348,184,244,104,348,
    244,366,184,184,104,149,57,181,
    57,184,104,149,111,146,267,267,17,111,146,267,
    378,46,119,137,355,376
};

/*
 * 每个字符对应的拼音在上表中的索引，
 * pinyin_index[n+1] - pinyin_index[n] = 第n个字的拼音个数。
 **/
uint16_t pinyin_index[] = {
0,1,3,6,7,8,9,10,
12,13,14,15,16,18,20,21,
22,23,24,25,26,28,29,30,
31,32,33,34,35,36,37,38,
39,40,41,42,43,44,45,46,
47,48,49,50,51,52,53,54,
55,56,58,59,60,61,62,63,
64,65,66,67,68,69,70,71,
72,73,75,76,77,78,79,80,
82,86,87,88,89,90,91,92,
93,95,96,97,98,99,100,101,
103,105,106,108,109,111,112,113,
114,115,116,117,118,119,120,121,
122,123,124,125,126,127,128,129,
130,131,132,133,134,135,136,137,
138,139,140,141,142,143,144,146,
148,149,150,151,152,153,154,156,
158,159,160,161,162,163,164,165,
166,167,168,169,170,171,172,173,
174,175,176,177,179,180,182,183,
185,186,188,189,190,191,192,193,
194,196,197,198,199,200,201,202,
203,204,205,207,208,209,210,211,
212,213,215,216,217,218,219,220,
221,223,224,225,226,227,228,229,
231,232,233,234,235,236,237,239,
240,242,243,244,245,247,248,249,
250,251,252,253,256,257,258,259,
260,262,264,265,266,267,268,269,
271,272,273,274,275,276,277,278,
280,282,283,284,285,286,287,288,
290,292,293,294,295,296,298,299,
300,301,302,303,304,305,306,307,
308,309,310,311,312,313,314,316,
317,318,320,322,323,324,325,326,
327,328,329,331,332,333,334,335,
336,338,340,341,342,343,344,345,
347,348,349,350,351,352,354,355,
358,359,360,361,362,363,364,365,
366,367,368,370,371,373,376,377,
378,379,381,382,384,386,387,388,
389,390,391,392,394,395,396,397,
398,399,400,401,404,406,407,408,
409,410,411,413,417,418,420,421,
422,423,424,425,426,427,428,429,
430,431,432,434,435,436,437,438,
439,440,441,442,443,445,446,447,
448,449,450,451,452,453,454,455,
456,457,458,459,460,461,462,463,
464,465,466,467,468,469,470,471,
472,473,474,475,476,477,478,479,
481,482,483,484,485,486,487,488,
490,491,492,493,494,495,497,498,
501,502,503,504,505,506,507,508,
509,510,512,513,514,515,516,517,
518,520,521,522,524,525,526,527,
529,530,531,532,533,534,535,536,
537,538,539,540,541,542,543,544,
546,547,548,549,550,551,552,553,
554,555,556,557,558,559,560,562,
564,565,567,568,569,570,572,573,
574,575,576,577,578,579,580,581,
582,583,584,585,586,587,588,590,
591,592,593,594,595,596,597,598,
600,601,602,603,604,605,607,608,
609,610,611,612,613,614,615,616,
617,618,619,620,621,622,623,624,
625,627,628,629,630,631,632,634,
635,636,637,638,639,640,641,642,
643,644,645,646,647,648,650,651,
652,653,654,655,656,657,658,659,
660,661,662,663,664,665,666,667,
668,669,671,672,673,674,676,677,
678,680,681,682,683,684,685,686,
687,688,689,691,692,693,694,695,
696,697,698,699,700,701,702,703,
704,705,706,708,709,710,711,712,
713,714,715,716,717,718,719,720,
721,722,723,725,726,729,730,731,
732,733,734,735,737,738,739,740,
741,743,744,746,747,748,749,750,
751,752,753,754,756,757,758,759,
760,761,762,763,764,766,768,770,
771,772,773,774,775,776,778,779,
780,781,782,783,784,785,786,787,
788,789,790,791,792,793,794,796,
797,798,799,800,802,803,804,806,
807,808,809,810,811,812,813,814,
815,816,817,819,820,822,823,824,
825,826,827,828,829,830,831,832,
833,834,835,836,837,838,839,840,
841,842,843,844,845,847,850,851,
852,853,854,855,857,858,860,861,
862,863,864,866,868,869,870,872,
873,874,875,876,877,878,879,880,
881,882,884,885,886,887,888,889,
890,891,892,893,896,897,898,899,
900,901,902,903,904,905,906,907,
908,909,910,911,912,913,914,915,
917,918,919,920,921,922,923,924,
925,926,927,928,929,930,931,932,
933,934,935,936,937,938,939,940,
941,942,943,944,945,947,948,950,
951,952,953,954,955,956,957,958,
959,960,961,962,963,964,965,966,
967,968,969,972,973,976,977,978,
980,981,984,985,986,987,988,989,
990,991,993,994,995,996,997,998,
999,1000,1002,1003,1004,1005,1006,1007,
1008,1009,1010,1012,1013,1014,1016,1017,
1018,1019,1020,1021,1022,1023,1024,1026,
1027,1028,1030,1031,1032,1033,1034,1035,
1036,1037,1038,1039,1040,1041,1043,1044,
1045,1046,1047,1048,1049,1050,1051,1052,
1053,1054,1055,1057,1058,1059,1060,1061,
1062,1064,1065,1066,1067,1068,1069,1070,
1071,1072,1073,1074,1075,1076,1077,1078,
1079,1080,1081,1082,1083,1084,1085,1086,
1088,1089,1090,1091,1092,1093,1094,1095,
1096,1097,1098,1099,1100,1101,1102,1103,
1104,1106,1107,1108,1109,1110,1111,1112,
1113,1115,1116,1117,1118,1119,1120,1121,
1122,1123,1124,1125,1126,1127,1128,1129,
1130,1131,1132,1133,1134,1135,1136,1137,
1138,1139,1140,1141,1142,1143,1144,1145,
1146,1147,1148,1149,1150,1151,1152,1153,
1154,1155,1156,1157,1158,1159,1161,1162,
1163,1164,1166,1167,1168,1169,1170,1171,
1172,1173,1174,1175,1176,1177,1178,1179,
1180,1181,1182,1183,1184,1185,1186,1187,
1188,1189,1190,1191,1192,1193,1194,1195,
1196,1197,1198,1199,1200,1201,1202,1203,
1204,1205,1206,1207,1208,1209,1210,1211,
1212,1214,1215,1216,1217,1218,1219,1220,
1221,1222,1223,1224,1225,1226,1227,1228,
1229,1231,1233,1234,1235,1236,1237,1238,
1239,1240,1241,1242,1243,1244,1245,1246,
1247,1248,1249,1251,1253,1254,1255,1257,
1258,1259,1260,1261,1262,1263,1264,1265,
1266,1267,1268,1269,1270,1271,1273,1274,
1275,1276,1278,1280,1281,1282,1284,1285,
1286,1287,1288,1289,1290,1292,1293,1294,
1296,1297,1298,1299,1301,1302,1303,1304,
1305,1307,1309,1310,1311,1312,1313,1314,
1316,1317,1318,1319,1320,1321,1322,1323,
1324,1325,1326,1327,1329,1330,1331,1332,
1333,1334,1335,1336,1337,1338,1339,1340,
1341,1342,1343,1344,1345,1346,1347,1348,
1349,1350,1351,1352,1353,1355,1356,1358,
1359,1360,1362,1363,1364,1365,1366,1367,
1368,1369,1370,1372,1373,1374,1375,1376,
1377,1378,1379,1380,1381,1382,1383,1384,
1385,1386,1388,1389,1391,1392,1393,1394,
1395,1396,1397,1398,1399,1400,1401,1402,
1403,1404,1405,1407,1408,1409,1410,1411,
1412,1413,1414,1415,1416,1418,1419,1420,
1421,1422,1423,1424,1425,1426,1427,1429,
1430,1431,1432,1433,1434,1435,1437,1438,
1439,1440,1441,1442,1443,1444,1445,1446,
1448,1449,1450,1451,1452,1453,1454,1455,
1456,1457,1458,1459,1460,1461,1462,1463,
1464,1465,1466,1467,1469,1470,1471,1472,
1473,1474,1475,1476,1477,1478,1479,1480,
1481,1482,1484,1485,1486,1487,1488,1489,
1490,1491,1492,1493,1494,1495,1496,1497,
1498,1500,1501,1502,1503,1504,1505,1507,
1508,1509,1511,1512,1513,1514,1515,1516,
1517,1518,1519,1521,1522,1524,1525,1526,
1527,1529,1530,1531,1532,1533,1534,1535,
1536,1537,1538,1539,1540,1541,1542,1543,
1544,1545,1546,1548,1549,1550,1553,1554,
1556,1557,1558,1559,1561,1563,1565,1566,
1567,1568,1570,1571,1572,1573,1574,1575,
1576,1577,1578,1579,1580,1582,1583,1584,
1585,1586,1587,1588,1589,1590,1591,1592,
1593,1594,1595,1596,1597,1598,1599,1600,
1601,1602,1603,1606,1608,1609,1610,1611,
1613,1615,1616,1617,1618,1619,1620,1621,
1623,1624,1625,1626,1627,1629,1630,1631,
1632,1633,1634,1635,1636,1637,1638,1639,
1640,1641,1642,1643,1644,1645,1646,1648,
1649,1650,1651,1652,1653,1654,1655,1656,
1657,1658,1660,1661,1662,1663,1664,1666,
1668,1670,1672,1673,1674,1675,1676,1677,
1679,1680,1681,1685,1689,1693,1697,1699,
1700,1701,1702,1703,1704,1705,1706,1707,
1708,1709,1710,1711,1712,1713,1717,1718,
1719,1720,1721,1722,1723,1724,1725,1727,
1728,1729,1730,1731,1732,1733,1735,1736,
1737,1739,1740,1741,1742,1744,1745,1746,
1747,1748,1749,1750,1751,1752,1753,1755,
1756,1757,1758,1759,1760,1761,1762,1763,
1764,1765,1767,1768,1769,1770,1772,1773,
1774,1776,1777,1778,1780,1781,1782,1783,
1784,1785,1786,1787,1789,1790,1791,1793,
1794,1795,1796,1797,1798,1799,1800,1801,
1802,1803,1805,1806,1807,1809,1810,1812,
1813,1814,1815,1817,1818,1819,1821,1822,
1823,1824,1826,1828,1829,1830,1831,1833,
1836,1837,1838,1840,1841,1842,1845,1847,
1848,1849,1850,1851,1852,1853,1854,1855,
1856,1857,1858,1859,1860,1861,1862,1863,
1864,1866,1868,1869,1870,1871,1872,1873,
1875,1877,1880,1882,1883,1884,1885,1886,
1887,1888,1890,1892,1893,1894,1895,1896,
1897,1898,1899,1900,1902,1903,1904,1906,
1907,1909,1911,1913,1914,1917,1920,1921,
1922,1923,1924,1926,1927,1928,1929,1931,
1932,1934,1936,1937,1938,1939,1940,1941,
1942,1943,1944,1945,1948,1951,1952,1953,
1954,1955,1956,1957,1958,1959,1960,1962,
1963,1964,1965,1966,1967,1968,1969,1970,
1971,1972,1973,1974,1975,1976,1978,1979,
1980,1981,1982,1983,1984,1985,1988,1989,
1992,1994,1995,1996,1998,1999,2000,2002,
2003,2004,2006,2007,2008,2011,2013,2014,
2015,2016,2017,2018,2019,2020,2021,2022,
2023,2024,2025,2026,2028,2029,2030,2031,
2033,2034,2035,2036,2037,2038,2040,2041,
2042,2043,2044,2045,2046,2049,2050,2051,
2052,2053,2055,2056,2057,2058,2059,2061,
2062,2063,2064,2067,2068,2069,2070,2071,
2072,2073,2076,2077,2078,2080,2081,2082,
2084,2085,2086,2087,2088,2089,2090,2092,
2094,2095,2096,2097,2098,2100,2101,2102,
2103,2104,2105,2107,2109,2112,2113,2114,
2115,2116,2117,2118,2119,2120,2121,2122,
2123,2124,2125,2126,2127,2128,2130,2131,
2132,2133,2134,2135,2136,2137,2138,2139,
2140,2141,2143,2144,2146,2148,2149,2150,
2151,2152,2153,2155,2156,2157,2158,2161,
2162,2163,2164,2165,2166,2167,2168,2169,
2170,2171,2173,2174,2175,2176,2178,2179,
2180,2181,2183,2184,2185,2186,2190,2191,
2192,2194,2198,2199,2200,2201,2202,2203,
2204,2206,2208,2210,2211,2213,2214,2215,
2216,2217,2218,2219,2220,2221,2222,2223,
2224,2226,2227,2228,2230,2231,2232,2233,
2234,2235,2236,2237,2238,2241,2242,2243,
2244,2245,2246,2247,2248,2249,2250,2251,
2252,2253,2254,2255,2256,2257,2258,2259,
2260,2261,2262,2263,2265,2266,2267,2268,
2270,2273,2274,2275,2276,2278,2279,2280,
2281,2282,2283,2284,2288,2289,2291,2292,
2293,2295,2296,2297,2298,2299,2301,2302,
2303,2304,2305,2306,2307,2309,2310,2313,
2315,2318,2319,2320,2322,2323,2324,2325,
2326,2327,2328,2329,2330,2331,2332,2333,
2335,2336,2337,2338,2341,2343,2344,2345,
2346,2347,2348,2350,2351,2353,2354,2355,
2356,2357,2358,2360,2361,2362,2363,2364,
2366,2367,2369,2370,2373,2374,2375,2376,
2377,2378,2379,2380,2381,2382,2383,2384,
2386,2388,2389,2390,2391,2392,2393,2394,
2396,2397,2398,2399,2400,2401,2402,2403,
2404,2405,2406,2408,2410,2411,2412,2413,
2414,2415,2416,2417,2418,2420,2422,2423,
2425,2426,2427,2428,2429,2430,2431,2432,
2433,2435,2436,2437,2438,2439,2440,2441,
2442,2444,2445,2447,2448,2449,2450,2452,
2453,2454,2455,2456,2457,2458,2459,2460,
2461,2462,2463,2464,2465,2466,2467,2468,
2469,2470,2471,2473,2474,2475,2476,2477,
2478,2479,2480,2481,2482,2485,2487,2489,
2491,2492,2493,2494,2496,2497,2498,2499,
2500,2503,2504,2505,2507,2508,2509,2510,
2511,2512,2513,2514,2516,2517,2518,2519,
2520,2521,2522,2523,2524,2525,2526,2527,
2528,2529,2530,2531,2532,2533,2534,2536,
2537,2538,2539,2540,2541,2542,2545,2546,
2547,2548,2550,2551,2552,2553,2554,2555,
2557,2558,2559,2560,2561,2562,2563,2564,
2565,2566,2567,2568,2569,2571,2572,2573,
2575,2576,2577,2579,2581,2584,2585,2586,
2587,2588,2589,2590,2592,2593,2594,2595,
2598,2599,2600,2601,2602,2605,2606,2607,
2608,2609,2611,2612,2613,2614,2615,2616,
2617,2618,2619,2620,2621,2622,2623,2624,
2625,2626,2627,2628,2629,2630,2631,2632,
2633,2634,2635,2636,2637,2639,2640,2641,
2642,2643,2644,2645,2647,2648,2649,2650,
2651,2652,2653,2654,2657,2658,2659,2660,
2661,2664,2665,2666,2667,2668,2669,2670,
2672,2673,2674,2675,2676,2677,2679,2680,
2681,2682,2683,2684,2685,2687,2688,2689,
2690,2691,2692,2693,2694,2695,2696,2697,
2698,2699,2700,2702,2703,2704,2705,2706,
2707,2708,2709,2710,2711,2712,2713,2714,
2715,2716,2717,2718,2719,2720,2721,2722,
2723,2725,2726,2727,2728,2730,2731,2732,
2734,2735,2736,2737,2738,2739,2740,2741,
2742,2743,2744,2745,2746,2748,2749,2750,
2751,2752,2753,2754,2755,2756,2757,2758,
2759,2760,2762,2763,2765,2766,2767,2768,
2769,2771,2773,2774,2775,2776,2777,2778,
2779,2780,2781,2782,2784,2785,2786,2787,
2788,2789,2790,2791,2792,2793,2794,2796,
2797,2799,2801,2802,2804,2805,2806,2807,
2810,2811,2812,2813,2814,2816,2817,2818,
2819,2820,2821,2822,2823,2824,2825,2826,
2827,2828,2829,2831,2832,2833,2834,2835,
2836,2837,2838,2839,2840,2841,2842,2843,
2844,2845,2846,2847,2848,2849,2850,2851,
2852,2853,2854,2855,2857,2858,2859,2860,
2861,2862,2863,2864,2865,2866,2867,2868,
2869,2870,2871,2872,2873,2875,2876,2877,
2878,2879,2880,2881,2882,2883,2884,2885,
2886,2887,2888,2889,2890,2891,2892,2893,
2894,2895,2896,2897,2898,2899,2900,2901,
2902,2903,2904,2905,2906,2907,2908,2909,
2910,2911,2912,2913,2914,2915,2916,2917,
2918,2920,2921,2923,2924,2925,2926,2927,
2928,2929,2930,2931,2932,2934,2936,2938,
2939,2940,2942,2943,2945,2946,2947,2948,
2950,2952,2953,2954,2955,2957,2958,2959,
2960,2961,2962,2963,2964,2965,2966,2967,
2968,2969,2970,2971,2973,2974,2976,2977,
2978,2979,2981,2982,2984,2985,2986,2987,
2988,2989,2991,2992,2994,2995,2996,2998,
2999,3000,3001,3002,3003,3004,3005,3006,
3007,3008,3009,3010,3011,3012,3013,3014,
3015,3016,3017,3018,3020,3021,3022,3024,
3025,3026,3029,3030,3031,3032,3034,3035,
3036,3037,3038,3039,3040,3041,3042,3043,
3044,3045,3048,3049,3050,3051,3054,3055,
3056,3059,3060,3061,3062,3063,3064,3065,
3066,3067,3068,3069,3070,3071,3072,3073,
3074,3075,3076,3077,3078,3079,3080,3081,
3082,3083,3084,3085,3086,3087,3088,3089,
3090,3091,3092,3093,3094,3095,3096,3097,
3098,3099,3100,3101,3102,3103,3104,3105,
3106,3107,3108,3109,3110,3111,3112,3114,
3115,3116,3117,3118,3119,3120,3121,3122,
3123,3124,3125,3126,3128,3129,3130,3131,
3132,3133,3134,3135,3136,3137,3138,3139,
3140,3142,3145,3146,3147,3149,3150,3151,
3152,3153,3154,3155,3156,3157,3158,3160,
3161,3162,3163,3164,3165,3167,3168,3169,
3170,3171,3172,3173,3174,3175,3176,3177,
3178,3179,3180,3181,3182,3183,3184,3185,
3186,3187,3188,3189,3190,3192,3194,3195,
3197,3198,3199,3200,3202,3203,3204,3206,
3207,3208,3209,3210,3211,3212,3213,3214,
3215,3216,3217,3218,3219,3220,3222,3223,
3225,3226,3227,3228,3229,3230,3231,3234,
3235,3236,3237,3238,3239,3240,3241,3242,
3244,3245,3246,3247,3248,3249,3250,3251,
3252,3253,3254,3255,3256,3257,3258,3259,
3260,3261,3262,3263,3264,3265,3266,3267,
3268,3269,3270,3271,3272,3273,3274,3275,
3276,3277,3278,3279,3281,3282,3283,3284,
3285,3286,3287,3288,3289,3290,3291,3292,
3293,3294,3295,3297,3299,3301,3303,3305,
3306,3307,3308,3309,3310,3311,3312,3313,
3314,3316,3317,3318,3319,3320,3321,3322,
3324,3326,3327,3328,3329,3330,3331,3332,
3333,3334,3335,3336,3337,3338,3339,3340,
3343,3344,3345,3346,3347,3348,3349,3350,
3352,3353,3354,3356,3357,3358,3360,3361,
3362,3363,3366,3367,3368,3369,3370,3373,
3374,3375,3376,3377,3379,3380,3381,3382,
3384,3385,3386,3387,3388,3389,3391,3392,
3393,3394,3397,3398,3401,3402,3403,3404,
3405,3407,3408,3409,3410,3411,3412,3414,
3415,3416,3417,3418,3419,3422,3424,3425,
3427,3428,3429,3430,3431,3432,3433,3434,
3435,3436,3437,3439,3441,3442,3444,3445,
3446,3447,3449,3450,3451,3452,3453,3454,
3455,3456,3457,3458,3460,3461,3462,3463,
3464,3465,3466,3467,3469,3470,3471,3473,
3474,3475,3476,3477,3478,3480,3481,3482,
3483,3484,3485,3488,3489,3491,3492,3493,
3494,3496,3497,3498,3499,3500,3501,3502,
3504,3505,3506,3507,3508,3509,3510,3511,
3512,3513,3514,3515,3517,3518,3519,3522,
3524,3525,3526,3527,3528,3529,3530,3531,
3532,3533,3534,3535,3536,3537,3538,3539,
3540,3541,3542,3543,3544,3545,3546,3547,
3548,3549,3550,3551,3552,3553,3554,3555,
3556,3557,3558,3559,3560,3561,3562,3563,
3564,3565,3567,3568,3570,3572,3574,3575,
3576,3577,3578,3579,3580,3581,3582,3583,
3584,3586,3588,3589,3590,3591,3592,3594,
3595,3596,3597,3599,3600,3601,3602,3603,
3604,3605,3606,3607,3608,3609,3610,3611,
3612,3613,3614,3615,3616,3617,3619,3620,
3621,3622,3623,3625,3626,3627,3629,3630,
3631,3632,3633,3634,3635,3637,3639,3641,
3642,3643,3644,3645,3646,3647,3648,3649,
3650,3651,3652,3654,3655,3656,3658,3659,
3660,3661,3662,3663,3664,3665,3666,3667,
3669,3670,3671,3672,3673,3674,3675,3677,
3678,3679,3680,3681,3682,3685,3686,3687,
3688,3689,3690,3691,3692,3693,3694,3695,
3697,3698,3699,3700,3701,3702,3703,3704,
3705,3707,3708,3709,3710,3712,3713,3714,
3715,3717,3718,3719,3720,3722,3724,3725,
3726,3727,3728,3729,3731,3732,3733,3734,
3735,3736,3737,3738,3739,3740,3741,3742,
3743,3745,3746,3747,3748,3750,3751,3753,
3754,3755,3756,3758,3759,3760,3761,3763,
3764,3765,3766,3767,3768,3769,3770,3771,
3772,3774,3775,3776,3777,3778,3779,3780,
3781,3782,3783,3784,3785,3786,3787,3788,
3789,3791,3792,3793,3794,3795,3796,3797,
3798,3799,3800,3801,3802,3803,3804,3806,
3807,3809,3810,3811,3812,3813,3814,3815,
3816,3817,3818,3819,3820,3821,3822,3823,
3824,3826,3827,3828,3829,3830,3832,3833,
3834,3835,3836,3837,3838,3839,3840,3841,
3842,3843,3844,3845,3846,3847,3848,3849,
3850,3851,3852,3853,3854,3855,3856,3857,
3858,3859,3860,3861,3862,3863,3864,3865,
3866,3868,3869,3870,3871,3872,3873,3874,
3875,3876,3877,3878,3879,3880,3881,3882,
3883,3884,3885,3886,3887,3888,3889,3890,
3891,3892,3893,3894,3895,3896,3897,3898,
3899,3900,3901,3902,3903,3904,3905,3906,
3907,3910,3911,3912,3913,3914,3915,3916,
3917,3918,3919,3920,3923,3924,3925,3926,
3927,3928,3929,3930,3931,3932,3933,3934,
3935,3936,3937,3938,3939,3940,3941,3942,
3943,3944,3945,3946,3947,3948,3949,3950,
3951,3952,3953,3954,3955,3956,3957,3958,
3959,3960,3961,3962,3963,3964,3966,3967,
3968,3970,3972,3973,3974,3975,3976,3978,
3979,3980,3981,3982,3983,3984,3985,3987,
3988,3989,3991,3992,3993,3994,3995,3996,
3997,3998,3999,4000,4001,4002,4003,4004,
4006,4007,4008,4009,4011,4012,4013,4014,
4015,4016,4018,4019,4020,4021,4022,4023,
4024,4025,4026,4027,4028,4029,4030,4031,
4032,4033,4035,4036,4037,4038,4039,4040,
4041,4042,4043,4045,4046,4047,4048,4049,
4050,4051,4052,4053,4055,4056,4057,4058,
4059,4060,4061,4062,4064,4065,4066,4067,
4068,4069,4070,4071,4072,4073,4074,4075,
4076,4077,4078,4079,4080,4081,4082,4083,
4084,4085,4086,4088,4089,4090,4091,4093,
4094,4095,4096,4097,4098,4099,4100,4101,
4103,4104,4105,4106,4107,4108,4109,4110,
4111,4112,4113,4114,4115,4116,4117,4118,
4119,4120,4121,4122,4123,4124,4125,4127,
4128,4129,4130,4131,4132,4133,4134,4135,
4136,4138,4139,4140,4141,4142,4143,4144,
4145,4146,4147,4148,4149,4150,4151,4152,
4153,4154,4155,4156,4157,4158,4159,4160,
4161,4162,4163,4164,4165,4166,4168,4169,
4170,4171,4172,4173,4174,4177,4179,4180,
4181,4182,4184,4185,4186,4188,4189,4190,
4191,4193,4194,4195,4196,4197,4198,4199,
4200,4201,4202,4203,4204,4205,4206,4207,
4208,4209,4210,4211,4212,4213,4214,4215,
4216,4220,4221,4222,4223,4224,4225,4226,
4227,4228,4229,4230,4231,4232,4233,4234,
4235,4236,4237,4239,4240,4241,4242,4244,
4246,4247,4248,4249,4250,4251,4252,4253,
4254,4255,4256,4257,4258,4259,4260,4261,
4263,4264,4265,4266,4267,4268,4269,4270,
4271,4272,4273,4274,4276,4277,4278,4280,
4282,4283,4284,4285,4286,4287,4288,4289,
4290,4291,4292,4293,4294,4296,4297,4299,
4301,4303,4304,4305,4306,4307,4308,4309,
4310,4312,4313,4314,4315,4316,4318,4319,
4320,4321,4322,4324,4325,4326,4327,4328,
4329,4330,4331,4332,4333,4334,4335,4336,
4337,4338,4339,4340,4341,4342,4343,4344,
4345,4346,4347,4348,4349,4350,4351,4352,
4353,4354,4355,4356,4357,4358,4359,4360,
4361,4363,4364,4365,4366,4367,4368,4369,
4370,4371,4372,4373,4374,4375,4376,4377,
4378,4379,4380,4381,4382,4383,4384,4385,
4386,4387,4388,4389,4390,4391,4392,4393,
4395,4396,4397,4399,4400,4401,4402,4403,
4404,4405,4406,4408,4409,4410,4411,4412,
4413,4414,4416,4417,4418,4419,4420,4421,
4422,4423,4424,4425,4426,4428,4429,4430,
4431,4432,4433,4434,4436,4437,4438,4439,
4440,4441,4442,4443,4444,4445,4446,4447,
4448,4449,4450,4451,4452,4453,4454,4455,
4456,4457,4458,4459,4460,4461,4462,4463,
4464,4465,4466,4467,4468,4469,4470,4471,
4472,4473,4474,4476,4477,4478,4479,4480,
4481,4482,4483,4485,4486,4487,4488,4489,
4490,4491,4492,4493,4494,4495,4497,4498,
4499,4500,4501,4502,4503,4504,4506,4507,
4508,4509,4510,4511,4512,4514,4516,4517,
4518,4519,4520,4521,4523,4525,4526,4527,
4528,4529,4531,4532,4533,4534,4535,4536,
4537,4538,4539,4540,4541,4543,4545,4546,
4547,4548,4550,4551,4552,4553,4554,4555,
4556,4557,4558,4559,4560,4561,4562,4563,
4564,4565,4566,4567,4569,4571,4572,4574,
4575,4576,4577,4579,4580,4581,4582,4583,
4584,4586,4587,4588,4589,4590,4591,4592,
4593,4594,4595,4596,4597,4598,4600,4601,
4602,4603,4604,4605,4607,4609,4610,4611,
4612,4613,4614,4615,4616,4617,4618,4619,
4621,4622,4623,4624,4625,4626,4627,4628,
4629,4630,4631,4632,4633,4634,4635,4636,
4637,4639,4641,4642,4643,4644,4645,4647,
4648,4649,4650,4651,4652,4653,4654,4655,
4656,4658,4660,4663,4664,4665,4666,4667,
4668,4669,4670,4671,4672,4673,4674,4675,
4676,4677,4679,4682,4683,4684,4685,4687,
4688,4689,4690,4691,4692,4693,4694,4695,
4696,4697,4698,4699,4700,4701,4702,4703,
4704,4705,4706,4707,4708,4709,4710,4711,
4712,4714,4715,4716,4717,4718,4719,4720,
4721,4722,4723,4724,4725,4726,4727,4730,
4731,4732,4733,4734,4735,4736,4737,4738,
4740,4741,4742,4743,4744,4745,4746,4747,
4748,4749,4750,4751,4752,4753,4754,4755,
4756,4757,4758,4759,4760,4761,4762,4763,
4764,4765,4767,4768,4770,4771,4772,4773,
4774,4775,4776,4777,4778,4779,4780,4781,
4782,4783,4784,4785,4786,4787,4788,4789,
4790,4791,4792,4793,4794,4795,4796,4797,
4798,4799,4801,4802,4803,4805,4806,4807,
4808,4809,4811,4813,4814,4815,4816,4817,
4818,4819,4820,4821,4822,4823,4824,4825,
4826,4827,4828,4829,4830,4831,4832,4833,
4834,4835,4836,4837,4840,4841,4842,4843,
4844,4845,4846,4847,4848,4849,4850,4851,
4852,4853,4854,4856,4857,4858,4859,4860,
4861,4862,4863,4864,4865,4867,4868,4869,
4870,4871,4872,4873,4874,4875,4876,4877,
4878,4879,4880,4881,4882,4883,4884,4885,
4887,4888,4889,4890,4891,4892,4895,4896,
4897,4898,4900,4901,4902,4903,4905,4906,
4907,4908,4909,4910,4911,4912,4914,4915,
4916,4917,4918,4919,4920,4921,4922,4923,
4924,4925,4926,4927,4928,4929,4930,4932,
4933,4934,4935,4936,4937,4940,4941,4942,
4943,4945,4946,4947,4948,4950,4951,4952,
4953,4954,4955,4956,4957,4958,4959,4960,
4961,4962,4963,4964,4965,4966,4967,4970,
4971,4973,4974,4975,4976,4977,4978,4979,
4980,4981,4983,4984,4985,4986,4987,4988,
4989,4990,4991,4992,4993,4994,4995,4996,
4997,4998,4999,5000,5001,5002,5003,5004,
5006,5007,5008,5009,5010,5011,5012,5013,
5014,5015,5016,5017,5018,5019,5020,5021,
5022,5023,5025,5026,5027,5028,5029,5030,
5031,5032,5033,5034,5035,5037,5038,5039,
5040,5041,5042,5043,5044,5045,5046,5047,
5048,5049,5050,5051,5052,5053,5054,5055,
5056,5057,5058,5059,5060,5061,5062,5063,
5066,5067,5069,5070,5071,5072,5073,5074,
5075,5076,5077,5078,5079,5080,5081,5083,
5084,5085,5086,5088,5089,5090,5091,5092,
5094,5095,5097,5099,5100,5101,5102,5104,
5105,5106,5107,5108,5109,5110,5111,5112,
5113,5115,5116,5118,5119,5120,5121,5122,
5123,5125,5126,5127,5128,5129,5130,5131,
5132,5133,5134,5135,5136,5137,5138,5139,
5140,5141,5143,5144,5145,5146,5147,5148,
5149,5150,5151,5152,5153,5154,5156,5157,
5159,5160,5161,5162,5163,5165,5167,5168,
5170,5171,5172,5173,5174,5175,5176,5177,
5178,5179,5180,5181,5182,5183,5184,5185,
5186,5187,5188,5189,5190,5191,5194,5196,
5197,5198,5199,5200,5201,5202,5203,5204,
5207,5208,5209,5210,5211,5212,5213,5215,
5216,5217,5218,5219,5220,5221,5223,5225,
5226,5227,5228,5229,5230,5231,5232,5233,
5234,5235,5236,5238,5239,5241,5243,5244,
5245,5247,5248,5250,5251,5252,5253,5254,
5255,5256,5257,5259,5260,5261,5262,5263,
5264,5265,5266,5268,5269,5270,5271,5272,
5273,5274,5275,5277,5279,5281,5283,5284,
5285,5286,5287,5289,5290,5291,5292,5294,
5296,5297,5298,5299,5300,5301,5302,5303,
5304,5305,5306,5308,5309,5311,5312,5313,
5315,5316,5317,5318,5321,5323,5324,5325,
5326,5327,5329,5330,5331,5333,5334,5336,
5337,5338,5339,5340,5341,5342,5343,5344,
5345,5346,5347,5348,5350,5351,5353,5354,
5355,5357,5358,5359,5361,5362,5363,5364,
5366,5368,5369,5372,5373,5374,5376,5378,
5379,5380,5381,5383,5384,5385,5386,5387,
5388,5389,5390,5391,5392,5393,5394,5396,
5397,5398,5399,5400,5401,5403,5404,5406,
5407,5409,5410,5411,5412,5413,5414,5415,
5416,5417,5419,5420,5421,5422,5423,5424,
5425,5426,5427,5428,5429,5431,5432,5433,
5434,5435,5436,5437,5440,5442,5443,5444,
5446,5447,5448,5449,5450,5451,5452,5453,
5454,5455,5456,5457,5458,5459,5460,5461,
5462,5463,5464,5465,5466,5467,5468,5470,
5471,5472,5473,5474,5475,5476,5477,5479,
5480,5481,5482,5483,5484,5485,5486,5487,
5488,5489,5491,5493,5494,5495,5496,5498,
5499,5501,5502,5503,5504,5505,5506,5508,
5509,5510,5511,5512,5513,5514,5515,5516,
5518,5519,5520,5521,5522,5523,5525,5526,
5527,5528,5529,5530,5531,5532,5534,5535,
5536,5537,5538,5539,5540,5541,5542,5543,
5544,5545,5546,5547,5548,5549,5550,5551,
5552,5553,5554,5555,5556,5557,5558,5559,
5560,5561,5562,5563,5564,5565,5566,5567,
5568,5569,5570,5571,5572,5574,5575,5576,
5578,5580,5581,5582,5584,5586,5587,5588,
5589,5590,5592,5593,5595,5596,5597,5598,
5599,5600,5602,5604,5605,5606,5607,5608,
5609,5610,5611,5612,5613,5614,5615,5617,
5618,5619,5620,5621,5622,5623,5624,5625,
5626,5627,5628,5629,5630,5631,5632,5633,
5635,5636,5637,5638,5639,5640,5641,5642,
5643,5644,5646,5647,5648,5649,5650,5651,
5652,5653,5654,5656,5660,5661,5662,5665,
5666,5668,5669,5670,5671,5672,5673,5674,
5675,5676,5678,5679,5680,5681,5682,5683,
5684,5685,5687,5688,5689,5691,5692,5693,
5694,5695,5696,5697,5698,5699,5700,5701,
5702,5703,5704,5705,5706,5707,5708,5710,
5711,5713,5714,5715,5716,5717,5719,5720,
5721,5722,5724,5726,5727,5728,5729,5730,
5731,5733,5734,5736,5737,5738,5739,5740,
5741,5742,5743,5744,5745,5746,5747,5748,
5749,5750,5751,5752,5753,5754,5755,5756,
5757,5758,5759,5760,5761,5763,5764,5765,
5766,5767,5768,5769,5770,5771,5772,5773,
5774,5776,5778,5780,5781,5782,5783,5784,
5785,5786,5787,5788,5789,5790,5791,5792,
5793,5794,5795,5796,5797,5798,5799,5800,
5801,5802,5803,5804,5805,5806,5809,5810,
5811,5812,5813,5815,5816,5817,5818,5821,
5822,5823,5824,5826,5827,5828,5829,5830,
5831,5832,5833,5834,5835,5836,5837,5838,
5839,5840,5841,5842,5843,5844,5845,5846,
5847,5848,5850,5851,5852,5853,5854,5855,
5857,5858,5860,5861,5862,5863,5864,5866,
5867,5868,5869,5870,5871,5872,5873,5874,
5875,5876,5877,5878,5879,5880,5881,5882,
5883,5884,5885,5886,5887,5888,5891,5892,
5893,5895,5896,5898,5899,5900,5901,5902,
5903,5904,5905,5906,5907,5908,5909,5910,
5911,5912,5913,5914,5915,5916,5917,5918,
5919,5920,5921,5922,5923,5924,5925,5926,
5927,5928,5929,5931,5932,5933,5935,5937,
5939,5940,5941,5942,5943,5944,5945,5946,
5948,5949,5950,5951,5952,5953,5954,5955,
5956,5957,5958,5959,5960,5961,5962,5963,
5964,5965,5966,5967,5968,5969,5970,5971,
5972,5973,5974,5975,5976,5977,5978,5979,
5981,5982,5984,5986,5987,5988,5989,5990,
5991,5992,5993,5994,5995,5996,5997,5998,
5999,6000,6002,6003,6004,6006,6007,6008,
6009,6010,6011,6012,6013,6014,6015,6017,
6018,6021,6022,6024,6026,6027,6029,6030,
6031,6032,6033,6037,6039,6041,6044,6045,
6046,6047,6050,6054,6055,6056,6057,6058,
6059,6060,6061,6062,6063,6064,6065,6066,
6067,6068,6071,6073,6074,6075,6076,6077,
6079,6081,6082,6083,6086,6087,6088,6089,
6090,6091,6092,6095,6096,6097,6098,6099,
6101,6102,6103,6104,6105,6107,6108,6109,
6110,6114,6115,6116,6117,6118,6119,6120,
6121,6123,6125,6126,6127,6128,6130,6131,
6132,6133,6134,6136,6137,6138,6139,6140,
6142,6143,6144,6146,6147,6148,6150,6151,
6152,6153,6154,6156,6157,6158,6159,6160,
6162,6163,6165,6166,6167,6168,6169,6170,
6171,6172,6174,6176,6177,6178,6179,6181,
6182,6183,6184,6185,6186,6188,6189,6190,
6191,6192,6193,6194,6197,6198,6199,6200,
6202,6204,6205,6208,6209,6210,6211,6212,
6213,6214,6215,6216,6217,6218,6219,6220,
6221,6222,6224,6225,6226,6228,6229,6231,
6232,6233,6234,6235,6236,6237,6239,6241,
6242,6245,6246,6249,6250,6251,6253,6255,
6256,6257,6258,6259,6260,6262,6263,6265,
6266,6267,6268,6269,6270,6271,6272,6274,
6275,6277,6278,6279,6280,6281,6283,6284,
6285,6286,6287,6288,6289,6290,6292,6293,
6295,6296,6297,6298,6299,6300,6301,6302,
6303,6304,6306,6307,6308,6309,6310,6311,
6312,6313,6315,6317,6319,6320,6321,6322,
6324,6325,6326,6327,6328,6329,6330,6332,
6333,6334,6336,6337,6338,6339,6340,6341,
6343,6346,6347,6349,6351,6352,6353,6354,
6355,6356,6357,6360,6362,6364,6365,6366,
6369,6370,6371,6372,6373,6374,6376,6377,
6378,6379,6380,6381,6382,6383,6385,6386,
6387,6388,6389,6390,6391,6393,6394,6395,
6396,6398,6399,6401,6402,6403,6404,6405,
6406,6407,6408,6409,6411,6414,6415,6417,
6418,6419,6420,6421,6422,6423,6425,6426,
6427,6428,6429,6430,6431,6432,6433,6434,
6435,6436,6437,6438,6439,6441,6442,6443,
6444,6445,6446,6447,6448,6451,6453,6455,
6457,6458,6459,6460,6461,6462,6463,6464,
6465,6466,6467,6468,6471,6472,6473,6474,
6475,6476,6477,6478,6479,6480,6481,6482,
6483,6485,6486,6490,6491,6492,6493,6494,
6495,6496,6498,6500,6501,6502,6503,6504,
6505,6506,6507,6509,6510,6513,6514,6515,
6516,6518,6519,6520,6521,6523,6524,6525,
6526,6528,6529,6530,6531,6532,6534,6535,
6537,6538,6539,6541,6544,6545,6547,6548,
6549,6550,6551,6552,6553,6554,6556,6557,
6558,6560,6561,6564,6565,6566,6568,6569,
6570,6571,6572,6573,6574,6575,6576,6577,
6578,6579,6580,6581,6582,6583,6584,6585,
6586,6587,6588,6590,6591,6592,6593,6594,
6595,6596,6598,6600,6601,6602,6604,6605,
6606,6607,6610,6613,6614,6615,6616,6617,
6618,6619,6620,6621,6622,6623,6624,6625,
6626,6627,6629,6630,6633,6634,6635,6636,
6637,6638,6640,6641,6642,6643,6644,6646,
6648,6649,6650,6651,6653,6654,6655,6656,
6657,6658,6659,6660,6661,6663,6664,6665,
6666,6667,6668,6669,6672,6674,6675,6680,
6681,6682,6683,6684,6686,6687,6690,6693,
6695,6696,6697,6698,6699,6700,6701,6702,
6703,6704,6706,6708,6709,6710,6711,6712,
6713,6715,6717,6719,6720,6721,6722,6724,
6725,6727,6728,6730,6731,6732,6735,6736,
6738,6739,6740,6741,6745,6747,6748,6749,
6750,6751,6752,6753,6754,6755,6756,6757,
6758,6759,6760,6761,6762,6763,6766,6767,
6768,6769,6770,6771,6772,6773,6774,6775,
6777,6778,6779,6780,6781,6783,6785,6786,
6787,6788,6789,6790,6792,6793,6794,6795,
6796,6798,6799,6800,6801,6802,6803,6805,
6806,6807,6809,6810,6811,6812,6813,6814,
6815,6816,6817,6818,6819,6820,6823,6825,
6826,6827,6828,6829,6830,6831,6832,6833,
6835,6836,6837,6838,6839,6840,6841,6842,
6844,6845,6846,6847,6848,6849,6850,6853,
6855,6857,6858,6859,6860,6861,6862,6863,
6864,6865,6866,6867,6868,6869,6870,6871,
6872,6873,6874,6875,6876,6877,6878,6879,
6880,6881,6882,6883,6885,6886,6887,6888,
6889,6891,6892,6893,6894,6895,6898,6899,
6902,6903,6904,6905,6907,6908,6909,6910,
6911,6913,6914,6916,6918,6919,6920,6922,
6923,6924,6925,6927,6928,6929,6931,6932,
6933,6934,6935,6936,6937,6938,6940,6941,
6942,6944,6945,6947,6948,6949,6950,6952,
6953,6954,6955,6956,6957,6958,6960,6961,
6962,6963,6965,6968,6969,6970,6971,6972,
6973,6974,6975,6976,6977,6979,6980,6981,
6982,6983,6984,6985,6986,6988,6989,6990,
6991,6992,6993,6995,6998,6999,7000,7001,
7002,7003,7004,7005,7007,7008,7009,7010,
7011,7012,7013,7015,7016,7017,7018,7019,
7020,7022,7023,7024,7025,7026,7027,7029,
7032,7033,7035,7037,7038,7039,7040,7041,
7042,7044,7046,7047,7049,7050,7051,7052,
7053,7055,7056,7057,7059,7060,7061,7062,
7063,7065,7067,7068,7069,7071,7072,7073,
7074,7075,7076,7077,7078,7079,7081,7082,
7083,7084,7085,7086,7087,7089,7090,7092,
7093,7094,7095,7096,7097,7098,7099,7100,
7101,7102,7104,7106,7107,7108,7109,7111,
7112,7113,7114,7115,7116,7117,7118,7119,
7120,7121,7122,7123,7124,7125,7126,7127,
7128,7129,7130,7132,7133,7135,7136,7137,
7139,7140,7142,7143,7144,7145,7146,7147,
7149,7150,7151,7152,7153,7154,7155,7156,
7157,7158,7159,7160,7161,7162,7163,7164,
7165,7166,7167,7168,7169,7170,7171,7172,
7173,7174,7175,7176,7177,7178,7179,7180,
7181,7182,7183,7184,7185,7186,7187,7188,
7189,7191,7192,7194,7197,7198,7199,7200,
7201,7202,7203,7204,7205,7206,7208,7209,
7210,7211,7212,7213,7214,7215,7216,7217,
7218,7219,7220,7221,7222,7223,7224,7225,
7226,7227,7228,7229,7230,7231,7232,7233,
7234,7235,7236,7237,7238,7239,7240,7241,
7242,7243,7244,7245,7246,7247,7248,7249,
7250,7251,7252,7253,7254,7255,7256,7257,
7258,7259,7260,7261,7263,7264,7265,7266,
7267,7268,7269,7270,7271,7272,7273,7274,
7275,7276,7277,7278,7280,7281,7282,7283,
7285,7286,7287,7288,7289,7290,7291,7292,
7293,7294,7295,7296,7297,7298,7299,7300,
7301,7302,7303,7304,7305,7306,7307,7308,
7310,7311,7312,7313,7314,7315,7316,7317,
7318,7319,7320,7321,7322,7323,7324,7325,
7327,7328,7329,7330,7331,7332,7334,7335,
7336,7337,7338,7339,7341,7342,7343,7344,
7345,7346,7347,7348,7349,7350,7351,7352,
7353,7354,7355,7356,7357,7358,7359,7360,
7362,7363,7364,7365,7366,7367,7369,7370,
7371,7372,7373,7374,7375,7376,7377,7378,
7379,7380,7381,7382,7383,7384,7385,7386,
7387,7388,7389,7390,7391,7392,7393,7394,
7395,7396,7397,7398,7399,7401,7402,7403,
7404,7405,7406,7407,7408,7409,7410,7411,
7412,7413,7414,7415,7416,7417,7418,7419,
7420,7421,7422,7423,7424,7425,7426,7427,
7428,7429,7430,7431,7432,7433,7434,7435,
7436,7437,7438,7439,7440,7441,7443,7444,
7445,7446,7447,7448,7449,7450,7451,7452,
7453,7454,7455,7456,7457,7458,7459,7460,
7461,7462,7463,7464,7465,7466,7467,7468,
7469,7470,7471,7472,7473,7474,7476,7478,
7479,7480,7483,7484,7486,7487,7489,7490,
7492,7493,7494,7495,7496,7498,7499,7500,
7502,7505,7506,7508,7509,7510,7511,7512,
7513,7515,7516,7519,7520,7521,7523,7524,
7526,7527,7528,7529,7531,7532,7533,7534,
7535,7536,7538,7539,7540,7541,7542,7544,
7546,7547,7549,7550,7551,7553,7554,7555,
7558,7559,7561,7562,7563,7564,7565,7566,
7567,7568,7569,7570,7571,7572,7574,7575,
7576,7577,7578,7579,7580,7581,7582,7583,
7584,7585,7586,7588,7590,7592,7594,7595,
7596,7597,7598,7600,7601,7602,7606,7607,
7608,7609,7610,7611,7612,7613,7614,7615,
7616,7617,7618,7619,7622,7624,7625,7627,
7628,7629,7630,7631,7632,7633,7634,7635,
7636,7638,7639,7640,7642,7643,7644,7645,
7646,7647,7648,7650,7651,7652,7653,7654,
7655,7657,7658,7659,7661,7662,7663,7664,
7665,7666,7668,7669,7670,7671,7672,7673,
7674,7675,7676,7677,7678,7679,7681,7683,
7684,7685,7686,7687,7688,7690,7691,7692,
7693,7694,7695,7696,7697,7698,7699,7701,
7702,7703,7705,7706,7707,7708,7709,7710,
7711,7713,7715,7716,7718,7719,7720,7721,
7722,7723,7725,7728,7729,7730,7732,7733,
7734,7736,7737,7738,7739,7740,7742,7743,
7745,7746,7747,7748,7749,7750,7751,7752,
7753,7754,7755,7756,7757,7759,7760,7762,
7763,7764,7765,7767,7769,7771,7773,7774,
7775,7777,7778,7779,7780,7781,7786,7789,
7790,7791,7792,7794,7795,7796,7797,7798,
7799,7801,7803,7804,7806,7807,7808,7809,
7810,7811,7812,7813,7814,7815,7819,7820,
7821,7822,7823,7824,7825,7826,7827,7829,
7830,7831,7832,7833,7834,7836,7837,7839,
7840,7841,7842,7843,7844,7845,7847,7848,
7850,7851,7853,7854,7855,7856,7857,7858,
7860,7861,7862,7863,7865,7866,7867,7868,
7870,7871,7872,7873,7874,7875,7876,7877,
7878,7880,7881,7883,7884,7885,7886,7887,
7888,7889,7891,7892,7893,7894,7895,7896,
7897,7898,7899,7900,7902,7903,7904,7905,
7906,7907,7908,7909,7910,7912,7913,7914,
7915,7916,7918,7920,7921,7922,7923,7924,
7925,7926,7928,7929,7930,7931,7932,7933,
7935,7936,7937,7938,7939,7940,7942,7943,
7945,7946,7947,7948,7950,7951,7952,7953,
7954,7955,7956,7957,7958,7959,7960,7961,
7962,7964,7965,7966,7967,7968,7969,7970,
7971,7972,7973,7974,7977,7979,7980,7982,
7983,7984,7985,7986,7987,7988,7990,7991,
7992,7993,7994,7995,7996,7997,7998,7999,
8000,8001,8002,8004,8005,8006,8007,8008,
8009,8010,8012,8013,8014,8015,8016,8017,
8018,8019,8020,8021,8022,8024,8025,8026,
8027,8028,8030,8031,8032,8035,8036,8037,
8038,8039,8040,8041,8042,8043,8044,8045,
8047,8048,8049,8050,8051,8052,8054,8055,
8057,8058,8061,8062,8065,8066,8067,8068,
8069,8070,8072,8073,8074,8075,8076,8077,
8078,8079,8080,8081,8084,8085,8086,8087,
8088,8089,8090,8091,8092,8094,8095,8096,
8098,8099,8101,8102,8103,8105,8106,8107,
8109,8110,8112,8113,8114,8115,8117,8118,
8119,8120,8121,8122,8123,8125,8126,8129,
8130,8131,8133,8135,8136,8138,8139,8141,
8142,8143,8144,8145,8146,8147,8148,8149,
8150,8151,8152,8153,8154,8155,8156,8157,
8158,8159,8160,8161,8162,8163,8164,8167,
8168,8169,8170,8171,8172,8173,8174,8175,
8177,8178,8179,8181,8182,8183,8184,8185,
8187,8188,8190,8191,8192,8193,8194,8195,
8196,8197,8198,8200,8201,8202,8203,8204,
8206,8207,8208,8209,8210,8211,8212,8213,
8214,8215,8216,8217,8218,8219,8220,8222,
8223,8224,8225,8226,8228,8229,8230,8231,
8232,8233,8234,8235,8236,8238,8240,8241,
8243,8244,8245,8247,8248,8249,8250,8251,
8253,8254,8256,8257,8258,8260,8261,8262,
8264,8265,8266,8267,8268,8269,8271,8272,
8273,8274,8275,8276,8277,8278,8280,8281,
8282,8283,8284,8285,8286,8288,8289,8290,
8291,8294,8296,8297,8298,8299,8300,8301,
8303,8304,8305,8306,8307,8308,8310,8311,
8312,8313,8314,8315,8316,8317,8318,8319,
8320,8321,8322,8323,8324,8325,8326,8327,
8328,8329,8330,8331,8332,8333,8334,8335,
8336,8337,8338,8339,8340,8341,8342,8343,
8344,8345,8346,8347,8348,8349,8350,8351,
8352,8353,8354,8355,8356,8357,8358,8359,
8362,8363,8364,8365,8366,8367,8368,8369,
8370,8371,8373,8374,8376,8377,8378,8379,
8381,8382,8383,8385,8387,8388,8389,8391,
8392,8393,8394,8395,8398,8399,8401,8402,
8404,8405,8407,8408,8409,8410,8411,8412,
8413,8414,8415,8416,8417,8418,8419,8421,
8422,8423,8424,8428,8429,8430,8432,8434,
8435,8436,8437,8438,8439,8440,8441,8442,
8443,8444,8445,8446,8447,8449,8450,8451,
8452,8453,8454,8455,8457,8458,8459,8460,
8461,8462,8464,8466,8467,8468,8469,8470,
8472,8473,8474,8475,8476,8477,8478,8479,
8480,8481,8482,8483,8484,8485,8486,8487,
8488,8489,8490,8491,8492,8493,8494,8495,
8496,8497,8498,8499,8501,8502,8503,8504,
8505,8507,8508,8509,8510,8511,8512,8513,
8515,8516,8517,8518,8519,8521,8522,8524,
8525,8526,8527,8529,8530,8531,8533,8534,
8535,8536,8537,8538,8539,8540,8541,8543,
8545,8547,8548,8550,8551,8552,8553,8554,
8555,8556,8557,8558,8559,8560,8561,8562,
8563,8564,8565,8566,8568,8569,8570,8572,
8573,8574,8575,8576,8577,8578,8579,8580,
8581,8582,8583,8584,8585,8586,8587,8588,
8589,8590,8591,8592,8593,8594,8595,8596,
8597,8599,8600,8601,8602,8604,8605,8606,
8607,8608,8611,8612,8613,8614,8615,8616,
8617,8618,8619,8620,8621,8622,8623,8626,
8627,8628,8629,8630,8632,8633,8634,8636,
8638,8639,8640,8641,8643,8644,8647,8648,
8649,8650,8651,8652,8653,8654,8655,8656,
8657,8658,8659,8660,8661,8662,8663,8666,
8667,8668,8669,8670,8671,8672,8673,8675,
8677,8678,8679,8680,8681,8682,8683,8684,
8686,8687,8689,8690,8691,8692,8693,8694,
8695,8696,8697,8698,8699,8700,8702,8703,
8704,8706,8707,8708,8709,8711,8712,8713,
8714,8715,8716,8717,8718,8719,8720,8721,
8722,8723,8724,8725,8727,8728,8729,8730,
8731,8733,8734,8735,8736,8737,8738,8739,
8740,8741,8742,8743,8744,8745,8746,8747,
8748,8749,8750,8751,8752,8753,8754,8755,
8756,8757,8758,8759,8760,8761,8763,8765,
8766,8767,8768,8771,8772,8774,8776,8777,
8779,8780,8781,8782,8783,8784,8785,8787,
8788,8790,8791,8792,8794,8796,8797,8798,
8800,8801,8803,8804,8805,8806,8808,8809,
8810,8811,8812,8813,8814,8815,8816,8817,
8819,8820,8821,8822,8823,8824,8825,8826,
8827,8828,8830,8831,8833,8834,8835,8836,
8837,8838,8839,8840,8841,8842,8843,8844,
8845,8846,8847,8848,8849,8850,8852,8853,
8854,8856,8857,8858,8859,8860,8861,8862,
8863,8864,8865,8866,8867,8868,8869,8871,
8872,8873,8874,8875,8876,8877,8878,8879,
8880,8881,8882,8883,8884,8885,8886,8887,
8888,8889,8891,8892,8893,8894,8896,8898,
8900,8901,8903,8904,8905,8906,8907,8908,
8909,8910,8911,8912,8913,8914,8915,8916,
8917,8919,8920,8921,8922,8923,8924,8925,
8926,8927,8928,8929,8930,8931,8932,8933,
8935,8938,8939,8940,8942,8944,8946,8947,
8948,8949,8950,8951,8952,8953,8954,8955,
8956,8957,8958,8959,8960,8962,8963,8964,
8965,8966,8967,8969,8970,8971,8972,8973,
8974,8975,8976,8977,8978,8979,8980,8981,
8982,8983,8984,8985,8986,8987,8988,8989,
8990,8991,8992,8993,8994,8995,8997,8998,
8999,9000,9001,9002,9003,9004,9005,9006,
9007,9009,9010,9011,9012,9013,9014,9015,
9016,9017,9018,9019,9020,9021,9022,9023,
9024,9025,9026,9027,9028,9029,9030,9031,
9033,9034,9035,9036,9038,9039,9040,9041,
9042,9043,9044,9045,9046,9048,9049,9050,
9051,9052,9053,9054,9055,9056,9057,9058,
9059,9060,9061,9062,9063,9064,9065,9066,
9067,9068,9069,9070,9071,9072,9073,9075,
9076,9077,9078,9079,9080,9081,9082,9083,
9085,9086,9087,9088,9090,9091,9092,9093,
9094,9095,9096,9097,9099,9100,9101,9102,
9104,9105,9106,9107,9108,9109,9110,9111,
9112,9113,9114,9115,9116,9117,9118,9119,
9120,9121,9122,9123,9124,9126,9128,9129,
9130,9131,9133,9134,9135,9136,9137,9138,
9139,9140,9141,9142,9143,9144,9147,9149,
9150,9152,9153,9154,9156,9158,9159,9160,
9161,9162,9163,9164,9165,9166,9167,9168,
9169,9171,9172,9173,9174,9175,9176,9177,
9178,9179,9181,9183,9185,9186,9188,9189,
9191,9192,9193,9194,9195,9196,9198,9199,
9200,9201,9203,9204,9205,9206,9207,9208,
9209,9210,9211,9212,9213,9214,9215,9216,
9217,9219,9220,9222,9223,9224,9226,9228,
9229,9230,9231,9232,9233,9234,9235,9236,
9237,9238,9239,9240,9241,9243,9244,9246,
9247,9248,9249,9251,9252,9254,9255,9256,
9258,9259,9261,9262,9263,9264,9265,9266,
9267,9268,9270,9271,9272,9273,9274,9275,
9276,9277,9278,9279,9280,9281,9282,9283,
9285,9286,9287,9288,9289,9291,9292,9293,
9294,9295,9296,9297,9298,9299,9300,9301,
9303,9304,9305,9307,9308,9309,9311,9312,
9313,9314,9315,9316,9317,9318,9319,9320,
9321,9322,9323,9324,9325,9326,9327,9328,
9329,9330,9331,9333,9336,9337,9338,9339,
9341,9342,9343,9344,9345,9346,9349,9350,
9351,9352,9353,9354,9355,9356,9357,9358,
9359,9360,9361,9362,9363,9365,9366,9367,
9369,9370,9371,9372,9373,9374,9375,9376,
9377,9378,9379,9380,9381,9382,9383,9384,
9385,9386,9387,9388,9389,9390,9392,9393,
9394,9395,9396,9397,9398,9399,9401,9402,
9403,9404,9405,9407,9408,9409,9410,9411,
9412,9413,9414,9416,9417,9418,9419,9420,
9422,9423,9424,9425,9426,9427,9428,9429,
9431,9432,9433,9434,9435,9436,9437,9438,
9439,9440,9441,9443,9444,9445,9446,9447,
9448,9449,9450,9451,9452,9453,9455,9456,
9457,9458,9459,9460,9461,9462,9463,9464,
9465,9466,9467,9468,9469,9471,9472,9473,
9474,9475,9476,9478,9479,9480,9481,9482,
9483,9484,9485,9486,9487,9488,9489,9490,
9491,9492,9494,9495,9496,9497,9498,9499,
9500,9501,9502,9503,9504,9505,9507,9508,
9509,9510,9511,9512,9514,9517,9518,9519,
9520,9521,9522,9523,9524,9525,9526,9527,
9528,9529,9531,9532,9533,9534,9535,9536,
9537,9538,9540,9541,9542,9544,9545,9546,
9547,9548,9549,9550,9551,9552,9553,9556,
9557,9558,9559,9560,9561,9563,9564,9565,
9566,9568,9569,9571,9572,9573,9574,9575,
9576,9578,9579,9580,9581,9582,9583,9584,
9585,9587,9588,9590,9592,9593,9594,9595,
9596,9597,9598,9599,9600,9601,9602,9603,
9604,9605,9606,9608,9611,9612,9613,9614,
9615,9616,9617,9618,9619,9620,9621,9623,
9624,9625,9627,9628,9629,9630,9631,9632,
9633,9634,9635,9636,9637,9638,9639,9641,
9642,9644,9645,9646,9647,9648,9649,9651,
9653,9654,9655,9656,9657,9658,9659,9660,
9662,9663,9664,9665,9666,9667,9668,9669,
9670,9671,9673,9674,9675,9676,9677,9678,
9679,9680,9681,9682,9683,9684,9685,9686,
9687,9688,9689,9690,9691,9692,9693,9694,
9697,9698,9699,9700,9701,9702,9703,9704,
9705,9706,9707,9708,9709,9710,9711,9712,
9713,9714,9715,9716,9718,9719,9720,9721,
9722,9723,9724,9726,9728,9730,9731,9733,
9735,9736,9737,9738,9739,9741,9742,9743,
9745,9746,9747,9748,9749,9750,9751,9752,
9753,9754,9755,9756,9758,9760,9761,9762,
9763,9764,9765,9766,9767,9768,9769,9771,
9772,9773,9774,9775,9778,9779,9780,9781,
9782,9783,9784,9786,9787,9788,9789,9790,
9791,9792,9793,9794,9795,9796,9797,9798,
9799,9800,9801,9802,9803,9804,9805,9806,
9807,9808,9810,9811,9812,9813,9814,9815,
9816,9817,9818,9820,9821,9822,9823,9824,
9825,9827,9828,9829,9830,9831,9832,9833,
9834,9836,9837,9840,9841,9842,9843,9845,
9846,9847,9848,9849,9850,9851,9852,9853,
9854,9855,9856,9857,9858,9860,9861,9862,
9863,9864,9865,9866,9867,9868,9869,9870,
9871,9872,9873,9874,9875,9876,9877,9878,
9879,9880,9881,9882,9883,9884,9885,9886,
9887,9889,9890,9891,9892,9893,9894,9895,
9896,9897,9898,9899,9900,9901,9902,9903,
9904,9905,9906,9907,9908,9909,9910,9912,
9913,9914,9915,9916,9917,9918,9919,9920,
9921,9922,9924,9925,9926,9927,9928,9929,
9930,9931,9932,9933,9934,9935,9936,9937,
9938,9940,9941,9942,9943,9944,9945,9946,
9948,9950,9951,9952,9953,9955,9956,9957,
9959,9960,9961,9962,9964,9965,9966,9967,
9968,9970,9971,9972,9973,9974,9975,9976,
9977,9979,9980,9981,9982,9983,9984,9986,
9987,9989,9990,9992,9993,9994,9995,9996,
9997,9998,9999,10000,10001,10002,10004,10005,
10006,10007,10008,10010,10011,10012,10013,10015,
10016,10018,10019,10020,10021,10023,10024,10025,
10026,10028,10029,10030,10032,10033,10035,10036,
10037,10038,10039,10040,10041,10042,10043,10044,
10045,10046,10047,10048,10049,10051,10052,10053,
10054,10055,10056,10057,10058,10059,10061,10062,
10063,10064,10065,10067,10068,10069,10070,10071,
10072,10073,10074,10076,10077,10078,10079,10080,
10081,10083,10084,10085,10086,10088,10089,10090,
10091,10092,10093,10094,10096,10097,10098,10100,
10101,10102,10103,10104,10105,10106,10107,10108,
10109,10110,10112,10113,10114,10115,10116,10117,
10118,10119,10120,10121,10122,10124,10125,10127,
10128,10130,10131,10133,10134,10135,10136,10138,
10139,10140,10141,10142,10143,10145,10146,10147,
10148,10149,10150,10151,10152,10153,10154,10156,
10158,10159,10160,10162,10163,10164,10165,10166,
10167,10168,10170,10171,10172,10174,10177,10178,
10180,10181,10182,10183,10184,10185,10186,10187,
10188,10189,10190,10191,10193,10195,10197,10198,
10199,10200,10201,10202,10203,10204,10205,10207,
10208,10209,10210,10211,10212,10213,10214,10216,
10217,10218,10220,10221,10223,10224,10225,10226,
10227,10228,10229,10230,10231,10232,10233,10234,
10235,10236,10237,10239,10240,10242,10243,10244,
10246,10247,10248,10249,10250,10251,10252,10253,
10254,10255,10256,10257,10259,10260,10261,10262,
10263,10264,10265,10266,10267,10269,10270,10271,
10272,10273,10274,10277,10278,10279,10280,10281,
10282,10283,10284,10285,10286,10287,10288,10289,
10290,10291,10292,10294,10295,10296,10297,10298,
10299,10300,10302,10303,10304,10305,10306,10307,
10308,10309,10310,10311,10312,10313,10315,10316,
10317,10318,10319,10320,10321,10323,10324,10325,
10326,10327,10328,10329,10330,10331,10332,10333,
10334,10335,10337,10338,10339,10340,10341,10342,
10343,10344,10345,10346,10348,10349,10351,10352,
10353,10354,10355,10356,10357,10358,10359,10360,
10361,10363,10364,10365,10366,10367,10368,10369,
10370,10371,10372,10373,10374,10375,10376,10377,
10378,10379,10380,10381,10382,10383,10384,10385,
10386,10387,10388,10390,10391,10392,10393,10395,
10396,10398,10399,10400,10401,10402,10403,10405,
10406,10407,10408,10409,10410,10411,10412,10413,
10414,10415,10416,10417,10418,10420,10421,10422,
10423,10424,10425,10426,10427,10428,10429,10430,
10431,10432,10433,10434,10435,10437,10438,10439,
10440,10441,10443,10444,10445,10446,10447,10448,
10449,10450,10451,10452,10453,10454,10455,10456,
10457,10458,10459,10460,10461,10462,10463,10464,
10465,10467,10468,10470,10471,10472,10473,10474,
10475,10476,10477,10478,10479,10480,10481,10482,
10483,10484,10485,10486,10487,10489,10490,10492,
10493,10494,10495,10496,10497,10499,10500,10502,
10504,10505,10506,10507,10508,10509,10510,10511,
10512,10513,10514,10515,10516,10517,10518,10519,
10520,10521,10522,10523,10525,10526,10528,10529,
10531,10532,10533,10534,10535,10537,10538,10539,
10541,10542,10543,10544,10545,10546,10547,10548,
10549,10551,10552,10553,10554,10555,10557,10558,
10559,10560,10561,10562,10563,10564,10565,10566,
10569,10570,10571,10572,10573,10574,10576,10577,
10578,10579,10580,10581,10582,10584,10586,10587,
10588,10589,10590,10591,10592,10593,10594,10595,
10596,10598,10599,10600,10601,10602,10603,10604,
10605,10606,10607,10608,10609,10610,10611,10612,
10613,10614,10615,10616,10617,10618,10619,10620,
10621,10622,10623,10624,10625,10626,10627,10629,
10630,10631,10632,10633,10634,10635,10636,10637,
10639,10640,10641,10642,10643,10644,10645,10646,
10647,10648,10649,10650,10651,10652,10653,10654,
10655,10656,10657,10658,10659,10661,10663,10664,
10665,10666,10667,10669,10671,10672,10673,10674,
10675,10676,10677,10678,10679,10680,10681,10682,
10684,10685,10686,10687,10688,10689,10690,10691,
10692,10693,10694,10695,10696,10697,10698,10699,
10700,10702,10703,10704,10705,10706,10707,10708,
10709,10710,10711,10712,10716,10717,10718,10719,
10720,10721,10723,10724,10725,10726,10727,10728,
10729,10730,10731,10732,10733,10734,10735,10736,
10737,10738,10739,10740,10741,10743,10744,10745,
10746,10747,10748,10749,10750,10751,10752,10753,
10754,10755,10756,10757,10758,10759,10760,10761,
10762,10763,10764,10765,10766,10767,10768,10769,
10770,10771,10772,10773,10775,10777,10778,10779,
10780,10781,10783,10784,10785,10787,10788,10789,
10790,10791,10792,10793,10795,10796,10797,10798,
10799,10800,10801,10802,10803,10804,10805,10806,
10807,10808,10809,10810,10811,10812,10813,10814,
10815,10816,10817,10819,10821,10822,10823,10824,
10826,10827,10828,10829,10830,10831,10832,10833,
10834,10835,10836,10837,10838,10839,10840,10841,
10842,10843,10844,10845,10846,10847,10848,10849,
10850,10851,10852,10853,10854,10855,10856,10857,
10858,10859,10861,10862,10863,10864,10865,10866,
10868,10869,10870,10871,10872,10873,10874,10876,
10878,10879,10880,10881,10882,10883,10884,10885,
10886,10887,10888,10889,10891,10892,10893,10894,
10895,10897,10898,10899,10900,10901,10902,10903,
10904,10905,10906,10907,10908,10909,10910,10911,
10912,10913,10914,10915,10916,10917,10918,10920,
10921,10922,10923,10924,10925,10926,10928,10929,
10930,10931,10932,10933,10934,10935,10936,10937,
10938,10939,10940,10941,10943,10944,10945,10946,
10947,10948,10949,10950,10951,10952,10953,10954,
10955,10956,10957,10958,10959,10960,10961,10963,
10964,10965,10966,10967,10969,10971,10972,10973,
10974,10975,10976,10978,10980,10981,10982,10983,
10985,10986,10987,10988,10989,10990,10991,10992,
10993,10994,10995,10996,10999,11001,11002,11003,
11005,11006,11007,11008,11009,11010,11011,11012,
11013,11014,11015,11016,11017,11018,11019,11020,
11021,11022,11023,11025,11026,11027,11028,11029,
11031,11032,11033,11035,11036,11037,11038,11039,
11040,11041,11042,11043,11044,11045,11047,11049,
11050,11051,11052,11053,11054,11055,11056,11057,
11058,11059,11060,11061,11062,11063,11064,11065,
11066,11068,11069,11070,11071,11072,11073,11074,
11076,11077,11078,11079,11080,11081,11082,11083,
11084,11085,11086,11088,11089,11090,11091,11092,
11093,11094,11095,11096,11097,11098,11099,11100,
11101,11102,11103,11104,11105,11106,11107,11108,
11109,11112,11113,11117,11118,11119,11121,11122,
11123,11124,11125,11126,11127,11129,11130,11131,
11132,11133,11134,11135,11136,11137,11138,11139,
11140,11141,11142,11143,11145,11146,11147,11148,
11149,11150,11151,11152,11153,11154,11155,11156,
11157,11158,11159,11160,11161,11162,11163,11164,
11166,11167,11169,11170,11171,11172,11173,11176,
11177,11178,11179,11180,11181,11182,11183,11184,
11185,11186,11188,11189,11191,11192,11193,11195,
11196,11197,11198,11199,11200,11201,11202,11203,
11205,11206,11207,11208,11209,11210,11211,11212,
11214,11215,11216,11217,11218,11219,11220,11221,
11222,11223,11224,11225,11226,11227,11228,11229,
11230,11231,11232,11234,11235,11236,11237,11238,
11240,11241,11242,11244,11245,11246,11247,11248,
11250,11251,11252,11254,11255,11256,11257,11258,
11259,11260,11262,11263,11264,11265,11267,11268,
11269,11270,11272,11273,11274,11275,11276,11277,
11278,11279,11280,11281,11282,11283,11284,11286,
11288,11289,11290,11291,11292,11293,11294,11295,
11296,11297,11298,11299,11300,11301,11302,11303,
11304,11305,11306,11307,11308,11309,11310,11311,
11312,11313,11314,11316,11317,11318,11319,11320,
11321,11322,11323,11324,11325,11326,11327,11328,
11329,11330,11331,11333,11334,11335,11336,11338,
11339,11340,11341,11342,11343,11344,11345,11346,
11347,11348,11350,11351,11352,11353,11354,11355,
11356,11357,11358,11359,11360,11361,11362,11363,
11364,11365,11366,11367,11368,11369,11371,11372,
11374,11375,11376,11377,11378,11379,11380,11381,
11383,11384,11385,11387,11389,11390,11391,11392,
11393,11394,11395,11396,11397,11398,11399,11400,
11401,11402,11403,11404,11405,11406,11407,11408,
11409,11410,11411,11412,11413,11414,11415,11416,
11418,11419,11420,11421,11422,11423,11424,11425,
11426,11427,11428,11429,11430,11431,11432,11434,
11435,11436,11437,11439,11440,11441,11442,11443,
11444,11445,11446,11447,11448,11449,11450,11451,
11452,11453,11454,11455,11456,11457,11458,11459,
11460,11461,11462,11463,11464,11465,11466,11467,
11468,11469,11471,11473,11474,11475,11476,11477,
11478,11480,11482,11483,11484,11485,11486,11487,
11488,11489,11490,11491,11492,11493,11495,11496,
11497,11498,11499,11500,11501,11502,11503,11504,
11505,11506,11507,11508,11509,11510,11511,11512,
11513,11514,11515,11517,11518,11519,11520,11521,
11522,11523,11524,11525,11526,11527,11528,11529,
11530,11531,11532,11533,11534,11535,11536,11537,
11539,11540,11541,11542,11543,11544,11545,11546,
11548,11550,11551,11552,11553,11554,11555,11557,
11558,11559,11560,11561,11562,11563,11564,11565,
11566,11567,11568,11569,11570,11571,11572,11573,
11574,11575,11576,11577,11578,11579,11581,11582,
11584,11585,11586,11587,11590,11591,11592,11593,
11595,11596,11598,11599,11600,11601,11602,11603,
11604,11606,11608,11609,11610,11611,11612,11613,
11614,11615,11616,11617,11618,11619,11620,11621,
11622,11623,11624,11625,11626,11627,11628,11629,
11630,11631,11632,11633,11634,11635,11636,11637,
11638,11639,11640,11641,11642,11643,11644,11645,
11646,11647,11648,11649,11650,11651,11652,11653,
11654,11655,11656,11657,11658,11659,11660,11661,
11662,11663,11664,11665,11666,11667,11668,11669,
11670,11671,11672,11673,11674,11675,11676,11677,
11678,11679,11680,11681,11682,11684,11685,11686,
11687,11690,11691,11693,11694,11696,11697,11699,
11700,11701,11703,11704,11705,11706,11707,11708,
11709,11710,11711,11712,11713,11714,11715,11716,
11717,11719,11720,11721,11722,11723,11724,11725,
11727,11728,11729,11730,11731,11733,11734,11735,
11737,11739,11740,11741,11742,11745,11746,11747,
11748,11750,11751,11753,11754,11756,11757,11758,
11759,11760,11761,11762,11763,11764,11765,11766,
11767,11768,11769,11770,11771,11772,11773,11774,
11775,11776,11777,11778,11779,11780,11781,11783,
11784,11785,11786,11787,11790,11792,11793,11794,
11795,11797,11799,11800,11801,11802,11803,11804,
11805,11806,11808,11809,11810,11811,11812,11813,
11815,11817,11818,11819,11820,11821,11822,11824,
11825,11827,11828,11829,11830,11831,11832,11833,
11834,11835,11836,11837,11838,11839,11840,11841,
11842,11843,11844,11846,11847,11848,11849,11850,
11851,11852,11854,11855,11856,11857,11858,11859,
11860,11861,11862,11863,11865,11866,11867,11868,
11871,11872,11874,11875,11876,11877,11878,11879,
11880,11881,11882,11883,11884,11885,11886,11887,
11888,11889,11890,11891,11892,11893,11894,11895,
11896,11897,11898,11899,11900,11901,11903,11904,
11905,11906,11907,11908,11910,11911,11912,11914,
11917,11918,11919,11920,11921,11922,11923,11925,
11926,11927,11928,11929,11930,11931,11932,11933,
11934,11936,11937,11938,11939,11940,11941,11942,
11943,11944,11945,11946,11947,11948,11950,11951,
11952,11953,11954,11955,11956,11957,11958,11959,
11960,11961,11962,11963,11965,11966,11968,11969,
11971,11973,11974,11975,11976,11977,11978,11979,
11980,11981,11982,11983,11984,11985,11986,11987,
11988,11989,11990,11991,11992,11993,11994,11995,
11996,11997,11998,11999,12000,12002,12003,12004,
12005,12006,12008,12009,12010,12011,12012,12013,
12014,12016,12017,12018,12019,12020,12021,12022,
12023,12025,12026,12027,12028,12029,12030,12031,
12032,12033,12034,12035,12036,12037,12038,12039,
12040,12041,12042,12043,12044,12045,12046,12047,
12048,12049,12050,12051,12052,12053,12054,12055,
12056,12057,12058,12059,12060,12061,12062,12063,
12064,12066,12067,12068,12069,12071,12073,12074,
12075,12076,12077,12078,12080,12081,12082,12083,
12084,12086,12087,12088,12089,12090,12091,12092,
12093,12094,12095,12096,12097,12098,12099,12100,
12101,12102,12103,12104,12105,12106,12107,12108,
12109,12110,12111,12114,12115,12116,12117,12118,
12119,12120,12121,12122,12123,12124,12125,12126,
12127,12128,12129,12130,12132,12133,12136,12137,
12138,12139,12140,12141,12142,12143,12144,12145,
12146,12147,12148,12149,12150,12151,12152,12153,
12154,12155,12156,12157,12158,12159,12160,12163,
12164,12165,12166,12167,12169,12170,12171,12172,
12173,12174,12175,12176,12177,12178,12179,12180,
12181,12182,12184,12185,12186,12187,12188,12189,
12190,12191,12192,12193,12195,12196,12197,12198,
12200,12201,12202,12203,12205,12206,12207,12208,
12209,12210,12212,12213,12215,12216,12217,12218,
12219,12220,12221,12222,12223,12224,12225,12226,
12228,12229,12231,12232,12233,12234,12235,12236,
12239,12240,12242,12243,12244,12246,12247,12248,
12249,12250,12251,12252,12254,12255,12256,12257,
12258,12259,12260,12262,12264,12265,12267,12268,
12269,12271,12272,12273,12274,12277,12278,12279,
12280,12281,12282,12283,12284,12285,12286,12287,
12288,12291,12292,12293,12294,12297,12298,12299,
12300,12301,12303,12304,12305,12306,12307,12308,
12309,12310,12311,12312,12313,12314,12315,12316,
12317,12318,12319,12320,12321,12322,12323,12324,
12326,12327,12328,12330,12331,12332,12334,12335,
12336,12337,12338,12341,12342,12343,12344,12345,
12346,12347,12348,12349,12350,12351,12352,12353,
12354,12355,12356,12357,12358,12359,12360,12361,
12362,12363,12364,12365,12366,12367,12368,12371,
12372,12373,12375,12376,12377,12378,12379,12380,
12382,12383,12384,12386,12387,12388,12389,12390,
12392,12393,12394,12395,12396,12398,12399,12401,
12402,12403,12404,12405,12406,12408,12409,12410,
12411,12412,12413,12414,12415,12416,12417,12418,
12420,12421,12422,12423,12424,12426,12427,12429,
12430,12431,12432,12433,12434,12435,12436,12437,
12439,12440,12441,12442,12444,12445,12446,12448,
12449,12450,12451,12452,12453,12455,12456,12457,
12458,12459,12460,12461,12462,12464,12465,12466,
12467,12468,12469,12470,12471,12474,12477,12480,
12481,12482,12483,12484,12485,12486,12487,12488,
12489,12490,12491,12492,12493,12494,12495,12496,
12497,12498,12499,12500,12502,12503,12504,12505,
12506,12507,12508,12509,12510,12513,12514,12515,
12516,12517,12518,12519,12520,12521,12522,12523,
12524,12525,12527,12528,12529,12531,12532,12533,
12536,12537,12538,12539,12540,12541,12542,12543,
12544,12545,12546,12547,12548,12549,12550,12551,
12552,12553,12554,12556,12557,12558,12559,12560,
12561,12562,12564,12565,12566,12568,12569,12570,
12571,12572,12573,12574,12575,12576,12577,12578,
12579,12580,12581,12582,12583,12584,12585,12586,
12587,12588,12589,12590,12591,12593,12594,12595,
12596,12597,12598,12600,12601,12603,12607,12608,
12609,12610,12612,12613,12614,12615,12617,12618,
12619,12620,12622,12623,12624,12625,12626,12629,
12630,12631,12632,12633,12634,12635,12637,12639,
12641,12642,12643,12644,12645,12646,12647,12648,
12649,12650,12651,12652,12653,12654,12655,12656,
12657,12658,12659,12660,12661,12662,12663,12666,
12667,12668,12669,12671,12672,12673,12674,12675,
12676,12677,12678,12680,12681,12683,12684,12685,
12687,12688,12689,12690,12691,12692,12693,12694,
12695,12696,12698,12699,12700,12701,12702,12703,
12704,12705,12706,12707,12709,12710,12711,12712,
12713,12714,12716,12717,12718,12719,12720,12722,
12723,12724,12725,12726,12727,12728,12729,12730,
12731,12732,12733,12734,12736,12737,12738,12739,
12741,12742,12743,12744,12746,12747,12749,12752,
12753,12755,12756,12757,12758,12759,12762,12763,
12765,12766,12769,12770,12771,12772,12773,12774,
12775,12776,12777,12778,12779,12780,12782,12783,
12784,12785,12786,12787,12788,12789,12790,12791,
12792,12793,12795,12796,12797,12798,12799,12800,
12801,12802,12804,12805,12806,12807,12808,12809,
12810,12811,12812,12813,12815,12817,12818,12819,
12820,12821,12822,12823,12824,12825,12826,12827,
12828,12829,12831,12833,12834,12835,12836,12837,
12838,12840,12841,12842,12843,12844,12845,12846,
12847,12849,12850,12851,12852,12853,12854,12855,
12856,12857,12858,12859,12860,12861,12863,12864,
12865,12866,12867,12868,12869,12870,12871,12872,
12873,12874,12875,12876,12877,12878,12879,12880,
12881,12882,12883,12884,12885,12886,12887,12888,
12889,12890,12891,12892,12893,12894,12895,12896,
12898,12899,12900,12902,12903,12904,12905,12906,
12907,12908,12909,12910,12911,12912,12914,12915,
12916,12917,12918,12919,12920,12921,12922,12923,
12924,12925,12926,12927,12928,12929,12930,12931,
12932,12933,12934,12935,12936,12937,12939,12940,
12941,12942,12944,12945,12946,12947,12948,12949,
12950,12951,12952,12953,12954,12955,12956,12957,
12958,12959,12960,12961,12962,12963,12965,12967,
12968,12969,12970,12971,12972,12973,12974,12975,
12976,12977,12978,12979,12981,12983,12984,12985,
12986,12987,12988,12989,12990,12991,12992,12993,
12994,12995,12996,12997,12998,12999,13000,13001,
13002,13003,13004,13006,13007,13008,13009,13010,
13011,13012,13013,13014,13015,13016,13017,13018,
13019,13020,13021,13023,13024,13025,13026,13027,
13028,13029,13030,13031,13032,13033,13035,13036,
13038,13039,13040,13041,13042,13043,13045,13046,
13048,13049,13050,13051,13052,13053,13054,13055,
13056,13058,13059,13060,13061,13062,13063,13064,
13065,13066,13067,13068,13069,13070,13071,13072,
13073,13074,13075,13076,13077,13078,13079,13081,
13082,13084,13085,13087,13089,13090,13091,13092,
13093,13094,13095,13096,13097,13098,13099,13100,
13102,13103,13104,13105,13106,13108,13109,13110,
13111,13112,13113,13114,13115,13117,13118,13119,
13120,13121,13122,13123,13124,13125,13126,13127,
13128,13130,13131,13132,13133,13135,13136,13137,
13138,13139,13142,13143,13144,13145,13146,13147,
13149,13150,13151,13152,13154,13155,13156,13157,
13158,13159,13161,13162,13163,13166,13168,13169,
13170,13171,13172,13173,13174,13175,13177,13178,
13179,13180,13181,13182,13183,13184,13185,13186,
13189,13191,13192,13193,13194,13195,13196,13197,
13198,13199,13200,13201,13202,13203,13204,13205,
13206,13208,13209,13210,13212,13214,13215,13216,
13217,13218,13219,13220,13221,13222,13223,13224,
13225,13226,13227,13229,13230,13231,13232,13234,
13235,13236,13238,13239,13241,13242,13243,13244,
13245,13246,13247,13248,13249,13250,13251,13252,
13253,13254,13255,13256,13257,13258,13259,13260,
13261,13262,13263,13264,13266,13269,13270,13271,
13272,13274,13275,13276,13277,13278,13280,13281,
13282,13283,13284,13285,13286,13287,13288,13289,
13290,13291,13292,13293,13294,13295,13296,13297,
13298,13300,13301,13302,13304,13305,13306,13307,
13308,13309,13310,13311,13312,13315,13316,13317,
13318,13319,13320,13321,13322,13324,13326,13328,
13329,13330,13331,13332,13333,13334,13335,13336,
13337,13338,13339,13340,13341,13342,13343,13344,
13345,13347,13348,13349,13350,13351,13352,13353,
13354,13356,13357,13358,13359,13360,13361,13362,
13363,13364,13365,13366,13367,13368,13369,13370,
13372,13373,13374,13375,13376,13377,13378,13379,
13380,13381,13382,13384,13385,13386,13387,13388,
13389,13390,13391,13393,13395,13396,13397,13398,
13399,13400,13401,13402,13403,13404,13405,13406,
13407,13408,13409,13410,13411,13412,13414,13415,
13416,13417,13418,13421,13423,13424,13425,13426,
13427,13428,13429,13432,13433,13434,13435,13436,
13437,13438,13439,13440,13441,13442,13443,13444,
13445,13446,13447,13448,13449,13450,13451,13453,
13454,13456,13457,13458,13459,13460,13461,13462,
13463,13464,13465,13466,13467,13468,13469,13470,
13471,13472,13473,13474,13475,13476,13477,13478,
13479,13480,13481,13482,13483,13484,13485,13486,
13487,13488,13489,13490,13491,13492,13493,13494,
13495,13496,13497,13498,13499,13500,13501,13502,
13503,13505,13506,13507,13508,13509,13510,13511,
13512,13513,13514,13515,13516,13517,13518,13519,
13520,13521,13522,13523,13524,13525,13526,13527,
13528,13529,13530,13531,13532,13533,13534,13535,
13536,13537,13541,13542,13543,13545,13546,13547,
13548,13550,13552,13553,13554,13556,13557,13558,
13559,13561,13562,13563,13564,13565,13566,13567,
13568,13570,13571,13572,13573,13574,13575,13576,
13577,13578,13579,13581,13582,13583,13584,13585,
13586,13587,13588,13589,13590,13591,13592,13593,
13594,13595,13596,13597,13598,13599,13600,13601,
13603,13604,13607,13608,13609,13610,13611,13614,
13615,13616,13617,13618,13619,13620,13621,13622,
13623,13624,13625,13626,13627,13628,13629,13630,
13631,13632,13633,13634,13635,13636,13637,13638,
13639,13640,13641,13642,13643,13644,13645,13646,
13647,13648,13649,13652,13654,13655,13656,13657,
13659,13660,13661,13662,13663,13665,13666,13667,
13668,13669,13670,13671,13672,13674,13675,13676,
13677,13680,13683,13684,13686,13687,13688,13689,
13691,13692,13693,13695,13696,13697,13698,13699,
13700,13701,13702,13703,13704,13705,13706,13708,
13709,13710,13711,13712,13713,13714,13715,13716,
13717,13718,13719,13721,13723,13724,13725,13726,
13727,13728,13729,13730,13731,13732,13733,13734,
13735,13737,13738,13739,13740,13741,13742,13743,
13744,13745,13746,13747,13748,13749,13750,13751,
13752,13753,13754,13755,13756,13757,13758,13759,
13761,13762,13763,13764,13765,13766,13767,13768,
13769,13771,13772,13773,13774,13775,13776,13777,
13778,13779,13780,13781,13784,13786,13787,13788,
13789,13790,13791,13792,13793,13794,13795,13796,
13797,13798,13799,13800,13801,13802,13803,13804,
13805,13806,13807,13808,13809,13810,13811,13812,
13813,13814,13815,13816,13817,13818,13819,13820,
13821,13822,13823,13824,13825,13826,13827,13828,
13829,13830,13831,13832,13833,13834,13835,13837,
13839,13840,13841,13842,13843,13844,13845,13846,
13847,13848,13849,13850,13851,13852,13853,13854,
13855,13856,13857,13858,13859,13860,13861,13862,
13863,13864,13865,13867,13869,13870,13872,13873,
13874,13875,13876,13877,13878,13879,13880,13881,
13882,13883,13884,13885,13886,13887,13888,13889,
13890,13891,13892,13893,13894,13895,13896,13897,
13898,13899,13901,13902,13903,13904,13905,13906,
13907,13908,13909,13910,13911,13912,13913,13914,
13915,13916,13917,13918,13920,13921,13922,13923,
13924,13925,13926,13927,13928,13930,13932,13933,
13934,13935,13936,13937,13939,13940,13941,13942,
13943,13944,13945,13946,13947,13948,13949,13950,
13951,13952,13953,13954,13955,13956,13957,13958,
13959,13961,13962,13963,13965,13966,13967,13968,
13969,13970,13971,13972,13973,13975,13977,13978,
13980,13981,13982,13983,13984,13985,13986,13987,
13988,13989,13990,13992,13993,13994,13996,13997,
13998,13999,14000,14001,14002,14003,14004,14005,
14006,14007,14008,14009,14010,14011,14013,14014,
14015,14016,14017,14018,14019,14020,14021,14023,
14024,14025,14026,14027,14028,14030,14031,14032,
14033,14034,14035,14036,14037,14038,14039,14041,
14042,14043,14044,14045,14046,14047,14048,14049,
14050,14051,14052,14053,14054,14055,14056,14057,
14058,14059,14060,14061,14063,14065,14066,14067,
14068,14069,14071,14072,14073,14075,14076,14077,
14079,14081,14083,14084,14087,14088,14089,14091,
14093,14094,14097,14098,14099,14100,14101,14102,
14104,14105,14106,14107,14108,14109,14110,14111,
14112,14113,14114,14116,14120,14121,14122,14123,
14124,14125,14126,14127,14128,14130,14132,14133,
14134,14135,14136,14137,14138,14139,14140,14141,
14142,14143,14144,14145,14146,14147,14148,14150,
14151,14152,14153,14154,14155,14157,14159,14160,
14162,14163,14164,14165,14166,14167,14168,14169,
14170,14172,14173,14176,14177,14178,14179,14181,
14182,14183,14184,14185,14186,14187,14188,14189,
14190,14192,14193,14194,14195,14196,14198,14199,
14200,14201,14202,14205,14206,14207,14208,14209,
14210,14211,14212,14213,14214,14215,14216,14217,
14218,14219,14221,14222,14223,14224,14225,14226,
14227,14228,14229,14230,14231,14232,14234,14235,
14237,14238,14239,14241,14242,14244,14245,14246,
14247,14248,14249,14250,14251,14252,14253,14254,
14255,14258,14259,14260,14261,14262,14263,14265,
14267,14268,14269,14271,14272,14274,14275,14276,
14277,14278,14279,14280,14281,14282,14283,14284,
14285,14286,14287,14288,14289,14290,14291,14293,
14294,14295,14296,14299,14300,14301,14303,14304,
14307,14308,14309,14310,14311,14312,14313,14314,
14315,14316,14318,14319,14321,14322,14323,14324,
14325,14326,14327,14328,14329,14330,14331,14333,
14335,14336,14337,14338,14340,14341,14342,14343,
14344,14346,14347,14348,14349,14350,14351,14352,
14353,14354,14356,14357,14358,14359,14360,14365,
14368,14369,14370,14371,14372,14373,14374,14375,
14376,14377,14378,14379,14380,14381,14382,14383,
14384,14386,14387,14388,14389,14390,14391,14392,
14393,14394,14395,14396,14397,14398,14399,14400,
14401,14402,14403,14404,14406,14407,14408,14409,
14410,14412,14413,14414,14416,14417,14420,14421,
14422,14423,14424,14425,14426,14427,14428,14429,
14430,14432,14433,14434,14435,14436,14438,14439,
14440,14441,14442,14443,14444,14445,14446,14447,
14448,14449,14450,14451,14452,14453,14455,14456,
14457,14458,14459,14463,14464,14465,14466,14467,
14468,14469,14470,14472,14473,14475,14477,14479,
14480,14481,14482,14483,14484,14485,14486,14487,
14488,14490,14491,14492,14493,14494,14495,14497,
14498,14499,14501,14502,14503,14504,14505,14506,
14507,14508,14509,14510,14511,14512,14513,14514,
14515,14516,14517,14518,14519,14520,14521,14522,
14523,14524,14525,14526,14527,14528,14529,14530,
14531,14532,14534,14535,14536,14538,14539,14540,
14541,14542,14543,14544,14545,14546,14547,14548,
14549,14550,14551,14552,14553,14554,14555,14556,
14557,14559,14560,14561,14562,14563,14564,14565,
14566,14567,14568,14569,14570,14572,14573,14574,
14576,14577,14578,14579,14580,14581,14582,14583,
14584,14585,14587,14589,14590,14591,14592,14593,
14595,14596,14597,14598,14599,14600,14601,14602,
14603,14604,14605,14606,14607,14608,14609,14612,
14613,14614,14615,14616,14617,14618,14619,14620,
14621,14622,14624,14629,14630,14631,14632,14633,
14634,14635,14636,14638,14639,14641,14642,14643,
14644,14645,14646,14647,14648,14649,14650,14651,
14652,14653,14654,14655,14656,14657,14658,14659,
14660,14661,14662,14663,14664,14665,14666,14667,
14668,14669,14670,14671,14672,14673,14674,14675,
14676,14677,14678,14679,14680,14681,14682,14683,
14684,14685,14686,14688,14689,14690,14691,14692,
14693,14694,14695,14696,14698,14699,14700,14701,
14702,14703,14704,14705,14706,14707,14708,14709,
14711,14712,14713,14714,14715,14716,14717,14718,
14719,14720,14721,14722,14723,14724,14725,14726,
14727,14728,14729,14731,14732,14733,14734,14735,
14737,14738,14739,14740,14741,14742,14743,14744,
14745,14746,14748,14749,14751,14752,14753,14754,
14755,14756,14757,14758,14759,14760,14761,14762,
14763,14764,14765,14766,14768,14771,14772,14773,
14774,14775,14776,14777,14778,14779,14780,14781,
14782,14783,14784,14785,14786,14787,14788,14790,
14791,14792,14793,14794,14795,14796,14797,14798,
14799,14800,14801,14802,14803,14804,14807,14808,
14809,14810,14812,14813,14814,14815,14816,14817,
14818,14820,14821,14822,14823,14824,14825,14826,
14828,14829,14830,14832,14833,14834,14835,14836,
14837,14838,14839,14840,14841,14842,14843,14844,
14845,14846,14847,14848,14849,14850,14852,14853,
14854,14855,14856,14857,14858,14859,14860,14861,
14862,14863,14864,14865,14866,14867,14868,14870,
14871,14872,14873,14874,14875,14876,14877,14879,
14881,14882,14884,14885,14886,14887,14888,14889,
14890,14891,14893,14894,14896,14897,14899,14900,
14901,14902,14903,14904,14905,14907,14908,14909,
14910,14911,14912,14913,14915,14916,14917,14918,
14919,14920,14921,14922,14923,14924,14925,14926,
14927,14928,14929,14931,14932,14933,14934,14935,
14936,14937,14938,14939,14940,14941,14942,14943,
14944,14945,14948,14949,14950,14951,14952,14953,
14955,14956,14958,14959,14960,14961,14962,14963,
14964,14965,14966,14967,14968,14970,14971,14972,
14973,14974,14975,14976,14977,14978,14979,14980,
14981,14982,14983,14984,14985,14986,14987,14988,
14989,14990,14991,14992,14993,14994,14995,14996,
14997,14998,14999,15000,15002,15003,15004,15005,
15006,15007,15008,15009,15010,15011,15012,15013,
15014,15015,15016,15017,15019,15020,15021,15022,
15023,15024,15026,15027,15028,15029,15030,15031,
15032,15033,15034,15035,15036,15038,15039,15040,
15041,15042,15043,15044,15046,15047,15048,15049,
15050,15051,15052,15053,15055,15056,15057,15058,
15059,15060,15061,15062,15063,15064,15065,15068,
15069,15070,15071,15072,15073,15074,15075,15076,
15077,15078,15079,15080,15081,15082,15083,15084,
15086,15087,15088,15089,15090,15092,15093,15094,
15096,15098,15099,15100,15102,15103,15105,15106,
15107,15108,15109,15110,15111,15112,15113,15114,
15115,15116,15117,15118,15119,15120,15121,15122,
15123,15124,15125,15126,15127,15128,15129,15130,
15131,15132,15133,15134,15135,15136,15137,15138,
15139,15140,15141,15142,15143,15144,15146,15147,
15149,15150,15151,15152,15153,15154,15155,15156,
15157,15158,15160,15161,15162,15163,15164,15165,
15166,15167,15168,15169,15170,15171,15172,15173,
15174,15175,15176,15177,15178,15179,15180,15181,
15182,15183,15184,15185,15186,15187,15188,15189,
15190,15191,15192,15193,15194,15195,15196,15197,
15198,15199,15200,15201,15202,15203,15204,15205,
15206,15207,15208,15209,15210,15211,15212,15213,
15214,15215,15216,15217,15218,15219,15220,15221,
15222,15223,15224,15226,15227,15229,15230,15231,
15232,15233,15234,15235,15236,15237,15238,15239,
15240,15241,15242,15243,15244,15245,15246,15247,
15248,15249,15250,15251,15252,15253,15254,15255,
15256,15257,15258,15259,15260,15261,15262,15263,
15264,15265,15266,15267,15268,15269,15270,15271,
15272,15273,15274,15275,15276,15277,15278,15279,
15280,15281,15282,15283,15284,15285,15286,15287,
15288,15289,15290,15291,15292,15293,15294,15295,
15296,15297,15298,15299,15300,15302,15303,15304,
15305,15306,15307,15308,15309,15310,15311,15314,
15315,15316,15317,15318,15319,15320,15321,15322,
15323,15324,15325,15326,15327,15328,15329,15330,
15331,15333,15334,15335,15336,15338,15339,15340,
15341,15342,15343,15344,15346,15347,15348,15349,
15350,15351,15352,15354,15355,15356,15357,15358,
15359,15361,15362,15363,15364,15365,15366,15367,
15368,15369,15370,15372,15373,15374,15375,15376,
15377,15378,15379,15380,15381,15382,15383,15384,
15385,15386,15388,15389,15390,15392,15393,15395,
15397,15398,15400,15401,15402,15403,15405,15406,
15407,15408,15410,15411,15412,15413,15414,15415,
15417,15419,15420,15421,15422,15423,15424,15425,
15426,15427,15428,15429,15430,15431,15432,15433,
15434,15435,15436,15437,15438,15439,15440,15441,
15442,15443,15444,15445,15446,15447,15448,15449,
15450,15451,15452,15453,15454,15455,15456,15457,
15458,15459,15460,15461,15462,15463,15464,15465,
15466,15467,15468,15469,15470,15471,15472,15473,
15474,15475,15476,15477,15478,15479,15480,15481,
15482,15483,15484,15485,15486,15487,15488,15489,
15490,15491,15492,15493,15494,15495,15496,15497,
15498,15499,15500,15501,15502,15503,15504,15505,
15506,15507,15508,15509,15510,15511,15512,15513,
15514,15515,15516,15517,15518,15519,15520,15521,
15522,15523,15524,15525,15526,15527,15528,15529,
15530,15531,15532,15533,15534,15535,15536,15537,
15538,15539,15540,15541,15542,15543,15544,15545,
15546,15547,15548,15549,15550,15551,15553,15554,
15555,15556,15557,15558,15559,15560,15561,15562,
15563,15564,15565,15566,15567,15568,15569,15570,
15571,15573,15574,15575,15576,15578,15579,15580,
15581,15583,15584,15586,15587,15588,15589,15590,
15591,15592,15593,15594,15595,15596,15598,15599,
15601,15602,15603,15604,15606,15607,15608,15609,
15610,15611,15612,15614,15615,15617,15618,15619,
15620,15621,15622,15623,15624,15625,15626,15628,
15629,15630,15631,15633,15634,15635,15636,15637,
15638,15639,15640,15641,15642,15643,15644,15645,
15646,15648,15649,15650,15651,15652,15654,15655,
15656,15657,15659,15660,15662,15663,15664,15665,
15666,15667,15668,15669,15671,15672,15674,15675,
15677,15678,15680,15681,15682,15683,15684,15685,
15686,15687,15688,15690,15691,15693,15694,15695,
15696,15697,15699,15700,15701,15703,15704,15705,
15706,15707,15708,15709,15710,15711,15712,15713,
15714,15715,15716,15717,15718,15719,15720,15721,
15722,15723,15724,15725,15726,15727,15728,15729,
15730,15731,15732,15733,15734,15735,15736,15737,
15738,15739,15740,15741,15742,15743,15744,15745,
15746,15747,15748,15749,15750,15751,15752,15753,
15754,15755,15756,15757,15758,15759,15760,15761,
15762,15763,15764,15765,15766,15767,15768,15769,
15770,15771,15772,15773,15774,15775,15776,15777,
15778,15779,15780,15781,15782,15783,15784,15785,
15786,15787,15789,15790,15791,15792,15793,15794,
15795,15796,15797,15798,15799,15800,15801,15802,
15803,15804,15805,15806,15807,15808,15810,15811,
15812,15814,15815,15816,15817,15818,15819,15820,
15821,15822,15823,15824,15825,15826,15827,15828,
15829,15830,15831,15832,15833,15834,15835,15836,
15837,15838,15839,15840,15841,15842,15843,15844,
15845,15846,15847,15848,15849,15850,15851,15853,
15854,15855,15857,15858,15859,15860,15861,15862,
15863,15865,15866,15867,15868,15869,15870,15873,
15874,15875,15876,15877,15878,15880,15881,15883,
15884,15886,15888,15889,15891,15894,15895,15896,
15897,15898,15899,15900,15901,15902,15903,15905,
15906,15907,15908,15909,15910,15911,15912,15913,
15914,15916,15917,15918,15919,15920,15921,15923,
15924,15925,15926,15927,15928,15929,15931,15932,
15933,15934,15936,15937,15938,15939,15940,15941,
15942,15943,15944,15945,15946,15947,15948,15949,
15950,15951,15952,15953,15954,15955,15956,15957,
15958,15960,15961,15962,15963,15965,15968,15969,
15970,15971,15972,15973,15975,15976,15977,15978,
15979,15980,15981,15982,15984,15985,15986,15987,
15988,15989,15990,15991,15992,15993,15994,15995,
15996,15997,15998,15999,16000,16001,16002,16003,
16004,16005,16006,16007,16008,16009,16010,16011,
16012,16013,16014,16015,16016,16017,16018,16019,
16020,16021,16022,16023,16024,16025,16026,16027,
16028,16029,16030,16031,16032,16033,16034,16035,
16036,16037,16038,16039,16040,16041,16042,16043,
16044,16045,16046,16047,16048,16049,16052,16053,
16054,16055,16056,16057,16058,16059,16060,16061,
16062,16063,16064,16065,16066,16067,16068,16069,
16070,16071,16072,16073,16074,16075,16076,16077,
16080,16081,16082,16083,16084,16085,16086,16087,
16088,16089,16090,16091,16092,16093,16094,16095,
16096,16097,16098,16099,16100,16101,16102,16103,
16104,16105,16106,16107,16108,16109,16110,16111,
16112,16113,16114,16115,16116,16117,16118,16119,
16120,16121,16122,16123,16124,16125,16126,16127,
16128,16129,16131,16132,16133,16134,16135,16136,
16137,16138,16139,16140,16141,16142,16143,16144,
16145,16147,16148,16149,16150,16151,16153,16154,
16155,16156,16157,16158,16160,16161,16162,16163,
16164,16165,16167,16168,16169,16170,16171,16172,
16173,16174,16175,16176,16177,16178,16179,16180,
16181,16182,16183,16184,16185,16186,16187,16188,
16189,16190,16191,16192,16193,16194,16195,16196,
16197,16198,16199,16200,16203,16204,16205,16206,
16207,16208,16209,16210,16211,16213,16214,16215,
16216,16217,16218,16219,16220,16221,16222,16223,
16224,16225,16226,16227,16228,16229,16230,16231,
16234,16235,16236,16237,16238,16239,16240,16241,
16242,16243,16244,16247,16248,16249,16250,16251,
16252,16253,16254,16255,16257,16259,16260,16261,
16262,16263,16264,16265,16266,16267,16268,16269,
16270,16271,16272,16273,16274,16275,16276,16277,
16278,16279,16280,16281,16283,16284,16285,16286,
16287,16288,16289,16290,16291,16292,16293,16294,
16295,16296,16297,16298,16299,16300,16301,16302,
16303,16304,16305,16306,16307,16308,16309,16310,
16311,16312,16313,16314,16315,16316,16317,16318,
16319,16320,16321,16322,16323,16324,16325,16326,
16327,16328,16330,16331,16332,16333,16334,16335,
16336,16337,16338,16339,16340,16341,16342,16343,
16344,16345,16346,16347,16348,16349,16350,16351,
16352,16353,16354,16355,16356,16357,16358,16359,
16360,16361,16362,16363,16364,16365,16366,16367,
16368,16369,16370,16371,16372,16373,16374,16375,
16376,16377,16378,16379,16380,16381,16382,16383,
16384,16385,16386,16387,16388,16389,16390,16391,
16392,16393,16394,16396,16397,16399,16400,16401,
16402,16403,16404,16405,16407,16408,16410,16411,
16412,16413,16414,16415,16416,16418,16419,16420,
16421,16422,16423,16424,16425,16426,16427,16428,
16430,16431,16432,16433,16434,16435,16436,16437,
16439,16440,16441,16442,16443,16444,16445,16446,
16447,16448,16449,16450,16452,16453,16455,16457,
16458,16459,16460,16462,16463,16464,16465,16466,
16467,16468,16469,16470,16472,16473,16474,16475,
16477,16478,16480,16481,16483,16484,16485,16486,
16488,16489,16490,16491,16492,16493,16494,16495,
16496,16497,16498,16499,16500,16501,16502,16503,
16504,16505,16506,16507,16508,16509,16510,16511,
16512,16514,16515,16516,16517,16518,16519,16520,
16521,16522,16523,16524,16525,16526,16527,16528,
16529,16531,16532,16533,16534,16535,16536,16537,
16538,16539,16540,16541,16542,16543,16544,16545,
16546,16547,16548,16549,16551,16552,16553,16554,
16555,16556,16557,16558,16559,16560,16561,16562,
16563,16564,16565,16566,16567,16568,16569,16570,
16571,16572,16573,16574,16575,16576,16577,16578,
16579,16580,16581,16582,16583,16584,16585,16586,
16587,16588,16589,16590,16591,16592,16593,16594,
16595,16596,16597,16598,16599,16600,16601,16602,
16603,16604,16605,16606,16607,16608,16609,16610,
16611,16612,16613,16614,16615,16616,16617,16618,
16619,16620,16621,16622,16623,16624,16625,16626,
16627,16628,16629,16630,16631,16632,16633,16634,
16635,16636,16637,16638,16639,16640,16641,16642,
16643,16644,16645,16646,16647,16648,16649,16650,
16651,16652,16653,16654,16655,16656,16657,16658,
16659,16660,16661,16662,16663,16664,16665,16666,
16667,16668,16669,16670,16671,16673,16674,16675,
16676,16677,16678,16679,16680,16682,16683,16684,
16685,16686,16687,16688,16689,16691,16692,16694,
16696,16697,16699,16700,16701,16702,16703,16704,
16705,16706,16707,16708,16710,16711,16712,16713,
16714,16715,16716,16717,16718,16719,16720,16722,
16723,16724,16725,16726,16727,16728,16729,16730,
16731,16732,16733,16734,16735,16738,16739,16741,
16742,16743,16744,16745,16746,16747,16748,16749,
16751,16752,16753,16754,16755,16756,16757,16758,
16759,16760,16761,16763,16764,16765,16766,16768,
16769,16770,16771,16772,16773,16774,16775,16776,
16777,16778,16780,16781,16782,16784,16785,16786,
16787,16788,16789,16791,16792,16793,16794,16795,
16796,16797,16798,16799,16800,16801,16802,16803,
16804,16805,16806,16807,16808,16809,16810,16812,
16813,16814,16815,16816,16817,16818,16819,16820,
16821,16822,16823,16824,16825,16826,16827,16828,
16829,16830,16831,16832,16833,16834,16835,16836,
16837,16838,16839,16840,16841,16842,16843,16844,
16845,16846,16847,16848,16849,16850,16851,16852,
16853,16854,16855,16856,16857,16858,16859,16860,
16861,16862,16863,16864,16865,16866,16867,16868,
16869,16870,16871,16872,16873,16874,16875,16876,
16877,16878,16879,16880,16881,16882,16883,16884,
16885,16886,16887,16888,16889,16890,16891,16892,
16893,16894,16895,16896,16897,16898,16899,16900,
16901,16902,16903,16904,16905,16906,16907,16908,
16909,16910,16911,16912,16913,16914,16915,16916,
16917,16918,16919,16920,16921,16922,16923,16924,
16925,16926,16927,16928,16929,16930,16931,16932,
16933,16934,16935,16936,16937,16938,16939,16940,
16941,16942,16943,16944,16945,16946,16947,16948,
16950,16951,16952,16953,16954,16955,16956,16957,
16958,16959,16960,16961,16962,16963,16965,16966,
16967,16968,16969,16971,16973,16974,16976,16977,
16978,16979,16980,16981,16982,16983,16984,16985,
16986,16987,16988,16989,16991,16992,16993,16994,
16995,16996,16997,16999,17000,17001,17002,17004,
17005,17006,17008,17009,17010,17011,17012,17013,
17014,17015,17016,17017,17018,17019,17020,17022,
17023,17024,17025,17026,17027,17028,17029,17030,
17031,17032,17033,17035,17036,17037,17038,17039,
17040,17041,17042,17043,17044,17045,17046,17047,
17048,17049,17050,17051,17052,17053,17054,17056,
17058,17059,17060,17061,17063,17064,17065,17066,
17067,17068,17069,17070,17071,17072,17073,17074,
17075,17076,17077,17078,17079,17080,17081,17082,
17083,17084,17085,17086,17087,17088,17089,17090,
17091,17092,17093,17094,17095,17096,17097,17098,
17099,17100,17101,17102,17103,17104,17105,17106,
17107,17108,17109,17110,17111,17112,17113,17114,
17115,17116,17117,17118,17119,17120,17121,17122,
17123,17124,17125,17126,17127,17128,17129,17130,
17131,17132,17133,17134,17135,17136,17137,17138,
17139,17140,17141,17142,17143,17144,17145,17146,
17147,17149,17150,17151,17152,17153,17154,17155,
17156,17157,17158,17159,17160,17162,17163,17164,
17165,17166,17167,17168,17169,17170,17171,17172,
17173,17174,17175,17176,17177,17178,17179,17180,
17181,17182,17183,17184,17185,17186,17187,17188,
17189,17190,17191,17192,17193,17194,17195,17196,
17197,17199,17200,17201,17202,17203,17204,17205,
17206,17207,17208,17209,17210,17211,17212,17213,
17214,17215,17216,17217,17218,17219,17220,17221,
17222,17223,17224,17225,17226,17228,17229,17230,
17231,17232,17233,17234,17236,17238,17239,17240,
17241,17242,17243,17244,17245,17246,17247,17248,
17249,17250,17251,17252,17253,17254,17255,17257,
17258,17259,17260,17261,17263,17264,17265,17266,
17267,17268,17269,17271,17275,17276,17277,17278,
17280,17282,17284,17285,17286,17287,17288,17289,
17291,17292,17293,17294,17295,17296,17297,17298,
17299,17300,17301,17302,17303,17304,17305,17306,
17307,17308,17309,17310,17312,17313,17314,17315,
17316,17317,17318,17319,17320,17321,17322,17323,
17324,17325,17326,17327,17328,17329,17330,17331,
17332,17334,17335,17336,17337,17338,17339,17340,
17342,17343,17344,17345,17347,17348,17349,17350,
17352,17353,17354,17356,17357,17359,17360,17361,
17362,17363,17364,17365,17366,17367,17368,17369,
17370,17371,17372,17373,17374,17375,17376,17377,
17378,17379,17380,17381,17382,17383,17384,17385,
17386,17387,17388,17389,17390,17391,17392,17393,
17394,17395,17396,17397,17398,17399,17400,17401,
17402,17403,17404,17405,17406,17407,17408,17409,
17410,17411,17412,17413,17414,17415,17416,17417,
17418,17419,17420,17421,17422,17423,17424,17425,
17426,17427,17428,17429,17430,17431,17432,17433,
17434,17435,17436,17437,17438,17439,17440,17441,
17442,17443,17444,17445,17446,17447,17448,17449,
17450,17451,17452,17453,17454,17455,17456,17457,
17458,17459,17460,17461,17462,17463,17464,17465,
17466,17467,17468,17469,17470,17471,17472,17473,
17474,17475,17476,17477,17478,17479,17480,17481,
17482,17483,17484,17485,17486,17487,17488,17489,
17490,17491,17492,17493,17495,17496,17497,17498,
17500,17501,17502,17503,17505,17506,17507,17508,
17509,17511,17512,17513,17514,17515,17517,17518,
17519,17520,17521,17523,17524,17525,17526,17527,
17528,17529,17530,17531,17532,17533,17534,17535,
17536,17537,17538,17540,17541,17542,17543,17544,
17545,17546,17547,17548,17549,17550,17552,17554,
17555,17556,17557,17559,17560,17561,17562,17563,
17564,17565,17567,17568,17569,17570,17571,17572,
17573,17574,17576,17577,17578,17579,17580,17581,
17582,17583,17584,17586,17587,17588,17589,17590,
17591,17592,17593,17594,17595,17597,17598,17599,
17601,17602,17603,17604,17606,17607,17608,17609,
17611,17612,17613,17614,17615,17616,17618,17619,
17620,17621,17622,17623,17624,17625,17626,17627,
17629,17630,17631,17632,17634,17635,17636,17637,
17638,17639,17640,17641,17642,17643,17644,17645,
17646,17647,17648,17649,17650,17651,17652,17653,
17654,17655,17656,17657,17658,17659,17660,17661,
17662,17663,17664,17665,17666,17667,17668,17669,
17670,17671,17672,17673,17674,17675,17676,17677,
17678,17679,17680,17681,17682,17683,17684,17685,
17686,17687,17688,17689,17690,17691,17692,17693,
17694,17695,17696,17697,17698,17699,17700,17701,
17702,17703,17704,17705,17706,17707,17708,17709,
17710,17711,17712,17713,17714,17715,17716,17717,
17718,17719,17720,17721,17722,17723,17724,17725,
17726,17727,17728,17729,17730,17731,17732,17733,
17734,17735,17736,17737,17738,17739,17740,17741,
17742,17743,17744,17745,17746,17747,17748,17749,
17750,17751,17752,17753,17754,17755,17756,17757,
17758,17759,17760,17761,17762,17763,17764,17765,
17766,17767,17768,17770,17773,17774,17775,17776,
17777,17778,17779,17780,17781,17782,17783,17784,
17785,17786,17787,17789,17790,17791,17792,17794,
17795,17796,17797,17798,17799,17800,17801,17802,
17803,17804,17805,17806,17807,17808,17809,17810,
17811,17812,17813,17816,17817,17820,17821,17822,
17823,17825,17826,17827,17829,17830,17831,17832,
17833,17834,17835,17837,17838,17839,17841,17842,
17844,17846,17847,17848,17849,17850,17851,17852,
17853,17854,17855,17856,17857,17858,17859,17863,
17864,17865,17866,17867,17868,17870,17872,17873,
17874,17875,17876,17877,17878,17879,17880,17881,
17883,17884,17885,17886,17888,17889,17890,17891,
17892,17893,17894,17895,17896,17897,17898,17899,
17900,17903,17904,17905,17906,17907,17908,17909,
17910,17911,17912,17913,17914,17915,17916,17917,
17918,17919,17920,17921,17922,17923,17924,17925,
17926,17927,17928,17929,17930,17931,17932,17933,
17934,17935,17936,17937,17938,17939,17940,17941,
17942,17943,17944,17945,17946,17947,17948,17949,
17950,17951,17952,17953,17954,17955,17956,17957,
17958,17959,17960,17961,17962,17963,17964,17965,
17966,17967,17968,17969,17970,17971,17972,17973,
17974,17975,17976,17977,17978,17979,17980,17981,
17982,17983,17984,17985,17986,17987,17988,17989,
17990,17991,17992,17993,17994,17995,17996,17997,
17998,17999,18000,18001,18002,18003,18004,18005,
18006,18007,18008,18009,18010,18011,18012,18013,
18014,18015,18016,18017,18018,18019,18020,18021,
18022,18023,18024,18025,18026,18027,18028,18029,
18030,18031,18032,18033,18034,18035,18036,18037,
18038,18039,18040,18041,18042,18043,18044,18045,
18046,18047,18048,18050,18051,18052,18053,18054,
18055,18056,18058,18059,18060,18061,18062,18063,
18064,18065,18066,18067,18068,18069,18070,18071,
18072,18073,18074,18075,18076,18077,18078,18079,
18080,18081,18082,18083,18084,18085,18086,18087,
18088,18089,18090,18091,18092,18093,18094,18095,
18096,18097,18098,18099,18100,18101,18102,18103,
18104,18106,18107,18108,18109,18110,18111,18112,
18113,18114,18115,18116,18117,18118,18119,18121,
18122,18123,18124,18125,18126,18127,18128,18129,
18130,18131,18132,18134,18135,18136,18137,18138,
18139,18141,18142,18143,18144,18145,18146,18147,
18148,18149,18150,18151,18152,18153,18154,18156,
18157,18158,18159,18161,18162,18163,18164,18165,
18166,18167,18168,18169,18170,18173,18174,18176,
18177,18178,18179,18180,18182,18183,18184,18185,
18186,18187,18188,18189,18192,18193,18194,18195,
18196,18197,18198,18199,18200,18201,18202,18203,
18204,18205,18206,18207,18208,18209,18210,18211,
18212,18213,18214,18215,18216,18217,18218,18219,
18220,18221,18222,18223,18224,18225,18226,18227,
18228,18229,18230,18231,18232,18233,18234,18235,
18236,18237,18238,18239,18240,18241,18242,18243,
18244,18245,18246,18247,18248,18249,18250,18251,
18252,18253,18255,18256,18257,18258,18259,18260,
18261,18262,18263,18264,18265,18266,18267,18268,
18269,18270,18271,18272,18273,18274,18275,18276,
18277,18278,18279,18280,18281,18282,18283,18284,
18285,18286,18287,18288,18290,18291,18292,18293,
18294,18295,18296,18297,18298,18299,18300,18301,
18302,18303,18304,18305,18306,18307,18308,18309,
18310,18311,18312,18313,18314,18315,18316,18317,
18318,18319,18320,18321,18322,18323,18324,18325,
18327,18329,18332,18333,18334,18335,18336,18337,
18338,18339,18340,18341,18342,18343,18344,18345,
18347,18348,18349,18350,18351,18352,18353,18354,
18356,18357,18358,18359,18360,18361,18362,18363,
18364,18365,18366,18367,18368,18370,18371,18372,
18373,18374,18375,18376,18377,18378,18379,18380,
18381,18383,18384,18385,18386,18387,18388,18389,
18390,18391,18393,18394,18395,18396,18397,18398,
18399,18401,18402,18403,18404,18405,18406,18407,
18408,18409,18410,18411,18412,18413,18414,18415,
18416,18417,18418,18419,18420,18421,18422,18423,
18424,18425,18426,18427,18428,18429,18430,18431,
18432,18433,18434,18435,18436,18437,18438,18439,
18441,18442,18443,18444,18445,18446,18447,18448,
18449,18450,18451,18453,18454,18455,18456,18457,
18458,18459,18460,18461,18463,18464,18465,18466,
18467,18468,18469,18470,18471,18472,18473,18474,
18475,18476,18477,18478,18479,18480,18481,18482,
18483,18484,18485,18486,18487,18488,18489,18490,
18491,18493,18494,18495,18496,18497,18498,18499,
18500,18501,18502,18503,18504,18505,18506,18507,
18508,18509,18510,18512,18513,18514,18515,18516,
18517,18518,18519,18520,18521,18522,18523,18525,
18526,18527,18528,18529,18530,18531,18532,18533,
18534,18535,18536,18537,18538,18539,18540,18541,
18542,18543,18544,18545,18546,18547,18548,18549,
18550,18551,18552,18553,18554,18555,18556,18557,
18558,18559,18560,18561,18562,18563,18564,18565,
18566,18567,18568,18569,18570,18571,18572,18573,
18574,18575,18576,18577,18578,18579,18580,18581,
18582,18583,18584,18585,18586,18587,18588,18589,
18590,18591,18592,18593,18595,18597,18598,18599,
18600,18601,18602,18603,18605,18607,18608,18609,
18611,18612,18613,18614,18615,18616,18617,18618,
18619,18620,18621,18622,18623,18624,18625,18628,
18629,18631,18632,18633,18635,18636,18637,18638,
18639,18641,18642,18643,18644,18645,18646,18647,
18649,18650,18651,18652,18653,18654,18656,18658,
18659,18660,18662,18663,18664,18665,18667,18668,
18669,18670,18672,18674,18675,18676,18677,18678,
18679,18681,18682,18683,18684,18685,18686,18687,
18688,18689,18690,18691,18692,18693,18696,18697,
18698,18700,18702,18703,18704,18705,18706,18708,
18709,18710,18711,18714,18715,18716,18717,18718,
18719,18720,18721,18722,18723,18724,18725,18726,
18727,18729,18730,18731,18733,18734,18735,18736,
18737,18738,18739,18740,18741,18742,18743,18745,
18746,18747,18748,18749,18750,18751,18752,18753,
18754,18755,18756,18757,18758,18759,18760,18761,
18762,18763,18764,18765,18766,18767,18768,18769,
18770,18771,18772,18773,18774,18775,18776,18777,
18778,18779,18780,18781,18782,18783,18784,18785,
18786,18787,18788,18789,18790,18791,18792,18793,
18794,18795,18796,18797,18798,18799,18800,18801,
18802,18803,18804,18805,18806,18807,18808,18809,
18810,18811,18812,18813,18814,18815,18816,18817,
18818,18819,18820,18821,18822,18823,18824,18825,
18826,18827,18828,18829,18830,18831,18832,18833,
18834,18835,18836,18837,18838,18839,18840,18841,
18842,18843,18844,18845,18846,18847,18848,18849,
18850,18851,18852,18853,18854,18855,18856,18857,
18858,18859,18860,18861,18862,18863,18864,18865,
18866,18867,18868,18869,18870,18871,18872,18873,
18874,18875,18876,18877,18878,18879,18880,18881,
18882,18883,18884,18885,18886,18887,18888,18889,
18890,18891,18892,18894,18895,18896,18897,18898,
18899,18900,18901,18902,18903,18904,18905,18906,
18907,18909,18910,18911,18912,18913,18914,18915,
18916,18917,18918,18919,18920,18921,18922,18923,
18924,18925,18926,18927,18928,18929,18930,18931,
18932,18933,18934,18935,18936,18937,18938,18939,
18940,18941,18942,18943,18944,18945,18946,18947,
18948,18949,18950,18952,18955,18956,18957,18958,
18959,18961,18962,18963,18965,18966,18967,18968,
18969,18970,18971,18972,18973,18974,18975,18976,
18977,18978,18979,18980,18981,18982,18983,18984,
18985,18986,18987,18988,18989,18990,18991,18992,
18993,18994,18995,18996,18997,18998,18999,19000,
19001,19002,19003,19004,19005,19007,19008,19009,
19010,19011,19012,19013,19014,19015,19016,19017,
19018,19019,19020,19021,19022,19023,19024,19025,
19026,19027,19028,19029,19030,19031,19032,19033,
19034,19035,19036,19037,19038,19039,19040,19041,
19042,19043,19044,19045,19046,19047,19048,19049,
19050,19051,19052,19053,19054,19055,19056,19057,
19058,19059,19060,19061,19062,19063,19064,19065,
19066,19067,19068,19069,19070,19071,19072,19073,
19074,19075,19076,19077,19078,19079,19080,19081,
19082,19083,19084,19085,19086,19087,19088,19089,
19090,19091,19092,19093,19094,19095,19096,19097,
19098,19099,19100,19101,19102,19103,19104,19105,
19106,19107,19108,19109,19110,19111,19112,19114,
19115,19116,19117,19118,19119,19121,19122,19123,
19124,19125,19126,19127,19128,19129,19130,19131,
19132,19133,19134,19135,19136,19137,19138,19139,
19140,19141,19142,19143,19144,19145,19146,19147,
19148,19149,19150,19151,19152,19153,19154,19155,
19156,19157,19158,19159,19160,19161,19162,19163,
19165,19166,19167,19168,19169,19170,19171,19172,
19174,19175,19176,19177,19178,19179,19180,19181,
19182,19183,19184,19185,19186,19187,19188,19189,
19190,19191,19192,19193,19194,19195,19196,19197,
19198,19199,19200,19202,19203,19204,19205,19206,
19207,19208,19209,19210,19211,19212,19213,19214,
19215,19216,19217,19218,19220,19221,19222,19223,
19224,19225,19226,19227,19228,19229,19230,19231,
19232,19234,19236,19237,19238,19239,19240,19241,
19242,19243,19244,19245,19247,19248,19249,19250,
19251,19252,19253,19254,19256,19258,19259,19260,
19261,19262,19263,19264,19265,19266,19267,19268,
19269,19270,19271,19272,19273,19274,19276,19277,
19278,19279,19280,19281,19282,19283,19284,19285,
19286,19287,19288,19289,19290,19291,19292,19293,
19294,19295,19296,19297,19298,19299,19300,19301,
19302,19303,19304,19305,19306,19307,19308,19309,
19310,19311,19312,19313,19314,19315,19316,19317,
19318,19319,19320,19321,19322,19323,19324,19325,
19326,19327,19328,19329,19330,19331,19332,19333,
19334,19335,19336,19337,19338,19339,19340,19341,
19342,19343,19344,19345,19346,19347,19348,19350,
19351,19352,19353,19354,19355,19356,19357,19358,
19359,19360,19361,19362,19363,19364,19365,19366,
19367,19368,19369,19370,19371,19372,19373,19374,
19375,19376,19377,19378,19379,19380,19381,19382,
19383,19384,19385,19386,19387,19388,19389,19390,
19391,19392,19393,19394,19395,19396,19397,19398,
19399,19400,19401,19402,19403,19404,19405,19406,
19407,19408,19409,19410,19411,19413,19414,19415,
19416,19417,19418,19419,19420,19422,19423,19424,
19425,19426,19427,19428,19429,19430,19431,19432,
19433,19434,19435,19436,19437,19438,19439,19440,
19442,19443,19444,19445,19447,19448,19449,19450,
19451,19452,19453,19458,19459,19460,19461,19462,
19463,19464,19465,19466,19467,19468,19469,19470,
19471,19472,19473,19474,19475,19476,19477,19478,
19479,19480,19481,19483,19484,19486,19487,19488,
19490,19491,19492,19493,19494,19495,19496,19497,
19498,19499,19500,19501,19502,19503,19504,19505,
19506,19507,19508,19509,19510,19511,19512,19513,
19514,19515,19516,19517,19518,19519,19520,19522,
19523,19524,19526,19527,19528,19529,19530,19531,
19532,19533,19534,19535,19536,19537,19538,19539,
19540,19541,19542,19543,19544,19545,19547,19548,
19549,19550,19551,19552,19553,19554,19555,19556,
19557,19558,19559,19560,19561,19562,19563,19564,
19565,19566,19567,19568,19569,19570,19571,19572,
19573,19574,19575,19576,19577,19578,19579,19580,
19581,19582,19583,19584,19585,19586,19587,19588,
19589,19590,19591,19592,19593,19594,19595,19596,
19597,19598,19599,19600,19601,19602,19603,19604,
19605,19606,19607,19608,19609,19610,19611,19612,
19613,19614,19615,19616,19617,19618,19619,19620,
19621,19622,19623,19624,19625,19626,19627,19628,
19629,19630,19631,19632,19633,19634,19635,19636,
19637,19638,19639,19640,19641,19642,19643,19644,
19645,19646,19647,19648,19649,19650,19651,19652,
19653,19654,19655,19656,19657,19658,19659,19660,
19661,19662,19663,19664,19665,19666,19667,19668,
19669,19670,19671,19672,19673,19674,19675,19676,
19678,19679,19680,19681,19682,19683,19684,19685,
19686,19688,19689,19690,19691,19692,19693,19694,
19695,19696,19697,19698,19699,19701,19702,19703,
19705,19706,19707,19708,19709,19710,19711,19712,
19713,19714,19715,19716,19717,19718,19719,19721,
19722,19723,19724,19725,19726,19727,19728,19729,
19730,19731,19733,19734,19736,19737,19738,19741,
19742,19743,19744,19745,19746,19747,19748,19749,
19750,19751,19752,19753,19754,19755,19756,19757,
19758,19759,19760,19761,19762,19763,19765,19766,
19767,19769,19770,19771,19772,19773,19774,19775,
19776,19777,19778,19779,19780,19781,19782,19783,
19784,19785,19786,19787,19788,19789,19791,19792,
19793,19794,19795,19797,19799,19800,19801,19802,
19803,19806,19808,19809,19810,19811,19812,19814,
19815,19816,19817,19818,19819,19822,19824,19826,
19827,19828,19829,19830,19831,19832,19833,19834,
19835,19836,19837,19838,19839,19840,19841,19842,
19843,19844,19845,19846,19847,19848,19849,19850,
19851,19852,19853,19854,19855,19856,19857,19858,
19859,19860,19861,19862,19863,19864,19865,19866,
19867,19868,19869,19870,19871,19872,19873,19874,
19875,19876,19877,19878,19879,19880,19881,19882,
19883,19884,19885,19886,19887,19888,19889,19890,
19891,19892,19893,19894,19895,19896,19897,19898,
19899,19900,19901,19902,19903,19904,19905,19906,
19907,19908,19909,19910,19911,19912,19913,19914,
19915,19916,19917,19918,19919,19920,19921,19922,
19923,19924,19925,19926,19927,19928,19929,19930,
19931,19932,19933,19934,19935,19936,19937,19938,
19939,19940,19941,19942,19943,19944,19945,19946,
19947,19948,19949,19950,19951,19953,19954,19955,
19956,19957,19958,19959,19960,19962,19963,19964,
19965,19966,19968,19970,19971,19972,19973,19974,
19976,19978,19980,19981,19982,19983,19984,19985,
19987,19988,19989,19992,19996,19997,19998,19999,
20000,20001,20002,20003,20004,20005,20006,20007,
20008,20009,20010,20011,20012,20013,20014,20015,
20016,20017,20018,20019,20020,20022,20023,20024,
20025,20027,20028,20029,20030,20031,20035,20037,
20039,20042,20043,20045,20046,20048,20049,20050,
20051,20052,20053,20054,20055,20057,20058,20059,
20060,20061,20064,20065,20066,20067,20068,20070,
20072,20074,20075,20076,20077,20079,20080,20082,
20083,20084,20085,20086,20087,20088,20089,20090,
20091,20092,20093,20094,20095,20096,20097,20098,
20099,20100,20101,20102,20103,20104,20105,20106,
20108,20109,20110,20112,20113,20114,20115,20116,
20117,20118,20119,20120,20121,20122,20123,20124,
20125,20126,20127,20128,20129,20130,20131,20132,
20133,20134,20135,20136,20137,20138,20139,20140,
20141,20142,20143,20144,20145,20146,20147,20148,
20149,20150,20151,20152,20153,20154,20155,20156,
20157,20158,20159,20160,20161,20162,20163,20164,
20165,20166,20167,20168,20169,20170,20171,20172,
20173,20174,20175,20176,20177,20178,20179,20180,
20181,20182,20183,20184,20185,20186,20187,20188,
20189,20190,20191,20192,20193,20194,20195,20196,
20197,20198,20199,20200,20201,20202,20203,20204,
20205,20206,20207,20208,20209,20210,20211,20212,
20213,20214,20215,20216,20217,20218,20219,20220,
20221,20222,20223,20224,20225,20226,20227,20228,
20229,20230,20231,20232,20233,20234,20235,20236,
20237,20238,20239,20240,20241,20242,20243,20244,
20245,20246,20247,20248,20249,20250,20251,20252,
20253,20255,20257,20258,20259,20260,20262,20263,
20264,20266,20269,20270,20271,20272,20273,20274,
20276,20278,20279,20280,20281,20282,20283,20285,
20286,20287,20288,20289,20290,20291,20292,20293,
20294,20295,20296,20297,20299,20301,20302,20303,
20304,20305,20306,20307,20308,20309,20310,20311,
20312,20313,20314,20315,20316,20317,20318,20319,
20320,20321,20322,20323,20325,20326,20327,20328,
20329,20330,20332,20333,20334,20335,20336,20337,
20338,20339,20340,20343,20344,20345,20347,20348,
20349,20350,20351,20352,20353,20354,20355,20356,
20357,20358,20359,20360,20361,20362,20363,20364,
20365,20366,20368,20369,20370,20371,20372,20373,
20374,20375,20376,20377,20378,20379,20380,20381,
20382,20385,20386,20387,20388,20389,20390,20393,
20394,20395,20396,20397,20398,20399,20400,20401,
20402,20403,20404,20405,20406,20407,20408,20409,
20410,20411,20412,20413,20414,20415,20416,20417,
20418,20419,20420,20421,20422,20423,20424,20425,
20426,20427,20428,20429,20430,20431,20432,20433,
20434,20435,20436,20437,20438,20439,20440,20441,
20442,20443,20444,20445,20446,20447,20448,20449,
20450,20451,20452,20454,20455,20456,20457,20458,
20459,20460,20461,20462,20463,20464,20465,20466,
20467,20468,20469,20470,20471,20472,20473,20474,
20475,20476,20477,20478,20479,20480,20481,20482,
20483,20484,20485,20486,20487,20488,20489,20490,
20491,20492,20493,20494,20495,20496,20497,20498,
20499,20500,20501,20502,20503,20504,20505,20506,
20507,20508,20509,20510,20511,20512,20513,20514,
20515,20516,20517,20518,20519,20520,20521,20522,
20523,20524,20525,20526,20527,20528,20529,20530,
20531,20532,20533,20534,20535,20536,20537,20538,
20539,20540,20542,20543,20544,20545,20546,20547,
20548,20549,20550,20551,20552,20553,20554,20555,
20556,20557,20558,20559,20560,20561,20563,20564,
20565,20566,20567,20568,20569,20570,20571,20572,
20574,20575,20576,20577,20578,20579,20580,20581,
20582,20583,20584,20585,20586,20587,20588,20589,
20591,20592,20593,20594,20595,20596,20598,20599,
20600,20601,20602,20604,20605,20606,20607,20608,
20609,20610,20611,20612,20613,20614,20615,20616,
20617,20618,20619,20620,20622,20623,20624,20625,
20626,20627,20628,20629,20631,20633,20634,20638,
20639,20640,20641,20643,20646,20647,20648,20649,
20650,20651,20652,20653,20654,20655,20656,20657,
20658,20659,20660,20661,20662,20663,20664,20665,
20666,20667,20668,20669,20670,20671,20672,20673,
20674,20675,20676,20677,20678,20679,20680,20681,
20682,20683,20684,20685,20686,20687,20688,20689,
20690,20691,20692,20693,20694,20695,20696,20697,
20698,20699,20700,20701,20702,20703,20704,20705,
20706,20707,20708,20709,20710,20711,20712,20713,
20714,20715,20716,20717,20718,20719,20720,20721,
20722,20723,20724,20725,20726,20727,20728,20729,
20730,20731,20732,20733,20734,20735,20736,20737,
20738,20739,20740,20741,20742,20743,20744,20745,
20746,20747,20748,20749,20750,20751,20752,20753,
20754,20755,20756,20757,20758,20759,20760,20761,
20762,20763,20764,20765,20766,20767,20768,20769,
20770,20771,20772,20773,20774,20775,20776,20777,
20778,20779,20780,20781,20782,20783,20784,20785,
20787,20788,20789,20790,20791,20792,20793,20794,
20796,20797,20798,20799,20800,20802,20803,20804,
20805,20807,20808,20809,20810,20811,20812,20813,
20814,20815,20816,20818,20819,20820,20821,20822,
20823,20824,20825,20826,20827,20829,20830,20832,
20833,20834,20835,20836,20837,20839,20840,20841,
20843,20844,20845,20847,20848,20849,20850,20851,
20852,20853,20854,20855,20856,20857,20858,20859,
20860,20861,20862,20863,20864,20866,20867,20868,
20869,20870,20871,20874,20875,20876,20877,20879,
20880,20881,20882,20883,20884,20885,20887,20888,
20889,20890,20891,20892,20893,20894,20895,20896,
20897,20898,20899,20900,20902,20903,20904,20906,
20909,20910,20911,20912,20913,20914,20915,20916,
20917,20918,20919,20920,20921,20923,20924,20925,
20926,20927,20928,20929,20930,20931,20932,20933,
20934,20935,20936,20937,20938,20939,20940,20941,
20942,20943,20944,20945,20946,20947,20948,20949,
20951,20952,20953,20954,20955,20956,20957,20958,
20959,20960,20961,20962,20963,20964,20965,20966,
20967,20968,20969,20970,20971,20972,20973,20974,
20975,20976,20977,20978,20979,20980,20981,20982,
20983,20984,20985,20986,20987,20988,20989,20990,
20991,20992,20993,20994,20995,20996,20997,20998,
20999,21000,21001,21002,21003,21004,21005,21006,
21008,21009,21010,21013,21014,21015,21016,21017,
21018,21019,21020,21021,21022,21023,21025,21026,
21027,21028,21029,21030,21031,21032,21033,21034,
21035,21036,21037,21038,21039,21040,21041,21042,
21043,21044,21045,21046,21047,21048,21049,21050,
21051,21052,21053,21054,21055,21056,21057,21058,
21059,21060,21061,21062,21063,21064,21065,21066,
21067,21068,21069,21070,21071,21072,21073,21074,
21075,21076,21077,21078,21081,21082,21083,21084,
21085,21086,21087,21089,21091,21092,21093,21094,
21095,21096,21099,21100,21102,21103,21104,21105,
21107,21108,21109,21110,21111,21112,21113,21114,
21115,21116,21117,21118,21119,21120,21121,21122,
21123,21124,21126,21127,21128,21129,21130,21131,
21132,21133,21134,21135,21137,21138,21139,21141,
21143,21145,21148,21150,21151,21152,21154,21156,
21157,21159,21160,21161,21162,21163,21164,21165,
21166,21167,21168,21169,21171,21172,21173,21174,
21175,21176,21177,21178,21179,21180,21181,21182,
21183,21184,21185,21187,21188,21189,21190,21191,
21193,21194,21195,21196,21198,21199,21200,21201,
21202,21203,21204,21205,21206,21207,21209,21210,
21211,21212,21213,21214,21215,21216,21217,21218,
21219,21220,21221,21223,21224,21225,21226,21227,
21228,21229,21230,21231,21232,21233,21234,21235,
21236,21237,21238,21239,21240,21241,21242,21243,
21244,21245,21246,21247,21248,21249,21250,21251,
21252,21253,21254,21255,21256,21257,21258,21259,
21260,21261,21262,21263,21264,21265,21266,21267,
21268,21269,21270,21271,21272,21273,21274,21275,
21276,21277,21278,21280,21281,21282,21283,21284,
21285,21286,21287,21288,21289,21290,21291,21292,
21293,21294,21295,21296,21297,21298,21299,21300,
21301,21302,21303,21304,21305,21306,21307,21308,
21309,21310,21311,21312,21314,21315,21316,21317,
21318,21319,21320,21321,21322,21323,21324,21325,
21326,21327,21328,21329,21330,21331,21332,21333,
21334,21335,21336,21337,21338,21339,21340,21341,
21342,21343,21344,21345,21346,21347,21348,21349,
21350,21351,21352,21353,21354,21355,21356,21357,
21358,21359,21360,21361,21362,21363,21364,21365,
21366,21369,21370,21372,21373,21374,21375,21376,
21377,21378,21379,21380,21381,21383,21385,21386,
21387,21389,21391,21392,21393,21394,21396,21400,
21401,21402,21403,21405,21406,21407,21408,21409,
21410,21411,21412,21414,21415,21416,21417,21418,
21419,21421,21422,21423,21424,21425,21426,21427,
21428,21430,21431,21432,21433,21434,21435,21436,
21437,21438,21439,21440,21441,21442,21443,21444,
21445,21446,21447,21448,21449,21450,21451,21453,
21454,21455,21456,21457,21458,21459,21460,21461,
21462,21464,21465,21466,21467,21468,21469,21470,
21471,21472,21473,21474,21475,21476,21477,21478,
21479,21480,21481,21482,21484,21485,21486,21487,
21488,21489,21490,21491,21492,21493,21494,21495,
21496,21497,21498,21499,21500,21501,21502,21503,
21504,21505,21506,21507,21508,21509,21510,21511,
21512,21513,21514,21515,21516,21517,21518,21519,
21520,21521,21522,21523,21524,21525,21526,21527,
21528,21529,21530,21531,21532,21533,21534,21535,
21536,21537,21538,21539,21540,21541,21542,21543,
21544,21545,21546,21547,21548,21549,21550,21551,
21552,21553,21554,21555,21556,21557,21558,21559,
21560,21561,21562,21563,21564,21565,21566,21567,
21568,21569,21570,21571,21572,21573,21574,21575,
21576,21577,21578,21579,21580,21581,21582,21583,
21584,21585,21586,21587,21588,21589,21590,21591,
21592,21593,21594,21595,21596,21597,21598,21599,
21600,21601,21602,21603,21604,21605,21606,21607,
21608,21609,21610,21611,21612,21613,21614,21615,
21616,21617,21618,21619,21620,21621,21622,21623,
21624,21625,21626,21627,21628,21629,21630,21631,
21632,21633,21634,21635,21636,21637,21638,21639,
21640,21642,21645,21646,21647,21649,21650,21651,
21652,21653,21655,21656,21657,21658,21659,21660,
21661,21662,21663,21664,21665,21666,21667,21668,
21669,21670,21671,21672,21673,21675,21676,21677,
21678,21679,21681,21682,21683,21684,21685,21686,
21687,21689,21690,21691,21692,21693,21694,21695,
21696,21697,21698,21699,21700,21701,21702,21704,
21705,21706,21707,21709,21710,21711,21712,21713,
21714,21715,21716,21717,21718,21719,21720,21721,
21722,21723,21724,21725,21726,21727,21728,21729,
21730,21731,21732,21733,21734,21735,21736,21737,
21740,21741,21742,21743,21744,21745,21746,21747,
21748,21749,21750,21751,21752,21754,21755,21756,
21757,21758,21759,21760,21762,21764,21767,21768,
21769,21770,21771,21772,21773,21774,21775,21776,
21777,21778,21779,21780,21781,21782,21783,21784,
21785,21786,21787,21788,21789,21790,21791,21792,
21793,21794,21795,21796,21797,21798,21799,21800,
21801,21802,21803,21804,21805,21806,21807,21808,
21809,21810,21811,21812,21813,21814,21815,21816,
21817,21818,21819,21820,21821,21822,21823,21824,
21825,21826,21827,21828,21829,21830,21831,21832,
21833,21834,21835,21836,21837,21838,21839,21840,
21841,21842,21843,21844,21845,21846,21847,21848,
21849,21850,21851,21852,21853,21854,21855,21856,
21857,21858,21859,21860,21861,21862,21863,21864,
21865,21866,21867,21868,21869,21870,21871,21872,
21874,21875,21876,21877,21878,21879,21880,21881,
21882,21883,21884,21885,21886,21887,21888,21889,
21890,21891,21892,21893,21894,21895,21896,21897,
21898,21899,21901,21902,21903,21904,21905,21906,
21907,21908,21909,21910,21911,21912,21913,21914,
21915,21916,21917,21918,21919,21920,21921,21922,
21923,21924,21925,21926,21927,21928,21929,21930,
21931,21932,21933,21934,21935,21936,21937,21938,
21939,21940,21941,21942,21943,21944,21945,21946,
21948,21949,21950,21951,21952,21953,21954,21956,
21957,21958,21960,21962,21963,21964,21965,21967,
21968,21970,21971,21972,21973,21974,21975,21976,
21977,21978,21979,21980,21982,21984,21985,21986,
21987,21988,21989,21990,21991,21992,21993,21994,
21995,21996,21997,21998,21999,22000,22001,22002,
22003,22005,22006,22007,22008,22009,22010,22011,
22012,22013,22014,22015,22016,22017,22018,22019,
22020,22022,22023,22024,22025,22026,22027,22028,
22029,22030,22031,22032,22033,22034,22035,22036,
22037,22038,22039,22041,22042,22043,22044,22045,
22046,22047,22048,22049,22050,22051,22052,22053,
22054,22055,22056,22057,22058,22059,22060,22061,
22062,22063,22064,22065,22066,22067,22068,22069,
22070,22071,22072,22073,22074,22075,22076,22077,
22078,22079,22080,22081,22082,22083,22084,22085,
22086,22087,22088,22089,22090,22091,22092,22093,
22094,22095,22096,22097,22098,22099,22100,22101,
22102,22103,22104,22105,22106,22107,22108,22109,
22110,22111,22112,22113,22114,22115,22116,22117,
22118,22119,22120,22121,22122,22123,22124,22125,
22126,22127,22128,22129,22130,22131,22132,22133,
22134,22135,22136,22137,22138,22139,22140,22141,
22142,22143,22144,22145,22146,22147,22148,22149,
22150,22151,22152,22153,22154,22155,22156,22157,
22158,22159,22160,22161,22162,22163,22164,22165,
22166,22167,22168,22169,22170,22171,22172,22173,
22174,22175,22176,22177,22178,22179,22180,22181,
22182,22183,22184,22185,22186,22187,22188,22189,
22190,22191,22192,22193,22194,22195,22196,22197,
22198,22199,22200,22201,22202,22203,22204,22205,
22206,22208,22209,22210,22211,22212,22213,22214,
22215,22216,22217,22218,22219,22220,22221,22222,
22223,22224,22225,22226,22228,22229,22230,22231,
22232,22233,22235,22236,22237,22238,22239,22240,
22241,22242,22243,22244,22245,22246,22247,22248,
22249,22250,22252,22253,22254,22255,22256,22257,
22258,22259,22260,22261,22262,22263,22264,22265,
22266,22267,22268,22269,22270,22271,22272,22273,
22274,22275,22276,22277,22278,22279,22280,22281,
22282,22283,22284,22285,22286,22287,22288,22289,
22290,22291,22292,22294,22296,22297,22298,22299,
22300,22301,22302,22304,22305,22306,22307,22308,
22309,22310,22311,22312,22313,22314,22315,22316,
22317,22318,22319,22320,22321,22322,22323,22324,
22325,22326,22327,22328,22329,22330,22331,22332,
22333,22334,22335,22336,22337,22338,22339,22340,
22341,22342,22343,22344,22345,22346,22347,22348,
22349,22350,22351,22352,22353,22354,22355,22356,
22357,22358,22359,22361,22362,22363,22364,22365,
22366,22367,22368,22369,22370,22371,22372,22373,
22374,22375,22376,22377,22378,22379,22380,22381,
22382,22383,22384,22385,22386,22387,22388,22389,
22390,22391,22392,22393,22394,22395,22396,22397,
22398,22399,22400,22401,22402,22403,22404,22405,
22406,22407,22408,22409,22410,22411,22412,22413,
22414,22415,22416,22417,22418,22419,22420,22421,
22422,22423,22424,22425,22426,22427,22428,22429,
22430,22431,22432,22433,22434,22435,22436,22437,
22438,22439,22440,22441,22442,22443,22444,22446,
22447,22448,22449,22451,22452,22453,22454,22455,
22456,22457,22458,22459,22460,22461,22462,22463,
22464,22466,22467,22468,22469,22470,22471,22472,
22473,22474,22475,22476,22477,22478,22480,22481,
22482,22483,22484,22485,22486,22487,22491,22492,
22493,22494,22495,22496,22497,22498,22499,22500,
22501,22502,22503,22504,22505,22506,22507,22508,
22509,22510,22511,22512,22513,22514,22515,22517,
22518,22519,22520,22521,22522,22523,22524,22525,
22526,22527,22528,22529,22530,22531,22532,22533,
22534,22535,22536,22537,22538,22539,22540,22541,
22542,22543,22544,22545,22547,22548,22549,22550,
22551,22552,22553,22554,22555,22556,22557,22558,
22559,22560,22561,22562,22563,22564,22565,22566,
22567,22568,22569,22570,22571,22572,22573,22574,
22575,22576,22577,22578,22579,22580,22581,22582,
22583,22584,22585,22586,22587,22588,22589,22590,
22591,22592,22593,22594,22595,22596,22597,22598,
22599,22600,22601,22602,22603,22604,22605,22606,
22607,22608,22609,22610,22611,22612,22613,22614,
22615,22616,22617,22618,22619,22620,22621,22622,
22623,22624,22625,22626,22627,22628,22629,22630,
22631,22632,22633,22634,22635,22636,22637,22638,
22639,22640,22641,22642,22643,22644,22645,22646,
22647,22648,22649,22650,22651,22652,22653,22654,
22655,22656,22657,22658,22659,22660,22661,22662,
22663,22664,22665,22666,22667,22668,22669,22670,
22671,22672,22673,22674,22675,22676,22677,22678,
22679,22680,22681,22682,22683,22684,22685,22686,
22687,22688,22689,22690,22691,22692,22693,22694,
22695,22696,22697,22698,22699,22700,22701,22702,
22703,22704,22705,22706,22707,22709,22710,22711,
22712,22713,22714,22715,22716,22718,22719,22720,
22721,22722,22724,22725,22726,22727,22728,22729,
22730,22731,22732,22733,22734,22735,22736,22737,
22738,22739,22740,22741,22742,22743,22744,22745,
22746,22747,22748,22749,22750,22751,22752,22753,
22754,22755,22756,22757,22758,22759,22760,22761,
22762,22763,22764,22765,22766,22767,22768,22769,
22770,22771,22772,22773,22775,22776,22777,22778,
22779,22780,22781,22782,22783,22784,22785,22786,
22787,22788,22789,22790,22791,22792,22793,22794,
22795,22796,22797,22798,22799,22801,22802,22803,
22804,22805,22806,22807,22809,22810,22812,22813,
22814,22815,22816,22817,22818,22819,22821,22822,
22823,22824,22827,22828,22829,22830,22831,22832,
22833,22834,22835,22837,22839,22840,22843,22844,
22845,22846,22847,22848,22849,22850,22851,22852,
22853,22854,22855,22856,22857,22858,22859,22860,
22861,22862,22863,22864,22865,22866,22867,22868,
22869,22870,22871,22872,22873,22874,22875,22876,
22877,22878,22879,22880,22881,22882,22883,22884,
22885,22886,22887,22888,22889,22890,22891,22892,
22893,22894,22895,22896,22897,22898,22899,22900,
22901,22902,22903,22904,22905,22906,22907,22908,
22909,22910,22911,22912,22913,22914,22915,22916,
22917,22918,22919,22920,22921,22922,22923,22924,
22925,22926,22927,22928,22929,22930,22931,22932,
22933,22934,22935,22936,22937,22938,22939,22940,
22941,22942,22943,22944,22945,22946,22947,22948,
22949,22950,22951,22952,22953,22954,22955,22956,
22957,22958,22959,22960,22961,22962,22963,22964,
22965,22966,22967,22968,22969,22970,22971,22972,
22973,22974,22975,22976,22977,22978,22979,22980,
22981,22982,22983,22984,22985,22986,22987,22988,
22989,22990,22991,22992,22993,22994,22995,22996,
22998,22999,23001,23002,23003,23004,23006,23007,
23008,23009,23011,23012,23014,23015,23016,23017,
23018,23020,23021,23022,23023,23025,23026,23028,
23029,23030,23031,23032,23033,23034,23035,23036,
23037,23038,23039,23040,23042,23043,23045,23046,
23047,23048,23049,23050,23051,23052,23053,23054,
23055,23056,23057,23058,23059,23060,23061,23062,
23063,23064,23065,23066,23067,23068,23069,23070,
23071,23072,23073,23074,23075,23077,23078,23079,
23080,23081,23082,23083,23084,23085,23086,23087,
23088,23089,23090,23091,23092,23093,23094,23095,
23096,23097,23098,23099,23100,23101,23102,23103,
23104,23105,23106,23107,23108,23109,23110,23111,
23112,23113,23114,23115,23116,23117,23118,23119,
23120,23121,23122,23123,23124,23125,23126,23127,
23128,23129,23130,23131,23132,23133,23134,23135,
23136,23137,23138,23139,23140,23141,23142,23143,
23144,23145,23146,23147,23148,23149,23150,23151,
23152,23153,23154,23155,23156,23157,23158,23159,
23160,23161,23162,23163,23164,23165,23166,23167,
23168,23169,23170,23171,23172,23173,23174,23175,
23176,23177,23178,23179,23180,23181,23182,23183,
23184,23185,23186,23187,23188,23189,23190,23191,
23192,23193,23194,23195,23196,23197,23198,23199,
23200,23201,23202,23203,23204,23205,23206,23207,
23208,23209,23210,23211,23212,23213,23214,23215,
23216,23217,23218,23219,23220,23221,23222,23223,
23224,23225,23226,23227,23228,23229,23230,23231,
23232,23233,23234,23235,23236,23237,23238,23239,
23240,23241,23242,23243,23245,23246,23247,23249,
23251,23252,23253,23254,23255,23256,23257,23258,
23259,23260,23261,23262,23263,23264,23265,23266,
23267,23268,23269,23270,23271,23272,23273,23274,
23275,23276,23277,23278,23279,23280,23281,23282,
23283,23284,23285,23286,23287,23288,23289,23290,
23291,23292,23293,23294,23295,23296,23297,23298,
23299,23300,23301,23302,23303,23305,23307,23308,
23309,23310,23311,23312,23313,23314,23315,23316,
23317,23318,23319,23320,23321,23322,23323,23324,
23325,23326,23327,23328,23329,23330,23331,23332,
23333,23334,23335,23336,23337,23338,23339,23340,
23341,23342,23343,23344,23345,23346,23347,23348,
23349,23350,23351,23352,23353,23354,23355,23357,
23358,23360,23361,23362,23363,23364,23366,23367,
23368,23369,23370,23371,23372,23373,23376,23379,
23380,23381,23382,23383,23384,23385,23386,23387,
23388,23389,23390,23391,23392,23393,23394,23395,
23396,23397,23398,23399,23400,23401,23402,23403,
23404,23405,23406,23407,23408,23409,23410,23411,
23412,23413,23414,23415,23416,23417,23418,23419,
23420,23421,23422,23423,23424,23425,23426,23427,
23428,23429,23430,23431,23432,23433,23434,23435,
23436,23437,23438,23439,23440,23441,23442,23443,
23444,23445,23446,23447,23448,23449,23450,23451,
23452,23453,23454,23456,23457,23458,23459,23460,
23461,23465,23466,23467,23468,23469,23470,23471,
23472,23473,23474,23475,23476,23477,23478,23479,
23480,23481,23482,23483,23484,23485,23486,23487,
23488,23489,23490,23491,23492,23493,23494,23495,
23496,23497,23498,23499,23500,23501,23502,23503,
23504,23505,23506,23507,23508,23509,23510,23511,
23512,23513,23514,23516,23517,23518,23519,23520,
23521,23523,23524,23525,23526,23527,23528,23529,
23531,23532,23533,23534,23535,23536,23537,23538,
23539,23540,23541,23542,23543,23546,23547,23548,
23551,23552,23553,23554,23555,23556,23557
};

int
cdos_utf8_to_pinyin (const char *utf8, char **result)
{
    GError *error = NULL;
    glong items_read = 0;
    glong items_write = 0;

    char *out = NULL;
    int len = 0;
    gboolean prev_is_ascii = FALSE;

    if (!result)
        return -1;

    gunichar *dst = g_utf8_to_ucs4 (utf8, -1, &items_read, &items_write, &error);

    if (error)
    {
        g_debug ("%s\n", error->message);
        g_error_free (error);
        return -1;
    }
    if (items_write == items_read)
    {
        return -1;
    }
    for (int i = 0; i < items_write; i++)
    {
        gunichar unichar = dst[i];
        if (unichar < 0x80)
        {
            ++len;
            prev_is_ascii = TRUE;
            continue;
        }
        else if (unichar < 0x4e00 || unichar > 0x9fa5)
        {
            gchar out[7];
            g_unichar_to_utf8 (unichar, out);
            g_debug ("%s: %s (not yet supported)", __func__, out);
            len += strlen(out) + 1;
            if (prev_is_ascii)
                ++len;
            prev_is_ascii = FALSE;
            continue;
        }
        unichar = dst[i] - 0x4e00;
        uint16_t idx = pinyin_index[unichar];
        uint16_t n = pinyin_index[unichar + 1] - idx;
        for (int j = 0; j < n; j++)
        {
            len += strlen(pinyin_table[pinyin_data[idx + j]]) + 1;
        }
        if (prev_is_ascii)
            ++len;
        prev_is_ascii = FALSE;
    }
    out = g_malloc(len + 1);
    len = 0;
    prev_is_ascii = FALSE;
    for (int i = 0; i < items_write; i++)
    {
        gunichar unichar = dst[i];
        if (unichar < 0x80)
        {
            out[len] = (char)unichar;
            ++len;
            prev_is_ascii = TRUE;
            continue;
        }
        else if (unichar < 0x4e00 || unichar > 0x9fa5)
        {
            gchar o[7];
            g_unichar_to_utf8 (unichar, o);
            if (prev_is_ascii)
            {
                out[len++] = ' ';
                prev_is_ascii = FALSE;
            }
            for (char *p = o; *p; p++)
            {
                out[len++] = *p;
            }
            out[len++] = ' ';
            continue;
        }
        if (prev_is_ascii)
        {
            out[len++] = ' ';
            prev_is_ascii = FALSE;
        }
        unichar = dst[i] - 0x4e00;
        uint16_t idx = pinyin_index[unichar];
        uint16_t n = pinyin_index[unichar + 1] - idx;
        for (int j = 0; j < n; j++)
        {
            const char *py = pinyin_table[pinyin_data[idx + j]];
            for (const char *p = py; *p; p++)
            {
                out[len++] = *p;
            }
            if (n > 1 && j < n - 1)
                out[len++] = -1;
        }
        if (i < items_write - 1)
            out[len++] = ' ';
    }
    out[len] = '\0';
    *result = out;
    return len;
}

const char *
cdos_utf8_to_pinyin_fast (const char *from)
{
    if (!from)
        return NULL;
    const char *result;
    static GHashTable *hz_py = NULL;
    if (!hz_py)
        hz_py = g_hash_table_new (g_str_hash, g_str_equal);

    result = g_hash_table_lookup (hz_py, from);
    if (result)
        return result;

    char *res = NULL;
    int ret = cdos_utf8_to_pinyin (from, &res);
    if (ret > -1)
    {
        g_hash_table_insert (hz_py, g_strdup (from), res);
        return res;
    }
    else
    {
        return NULL;
    }
}

typedef struct _PinyinTrie {
    struct _PinyinTrie *py[26];
    int is_leaf;
}PinyinTrie;

static PinyinTrie *pinyin_tree[26] = {NULL};

static void
build_trie_tree (void)
{
    for (int i = 0; i < sizeof(pinyin_table) / sizeof(pinyin_table[0]); i++)
    {
        const char *pinyin = pinyin_table[i];
        int idx = pinyin[0] - 'a';
        if (!pinyin_tree[idx])
            pinyin_tree[idx] = g_new0 (PinyinTrie, 1);
        PinyinTrie *tree = pinyin_tree[idx];
        const char *p = pinyin + 1;
        while (*p)
        {
            idx = p[0] - 'a';
            ++p;
            if (!tree->py[idx])
            {
                tree->py[idx] = g_new0 (PinyinTrie, 1);
                if (*p == '\0')
                    tree->py[idx]->is_leaf = 1;
            }
            tree = tree->py[idx];
        }
    }
}

static int
search_pinyin (const char *str)
{
    static int tree_inited = 0;
    if (!tree_inited)
    {
        build_trie_tree ();
        tree_inited = 1;
    }
    if (!str[0])
        return 0;

    char a = str[0];
    if (a > 'A' && a < 'Z')
        a += 32;
    if (a < 'a' || a > 'z')
        return 0;

    int idx = a - 'a';
    PinyinTrie *tree = pinyin_tree[idx];
    if (!tree)
        return 0;

    int length = 1, prev_leaf_length = 1;
    const char *p = str + 1;
    while (*p)
    {
        char a = p[0];
        /*
        if (a == -1)
        {
            ++p;
            ++prev_leaf_length;
            while (*p && *p != ' ')
            {
                ++p;
                ++prev_leaf_length;
            }
            return prev_leaf_length;
        }
        */
        if (a > 'A' && a < 'Z')
            a += 32;
        else if (a < 'a' || a > 'z')
            break;

        idx = a - 'a';
        if (!tree->py[idx])
        {
            break;
        }
        else
        {
            tree = tree->py[idx];
            ++p;
            ++length;
            if (tree->is_leaf || a == 'h')
                prev_leaf_length = length;
        }
    }
    return prev_leaf_length;
}

static int
match_buf (const char *buf1, const char *buf2, int len1, int len2)
{
    if (len1 < len2)
        return 0;
    int n = len2;
    /* zh - ch, ju - ji ... */
    if (strncmp (buf1, buf2, n))
        return 0;
    else if (len1 == len2)
        return 1;
    /* z - z; zhi - zh, ... */
    if (n == 1 || (n == 2 && buf1[1] == 'h'))
        return 1;
    /* 形式如jia - jian - jiang */
    return 0;
}

/* 字符串匹配,例如
 * str2: z 匹配 str1: z* 
 * li 不匹配 liang
 **/
static int
match_string (const char *str1, const char *str2)
{
    int len1 = 0, len2 = 0;
    const char *ptr1 = str1, *ptr2 = str2;
    int i;

    /*
    char py_map[] = {0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 1, 1, 1, 1,
                     0, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1};
    */

    while(*ptr2 && *ptr1)
    {
        len2 = search_pinyin (ptr2);
        if (len2 == 0)
        {
            len2 = 1;
        }
        char buf1[7], buf2[7];
        for (i = 0; i < len2; i++)
            buf2[i] = ptr2[i];
        buf2[i] = '\0';
        ptr2 += len2;

        if (ptr1 == str1)
        {
            while (*str1 == ' ')
                str1++;
            len1 = search_pinyin (str1);
            if (len1 == 0)
                len1 = 1;
            str1 += len1;
        }

        while (*ptr1 == ' ')
            ptr1++;
        len1 = search_pinyin (ptr1);
        if (len1 == 0)
        {
            len1 = 1;
        }
        for (i = 0; i < len1; i++)
            buf1[i] = ptr1[i];
        buf1[i] = '\0';
        ptr1 += len1;
        if (!match_buf (buf1, buf2, len1, len2))
        {
            int reset = 1;
            if (*ptr1 == -1)
            {
                ++ptr1;
                while (*ptr1 && *ptr1 != ' ')
                {
                    if (*ptr1 == -1)
                        ptr1++;
                    len1 = search_pinyin (ptr1);
                    if (len1 == 0)
                        break;
                    for (i = 0; i < len1; i++)
                        buf1[i] = ptr1[i];
                    buf1[i] = '\0';
                    ptr1 += len1;
                    if (reset && match_buf (buf1, buf2, len1, len2))
                    {
                        reset = 0;
                    }
                }
            }
            if (reset)
            {
                ptr1 = str1;
                ptr2 = str2;
            }
        }
        else if (*ptr1 == -1) /* 跳过多音字其余音节 */
        {
            ++ptr1;
            while (*ptr1 && *ptr1 != ' ')
                ptr1++;
        }
    }
    if (*ptr2)
        return 0;
    else
        return 1;
}

static gchar *
strdown (const gchar *src)
{
    if (!src)
         return NULL;

    gchar *dst = g_strdup (src);
    for (gchar *ptr = dst; *ptr != '\0'; ptr++)
    {
        if (*ptr >= 'A' && *ptr <= 'Z')
            *ptr = *ptr - ('A' - 'a');
    }
    return dst;
}

int
cdos_pinyin_match (const char *source, const char *pattern, int fuzzy)
{
    if (!source || !pattern)
        return 0;

    if (!fuzzy)
        return strstr (source, pattern) != NULL;
    if (strstr (source, pattern))
        return 1;

    const char *pinyin = cdos_utf8_to_pinyin_fast (source);
    if (!pinyin)
        pinyin = source;
    g_autofree gchar *py = strdown (pinyin);
    return match_string (py, pattern);
}

static int
cmp_buf (const char *buf1, const char *buf2, int len1, int len2)
{
    return strcmp (buf1, buf2);
    /*
    int n = len2 > len1 ? len1 : len2;
    int ret;
    if ((ret = strncmp (buf1, buf2, n)))
        return ret;
    else if (len1 == len2)
        return 0;
    if (n == 1 || (n == 2 && buf1[1] == 'h' && buf2[1] == 'h'))
        return 0;
    return len2 - len1;
    */
}

int
cdos_pinyin_compare (const char *str1, const char *str2)
{
    const char *ptr1 = str1, *ptr2 = str2;
    int ret = 0;

    if (!str1 && str2)
        return -1;
    else if (str1 && !str2)
        return 1;
    else if (!str1 && !str2)
        return 0;

    while(*ptr2 && *ptr1)
    {
        int len1 = 0, len2 = 0;
        const char *p;
        char buf1[46], buf2[46];

        while (*ptr2 == ' ')
            ptr2++;
        for (p = ptr2; *p && *p != ' ' && *p != -1; p++, len2++)
        {
            buf2[len2] = *p;
        }
        buf2[len2] = '\0';
        ptr2 = p;

        while (*ptr1 == ' ')
            ptr1++;
        for (p = ptr1; *p && *p != ' ' && *p != -1; p++, len1++)
            buf1[len1] = *p;
        buf1[len1] = '\0';
        ptr1 = p;

        ret = cmp_buf (buf1, buf2, len1, len2);
        if (!ret)
        {
            if (*ptr1 == -1) /* 跳过多音字其余音节 */
            {
                ++ptr1;
                while (*ptr1 && *ptr1 != ' ')
                    ptr1++;
            }
            if (*ptr2 == -1) /* 跳过多音字其余音节 */
            {
                ++ptr2;
                while (*ptr2 && *ptr2 != ' ')
                    ptr2++;
            }
        }
        else
            return ret;
    }
    if (*ptr2)
        return 1;
    else if (*ptr1)
        return -1;
    else
        return 0;
}

#if 0
int main (int argc, char **argv)
{
    if (argc < 2)
        return 0;

    char *str = argv[1];
    setlocale(LC_ALL, "");

    char *res = NULL;
    if (cdos_utf8_to_pinyin (str, &res) > -1)
    {
        printf("%s\n", res);
    }
    else
        printf("%s\n", str);
    if (argc == 3)
    {
        str = argv[2];
        printf("%d\n", search_pinyin(argv[2]));
        int len = 0;
        while(*str)
        {
            len = search_pinyin(str);
            if (len == 0)
            {
                str++;
                continue;
            }
            printf("%s:%d\n", str, len);
            str += len;
        }
        for (int i = 0; i < 400; i++)
        {
            cdos_pinyin_match (argv[1], pinyin_table[i], 1);
        }
        printf("%s %s: %d\n", argv[1], argv[2], cdos_pinyin_match (argv[1], argv[2], 1));
    }
    //while(1)
    //    sleep(10);

    return 0;
}
#endif
